;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;		=Disassembly of $0D mask for 93-95 PCMs= 
;
;		By: Dimented24x7 
;
;		Special thanks to the originator of the first hack as 
;		as well as the whole DIY-Prom crew at Thirdgen.org for 
;		making this possible. 
;
;		Disclaimer: No guarantee of accuracy can be made. 
;		This is intended for educational purposes only and 
;		you use the information contained within at your 
;		own risk. In other words, if your car blows up, 
;		it ain't my fault :) 
;
; Last revised: 11/16/07
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;Some conversion stuff...
;
;>SPARK
; -SA           = X * .351
;>RPM
; -RPM	        = X / 25
; -Scaled RPM   = X / 12.5, (X - 64) x 25
;>TPS
; -A/D TPS	= X * .0039
; -TPS		=(L5B26 x (A/D TPS-base TPS))/64   
;>MAP/VAC
; -SdMap        = ((ADC - 26) x 151) / 128
; -kPa          = (SDMap x .313) + 20.0
; -kPa          = (ADC x 0.369) + 10.415
; -A/D vac	= 256 - 1.18 x (A/D Baro - A/D MAP)
; -VACkPa       = .313 x (256 - A/D Vac)
;
;>Coolant counts conversion table for raw A/D value
;
;	 A/D 	degrees		A/D
; 	counts     C		Inv
; 	------- -------		---
; 	255	    -40		  0
; 	251	    -30		  4
; 	250	    -25		  5
; 	247	    -20		  8
; 	245	    -15		 10
; 	241	    -10		 14
; 	237	    - 5		 18
; 	231	      0		 24
; 	225	      5		 30
; 	218	     10		 37
; 	209	     15		 46
; 	199	     20		 56
; 	189	     25		 66
; 	177	     30		 78
; 	165	     35		 90
; 	152	     40		103
; 	139	     45		116
; 	126	     50		129
; 	114	     55		141
; 	102	     60		153
; 	92	     65		163
; 	81	     70		174
; 	72	     75		183
; 	64	     80		191
; 	56	     85		199
; 	50	     90		205
; 	44	     95		211
; 	39	    100		216
; 	34	    105		221
; 	30	    110		225
; 	26	    115		229
; 	23	    120		232
; 	21	    125		234
; 	18	    130		237
; 	16	    135		239
; 	14	    140		241
; 	13	    145		242
; 	12	    150		243
; 	0	    200		255
;
;>Temp conversion for stored linear counts
; -Deg C = .746 x counts - 40
;
;>Filtered value at any loop pass n is:
; n=0
;  Filtered value=initial value
;
; n=1,2,3...infinity
;  Filtered value=initial value + (1-FC) x sum([FC]^k,k=0,n-1) x 
;		  (end value - initial value)
;
; Note: Only valid while intial and end values remain constant 
;
;>General PWM hardware output
;
; MSB = Divider for switching speed, b7 must always be 1
;	
;  Example:
;   111 1111 =   32 Hz
;   011 1111 =   64 Hz
;   001 1111 =  128 Hz
;  etc.	
;
; LSB = Duty cycle
;
;  %DC = val / 256
;	  	
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
	ORG	$0000	
;
	FCB	$FF			;Holding byte 
;
;~~~~~~~~~~~~~~~~~~~~~~~~~
;
; RAM Addresses
;
;~~~~~~~~~~~~~~~~~~~~~~~~~
;
L0000	EQU	$0000			;Status word
					; b7=MPU has reset/powered up
					; b6=MPU clock failure
					; b5=COP timed out	
					; b4=Illegal opcode encountered
					; b3=SWI requested
					; b2=Extended interrupt requested
					; b1=Factory test in effect
					; b0=Main vector loop initiated
;
L0001	EQU	$0001			;
;
L0002	EQU	$0002			;Loop counter
L0003	EQU	$0003			;Loop counter, one delayed
L0004	EQU	$0004			;Status word
					; b7=EST circuit failure
					; b6=
					; b5=
					; b4=Hot restarts in effect
					; b3=Bad shutdown
					; b2=RAM refresh incomplete
					; b1=
					; b0=O2 functional
;
L0005	EQU	$0005			;Status word
					; b7=MAP AE in effect
					; b6=TPS AE in effect
					; b5=
					; b4=EPROM failure
					; b3=
					; b2=
					; b1=
					; b0=
;
L0006	EQU	$0006			;Cool temp, linear
L0007	EQU	$0007			;Current IAC steps
L0008	EQU	$0008			;Desired IAC steps
L0009	EQU	$0009			;Status word
					; b7=
					; b6=Throttle kicker failed diags.
					; b5=Stable idle attained, A/C on
					; b4=Stable idle attained, A/C off
					; b3=
					; b2, 0=IAC reset requested
					; b1=First run flag for IAC flow kickdown logic
					; b0, 0=IAC reset in progress
;
L000A	EQU	$000A			;IAC control word
					; b7=
					; b6=
					; b5=
					; b4, 1=Enable IAC
					; b3, 1=Coil state B on
					; b2, 1=Coil state A on
					; b1=
					; b0, 1=Extend IAC
;
;-Stored flagged malfunction words
;
L000B	EQU	$000B			;Stored flagged errors for error word #1
L000C	EQU	$000C			;Stored flagged errors for error word #2
L000D	EQU	$000D			;Stored flagged errors for error word #3
L000E	EQU	$000E			;Stored flagged errors for error word #4
L000F	EQU	$000F			;Stored flagged errors for error word #5
L0010	EQU	$0010			;Stored flagged errors for error word #6
L0011	EQU	$0011			;Stored flagged errors for error word #7
L0012	EQU	$0012			;Stored flagged errors for error word #8
L0013	EQU	$0013			;Stored flagged errors for error word #9
;
L0014	EQU	$0014			;# of error free startups
L0015	EQU	$0015			;Flagged malfunction word checksum
L0016	EQU	$0016			;Error word #1
					; b7=Error 13, O2 sensor
					; b6=
					; b5=
					; b4=Err 16, VSS buffer error
					; b3=
					; b2=
					; b1=
					; b0=Err 21, high TPS
;
L0017	EQU	$0017			;Error word #2
					; b7=Error 22, low TPS
					; b6=Error 23, low MAT
					; b5=Error 24, VSS signal lost (manual)
					; b4=Error 25, high MAT
					; b3=
					; b2=
					; b1=
					; b0=
;
L0018	EQU	$0018			;Error word #3
					; b7= Error 31, Governor
					; b6= Error 32, EGR
					; b5= Error 33, MAP sens. high
					; b4= Error 34, MAP sens. low
					; b3= Error 35, IAC malf.
					; b2= Error 36, Throttle kicker malf.
					; b1= Error 37, TCC brake switch stuck on
					; b0= Error 38, TCC brake switch stuck off
;
L0019	EQU	$0019			;Error word #4
					; b7=
					; b6=Error 41, 1x cam sensor
					; b5=Error 42, EST circuit failure
					; b4=Error 43, Knock sensor circuit
					; b3=Error 44, Lean O2
					; b2=Error 45, O2 rich
					; b1=Error 46, VATS
					; b0=
;	
L001A	EQU	$001A			;Error word #5
					; b7=Error 48,
					; b6=Error 49,
					; b5=Error 51, EEPROM checksum fail
					; b4=Error 52, High battery voltage, extended test
					; b3=Error 53, High battery voltage
					; b2=Error 54, Low fuel pump volts/faulty relay
					; b1=Error 55, Faulty PCM
					; b0=Error 56, quad driver B fault
;
L001B	EQU	$001B			;Error word #6
                    			; b7 1 = ERR 57,
                    			; b6 1 = ERR 58, high trans temp
                    			; b5 1 = ERR 59, low trans temp
                    			; b4 1 = ERR 61,
                    			; b3 1 = ERR 62, 
                    			; b2 1 = ERR 63, high baro pressure
                    			; b1 1 = ERR 64, low baro pressure
                    			; b0 1 = ERR 65,
; 	
L001C	EQU	$001C			;Error word #7
					; b7=Error 66, 3 -> 2 shift sol. quad driver failure
					; b6=Error 67, TCC solenoid failure
					; b5=Error 68, TCC slipping/not engaged
					; b4=Error 69, TCC failed to release
					; b3=
					; b2=
					; b1=Error 73, force motor fault 
					; b0=
;				
L001D	EQU	$001D			;Error word #8
					; b7=Error 75, low system voltage
					; b6=Error 76, 
					; b5=Error 77, MNP switch
					; b4=Error 78, 
					; b3=Error 79, Prolonged high trans temp
					; b2=Error 81, 2 -> 3 shift sol. failure
					; b1=Error 82, 1 -> 2 shift sol. failure
					; b0=Error 83, Shift solenoid driver failure
;
L001E	EQU	$001E			;Error word #9
					; b7=
					; b6=
					; b5=Error 86, engine speed to MPH ratio low
					; b4=Error 87, engine speed to MPH ratio high
					; b3=
					; b2=Error 89, excessive transmission slip detected
					; b1= 
					; b0=
;
L001F	EQU	$001F			;Error 21 fault counter
L0020	EQU	$0020			;Error 22 fault counter
L0021	EQU	$0021			;Error 24 counter
L0022	EQU	$0022			;Error 28 counter
L0023	EQU	$0023			;Error 37 counter
L0024	EQU	$0024			;Error 38 counter
L0025	EQU	$0025			;Error 68 counter
L0026	EQU	$0026			;Error 66 counter
L0027	EQU	$0027			;Error 67 counter
L0028	EQU	$0028			;Error 68 counter
L0029	EQU	$0029			;Error 72 counter
L002A	EQU	$002A			;Error 73 counter
L002B	EQU	$002B			;Error 81 counter
L002C	EQU	$002C			;Error 82 counter
L002D	EQU	$002D			;Error 83 counter
L002E 	EQU	$002E			;Error 85 counter
L002F	EQU	$002F			;Error 86 counter 
L0030	EQU	$0030			;Error 87 counter
L0031	EQU	$0031			;Error 89 counter
L0032	EQU	$0032			;Error 89 counter (2nd gear)
L0033	EQU	$0033			;Error 89 counter (3rd gear)
L0034	EQU	$0034			;Error 89 counter (4th gear)
L0035	EQU	$0035			;
L0036	EQU	$0036			;Status word
					; b7=High idle speed
					; b6=Throttle kicker has previously been active
					; b5=P/S compensation in effect
					; b4=Stall saver in effect
					; b3=Closed loop idle in effect
					; b2=Closed loop idle quals met
					; b1, 0=park/neutral, 1=drive
					; b0, 0=A/C off, 1=A/C on
;
L0037	EQU	$0037			;Status word
					; b7, 1=positive derivative idle term
					; b6=Limiting idle proportional action
					; b5=
					; b4=Throttle kicker disable req. flag
					; b3=Throttle kicker disable req. flag
					; b2, 1=Thottle kicker is active
					; b1=reduce thtottle follower action
					; b0=warm engine flag, throttle follower
;
L0038	EQU	$0038			;Status word
					; b7=
					; b6=
					; b5=Sign of added DFCO IAC airflow
					; b4=Initial stored IAC flow updated
					; b3=Update idle int. flag, drive, A/C on
					; b2=Update idle int. flag, drive, no A/C
					; b1=Update idle int. flag, P/N, A/C on
					; b0=Update idle int. flag, P/N, no A/C	
;				
L0039	EQU	$0039			;Status word
					; b7=Throttle open flag
					; b6=Manditory post-DFCO T/F added flow update
					; b5=
					; b4=
					; b3=
					; b2=Throttle kicker diags. performed
					; b1=Post throttle kicker base TPS updated
					; b0=Throttle kicker base TPS updated
;	
L003A	EQU	$003A			;Hardware status word 
					; b7=
					; b6=Bad RAM checksum
					; b5=Loop overran
					; b4=Stack overflow
					; b3=Data failed to be pulled from stack
					; b2=Improper hardware init.
					; b1=EPROM failure
					; b0=A/D failure
;
L003B	EQU	$003B			;Serial data status word
					; b7=Double byte transmission in progress
					; b6=ALDL transmission in progress
					; b5=
					; b4=In ALDL mode 4 (engine diags.)
					; b3=In ALDL mode 4 (transmission diags.)
					; b2=Diagnostics disabled, shutdown in progress
					; b1=In ALDL mode 10 (engine diag.)
					; b0=In ALDL mode 10 (trans diag.)
;
L003C	EQU	$003C			;Status word
					; b7=
					; b6=Codes remaining to be flashed through SES
					; b5=Blink out 1's portion of error code flag
					; b4=
					; b3=
					; b2=
					; b1=Error code not set for current pass
					; b0=
;
L003D	EQU	$003D			;Status word
					; b7=Deliver async pulse
					; b6=Accel enrich active
					; b5=PE underway
					; b4=Decel enlean underway
					; b3=Delay BLM update
					; b2=BLM cell change
					; b1=VATS passed/failed
					; b0=no delta speed entry into PE allowed
;
L003E	EQU	$003E			;Status word
					; b7=In closed loop
					; b6=
					; b5=
					; b4=In async fire
					; b3=Cold choke decay delay complete
					; b2=IAC disabled, batt volts out of range
					; b1=Enabling BLM update
					; b0=Deliver post-DFCO AE pulse
;
L003F	EQU	$003F			;Status word
					; b7=Short BPW flag
					; b6, 1=clear initial idle terms
					; b5=bypass PE delay next pass
					; b4=Sync/async. flag
					; b3=RPMs outside desired idle range (O2 handling)
					; b2=High idle speed, (O2 handling)
					; b1=RPMs changing too rapildy for idle (O2 hand'g)
					; b0=RPMs falling (O2 handling)
;
L0040	EQU	$0040			;
L0041	EQU	$0041			;Status word
					; b7=CCP solenoid energized
					; b6=
					; b5=In P/N
					; b4=A/C on
					; b3=A/C AE delivered
					; b2=
					; b1=MAT SA corr., 1=negative, 0=positive
					; b0=Increment closed linear EGR position
;
L0042	EQU	$0042			;ALDL diag. status word
					; b7=
					; b6=
					; b5=
					; b4=
					; b3=
					; b2=
					; b1=Transmission diagnostics flag
					; b0=Engine diagnostics flag
;
L0043	EQU	$0043			;Status word
					; b7=
					; b6=
					; b5=SES on/off toggle flag
					; b4=Change in fueling state
					; b3=SES fueling diags. in effect
					; b2=SES being used to relay fueling info
					; b1=Engine run time > 10 secs
					; b0=
;
L0044	EQU	$0044			;Status word
					; b7=EST circuit failure
					; b6=EST monitor
					; b5=High MAT temps present (O2 diags.)
					; b4=Ignition off
					; b3=First valid DRP recieved
					; b2=
					; b1=Limit closed loop correction when AE in effect 
					; b0=Factory test required
;
L0045	EQU	$0045			;ALDL diag. status word
					; b7=Engine error codes reset
					; b6=Transmission error codes reset
					; b5=
					; b4=BLMs reset
					; b3=
					; b2=
					; b1=
					; b0=
;
L0046	EQU	$0046			;Status word
					; b7=Have entered closed loop since startup
					; b6=BLM cells reset
					; b5=
					; b4=Fuel cut off in effect
					; b3=DFCO in effect
					; b2, 1=AIR enabled
					; b1, 1=Long term fueling is rich, 0=lean 
					; b0=
;
L0047	EQU	$0047			;Status word
					; b7=
					; b6=
					; b5=TCC status
					; b4=A/C status
					; b3=
					; b2=
					; b1=
					; b0=Load transient MAP filter coeff
;
L0048	EQU	$0048			;	
L0049	EQU	$0049			;
L004A	EQU	$004A			;	
L004B	EQU	$004B			;I/O word
					; b7, TCC/throttle kicker, 0=on, 1=off
					; b6=
					; b5=
					; b4=
					; b3=3->2 downshift feedback, 0=0 DC/short ckt, 
					;			      1=full DC/open ckt.
					; b2=TCC feedback, 0=on/short ckt, 
					;		   1=off/open ckt.
					; b1=2->3 shift feedback, 0=on/short ckt, 
					;			  1=off/open ckt.
					; b0=1->2 shift feedback, 0=on/short ckt, 
					;			  1=off/open ckt.
;
L004C	EQU	$004C			;I/O port D word 
					; b7=
					; b6=
					; b5=
					; b4=IAC enable	
					; b3=IAC coil state B
					; b2=IAC coil state A
					; b1=2 -> 3 shift solenoid
					; b0=1 -> 2 shift solenoid
;
L004D	EQU	$004D			;I/O port C word
					; b7=
					; b6=
					; b5, 0=Park/Neutral
					; b4=Range sw C, Open=1, Closed=0
					; b3=Range sw B, Open=1, Closed=0
					; b2=Range sw A, Open=1, Closed=0
					; b1, 1=brake switch off
					; b0, 1=A/C req.
;
L004E	EQU	$004E			;Status word
					; b7=
					; b6=
					; b5=
					; b4=
					; b3=
					; b2=
					; b1, 1=use cool linear temp table for trans temp  
					; b0, 1=use cool linear temp table for engine temp
;
L004F	EQU	$004F			;Status word
					; b7=Engine running
					; b6=EST bypass diag. already performed
					; b5=Reset INT on VE table change
					; b4=Engine above threshold for cranking or off
					; b3=Switched to open VE table
					; b2=Major loop overran 6.25 msec period
					; b1=
					; b0, 1=Negative SA, 0=Positive SA
;
L0050	EQU	$0050			;Status word
					; b7=Engine idling flag
					; b6=Idle SA enabled
					; b5=DRP occured
					; b4=In diagnostics mode
					; b3=In ALDL mode
					; b2=DRP occured, 6.25 msec test
					; b1=Open loop idle enabled
					; b0=Sync. MAP reads
;
L0051	EQU	$0051			;Status word
					; b7=Trans. from double. to single fire in prog.
					; b6=All SA has been ramped in
					; b5=DFCO SA correction enabled
					; b4=High batt. volts
					; b3=Shutdown in effect
					; b2=
					; b1=
					; b0=In single fire mode
;
L0052	EQU	$0052			;Status word
					; b7=In quasi-closed loop
					; b6=Cat overtemp flag
					; b5=A/C anticipated to be on
					; b4=A/C disabled due to high rpms
					; b3=CPI tuning valve enabled
					; b2=Engine idling (BLM routine)
					; b1=Low knock activity (low octaine control)
					; b0=High knock activity (low octaine control)
;
L0053	EQU	$0053			;Status word
					; b7=
					; b6=Single fire alt. exit desired
					; b5=First injector firing pending
					; b4=P/S SA compensation in effect
					; b3=Trans abuse indicated, limiting engine RPMs
					; b2=Shiftlight enabled
					; b1=Skip min delta TPS check, decel enlean
					; b0, 1=engine idle overspeed, 0=engine idle underspeed
;
L0054	EQU	$0054			;
L0055	EQU	$0055			;Fuel pump volts x 10
;
;-Stored error words
;
L0056	EQU	$0056			;Stored error word #1 
L0057	EQU	$0057			;Stored error word #2 
L0058	EQU	$0058			;Stored error word #3 
L0059	EQU	$0059			;Stored error word #4 
L005A	EQU	$005A			;Stored error word #5
L005B	EQU	$005B			;Stored error word #6
L005C	EQU	$005C			;Stored error word #7 
L005D	EQU	$005D			;Stored error word #8 
L005E	EQU	$005E			;Stored error word #9 
;
L005F	EQU	$005F			;DRP period
L0060	EQU	$0060			;
L0061	EQU	$0061			;Scaled RPMs, 0-2400 in 12.5 rpm inc., 2400-4800 in 25 rpm inc.
L0062	EQU	$0062			;RPM/25
L0063	EQU	$0063			;MSB, filtered RPM/12.5
L0064	EQU	$0064			;LSB, filtered RPM/12.5
L0065	EQU	$0065			;RPM/12.5, one delayed
L0066	EQU	$0066			;RPM/12.5, one delayed, O2 handling
L0067	EQU	$0067			;RPM/6.25
L0068	EQU	$0068			;MSB, filtered RPM/6.25
L0069	EQU	$0069			;LSB, filtered RPM/6.25
L006A	EQU	$006A			;MSB, delayed filtered RPM/6.25
L006B	EQU	$006B			;LSB, delayed filtered RPM/6.25
L006C	EQU	$006C			;Transient RPM ratio
L006D	EQU	$006D			;Status word
					; b7=	
					; b6=
					; b5=
					; b4=
					; b3=EGR position error sign, 0=plus/1=minus
					; b2=
					; b1=
					; b0=
;
L006E	EQU	$006E			;Status word
					; b7, 1=EGR enabled
					; b6, 1=RPM threshold for EGR met
					; b5, 1=TPS threshold for EGR met
					; b4, 1=MAP threshold for EGR met
					; b3=Accel enrich now active
					; b2, 1=Vac threshold for EGR met
					; b1=Knock prevention in effect
					; b0=INT reset/BLM cell change (EGR diag.)
;
L006F	EQU	$006F			;Status word
					; b7=One second has passed (error handling)
					; b6=Error, high MAP
					; b5=Error, low TPS
					; b4=Error 31, gov'r previously set
					; b3=Error, low MAP
					; b2=Error, High TPS
					; b1=
					; b0=
;
L0070	EQU	$0070			;Status word
					; b7=
					; b6=
					; b5=
					; b4=
					; b3=EGR diag. cycle performed flag
					; b2=EGR diags. in progress
					; b1=
					; b0=EGR failed diagnostics
;
L0071	EQU	$0071			;Status word
					; b7=Diagnostics in effect
					; b6=Run time threshold met for O2 diagnostics
					; b5=
					; b4=
					; b3=
					; b2=INT just reset (O2 diag.)
					; b1=
					; b0=SES commanded to be on
;
L0072	EQU	$0072			;Status word
					; b7=
					; b6=O2 lean flag
					; b5=Linear EGR failed diags flag
					; b4=
					; b3=
					; b2=High MAT temp flag (O2 diags.)
					; b1=Fuel pump volts diagnostics in progress
					; b0=Low fuel pump volts/relay failure
;
L0073	EQU	$0073			;CPU status
					; b7=
					; b6=
					; b5=
					; b4=
					; b3=
					; b2= VSS pulse ocured
					; b1=
					; b0=
;
L0074	EQU	$0074			;
L0075	EQU	$0075			;Governor status word
					; b7=In MPH govern
					; b6=MPH overspeed
					; b5=
					; b4=
					; b3=
					; b2=RPM overspeed
					; b1=Post derivative TPS return
					; b0=RPM govern in effect
;
L0076	EQU	$0076			;Status word
					; b7, 0=cruise cont'l active
					; b6=
					; b5=
					; b4=
					; b3=
					; b2=
					; b1=
					; b0=
;
L0077	EQU	$0077			;Stored state of L0076, one delayed
L0078	EQU	$0078			;MSB, filtered cat reaction temp
L0079	EQU	$0079			;LSB. filterec cat reaction temp
L007A	EQU	$007A			;Cat reaction temp
L007B	EQU	$007B			;Flow based cat raction temp				
L007C	EQU	$007C			;Cat reaction temperature filter coeff.
L007D	EQU	$007D			;Cat reaction AFR temperature bias
L007E	EQU	$007E			;Cat overtemp test timer
L007F	EQU	$007F			;Status word
					; b7=3->2 shift still in progress
					; b6=
					; b5=
					; b4=Trans downshift flag
					; b3=Downshift is underway
					; b2=
					; b1=Trans upshift flag
					; b0=Upshift is underway
;
L0080	EQU	$0080			;Shift status word
					; b7=ALDL downshift points in use
					; b6=Manual downshift pattern active
					; b5=Kickdown downshift pattern active
					; b4=Standard downshift pattern active
					; b3=ALDL upshift points in use
					; b2=Manual upshift pattern active
					; b1=Kickdown upshift pattern active
					; b0=Standard upshift pattern active
;
L0081	EQU	$0081			;Trans shift status word
					; b7=Shift is complete
					; b6=Heads up requested
					; b5=Drive range select during 2->4 jump shift
					; b4=Shift has started
					; b3= 
					; b2=
					; b1=Trans shift status flagged
					; b0=Slope of derivative is negative
;
L0082	EQU	$0082			;Status word
					; b7=Powertrain is in motion
					; b6=ELCO inst. prom is present
					; b5=
					; b4=NV ram update allowed
					; b3=Ign. power is on
					; b2=Force motor pulse in progress
					; b1=
					; b0=Batt volts too low to flag force motor fault 
;
L0083	EQU	$0083			;Trans status word
					; b7=
					; b6=
					; b5=
					; b4=
					; b3=
					; b2=
					; b1=Hot trans temps present
					; b0=Disallow shift into OD
;
L0084	EQU	$0084			;Status word
					; b7=
					; b6=
					; b5=3->2 shift solenoid already flagged as operational
					; b4=3->2 shift solenoid operational flag
					; b3=
					; b2=Output speed present (3->2 sol.)
					; b1=Output speed lost (3->2 sol.)
					; b0=Manual range selected (3->2 sol.) 
;
L0085	EQU	$0085			;TCC status word
					; b7=Excessive negative TCC slip present
					; b6=TCC being applied
					; b5=TCC fully locked/commanded to lock
					; b4=TCC being released
					; b3=Use high speed TCC TPS thresholds
					; b2=High transmission temp
					; b1=TCC output state valid
					; b0= 
;
L0086	EQU	$0086			;Status word
					; b7=
					; b6=
					; b5=TCC to remain unlocked
					; b4=PWM TCC diag. override in effect
					; b3=Insufficient time to lock TCC 
					; b2=Lock TCC
					; b1=TCC locked/being applied
					; b0=TCC fully unlocked
;
L0087	EQU	$0087			;Status word
					; b7=Delaying TCC lockup
					; b6=Insufficient time to relock TCC
					; b5=TCC disabled due to rapid decel
					; b4=PWM TCC to be locked
					; b3=Repeat unlock/relock prevention in effect
					; b2=TCC commanded to lock
					; b1=TCC commanded to unlock
					; b0=PWM'd TCC locked
;
L0088	EQU	$0088			;Status word
					;*Set items bar TCC lockup*				
					; b7=Excessive negative torque converter slip
					; b6=A/C on
					; b5=Transmission recently downshifted
					; b4=Rapid decel present
					; b3=Brakes have been applied
					; b2=Cold engine temps present
					; b1=Cold trans temps present
					; b0=In manual range
;
L0089	EQU	$0089			;Status word
					;*Set items bar TCC lockup*				
					; b7=
					; b6=
					; b5=
					; b4=
					; b3=
					; b2=Applicable errors have been set
					; b1=Repeat locking/unlocking prevention in effect
					; b0=Transmission recently upshifted
;					
L008A	EQU	$008A			;Status word
					; b7=Force motor fault present
					; b6=Desired line pressure rising
					; b5=In forward range
					; b4=In reverse
					; b3=Selected range has changed flag
					; b2=Engine braking in effect
					; b1=Forward range selected
					; b0=Upshift flag 
;
L008B	EQU	$008B			;Status word 
					;*Set items bar adaptive modifier updates*
					; b7=Excessive change in TPS
					; b6=Short shift times present
					; b5=Excessive transmission slip present
					; b4=Manual first or second selected
					; b3=Engine braking in effect
					; b2=TPS out of range for adaptive modifier update
					; b1=Trans temp out of range
					; b0=Trans overtemp, manual pattern selected, or 
					;    pending shift 
;
L008C	EQU	$008C			;Status word
					;*Set items bar adaptive modifier updates*
					; b7=Shift is flaring
					; b6=
					; b5=TCC being released/applied
					; b4=
					; b3=Errors applicable to adaptive modifier set
					; b2=Change in speed to great 
					; b1=
					; b0=Time between shifts too short
;
L008D	EQU	$008D			;Status word
					;*Set items bar adaptive modifier updates*			;	
					; b7=
					; b6=
					; b5=
					; b4=
					; b3=
					; b2=
					; b1=
					; b0=In 4WD low
;
L008E	EQU	$008E			;Status word
					; b7=Vehicle in motion flag (error 38)
					; b6=Vehicle in decelerating/stopped flag (error 38)
					; b5=Vehicle in motion flag (errpr 37)
					; b4=Vehicle stopped flag (error 37)
					; b3=
					; b2=Error 24 logged for this session
					; b1=Error 22 logged for this session
					; b0=Error 21 logged for this session
; 
L008F	EQU	$008F			;Status word
					; b7=Error 67 logged for this session
					; b6=Error 77 diags. performed
					; b5=Error 73 logged for this session
					; b4=Error 72 logged for this session
					; b3=Error 69 logged for this session
					; b2=Error 68 logged for this session
					; b1=Error 53 reset performed
					; b0=
;
L0090	EQU	$0090			;Status word
					; b7=Error 66 logged for this session
					; b6=Error 89 logged for this session
					; b5=error 87 logged for this session
					; b4=Error 86 logged for this session
					; b3=
					; b2=Error 83 logged for this session
					; b1=Error 82 logged for this session
					; b0=Error 81 logged for this session
L0091	EQU	$0091			;
L0092	EQU	$0092			;Status word
					; b7=Heat related transmission errors set
					; b6=External errors applicalbe to TCC lockup set
					; b5=External errors applicable to line pressure/
					;    force motor set
					; b4=Errors applicable to adapive modifier set
					; b3=General trans related errors set
					; b2=Trans. press. switch man. error
					; b1=VSS/TOS errors present
					; b0=Transmission TPS error
;
L0093	EQU	$0093			;Status word
					; b7=
					; b6=
					; b5=
					; b4=
					; b3=Error, high system voltage present
					; b2=Error, trans output speed loss
					; b1=Shift error
					; b0=External errors set
;
L0094	EQU	$0094			;Status word
					; b7=
					; b6=
					; b5=
					; b4=
					; b3=High system voltage present
					; b2, 0=bump idle tps when in decel
					; b1=
					; b0=
;
L0095	EQU	$0095			;Stored I/O port C status
L0096	EQU	$0096			;Status word
					; b7=Cruise control active
					; b6=
					; b5=4WD low active
					; b4=Enable kickdown
					; b3=
					; b2=
					; b1=A/C has been requested
					; b0=Brakes applied
;
L0097	EQU	$0097			;Previous state of L0096
L0098	EQU	$0098			;I/O feedback status word
L0099	EQU	$0099			;Trans shift pattern word
					; b7=
					; b6=
					; b5=
					; b4=
					; b3, 1=Illigal shift pattern
					; b2, 1=Manual pattern requested
					; b1, 1=Performance pattern requested
					; b0, 1=Normal pattern requested
;
L009A	EQU	$009A			;Previous shift pattern 
;
L009B	EQU	$009B			;Range select timer
;
L009C	EQU	$009C			;Range select word
					; b7=Invalid range 
					; b6=P/N
					; b5=Reverse
					; b4=Invalid range
					; b3=Forward D4
					; b2=Forward D3
					; b1=Forward D2
					; b0=Forward D1
;
L009D	EQU	$009D			;Previous state of L009C
L009E	EQU	$009E			;
L009F	EQU	$009F			;A/D transmission temp
L00A0	EQU	$00A0			;A/D TPS voltage
L00A1	EQU	$00A1			;A/D batt. voltage
L00A2	EQU	$00A2			;Linaer cool temp
L00A3	EQU	$00A3			;Engine temp for transmission
L00A4	EQU	$00A4			;
L00A5	EQU	$00A5			;Cold temp trans term
L00A6	EQU	$00A6			;MSB, trans baro term, filtered
L00A7	EQU	$00A7			;LSB, trans baro term, filtered
L00A8	EQU	$00A8			;
L00A9	EQU	$00A9			;
L00AA	EQU	$00AA			;MSB, transmission TPS, filtered
L00AB	EQU	$00AB			;LSB, transmission TPS, filtered
L00AC	EQU	$00AC			;
L00AD	EQU	$00AD			;
L00AE	EQU	$00AE			;Trans temp, linear
L00AF	EQU	$00AF			;MSB, Trans temp, filtered
L00B0	EQU	$00B0			;LSB, Trans temp, filtered
L00B1	EQU	$00B1			;MSB, engine RPM x 8
L00B2	EQU	$00B2			;LSB, engine RPM x 8
L00B3	EQU	$00B3			;
L00B4	EQU	$00B4			;
L00B5	EQU	$00B5			;
L00B6	EQU	$00B6			;
L00B7	EQU	$00B7			;MSB, Trans Input Speed RPM x 8
L00B8	EQU	$00B8			;LSB, Trans Input Speed RPM x 8
L00B9	EQU	$00B9			;
L00BA	EQU	$00BA			;
L00BB	EQU	$00BB			;
L00BC	EQU	$00BC			;MSB, Trans Input Speed RPM x 8, filtered
L00BD	EQU	$00BD			;LSB, Trans Input Speed RPM x 8, filtered
L00BE	EQU	$00BE			;MSB, Trans Input Speed RPM x 8
L00BF	EQU	$00BF			;LSB, Trans Input Speed RPM x 8
L00C0	EQU	$00C0			;MSB, Trans Input Speed RPM x 8, filtered
L00C1	EQU	$00C1			;LSB, Trans Input Speed RPM x 8, filtered
L00C2	EQU	$00C2			;Stored pulse counts, one delayed
L00C3	EQU	$00C3			;MSB, free running counter, one delayed
L00C4	EQU	$00C4			;LSB, free running counter, one delayed
L00C5	EQU	$00C5			;Timer to wait for TOS signal
L00C6	EQU	$00C6			;MSB, TOS RPM x 8
L00C7	EQU	$00C7			;LSB, TOS RPM x 8
L00C8	EQU	$00C8			;MSB, free running counter val. (trans output speed)
L00C9	EQU	$00C9			;LSB, free running counter val. (trans output speed)
L00CA	EQU	$00CA			;Stored accumulated pulse counts (trans output speed)
L00CB	EQU	$00CB			;TOS RPM x 8
L00CC	EQU	$00CC			;
L00CD	EQU	$00CD			;MSB, TOS RPM x 8, filtered
L00CE	EQU	$00CE			;LSB, TOS RPM x 8, filtered
L00CF	EQU	$00CF			;MSB, stored TOS RPM x 8
L00D0	EQU	$00D0			;LSB, stored TOS RPM x 8
L00D1	EQU	$00D1			;MSB, MPH, filtered
L00D2	EQU	$00D2			;LSB, MPH, filtered
L00D3	EQU	$00D3			;
L00D4	EQU	$00D4			;Shift flare timer
L00D5	EQU	$00D5			;Shift timer
L00D6	EQU	$00D6			;Shift duration timer
L00D7	EQU	$00D7			;Shift time error 
L00D8	EQU	$00D8			;Adaptive modifier clamping pressure
L00D9	EQU	$00D9			;Stored underpressure for current shift
L00DA	EQU	$00DA			;Stored overpressure for current shift
L00DB	EQU	$00DB			;Current change to adaptive modifier
L00DC	EQU	$00DC			;Desired line pressure
L00DD	EQU	$00DD			;
L00DE	EQU	$00DE			;
L00DF	EQU	$00DF			;
L00E0	EQU	$00E0			;
L00E1	EQU	$00E1			;
L00E2	EQU	$00E2			;Engine braking delay timer
L00E3	EQU	$00E3			;
L00E4	EQU	$00E4			;
L00E5	EQU	$00E5			;MSB, filtered torque converter slip (RPM x 8)
L00E6	EQU	$00E6			;LSB, filtered torque converter slip (RPM x 8)
L00E7	EQU	$00E7			;MSB, magnitude of torque converter slip 
L00E8	EQU	$00E8			;LSB, magnitude of torque converter slip 
L00E9	EQU	$00E9			;Stored torque converter slip
L00EA	EQU	$00EA			;
L00EB	EQU	$00EB			;MSB, PWM TCC operating point
L00EC	EQU	$00EC			;LSB, PWM TCC operating point
L00ED	EQU	$00ED			;MSB, accumulated PWM TCC DC
L00EE	EQU	$00EE			;LSB, accumulated PWM TCC DC
L00EF	EQU	$00EF			;
L00F0	EQU	$00F0			;
L00F1	EQU	$00F1			;MPH, one delayed
L00F2	EQU	$00F2			;
L00F3	EQU	$00F3			;MSB, gear ratio x 16384
L00F4	EQU	$00F4			;LSB, gear ratio x 16384
L00F5	EQU	$00F5			;Brake status min change state timer
L00F6	EQU	$00F6			;Force motor fault timer
L00F7	EQU	$00F7			;
L00F8	EQU	$00F8			;MSB, engine run time
L00F9	EQU	$00F9			;LSB, engine run time
L00FA	EQU	$00FA			;Timer for powertrain in motion flag
L00FB	EQU	$00FB			;
L00FC	EQU	$00FC			;
L00FD	EQU	$00FD			;
L00FE	EQU	$00FE			;
L00FF	EQU	$00FF			;
L0100	EQU	$0100			;
L0101	EQU	$0101			;
L0102	EQU	$0102			;
L0103	EQU	$0103			;
;
;-ALDL MPH upshift points
;
L0104	EQU	$0104			;
L0105	EQU	$0105			;
L0106	EQU	$0106			;
;
;-ALDL MPH downshift points
;
L0107	EQU	$0107			;2 -> 1
L0108	EQU	$0108			;3 -> 2
L0109	EQU	$0109			;4 -> 3
;
L010A	EQU	$010A			;PWM TCC diag. DC
L010B	EQU	$010B			;
L010C	EQU	$010C			;
L010D	EQU	$010D			;
L010E	EQU	$010E			;
L010F	EQU	$010F			;
L0110	EQU	$0110			;
L0111	EQU	$0111			;Diag. transmisison line pressure
L0112	EQU	$0112			;
L0113	EQU	$0113			;
L0114	EQU	$0114			;
L0115	EQU	$0115			;
L0116	EQU	$0116			;
L0117	EQU	$0117			;
L0118	EQU	$0118			;3->2 shift solenoid diag. DC
;
L0119	EQU	$0119			;TCC coast release timer
L011A	EQU	$011A			;
L011B	EQU	$011B			;Accumulated TCC relock period timer
L011C	EQU	$011C			;
L011D	EQU	$011D			;MSB, TCC relock delay timer
L011E	EQU	$011E			;LSB, TCC relock delay timer
L011F	EQU	$011F			;MSB, accumulated TCC relock delay timer
L0120	EQU	$0120			;LSB, accumulated TCC relock delay timer
L0121	EQU	$0121			;Rapid decel flag reset timer
L0122	EQU	$0122			;TCC upshift disable timer
L0123	EQU	$0123			;TCC engage/release shift delay timer
L0124	EQU	$0124			;MSB, PWM TCC DC
L0125	EQU	$0125			;LSB, PWM TCC DC
L0126	EQU	$0126			;TCC MPH threshold
L0127	EQU	$0127			;MSB, DC for PWM TCC to be fully locked 
L0128	EQU	$0128			;LSB, DC for PWM TCC to be fully locked
L0129	EQU	$0129			;TCC engage/release timer
L012A	EQU	$012A			;TCC engage/release delay timer
L012B	EQU	$012B			;Manditory TCC timer
L012C	EQU	$012C			;
L012D	EQU	$012D			;
L012E	EQU	$012E			;Desired 3->2 solenoid duty cycle
L012F	EQU	$012F			;3->2 solenoid output switching speed
L0130	EQU	$0130			;3-2 shift modulation timer
L0131	EQU	$0131			;4 -> 2 shift limiting timer
;
;-Stored shift duration times
;
L0132	EQU	$0132			;1 -> 2
L0133	EQU	$0133			;2 -> 3
L0134	EQU	$0134			;3 -> 4
;
;-Stored shift time error
;
L0135	EQU	$0135			;1 -> 2
L0136	EQU	$0136			;2 -> 3
L0137	EQU	$0137			;3 -> 4
;
L0138	EQU	$0138			;Start of shift time
L0139	EQU	$0139			;Post shift timer
L013A	EQU	$013A			;MSB, accumulated line pressure
L013B	EQU	$013B			;LSB, accumulated line pressure
L013C	EQU	$013C			;Added low speed line pressure
L013D	EQU	$013D			;Momentary added line pressure timer
L013E	EQU	$013E			;
L013F	EQU	$013F			;
L0140	EQU	$0140			;Final low speed line pressure
L0141	EQU	$0141			;Final transmission line pressure
L0142	EQU	$0142			;
L0143	EQU	$0143			;Stored line pressure, one delayed
L0144	EQU	$0144			;Adaptive pressure modifer 
L0145	EQU	$0145			;Adapytive modifer cell #
L0146	EQU	$0146			;Force motor %DC
L0147	EQU	$0147			;Desired force motor current
L0148	EQU	$0148			;A/D force motor current
L0149	EQU	$0149			;Stored force motor current error term
L014A	EQU	$014A			;MSB, force motor integrator
L014B	EQU	$014B			;LSB, force motor integrator
L014C	EQU	$014C			;Trans temp value used for force motor prop. gain lookup
L014D	EQU	$014D			;Current force motor error proportional gain
L014E	EQU	$014E			;Force motor error prop. gain, one delayed
L014F	EQU	$014F			;
L0150	EQU	$0150			;
L0151	EQU	$0151			;
L0152	EQU	$0152			;
L0153	EQU	$0153			;
L0154	EQU	$0154			;
L0155	EQU	$0155			;
L0156	EQU	$0156			;
L0157	EQU	$0157			;
L0158	EQU	$0158			;Time DFCO active, O2 diags.
L0159	EQU	$0159			;Error 14 timer
L015A	EQU	$015A			;Error 15 timer
L015B	EQU	$015B			;Error 21 timer
L015C	EQU	$015C			;Error 22 timer
L015D	EQU	$015D			;Error 24 timer
L015E	EQU	$015E			;
L015F	EQU	$015F			;
L0160	EQU	$0160			;
L0161	EQU	$0161			;
L0162	EQU	$0162			;MSB, error 52 timer
L0163	EQU	$0163			;LSB, error 52 timer
L0164	EQU	$0164			;Error 53 timer
L0165	EQU	$0165			;Trans temp high error timer
L0166	EQU	$0166			;Trans temp low error timer
L0167	EQU	$0167			;Error 63 timer
L0168	EQU	$0168			;Error 64 timer
L0169	EQU	$0169			;Error 66 timer
L016A	EQU	$016A			;Error 67 timer
L016B	EQU	$016B			;Error 68 timer
L016C	EQU	$016C			;Error 69 timer
L016D	EQU	$016D			;Error 71 timer
L016E	EQU	$016E			;Error 72 timer
L016F	EQU	$016F			;Error 73 timer
L0170	EQU	$0170			;
L0171	EQU	$0171			;Error 75 timer
L0172	EQU	$0172			;
L0173	EQU	$0173			;Error 79 timer
L0174	EQU	$0174			;
L0175	EQU	$0175			;Error 81 timer
L0176	EQU	$0176			;Error 82 timer
L0177	EQU	$0177			;Error 83 timer
L0178	EQU	$0178			;
L0179	EQU	$0179			;Error 86 timer
L017A	EQU	$017A			;
L017B	EQU	$017B			;Vehicle coasting timer
L017C	EQU	$017C			;Vehicle in motion timer
L017D	EQU	$017D			;Vehicle decelerating/stopped timer
L017E	EQU	$017E			;Vehicle in motion timer
L017F	EQU	$017F			;
L0180	EQU	$0180			;Stored MPH shift threshold
L0181	EQU	$0181			;
L0182	EQU	$0182			;TCC downshift disable timer
L0183	EQU	$0183			;TCC rapid decel timer
L0184	EQU	$0184			;TCC accel release timer
L0185	EQU	$0185			;Temp store for PWM TCC routines
L0186	EQU	$0186			;Temp store for PWM TCC routines
L0187	EQU	$0187			;Temp store for PWM TCC routines
L0188	EQU	$0188			;Roll-over tracking for PWM TCC routines
L0189	EQU	$0189			;
L018A	EQU	$018A			;
L018B	EQU	$018B			;Trans TPS at start of shift
L018C	EQU	$018C			;Jump shift timer
L018D	EQU	$018D			;Jump shift 2->4 timer
L018E	EQU	$018E			;Jump shift compensation apply timer
L018F	EQU	$018F			;Jump shift pressure compensation
L0190	EQU	$0190			;MSB, RPM/MPH ratio x 4
L0191	EQU	$0191			;LSB, RPM/MPH ratio x 4
L0192	EQU	$0192			;A/D coolant
L0193	EQU	$0193			;
L0194	EQU	$0194			;
L0195	EQU	$0195			;
L0196	EQU	$0196			;
L0197	EQU	$0197			;Current %TPS, for engine
L0198	EQU	$0198			;%TPS, one delayed
L0199	EQU	$0199			;%TPS, two delayed
L019A	EQU	$019A			;%TPS, three delayed
L019B	EQU	$019B			;%TPS, four delayed
L019C	EQU	$019C			;Error 72 delay timer
L019D	EQU	$019D			;MSB, engine RPM x 8
L019E	EQU	$019E			;LSB, engine RPM x 8
;
L019F	EQU	$019F			;Start of sampled engine RPMs x 8
L01A0	EQU	$01A0			;
L01A1	EQU	$01A1			;
L01A2	EQU	$01A2			;
L01A3	EQU	$01A3			;
L01A4	EQU	$01A4			;
L01A5	EQU	$01A5			;
L01A6	EQU	$01A6			;
L01A7	EQU	$01A7			;
L01A8	EQU	$01A8			;
L01A9	EQU	$01A9			;
L01AA	EQU	$01AA			;
L01AB	EQU	$01AB			;
L01AC	EQU	$01AC			;
L01AD	EQU	$01AD			;
L01AE	EQU	$01AE			;
L01AF	EQU	$01AF			;
L01B0	EQU	$01B0			;
L01B1	EQU	$01B1			;
L01B2	EQU	$01B2			;
L01B3	EQU	$01B3			;
L01B4	EQU	$01B4			;
L01B5	EQU	$01B5			;
L01B6	EQU	$01B6			;
L01B7	EQU	$01B7			;
L01B8	EQU	$01B8			;
L01B9	EQU	$01B9			;
L01BA	EQU	$01BA			;
L01BB	EQU	$01BB			;
L01BC	EQU	$01BC			;
L01BD	EQU	$01BD			;
L01BE	EQU	$01BE			;
L01BF	EQU	$01BF			;
L01C0	EQU	$01C0			;
L01C1	EQU	$01C1			;
L01C2	EQU	$01C2			;
L01C3	EQU	$01C3			;
L01C4	EQU	$01C4			;
L01C5	EQU	$01C5			;End sampled engine RPMs x 8
;
L01C6	EQU	$01C6			;
L01C7	EQU	$01C7			;
L01C8	EQU	$01C8			;
L01C9	EQU	$01C9			;Engine RPM sample counter
L01CA	EQU	$01CA			;Upper byte, average sum of change in engine RPM
L01CB	EQU	$01CB			;Middle byte, average sum of change in engine RPM
L01CC	EQU	$01CC			;Lower byte, average sum of change in engine RPM
L01CD	EQU	$01CD			;Upper byte, average sum, last pass
L01CE	EQU	$01CE			;Middle byte, average sum, last pass
L01CF	EQU	$01CF			;Lower byte, average sum, last pass
L01D0	EQU	$01D0			;Delta engine RPMs, ALDL diags.
L01D1	EQU	$01D1			;Desired % liner EGR position
L01D2	EQU	$01D2			;Desired %EGR 
L01D3	EQU	$01D3			;Delta pressure across EGR
L01D4	EQU	$01D4			;%EGR flow
L01D5	EQU	$01D5			;Filtered %EGR flow, MSB
L01D6	EQU	$01D6			;Filtered %EGR flow, LSB
L01D7	EQU	$01D7			;Linear EGR position
L01D8	EQU	$01D8			;MSB, A/D closed EGR pintle pos, lag filtered
L01D9	EQU	$01D9			;LSB, A/D closed EGR pintle pos, lag filtered
L01DA	EQU	$01DA			;MSB, filtered % liner EGR
L01DB	EQU	$01DB			;LSB, filtered % liner EGR
L01DC	EQU	$01DC			;% linear EGR
L01DD	EQU	$01DD			;Linear EGR position error
L01DE	EQU	$01DE			;Desired EGR gain factor
L01DF	EQU	$01DF			;MSB, EGR correction factor
L01E0	EQU	$01E0			;LSB, EGR correction factor
L01E1	EQU	$01E1			;MSB, EGR proportional term
L01E2	EQU	$01E2			;LSB, EGR proportional term
L01E3	EQU	$01E3			;SD MAP
L01E4	EQU	$01E4			;SD MAP, one delayed
L01E5	EQU	$01E5			;SD MAP, two delayed
L01E6	EQU	$01E6			;A/D MAP, Filtered, MSB
L01E7	EQU	$01E7			;A/D MAP, Filtered, LSB
L01E8	EQU	$01E8			;Transient MAP apply timer
L01E9	EQU	$01E9			;A/D MAP
L01EA	EQU	$01EA			;AE SD MAP, filtered per cool temp, MSB
L01EB	EQU	$01EB			;AE SD MAP, filtered per cool temo, LSB
L01EC	EQU	$01EC			;Vacuum
L01ED	EQU	$01ED			;Delta MAP for AE
L01EE	EQU	$01EE			;Slo filtered MAP filter coeff.
L01EF	EQU	$01EF			;Baro val. for lookup
L01F0	EQU	$01F0			;RPM threshold for PE, based on Baro
L01F1	EQU	$01F1			;Baro factor for MAP/baro ratio calc
L01F2	EQU	$01F2			;MAP/baro ratio
L01F3	EQU	$01F3			;MSB, Slow filtered O2
L01F4	EQU	$01F4			;LSB, Slow filtered O2
L01F5	EQU	$01F5			;MSB, Slow filtered O2, based on airflow
L01F6	EQU	$01F6			;LSB, Slow filtered O2, based on airflow
L01F7	EQU	$01F7			;A/D O2, one delayed
L01F8	EQU	$01F8			;A/D O2 voltage
L01F9	EQU	$01F9			;CCP duty cycle
L01FA	EQU	$01FA			;Closed loop base CCP duty cycle multiplier
L01FB	EQU	$01FB			;CCP update timer
L01FC	EQU	$01FC			;% TPS
L01FD	EQU	$01FD			;%TPS, one delayed
L01FE	EQU	$01FE			;TPS, one delayed
L01FF	EQU	$01FF			;AE TPS, filtered per cool temp, MSB
L0200	EQU	$0200			;AE TPS, filtered per cool temp, LSB
L0201	EQU	$0201			;MSB, slow filtered %TPS
L0202	EQU	$0202			;LSB, slow filtered %TPS
L0203	EQU	$0203			;TPS threshold for WOT
L0204	EQU	$0204			;Fast PE delay timer countdown rate
L0205	EQU	$0205			;TPS threshold for PE
L0206	EQU	$0206			;PE TPS baro factor
L0207	EQU	$0207			;Delta TPS AE baro factor
L0208	EQU	$0208			;AE TPS filter coeff. vs. cool temp
L0209	EQU	$0209			;MSB, DRP, two delayed
L020A	EQU	$020A			;LSB, DRP, two delayed
L020B	EQU	$020B			;MSB, DRP period
L020C	EQU	$020C			;LSB, DRP period
L020D	EQU	$020D			;Dynamic dwell
L020F	EQU	$020F			;MSB, outputted dwell
L0210	EQU	$0210			;LSB, outputted dwell
L0211	EQU	$0211			;MSB, total SA
L0212	EQU	$0212			;LSB, total SA
L0213	EQU	$0213			;
L0214	EQU	$0214			;
L0215	EQU	$0215			;MSB, SA ramping multiplier
L0216	EQU	$0216			;LSB, SA ramping multiplier
L0217	EQU	$0217			;MSB, Filtered EGR SA correction
L0218	EQU	$0218			;LSB, Filtered EGR SA correction
L0219	EQU	$0219			;MSB, commanded SA
L021A	EQU	$021A			;LSB, commanded SA
L021B	EQU	$021B			;SA from table lookup
L021C	EQU	$021C			;Cool temp SA correction
L021D	EQU	$021D			;MAT temp for look up
L021E	EQU	$021E			;Altitude SA correction
L021F	EQU	$021F			;PE SA adder
L0220	EQU	$0220			;MSB, final SA
L0221	EQU	$0221			;LSB, final SA
L0222	EQU	$0222			;Startup SA decay multiplier
L0223	EQU	$0223			;Initial added SA decay period
L0224	EQU	$0224			;SA latency correction
L0225	EQU	$0225			;DRP period counter
L0226	EQU	$0226			;MSB, knock counts, one delayed
L0227	EQU	$0227			;LSB, knock counts, one delayed
L0228	EQU	$0228			;MSB, EST circuit info
L0229	EQU	$0229			;LSB, EST circuit info
L022A	EQU	$022A			;Error 43 timer
L022B	EQU	$022B			;DFCO SA correction
L022C	EQU	$022C			;Low octaine SA ramping multiplier
L022D	EQU	$022D			;Low octaine SA retard
L022E	EQU	$022E			;Low octane base SA retard
L022F	EQU	$022F			;Knock retard x 2
L0230	EQU	$0230			;Knock counts
L0231	EQU	$0231			;Burst retard timer
L0232	EQU	$0232			;INTegrator
L0233	EQU	$0233			;DRP counter, used for setting engine running flag
L0234	EQU	$0234			;Transition from crank to run delay DRP counter	
L0235	EQU	$0235			;
L0236	EQU	$0236			;
L0237	EQU	$0237			;10's portion of numerical error code to blink out
L0238	EQU	$0238			;1's portion of numerical error code to blink out
L0239	EQU	$0239			;Current error code to blink through SES
L023A	EQU	$023A			;Set errors, SES code display logic
L023B	EQU	$023B			;SES blink timer
L023C	EQU	$023C			;Error 44 timer
L023D	EQU	$023D			;MSB, error 45 timer
L023E	EQU	$023E			;LSB, error 45 timer
L023F	EQU	$023F			;Error 13 timer
L0240	EQU	$0240			;Error 23 counter
L0241	EQU	$0241			;Error 25 counter
L0242	EQU	$0242			;EGR diag. cycling timer
L0243	EQU	$0243			;Delay timer for EGR diags.
L0244	EQU	$0244			;EGR INT diags. timer
L0245	EQU	$0245			;Stored INT reference for EGR diags.
L0246	EQU	$0246			;EGR fail timer
L0247	EQU	$0247			;Pintle position error diag. failure timer
L0248	EQU	$0248			;Linear EGR diag. cycle timer
L0249	EQU	$0249			;
L024A	EQU	$024A			;Linear EGR position low error timer
L024B	EQU	$024B			;Error 33 timer
L024C   EQU	$024C           	;Err. 34 timer
L024D	EQU	$024D			;Err. 54 timer
L024E	EQU	$024E			;Error log timer
L024F	EQU	$024F			;
L0250	EQU	$0250			;
L0251	EQU	$0251			;
L0252	EQU	$0252			;MAT temp
L0253	EQU	$0253			;A/D MAT
L0254	EQU	$0254			;
L0255	EQU	$0255			;
L0256	EQU	$0256			;
L0257	EQU	$0257			;Mass of air in cylinder, MSB
L0258	EQU	$0258			;Mass of air in cylinder, LSB
L0259	EQU	$0259			;Inv. air density, MSB
L025A	EQU	$025A			;Inv. air density, LSB
L025B	EQU	$025B			;Max mass of air in cyl., MSB
L025C	EQU	$025C			;Max mass of air in cyl., LSB
L025D	EQU	$025D			;Async pulse time, MSB
L025E	EQU	$025E			;Async pulse time, LSB
L025F	EQU	$025F			;AE temperature correction factor
L0260	EQU	$0260			;AE temp correction mult.
L0261	EQU	$0261			;MAP AE PW
L0262	EQU	$0262			;TPS AE PW
L0263	EQU	$0263			;Filtering execution timer
L0264	EQU	$0264			;O2 dwell timer
L0265	EQU	$0265			;Decel enlean cool factor
L0266	EQU	$0266			;Min DFCO rpm
L0267	EQU	$0267			;DFCO BPW multiplier
L0268	EQU	$0268			;DFCO exit PW timer
L0269	EQU	$0269			;DFCO delay timer
L026A	EQU	$026A			;Current BLM cell number
L026B	EQU	$026B			;Current BLM 
L026C	EQU	$026C			;BLM, one delayed
L026D	EQU	$026D			;AFR
L026E	EQU	$026E			;Stored AFR
L026F	EQU	$026F			;MSB, BPW
L0270	EQU	$0270			;LSB, BPW
L0271	EQU	$0271			;Sync BPW, MSB
L0272	EQU	$0272			;Sync BPS, LSB
L0273	EQU	$0273			;MSB, TOC 5 value
L0274	EQU	$0274			;LSB, TOC 4 value
L0275	EQU	$0275			;EGR BPW correction, MSB
L0276	EQU	$0276			;EGR BPW correction, LSB
L0277	EQU	$0277			;MSB, BPW, CPI/PFI mode
L0278	EQU	$0278			;LSB, BPW, CPI/PFI mode
L0279	EQU	$0279			;PW compensation for batt. volts
L027A	EQU	$027A			;MSB, kPa x 256
L027B	EQU	$027B			;LSB, kPa x 256
L027C	EQU	$027C			;Accumulated asnyc BPW, MSB
L027D	EQU	$027D			;Accumulated async BPW, LSB
L027E	EQU	$027E			;Accumulated async BPW, remainder
L027F	EQU	$027F			;Asnyc BPW, MSB
L0280	EQU	$0280			;Async BPW, LSB
L0281	EQU	$0281			;Async BPW, stored remainder
L0282	EQU	$0282			;INT timer
L0283	EQU	$0283			;MSB, time since last DRP
L0284	EQU	$0284			;LSB. time since last DRP
L0285	EQU	$0285			;INT delay term for idling
L0286	EQU	$0286			;delta speed PE entry timer
L0287	EQU	$0287			;MSB, filtered AFR
L0288	EQU	$0288			;LSB, filtered AFR
L0289	EQU	$0289			;Time to filtere AFR on idle to run
L028A	EQU	$028A			;AFR crank transition ratio decay rate
L028B	EQU	$028B			;Elapsed DRPs
L028C	EQU	$028C			;PE timer
L028D	EQU	$028D			;Fuel cut off delay timer
L028E	EQU	$028E			;INT delay timer term
L028F	EQU	$028F			;Mean rich/lean O2 threshold
L0290	EQU	$0290			;Rich O2 threshold
L0291	EQU	$0291			;Lean O2 threshold
L0292	EQU	$0292			;Closed loop fuel correction term
L0293	EQU	$0293			;Choke AFR decay delay period
L0294	EQU	$0294			;Idle timer
L0295	EQU	$0295			;Open loop idle AFR 
L0296	EQU	$0296			;BLM update delay timer
L0297	EQU	$0297			;BLM loop timer
L0298	EQU	$0298			;O2 cross-counts
L0299	EQU	$0299			;PE reset/delay timer
L029A	EQU	$029A			;PE delay timer
L029B	EQU	$029B			;PE ramping multiplier
L029C	EQU	$029C			;Time in PE, AIR inj. management
L029D	EQU	$029D			;Proportional term
L029E	EQU	$029E			;Scaled airflow
L029F	EQU	$029F			;Duration in current fueling state
L02A0	EQU	$02A0			;Cold park to drive transition period
L02A1	EQU	$02A1			;MSB, shutdown timer
L02A2	EQU	$02A2			;LSB, shutdown timer
L02A3	EQU	$02A3			;O2 error
L02A4	EQU	$02A4			;Stored INT delay
L02A5	EQU	$02A5			;Initial cool temp (engine off)
L02A6	EQU	$02A6			;Range limited cool temp
L02A7	EQU	$02A7			;MSB, MPH, filtered
L02A8	EQU	$02A8			;LSB, MPH, filtered
L02A9	EQU	$02A9			;MSB, governor TPS
L02AA	EQU	$02AA			;LSB, governor TPS
L02AB	EQU	$02AB			;MSB, Gov'r actuator integral term
L02AC	EQU	$02AC			;LSB, Gov'r actuator integral term
L02AD	EQU	$02AD			;Governor integral term
L02AF	EQU	$02AF			;Previous governor RPM/25
L02B0	EQU	$02B0			;Delta governor RPM/25
L02B1	EQU	$02B1			;Governor RPM/25
L02B2	EQU	$02B2			;
L02B3	EQU	$02B3			;MSB, RPM prop. term
L02B4	EQU	$02B4			;LSB, RPM prop. term
L02B5	EQU	$02B5			;MPH prop. term
L02B6	EQU	$02B6			;Governor RPM error
L02B7	EQU	$02B7			;Governor MPH error
L02B8	EQU	$02B8			;MSB, gov'r actuator prop. term
L02B9	EQU	$02B9			;LSB, gov'r actuator prop. term
L02BA	EQU	$02BA			;MSB, Gov'r actuator integral factor
L02BB	EQU	$02BB			;LSB, Gov'r actuator integral factor
L02BC	EQU	$02BC			;Desired outputted TPS, governor
L02BD	EQU	$02BD			;Gov'r actuator TPS error
L02BE	EQU	$02BE			;MSB, pending RAM addr. block to be refreshed
L02BF	EQU	$02BF			;LSB, pending RAM addr. block to be refreshed
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Idle integral cells, P/N
;~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;-A/C off
;
L02C0	EQU	$02C0			;MSB, current idle integral term
L02C1	EQU	$02C1			;LSB, current idle integral term
L02C2	EQU	$02C2			;Steady state idle integral term
;
;-A/C on
;
L02C3	EQU	$02C3			;MSB, current idle integral term
L02C4	EQU	$02C4			;LSB, current idle integral term
L02C5	EQU	$02C5			;Steady state idle integral term
;
L02C6	EQU	$02C6			;MSB, coupled IAC flow vs. cool temp
L02C7	EQU	$02C7			;LSB, coupled IAC flow vs. cool temp
L02C8	EQU	$02C8			;MSB, IAC flow vs. cool temp
L02C9	EQU	$02C9			;LSB, IAC flow vs. cool temp 	
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Idle integral cells, drive
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;-A/C off
;
L02CA	EQU	$02CA			;MSB, current idle integral term
L02CB	EQU	$02CB			;LSB, current idle integral term
L02CC	EQU	$02CC			;Steady state idle integral term
;
;-A/C on
;
L02CD	EQU	$02CD			;MSB, current idle integral term
L02CE	EQU	$02CE			;LSB, current idle integral term
L02CF	EQU	$02CF			;Steady state idle integral term
;
L02D0	EQU	$02D0			;MSB, coupled IAC flow vs. cool temp
L02D1	EQU	$02D1			;LSB, coupled IAC flow vs. cool temp
L02D2	EQU	$02D2			;MSB, IAC flow vs. cool temp
L02D3	EQU	$02D3			;LSB, IAC flow vs. cool temp 
;
;-End of integrator cells
;
L02D4	EQU	$02D4			;Throttle kicker base TPS
L02D5	EQU	$02D5			;Post throttle kicker base TPS
;
;~~~~~~~~~~~
;-BLM cells
;~~~~~~~~~~~
;
L02D6	EQU	$02D6			;BLM cell #0
L02D7	EQU	$02D7			;BLM cell #1
L02D8	EQU	$02D8			;BLM cell #2
L02D9	EQU	$02D9			;BLM cell #3
L02DA	EQU	$02DA			;BLM cell #4
L02DB	EQU	$02DB			;BLM cell #5
L02DC	EQU	$02DC			;BLM cell #6
L02DD	EQU	$02DD			;BLM cell #7
L02DE	EQU	$02DE			;BLM cell #8
L02DF	EQU	$02DF			;BLM cell #9
L02E0	EQU	$02E0			;BLM cell #10
L02E1	EQU	$02E1			;BLM cell #11
L02E2	EQU	$02E2			;BLM cell #12
L02E3	EQU	$02E3			;BLM cell #13
L02E4	EQU	$02E4			;BLM cell #14
L02E5	EQU	$02E5			;BLM cell #15
;
;-Start of BLM idle cells
;
L02E6	EQU	$02E6			;BLM cell #16
L02E7	EQU	$02E7			;BLM cell #17
L02E8	EQU	$02E8			;BLM cell #18
L02E9	EQU	$02E9			;BLM cell #19
L02EA	EQU	$02EA			;BLM cell #20
;
;-End of BLM cells
;
L02EB	EQU	$02EB			;Stored choke AFR
L02EC	EQU	$02EC			;AFR crank trans. ratio
L02ED	EQU	$02ED			;Choke AFR decay delay timer
L02EE	EQU	$02EE			;Initial SA
L02EF	EQU	$02EF			;Initial SA decay delay timer
L02F0	EQU	$02F0			;DRP counter
L02F1	EQU	$02F1			;Stored DRP counts
L02F2	EQU	$02F2			;Crank fuel PW, MSB
L02F3	EQU	$02F3			;Crank fuel PW, LSB
L02F4	EQU	$02F4			;
L02F5	EQU	$02F5			;
L02F6	EQU	$02F6			;
L02F7	EQU	$02F7			;
L02F8	EQU	$02F8			;
L02F9	EQU	$02F9			;
L02FA	EQU	$02FA			;
L02FB	EQU	$02FB			;
L02FC	EQU	$02FC			;
L02FD	EQU	$02FD			;
L02FE	EQU	$02FE			;
L02FF	EQU	$02FF			;
L0300	EQU	$0300			;
L0301	EQU	$0301			;
L0302	EQU	$0302			;
L0303	EQU	$0303			;
L0304	EQU	$0304			;
L0305	EQU	$0305			;
L0306	EQU	$0306			;
L0307	EQU	$0307			;
L0308	EQU	$0308			;
L0309	EQU	$0309			;
L030A	EQU	$030A			;
L030B	EQU	$030B			;
L030C	EQU	$030C			;
L030D	EQU	$030D			;
L030E	EQU	$030E			;
L030F	EQU	$030F			;
L0310	EQU	$0310			;
L0311	EQU	$0311			;
L0312	EQU	$0312			;
L0313	EQU	$0313			;
L0314	EQU	$0314			;
L0315	EQU	$0315			;
L0316	EQU	$0316			;Baro
L0317	EQU	$0317			;Filtered baro, MSB
L0318	EQU	$0318			;Filtered baro, LSB
L0319	EQU	$0319			;Base TPS for closed throttle, MSB
L031A	EQU	$031A			;Base TPS for closed throttle, LSB
;
;~~~~~~~~~~~~~~~~~~~~~~~~~
; Adaptive modifier cells 
;~~~~~~~~~~~~~~~~~~~~~~~~~
;
;-1 -> 2 shift
;
L031B	EQU	$031B			;Cell #0
L031C	EQU	$031C			;Cell #1
L031D	EQU	$031D			;Cell #2
L031E	EQU	$031E			;Cell #3
;
;-2 -> 3 shift
;
L031F	EQU	$031F			;Cell #0
L0320	EQU	$0320			;Cell #1
L0321	EQU	$0321			;Cell #2
L0322	EQU	$0322			;Cell #3
;
;-3 -> 4 shift
;
L0323	EQU	$0323			;Cell #0
L0324	EQU	$0324			;Cell #1
L0325	EQU	$0325			;Cell #2
L0326	EQU	$0326			;Cell #3
;
L0327	EQU	$0327			;Net change in adaptive modifier
L0328	EQU	$0328			;			
L0329	EQU	$0329			;
L032A	EQU	$032A			;
L032B	EQU	$032B			;
L032C	EQU	$032C			;
L032D	EQU	$032D			;
L032E	EQU	$032E			;
L032F	EQU	$032F			;
L0330	EQU	$0330			;
L0331	EQU	$0331			;
L0332	EQU	$0332			;
L0333	EQU	$0333			;
L0334	EQU	$0334			;
L0335	EQU	$0335			;
L0336	EQU	$0336			;
L0337	EQU	$0337			;
L0338	EQU	$0338			;
L0339	EQU	$0339			;
L033A	EQU	$033A			;
L033B	EQU	$033B			;
L033C	EQU	$033C			;
L033D	EQU	$033D			;
L033E	EQU	$033E			;
L033F	EQU	$033F			;
L0340	EQU	$0340			;
L0341	EQU	$0341			;
L0342	EQU	$0342			;
L0343	EQU	$0343			;
L0344	EQU	$0344			;
L0345	EQU	$0345			;
L0346	EQU	$0346			;
L0347	EQU	$0347			;
L0348	EQU	$0348			;
L0349	EQU	$0349			;
L034A	EQU	$034A			;
L034B	EQU	$034B			;
L034C	EQU	$034C			;
L034D	EQU	$034D			;
L034E	EQU	$034E			;
L034F	EQU	$034F			;
L0350	EQU	$0350			;
L0351	EQU	$0351			;
L0352	EQU	$0352			;
L0353	EQU	$0353			;
L0354	EQU	$0354			;
L0355	EQU	$0355			;
L0356	EQU	$0356			;
L0357	EQU	$0357			;
L0358	EQU	$0358			;
L0359	EQU	$0359			;
L035A	EQU	$035A			;
L035B	EQU	$035B			;
L035C	EQU	$035C			;
L035D	EQU	$035D			;
L035E	EQU	$035E			;
L035F	EQU	$035F			;
L0360	EQU	$0360			;
L0361	EQU	$0361			;
L0362	EQU	$0362			;ALDL message length counter
L0363	EQU	$0363			;ALDL message checksum
L0364	EQU	$0364			;Pointer to ALDL transmission string
L0365	EQU	$0365			;
L0366	EQU	$0366			;ALDL diag. ID
L0367	EQU	$0367			;ALDL message length
L0368	EQU	$0368			;Stored LSB for double byte ALDL transmission
L0369	EQU	$0369			;ALDL inactivity timer
L036A	EQU	$036A			;
L036B	EQU	$036B			;
L036C	EQU	$036C			;ALDL mode 4 re-enable delay timer
L036D	EQU	$036D			;
L036E	EQU	$036E			;ALDL data buffer
					;^
					;...
					;^
					;^
					;^
					;End of ALDL data buffer
;
L0390	EQU	$0390			;Engine diagnostics data buffer	
L0391	EQU	$0391			;...
L0392	EQU	$0392			;^
L0393	EQU	$0393			;^
L0394	EQU	$0394			;^
L0395	EQU	$0395			;^
L0396	EQU	$0396			;^
L0397	EQU	$0397			;^
L0398	EQU	$0398			;^
L0399	EQU	$0399			;^
L039A	EQU	$039A			;End of engine diag. data buffer
;
L039B	EQU	$039B			;Transmission diagnostics data buffer
L039C	EQU	$039C			;^
L039D	EQU	$039D			;...
L039E	EQU	$039E			;^
L039F	EQU	$039F			;^
L03A0	EQU	$03A0			;^
L03A1	EQU	$03A1			;^
L03A2	EQU	$03A2			;^
L03A3	EQU	$03A3			;^
L03A4	EQU	$03A4			;^
L03A5	EQU	$03A5			;End of transmission diag. data buffer
;
L03B2	EQU	$03B2			;End of stack/check byte
					;^
					;...
					;^
					;^
					;^
L03FF	EQU	$03FF			;Start of stack
;
;~~~~~~~~~~~~~~~~~~~~~~
; HU RAM (ALDL mode 4)
;~~~~~~~~~~~~~~~~~~~~~~
;
L0400	EQU	$0400			;
L0401	EQU	$0401			;
L0402	EQU	$0402			;
L0403	EQU	$0403			;
L0404	EQU	$0404			;
L0405	EQU	$0405			;
L0406	EQU	$0406			;
L0407	EQU	$0407			;
L0408	EQU	$0408			;
L0409	EQU	$0409			;
L040A	EQU	$040A			;
L040B	EQU	$040B			;
L040C	EQU	$040C			;
L040D	EQU	$040D			;
L040E	EQU	$040E			;
L040F	EQU	$040F			;
L0410	EQU	$0410			;
L0411	EQU	$0410			;
L0412	EQU	$0410			;
L0413	EQU	$0410			;
L0414	EQU	$0414			;
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Start of extended RAM addresses
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;
L0800	EQU	$0800			;
L0801	EQU	$0801			;
L0802	EQU	$0802			;
L0803	EQU	$0803			;
L0804	EQU	$0804			;
L0805	EQU	$0805			;
L0806	EQU	$0806			;
L0807	EQU	$0807			;
L0808	EQU	$0808			;
L0809	EQU	$0809			;
L080A	EQU	$080A			;
L080B	EQU	$080B			;
L080C	EQU	$080C			;# of VSS pulses recieved since last update
L080D	EQU	$080D			;MSB, VSS counter
L080E	EQU	$080E			;LSB, VSS counter
L080F	EQU	$080F			;MSB, VSS counter, one delayed
L0810	EQU	$0810			;LSB, VSS counter, one delayed
L0811	EQU	$0811			;VSS pulse timer
L0812	EQU	$0812			;MSB, current MPH
L0813	EQU	$0813			;LSB, current MPH
L0814	EQU	$0814			;MSB, stored MPH
L0815	EQU	$0815			;LSB, stored MPH
L0816	EQU	$0816			;
L0817	EQU	$0817			;Error 16 timer
L0818	EQU	$0818			;Delta MPH for PE entry
L0819	EQU	$0819			;
L081A	EQU	$081A			;MPH, one delayed
L081B	EQU	$081B			;
L081C	EQU	$081C			;Error 24 timer
L081D	EQU	$081D			;INT lock timer, AE
L081E	EQU	$081E			;Exec. delay timer
L081F	EQU	$081F			;MSB, time based delay for inj. firing
L0820	EQU	$0820			;LSB, time based delay for inj. firing 
L0821	EQU	$0821			;MSB, TOC 5 timer val.
L0822	EQU	$0822			;LSB, TOC 5 timer val.
L0823	EQU	$0823			;MSB, TOC 4 timer val.
L0824	EQU	$0824			;LSB, TOC 4 timer val.
L0825	EQU	$0825			;MSB, TOC 5 timer val.
L0826	EQU	$0826			;LSB, TOC 5 timer val.
L0827	EQU	$0827			;MSB, TOC 4 timer val.
L0828	EQU	$0828			;LSB, TOC 4 timer val.
L0829	EQU	$0829			;DRP counter
L082A	EQU	$082A			;Time in double fire mode 
L082B	EQU	$082B			;Delta TPS for decel enlean
L082C	EQU	$082C			;Initial AFR enrichment timer
L082D	EQU	$082D			;A/D coolant
L082E	EQU	$082E			;A/D MAP, A/D conv.
L082F	EQU	$082F			;ALDL switch voltage
L0830	EQU	$0830			;Enable A/C timer
L0831	EQU	$0831			;Re-enable A/C timer
L0832	EQU	$0832			;High rpm A/C disable timer
L0833	EQU	$0833			;High rpm A/C re-enable timer
L0834	EQU	$0834			;High TPS A/C disable timer
L0835	EQU	$0835			;
L0836	EQU	$0836			;
L0837	EQU	$0837			;Stored RPM/12.5
L0838	EQU	$0838			;MSB, stored RPM/12.5, filtered
L0839	EQU	$0839			;LSB, stored RPM/12.5, filtered
L083A	EQU	$083A			;RPM error/12.5
L083B	EQU	$083B			;Stored SA derivative term for manual trans
L083C	EQU	$083C			;Final MAT SA correction
L083D	EQU	$083D			;MSB, stored RPM/25
L083E	EQU	$083E			;LSB, stored RPM/25
L083F	EQU	$083F			;MSB, stored static dwell
L0840	EQU	$0840			;LSB, stored static dwell
L0841	EQU	$0841			;Low octane RPM mult.
L0842	EQU	$0842			;Burst knock retard
L0843	EQU	$0843			;Knock attack rate
L0844	EQU	$0844			;Max knock retard
L0845	EQU	$0845			;Gear # (S/L)
L0846	EQU	$0846			;S/L activation delay timer
L0847	EQU	$0847			;MSB, stored pulse counts from last pass (S/L)
L0848	EQU	$0848			;LSB, stored pulse counts from last pass (S/L) 
L0849	EQU	$0849			;MSB, delta pulse counts (S/L)
L084A	EQU	$084A			;LSB, delta pulse counts (S/L)
L084B	EQU	$084B			;Engine trans RPM ratio (S/L)
L084C	EQU	$084C			;Cat converter overtemp timer
L084D	EQU	$084D			;Trans abuse test timer
L084E	EQU	$084E			;Err. 41 counter
L084F	EQU	$084F			;MSB, EGR delta integral term
L0850	EQU	$0850			;LSB, EGR delta integral term
L0851	EQU	$0851			;MSB, TIC 2 timer val (VATS)
L0852	EQU	$0852			;LSB, TIC 2 timer val (VATS)
L0853	EQU	$0853			;TCC relock delay timer
L0854	EQU	$0854			;
L0855	EQU	$0855			;MAT, linearized A/D output
L0856	EQU	$0856			;Error 46 timer
L0857	EQU	$0857			;Desired idle RPMs
L0858	EQU	$0858			;Idle RPM error
L0859	EQU	$0859			;MSB, filtered idle RPM error
L085A	EQU	$085A			;LSB, filtered idle RPM error			
L085B	EQU	$085B			;Idle purge timer
L085C	EQU	$085C			;Added desired idle speed
L085D	EQU	$085D			;Decay period for added idle speed
L085E	EQU	$085E			;Timer to begin decay of added desired idle speed
L085F	EQU	$085F			;Filtered desired idle speed
L0860	EQU	$0860			;Added RPMs for high A/C pressure
L0861	EQU	$0861			;High pressure A/C compensation delay timer
L0862	EQU	$0862			;MSB, stored idle integrator flow
L0863	EQU	$0863			;LSB, stored idle integrator flow
L0864	EQU	$0864			;Closed loop idle enable delay timer
L0865	EQU	$0865			;
L0866	EQU	$0866			;Closed loop idle delay, low idle
L0867	EQU	$0867			;Closed loop idle delay, high idle
L0868	EQU	$0868			;Stall saver activation threshold
L0869	EQU	$0869			;Stall saver duration timer
L086A	EQU	$086A			;Stored steady state idle integral term
L086B	EQU	$086B			;Added initial IAC flow after startup
L086C	EQU	$086C			;Decay period for initial added IAC flow
L086D	EQU	$086D			;Proportional idle term
L086E	EQU	$086E			;MSB, derivative idle term
L086F	EQU	$086F			;LSB, derivative idle term
L0870	EQU	$0870			;Final derivative idle flow term
L0871	EQU	$0871			;MSB, added coupled IAC flow vs. cool temp
L0872	EQU	$0872			;LSB, added coupled IAC flow vs. cool temp
L0873	EQU	$0873			;MSB, stored IAC flow vs. cool temp
L0874	EQU	$0874			;LSB, stored IAC flow vs. cool temp
L0875	EQU	$0875			;IAC flow decay delay timer
L0876	EQU	$0876			;Added IAC flow for A/C on/off transition
L0877	EQU	$0877			;A/C on/off transition timer
L0878	EQU	$0878			;A/C on/off transition flow decay period
L0879	EQU	$0879			;Added IAC flow for trans. shift
L087A	EQU	$087A			;Trans shift transition period
L087B	EQU	$087B			;Added power steering compensation flow
L087C	EQU	$087C			;P/S compensation IAC flow decay period
L087D	EQU	$087D			;MSB, IAC flow, one delayed
L087E	EQU	$087E			;LSB, IAC flow, one delayed
L087F	EQU	$087F			;IAC accel enrich
L0880	EQU	$0880			;MSB, added throttle follower IAC flow
L0881	EQU	$0881			;LSB, added throttle follower IAC flow
L0882	EQU	$0882			;MSB, hi res filtered TPS
L0883	EQU	$0883			;LSB, hi res filtered TPS
L0884	EQU	$0884			;TPS filtering period
L0885	EQU	$0885			;Stored upper limit for closed TPS
L0886	EQU	$0886			;Timer for reducing T/F action during clsd. throt.
L0887	EQU	$0887			;Added IAC flow when in DFCO
L0888	EQU	$0888			;DFCO flow decay rate 
L0889	EQU	$0889			;MSB, finalized IAC flow
L088A	EQU	$088A			;LSB, finalized IAC flow
L088B	EQU	$088B			;Desired IAC airflow, percentage of total flow
L088D	EQU	$088D			;Throttle kicker timer
L088E	EQU	$088E			;Error 35 timer
;
;-Updated idle integral flow, P/N
;
L088F	EQU	$088F			;MSB, updated integral flow, no A/C
L0890	EQU	$0890			;LSB, updated integral flow, no A/C
L0891	EQU	$0891			;MSB, updated integral flow, A/C
L0892	EQU	$0892			;LSB, updated integral flow, A/C
L0893	EQU	$0893			;MSB, Updated initial IAC flow vs. cool temp
L0894	EQU	$0894			;LSB, Updated initial IAC flow vs. cool temp
;
;-Updated idle integral flow, drive
;
L0895	EQU	$0895			;MSB, updated integral flow, no A/C
L0896	EQU	$0896			;LSB, updated integral flow, no A/C
L0897	EQU	$0897			;MSB, updated integral flow, A/C
L0898	EQU	$0898			;LSB, updated integral flow, A/C
L0899	EQU	$0899			;MSB, Updated initial IAC flow vs. cool temp
L089A	EQU	$089A			;LSB, Updated initial IAC flow vs. cool temp
;
L0F00	EQU	$0F00			;
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; Location of ALDL mode 4 jump table and code in PROM
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;-Not present in this calibration, dummy RTS table provided for this disassembly
;
	ORG	$1800		
;
L1800	NOP				;
	NOP				;
	RTS				;Return
;
L1803	NOP				;
	NOP				;
	RTS				;Return
;
L1806	NOP				;
	NOP				;
	RTS				;Return
;	
L1809	NOP				;
	NOP				;
	RTS				;Return
;
L180C	NOP				;
	NOP				;
	RTS				;Return
;
L180F	NOP				;
	NOP				;
	RTS				;Return
;
L1812	NOP				;
	NOP				;
	RTS				;Return
;
L1815	NOP				;
	NOP				;
	RTS				;
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; Registers
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~
;
L3000	EQU	$3000			;Port A/Start of CPU regs.
L3002	EQU	$3002			;
L3008	EQU	$3008			;Port D
					; b7=
					; b6=
					; b5=
					; b4=
					; b3=
					; b2=A/C pressure switch
					; b1=
					; b0=
;
L3010	EQU	$3010			;MSB, 131 kHz free running TIS counter
L3011	EQU	$3011			;LSB, 131 kHz free running TIS counter
L300E	EQU	$300E			;MSB, 16 bit free running CPU counter
L300F	EQU	$300F			;LSB, 16 bit free running CPU counter
L3012	EQU	$3012			;TIC 2
L3016	EQU	$3016			;TOC 1
L301A	EQU	$301A			;
L3022	EQU	$3022			;
L3023	EQU	$3023			;
L3024	EQU	$3024			;
L3025	EQU	$3025			;
L3027	EQU	$3027			;TIS pulse accumulator
L302D	EQU	$302D			;SCI control register 2
L302E	EQU	$302E			;SCI status register
L302F	EQU	$302F			;SCI data register
L3030	EQU	$3030			;A/D Control Reg
L3031	EQU	$3031			;A/D result reg 1
L3032	EQU	$3032			;A/D result reg 2
L3033	EQU	$3033			;A/D result reg 3
L3034	EQU	$3034			;A/D result reg 4
L3039	EQU	$3039			;
L303A	EQU	$303A			;
L303F	EQU	$303F			;
L3060	EQU	$3060			;I/O
					; b7=TCC/Throttle kicker
					; b6=
					; b5=
					; b4=
					; b3=
					; b2=
					; b1=2 -> 3 shift solenoid feedback (+12V = 1)
					; b0=1 -> 2 shift solenoid feedback (+12V = 1)
;	
L3061	EQU	$3061			;
L3062	EQU	$3062			;I/O port D?
L3063	EQU	$3063			;
L3064	EQU	$3064			;I/O port C
L3065	EQU	$3065			;
L3066	EQU	$3066			;
L3067	EQU	$3067			;
L3068	EQU	$3068			;
L306A	EQU	$306A			;PWM TCC output
L306C	EQU	$306C			;3->2 sol./shift light/governor
L306E	EQU	$306E			;
L306F	EQU	$306F			;Hardware, fuel pump relay control
L30BC	EQU	$30BC			;
L30C0	EQU	$30C0			;
L30C6	EQU	$30C6			;
L30E5	EQU	$30E5			;
L30F3	EQU	$30F3			;
L30F7	EQU	$30F7			;
L30F8	EQU	$30F8			;
L319D	EQU	$319D			;
L319E	EQU	$319E			;
L3211	EQU	$3211			;
L3213	EQU	$3213			;
L3124	EQU	$3124			;
L3224	EQU	$3224			;
L3244	EQU	$3244			;
L3226	EQU	$3226			;
L326D	EQU	$326D			;
L326F	EQU	$326F			;
L3FC0	EQU	$3FC0			;DRP counter
L3FC2	EQU	$3FC2			;65kHz free running TOS counter		
L3FC4	EQU	$3FC4			;VSS counter
L3FC6	EQU	$3FC6			;alternate TOS pulse accumulator
L3FC8	EQU	$3FC8			;
L3FCA	EQU	$3FCA			;EST circuit info
L3FCC	EQU	$3FCC			;
L3FCE	EQU	$3FCE			;Reg. for TBI injector hardware
L3FD4	EQU	$3FD4			;
L3FD6	EQU	$3FD6			;
L3FD8	EQU	$3FD8			;
L3FDA	EQU	$3FDA			;
L3FDC	EQU	$3FDC			;
L3FDE	EQU	$3FDE			;
L3FE0	EQU	$3FE0			;TOS pulse accumulator
L3FE4	EQU	$3FE4			;
L3FE5	EQU	$3FE5			;
L3FE6	EQU	$3FE6			;
L3FE7	EQU	$3FE7			;
L3FE8	EQU	$3FE8			;
L3FE9	EQU	$3FE9			;
L3FEC	EQU	$3FEC			;
L3FED	EQU	$3FED			;
L3FF2	EQU	$3FF2			;
L3FF6	EQu	$3FF6			;EST fall counter
L3FF8	EQU	$3FF8			;Alternate 65 kHz free running TOS counter
L3FFA	EQU	$3FFA			;CPU stat. reg.
L3FFC	EQU	$3FFC			;I/O port D?
L3FEA	EQU	$3FEA			;
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; Start of calibration
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
	ORG $4000		
;
L4000	FDB $23AF		;ID 
;
L4002   FDB $FFFF		;Platform ID  
L4004   FDB $FFFF		;Platform ID   
;
L4006   FDB $3087		;Check sum
L4008	FCB $0D			;Mask ID byte   
				;
L4009   FDB 0481		;6 secs. delay till shutdown
;         
;-AIR FUEL MODE WORDS
;
L400B	FCB $0C			;0000 1100, option byte 
				; b7, 1 = Skip delta TPS check when already in decel enlean
                    		; b6, 1 = MAT sensor
                    		; b5, 1 = 180 deg offset
                    		; b4, 1 = ASDF crank
                    		; b3, 1 = Limit closed loop corr. when AE in effect  
                    		; b2, 1 = Syncronous fuel at idle (TBI)
                    		; b1, 1 = Enable AIR
                    		; b0, 1 = CPI/PFI MODE
;	
L400C	FCB $93			;1001 0011, option byte  
                    		; b7, 1 = Enable CCP purge
                    		; b6, 1 = Contitional INT reset on BLM cell change
                    		; b5, 1 = INT reset on AE
                    		; b4, 1 = INT reset in BLM cell change
                    		; b3, 1 = ASDF 
                    		; b2, 1 = CRANK FUEL ALL INJ'S EACH DRP
                    		; b1, 1 = ERR 44/45 BLM LMT
				; b0, 1 = SYNC MAP SENSOR READS
;  
L400D	FCB $A4			;1010 0100, option byte  
                    		; b7, 1 = SINGLE PASS EGR TEST
                    		; b6, 1 = VATS
                    		; b5, 1 = Ese table to look up EGR flow ratio
                    		; b4, 1 = EGR = 0 AT IDLE
                    		; b3, 1 = Disable EGR in open loop
				; b2, 1 = BACK PRESSURE EGR
                    		; b1, 1 = linear EGR, 
				;     0 = Electrinic Vacuum Regulator Valve EGR
                    		; b0, 1 = look up AFR when in open loop
;
L400E   FCB $00			;0000 0000, option byte
				; b7 1 = not used
				; b6 1 = not used
				; b5 1 = LATCH ERR 45
				; b4 1 = USE L496F WITH ASYNC FUEL DELIVERY
				; b3 1 = VAR TUNE CNT'L
				; b2 1 = SHIFT LIGHT ENABLE 
				; b1 1 = Use MAP/Baro ratio for DFCO
				; b0 1 = Use MAP/Baro ratio for BLM cell bounds
; 
L400F	FCB $18			;Option byte, 0001 1000
				; b7=TCC option (1 = MAN, 0 = TCC)
				; b6=TCC only (Non-CC transmission)
                    		; b5= 
                    		; b4=Enable Cat. reaction overtemp protection
                    		; b3=Use knock retard
                    		; b2, 1 = A/C clutch cont., 0 = Var tune
                    		; b1=
                    		; b0, 1=Rev limiter                    
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Adaptive A/C clutch control params
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; All timer values are seconds x 10
;
L4010   FCB  0			;Upper TPS threshold for A/C to come on
L4011   FCB  0      		;Upper TPS threshold for A/C to remain on
;
L4012   FCB  1      		;.1 sec, min time to enable A/C after A/C requested
L4013   FCB  0      		;0 sec, min time to re-enable A/C 
L4014   FCB  0      		;0 sec, time delay for A/C to come back on after launch mode
L4015   FCB  0      		;0 sec. time delay for A/C to be disabled if high RPMs
L4016   FCB  0      		;0 sec, min time to re-enable A/C after high rpms
;
L4017   FCB  255		;6375 RPM, upper rpm limit for A/C to come on 
L4018   FCB  254		;6350 RPM, upper rpm limit for A/C to remain on
;
L4019   FCB  254		;6350 RPM, upper RPM threshold to disable A/C
;
L401A   FCB  254		;150 deg C, upper cool temp limit for A/C to come on
L401B   FCB  253		;150 deg C, upper cool temp limit for A/C to remain on
;
L401C   FCB  0			;0 MPH, Lower MPH limit for launch mode if A/C on 
L401D   FCB  0			;0 MPH, Lower MPH limit for launch mode for A/C to come on
;
L401E   FCB  254		;99.2% TPS, lower TPS limit for launch mode for A/C to come on
L401F   FCB  255		;99.2% TPS, lower TPS limit for launch mode for A/C on
;
L4020   FDB  0000		;0 sec, min time to re-enable A/C after high TPS
L4022   FDB  0000   		;0 sec, min run time for compressor to come on   
;
;~~~~~~~~~~~~~~~~~~~~~~~~
;-Closed loop CCP params
;~~~~~~~~~~~~~~~~~~~~~~~~
;
L4024   FCB  240		;5 kPa vac, lower vac threshold to allow CCP DC updates
L4025   FCB  1			;0.4 sec, min time between CCP DC updates
;
;-Desired integrator window during CCP 
;
L4026   FCB 123			;Upper INT bound to begin increasing CCP DC
L4027   FCB 115			;Lower INT bound to begin decreasing CCP DC
;
L4028   FCB 0			;0, multiplier for CCP DC in open loop
L4029   FCB 13			;2.5% per 200 msec, rate to increase CCP if outside INT window 
L402A   FCB 26			;5.1% per 200 msec, rate to decrease CCP if outside INT 
				;window
;
;~~~~~~~~~~~~~~~~~~~~~~~~~
;-Params for allowing CCP
;~~~~~~~~~~~~~~~~~~~~~~~~~
;
L402B   FCB 120			;50 deg C, min temp for CCP
L402C   FCB 160			;80 deg C, cool temp threshold for full DC purge on init. run
L402D   FDB 0004		;Engine run time, duration for full DC purge on init. run
L402F   FCB 10			;Lower speed threshpld to clear CCP DC multiplier if CCP not
				;ok'd
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Hyst. pairs for enabling/disabling CCP
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
L4030   FCB  4			;4 MPH, speed threshold to enable CCP
L4031   FCB  35			;35 Kpa, MAP threshold to enable CCP
L4032   FCB  8			;3% TPS, TPS threshold to enable CCP
;
L4033   FCB  3			;3 MPH, speed threshold to disable CCP
L4034   FCB  32			;32 kPa, MAP threshold to disable CCP
L4035   FCB  6			;2.3% TPS, TPS threshold to disable CCP
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Base closed loop CCP duty cycle vs. airflow
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;	%duty cycle = 
;	val x .390		;gms/sec		
;
L4036   FCB  180		;  0
	FCB  255		; 16
	FCB  255		; 32
	FCB  255		; 48
	FCB  255		; 64
	FCB  255		; 80
	FCB  255		; 96
	FCB  255		;112
	FCB  255		;128
	FCB  255		;144
	FCB  255		;160
	FCB  255		;176
	FCB  255		;192
	FCB  255		;208
	FCB  255		;224
	FCB  255		;240
	FCB  255		;256
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Minimum allowed CCP dutycycle vs. airflow
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;	%duty cycle = 
;	val x .390		;gms/sec		
;
L4047   FCB  90			;  0
	FCB  128		; 16
	FCB  166		; 32
	FCB  179		; 48
	FCB  192		; 64
	FCB  205		; 80
	FCB  218		; 96
	FCB  255		;112
	FCB  255		;128
	FCB  255		;144
	FCB  255		;160
	FCB  255		;176
	FCB  255		;192
	FCB  255		;208
	FCB  255		;224
	FCB  255		;240
	FCB  255		;256
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Table of BLM cells that purge is allowed in
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;	logic, 1=true, 0=false	;BLM cell #
;
L4058   FCB  0			; 0
	FCB  0			; 1 
	FCB  0			; 2
	FCB  0			; 3
	FCB  0			; 4
	FCB  1			; 5
	FCB  1			; 6
	FCB  1			; 7
	FCB  1			; 8
	FCB  1			; 9
	FCB  1			;10
	FCB  1			;11
	FCB  1			;12
	FCB  1			;13
	FCB  1			;14
	FCB  1			;15
	FCB  0			;16
	FCB  0			;17
	FCB  0			;18
	FCB  0			;19
	FCB  0			;20
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Catalytic converter overtemp protection
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; Deg C = value x 3 + 300
;
;-Thresholds used when in closed loop
;
L406D   FCB 255			;1065 deg C, upper allowed cat. converter reaction temp 
L406E   FCB 255			;1065 deg C, upper allowed cat. reaction temp to clear
				;overtemp flag
;
L406F   FCB 255			;1065 deg C, maximum allowed cat. reaction temp
L4070   FCB 255			;1065 deg C, lower reaction temp to clear overtemp flag
;
L4071   FCB 167			;85 deg C, threshold to init cat temp to default value
L4072   FCB 33      		;399 deg C, default cat temp when cool > L4071, temp = val x 3 + 300
;
L4073   FCB 10			;Max time allowed for overtemp before flagging overtemp
				;condition
L4074   FCB 255			;99.6% TPS, upper TPS threshold to mainain current cat reaction
				;temp state after period of time 
L4075   FCB 0			;0 RPM, lower RPM threshold to maintain current cat reaction
				;temp state after period of time    
L4076   FCB 255			;255 sec, time after above thresholds met to stop testing
				;cat. converter for overheating and retain current
				;temperature state set during previous test
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Catalytic converter raction temperature bias vs. AFR
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;	coeff = val/256		;gms/sec
;
L4077   FCB  1			;  0
	FCB  4			; 16
	FCB  5			; 32
	FCB  6			; 48
	FCB  6			; 64
	FCB  6			; 80
	FCB  7			; 96
	FCB  18			;112
	FCB  20			;128
	FCB  22			;144
	FCB  22			;160
	FCB  22			;176
	FCB  22			;192
	FCB  22			;208
	FCB  22			;224
	FCB  22			;240
	FCB  22			;256
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Catalytic converter raction temperature bias vs. AFR
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;	Deg C = value x3 + 300	;AFR
;
L4088   FCB  101		;10.7
	FCB  105		;11.5
	FCB  115		;12.3
	FCB  121		;13.1
	FCB  112		;13.9
	FCB  128		;14.7
	FCB  112		;15.5
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Flow based catalytic converter reaction temperature 
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
L408F   FCB   0			;0 RPM, row offset
	FCB   0			;0 gms/sec, col. offset
	FCB   17		;# cols. per row
;
; Deg C = value x3 + 300
;
;    gms/sec: 0   16   32   48   64   80   96  112  128  144  160  176  192  208  224  240  256;  RPM
;
L4092	FCB  27,  75,  99, 120, 145, 170, 174, 193, 185, 185, 185, 185, 185, 185, 185, 185, 185;    0  
	FCB  27,  75,  99, 120, 145, 170, 174, 193, 185, 185, 185, 185, 185, 185, 185, 185, 185;  800
	FCB  27,  90, 110, 169, 149, 174, 178, 197, 189, 195, 195, 195, 195, 195, 195, 195, 195; 1600  
	FCB  27, 105, 122, 135, 182, 207, 211, 230, 222, 227, 227, 227, 227, 227, 227, 227, 227; 2400 
	FCB  27, 105, 146, 163, 171, 196, 200, 219, 211, 219, 219, 219, 219, 219, 219, 219, 219; 3200
	FCB 102,  85, 150, 176, 181, 187, 187, 187, 187, 187, 187, 187, 187, 187, 187, 187, 187; 4000
	FCB 102,  85, 150, 176, 181, 187, 187, 187, 187, 187, 187, 187, 187, 187, 187, 187, 255; 4800
	FCB 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255; 5600
	FCB 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255; 6400
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-CPI variable tuning valve params
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
L412B   FCB 0			;RPM/25, lower hyst. bound to disable tuning valve
L412C   FCB 0			;RPM/25, lower hyst. bound for valve to remain disabled
;   
L412D   FCB 0			;RPM/25, upper hyst. bound to disable tuning valve   
L412E   FCB 0			;RPM/25, upper hyst. bound for valve to remain disabled
;
L412F   FCB 0			;% TPS, lower hyst. bound to disable tuning valve
L4130   FCB 0			;% TPS, lower hyst. bound for valve to remain disabled
;  
L4131   FCB 0			;-40 deg C, lower cool temp threshold to enable tuning valve
;
;~~~~~~~~~~~
;-VSS param
;~~~~~~~~~~~
;
L4132   FCB 2   		;2 mph, lower allowed threshold for vehicle speed
;
;~~~~~~~~~~~~~~~~~~
;-Spark/DRP params
;~~~~~~~~~~~~~~~~~~
;
L4133   FCB 0			;Base SA 
L4134	FDB 2458		;400 RPM, DRP threshold for engine to be running
L4136	FDB 3574		;275 RPM, DRP threshold to transition from crank to run
;
;~~~~~~~~~~~~~~~~~~~
;-MAP filter params
;~~~~~~~~~~~~~~~~~~~
;
L4138	FCB   200		;Standard MAP filter coeff.
L4139	FCB   250		;Transient MAP filter coeff. 
L413A   FCB   240		;3 secs, transient MAP apply timer val
L413B   FCB   1  		;12.5 rpm, delta for transient MAP 
;
;~~~~~~~~~
;-SA bias
;~~~~~~~~~
;
L413C	FCB 28			;10 deg, main SA bias
L413D	FCB 57			;20 deg, cool temp corr. SA bias
L413E   FCB 28  		;10 deg, 
L413F	FCB 0			;0 Deg, EGR bias
L4140	FCB 0			;0 Deg, bias for MAT SA corr.
;
L4141	FCB  167		; 85c COOL, COLD THRESH FOR NEG SPK not used,
;    
L4142	FCB  8      		;8, number of cylinders 
L4143	FCB  6			; 6 NUM CNT'S SPK RUN FLAG SET IF RPM GT L4134 (400) 
L4144	FCB  2			;#DRPs to transition from crank to run
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-SA advance and retard limits
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
L4145   FDB 0119		;42 deg, max SA;
L4147	FDB 65508   		;-10.0 deg, max allowable retard  
L4149	FDB 65514		;-7.4 deg, max allowable retard when in DFCO
L414B	FDB 65514		;-7.4 deg, max retard during torque management fuel cutoff 
;
L414D	FCB 15			;Filter coeff. for SA ramping multiplier, used when 
				;transitioning from crank to run
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Decel Fuel Cut Off SA corection parameters
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
L414E   FCB 10			;Low %TPS SA ramping mult. filter coeff.  
L414F   FCB 20      		;High %TPS SA ramping mult. filter coeff. 
L4150   FCB 25      		;9.8%, TPS threshold to use high %TPS filter coeff.  
L4151   FCB 28			;10 deg, DFCO SA 
;
L4152   FCB 240			;Filter coeff for RPM/12.5
;
;~~~~~~~~~~~~~~~~~~~
;-Pseudo-Baro quals
;~~~~~~~~~~~~~~~~~~~
;
L4153	FCB 224			;4000 RPM, upper scaled rpm limit for pseudo-baro
L4154   FCB 64      		;1200 RPM, lower scaled rpm limit for pseudo-baro
L4155   FCB 140     		;54.7% TPS, min *TPS for pseudo-baro
L4156   FCB 5       		;2.0% TPS, Min delta %TPS for pseudo-baro
L4157   FCB 128     		;Baro filter coeff.
L4158   FCB 160     		;Min cool temp for pseudo-baro update
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Idle SA adjustment quals.
;~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
L4159   FCB 1			;1 MPH, upper limit for idle SA adjustments   
L415A   FCB 4			;1.6% TPS, upper limit for idle SA adjustments, idle SA ok'd  
L415B   FCB 6			;3.2% TPS, upper limit for idle SA adjustments, idle SA 
				;not ok'd   
L415C   FCB 100			;35 deg C, min temp for idle SA adjustments 
L415D   FDB 0030		;30 sec, min run time for idle SA adjustments 
L415F   FCB 0			;0 RPM, offset for desired RPMs if not idling to 
				;check RPM quals. for idle SA adjustments
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Power steering SA compensation
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
L4160   FCB 255			;151 deg C, cool temp threshold for P/S SA compensation
L4161   FDB 0000		;0 deg SA, fixed SA for P/S compensation
L4163   FCB 0			;0% TPS, upper TPS threshold for enabling P/S compensation
L4164   FCB 0			;0% TPS, threshold for disabling P/S compensation when active 
;
;~~~~~~~~~~~~~~~
;-Main SA table
;~~~~~~~~~~~~~~~
;
; SA = (val - bias) * .351
;
L4165   FCB   0			;0 RPM, row offset
	FCB   0			;0 kPa, col offset
	FCB  17			;# cols./row
;
;	kPa: 20   25   30   35   40   45   50   55   60   65   70   75   80   85   90   95  100 ;  RPM
;
	FCB  85,  85,  85,  85,  85,  80,  77,  63,  60,  54,  46,  40,  37,  34,  31,  28,  26 ;  400
	FCB  85,  85,  85,  85,  85,  80,  77,  63,  60,  54,  46,  40,  37,  34,  31,  28,  28	;  600
	FCB  85,  85,  85,  85,  85,  85,  85,  74,  68,  57,  54,  43,  40,  34,  31,  28,  28	;  800
	FCB  85,  85,  85,  85,  85,  85,  85,  80,  71,  65,  57,  48,  43,  37,  31,  28,  28	; 1000   
 	FCB  85,  85,  85,  85,  85,  85,  85,  80,  74,  68,  60,  54,  48,  43,  37,  34,  31	; 1200
  	FCB  88,  88,  88,  88,  88,  88,  88,  82,  80,  71,  65,  60,  54,  48,  43,  40,  40 ; 1600
	FCB  91,  91,  91,  91,  91,  91,  91,  85,  82,  74,  68,  63,  57,  54,  48,  46,  46	; 2000
	FCB  94,  94,  94,  94,  94,  94,  94,  91,  88,  80,  74,  68,  63,  60,  54,  54,  51	; 2400
	FCB 100, 100, 100, 100, 100, 100, 100,  94,  91,  82,  77,  71,  65,  63,  60,  57,  54 ; 2800
	FCB 102, 102, 102, 102, 102, 102, 102,  97,  94,  91,  80,  77,  74,  68,  65,  63,  60 ; 3200
	FCB 105, 105, 105, 105, 105, 105, 105, 102,  97,  91,  85,  80,  77,  71,  68,  65,  63	; 3600
	FCB 114, 114, 114, 114, 114, 114, 114, 114, 100,  94,  88,  85,  82,  77,  77,  74,  71	; 4000
	FCB 114, 114, 114, 114, 114, 114, 114, 114, 102,  97,  91,  88,  85,  80,  80,  80,  77 ; 4400
	FCB 114, 114, 114, 114, 114, 114, 114, 114, 102,  97,  91,  88,  85,  80,  77,  74,  71	; 4800
	FCB 114, 114, 114, 114, 114, 114, 114, 114, 102,  97,  94,  91,  88,  82,  80,  77,  74	; 5200         
	FCB 114, 114, 114, 114, 114, 114, 114, 114, 102,  97,  94,  91,  88,  82,  80,  77,  74 ; 5600
	FCB 114, 114, 114, 114, 114, 114, 114, 114, 102,  97,  94,  91,  88,  82,  80,  77,  74 ; 6000
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Closed throttle SA table
;~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; SA = (val - bias) * .351
;
L4289   FCB  0			;0 RPM, row offset
	FCB  0			;0 kPa, col offset
	FCB  17			;# of cols. per row
;
;	kPa: 20   25   30   35   40   45   50   55   60   65   70   75   80   85   90   95  100 ;  RPM
;
	FCB  85,  85,  85,  85,  85,  80,  77,  63,  60,  54,  46,  40,  37,  34,  31,  28,  26 ;  400
	FCB  71,  71,  71,  71,  77,  80,  77,  63,  60,  54,  46,  40,  37,  34,  31,  28,  28	;  600
	FCB  71,  71,  71,  71,  77,  85,  85,  74,  68,  57,  54,  43,  40,  34,  31,  28,  28	;  800
	FCB  85,  85,  85,  85,  85,  85,  85,  80,  71,  65,  57,  48,  43,  37,  31,  28,  28 ; 1000
	FCB  85,  85,  85,  85,  85,  85,  85,  80,  74,  68,  60,  54,  48,  43,  37,  34,  31 ; 1200
	FCB  88,  88,  88,  88,  88,  88,  88,  82,  80,  71,  65,  60,  54,  48,  43,  40,  40 ; 1600        
	FCB  91,  91,  91,  91,  91,  91,  91,  88,  85,  80,  71,  68,  60,  57,  48,  46,  46 ; 2000
	FCB  94,  94,  94,  94,  94,  94,  94,  91,  88,  80,  74,  68,  63,  60,  54,  54,  51	; 2400
	FCB 100, 100, 100, 100, 100, 100, 100,  94,  91,  82,  77,  71,  65,  63,  60,  57,  54 ; 2800
	FCB 102, 102, 102, 102, 102, 102, 102,  97,  94,  91,  80,  77,  74,  68,  65,  63,  60 ; 3200
	FCB 105, 105, 105, 105, 105, 105, 105, 102,  97,  91,  85,  80,  77,  71,  68,  65,  63 ; 3600
	FCB 114, 114, 114, 114, 114, 114, 114, 114, 100,  94,  88,  85,  82,  77,  74,  71,  68 ; 4000
	FCB 114, 114, 114, 114, 114, 114, 114, 114, 102,  97,  91,  88,  85,  80,  77,  74,  71 ; 4400
	FCB 114, 114, 114, 114, 114, 114, 114, 114, 102,  97,  91,  88,  85,  80,  77,  74,  71 ; 4800
	FCB 114, 114, 114, 114, 114, 114, 114, 114, 102,  97,  94,  91,  88,  82,  80,  77,  74 ; 5200
	FCB 114, 114, 114, 114, 114, 114, 114, 114, 102,  97,  94,  91,  88,  82,  80,  77,  74 ; 5600
	FCB 114, 114, 114, 114, 114, 114, 114, 114, 102,  97,  94,  91,  88,  82,  80,  77,  74 ; 6000
;
;~~~~~~~~~~~~~~~~~~~~~~~~~
;-Cool temp SA correction
;~~~~~~~~~~~~~~~~~~~~~~~~~
;
; SA = (val - bias) * .351
;
L43AD	FCB   0 		;zero val=use vac, non-zero val=use SD MAP
;
	FCB  16			;12 deg C, cool temp offset 
	FCB  0			;0 kPa, vac offset
	FCB  9			;# cols. per row
;
;	kPa: 80	  70   60   50   40   30   20   10    0	;Deg C
;
	FCB  91,  91,  91,  91,  91,  91, 100, 100, 100	;-28
	FCB  91,  91,  91,  91,  91,  91,  97,  97,  97 ;-16 
	FCB  85,  85,  85,  85,  85,  85,  94,  94,  94 ; -4 
	FCB  82,  82,  82,  82,  82,  82,  85,  85,  85 ;  8 
	FCB  54,  54,  54,  54,  54,  57,  60,  63,  63 ; 20 
	FCB  48,  48,  48,  48,  48,  51,  54,  57,  57 ; 32 
	FCB  46,  46,  46,  46,  46,  48,  51,  51,  51 ; 44 
	FCB  46,  46,  46,  46,  46,  48,  51,  51,  51 ; 56 
	FCB  48,  48,  48,  48,  48,  57,  57,  57,  57 ; 68 
	FCB  57,  57,  57,  57,  57,  57,  57,  57,  57 ; 80 
	FCB  57,  57,  57,  57,  57,  57,  57,  57,  57 ; 92 
	FCB  54,  54,  54,  51,  51,  48,  46,  46,  46 ;103   
	FCB  51,  48,  48,  43,  43,  43,  40,  40,  40 ;116 
	FCB  43,  43,  37,  34,  34,  34,  31,  31,  31 ;128 
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-SA correction mult. vs MAT
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; SA corr. = (val - bias) x .351 
;
L442F   FCB  0      ;Col. select, 0 = use vac, 1 = use RPM
;
	FCB  80     ;20 deg C, min MAT
	FCB  0      ;Min vac.
	FCB  12     ;# of cols.
;
;
;	kPa   55,   50,   45,   40,   35,   30,   25,   20,   15,   10,    5,    0	;Deg C
;

	FCB    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0	; 20 
	FCB    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0	; 32
	FCB    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0	; 44
	FCB    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0	; 55
	FCB    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0	; 68
	FCB    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0	; 80
	FCB    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0	; 92	
	FCB    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0	;104
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-MAT SA mult. for negative SA correction
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;	Mult. = val/256		;kPa
;
L4493	FCB  0			; 20  
	FCB  0			; 25  
	FCB  0      		; 30  
	FCB  0      		; 35  
	FCB  0      		; 40  
	FCB  0      		; 45  
	FCB  0      		; 50  
	FCB  0      		; 55  
	FCB  0      		; 60  
	FCB  0      		; 65  
	FCB  0      		; 70  
	FCB  0      		; 75  
	FCB  0      		; 80
	FCB  0      		; 85
	FCB  0      		; 90
	FCB  0     		; 95
	FCB  0      		;100
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-MAT SA mult. for positive SA correction
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;	Mult. = val/256		;kPa
;
L44A4	FCB  0			; 20  
	FCB  0			; 25  
	FCB  0      		; 30  
	FCB  0      		; 35  
	FCB  0      		; 40  
	FCB  0      		; 45  
	FCB  0      		; 50  
	FCB  0      		; 55  
	FCB  0      		; 60  
	FCB  0      		; 65  
	FCB  0      		; 70  
	FCB  0      		; 75  
	FCB  0      		; 80
	FCB  0      		; 85
	FCB  0      		; 90
	FCB  0     		; 95
	FCB  0      		;100
;
;~~~~~~~~~~~~~~~~~~~~~~~
;-PE spark corr. vs rpm
;~~~~~~~~~~~~~~~~~~~~~~~
;
;	Deg SA		;RPM
;

L44B5   FCB  0			;   0
	FCB  0			; 400
	FCB  0			; 800
	FCB  3			;1200
	FCB  6			;1600
	FCB  6			;2000
	FCB  6			;2400
	FCB  6			;2800
	FCB  6			;3200
	FCB  6			;3600
	FCB  6			;4000
	FCB  6			;4400
	FCB  6			;4800
	FCB  6			;5200
	FCB  6			;5600
	FCB  6			;6000
	FCB  6			;6400
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-SA decay period vs. airflow
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;	period = 5/value
;
L44C6   FCB 19			; 0
        FCB 14			;16
        FCB 8			;32
        FCB 6			;48
        FCB 4			;64
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Alt. SA correction vs. baro and vac
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; SA corr. = (val - bias) x .351 
;
	
L44CB	FCB  96			;75 kPa, baro row offset for lookup
	FCB  176		;25 kPa, vacuum col. offset for lookup
	FCB  6			;# cols. per row
;
;  kPa vac:  25   20   15   10    5    0 ;kPa baro 
;
	FCB  28,  28,  25,  25,  25,  28 ; 75     
	FCB  28,  28,  28,  28,  28,  28 ; 85
	FCB  28,  28,  28,  28,  28,  28 ; 95	
	FCB  28,  28,  28,  28,  28,  28 ;105
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Engine idle overspeed proportional SA correction vs RPM error
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;-Subtracted from SA to lower engine speed
;
L44E6  	FCB   96		;250 RPM, UL for table lookup
;
;	SA corr.=		
;	val x .351		;RPM error
;
	FCB   0			;  0.0  
	FCB   6			; 12.5
	FCB   9			; 25.0
	FCB  11			; 37.5
	FCB  14			; 50.0
	FCB  14			;150.0
	FCB  17			;250.0
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Engine idle overspeed SA multiplier vs. SD MAP
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
L44EE   FCB  128    ;60 kPa, UL for lookup
;
;	% SA corr. =
;	val x .391		;kPa
;
	FCB  255		;20
	FCB  255		;25
	FCB  255		;30
	FCB  255		;35
	FCB  244		;40
	FCB  244		;45
	FCB  232		;50
	FCB  175		;55
	FCB  128		;60
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Engine idle underspeed proportional SA correction vs RPM error
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;-Added SA to raise engine speed
;
L44F8   FCB 96			;250 rpm, UL for lookup
;
;	SA corr.=		
;	val x .351		;RPM error
;
	FCB   0     		;  0.0 
	FCB   3     		; 12.5
	FCB   6     		; 25.0 
	FCB   9     		; 37.5
	FCB  11    		; 50.0
	FCB   9     		;150.0 
	FCB   9     		;250.0
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Engine idle underspeed SA multiplier vs S/D MAP
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
L4500   FCB 128     		;60 kPa, UL for lookup
;
;	% SA corr. =
;	val x .391		;kPa
;
	FCB   244		; 20 
	FCB   232    		; 25
	FCB   232    		; 30
	FCB   232    		; 35
	FCB   205    		; 40
	FCB   166    		; 45
	FCB   102    		; 50
	FCB   32     		; 55
	FCB   0      		; 60
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Derivative spark/fuel RPM control params
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
L450A  FCB   57			;20 deg, SA bias for transient RPM idle SA corr. 
L450B  FCB   11			;3.9 deg, max idle SA correction 
L450C  FCB   64			;1600 RPM, upper limit for derivative RPM term
;
L450D  FCB   221		;Filter coeff, RPM/6.25
L450E  FCB   101		;Filter coeff, RPM derivative term
;
;-Limits for derivative fuel 
;
L450F  FCB   116		;.91, min value for transient RPM ratio
L4510  FCB   141		;1.1, max value for transient RPM ratio
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Transient RPM idle SA corr. vs. rpm ratio
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; 	SA = 
;	(val - bias) x .351	;RPM ratio
;
L4511   FCB  50			;0.75000
	FCB  51			;0.78125
	FCB  52			;0.81250     
	FCB  53			;0.84375
	FCB  54			;0.87500
	FCB  55			;0.90625
	FCB  56			;0.93750
	FCB  57			;0.96875
	FCB  57			;1.00000
	FCB  57			;1.03125
	FCB  58			;1.06250
	FCB  59			;1.09375
	FCB  60			;1.12500
	FCB  61			;1.15625
	FCB  62			;1.18750
	FCB  63			;1.21875
	FCB  64			;1.25000
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Initial added SA vs. cool temp
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
L4522	FCB 8			;9 line table 
;
;	Deg SA = val x .351	;Deg C		
;
L4523   FCB 28			;-40
	FCB 28			;-16
	FCB 28			;  8
	FCB 28			; 32
	FCB 20			; 56
	FCB 14			; 80
	FCB 14			;104
	FCB 14			;128
	FCB 14			;152
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Initial added SA decay delay vs. cool temp
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;
L452C	FCB 8			;9 line table
;
;	secs before decay = 
;	val/5			;Deg C
;
L452D   FCB 40			;-40
	FCB 40			;-16
	FCB 25			;  8
	FCB 15			; 32
	FCB 20			; 56
	FCB 10			; 80
	FCB 5			;104
	FCB 5			;128
	FCB 5			;152
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Choke SA decay multiplier vs. cool temp
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
L4536   FCB  8      ;9 line table
;
;	mult. = val/256		;Deg C
;
L4537   FCB  251		;-40
	FCB  251		;-16
	FCB  248		;  8
	FCB  248		; 32
	FCB  243		; 56
	FCB  235		; 80
	FCB  205		;104
	FCB  205		;128
	FCB  205		;152
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Spark latency correction vs. RPMs
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
L4540	FCB  192		;4800 rpm, UL for lookup
;
;       usecs = val x 15.26	;RPM
;
L4541	FCB   0			;   0    
	FCB   0			; 400 
	FCB   0			; 800 
	FCB   3			;1200  
	FCB   7			;1600   
	FCB   9			;2000
	FCB   9			;2400  
	FCB   12		;2800
	FCB   11		;3200
	FCB   9			;3600 
	FCB   10 		;4000 
	FCB   10     		;4400 
	FCB   9      		;4800    
;
;~~~~~~~~~~~~~~~~~~~~~~~
;-Baro correction table
;~~~~~~~~~~~~~~~~~~~~~~~
;
L454E	FCB   32    ; Min RPM Value, (1200)
	FCB   48    ; Min TPS Value, (37.5%)
	FCB   6     ; # columns/row
;
;-Added Baro counts
;
; 	%TPS   37.5, 50.0, 62.5, 75.0, 87.5,  100	;RPM			
;
	FCB	18,    10,    6,    5,    4,    3	;1200
	FCB	35,    17,   13,    9,    8,    6	;1800
	FCB	54,    27,   22,   16,   14,   11	;2400
	FCB	75,    44,   29,   20,   16,   13	;3000
	FCB	97,    54,   38,   26,   21,   16	;3600
	FCB    115,    62,   46,   32,   27,   21	;4200
	FCB    130,    70,   48,   33,   25,   20	;4800	
;
;~~~~~~~~~~~~~~~~~~~~~
;-Max PE knock retard
;~~~~~~~~~~~~~~~~~~~~~
;
;	Max pulled SA=
;	value x .351		;RPM
;
L457B   FCB 46			; 800  
	FCB 46  		;1600 
	FCB 46  		;2400 
	FCB 57  		;3200 
	FCB 57  		;4000
	FCB 57  		;4800
	FCB 57  		;5600
	FCB 57  		;6400
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Max Non-PE retard vs Vac
;~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;
;	Max pulled SA=
;	value x .351		;kPa
;
L4583   FCB 0			; 60  
        FCB 23  		; 50
        FCB 46  		; 40
        FCB 46  		; 30
        FCB 46  		; 20
        FCB 46  		; 10
        FCB 46  		;  0
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Knock attack rate vs. rpm
;~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;	deg/msec = val. x
;		   .0225 	
;	       			; RPM
;			
L458A   FCB  2			;   0
	FCB  2      		; 400
	FCB  10     		; 800
	FCB  15     		;1200
	FCB  15     		;1600
	FCB  15     		;2000
	FCB  15     		;2400
	FCB  21     		;2800
	FCB  21     		;3200
	FCB  21     		;3600
	FCB  21     		;4000
	FCB  21     		;4400
	FCB  21     		;4800
	FCB  21     		;5200
	FCB  21     		;5600
	FCB  21     		;6000
	FCB  21   	  	;6400
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Retard recovery rate vs. RPM
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;	% recovery/sec = 
;	value x 1.953		;RPM
;
L459B   FCB  220		;   0
	FCB  220		; 400
	FCB  48			; 800
	FCB  19			;1200
	FCB  19			;1600
	FCB  19			;2000
	FCB  18			;2400
	FCB  18			;2800
	FCB  16			;3200
	FCB  16			;3600
	FCB  16			;4000
	FCB  16			;4400
	FCB  16			;4800
	FCB  16			;5200
	FCB  16			;5600
	FCB  16			;6000
	FCB  16			;6400
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Low octaine retard params
;~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
L45AC   FCB 160			;80 deg C, min temp for low octaine retard
L45AD   FCB 96			;50 kPa, lower SD MAP threshold for low octaine retard 
L45AE   FCB 64			;20 kPa, max rate of change in SD MAP for low octaine retard
;
L45AF   FCB 14			;2.5 deg, lower SA retard threshold to set high knock activity  
L45B0   FCB 11			;1.9 deg, upper SA retard threshold to set low knock activity
;
L45B1   FCB 25			;9.8 %/sec, ramping rate to bring in base low octaine ret. SA
L45B2   FCB 25			;9.8 %/sec, ramping rate to remove base low octaine retard SA
;
L45B3   FCB 14			;5 deg, base low octaine retard SA
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Low octane spark retard mult. vs RPM
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; Mult = val/2.56		; RPM
;
L45B4   FCB  0      		;   0
	FCB  0     		; 800
	FCB  208    		;1600
	FCB  230    		;2400
	FCB  255    		;3200
	FCB  255    		;4000
	FCB  255    		;4800
	FCB  255    		;5600
	FCB  255    		;6400
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Low octane SA mult. vs SD MAP
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;	Mult.			;kPa		
;
L45BD   FCB  0			; 20
	FCB  0			; 25
	FCB  0			; 30
	FCB  0			; 35
	FCB  0			; 40
	FCB  0			; 45
	FCB  128		; 55
	FCB  204		; 55
	FCB  255		; 60
	FCB  255		; 65
	FCB  255		; 70
	FCB  255		; 75
	FCB  255		; 80
	FCB  255		; 85
	FCB  255		; 90
	FCB  255		; 95
	FCB  255		;100
;
;~~~~~~~~~~~~~~~~~~~~~~~~
; Quals for knock retard
;~~~~~~~~~~~~~~~~~~~~~~~~
;
L45CE	FCB 2			;2 MPH, min speed for knock retard 
L45CF   FCB 56			;700 RPM, min RPM threshold for knock retard 
L45D0   FCB 153			;75 deg C, min cool temp for knock retard 
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Min knock counts vs. RPM
;~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; Subtracted out from knock 
; counts before store
;
;	knock counts		;RPM
;
L45D1   FCB  48			;   0
	FCB  48			; 100
	FCB  48			; 200
	FCB  48			; 300
	FCB  48			; 400
	FCB  48			; 500
	FCB  48			; 600
	FCB  48			; 700
	FCB  19			; 800
	FCB  19			; 900
	FCB  19			;1000
	FCB  16			;1100
	FCB  16			;1200
	FCB  16			;1300
	FCB  16			;1400
	FCB  16			;1500
	FCB  16			;1600
	FCB  16			;1700
	FCB  16			;1800
	FCB  16			;1900
	FCB  21			;2000
	FCB  21			;2100
	FCB  21			;2200
	FCB  21			;2300
	FCB  21			;2400
	FCB  21			;2500
	FCB  21			;2600
	FCB  21			;2700
	FCB  41			;2800
	FCB  41			;2900
	FCB  41			;3000
	FCB  41			;3100
	FCB  41			;3200
	FCB  41			;3300
	FCB  41			;3400
	FCB  41			;3500
	FCB  41			;3600
	FCB  32			;3700
	FCB  28			;3800
	FCB  25			;3900
	FCB  21			;4000
	FCB  16			;4100
	FCB  16			;4200
	FCB  16			;4300
	FCB  16			;4400
	FCB  16			;4500
	FCB  16			;4600
	FCB  16			;4700
	FCB  16			;4800
	FCB  16			;4900
	FCB  16			;5000
	FCB  16			;5100
	FCB  16			;5200
	FCB  16			;5300
	FCB  16			;5400
	FCB  16			;5500
	FCB  16			;5600
	FCB  16			;5700
	FCB  16			;5800
	FCB  16			;5900
	FCB  16			;6000
	FCB  16			;6100
	FCB  16			;6200
	FCB  16			;6300
	FCB  16			;6400
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Added knock count bias vs. RPM
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;	Knock count bias	;RPM
;
L4612   FCB  0			;   0
	FCB  0			; 100
	FCB  0			; 200
	FCB  0			; 300
	FCB  0			; 400
	FCB  0			; 500
	FCB  0			; 600
	FCB  0			; 700
	FCB  48			; 800
	FCB  48			; 900
	FCB  48			;1000
	FCB  48			;1100
	FCB  48			;1200
	FCB  48			;1300
	FCB  48			;1400
	FCB  48			;1500
	FCB  48			;1600
	FCB  48			;1700
	FCB  48			;1800
	FCB  48			;1900
	FCB  48			;2000
	FCB  48			;2100
	FCB  48			;2200
	FCB  48			;2300
	FCB  48			;2400
	FCB  48			;2500
	FCB  48			;2600
	FCB  48			;2700
	FCB  48			;2800
	FCB  48			;2900
	FCB  48			;3000
	FCB  48			;3100
	FCB  48			;3200
	FCB  48			;3300
	FCB  48			;3400
	FCB  64			;3500
	FCB  64			;3600
	FCB  64			;3700
	FCB  64			;3800
	FCB  64			;3900
	FCB  64			;4000
	FCB  64			;4100
	FCB  64			;4200
	FCB  64			;4300
	FCB  64			;4400
	FCB  64			;4500
	FCB  64			;4600
	FCB  64			;4700
	FCB  64			;4800
	FCB  64			;4900
	FCB  64			;5000
	FCB  64			;5100
	FCB  64			;5200
	FCB  64			;5300
	FCB  64			;5400
	FCB  64			;5500
	FCB  64			;5600
	FCB  64			;5700
	FCB  64			;5800
	FCB  64			;5900
	FCB  64			;6000
	FCB  64			;6100
	FCB  64			;6200
	FCB  64			;6300
	FCB  64			;6400
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Knock window delay time vs. RPM
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; msec = Table val/131.072
;
L4653   FDB  0426		;   0
	FDB  0426		; 100
	FDB  0426   		; 200
	FDB  0426   		; 300
	FDB  0426   		; 400        
	FDB  0426   		; 500
	FDB  0426   		; 600
	FDB  0426   		; 700
	FDB  0426   		; 800   
	FDB  0426   		; 900
	FDB  0426   		;1000
	FDB  0426  		;1100
	FDB  0426   		;1200
	FDB  0426   		;1300        
	FDB  0426   		;1400
	FDB  0426   		;1500
	FDB  0426   		;1600
	FDB  0361   		;1700        
	FDB  0361   		;1800        
	FDB  0361   		;1900        
	FDB  0361   		;2000        
	FDB  0361   		;2100        
	FDB  0361   		;2200        
	FDB  0328   		;2300        
	FDB  0328   		;2400
	FDB  0328   		;2500        
	FDB  0328   		;2600        
	FDB  0328   		;2700        
	FDB  0328   		;2800        
	FDB  0328   		;2900        
	FDB  0328   		;3000        
	FDB  0328   		;3100        
	FDB  0328   		;3200        
	FDB  0262   		;3300        
	FDB  0262   		;3400        
	FDB  0262   		;3500        
	FDB  0262   		;3600        
	FDB  0262   		;3700        
	FDB  0262   		;3800        
	FDB  0262   		;3900        
	FDB  0262   		;4000        
	FDB  0262   		;4100
	FDB  0262   		;4200        
	FDB  0262   		;4300
	FDB  0262   		;4400        
	FDB  0219   		;4500        
	FDB  0219   		;4600
	FDB  0219   		;4700        
	FDB  0219   		;4800        
	FDB  0219   		;4900        
	FDB  0219   		;5000        
	FDB  0219   		;5100        
	FDB  0219   		;5200        
	FDB  0219   		;5300        
	FDB  0219   		;5400        
	FDB  0219   		;5500        
	FDB  0219   		;5600        
	FDB  0219   		;5700        
	FDB  0219   		;5800        
	FDB  0219   		;5900        
	FDB  0219   		;6000        
	FDB  0219   		;6100        
	FDB  0219   		;6200        
	FDB  0219   		;6300        
	FDB  0219   		;6400
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Knock prevention params.
;~~~~~~~~~~~~~~~~~~~~~~~~~~
;
L46D5   FCB 96			;50 kPa, upper limit for knock prevention
L46D6   FCB 128			;50% TPS, upper limit for knock prevention
L46D7   FCB 12			;4.7% delta TPS, min delta TPS for knock prevention 
L46D8   FCB 40			;500 msec, time for burst retard to be in effect 
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Degrees SA of burst knock retard vs. cool temp
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
L46D9   FCB 112			;110 deg C, UL for lookup
;
;	deg SA = 
;	val x .351		;Deg C
;
	FCB 0       		; 75
	FCB 3       		; 80
	FCB 6       		; 85
	FCB 6       		; 90
	FCB 6       		; 95
	FCB 9       		;100
	FCB 14      		;105
	FCB 17      		;110
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Engine SA control params, used with manual trans.
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;-RPMs rising
;
L46E2   FCB 0			;SA derivative gain
L46E3   FCB 0			;SA derivative term bias
L46E4   FCB 0			;Max allowable SA derivative term to be subtracted
;
;-RPMs falling
;
L46E5   FCB 0			;SA derivative gain  
L46E6   FCB 0			;SA derivative term bias  
L46E7   FCB 0			;Max allowable SA derivative term to be added   
L46E8   FCB 255			;151 deg C, lower temp. limit for der. control
L46E9   FCB 0			;-40 deg C, upper temp. limit for der. control
;
;-Filter coeff for RPMs/12.5, used in calc. delta RPMs
;
;	Filt. coeff.		;% TPS
;
L46EA   FCB 0			;  0
	FCB 0			; 25   
	FCB 0			; 50   
	FCB 0			; 75   
	FCB 0			;100   
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Quals for enabling/disabling EGR
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;-Startup cool temp EGR params
;
L46EF   FCB 73			;15 deg C, startup cool temp threshold for warm startup
L46F0   FCB 153			;75 deg C, cool temp to enable EGR if warm startup
L46F1   FCB 113			;45 deg C, cool temp to enable EGR if cold startup
;
;-MAT thresholds for EGR
;
L46F2   FCB 0			;-50 deg C, MAT threshold to enable EGR     
L46F3   FCB 0			;-50 deg C, MAT threshold to disable EGR
;
;-Speed threshold for EGR
;
L46F4   FCB 0			;0 MPH, min speed for EGR  
;
;-RPM thresholds for EGR
;
L46F5   FCB 32			;800 RPM, threshold to enable EGR   
L46F6   FCB 30			;750 RPM, threshold to disable EGR
;
;-TPS thresholds for EGR
;
L46F7   FCB 8			;3% TPS, threshold to enable EGR 
L46F8   FCB 5			;2% TPS, threshold to disable EGR
;
;-MAP thresholds for EGR
;
L46F9   FCB 45			;34 kPa, MAP threshold to enable EGR   
L46FA   FCB 38			;32 kPa, MAP threshold to disable EGR  
;
;-VAC thresholds for EGR
;				
L46FB   FCB 240			;5.0 kPa, vac threshold to enable EGR
L46FC   FCB 233			;7.2 kPa, vac threshold to disable EGR
;
L46FD   FCB 166			;65% TPS, max TPS for EGR 
L46FE   FCB 140			;14:1 AFR, max rich AFR for EGR
;
;-Min desired EGR
;
; Linear EGR = desired pintle pos.
; EVRV/back pressure type EGR = duty cycle
;
L46FF	FCB	51		;20%, min allowed desired EGR, less will be cleared
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Desired EGR vs. RPM and load
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;				 
L4700   FCB 0			;EGR load option
				;0 = vacuum
				;1 = SD MAP
				;2 = MAP/Baro ratio
;	 
L4701   FCB 32			;800 RPM offset
        FCB 0			;0 kPa, vac offset
        FCB 9			;# cols. per row
;
; Desired EGR = val x .391 (or pintle posit. for linear EGR)
;
;   Vac	kPa:  80   70   60   50   40   30   20   10    0 : RPM
;
L4704   FCB  255, 255, 255, 255, 255, 255, 255, 255, 255 ; 800
	FCB  255, 255, 255, 255, 255, 255, 255, 255, 255 ;1200
	FCB  255, 255, 255, 255, 255, 255, 255, 255, 255 ;1600
	FCB  255, 255, 255, 255, 255, 255, 255, 255, 255 ;2000
	FCB  255, 255, 255, 255, 255, 255, 255, 255, 255 ;2400
	FCB  255, 255, 255, 255, 255, 255, 255, 255, 255 ;2800
	FCB  255, 255, 255, 255, 255, 255, 255, 255, 255 ;3200
	FCB  255, 255, 255, 255, 255, 255, 255, 255, 255 ;3600
	FCB  255, 255, 255, 255, 255, 255, 255, 255, 255 ;4000
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Desired EGR correction factor vs. cool temp
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
L4755   FCB  176 		;92 deg C, upper limit for lookup
;
;	factor = val/128	;Deg C
;
	FCB  128    		; 20      
	FCB  128    		; 32        
	FCB  128    		; 48        
	FCB  128    		; 56      
	FCB  128    		; 68       
	FCB  128    		; 80         
	FCB  128    		; 92
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Desired EGR correction vs. SD MAP and vac
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
L475D   FCB   16		;30 kPa, MAP offset 
	FCB   96		;75 kPa, vac offset
	FCB   4			;# cols. per row
;
; Factor = value/128
;
;  Baro kPa:  75   85   95  105 :MAP kPa
;
L4760   FCB  128, 128, 128, 128 ; 30
	FCB  128, 128, 128, 128 ; 40
	FCB  128, 128, 128, 128 ; 50
	FCB  128, 128, 128, 128 ; 60
	FCB  128, 128, 128, 128 ; 70
	FCB  128, 128, 128, 128 ; 80
	FCB  128, 128, 128, 128 ; 90
	FCB  128, 128, 128, 128 ;100
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Ratio of EGR flow vs. MAP and load
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
L4780   FCB   16		;800 RPM, RPM offset 
        FCB   0			;0 kPa, min load value for lookup
        FCB   13		;# cols. per row
;
; EGR/total gms flow ratio = value x .0976
; 
;      kPa: 0.0  2.5  5.0  7.5 10.0 12.5 15.0 17.5 20.0 30.0 40.0 50.0 60.0:  RPM
;
L4783	FCB   0,   0,  15,  46, 100, 100, 100, 100, 100, 100, 100, 100, 100 ; 800  
	FCB   0,   0,  15,  46, 100, 100, 100, 100, 100, 100, 100, 100, 100 ;1600 
	FCB   0,   0,  15,  46, 100, 100, 100, 100, 100, 100, 100, 100, 100 ;2400 
	FCB   0,   0,  15,  46, 100, 100, 100, 100, 100, 100, 100, 100, 100 ;3200
	FCB   0,   0,  15,  46, 100, 100, 100, 100, 100, 100, 100, 100, 100 ;3600
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-%EGR flow filter coeff. vs airflow
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
L47C4   FCB 230			; 0 
	FCB 230			;16
	FCB 230			;32
	FCB 230			;48
	FCB 230			;64
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-EGR exhaust back pressure vs. airflow
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;	kPa = va lx .3125
;
L47C9   FCB  0			;  0
	FCB  0			; 16
	FCB  0			; 32
	FCB  0			; 48
	FCB  0			; 64
	FCB  0			; 80
	FCB  0			; 96
	FCB  0			;112
	FCB  0			;128
	FCB  0			;144
	FCB  0			;160
	FCB  0			;176
	FCB  0			;192
	FCB  0			;208
	FCB  0			;224
	FCB  0			;240
	FCB  0			;256
;
L47DA   FCB 0			;EGR flow scalar, used in calculation of %EGR flow
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-%EGR flow VS. delta kPa and %EGR 
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
L47DB   FCB   0			;0 kPa, delta Kpa row offset 
        FCB   0			;0%, %EGR col. offset 
        FCB   10		;# cols. per row
;
;      kPa:   0   5  10  15  20  30  40  50  60  70 ;%EGR
; 
	FCB   0,  0,  0,  0,  0,  0,  0,  0,  0,  0 ;  0
	FCB   0,  0,  0,  0,  0,  0,  0,  0,  0,  0 ; 10
	FCB   0,  0,  0,  0,  0,  0,  0,  0,  0,  0 ; 20
	FCB   0,  0,  0,  0,  0,  0,  0,  0,  0,  0 ; 30
	FCB   0,  0,  0,  0,  0,  0,  0,  0,  0,  0 ; 40
	FCB   0,  0,  0,  0,  0,  0,  0,  0,  0,  0 ; 50
	FCB   0,  0,  0,  0,  0,  0,  0,  0,  0,  0 ; 60
	FCB   0,  0,  0,  0,  0,  0,  0,  0,  0,  0 ; 70
	FCB   0,  0,  0,  0,  0,  0,  0,  0,  0,  0 ; 80
	FCB   0,  0,  0,  0,  0,  0,  0,  0,  0,  0 ; 90
	FCB   0,  0,  0,  0,  0,  0,  0,  0,  0,  0 ;100
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-%EGR flow baro corr. factor vs baro
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;	mult. 			;kPa	
;
L484C   FCB 255			; 75 
	FCB 255			; 85
	FCB 255			; 95
	FCB 255			;105
;
;-Linear EGR quals for EGR flow
;
L4850   FCB 0			;Min linear EGR position for EGR flow
L4851   FDB 0003		;3 secs, min time to enable EGR
;
;-Standard EGR quals for EGR flow
;
L4853   FCB 51			;20%, min %EGR for EGR flow
;
L4854   FCB 5			;EGR SA correction filter coeff.
;
;~~~~~~~~~~~~~~~~~~~
;-EGR SA correction
;~~~~~~~~~~~~~~~~~~~
;
L4855   FCB	1		;EGR load opt. byte, 1=vacuum, 0=%EGR flow
;	
L4856   FCB	16		;800 RPM, row offset
        FCB	0		;0 kPa, column offset
        FCB	13		;# cols per row							  
;
; SA = value x .391
;
;	kPa: 0.0  2.5  5.0  7.5 10.0 12.5 15.0 17.5 20.0 30.0 40.0 50.0 60.0 ; RPM
;
	FCB    0,   0,   0,   3,   3,   6,   9,   9,   9,   9,   9,   9,   9 ; 800 
	FCB    0,   0,   0,   0,   0,   0,   0,   9,   9,   9,   9,   9,   9 ;1600  		
   	FCB    0,   0,   0,   0,   0,   0,   0,   9,   9,   9,   9,   9,   9 ;2400          
	FCB    0,   0,   0,   3,   3,   6,   9,  11,  11,  11,  11,  11,  11 ;3200
	FCB    0,   0,   0,   3,   3,   6,   9,  11,  11,  11,  11,  11,  11 ;4000
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Digital linear EGR pintle position control calib.
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
L489A   FCB  0			;0, A/D val for EGR pintle pos. scalar
L489B   FCB  0			;Max allowed closed EGR pintle pos. 
L489C   FCB  0	    		;Min allowed closed EGR pintle pos.					
L489D   FCB  0	    		;Filter coeff. for EGR pinle position, EGR closing/closed 
L489E   FCB  0	    		;0, filter coeff. for %EGR
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-EGR init vs. % desired EGR
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;	
L489F   FCB 0			;# lines/table
;
;	Init val for EGR int.	;% desired EGR
;
	FCB 0  			;  0
	FCB 0  			; 25
	FCB 0  			; 50   
	FCB 0  			; 75     
	FCB 0  			;100    
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-EGR integral gain vs EGR error
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
L48A5   FCB 0			;# lines/table
;
;	gain = val/256		;%EGR error
;
	FCB 0   		;  0
	FCB 0   		; 25  
	FCB 0   		; 50     
	FCB 0   		; 75     
	FCB 0			;100 
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-EGR proprortional gain vs. EGR error 
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
L48AB   FCB 0   		;# lines/table 
;
;	gain = val/128		;%EGR error
;
	FCB 0   		;  0    
	FCB 0   		; 25     
	FCB 0   		; 50    
	FCB 0   		; 75     
	FCB 0 			;100 
;
;~~~~~~~~~~~~~~~
;-O2 parameters
;~~~~~~~~~~~~~~~
;
L48B1   FCB  15			;65 mV, lower limit for slow filtered O2
L48B2   FCB  208    		;903 mV, upper limit for slow filtered O2
L48B3   FCB  3			;Slow O2 filter coeff for idle
L48B4   FCB  104		;450 mV, init value for O2
;
;~~~~~~~~~~~~~~~~~~~~~~~~~
;-MAP Accel Enrich params
;~~~~~~~~~~~~~~~~~~~~~~~~~
;
L48B5   FCB  16			;5 kPa, delta MAP threshold for AE
L48B6   FCB  4      		;1.3 kPa, delta MAP threshold for AE if idling 
L48B7   FCB  204		;80% TPS, TPS treshold to double delta MAP AE threshold  
L48B8   FCB  255		;TPS threshold to double MAP AE
;
;~~~~~~~~~~~~~~~~~~~~~~~~~
;-TPS Accel Enrich params
;~~~~~~~~~~~~~~~~~~~~~~~~~
;
L48B9   FCB  5			;2%, TPS threshold for TPS AE
L48BA   FCB  147		;1.15, TPS AE off-idle multiplier
L48BB   FCB  5      		;MPH threshold to apply off-idle TPS AE mult.
;
;~~~~~~~~~~~~~~~~~~~~~
;-Misc. AE parameters
;~~~~~~~~~~~~~~~~~~~~~
;
L48BC   FCB  100		;6.104 msec, post DFCO AE pulse
L48BD   FCB  20			;1.2 msec A/C off to on accel enrich
L48BE   FCB  1			;0.1 sec, execution interval to filter MAP/TPS
L48BF   FCB  26			;Slow filtered TPS filter coeff
L48C0   FCB  10			;.125 sec, time for INT to remain locked after AE
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Closed loop fueling quals
;~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
L48C1   FCB  120		;50 deg C, upper temp limit for extended closed loop delay
L48C2   FCB  203		;112 deg C, initial cool temp threshold for hot restart
;
L48C3   FCB  15			;30 sec, min time for closed loop (cool engine)
L48C4   FCB  60			;120 sec, min time for closed loop (cold engine)
L48C5   FCB  15			;30 sec, min time for full closed loop (hot restart)
;
L48C6   FCB  56			;2 deg C, min temp for closed loop
;
L48C7   FCB  253		;150 deg C, upper limit for AFR lookup in closed loop 
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Max time O2 allowed to remain in window to be functional
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
L48C8   FCB  50			;10 secs, max time for O2 to dwell in window
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~
;-BLM update enable params
;~~~~~~~~~~~~~~~~~~~~~~~~~
;
L48C9   FCB  250		;78.75 kPa, upper vac limit for BLM upate
;
L48CA   FCB  21			;18.9 Kpa, lower MAP limit for BLM update
L48CB   FCB  240		;98.9 kPa, upper MAP limit for BLM update
;
L48CC   FCB  107		;40 deg C, min temp for BLM update enable
L48CD   FCB  140		;3500 RPM, upper RPM limit for BLM update enable
;
;~~~~~~~~~~~~~~~~~
;-Idle parameters
;~~~~~~~~~~~~~~~~~
;
L48CE   FCB  3			;3 MPH, upper limit for idle VE 
L48CF   FCB  4			;4 MPH, max speed to be idling 
L48D0   FCB  7			;2.7% TPS, threshold for idling
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Params for derivative idle RPM control
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
L48D1   FCB  5			;2%, max TPS for derivative RPM calc.
L48D2   FCB  2			;2 MPH, max speed for derivative RPM calc. 
L48D3   FCB  153		;75 deg C, min temp for derivative RPM calc.  
;
;~~~~~~~~~~~~~~~~~~~~~~
;-Open loop idle quals
;~~~~~~~~~~~~~~~~~~~~~~
;
L48D4   FCB  255		;255 sec, timer threshold to enable open loop idle
L48D5   FCB  0			;0 secs, timer threshold to transition to open loop idle AFR
L48D6   FDB  0015		;15 sec, time after startup to clear initial idle terms
L48D8   FCB  100		;35 dec C, threshold to remain in open loop idle
;
;~~~~~~~~~~~~~~~~~~~~~~
;-Open loop AFR params
;~~~~~~~~~~~~~~~~~~~~~~
;
L48D9   FCB  173		;90 deg C, temp threshold for open loop idle AFR bias
L48DA   FCB  0			;-40 deg C, lower cool temp for open loop AFR enrichment
L48DB   FCB  254		;151 dec C, upper cool temp for open loop AFR enrichment  
;
L48DC   FCB  200		;10 secs, time limit for initial AFR enrichment
;
L48DD   FCB  147		;14.7:1, stoich AFR 
;
L48DE   FCB  16			;1.6:1 AFR bias for open loop idle
				; Used prior to TPS action after start up
;
L48DF   FCB  147		;14.7:1 AFR, lower limit for quasi-closed loop
L48E0   FCB  10			;1.0:1, onitiial AFR enrichemnt  
L48E1   FCB  5			;0.5:1, initial AFR enrichment if in park/neutral
L48E2   FCB  6			;Filter coeff. for filtering AFR after transition from idle
;
;~~~~~~~~~~~~
;-BLM params
;~~~~~~~~~~~~
;
L48E3   FCB  8			;450 msec, min time to update BLMS
;
;                   _______|______|_______|______
;                          |      |       |
;                     12   |  13  |  14   |  15    
;                   _______|______|_______|______  80 Kpa MAP
;                          |      |       |
;                      8   |  9   |   10  |  11     
;                   _______|______|_______|______  50 Kpa MAP
;                          |      |       |
;                      4   |  5   |   6   |   7     
;                   _______|______|_______|______  31 Kpa MAP
;                          |      |       |
;                      0   |  1   |   2   |   3
;                   _______|______|_______|______
;                          |      |       |       
;         900  RPM   ______|      |       |
;                                 |       |
;        1600  RPM   _____________|       |
;                                         |
;        2200  RPM   _____________________|
;
;-RPM boundries for BLM cells
;
L48E4	FCB 36			; 900 RPM 
L48E5	FCB 64			;1600 RPM   
L48E6	FCB 88			;2200 RPM  
;
;-MAP boundries for BLM cells
;
L48E7	FCB 36			;23.6 kPa 
L48E8	FCB 96			;45.8 kPa 
L48E9	FCB 192			;81.2 kPa 
;
L48EA	FCB 3			;75 RPM, RPM hyst. to change BLM cells
L48EB	FCB 8			;2 kPa, SD MAP hyst. to change BLM cells  
;
L48EC	FCB $07			;0000 0111, Option byte
				; b7 1 = not used
                		; b6 1 = not used
                		; b5 1 = not used
                		; b4 1 = not used
                		; b3 1 = not used
                		; b2 1 = SEPARATE BLM PK/NEUT CELLS
                		; b1 1 = SEPARATE BLM A/C IDLE CELLS
                		; b0 1 = SEPARATE BLM IDLE CELLS
;
L48ED	FCB 4			;Delta INT threshold to update BLMs
L48EE	FCB 3			;Delta INT threshold to bar open loop idle
;
L48EF	FCB 105			;105, min allowed BLM 
L48F0	FCB 90			;90, Min allowed idle BLM
L48F1	FCB 1			;1, BLM step value 
L48F2	FCB 165			;160, Max allowed BLM
;
;-BLM init limits
;
L48F3	FCB 135			;Upper limit on BLM at init      
L48F4	FCB 126			;Lower limit on BLM at init
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-O2 window boundries to enable learn
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; Used to make sure O2 doesnt dwell around stoich, 
; indicating a fault with the O2 sensor. If O2
; dwells, learn is disabled after period of time.
;
;-Closed loop O2 window
;
L48F5	FCB 156			;677 mV, upper bound
L48F6	FCB 64			;277 mV, lower bound
;
;-Open loop O2 window
;
L48F7	FCB 166			;720 mV, upper bound
L48F8	FCB 64			;277 mV, lower bound 
;
;-Closed loop idle O2 window
;
L48F9	FCB 161			;720 mV, upper bound
L48FA	FCB 81			;352 mV, lower bound  
;
;-Open loop idle O2 window
;
L48FB	FCB 169			;733 mV, upper bound 
L48FC   FCB 81			;352 mV, lower bound  
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-INTegrator anti-windup params
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;-Params to set BLM cell change and cause INT reset
;
L48FD   FCB 120			;Lower INT thresh to flag BLM cell change when BLM rising
L48FE   FCB 133			;Upper INT thresh to flag BLM cell change when BLM falling
;
L48FF   FCB 3			;Delta BLM thresh to clear BLM cell change if going leaner 
L4900   FCB 5			;Delta BLM thresh to clear BLM cell change if going richer
;
L4901   FCB 48			;28.1 kPa lower MAP treshold for INT reset 
;
L4902	FCB 48			;1200 RPM, lower threshold not to reset INT   
L4903   FCB 100			;2500 RPM, upper treshold not to reset INT 
;
L4904   FCB 15			;15 MPH, speed threshold to reset INT   
L4905   FCB 124			;124, INT threshold to reset INT on VE table change
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-INT/Proportional term limits
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
L4906	FCB 40			;40, Min allowed INT  
L4907   FCB 158			;158, Max allowed INT		
L4908   FCB 10			;Proportional term limit when AE in effect      
;
L4909	FCB 160			;Lean AFR limit
;
;~~~~~~~~~~~~~~~~~~~~~
;-Power enrich params
;~~~~~~~~~~~~~~~~~~~~~
;
L490A	FCB 250			;3125 RPM, delta RPM value to bypass PE delay 
;
L490B   FCB 120			;50 deg C, lower cool temp threshold to bypass PE delay/
				;allow for AFR adjustments
L490C   FCB 193			;105 deg C, upper cool temp thresh to bypass PE delay
;
L490D   FCB 13			;5.1%, TPS hysteresis to remain in PE
L490E   FCB 170			;4259 RPM, upper RPM threshold for PE delay/AFR adj. 
L490F   FCB 15			;15 sec, time delay to reset after in PE
;
L4910   FCB 3			;150 msec, delay for PE to become active
L4911   FCB 150			;58.5%/pass, PE ramping rate
L4912   FCB 153			;74 deg C, cool temp thresh for cold PE
;;
L4913   FCB 50			;50 secs, threshold for extended PE\
;
L4914   FCB 195			;106c COOL, upper temp thresh to allow PE AFR adjustments
L4915	FCB 70			;1750 RPM, lower RPM limit for colt PE AFR enrich 
L4916	FCB 25			;25 MPH, MPH limit for cold PE AFR enrichment
;
L4917   FCB 12			;12 MPH min speed for delta MPH fast PE entry
L4918   FCB 255			;99.6%, TPS threshold for delta MPH fast PE entry
L4919   FCB 0       		;Timer thresh to continue to bar fast delta MPH PE entry
;
L491A   FCB 7			;0.7 AFR, cold PE AFR enrichment   
L491B   FCB 15			;1.5 AFR, base PE enleanment AFR  
L491C   FDB 0000		;0 MPH, delta MPH threshold for PE 
;
;~~~~~~~~~~~~~~~~~~~~~~
;-Choke AFR parameters
;~~~~~~~~~~~~~~~~~~~~~~
;
L491E   FCB 243			;95%, choke AFR decay multiplier
L491F   FCB 10			;2 sec, time after startup to decay choke AFR 
;
;~~~~~~~~~~~~~~~~~~~~~~~
; Async/Sync BPW params
;~~~~~~~~~~~~~~~~~~~~~~~
;
L4920	FDB 0046		;702 usec, threshold for asnyc to sync BPW  
				;Also threshold for double to single fire if CPI
L4922   FDB 0033		;500 usec, threshold to transition form double 
				;to single fire mode, CPI/PFI 
;
;-Manditory async thresholds, TBI only
;	
L4924   FCB 96			;45.8 kPa, MAP threshold to transition back to sync fire   
L4925   FCB 48			;1200 RPM, RPM threshold to transition back to sync fire
;
L4926   FCB 112			;51.7 kPa, MAP threshold to enter async fire
L4927   FCB 52			;1300 RPM, RPM threshold to enter async fire
;
L4928   FDB 0786		;12 msec, max async BPW per pass
L492A   FDB 0033		;500 usec, min async BPW 
;
;~~~~~~~~~~~~~~~~~~~~~
;-Decel enlean params
;~~~~~~~~~~~~~~~~~~~~~
;
L492C   FCB 12			;-4.7%, delta TPS threshold for decel enlean  
L492D   FCB 12      		;-14.8 kPa, delta MAP threshold for decel enlean 
;
L492E   FCB 128			;Multiplier for low speed decel enlean  
L492F   FCB 6			;6 MPH, MPH threshold for low speed decel enlean
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Decel Fuel Cut Off params
;~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
L4930	FCB 8			;200 RPM, DFCO RPM hyst.    
;
L4931	FCB 15			;15 MPH, min MPH to enter DFCO   
L4932	FCB 10			;10 MPH, min MPH to remain in DFCO
;
L4933	FCB 8			;13.3 kPa, Lower MAP threshold for manditory DFCO
L4934	FCB 56			;31.0 kPa, Upper MAP limit for DFCO
;
L4935	FDB 0506		;7.72 msec, additional AE if exiting DFCO 
L4937	FCB 65			;6.5 sec, min time to enter DFCO
L4938	FCB 32			;400 msec, duration to sustain post DFCO PW   
L4939	FCB 2			;0.8% per 12.5 msec, decay rate for DFCO multiplier
;
L493A	FCB 127			;55 deg C, min temp for DFCO 
L493B   FCB 254			;3175 RPM, delta RPM threshold for DFCO 
L493C   FCB 254     		;80 kPa, delta SD MAP threshold for DFCO 
;
;~~~~~~~~~~~~~~~
;-O2 PID params
;~~~~~~~~~~~~~~~
;
L493D   FCB 30			;130 mV, O2 mV hyst. for rich/lean  
;
L493E   FCB 6			;6 counts, min slow O2 error to update INT
;  
L493F   FCB 230			;3.59, scalar for airflow used in PID tables
;
;-Idle O2 params
;
L4940   FCB 110			;478 mV, idle rich O2 threshold
L4941   FCB 110			;478 mV, idle lean O2 threshold 
L4942   FCB 110			;478 mV, mean idle rich/lean O2 threshold
;
L4943   FCB 3			;75 msec, proportional duration if idling
;   
L4944   FCB 20			;500 msec, Proportional duration dead band
;
;~~~~~~~~~~~~~~~~~~~~
;-O2 PID idle params
;~~~~~~~~~~~~~~~~~~~~
;
;-Proportional gains if unstable idle present
;
L4945   FCB 4			;0.016, proportional gain if rich  
L4946   FCB 5			;0.020, proportional gain if lean
;
;-Proportional gains if stable idle present
;
L4947   FCB 16			;0.063, proportional gain if rich
L4948   FCB 20			;0.078, proportional gain if lean
				;(Also indexed for INT delay term)  
;
;-Integrator delay for stable idle 
;
L4949   FCB 20			;500 msec, INT delay term	
;
;-Delta RPM threshold for unstable idle
;				
L494A   FCB 2			;25 RPM, delta RPM threshold for unstable idle  
;		
L494B   FCB 255			;0.996, factor applied to O2 error if going richer 
L494C   FCB 255			;0.996, factor applied to O2 error if idling 
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Cold park to drive choke AFR params
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
L494D   FCB 40			;-10 deg C, upper cool temp limit for cold choke park to
				;drive transition    
L494E   FCB 25			;2.5 AFR, lower choke AFR limit during cold park to drive
				;transition
L494F   FCB 25			;5 sec, Park to drive transition period 
;
;~~~~~~~~~~~~~~~~~~~~
;-Hot restart params
;~~~~~~~~~~~~~~~~~~~~
;
L4950   FCB 180			;95 deg C, min previous shutdown temp for hot restard  
L4951   FCB 200     		;110 deg C, min temp for hot restart  					
L4952   FCB 10      		;1:1, AFR bias for hot restart
L4953   FDB 0015    		;15 secs, min time to clear hot restart status 
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Crank to run transition params
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
L4955   FCB 0			;0 DRPs, DRP threshold to transition from crank to run
L4956   FCB 0			;Mult/128, crank BPW mult. for 2nd inj. firing
L4957   FCB 0			;-40c, threshold to add AFR transition ratio in
L4958   FDB 0800		;10 sec., max time to run pump with no DRPs
L495A	FCB 3			;3 DRPs, number of DRPs between each execution of 
				;crank to run transition routine
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Speed/RPM limiter params
;~~~~~~~~~~~~~~~~~~~~~~~~~~
;
L495B	FCB 40      		;.5 sec delay to cut fuel if high speed present  
L495C	FCB 40			;1000 RPM, thresold to re-enable fuel if MPH overspeed  
;
L495D	FCB 192			;4800 RPM, RPM threshold to enter fuel cut off
L495E	FCB 164     		;4100 RPM, RPM threshold to exit fuel cut off    
;
L495F	FCB 172     		;4300 RPM, threshold to enter fuel cutoff if trans errors
L4960	FCB 154			;3850 RPM, threshold to exit fuel cutoff if trans errors
;
L4961	FCB 98			;98 MPH, speed threshold to enter fuel cut off
L4962	FCB 96      		;96 MPH, speed threshold to exit fuel cut off
;
;~~~~~~~~~~~~
;-BPW limits
;~~~~~~~~~~~~
;
L4963	FDB 0026		;397 usec, PW threshold for short BPW
L4965	FDB 0026		;397 usec, min allowed BPW
;
L4967	FDB 0026		;397 usec, BPW offset  (used with BPW bias table) 
;
L4969	FCB 0			;Deg = val/128, degree delay from DRP to inj. firing
;
L496A	FCB 0			;Min time to transition back to single fire mode
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-RPM threshold for PE vs. baro kPa
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;	RPM			;kPa
;
L496B   FCB 0			; 75
	FCB 0			; 85
	FCB 0			; 95
	FCB 0			;105
;
;~~~~~~~~~~~~~~~~~~~~~~~~
; Added BPW bias vs. BPW
;~~~~~~~~~~~~~~~~~~~~~~~~
;
; L400E, b4=1 selects use of table
;
;	usecs = val x 15.26	;usec PW
;
L496F   FCB 26			; 488 
	FCB 26			; 732
	FCB 26  		; 976
	FCB 26			;1220
	FCB 26			;1460
	FCB 26			;1708
	FCB 26			;1950
	FCB 26			;2197
	FCB 26			;2440
	FCB 26			;2685
	FCB 26			;2929
	FCB 26			;3170
	FCB 26			;3410
	FCB 26			;3660
	FCB 26			;3900 
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-BPW mult. vs. fuel pump volts
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;	mult. = val/128		
;
L497E   FCB  128		; 0.0
	FCB  128    		; 1.6
	FCB  160		; 3.2
	FCB  160		; 4.8
	FCB  160		; 6.4
	FCB  147		; 8.0
	FCB  147    		; 9.6
	FCB  141    		;11.2
	FCB  137   		;12.8
	FCB  128		;14.4
	FCB  126		;16.0
	FCB  122		;17.6
	FCB  120		;19.2
	FCB  118		;20.8
	FCB  116		;22.4
	FCB  114		;24.0
	FCB  112		;25.6
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Cat overtemp AFR limit vs. airflow
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;	AFR = val/10		;gms/sec
;
L498F   FCB  145		;  0
	FCB  130		; 16
	FCB  130		; 32
	FCB  128		; 48
	FCB  128		; 64
	FCB  128		; 80
	FCB  128		; 96
	FCB  128		;112
	FCB  128		;128
	FCB  128		;144
	FCB  128		;160
	FCB  128		;176
	FCB  128		;192
	FCB  128		;208
	FCB  128		;224
	FCB  128		;240
	FCB  128		;256
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-BPW alt. correction factor table
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
L49A0	FCB   0         ;10 kPa MAP, min row value
    	FCB   96        ;75 kPa baro, min col. value
    	FCB   4         ;# cols per row
;
; BPW corr. factor = value/128
;
;  baro kPa:  75   85   95  105 ;MAP kPa
;
	FCB  155, 137, 128, 128 ;10
	FCB  155, 136, 128, 128 ;20
	FCB  148, 135, 128, 128 ;30
	FCB  146, 134, 128, 128 ;40 
	FCB  145, 130, 128, 128 ;50
	FCB  138, 128, 128, 128 ;60
	FCB  135, 128, 128, 128 ;70
	FCB  135, 128, 128, 128 ;80
	FCB  135, 128, 128, 128 ;90
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Alt. comp. value for MAP
;~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; kPa = .625 x val + 5
;
;				;baro kPa
;	
L49C7	FCB 120			; 79.2
	FCB 136			; 89.2
	FCB 152			; 99.6
	FCB 168			;110.4
;
;~~~~~~~~~~~~~~~~~~~~~~~~
;-Open throttle VE table
;~~~~~~~~~~~~~~~~~~~~~~~~
;
L49CB	FCB   16		;400 RPM, min row val. for lookup
	FCB   0			;0 kPa, min MAP val. for lookup
	FCB   11		;# cols per row
;
; VE = val. x .390
;
;	kPa:  20   25	  30    35    40    50    60    70    80    90   100 ; RPM
;
	FCB   80,  105,  118,  130,  135,  159,  161,  167,  173,  180,  200 ; 400 
	FCB   95,  116,  144,  163,  173,  177,  183,  186,  190,  197,  210 ; 800
	FCB  122,  151,  175,  188,  190,  197,  207,  206,  210,  212,  214 ;1200
	FCB  143,  183,  193,  197,  212,  219,  225,  225,  225,  226,  220 ;1600
	FCB  150,  199,  204,  208,  210,  218,  224,  232,  232,  232,  235 ;2000
	FCB  150,  205,  210,  212,  214,  225,  232,  242,  242,  242,  240 ;2400
	FCB  150,  207,  210,  214,  215,  225,  235,  237,  238,  238,  235 ;2800
	FCB  153,  190,  205,  211,  212,  224,  242,  240,  240,  242,  242 ;3200
	FCB  150,  185,  190,  197,  207,  224,  230,  236,  240,  240,  245 ;3600
	FCB  125,  185,  190,  195,  207,  222,  235,  233,  231,  233,  220 ;4000
	FCB  125,  185,  190,  190,  203,  215,  220,  225,  230,  230,  230 ;4400
	FCB  125,  185,  190,  190,  193,  205,  210,  215,  220,  222,  222 ;4800
	FCB  125,  166,  190,  192,  193,  205,  210,  225,  230,  236,  236 ;5200
	FCB  125,  166,  190,  192,  193,  205,  210,  225,  230,  236,  236 ;5600
	FCB  125,  166,  190,  192,  193,  205,  210,  225,  230,  236,  236 ;6000
	FCB  125,  166,  190,  192,  193,  205,  210,  225,  230,  236,  236 ;6400
;
;~~~~~~~~~~~~~~~
;-Idle VE table
;~~~~~~~~~~~~~~~
;
L4A7E	FCB   32		;400 RPM, min row val. for lookup
	FCB   0			;20 kPa, min col. val. for lookup
	FCB   11		;# cols. per row
;
; VE = val. x .390
;
;	kPa:  20   25	  30    35    40    50    60    70    80    90   100 ; RPM
;
	FCB   85,  113,  128,  132,  135,  156,  161,  167,  173,  180,  200 ; 400
	FCB   95,  115,  137,  149,  153,  166,  175,  185,  186,  195,  205 ; 600
	FCB  105,  127,  156,  165,  168,  172,  189,  195,  200,  210,  210 ; 800
	FCB  115,  148,  177,  180,  179,  185,  199,  203,  208,  217,  212 ;1000
	FCB  128,  166,  195,  197,  198,  203,  209,  212,  217,  224,  214 ;1200
	FCB  138,  177,  197,  200,  201,  210,  214,  218,  223,  226,  218 ;1400
	FCB  146,  192,  202,  206,  207,  218,  219,  224,  230,  229,  222 ;1600
	FCB  151,  195,  205,  207,  210,  219,  223,  227,  233,  231,  227 ;1800
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-MAT/CTS blend multiplier vs. airflow
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; Larger multiplier = more CTS blended into MAT
;
;	Mult = val/128		;g/sec
;
L4AD9   FCB  0			;  0
	FCB  0			; 16
	FCB  0			; 32
	FCB  0			; 48
	FCB  0			; 64
	FCB  0			; 80
	FCB  0			; 96
	FCB  0			;112
	FCB  0			;128
	FCB  0			;144
	FCB  0			;160
	FCB  0			;176
	FCB  0			;192
	FCB  0			;208
	FCB  0			;224
	FCB  0			;240
	FCB  0			;256
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Inverse air density @ sea level vs. blended intake temp
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; l/g = (value x 128 + 29824) / 45064
;
;	val.			;Deg C
;
L4AEA   FCB  0			;-40
	FCB  12			;-28           
	FCB  24			;-16            
	FCB  36			;- 4            
	FCB  48			;  8            
	FCB  60			; 20         
	FCB  72			; 32           
	FCB  84			; 44
	FCB  96			; 56
	FCB  108		; 68          
	FCB  120		; 80          
	FCB  132		; 92          
	FCB  144		;104          
	FCB  156		;116          
	FCB  168		;128          
	FCB  180		;140         
	FCB  192		;152          
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Injector offset vs. batt volts
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;	usecs = val x 15.26	;Batt. volts	
;
L4AFB   FCB  26			; 0.0
	FCB  26			; 1.6
	FCB  50			; 3.2
	FCB  50 		; 4.8
	FCB  48			; 6.4
	FCB  37			; 8.0
	FCB  31			; 9.6
	FCB  26			;11.2
	FCB  26			;12.8
	FCB  26			;14.4
	FCB  22			;16.0
	FCB  19			;17.6
	FCB  19			;19.2
	FCB  19			;20.8
	FCB  19			;22.4
	FCB  19			;24.0
	FCB  19			;25.6
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Decel enlean factor vs. cool temp
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
L4B0C   FCB 8			;9 line table
;
;	factor = val/32		;Deg C
;    		
L4B0D   FCB  96			;-40
	FCB  80			;-16
	FCB  64			;  8
	FCB  48			; 32
	FCB  32			; 56
	FCB  32			; 80
	FCB  32			;104
	FCB  30			;128
	FCB  28			;152
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Decel enlean PW reduction vs. delta MAP
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;	%reduction = 
;	val x .396		;kPa
;
L4B16   FCB  0			; 0
	FCB  16			; 5
	FCB  32			;10
	FCB  36			;15
	FCB  40			;20
	FCB  48			;25
	FCB  56			;30
	FCB  64			;35
	FCB  72			;40
	FCB  80			;45
	FCB  88			;50
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Decel enlean PW redcutcion vs. delta TPS
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;	%reduction =
;	val x .396		;% TPS
;
L4B21   FCB  0 			;  0.0
	FCB  13			;  6.3
	FCB  26			; 12.5
	FCB  38			; 18.8
	FCB  51			; 25.0
	FCB  77			; 31.3
	FCB  102		; 37.5
	FCB  102		; 43.8
	FCB  102		; 50.0
	FCB  102		; 56.3
	FCB  102		; 62.5
	FCB  102		; 68.8
	FCB  102		; 75.0
	FCB  102		; 81.3
	FCB  102		; 87.5
	FCB  102		; 93.8
	FCB  102		;100.0
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-MAP filter coeff. vs cool temp
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;	Filter coeff.		;Deg C
;
L4B32   FCB  18     		;-28
	FCB  22     		;-16
	FCB  24     		; -4
	FCB  28     		;  8
	FCB  35     		; 20
	FCB  40     		; 32
	FCB  45     		; 44
	FCB  49     		; 56
	FCB  52     		; 68
	FCB  55     		; 80
	FCB  57     		; 92
	FCB  59     		;104
	FCB  63     		;116
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Sync accel enrich vs. delta MAP
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
L4B3F	FCB  4			;5 line table
;
	FCB  5			; 0
	FCB  10			;20
	FCB  12			;40
	FCB  17			;60
	FCB  20			;80
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Accel enrich vs. delta TPS
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;	usecs = val x 15.26	;Delta %TPS
;
L4B45   FCB  1			; 0.0
	FCB  8			; 3.1
	FCB  15			; 6.3
	FCB  30			; 9.4
	FCB  35			;12.5
	FCB  38			;15.6
	FCB  41			;18.8
	FCB  44			;21.9
	FCB  47			;25.0
	FCB  49			;28.1
	FCB  51			;31.3
	FCB  53			;34.4
	FCB  55			;37.5
	FCB  57			;40.6
	FCB  62			;43.8
	FCB  66			;46.9
	FCB  69			;50.0
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Delta TPS AE mult. factor vs. Baro
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;	Factor=val/128		;kPa		
;
L4B56	FCB 99			; 80
	FCB 121			; 90
	FCB 128			;100
	FCB 128			;110
;

;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-AE TPS filter coeff. vs cool temp
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;	corr. = val/256		;Deg C
;
L4B5A	FCB  38     		;-28
	FCB  40 		;-16
	FCB  42     		; -4
	FCB  44     		;  8
	FCB  46     		; 20
	FCB  56     		; 32
	FCB  64     		; 44
	FCB  76     		; 56
	FCB  102    		; 68
	FCB  128    		; 80
	FCB  150    		; 92
	FCB  160    		;104
	FCB  165    		;116
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Async accel enrich temp correction vs. cool temp
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;	factor=val/32		;Deg C
;
L4B67   FCB  192		;-40
	FCB  172		;-28
	FCB  163		;-16
	FCB  153		; -4
	FCB  132		;  8
	FCB  110		; 20
	FCB  93			; 32
	FCB  70			; 44
	FCB  54			; 56
	FCB  43			; 68
	FCB  33			; 80
	FCB  32			; 92
	FCB  32			;104
	FCB  24			;116
	FCB  24			;128
	FCB  24			;140
	FCB  24			;152
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-TPS accel entrich temp correction factor vs. MAT
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;	Factor = val/128	;Dec C	
;
L4B78   FCB  128		;-40
	FCB  128		;-28
	FCB  128		;-16
	FCB  128		; -4
	FCB  128		;  8
	FCB  128		; 20
	FCB  128		; 32
	FCB  128		; 44
	FCB  128		; 56
	FCB  128		; 68
	FCB  128		; 80
	FCB  128		; 92
	FCB  128		;104
	FCB  128		;116
	FCB  128		;128
	FCB  128		;140
	FCB  128		;152
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Sync Accel enrich temp correction vs cool temp
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;	mult. = val/32		;Cool temp
;
L4B89   FCB  160		;-40
	FCB  144		;-28
	FCB  128		;-16
	FCB  112		;- 4
	FCB  96			;  8
	FCB  88			; 20
	FCB  72			; 32
	FCB  56			; 44
	FCB  48			; 55
	FCB  40			; 67
	FCB  34			; 80
	FCB  32			; 91
	FCB  32			;103
	FCB  28			;115
	FCB  24			;127
	FCB  24			;139
	FCB  24			;150
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Delta MAP AE mult. vs RPM
;~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
L4B9A   FCB  128		;3200 RPM, upper lim for lookup
;
;	mult = val/128		;RPM	
;
L4B9B   FCB  215		; 400
	FCB  144		; 800
	FCB  107		;1200
	FCB  78			;1600
	FCB  62			;2000
	FCB  50			;2400
	FCB  45			;2800
	FCB  38			;3200
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Lean open loop AFR limit vs cool temp
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;		AFR x 10	;Deg C
;
L4BA3   FCB	120		;-40
	FCB	123    		;-28
	FCB	125    		;-16
	FCB	126    		; -4
	FCB	128    		;  8
	FCB	137    		; 20
	FCB	140    		; 32
	FCB	147    		; 44
	FCB	147    		; 56
	FCB	146    		; 68
	FCB	146    		; 80
	FCB	146    		; 92
	FCB	146    		;104
;
;~~~~~~~~~~~~~~~~~~~~~
;-Open loop AFR table
;~~~~~~~~~~~~~~~~~~~~~
;
L4BB0   FCB   0			;Opt bit for lookup, 0=Vac, 1=MAP
;
	FCB   0 		;0 deg C, min row val. for lookup
        FCB   0			;0 kPa, min vac/map val for lookup
        FCB   9			;# of cols. per row
;
;	Vac: 80   70   60   50   40   30   20   10    0 ;Deg C
;	
	FCB 125, 125, 125, 125, 125, 120, 115, 105,  95 ;-40 
	FCB 130, 130, 130, 130, 130, 125, 120, 110, 100 ;-28
	FCB 132, 132, 132, 132, 132, 127, 122, 112, 102 ;-16
	FCB 140, 140, 140, 140, 140, 135, 130, 120, 110 ;- 4
	FCB 145, 145, 145, 145, 145, 140, 135, 125, 115 ;  8
	FCB 160, 160, 160, 150, 145, 140, 140, 135, 130 ; 20
	FCB 160, 160, 160, 150, 145, 140, 140, 135, 130 ; 32
	FCB 160, 160, 160, 150, 145, 140, 140, 135, 130 ; 44
	FCB 160, 160, 160, 150, 145, 140, 140, 135, 130 ; 56
	FCB 160, 160, 160, 155, 150, 145, 140, 140, 130 ; 68
	FCB 160, 160, 160, 155, 150, 145, 140, 140, 130 ; 80
	FCB 160, 160, 160, 155, 150, 145, 140, 140, 130 ; 92
	FCB 160, 160, 160, 155, 150, 145, 140, 140, 130 ;104
;
;~~~~~~~~~~~~~~~~~~~~~~~~~
;-Choke AFR vs. cool temp
;~~~~~~~~~~~~~~~~~~~~~~~~~
;
; AFR is subtracted out to enrich the initial AFRs after startup
;
;	AFR = val/10		;Deg C
;
L4C29   FCB  80			;-40
	FCB  75			;-28
	FCB  70			;-16
	FCB  65			; -4
	FCB  50			;  8
	FCB  35			; 20
	FCB  30			; 32
	FCB  30			; 44
	FCB  30			; 56
	FCB  25			; 68
	FCB  20			; 80
	FCB  20			; 92
	FCB  20			;104
	FCB  20			;116
	FCB  20			;128
	FCB  20			;140
	FCB  20			;152
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Choke AFR decay period vs. cool temp
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;	period = 5/val		;Deg C
;
L4C3A   FCB  14			;-40
	FCB  14			;-28
	FCB  14			;-16
	FCB  13			; -4
	FCB  12			;  8
	FCB  11			; 20
	FCB  11			; 32
	FCB  11			; 44
	FCB  10			; 56
	FCB  10			; 68
	FCB  10			; 80
	FCB  10			; 92
	FCB  10			;104
	FCB  10			;116
	FCB  10			;128
	FCB  10			;140
	FCB  10			;152
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Choke AFR decay period mult. vs. airflow
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;	mult = val/128		;gms/sec
;
L4C4B   FCB  166		; 0
	FCB  134		;16
	FCB  96			;32
	FCB  64			;48
	FCB  32			;64
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-TPS threshold for WOT vs. RPM
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; PE entry is also expedited when TPS is above these
; values
;        		
L4C50   FCB 160			; 4000 RPM UPPER LMT
;
;	%TPS = 
;	value/.391		;RPM
;
	FCB  128		; 800
	FCB  192		;1200
	FCB  217		;1600
	FCB  217		;2000
	FCB  217		;2400
	FCB  217		;2800
	FCB  217		;3200
	FCB  217		;3600
	FCB  217		;4000
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-TPS threshold for PE enable vs. RPM
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
L4C5A   FCB  160    		;4000 RPM, UL for lookup
;
;	%TPS = val / .391	;RPM	
;
	FCB  102		; 800
	FCB  154		;1200
	FCB  141		;1600
	FCB  110		;2000
	FCB  110		;2400
	FCB  92			;2800
	FCB  92			;3200
	FCB  92			;3600
	FCB  85			;4000
;
;~~~~~~~~~~~~~~~~
;-PE AFR vs. RPM
;~~~~~~~~~~~~~~~~
;
;	AFR=val/10		;RPM		
;
L4C64   FCB  122		;   0
	FCB  122		; 400
	FCB  123		; 800
	FCB  124		;1200
	FCB  125		;1600
	FCB  126		;2000
	FCB  124		;2400
	FCB  121		;2800
	FCB  121		;3200
	FCB  124		;3600
	FCB  125		;4000
	FCB  126		;4400
	FCB  126		;4800
	FCB  126		;5200
	FCB  126		;5600
	FCB  126		;6000
	FCB  126		;6400
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Fast PE entry delay reduction vs. RPM
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;
L4C75   FCB 160			; 4000 RPM, upper limit for lookup
;
; Table represents fraction of time needed
; to enter PE relative to the time normally 
; needed. A value of 10 would be 1/10 the 
; time needed to normally enter PE when 
; these values are used instead to compute 
; the delay for PE. Value @ L490F deterimines
; time to enter/exit PE
;
;	Fract = 1/val		;RPM	
;
L4C76   FCB  50			; 800
	FCB  25			;1200
	FCB  25			;1600
	FCB  25			;2000
	FCB  25			;2400
	FCB  25			;2800
	FCB  25			;3200
	FCB  25			;3600
	FCB  25			;4000
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-PE TPS baro factor vs baro
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; Factor is delta TPS needed
; to bypass PE delay
;
;	delta TPS=
;	val/.391		;kPa
;
L4C7F   FCB 254			; 80
        FCB 254			; 90
        FCB 254			;100
        FCB 254			;110
;

    
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Min DFCO RPM vs cool temp
;~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;	RPM/25			;Deg C
;
L4C83   FCB  68     		;-28
	FCB  68     		; -4
	FCB  62     		; 20
	FCB  54     		; 44
	FCB  48     		; 68
	FCB  48     		; 92
	FCB  48     		;116
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-TPS threshold for DFCO vs. RPMs
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
L4C8A   FCB 144			;3600 RPM, UL for lookup 
;
;	%TPS = val x .369	;RPM
;
	FCB  3			; 800
	FCB  4			;1200
	FCB  5			;1600
	FCB  7			;2000
	FCB  10			;2400
	FCB  13			;2800
	FCB  13			;3200
	FCB  13			;3600
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Crank to run transition AFR vs. cool temp
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; Added or subtracted depending on cool temp threshold in L4957
;
;	AFR = val/10		;Deg C	
;
L4C93   FCB  180		;-40
	FCB  170		;-28
	FCB  160		;-16
	FCB  150		; -4
	FCB  100		;  8
	FCB  90			; 20
	FCB  85			; 32
	FCB  80			; 44
	FCB  65			; 56
	FCB  60			; 68
	FCB  50			; 80
	FCB  0			; 92
	FCB  0			;104
	FCB  0			;116
	FCB  0			;128
	FCB  0			;140
	FCB  0			;152
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Crank to run AFR decay rate vs. cool temp
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; 	AFR=val./10   		;Deg C
;
L4CA4	FCB  20			;-40
	FCB  20     		;-28
	FCB  20     		;-16
	FCB  20     		; -4
	FCB  20     		;  8
	FCB  20     		; 20
	FCB  20     		; 32
	FCB  20     		; 44
	FCB  20     		; 56
	FCB  20    		; 68
	FCB  20     		; 80
	FCB  20     		; 92
	FCB  20     		;104
	FCB  10     		;116
	FCB  10     		;128
	FCB  10     		;140
	FCB  10     		;152
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Crank to run AFR decay delay DRPs vs. cool temp
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;	Elapsed DRPs		;Deg C
;
L4CB5   FCB  12			;-40
	FCB  12			;-28
	FCB  12			;-16
	FCB  10			; -4
	FCB  8			;  8
	FCB  6			; 20
	FCB  6			; 32
	FCB  6			; 44
	FCB  6			; 56
	FCB  6			; 68
	FCB  5			; 80
	FCB  0			; 92
	FCB  0			;104
	FCB  0			;116
	FCB  0			;128
	FCB  0			;140
	FCB  0			;152
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Crank to run AFR decay delay counts multiplier vs DRPs
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;	mult. = val./256	;Accumulated DRPs prior to run
;
L4CC6   FCB  0			; 0
	FCB  128		; 4
	FCB  192		; 8
	FCB  204		;12
	FCB  255		;16
	FCB  255		;20
	FCB  255		;24
	FCB  255		;28
	FCB  255		;32 
	FCB  255		;36
	FCB  255		;40
	FCB  255		;44
	FCB  255		;48
	FCB  255		;52
	FCB  255		;56
	FCB  255		;60
	FCB  255		;64
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Integrator delay vs. scaled airflow*
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;	msec=valx25		;Airflow counts
;
L4CD7   FCB  16			;  0
	FCB  15			; 16
	FCB  14			; 32
	FCB  13			; 48
	FCB  12			; 64
	FCB  11			; 80
	FCB  10			; 96
	FCB  9			;112
	FCB  8			;128
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Mean rich/lean threshold vs. scaled airflow*
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;	mV=val x 4.34		;Airflow counts
;	
L4CE0   FCB  102		;  0
	FCB  100		; 16
	FCB  98			; 32
	FCB  98			; 48
	FCB  98			; 64
	FCB  98			; 80
	FCB  98			; 96
	FCB  98			;112
	FCB  98			;128
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Rich O2 threshold vs. scaled airflow*
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;	mV=val x 4.34		;Airflow counts
;
L4CE9   FCB  102		;  0
	FCB  100		; 16
	FCB  98			; 32
	FCB  98			; 48
	FCB  98			; 64
	FCB  98			; 80
	FCB  98			; 96
	FCB  98			;112
	FCB  98			;128
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Lean O2 threshold vs. scaled airflow*
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;	mV=val x 4.34		;Airflow counts
;
L4CF2   FCB  102		;  0
	FCB  100		; 16
	FCB  98			; 32
	FCB  98			; 48
	FCB  98			; 64
	FCB  98			; 80
	FCB  98			; 96
	FCB  98			;112
	FCB  98			;128
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Slow O2 filter coeff. vs scaled airflow*
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;	filter coeff.		;Airflow counts
;
L4CFB   FCB  2			;  0
	FCB  3			; 16
	FCB  4			; 32
	FCB  5			; 48
	FCB  6			; 64
	FCB  9			; 80
	FCB  11			; 96
	FCB  13			;112
	FCB  14			;128
;
;* Note: See 493F for grams/sec airflow scalar used 
;        to scale the airflow for table lookups.
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Base proportional counts vs. O2 error
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;	Counts			;O2 error counts
;
L4D04   FCB  4			; 0
	FCB  4			; 8
	FCB  4			;16
	FCB  4			;24
	FCB  4			;32
	FCB  4			;40
	FCB  4			;48
	FCB  5			;56
	FCB  7			;64
	FCB  8			;72
	FCB  9			;80
	FCB  10			;88
	FCB  11			;96
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Proportional duration vs. O2 error
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;	msec=val x 25		;Error counts
;
L4D11   FCB  20			; 0
	FCB  40			; 8
	FCB  60			;16
	FCB  80			;24
	FCB  100		;32
	FCB  128		;40
	FCB  255		;48
	FCB  255		;56
	FCB  255		;64
	FCB  255		;72
	FCB  255		;80
	FCB  255		;88
	FCB  255		;96
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Added proportional duration vs. scaled airflow*
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;	msec=val x 25		;Airflow counts
;
L4D1E   FCB  5			;  0
	FCB  10			; 16
	FCB  20			; 32
	FCB  255		; 48
	FCB  255		; 64
	FCB  255		; 80
	FCB  255		; 96
	FCB  255		;112
	FCB  255		;128
;
;* Note: See 493F for grams/sec airflow scalar used 
;        to scale the airflow for table lookups.
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Proportional flow gain factor table
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
L4D27   FCB   16		;400 RPM, min row val. for lookup 
        FCB   0			;20 kPa, min col. val. for lookup
        FCB   9			;# cols. per row
;
;       kPa: 20   30   40   50   60   70   80   90  100 ;RPM	
;
	FCB  12,  48,  64, 128, 128, 128, 128, 128, 128	; 400
	FCB  14,  56,  96, 128, 128, 128, 128, 128, 128 ; 800
	FCB  16,  64, 128, 128, 128, 128, 128, 128, 128 ;1200
	FCB  18,  64, 128, 128, 128, 128, 128, 128, 128 ;1600
	FCB  36, 128, 128, 128, 128, 128, 128, 128, 128 ;2000
	FCB  64, 128, 128, 128, 128, 128, 128, 128, 128 ;2400 
	FCB 128, 128, 128, 128, 128, 128, 128, 128, 128 ;2800 
	FCB 128, 128, 128, 128, 128, 128, 128, 128, 128 ;3200 
	FCB 128, 128, 128, 128, 128, 128, 128, 128, 128 ;3600 
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Integrator delay mult. vs. O2 error
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;	mult.=val/256		;O2 error
;
L4D7B  	FCB   255		; 0
	FCB   255		; 8
	FCB   255		;16 
	FCB   255		;24
	FCB   255		;32
	FCB   255		;40
	FCB   255		;48
	FCB   255		;56
	FCB   232		;64
	FCB   212		;72
	FCB   192		;80
	FCB   172		;88
	FCB   152		;96
;
;~~~~~~~~~~~~~~~~~~~~~~~
;-Inj./engine constants
;~~~~~~~~~~~~~~~~~~~~~~~
;
L4D88   FDB  6300		;7.69 g/s, injector flowrate
L4D8A   FCB  159		;.7125L, cylinder volume
;
;~~~~~~~~~~~~~~~~~~~
;-Crank fuel params
;~~~~~~~~~~~~~~~~~~~
;
L4D8B   FCB 15			;DRP threshold to use RPM based crank fuel multipliers
L4D8C   FCB 2			;Init crank DRP counter to 2 on bad shutdown
L4D8D   FCB 2			;2 DRPs, DRP counter limit for fuel pump relay failure
L4D8E   FCB 40			;4 volts, lower FP voltage limit to allow DRP counter to update
L4D8F   FCB 33			;-15 deg C, threshold for cold cranking
;
;~~~~~~~~~~~~~~~~~~~~~~~~
;-Crank PW vs. cool temp
;~~~~~~~~~~~~~~~~~~~~~~~~
;
L4D90   FDB 8093		;Scalar for table
;
;	msecs = 
;	[(val x scalar)/256] x	;Deg C
;	15.26 x 10^-3	
;
L4D92   FCB  254		;-40
	FCB  225		;-28
	FCB  195		;-16
	FCB  165		;- 4
	FCB  140		;  8
	FCB  110		; 20
	FCB  80			; 32
	FCB  53			; 44
	FCB  47			; 56
	FCB  40			; 68
	FCB  32			; 80
	FCB  28			; 92
	FCB  18			;104
	FCB  17			;116
	FCB  15			;128
	FCB  15			;140
	FCB  14			;152
;	
;~~~~~~~~~~~~~~~~~~~~~~~~~
;-Crank fuel mult vs. RPM
;~~~~~~~~~~~~~~~~~~~~~~~~~
;
;	mult. = val/256		;RPM
;
L4DA3  	FCB   255		;  0
	FCB   255		;100
	FCB   192		;200
	FCB   192		;300
	FCB   192		;400
	FCB   192		;500
	FCB   128		;600
	FCB   128		;700
	FCB   128		;800
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Hot restart crank fuel mult. vs RPM
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;	mult. = val/256		;RPM
;
L4DAC  	FCB   255   		;  0
	FCB   255   		;100
	FCB   166   		;200
	FCB   166   		;300
	FCB   128   		;400
	FCB   128   		;500
	FCB   128   		;600
	FCB   128   		;700
	FCB   128   		;800
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Crank fuel mult. vs. baro
;~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;	mult. = val / 128	;kPa
;
L4DB5   FCB  96			; 75
	FCB  115		; 85
	FCB  128		; 95
	FCB  134		;105
;
;~~~~~~~~~~~~~~~~~~~~~~~~~
;-Crank fuel mult. vs TPS
;~~~~~~~~~~~~~~~~~~~~~~~~~
;
L4DB9	FCB   8			;9 line table
;
;	mult. = val./64
;
L4DBA  	FCB   64		;  0.0
	FCB   67		; 12.5
	FCB   70		; 25.0
	FCB   80		; 37.5
	FCB   83		; 50.0
	FCB   83		; 62.5
	FCB   32		; 75.0
	FCB   3			; 87.5
	FCB   2			;100.0
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Cranking fuel delay vs. cool temp
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;	Delay			;Deg C
;
L4DC3   FCB	0		;-40
	FCB	0		;-28
	FCB	0		;-16
	FCB	0		; -4
	FCB	0		;  8
	FCB	0		; 20
	FCB	0		; 32
	FCB	0		; 44
	FCB	0		; 56
	FCB	0		; 68
	FCB	0		; 80
	FCB	0		; 92
	FCB	0		;104
	FCB	0		;116
	FCB	0		;128
	FCB	0		;140
	FCB	0		;152
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; Uses any one of the following tables, depending on the conditions
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Cold crank PW mult. vs DRPs
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; 
; Based on cranking fuel counter #2
;
; Temperature < threshold in L4D8F
;
;	Mult. = val./128	;DRPs
;
L4DD4   FCB	230		; 1
	FCB	209		; 2
	FCB	188		; 3
	FCB	166		; 4
	FCB	145		; 5
	FCB	124		; 6
	FCB	103		; 7
	FCB	 83		; 8
	FCB	 79		; 9
	FCB	 75		;10
	FCB	 70		;11
	FCB	 64		;12
	FCB	 60		;13
	FCB	 56		;14
	FCB	 53		;15
	FCB	 51		;16
	FCB	 51		;17
	FCB	 51		;18
	FCB	 51		;19
	FCB	 51		;20
	FCB	 51		;21
	FCB	 51		;22
	FCB	 51		;23
	FCB	 51		;24
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Non hot restart crank PW fuel mult. vs DRPs
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; 
; Based on cranking fuel counter #2
;
; Temperature >= threshold in L4D8F
;
;	mult. = val./128	;DRPs
;
L4DEC   FCB	192    		; 1
	FCB	166    		; 2
	FCB	128    		; 3
	FCB	 83		; 4
	FCB	 60		; 5
	FCB	 56		; 6
	FCB	 52		; 7
	FCB	 44		; 8
	FCB	 42		; 9
	FCB	 41		;10
	FCB	 39    		;11
	FCB	 38    		;12
	FCB	 38    		;13
	FCB	 38    		;14
	FCB	 38    		;15
	FCB 	 38    		;16
	FCB	 38    		;17
	FCB	 38    		;18
	FCB	 38    		;19
	FCB	 38    		;20 
	FCB	 44    		;21
	FCB	 50    		;22
	FCB	 57    		;23
	FCB	 64    		;24
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Hot restart crank PW mult. vs DRPs
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; Based on cranking fuel counter #2
;
;	mult. = val./128	;DRPs
;
L4E04  	FCB	128		; 1
	FCB	100   		; 2
	FCB   	 70    		; 3
	FCB   	 60    		; 4
	FCB   	 50    		; 5
	FCB   	 45    		; 6
	FCB   	 40    		; 7
	FCB   	 38    		; 8
	FCB   	 38    		; 9
	FCB   	 38    		;10
	FCB   	 38    		;11
	FCB   	 38    		;12
	FCB   	 38    		;13
	FCB   	 38    		;14
	FCB   	 38    		;15
	FCB   	 38    		;16
	FCB   	 40    		;17
	FCB   	 41    		;18
	FCB   	 43    		;19
	FCB   	 44    		;20
	FCB   	 46    		;21
	FCB   	 48    		;22
	FCB   	 50    		;23
	FCB   	 51    		;24
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Cycling crank PW mult. vs DRPs
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; Based on DRP counter #2, use when DRPs > 23
; 
;	Mult. = val./128	;DRPs, lower 3 bits of counter
;
L4E1C	FCB	 64		;0   
	FCB	 52    		;1   
	FCB	 44    		;2   
	FCB	 38    		;3   
	FCB	 35    		;4   
	FCB	 34    		;5   
	FCB	 33    		;6   
	FCB	 32    		;7   
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Time thresholds to log flagged errors
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
L4E24	FCB 10			;1 sec, time threshold to update flagged errors if no 
				;errors previously set and no new errors flagged  
L4E25   FCB 20			;2 secs, time threshold to update errors if errors have
				;been previously set and new errors have been flagged  
;
L4E26	FCB 100			;10 secs, time threshold to update flagged errors if 
				;errors are currently set no new errors have been flagged
L4E27	FCB 120			;12 secs, time threshold to update flagged errors if
				;errors are currently set and new errors have been flagged 
;
;~~~~~~~~~~~~~~~~~~~~~
;-Error 13: O2 sensor
;~~~~~~~~~~~~~~~~~~~~~
;
L4E28   FCB 60			;120 secs, engine run time req'd for O2 diags.
L4E29   FCB 10			;1 sec, upper limit for DFCO for O2 diags.
L4E2A   FCB 1			;.1 sec, max time allowed during O2 diags. for DFCO
				;to be active
;
L4E2B   FCB 81			;351 mV, lower O2 window bound for O2 to be active   
L4E2C   FCB 127			;551 mV, upper O2 window bound for O2 to be active
;
L4E2D   FCB 13			;5%, lower TPS threshold to set O2 error
L4E2E   FCB 30			;60 sec, time threshold to set O2 error
L4E2F	FCB 147			;70 deg C, lower cool temp to set O2 error
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Error 16: 2002 PPM VSS buffer signal fail
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
L4E30   FCB 2			;2 MPH, upper limit for diags/to set error
;
L4E31   FCB 38			;950 RPM, lower bound to set error
L4E32   FCB 176			;4400 RPM, upper bound to set error
;
L4E33   FCB 15			;16 kPa, upper MAP threshold for VSS buffer diags.
L4E34   FCB 140			;65 deg C, lower cool temp for VSS buffer diags.
L4E35   FCB 3			;3 secs, error timer threshold to set error
L4E36   FCB 5			;2% TPS, upper threshold to set error
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Threshold for setting error 23, low MAT
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
L4E37   FCB 0			;-40 deg C, lower MAT threshold to set error 23 
L4E38   FCB 0			;0 sec, sec = value/10, time threshold to flag error 23 
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Error 24: VSS (manual only)
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
L4E39   FCB 10			;1 sec, time threshold to set error 24, VSS 
L4E3A   FCB 10			;10 MPH, upper limit for diags/to set error
;
;~~~~~~~~~~~~~~~~~~~
;-Error 23: low MAT
;~~~~~~~~~~~~~~~~~~~
;
L4E3B   FDB 0000		;Engine run time, time before performing MAT diags.
L4E3D   FCB 0			;0 MPH, boundry for errors 23 and 25
L4E3E   FCB 0			;-40c, default MAT if error is flagged
;
;~~~~~~~~~~~~~~~~~~~~
;-Error 25: high MAT
;~~~~~~~~~~~~~~~~~~~~
;
L4E3F   FCB 0			;-40 deg C, upper MAT threshold to set error 25 
L4E40   FCB 0			;0 sec, sec = value/10, time threshold to flag error 23
;
;~~~~~~~~~~~~~~~~~~~~
;-Error 31: Governor
;~~~~~~~~~~~~~~~~~~~~
;
L4E41   FCB 0			;0%, governor actuator integral threshold to set error   
;
;~~~~~~~~~~~~~~~
;-Error 32: EGR
;~~~~~~~~~~~~~~~
;
L4E42   FCB 30			;30 secs, time between EGR diag. cycles 
;
L4E43   FCB 64			;34.0 kPa, lower vac threshold for diags.		 
L4E44   FCB 208			;87.1 kPa, upper vac threshold for diags.
;
L4E45   FCB 15			;6% TPS, lower TPS threshold to perform diags.
L4E46   FCB 77			;30% TPS, upper TPS threshold to perform diags.
;
L4E47   FCB 243			;95% desired EGR position threshold to perform diags.
;
L4E48   FCB 30			;3 secs, min time to perform diags.
L4E49   FCB 3			;3 secs, time threshold for EGR to fail diags
;
L4E4A   FCB 10			;4% TPS, max change in TPS allowed for EGR diags.
L4E4B   FCB 25			;2.5 secs, max time before EGR fails diags.
L4E4C   FCB 4			;4, min change in INT for EGR to be functional
;
L4E4D   FCB 45			;45 MPH, min speed for EGR test
;
L4E4E   FCB 12			;300 msec INTegrator delay if EGR diags. underway
;
;-Linear EGR portion
;
L4E4F   FCB 0			;0, min allowed A/D pintle position   
L4E50   FCB 0			;0%, max allowed pintle position error
L4E51   FCB 0			;0 secs, time threshold to set error TIME THRESH
;
L4E52   FCB 0			;0%, upper position limit to check pintle position 
				;error
L4E53   FCB 0			;0 secs, time between diag. cycles
L4E54   FCB 0			; 0 Sec's SET ERR IF A/D POSIT L.T. REQ (note: not used)  
L4E55   FCB 0			;0 secs, time threshold to fail diags.
;
;~~~~~~~~~~~~~~~~~~~~~~~~~
;-Error 33: MAP sensor hi
;~~~~~~~~~~~~~~~~~~~~~~~~~
;
L4E56   FCB 10			;4% TPS, upper threshold to set error
L4E57   FCB 156			;68 kPa, lower baro limit, MAP diags.
L4E58   FCB 50			;5 secs, time threshold to set error
L4E59   FCB 0			;0 RPM, scaled RPM threshold to set error 
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Error 34: MAP sensor low
;~~~~~~~~~~~~~~~~~~~~~~~~~~
;
L4E5A   FCB 5			;12.2 kPa, lower MAP limit to set error 
L4E5B   FCB 64			;1200 RPM, upper limit to set MAP error
L4E5C   FCB 50			;625 msecs, time threshold to set error
L4E5D   FCB 20			;8% TPS, upper limit to segt error
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Default MAP params, use when low/high MAP error in effect
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
L4E5E   FCB 218			;91 kPa, default MAP if not running/UL for default MAP value
L4E5F   FCB 112     		;1.75, default MAP TPS coeff.
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Default MAP bias vs. RPM
;~~~~~~~~~~~~~~~~~~~~~~~~~~
;
L4E60	FCB 64			;3200 RPM, UL for lookup
;
;	Bias val.		;RPM
;
        FCB 90      		; 800 
        FCB 35      		;1600
        FCB 7       		;2400  
        FCB 2       		;3200  
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~
; Error 41: CAM sens error
;~~~~~~~~~~~~~~~~~~~~~~~~~~
;
L4E65   FCB 0			;# DRPs without cam pulses to set error
;
;~~~~~~~~~~~~~~~~~~~
;-ERROR 42 EST FAIL
;~~~~~~~~~~~~~~~~~~~
;
L4E66	FCB 4			;Lower RPM for EST diags.
L4E67	FCB 0			;0 counts, delta EST info counter value to set error
L4E68	FCB 4			;# of EST errors to set code 42
L4E69	FCB 4			;Time threshold to set error
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Error 43 params, knock sensor
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
L4E6A	FCB 160			;16 sec. time threshold to set error
L4E6B   FCB 230			;4.5 volts, upper threshold to set error 
L4E6C   FCB 20			;0.390 volts. lower threshold to set error 
L4E6D   FCB 23			;4 deg, default SA for knock sensor failure
;
;~~~~~~~~~~~~~~~~~~~
;-Error 44: O2 lean
;~~~~~~~~~~~~~~~~~~~
;
L4E6E   FCB 34			;148 mV, low O2 threshold to set error
L4E6F   FCB 15			;15  secs, time threshold to set error
L4E70   FCB 200			;200 secs, time threshold to set error if lean O2 previously
				;flagged
L4E71	FCB 146			;14.6 AFR, lean AFR limit if error 44 present
;
;~~~~~~~~~~~~~~~~~~~
;-Error 45: O2 rich
;~~~~~~~~~~~~~~~~~~~
;
L4E72   FCB 255			;151 deg C, threshold for high MAT temps 
L4E73   FCB 173			;750 mV, O2 threshold to set error
L4E74   FDB 0060		;60 secs, timer threshold  to set error (normal MAT temps)
;
L4E76   FDB 0060		;60 secs, timer threshold  to set error (high MAT temps)
;
L4E78   FCB 13			;5%, upper %TPS range to proceed with diags.
L4E79   FCB 0			;0% TPS, lower %TPS threshold to skip diags.
;
;~~~~~~~~~~~~~~~~~~~~~~~
;-Error 46, VATS params
;~~~~~~~~~~~~~~~~~~~~~~~
;
L4E7A   FCB 128			;12.8 secs, time threshold to set error
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Error 54, low fuel pump volts/relay failure
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
L4E7B   FCB 5			;500 msec, time threshold to set error 54
L4E7C   FCB 50			;5 volts, lower FP volts threshold to pass init. test
L4E7D   FCB 80			;8 volts, lower FP volts threshold to set error (engine on)
L4E7E   FCB 20			;2 volts, lower batt. volts threshold to set error
;
L4E7F   FCB 2			;Slow filtered O2 coeff.
;
;~~~~~~~~~~~~~~~~~~~~~~
;-AIR management quals
;~~~~~~~~~~~~~~~~~~~~~~
;
L4E80   FCB 192			;2400 RPM, upper RPM threshold for AIR
L4E81   FCB 32			;22 kPa, min MAP threshold for AIR if MAP < DISABLE AIR INJ IF L.T. 22 Kpa 
L4E82   FCB 73			;15 deg C, min temp for AIR
L4E83   FCB 0			;Time threshold in PE to disable air (1 = 1/10 sec)
;
L4E84   FCB 135			;Filter coeff for MPH
;
;~~~~~~~~~~~~~
;-IAC params.
;~~~~~~~~~~~~~
;         
L4E85	FCB  $0A		;IAC opt. word, 0000 1010
				; b7, 1 = Stall saver thresholds are default rpms    
				;     0 = Stall saver thresholds are based on requested rpms    
				; b6=Use MAT based tables for A/C on/off transitions        
				; b5=Init INT with calib const., not running    
				; b4, 1=A/C high pressure switch in use           
				; b3=Use A/C off idle integral cells to reset A/C on integral cells    
				; b2=Throttle kicker is present
				; b1=Maintain T/F action during DFCO            
				; b0 - 1 = Man. trans, disable closed loop idle if quals not met                              
				;      0 = Auto trans, enable closed loop idle if rpms low  
;           
L4E86  FCB  $01			;IAC opt. word, 0000 0001		
				; b7=   
				; b6=    
				; b5=    
				; b4=                          
				; b3=   
				; b2=    
				; b1, 1=power steering switch in use   
				; b0, 1=init current idle integral cells using steady state cells   
;                      
L4E87	FCB  161		;81 deg C, min cool temp threshold to move IAC prior to
				;initial startup    
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-IAC counts vs. % desired airflow
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; Used to linearize IAC action
;
;	IAC counts		;% airflow
;
L4E88   FCB  0			;  0.0
	FCB  9			;  6.3
	FCB  19			; 12.5
	FCB  27			; 18.8
	FCB  37			; 25.0
	FCB  46			; 31.3
	FCB  56			; 37.5
	FCB  64			; 43.8
	FCB  72			; 50.0
	FCB  79			; 56.3
	FCB  84			; 62.5
	FCB  89			; 68.8
	FCB  94			; 75.0
	FCB  100		; 81.3
	FCB  109		; 87.5
	FCB  125		; 93.8
	FCB  150		;100.0
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-IAC baro compensation factor vs baro
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; kPa = .625 x value + 14.9
;
L4E99   FCB  72			;45 kPa, table offset
	FCB  72			;(60 kPa + 45 kPa offset) = 105 kPa, UL for lookup
	FCB  32			;5 kPa per line
;
; 	factor = val/128	;kPa
;	
L4E9C   FCB  166		; 60
	FCB  160		; 65
	FCB  155    		; 70
	FCB  151		; 75
	FCB  147		; 80
	FCB  143		; 85
	FCB  138		; 90
	FCB  130		; 95
	FCB  128		;100
	FCB  128		;105
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-IAC constants and default values
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
L4EA6 	FCB  150		;150 steps, park position for IAC
;
L4EA7 	FCB  46			;18% airflow, default value for P/N idle integral cells
;
L4EA8 	FCB  51			;20% airflow, default value for drive idle integral cells
L4EA9 	FCB  51			;20% airflow, added airflow for A/C on integral cells
				;if high engine temps present
L4EAA 	FCB  31			;12% airflow, added airflow for A/C on integral cells
				;if normal engine temps are present
;
L4EAB 	FCB  200		;110 deg C, threshold for high engine temp
;
L4EAC 	FCB  105		;10.5 volts, min batt volts for IAC operation 
;
L4EAD 	FCB  40			;40 MPH, lower speed threshold to perform IAC reset
				; (If non vol mem or IAC is lost)
;
L4EAE 	FCB  50			;19.5% IAC flow, added flow for startup
L4EAF 	FCB  1			;100 msec, decay period for initial added IAC flow 
L4EB0 	FCB  240		;240 secs, time until init. idle speed is decayed out
;
L4EB1 	FCB  3			;300 msec, decay period for added desired idle speed
				; PER 12.5 msecs INT PERIOD 
;
L4EB2 	FCB  192		;1.5, multiplier for added desired idle speed if in P/N 
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Initial idle airflow parameters
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
L4EB3   FCB  250		;97.6%, decay multiplier for intial IAC flows
L4EB4   FCB  245		;95.7%, decay multiplier for coupled flow, P/N
L4EB5   FCB  244		;95.3%, decay multiplier for coupled flow, drive 
L4EB6   FCB  10			;10 MPH, upper MPH limit to decay out initial IAC flow 
L4EB7   FCB  211		;82.4%, initial IAC flow vs. cool temp kickdown mult.
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-IAC %flow vs. cool temp (P/N)
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;	%flow = val x .391	;Deg C	
;
L4EB8   FCB  240		;-28
	FCB  230		;-16
	FCB  197		; -4
	FCB  187		;  8
	FCB  166		; 20
	FCB  141		; 32
	FCB  102		; 44
	FCB  77			; 56
	FCB  72			; 68
	FCB  64			; 80
	FCB  64			; 92
	FCB  72			;104
	FCB  115		;116
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-IAC %flow vs. cool temp (drive)
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;	%flow = val x .391	;Deg C	
;
L4EC5   FCB  218		;-28
	FCB  212		;-16
	FCB  171		; -4
	FCB  166		;  8
	FCB  158		; 20
	FCB  133		; 32
	FCB  100		; 44
	FCB  80			; 56
	FCB  75			; 68
	FCB  67			; 80
	FCB  67			; 92
	FCB  79			;104
	FCB  128		;116
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-IAC cold offset delay vs. cool temp 
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; Delay untill initial IAC flow is
; decayed out after startup
;
;	delay = val/10		;Deg C
;
L4ED2   FCB  210		;-28
	FCB  170		;-16
	FCB  160		; -4
	FCB  150		;  8
	FCB  140		; 20
	FCB  120		; 32
	FCB  75			; 44
	FCB  35			; 56
	FCB  25			; 68
	FCB  10			; 80
	FCB  5			; 92
	FCB  5			;104
	FCB  20			;116
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Decay delay multiplier vs airflow
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; Used for above table
;
;	mult. = val/128		;gms/sec
;
L4EDF   FCB  154		; 0
	FCB  128		;16
	FCB  115		;32
	FCB  102		;48
	FCB  96			;64
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Allowable differences between initial IAC flows in P/N and drive
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; Flows are not saved if significantly different from one another
;
;-In drive, allowable difference from flow in P/N
;
L4EE4   FCB  102		;+40%, max allowable difference
L4EE5   FCB  77			;-30%, min allowable difference 
;
;-In park/neutral, allowed difference from flow in drive
;
L4EE6   FCB  102		;+40%, max allowable difference
L4EE7   FCB  77			;-30%, min allowable difference
;
;~~~~~~~~~~~~~~~~~~~~~~~~
;-Closed loop idle quals
;~~~~~~~~~~~~~~~~~~~~~~~~
;
L4EE8   FCB 4			;1.56% TPS, upper limit for closed throttle 
L4EE9   FCB 3			;3 MPH,  upper MPH limit for closed loop idle 
L4EEA   FCB 20			;500 msec, closed loop enable delay if quals not met
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-IAC closed loop idle enable delay after start up vs. cool temp (high RPMs)
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;
;	secs. = val/10		;Deg C
;
L4EEB   FCB  30			;-28
	FCB  20			;-16
	FCB  20			; -4
	FCB  15			;  8
	FCB  15			; 20
	FCB  15			; 32
	FCB  15			; 44
	FCB  15			; 56
	FCB  10			; 68
	FCB  10			; 80
	FCB  10			; 92
	FCB  10			;104
	FCB  10			;116
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-IAC closed loop idle enable delay after start up vs. cool temp (low RPMs)
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;
;	secs. = val/10		;Deg C
;
L4EF8   FCB  50			;-28
	FCB  40			;-16
	FCB  30			; -4
	FCB  20			;  8
	FCB  20			; 20
	FCB  20			; 32
	FCB  10			; 44
	FCB  10			; 56
	FCB  5			; 68
	FCB  5			; 80
	FCB  5			; 92
	FCB  5			;104
	FCB  5			;116
;
;~~~~~~~~~~~~~~~~~~~~~~~
;-IAC transition params
;~~~~~~~~~~~~~~~~~~~~~~~
;
L4F05   FCB  5			;2%, delta IAC flow multiplier for A/C on to off transition
;
L4F06   FCB  1			;0.4%, min A/C on to A/C off transition IAC flow
L4F07   FCB  10			;4.0%, max A/C on to A/C off transition IAC flow
;
L4F08   FCB  8			;200 msec, A/C on to off transition period
L4F09   FCB  1			;20 msec, additional added delay for closed loop idle 
				;enable during A/C transition
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Delta IAC flow for A/C on/off transition vs. MAT
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;-Params for table lookup
;
L4F0A   FCB  0			;-40 deg C, offset for lookup
	FCB  0			;-40 deg C, upper limit for offset value used for lookup
	FCB  0			;0 deg C per line
;
;	% delta IAC flow 
;	= val/2.56		;Deg C
;
L4F0D   FCB  0			;N/A  
	FCB  0			;
	FCB  0			;
	FCB  0			;
	FCB  0			;
;

;
;~~~~~~~~~~~~~~~~~~~~~~~
;-IAC transition params
;~~~~~~~~~~~~~~~~~~~~~~~
;
L4F12 	FCB  77			;30%, flow transition multiplier for A/C change state
; 		
L4F13 	FCB  3			;1.1% flow, min allowed change in airflow for A/C 
				;on/off transition
L4F14 	FCB  13			;5.1% flow, max allowed change in airflow for A/C
				;on/off transition
;
L4F15 	FCB  40			;1000 msec, idle transition period for A/C on/off 
L4F16 	FCB  4			;100 msec, delay for closed loop idle after A/C 
				;on/off transition 
;
L4F17 	FCB  1			;2.6% flow, A/C on/off transition flow decay rate
L4F18 	FCB  3			;300 msec, A/C on/off transition flow decay perod 
;
;-Park to drive
;     				
L4F19 	FCB  10			;3.9% flow, added IAC flow for park to drive trans shift
L4F1A 	FCB  35			;875 msec, period for park to drive shift
L4F1B 	FCB  20			;500 msec, closed loop idle delay for park to drive shift
;
;-Drive to park
;
L4F1C 	FCB  2			;0.8% IAC flow for drive to park/neutral
L4F1D 	FCB  60			;1.5 sec, drive to park transition period
L4F1E 	FCB  60			;1.5 sec, closed loop delay for drive to park shift
;
L4F1F 	FCB  47			;-5 deg C, threshold for cold idle added delays	for
				;transmission shift from P/N or drive
;
;-Park to drive
;
L4F20 	FCB  20			;500 msec, added idle delay for closed loop when cold and 
				;have shifted from park to drive
L4F21 	FCB  60			;1.5 sec, additional park to drive shift transition delay
				;when cold
;
;-Drive to park
;	 			
L4F22 	FCB  20			;500 msec, added idle delay for closed loop when cold and 
				;have shifted from drive to park
L4F23 	FCB  20			;500 msec, additional park to drive shift transition delay
				;when cold
;
;-Power steering compensation
;
L4F24 	FCB  0			;0% flow, added flow for P/S compensation
L4F25 	FCB  0			;0 msec, 100 msec/count, closed loop idle delay if 
				;P/S compensation in effect 
L4F26 	FCB  0			;0 msec, 100 msec/count, P/S compensation IAC flow
				;decay period 
L4F27 	FCB  0			;0% flow, P/S compensation flow decay rate
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Desired idle speed vs. cool temp
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; In P/N and A/C is off
;
;	RPM = val x 12.5	;Deg C
;
L4F28   FCB  70			;-28
	FCB  70			;-16
	FCB  70			; -4
	FCB  70			;  8
	FCB  66			; 20
	FCB  62			; 32
	FCB  58			; 44
	FCB  56			; 56
	FCB  54			; 68
	FCB  53			; 80
	FCB  52			; 92
	FCB  52			; 98
	FCB  52			;104
	FCB  54			;110
	FCB  58			;116
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Desired idle speed vs. cool temp
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; In P/N and A/C is on
;
;	RPM = val x 12.5	;Deg C
;
L4F37   FCB  70			;-28
	FCB  70			;-16
	FCB  70			; -4
	FCB  70			;  8
	FCB  66			; 20
	FCB  62			; 32
	FCB  58			; 44
	FCB  56			; 56
	FCB  54			; 68
	FCB  53			; 80
	FCB  52			; 92
	FCB  52			; 98
	FCB  52			;104
	FCB  54			;110
	FCB  58			;116
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Desired idle speed vs. cool temp
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; In drive and A/C is off
;
;	RPM = val x 12.5	;Deg C
;
L4F46   FCB  54			;-28
	FCB  54			;-16
	FCB  54			; -4
	FCB  54			;  8
	FCB  52			; 20
	FCB  50			; 32
	FCB  48			; 44
	FCB  46			; 56
	FCB  44			; 68
	FCB  43			; 80
	FCB  42			; 92
	FCB  42			; 98
	FCB  42			;104
	FCB  46			;110
	FCB  48			;116
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Desired idle speed vs. cool temp
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; In drive and A/C is on
;
;	RPM = val x 12.5	;Deg C
;
L4F55   FCB  54			;-28
	FCB  54			;-16
	FCB  54			; -4
	FCB  54			;  8
	FCB  52			; 20
	FCB  50			; 32
	FCB  48			; 44
	FCB  46			; 56
	FCB  44			; 68
	FCB  43			; 80
	FCB  42			; 92
	FCB  42			; 98
	FCB  42			;104
	FCB  46			;110
	FCB  48			;116
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Added initial desired idle speed vs. cool temp
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;	RPM = val x 12.5	;Deg C
;
L4F64   FCB  4			;-28
	FCB  4			;-16
	FCB  4			; -4
	FCB  4			;  8
	FCB  4			; 20
	FCB  4			; 32
	FCB  4			; 44
	FCB  4			; 56
	FCB  4			; 68
	FCB  3			; 80
	FCB  1			; 92
	FCB  1			;104
	FCB  1			;116
	FCB  2			;128
	FCB  4			;140
;
L4F73   FCB 120			;.47, RPM error gain factor for rough idle
L4F74   FCB 20			;Filter coefficient for filtered idle RPM error
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Default RPM errors for transitions
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
L4F75   FCB  0			;0 RPM, upper default RPM error if throttle is open
L4F76   FCB  16			;200 RPM, default RPM error for P/N, drive trans shift
L4F77   FCB  20			;250 RPM, default RPM error for A/C on/off transition 
L4F78   FCB  20			;250 RPM, defualt RPM error if P/S active
;
L4F79   FCB  2			;25 RPM, RPM error threshold for rough idle
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Proportional low battery volts correction parameters
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
L4F7A   FCB  125		;12.5 volts, lower threshold to begin raising RPMs to 
				;compensate for low batt. volts	   
L4F7B   FCB  51			;25 RPM per volt, prop. gain for low batt volts.  
				;gain(RPM/volt) = val x (125/256)
L4F7C   FCB  3			;38 RPM, max added idle speed for low batt volts
;
;~~~~~~~~~~~~~~~~~~~~~~~~~
;-CCP compensation params
;~~~~~~~~~~~~~~~~~~~~~~~~~
;
L4F7D   FCB  107		;Upper BLM limit for idle purge compensation
L4F7E   FCB  20			;20 sec, min time before begining idle purge compensation
L4F7F   FCB  173		;90 deg C, Lower temp threshold for idle purge compensation
L4F80   FCB  6			;75 RPM, idle purge compensation, P/N, A/C off 
L4F81   FCB  4			;50 RPM, idle purge compensation, P/N, A/C on
L4F82   FCB  4			;50 RPM, idle purge compensation, drive, A/C off
L4F83   FCB  2			;25 RPM, idle purge compensation, drive, A/C on
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Params for raising idle if high A/C pressure detected
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
L4F84   FCB  80			;1000 RPM, desired RPM for high A/C pressure, P/N
L4F85   FCB  90			;1125 RPM, desired RPM for high A/C pressure, drive
L4F86   FCB  10			;Delay to add in desired RPMs if high load previously
				;present
L4F87   FCB  10			;Delay to add in desired RPMs if high load not previously
				;present
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Idle integral gain vs. RPM error
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; Idle speed low and in P/N
;
;	gain(%flow/RPM)=
;	val x .000122		;RPM error
;
L4F88   FCB  0			;  -0.0
	FCB  0			; -12.5
	FCB  3			; -25.0
	FCB  6			; -37.5
	FCB  6			; -50.0
	FCB  12			;-100.0
	FCB  16			;-150.0
	FCB  16			;-200.0
	FCB  16			;-300.0
	FCB  16			;-400.0
	FCB  16			;-500.0
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Idle integral gain vs. RPM error
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; Idle speed high and in P/N
;
;	gain(%flow/RPM)=
;	val x .000122		;RPM error
;	
L4F93   FCB  0			;  0.0
	FCB  0			; 12.5
	FCB  3			; 25.0
	FCB  6			; 37.5
	FCB  6			; 50.0
	FCB  6			;100.0
	FCB  6			;150.0
	FCB  6			;200.0
	FCB  6			;300.0
	FCB  6			;400.0
	FCB  6			;500.0
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Idle integral gain vs. RPM error
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; Idle speed low and in drive
;
;	gain(%flow/RPM)=
;	val x .000122		;RPM error
;
L4F9E   FCB  0 			;  -0.0
	FCB  0			; -12.5
	FCB  4			; -25.0
	FCB  6			; -37.0
	FCB  6			; -50.0
	FCB  8			;-100.0
	FCB  9			;-150.0
	FCB  10			;-200.0
	FCB  11			;-300.0
	FCB  16			;-400.0
	FCB  12			;-500.0
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Idle integral gain vs. RPM error
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; Idle speed high and in drive
;
;	gain(%flow/RPM)=
;	val x .000122		;RPM error
;
L4FA9   FCB  0			;  0.0
	FCB  0			; 12.5
	FCB  4			; 25.0
	FCB  6			; 37.5
	FCB  7			; 50.0
	FCB  8			;100.0
	FCB  9			;150.0
	FCB  10			;200.0
	FCB  11			;300.0
	FCB  16			;400.0
	FCB  12			;500.0
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Idle integrator parameters and thresholds
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
L4FB4   FCB	1		;12.5 RPM, upper RPM error limit for steady state idle
;
L4FB5	FCB	167		;85 deg C, lower temp threshold for steady state idle 
				;check
L4FB6	FCB	193		;105 deg C, upper temp threshold for steady state idle
				;check
;
;-Allowable integrator difference for A/C and non-A/C terms, P/N
;	
L4FB7	FCB   7			;2.7% flow, min allowable difference 
L4FB8   FCB  90			;35% flow, max allowable difference 
;
;-Allowable integrator difference for A/C and non-A/C terms, drive
;
L4FB9   FCB  7			;2.7% flow, min allowable difference
L4FBA   FCB  103		;40.2% flow, max allowable difference 
;
;-Allowable integrator difference for drive and P/N terms, both non-A/C and A/C
;				
L4FBB   FCB  38			;15% flow, max allowable difference
L4FBC   FCB  8			;-2.7% flow, min allowable difference 
;
;-Allowable integral difference between steady state and current integral terms,
; checked when not in closed loop idle
;
L4FBD   FCB  255		;99.6% flow, max allowable difference, no A/C
L4FBE   FCB  255		;99.6% flow, max allowable difference, A/C
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Quals for limiting idle proportional action
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
L4FBF   FCB  5			;5 MPH, lower threshold to limit proportional action
L4FC0   FCB  3			;3 MPH, upper threshold to restore proportional action
L4FC1   FCB  128		;1600 RPM, upper limit for RPM errror when limiting 
				;proportional action
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Idle proportional term vs. RPM error
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; Low idle and in P/N		
;
;	%flow = val x .391	;RPM error
;
L4FC2   FCB  0			;  -0.0            
	FCB  0			; -12.5
	FCB  0			; -25.0
	FCB  4			; -37.5
	FCB  6			; -50.0
	FCB  9			;-100.0
	FCB  12			;-150.0
	FCB  12			;-200.0
	FCB  12			;-300.0
	FCB  12			;-400.0
	FCB  12			;-500.0
;
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Idle proportional term vs. RPM error
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; High idle and in P/N		
;
;	%flow = val x .391	;RPM error
;
L4FCD   FCB  0			;  0.0
	FCB  0			; 12.5
	FCB  0			; 25.0
	FCB  1			; 37.5
	FCB  2			; 50.0
	FCB  4			;100.0
	FCB  5			;150.0
	FCB  6			;200.0
	FCB  7			;300.0
	FCB  8			;400.0
	FCB  9			;500.0
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Idle proportional term vs. RPM error
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; Low idle, A/C off, in drive, and open loop idle		
;
;	%flow = val x .391	;RPM error
;
L4FD8   FCB  0			;  -0.0
	FCB  0			; -12.5
	FCB  0			; -25.0
	FCB  0			; -37.5
	FCB  1			; -50.0 
	FCB  3			;-100.0  
	FCB  5			;-150.0  
	FCB  6			;-200.0 
	FCB  8			;-300.0  
	FCB  12			;-400.0  
	FCB  12			;-500.0   
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Idle proportional term vs. RPM error
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; High idle, A/C off, in drive, and open loop idle		
;
;	%flow = val x .391	;RPM error
;
L4FE3   FCB  0			;  0.0
	FCB  0			; 12.5
	FCB  0			; 25.0
	FCB  0			; 37.5
	FCB  0			; 50.0
	FCB  0			;100.0
	FCB  0			;150.0
	FCB  0			;200.0
	FCB  0			;300.0
	FCB  0			;400.0
	FCB  0			;500.0
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Idle proportional term vs. RPM error
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; Low idle, A/C off, in drive, and closed loop idle		
;
;	%flow = val x .391	;RPM error
;
L4FEE   FCB  0			;  -0.0
	FCB  0			; -12.5
	FCB  2			; -25.0
	FCB  5			; -37.5
	FCB  7			; -50.0
	FCB  10			;-100.0
	FCB  12			;-150.0
	FCB  12			;-200.0
	FCB  12			;-300.0
	FCB  12			;-400.0
	FCB  12			;-500.0
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Idle proportional term vs. RPM error
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; High idle, A/C off, in drive, and closed loop idle		
;
;	%flow = val x .391	;RPM error
;			
L4FF9   FCB  0			;  0.0
	FCB  0			; 12.5
	FCB  1			; 25.0
	FCB  2			; 37.5
	FCB  4			; 50.0
	FCB  5			;100.0
	FCB  6			;150.0
	FCB  7			;200.0
	FCB  8			;300.0
	FCB  9			;400.0
	FCB  12			;500.0
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Idle proportional term vs. RPM error
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; Low idle, A/C on, in drive, and open loop idle		
;
;	%flow = val x .391	;RPM error
;
L5004   FCB  0			;  -0.0
	FCB  0			; -12.5
	FCB  0			; -25.5
	FCB  0			; -37.5
	FCB  1			; -50.0
	FCB  3			;-100.0
	FCB  5			;-150.0
	FCB  6			;-200.0
	FCB  8			;-300.0
	FCB  12			;-400.0
	FCB  12			;-500.0	
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Idle proportional term vs. RPM error
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; High idle, A/C on, in drive, and open loop idle		
;
;	%flow = val x .391	;RPM error
;
L500F   FCB  0			;  0.0
	FCB  0			; 12.5	
	FCB  0			; 37.5
	FCB  0			; 25.0
	FCB  0			; 50.0
	FCB  0			;100.0
	FCB  0			;150.0
	FCB  0			;300.0
	FCB  0			;400.0
	FCB  0			;500.0
;
L5019	FCB  0			;Not used	
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Idle proportional term vs. RPM error
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; Low idle, A/C on, in drive, and closed loop		
;
;	%flow = val x .391	;RPM error
;
L501A   FCB  0			;  -0.0
	FCB  0			; -12.5
	FCB  3			; -25.0
	FCB  6			; -37.5
	FCB  9			; -50.0
	FCB  12			;-100.0
	FCB  15			;-150.0
	FCB  15			;-200.0
	FCB  15			;-300.0
	FCB  15			;-400.0
	FCB  12			;-500.0
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Idle proportional term vs. RPM error
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; High idle, A/C on, in drive, and closed loop		
;
;	%flow = val x .391	;RPM error
;
L5025   FCB  0			;  0.0
	FCB  0			; 12.5
	FCB  1			; 25.0
	FCB  3			; 37.5
	FCB  5			; 50.0
	FCB  6			;100.0 
	FCB  9			;150.0  
	FCB  12			;200.0 
	FCB  12			;300.0 
	FCB  14			;400.0  
	FCB  14			;500.0
;
;-Filter coeff. for limiting derivative idle action
;
L5030   FCB  32			;Coeff. for positive derivative action
L5031   FCB  16			;Coeff. for negative derivative action 
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Idle derivative term vs. transient RPM ratio
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;
;	%flow = (val - 128)/4	;Transient RPM ratio
;
L5032   FCB  128		;0.75000
	FCB  128		;0.78125
	FCB  128		;0.81250
	FCB  128		;0.84375
	FCB  128		;0.87500
	FCB  128		;0.90625
	FCB  128		;0.93750
	FCB  128		;0.96875
	FCB  128		;1.00000
	FCB  128		;1.03125
	FCB  140		;1.06250
	FCB  152		;1.09375
	FCB  164		;1.12500
	FCB  164		;1.15625
	FCB  164		;1.18750
	FCB  164		;1.18750
	FCB  164		;1.25000
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Stall saver activation thresholds
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; L4E85, b7==0, quals are % multipliers for desired idle to obtain RPM threshold
;        b7==1, quals are RPM thresholds
;
;-In park/neutral
;			 
L5043	FCB  166		;65% desired idle speed, A/C off
L5044   FCB  166		;65% desired idle speed, A/C on
;
;-In drive
;	
L5045   FCB  179		;70% desired idle speed, A/C off
L5046   FCB  179		;70% desired idle speed, A/C on 
;		 
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Added airflow for stall saver
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;-In park/neutral
;			 
L5047   FCB  6			;2.3%, added airflow, A/C off 
L5048   FCB  8			;3.0%, added airflow, A/C on
;
;-In drive
;			
L5049   FCB  10			;4.0%, added airflow, A/C off
L504A   FCB  13			;5.0%, added airflow, A/C on 
;
L504B   FCB  20			;0.5 sec, time for stall saver to be active, P/N
L504C   FCB  40			;1.0 sec, time for stall saver to be active, drive
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; IAC throttle follower gain vs. cool temp
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;	gain = val/64
;
L504D   FCB  96			;-28
	FCB  96			;-16
	FCB  80			; -4
	FCB  74			;  8
	FCB  70			; 20
	FCB  70			; 32
	FCB  70			; 44
	FCB  70			; 56
	FCB  70			; 68
	FCB  64			; 80		
	FCB  64			; 92		
	FCB  64			;104		
	FCB  58			;116	
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; IAC throttle follower gain vs. RPM
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;
L505A   FCB  8			;9 line table
;
;	gain = val/64		;RPM
;
L505B   FCB  110		;   0
	FCB  140		; 400
	FCB  140		; 800
	FCB  140		;1200
	FCB  140		;1600
	FCB  140		;2000
	FCB  140		;2400
	FCB  140		;2800
	FCB  140		;3200
;
L5064   FCB 37			;28.60%, max allowed TPS for P/N throttle follower
L5065   FCB 64			;31.25%, max allowed TPS for T/F in drive
L5066   FCB 37			;TPS throttle follower filtering coeff., P/N
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-TPS filtering period vs. MPH 
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; Used to filter TPS back down during throttle 
; closing for throttle follower and other items
;
;	secs = val/40		;MPH
;
L5067   FCB   8			; 0
	FCB  30			; 8
	FCB  40			;16
	FCB  60			;24
	FCB  60			;32
	FCB  60			;40
	FCB  60			;48
	FCB  60			;56
	FCB  60			;64
	FCB  60			;72
	FCB  60			;80
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-TPS filtering coeff. vs. MPH
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; Filter coeff. for filter TPS back down during throttle 
; closing for throttle follower and other items
;
;	Coeff.			;MPH
;
L5072   FCB  20			; 0
	FCB  12			; 8
	FCB  5			;16
	FCB  5			;24
	FCB  5			;32
	FCB  5			;40
	FCB  5			;48
	FCB  5			;56
	FCB  5			;64
	FCB  5			;72
	FCB  5			;80
;
L507D   FCB  4			;1.6% flow, min delta flow to update throttle follower flow
;
L507E   FCB  20			;26.3 kPa map threshold for scaling down TPS filter coeff.
L507F   FCB  128		;.5, multiplier for reducing filter coeff. when low MAP present 
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Lower limit for TPS to ensure some added throttle follower flow during closed throttle
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;	%TPS = val x .0977	;Deg C
;
L5080   FCB  24			;-28
	FCB  24			;-16
	FCB  24			; -4
	FCB  24			;  8
	FCB  24			; 20
	FCB  24			; 32
	FCB  24			; 44
	FCB  24			; 56
	FCB  24			; 68
	FCB  24			; 80
	FCB  24			; 92
	FCB  24			;104
	FCB  28			;116
;
L508D	FCB  40			;4 secs, time after throttle closes to limit
				;throttle follwer action
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Added flow for steady state integral cells on initilization
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
L508E   FCB  3			;1.0% flow, added to steady state integral on 
				;init, A/C off, drive
L508F   FCB  6			;2.3% flow, added to steady state integral on 
				;init, A/C on, drive
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Note from original hac:
;
;	SPECIAL IDLE AIR MODIFIERS PRIOR TO CAT LIGHT OFF
;
;	Also used by non-cat cals to reduce HC emissions
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
L5090   FCB  73			;15 deg c, lower cool temp limit for cool engine
L5091   FCB  113		;45 deg c, upper cool temp limit for cool engine
L5092   FDB  0350		;350 secs, max time to remain in cool engine mode
L5094   FCB  218		;85%, multiplier for reducing T/F action during cool
				;engine operation 
L5095   FCB  220		;86%, multiplier for reducing T/F action during warm
				;engine operation 
L5096   FCB  85			;1.33, multiplier for lower closed throttle TPS limit 
				;when in cool engine mode 
;
L5097   FCB  86			;1.34, added throttle follower gain for cool engine 
				;operation
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Accel enrich vs. delta %IAC flow
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
L5098   FCB  4			;5 line table
;
;	usec=val x 15.26	;Delta %IAC flow
;
L5099   FCB  0			; 0.00
	FCB  1			; 6.25
	FCB  5			;12.50
	FCB  10			;18.75
	FCB  15			;25.00	
;
;~~~~~~~~~~~~~~~~~~~
;-Params for IAC AE
;~~~~~~~~~~~~~~~~~~~
;
L509E   FCB  5			;1.9% flow, lower threshold for IAC AE
L509F   FCB  10			;3.9% TPS, upper threshold for IAC AE
;
L50A0   FDB  65535		;99.998% flow, Max IAC % flow in Pwr Enrich
				;Used by TBI sys to shut down IAC near WOT
				;to improve air fuel dist on dyno [from original hac]
;
L50A2   FCB  0			;0%, flow bias for added IAC flow when in DFCO
				;(following table)
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Additional IAC flow vs. RPM for DFCO
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;	%flow = val/.391	;RPM/25
;
L50A3   FCB  8			;   0
	FCB  46			; 400
	FCB  46			; 800
	FCB  51			;1200
	FCB  62			;1600
	FCB  66			;2000
	FCB  72			;2400
	FCB  77			;2800
	FCB  82			;3200
	FCB  90			;3600
	FCB  96			;4000
	FCB  96			;4400
	FCB  96			;4800
	FCB  96			;5200
	FCB  96			;5600
	FCB  96			;6000
	FCB  96     		;6400
;
;-DFCO flow decay rate per sec = decay ammount/decay time rate
;
L50B4   FCB  4			;16%, DFCO decay ammount
L50B5   FCB  3      		;300 msec, DFCO decay rate
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-IAC diagnostic (error 35) params
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
L50B6   FCB  9			;112.5 RPM, sustained idle RPM error to set IAC fault 
L50B7   FCB  150		;30 sec, time threshold to set IAC fault
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Quals for checking idle integral limits
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;
L50B8   FCB  167		;85 deg C, lower cool temp limit to check integrator					;
L50B9   FCB  166		;65% flow, max allowed idle integrator, no A/C
L50BA   FCB  255		;99.6% flow, max idle integrator, A/C
L50BB   FCB  8 			;100 RPM, error threshold for requesting IAC reset
				;and IAC reposition
L50BC   FCB  $0C		;Counter value, loop counter threshold to begin
				;adding/removing steps
;
;~~~~~~~~~~~~~~~~~~~~~~~~
;-Throttle kicker params
;~~~~~~~~~~~~~~~~~~~~~~~~
;
L50BD   FCB  0			;-40 deg C, temp threshold to enable throttle kicker
L50BE   FCB  0			;0% flow, subtracted bias when throttle kicker is active
L50BF   FCB  0			;1, airflow multiplier for throttle kicker FACTOR 
L50C0   FCB  0			;0 MPH, threshold to enable throttle kicker (low baro)
L50C1   FCB  0			;0% TPS, threshold to enable throttle kicker (low baro)
L50C2   FCB  0			;0 secs, min time for throttle kicker to be active 
;
L50C3   FCB  0			;0 kPa, upper baro threshold to disable throttle kicer
L50C4   FCB  0			;0 kPa, lower baro threshold to enable throttle kicker
;
L50C5   FCB  0			;0% flow, idle integrator thresh. for disabling T/K 
L50C6   FCB  0			;0% flow, lower idle integrator threshold for 
				;enabling throttle kicker (low baro)
;
L50C7   FCB  0			;RPM error threshold to disable throttle kicker (Drive)
L50C8   FCB  0			;RPM error threshold to disable throttle kicker (P/N)
					;
L50C9   FCB  0			;0 sec, min time for T/K to be active
L50CA   FCB  0			;0%, default base TPS for T/K
L50CB   FCB  0			;0%, base TPS offset for kicker enabled
L50CC   FCB  0			;0% min delta TPS threshold not to set err. 36
					;
L50CD   FCB  10			;(10 + 255) = 265 sec, min time befor re-enabling T/K
;
;~~~~~~~~~~~~~~~~~
;-Governor params
;~~~~~~~~~~~~~~~~~
;
L50CE   FCB  0      		;0 RPM, RPM threshold for engine overspeed
;
L50CF   FCB  0			;Min Governor RPM
;
L50D0   FCB  0			;0 MPH, MPH threshold for governor (0 = No MPH governing)
L50D1   FCB  0			;0 MPH, Governor MPH error threshold to exit MPH governing
L50D2   FCB  0			;0 RPM, threshold for governor overspeed light (RPM/25)
;
L50D3   FCB  0      		;RPM prop. gain term
L50D4   FCB  0     		;MPH prop. gain term
;
L50D5   FCB  0			;Desired TPS integral gain term for MPH overspeed
L50D6   FCB  0			;Desired TPS integral gain term for RPM overspeed
;
L50D7   FCB  0			;0%, upper limit for governor TPS
L50D8   FCB  0      		;Min desired TPS for overspeed
;
L50D9   FCB  0			;0%, min desired TPS for derivative action 
;
L50DA   FCB  0			;Filter coeff for governor rpm  
;
L50DB   FCB  0     		;TPS threshold for return gain factors 
L50DC   FCB  0			;High desired TPS return gain
L50DD   FCB  0      		;Low desired TPS return gain
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Governor integral gain factor vs desired TPS output
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
L50DE	FCB  64     		;50% TPS, upper limit
;
;	gain			;%TPS
;
	FCB	0		; 0.0
	FCB	0		;12.5
	FCB	0 		;25.0
	FCB	0		;37.5
	FCB	0		;50.0
;
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Gov'r derivative action thresholds vs. delta RPMs
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;
L50E4	FCB  80			;Line count, 10 lines
;
;	RPM threshold		;Delta RPMs
;
L50E5   FCB  0			;  0	
	FCB  0			; 80	
	FCB  0			;160 	
	FCB  0			;240	
	FCB  0			;320	
	FCB  0			;400	
	FCB  0			;480	
	FCB  0			;560	
	FCB  0			;640	
	FCB  0			;720	
	FCB  0			;800
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Governor, TPS derivative gain vs. cool temp
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;	gain			;Deg C		
;
L50F0   FCB  0			;-40
	FCB  0			;-16
	FCB  0			;  8
	FCB  0			; 32
	FCB  0			; 56
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Governor actuator proportional gain vs. TPS error
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
L50F5   FCB  8			;9 lines
;
;	gain			;%TPS error
;
	FCB  0			;  0.0
	FCB  0			; 12.5
	FCB  0			; 25.0
	FCB  0			; 37.5
	FCB  0			; 50.0
	FCB  0			; 62.5
	FCB  0			; 75.0
	FCB  0			; 87.5
	FCB  0			;100.0
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Governor actuator integral gain vs. TPS error
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;			
L50FF   FCB  8			;9 lines
;
;	gain			;%TPS error
;
L5100   FCB  0			;  0.0
	FCB  0			; 12.5
	FCB  0			; 25.0
	FCB  0			; 37.5
	FCB  0			; 50.0
	FCB  0			; 62.5
	FCB  0			; 75.0
	FCB  0			; 87.5
	FCB  0			;100.0
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Upper engine/trans RPM ratio threshold vs gear # (shift light)
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;	Ratio = val/10		;Gear #
;
L5109   FCB  37			;1st 
	FCB  21			;2nd
	FCB  12			;3rd
	FCB  9			;4th 
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Min RPM thresholds vs gear #
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;	RPM = val x 25		;Gear #
;
L510D   FCB  64			;1st
	FCB  64			;2nd 
	FCB  56			;3rd 
	FCB  55			;4th	
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-TPS load scalar vs gear #
;~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; 	mult. =  val / 128	;Gear #
;
L5111   FCB  90			;1st
	FCB  128		;2nd 
	FCB  128		;3rd 
	FCB  141		;4th 
;
;~~~~~~~~~~~~~~~~~~~~
;-Shift light params
;~~~~~~~~~~~~~~~~~~~~
;
L5115   FCB  73			;15 deg C, min temp for shiftlight 
;
L5116   FCB  8			;3% TPS, lower limit for shift light in low gears
L5117   FCB  8			;3% TPS, lower limit for shift light in high gears 
;
L5118   FCB  180		;4500 RPM, RPM threshold for manditory shift light
;
L5119   FCB  10			;1 second shift light activation delay, low gears
L511A   FCB  15			;1.5 second shift light activation delay, high gears
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; TPS load threshold for shift light (low gears, light currently on)
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;	% load = val/256	;RPM	
;
L511B   FCB  38			;1000
	FCB  38			;1400
	FCB  43			;1800
	FCB  66			;2200
	FCB  89			;2600
	FCB  112		;3000
	FCB  132		;3400
	FCB  204		;3800
	FCB  255		;4200
	FCB  255		;4600
	FCB  255		;5000
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; TPS load threshold for shift light (low gears, light currently off)
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;	% load = val/256	;RPM	
;
L5126   FCB  26			;1000
	FCB  26			;1400
	FCB  31			;1800
	FCB  54			;2200
	FCB  77			;2600
	FCB  100		;3000
	FCB  120		;3400
	FCB  192		;3800
	FCB  243		;4200
	FCB  255		;4600
	FCB  255		;5000
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; TPS load threshold for shift light (high gears, light currently on)
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;	% load = val/256	;RPM	
;
L5131   FCB  43			;1000
	FCB  43			;1400
	FCB  71			;1800
	FCB  102		;2200
	FCB  130		;2600
	FCB  178		;3000
	FCB  208		;3400
	FCB  230		;3800
	FCB  242		;4200
	FCB  255		;4600
	FCB  255		;5000
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; TPS load threshold for shift light (high gears, light currently off)
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;	% load = val/256	;RPM	
;
L513C   FCB  31			;1000
	FCB  41			;1400
	FCB  59			;1800
	FCB  90			;2200
	FCB  118		;2600
	FCB  166		;3000
	FCB  192		;3400
	FCB  218		;3800
	FCB  230		;4200
	FCB  255		;4600
	FCB  255		;5000
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-TPS S/L load threshold correction vs. baro kPa
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;	mul. = val/128		;kPa
;
L5147   FCB  109		; 75
	FCB  115		; 85
	FCB  128		; 95
	FCB  128		;105
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Params for trans anti-abuse routine
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
L514B   FCB  40			;500 msec, time for anti-abuse routine to be in effect
L514C   FCB  3      		;3 MPH, max speed for park to drive 
L514D   FCB  255    		;4775 RPM (scaled), RPM threshold to flag abuse
L514E   FCB  255    		;4775 RPM (scaled), RPM threshold to clear abuse flag 
L514F   FCB  51			;20% TPS, min TPS for anti-abuse routine
;
;~~~~~~~~~~~~~~~~~~~
;-TCC lockup params
;~~~~~~~~~~~~~~~~~~~
;
L5150   FCB  140		;65 deg C, lower temp limit for TCC lockup
L5151   FCB  5			;2% TPS hyst. if TCC locked, used with table L515E
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Delta TPS TCC limits to lock
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
L5152   FCB  54			;-.21% TPS/msec, throttle closing limit 
L5153   FCB  45			;.176% TPS/msec, throttle opening limit 
;
;~~~~~~~~~~~~~~~~~~
;-TCC relock delay
;~~~~~~~~~~~~~~~~~~
;					
L5154   FCB  20			;2 sec delay to relock after quals not met
L5155   FCB  25			;2.5 sec coast relock delay
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Manditory lockup thresholds
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
L5156   FCB  82			;82 MPH, threshold to lock TCC
L5157   FCB  79			;79 MPH, threshold to unlock TCC after mand. lockup
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-RPM thresholds for TCC lockup
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;	RPM/12.5
;
L5158   FCB  0			;0 rpm, limit for TCC to remain locked	
L5159   FCB  0			;0 rpm, limit for TCC to lock
;
;~~~~~~~~~~~~~~~~~~~~
;-TCC MPH thresholds
;~~~~~~~~~~~~~~~~~~~~
;
;-No A/C 
;
L515A   FCB  30			;30 mph, lower threshold for TCC to lock
L515B   FCB  28			;28 mph, lower threshold for TCC to remain locked			
;
;-A/C 
;			
L515C   FCB  35			;35 mph, lower threshold for TCC to lock
L515D   FCB  33			;33 mph, lower threshold for TCC to remain locked			
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Lower load limit for TCC to lock vs mph
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;	%TPS = val x .391	;MPH
;
L515E	FCB  13			; 0
	FCB  13			;10
	FCB  13			;20
	FCB  13			;30
	FCB  17			;40
	FCB  20			;50
	FCB  22			;60
	FCB  23			;70
	FCB  23			;80
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Upper load limit for TCC to remain locked vs MPH
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;	%TPS = val x .391	;MPH
;
L5167	FCB  64			;20
	FCB  64			;25
	FCB  73			;30
	FCB  87			;35
	FCB  110		;40
	FCB  128		;45
	FCB  140		;50
	FCB  158		;55
	FCB  158		;60
	FCB  158		;65
	FCB  158		;70
	FCB  158		;75
	FCB  158		;80
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Upper load limit for TCC to lock vs MPH
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;	%TPS = val x .391	;MPH
;
L5174  	FCB  51			;20
	FCB  51			;25
	FCB  57			;30
	FCB  64			;35
	FCB  72			;40
	FCB  88			;45
	FCB  96			;50
	FCB  128		;55
	FCB  128		;60
	FCB  128		;65
	FCB  128		;70
	FCB  128		;75
	FCB  128		;80
;
;~~~~~~~~~~~~~
;-VATS params
;~~~~~~~~~~~~~
;
; VATS period (msecs) = val x 7.63 x 10^-3 
;
L5181   FDB  4680		;35.71 msec, 28 Hz, lower freq. limit
L5183   FDB  4096		;31.25 msec, 32 Hz, upper freq. limit    
;
;**********************************************
;* $0D (BDWM) 8192 SERIAL DATA DCB'S 
;* PCM P/N 16196395, S/S BY PCM 16197427
;*
;**********************************************
;
;**********************************************
;* $0D (BDWM)  REMOTE BROADCAST MODE
;*  
;**********************************************
;
;---------------------------------------------
; REMOTE MESSAGE SCHEDULE TABLE 
; (POLLING FORMAT)
;
; (1 FOR EACH MINOR LOOP)
;
; 0000 ADDRESS'S ARE IGNNORED BY PGM
;---------------------------------------------
;
L5185  	FDB 0000		;  0, SEGMENT 0
	FDB 0000		;  1,
	FDB 0000		;  2,
	FDB 0000		;  3,
	FDB 0000		;  4,
	FDB 0000		;  5, 
	FDB 0000		;  6,
	FDB 0000		;  7,
	FDB 0000		;  8,
	FDB 0000		;  9,
	FDB 0000		; 10,
	FDB 0000		; 11,
	FDB 0000		; 12,
	FDB 0000		; 13,
	FDB 0000		; 14,
	FDB 0000		; 15,
;
;---------------------------------------------
; SCAN TOOL POLLING MESSAGE
;
;---------------------------------------------
;
L51A5  	FDB 0000		; NEXT MSG ENTRY ADDR
;
L51A7  	FCB $F0			; DEVICE CODE  
;
L51A8  	FCB $80			; MEM FLAG, TBL FM ROM
L51A9  	FCB 1			; NUM OF WORDS  
;
L51AA  	FDB L036E		; ADD OF OUTPUT BUFFER  
L51AC  	FDB $0000		; ADDR OF INPUT BUFFER 
;
L51AE  	FDB $0000		; ADDR DATA POINTER
;
L51B0  	FCB $F4			; ENGINE ID
;
;*******************************************
;* MSG ID $F4
;*
;*******************************************
;* 8192 ALDL MESSAGES FOLLOW XDE 5024 STD FORMAT,
;* ALSO SPECIAL FORMATS PER GM ALDL SPEC 12/14/87
;*
;* THE PCM IS A SLAVE, THUS NO OUTPUTED ALDL 
;* POLL MSG'S. THE PCM RESPONDS TO RX'ED ALDL MSGS
;*
;**********************************************
;
L51B1  FDB 65535		;Time threshold to prevent immediate re-entry into 
				;ALDL mode 4
;
L51B3  FDB $0000		;Manditory time to prevent re-entry into mode 4
;
;***********************************************
;* SERIAL DATA MESSAGES                         
;* DEVICE ID $F4								   
;*			                                   
;***********************************************
;
;  DEVICE ID $F4
;
; MSG RX'ED, ANSWER TO ALDL POLLING MSG
; TYPE $0D PCM
;----------------------------------------------
L51B5  	FDB  L539E		;Next ALDL entry addr.					
L51B7  	FCB  $F4		;Engine diag. ID				
L51B8  	FCB  $00		;
L51B9  	FCB  $80		;Number of bytes to xmit
;
L51BA	FDB  L036E		;Addr. of outgoing data buffer
L51BC  	FDB  L036E		;Addr. of incoming data buffer
;
;-Pointer for parameters for various ALDL modes
;					;
L51BE  	FDB  L51D6		;Mode 0, normal operating mode
L51C0  	FDB  L51E5		;Mode 1, diag. scan mode 
L51C2  	FDB  L537A		;  2, DEVICE ID $F4, MODE 2, 64 ADDR MEM DUMP 
L51C4  	FDB  L5383		;  3, DEVICE ID $F4, MODE 3, SEQUENTIAL 8 ADDR MEM DUMP 					
L51C6  	FDB  L538C		;  4, DEVICE ID $F4, MODE 4, CONTROLER MODE
L51C8  	FDB  $0000		;  5, not used  
L51CA  	FDB  $0000		;  6, not used 
L51CC  	FDB  $0000		;  7, not used
L51CE  	FDB  $0000		;  8, not used
L51D0  	FDB  $0000		;  9, not used
L51D2  	FDB  L5395		; 10, DEVICE ID $F4, MODE 10, ALDL ERR RESET
L51D4  	FDB  $0000		; 11, not used
;
;----------------------------------------------
; DEVICE ID $F4, MODE 0
;
;  RESET NORMAL COMM
;----------------------------------------------
;
L51D6  	FDB  $0000		; NEXT ENRTY ADDRESS
;
L51D8  	FCB  $F4		; DEVICE ID
;
L51D9  	FCB   00		; MEM FLAY
L51DA  	FCB    1		; BYTES TO TX
;
L51DB  	FDB  L036E		; ADDR OF OUTPUT BUFFER
L51DD  	FDB  L036E		; ADDR OF INPUT BUFFER
;
;----------------------------------------------
; MODE 1 SELECTOR TABLE
;
;
;----------------------------------------------
;
L51DF  	FDB  L51E5		; DEVICE ID $F4, MODE 1/0
L51E1  	FDB  L526C		; DEVICE ID $F4, MODE 1/1 
L51E3  	FDB  L52F3		; DEVICE ID $F4, MODE 1/2
;
;----------------------------------------------
; DEVICE ID $F4, MODE 1/0
; TYPE $0D PCM P/N 16196395, S/S BY PCM 16197427
;
;  SCAN TOOL DATA 
;----------------------------------------------
;
L51E5  	FDB  $0000		; NEXT ENRTY ADDRESS
;
L51E7  	FCB  $F4		; DEVICE ID
;
L51E8  	FCB  $80		; MEM FLAG, TBL FROM RAM
L51E9  	FCB   64		; BYTES TO TX
;
L51EA  	FDB  L036E		; ADDR OF OUTPUT BUFFER
L51EC  	FDB  L036E		; ADDR OF INPUT  BUFFER
;
L51EE  	FDB  L4000		;  1, EPROM ID,MSB
L51F0  	FDB  L4000+1		;  2, EPROM ID,LSB
;
L51F2  	FDB  L0004		; 3, NVMW, NON-VOL RAM MODE WD
				; b7 1 = ERR 42, EST MON
				; b6 1 = ERR 24, Vss, (not used)
				; b5 1 = IAC KICK DN, (not used)
				; b4 1 = HOT RESTART PROCEEDING
				; b3 1 = BAD SHUT DOWN
				; b2 1 = RAM REFRESH ERR OCCOURED
				; b1 1 = CLS LP TMR OK
				; b0 1 = o2 SENSOR RDY
;
L51F4  	FDB  L0036		; 4	 DIACMW2, NON-VOL IDLE CNT'L MD WD
				; b7 1 =  IDLE RPM TO HIGH
				; b6 1 =  THROTTLE KICKER HAS BEEN DISABLED ONCE    
				; b5 1 =  PW STEER PRESS LOAD IN WORK
				; b4 1 =  STALL SAVER IN WORK
				; b3 1 =  CLSD LP ON RPM ENABLED
				; b2 1 =  CLSD LP QUALS MET, (clsd tps & low mph)
				; b1 1 =  DRIVE, 0 = PK/NEUT
				; b0 1 =  A/C ON
;
L51F6  	FDB  L0007		; 5, IAC, PRESSENT MOTOR POSIT         
				; 0 to MAX IAC (typ 145)
;
L51F8  	FDB  L000B		; 6, ERROR WD 1  
				; b7 1 = ERR 13, o2 SENSOR 
				; b6 1 = ERR 14, COOL SENSOR, HIGH  
				; b5 1 = ERR 15, COOL SENSOR, LOW 
				; b4 1 = ERR 16, Vss BUFFER, (2002 PPM)
				; b3 1 = ERR 17, RPM SIGNAL PROBLEM (not used)    
				; b2 1 = ERR 18, CAM CRANK ERROR  (not used) 
				; b1 1 = ERR 19, (not used)
				; b0 1 = ERR 21, TPS SENSOR HIGH
;   
L51FA  	FDB  L000C		; 7, ERROR WD 2           	
				; b7 1 = ERR 22, LOW TPS    
				; b6 1 = ERR 23, LOW MAT    
				; b5 1 = ERR 24, LOW OUTPUT SPEED (Vss)  
				; b4 1 = ERR 25, MAT SENSOR HIGH     
				; b3 1 = ERR 26,  (not used)
				; b2 1 = ERR 27,  (not used)
				; b1 1 = ERR 28, PRESS SW MANIFOLD   
				; b0 1 = ERR 29, (not used)
;   
L51FC  	FDB  L000D		;  8, ERR MD WD 3		                		
				; b7 1 = ERR 31, GOVENOR    
				; b6 1 = ERR 32, EGR ERROR     
				; b5 1 = ERR 33, MAP SENSOR HI    
				; b4 1 = ERR 34, MAP SNENSOR LOW      
				; b3 1 = ERR 35, IAC ERROR    
				; b2 1 = ERR 36, IAC THROTTLE KICKER FAIL (not used)    
				; b1 1 = ERR 37, TCC BRAKE SW STUCK ON    
				; b0 1 = ERR 38, TCC BRAKE SW STUCK OFF    
;    
L51FE  	FDB  L000E		; 9,  ERR WD 4       	
				; b7 1 = ERR 39, TCC STUCK OFF  
				; b6 1 = ERR 41, 1x CAM SENSOR FAIL 
				; b5 1 = ERR 42, IGN ERROR (EST)
				; b4 1 = ERR 43, KNOCK SENSOR CKT    
				; b3 1 = ERR 44, o2 SENSOR LEAN
				; b2 1 = ERR 45, o2 SENSOR RICH 
				; b1 1 = ERR 46, VATS FAIL
				; b0 1 = ERR 47, (not used) 
;    
L5200  	FDB  L000F		; 10,   ERR WD 5   	
				; b7 1 = ERR 48, (not used) 
				; b6 1 = ERR 49, (not used) 
				; b5 1 = ERR 51, EPROM ERROR
				; b4 1 = ERR 52, SYS VDC HI-LONG TEST    
				; b3 1 = ERR 53, HI SYS VOLTAGE  
				; b2 1 = ERR 54, LOW FUEL PUMP VDV 
				; b1 1 = ERR 55, FAULTY COMPUTER (ADU)
				; b0 1 = ERR 56, QUAD DRIVER B FAULT (not used)   
;  
L5202  	FDB  L0008		; 11, IAC MOTOR               	
;    
L5204  	FDB  L0009		; 12, DIACMCW1, IDLE AIR CNT'L NV MODE WD
				; b7    (not used)  
				; b6  = 1st PASS OF ERR 36 HAS FAILED   
				; b5  = WARM IDLE STABLE, A/C ON
				; b4  = WARM IDLE STABLE, A/C OFF    
				; b3    (not used)      
				; b2  = R/S REQUESTED IF BIT CLEAR   
				; b1  = 1st DRIVE AWAY FLAG FOR IAC KICK DOWN LOG   
				; b0  = IAC MOTOR Reset IN WORK
;   
L5206  	FDB  L000A		; 13, DIACMCW2, IDLE AIR CNT'L MODE WD
       				; b7 1 = (not used)   
				; b6 1 = (not used)   
				; b5 1 = (not used)   
				; b4 1 = STEPPER MOTOR 1=ON/0=OFF 	 
				; b3 1 =  COIL B STATE ON 
				; b2 1 =  COIL A STATE ON 
				; b1 1 =  (not used)  
				; b0 1 =  MOTOR DIRECTION, 1=EXTEND  
;    
L5208  	FDB  L0037		; 14, DIACMW4  IAC CNT'L MD WD 
				; b7 1 = ADD DERIV TERM TO g/SEC FLOW, (0 = SUB)
				; b6 1 = PRORP LIMITING ATHORITY BEING EXERSIZED    
				; b5 1 = PART 2 OF DIAG TEST TO BE RUN  
				; b4 1 = THROTTLE KICKER BARO DISABLE REQUESTED    
				; b3 1 = THROTTLE KICKER DISABLE REQUESTED 
				; b2 1 = THROTTLE KICKER ACTIVE    
				; b1 1 = ETC * KONST    
				; b0 1 = ETC ONCE FLAG   
;  
L520A  	FDB  L00A2		; 15, COOL TEMP                     	    
				;      Deg  c = n*.75 - 40
				;      (deg F = 1.35N - 40)
;  
L520C  	FDB  L00A1		; 16, BATTERY A/D                   	    
				; VDC/10    
;  
L520E  	FDB  L00A0		; 17, TPS A/D	
				; BIN = VDC * 50        
;    
L5210  	FDB  L082E		; 18, A/D MAP VALUE	
			        ; BIN = VDC * 50     
;   
L5212  	FDB  L01F8		; 19, o2 VOLTS, (VDC/226)           	    
;   
L5214  	FDB  L003D		; 20, MWAF, AFR MD WORD 0,	
				; b7 1 = DELIVER ASYNC PULSE	
				; b6 1 = ACELLL ENR IS ACTIVE   
				; b5 1 = PWR ENR IS ACTIVE   
				; b4 1 = DECEL ENLEAN IS ACYIVE   
				; b3 1 = DELAY BLM UPDATE   
				; b2 1 = BLK LRN ADDR CHANGE 1 = CHANGED
				; b1 1 = VATS PASS/FAIL    
				; b0 1 = PWR ENR ENR FLAG    
; 
L5216  	FDB  L003B		; 21, SDMW, SERIAL DATA MD WORD	            
				; b7 1 = 2nd BYTE 8192 TX IS PENDING	
				; b6 1 = 8192 TX IN WORK   
				; b5 1 = 8192 TX OVERRUN   
				; b4 1 = IN MODE 4, ($F4),  (not used)   
				; b3 1 = IN MODE 4, ($F5),  (not used)  
				; b2 1 = XMISH DIAG DISABLED   
				; b1 1 = MODE 10 ID $F4 
       	            		; b0 1 = MODE 10 ID $F5      
;  
L5218  	FDB  L0044		; 22, MWBG, MINOR LOOP MODE WD
				; b7 1 = LOCKED IN ERR 42A	
				; b6 1 = 1st GOOD ERR 42A FLAG
				; b5 1 = HIGH MAT CONDITIONS OBSERVED
				; b4 1 = IGNITION OFF
				; b3 1 = 1st DRP VALID 
				; b2 1 = SKIP ERR 43 DUE TO ALDL
				; b1 1 = ACELL ENR CLAMP ACTIVE   
;   
L521A  	FDB  L004F		; 23, MW1, MINOR LOOP MODE WD 1
				;  b7 1 = ENGINE RUNNING	
				;  b6 1 = MAJOR LOOP EST MONITOR ENABLE
				;  b5 1 = VE INT RESET 
				;  b4 1 = RUN FUEL        
				;  b3 1 = OPEN TPS VE FLAG   
				;  b2 1 = LOOP OVERRAN 6,25 MS PERIOD
				;  b1 1 = CHECK ENG LAMP DELAY   
				;  b0 1 = RETARD, 0 = ADVANCE     
; 
L521C  	FDB  L0050		; 24,  MW2, MINOR LOOP MODE WD 2	   
				; b7 1 = IDLE FLAG	
				; b6 1 = IDLE SPARK ENABLED
				; b5 1 = RET PULSE OCCOURED
				; b4 1 = DIAG SW IN DIAG POSIT.    
				; b3 1 = DIAG SW IN FACTORY TEST 
				; b2 1 = DRP OCCOURED 6.25msec TEST   
				; b1 1 = OPN LOOP IDLE FLG FOR AIR SW ENGAGE AT IDLE   
				; b0 1 = SYNC MAP SENSOR READS     
;  
L521E  	FDB  L004D		; 25, I/O D PORT C, I/O PORT C             	  	
       	 		   	; b7 1 = FWD LOW SW (NO) 1=ON  
				; b6     (not used)
				; b5     (not used) 		  
				; b4 1 = RANGE SW 3 OFF 	  						  
				; b3 1 = RANGE SW 2 OFF 	  
				; b2 1 = RANGE SW 1 OFF 	  
				; b1 1 = BK SW ON    		  
				; b0 1 = A/C REQUEST ON 	  
				;
				; PRNDL TABLE:
				; RANGE1  RANGE2 RANGE3  GEAR
				; ---------------------------
				;   0       1      1     Low
				;   1       1      1     Drive2
 				;   1       1      0     Drive3
				;   1       0      0     Drive4
				;   1       0      1     P/N
				;   0       0      1     Reverse
				;
				;     
L5220  	FDB  L0046		; 26, CLCC MW, MAJOR LOOP MODE WD 2
				; b7 1 = HAS BEEN IN CLS LP ONCE SINCE START UP                          	 
				; b6 1 = NON VOL MEM BOMBED   	
				; b5 1 = DECEL FUEL C/O IAC FAST FILTER 
				; b4 1 = OVERSPEED FUEL SHUT OFF    
				; b3 1 = DECEL FUEL C/O    
				; b2 1 = AIR MANAGMENT OFF, 1=ON   
				; b1 1 = SLOW RICH/LEAN, 1 = RICH   
				; b0 1 = SYNC ACELL ENRICH     
;  
L5222  	FDB  L0316		; 27, ADBARO, BARO A/D                  	    
				;	  VDC = N x (5/256)
				;     kPA = (N x 28.06)/2.71
;
L5224  	FDB  L0041		; 28, LCCPWM, TCC A/C & EGR MD WD					                         
       	 			; b7 1 = CCP SOLENOID ON
				; b6 	 (not used)
				; b5 1 = PARK/NEUTRAL
				; b4 1 = A/C PRESSURE SW, (A/C ON)
				; b3 1 = A/C ACEL ENR ENABLED 
				; b2 1 =
				; b1 1 = SPK CORRECTION DUE TO MAT IS NEGITIVE
				; b0 1 = ADJ ... THE EGR OFFSET BY 2 b0 1 =	 (not used)
; 
L5226  	FDB  L003E		; 29,  NWAF1, AIR/FUEL MD WD 1                      	 	
				; b7 1 = CLOSED LOOP FLAG	
				; b6 1 = RICH/ LEAN, 1 = RICH   
				; b5 1 = CLOSED LOOP FOR O/L IDLE CONDITION 
				; b4 1 = ASYNC PULSE FLAG
				; b3 1 = A/F DECAY INT DONE FOR COLD P/D  
				; b2 1 = LOW BATTERY
				; b1 1 = BLM ENABLE    
				; b0 1 = DECELL FUEL C/O TPS ACEL ENRICH    
; 
L5228  	FDB  L0085		; 30, TCCMODE, TCC MODE WD	     
				;  b7 1 = NEG SLIP RECENTLY PREVENTS TCC APPLY
				;  b6 1 = TCC IS BEING APPLIED    
				;  b5 1 = TCC IS IN LOCK ADJ MODE   
				;  b4 1 = TCC IS IN RELEASE MODE SLIPPING   
				;  b3 1 = USE HYST FOR VEHICLE SPEED   
				;  b2 1 = LOW THRESH FOR COPE TCC SELECTED   
				;  b1 1 = TCC ENABLE SOLENIOD VALID   
				;  b0 1 = TCC IS BEING FORCED OFF
;   
L522A  	FDB  L02A7		; 31, FILTMPH,  MPH	 0 - 255 MPH                   	      
				;		MPH/1
				;		KPH = 1.61  
;  
L522C  	FDB  L004C		; 32, IODPORTB,  I/O D PORT B                        
				; b7 1 = M/CLR					
				; b6 1 = REF IRQ CLR   			
				; b5 1 = FORCE MOTOR ENABLE		
				; b4 1 = IAC ENABLE   										
				; b3 1 = IAC COIL B ON   
				; b2 1 = IAC COIL A ON  
				; b1 1 = 2 - 3 SOL ON,	SHIFT B
				; b0 1 = 1 - 2 SOL ON,	SHIFT A
				;
				;
				;  SHIFT B  SHIFT A  GEAR
				;  ----------------------
				;    0        0       3rd
				;    0        1       4th
				;    1        0       2nd
				;    1        1       1st
				;  ----------------------
;   
L522E  	FDB  L0055		; 33, FUEL PUMP                    	     
				; (VDC/10)      	
;   
L5230  	FDB  L0062		; 34, FUEL PUMP, (BATTERY VOLTS * 10)          			       	     
				; 		(VDC/10)     
;     
L5232  	FDB  $3209		; 35, OLDREFPER, LAST MINOR LP DRP FROM ECU (MSB)
L5234  	FDB  $3209+1		; 36, OLDREFPER+1, LAST MINOR LP DRP FROM ECU (LSB)
				;
				; uSEC / 15.26 8 65536
				;
				; RPM = 983040(MSB*256)+LSB    
				; RPM = ((65536 * 120)/8)/CAL    
				; RPM = 983040/CAL    
				;    
				; CAL = 65536 *(120/(RPM * NUM CYL)    
				;----------------------------- 
;    
L5236  	FDB  L01D2		; 37, EGRDC, EGR D.C                     
				; 	     256/100
;
L5238  	FDB  L0051		; 38, MISC MODE WD	                                   
       				; b7 1 =  TRANSITION FLAG (1 = TRANSITION IN WORK)
				; b6 1 =  START UP SPARK FILTER DONE
				; b5 1 =  DECEL FUEL C/P SPARK FILTER
				; b4 1 =  HIGH BATTERY VOLTAGE     
				; b3 1 =  POWER DOWN IN WORK 
				; b2 1 =  RETEST RAM IN BACKGROUND  
				; b1 1 =  SINGLE FIRE 1st TIME
				; b0 1 =  CPI/PFI SINGLE FIRE, (1 S/F)
;
L523A  	FDB  L30F8		; 39, TIMEENG, ENGINE RUNNING TIME SEC
L523C  	FDB  L30F8+1		; 40, TIMEENG+1, 
				;	  SEC/1 (16 BIT VALUE)
				; 	  SEC = (MSB * 256) + LSB))  	
;   
L523E  	FDB  L0857		; 41, DESSPD, DESIRED IDLE RPM/12.5               
;  
L5240  	FDB  L0197		; 42, NDTHROPS, TPS FOR ENGINE             	        
				; 	   (CURRENT NON-DEAULT VALUE)   
				;	   % * 2.56
;  
L5242  	FDB  L0052		; 43, MODE WD 4                        	        	
				; b7 1 = QUAIS CLSD LP ENABLED	
				; b6 1 = CAT OVER TEMP
				; b5 1 = A/C ANTICIPATING TO BE ON
				; b4 1 = A/C DISABLED DUE TO HIGH RPM
				; b3 1 = VARIABLE TUNING CNT'L
				; b2 1 = TPS CLOSED, (FOR BLM)
				; b1 1 = ZERO ACTIVITY FLAG (LO OCTANE)
				; b0 1 = HIGH KNOCK ACTIVITY     
; 
L5244  	FDB  L0053		; 44, MODE WD 5 		
				; b7  	 (not used)	
				; b6 1 = SINGLE FIRE ALT EXIT IS DESIRED	    
				; b5 1 = A INJECTOR FIRED AT 1st DRP    
				; b4 1 = PWR STEER CRAMP STALL SAVER    
				; b3 1 = HI RPM INDICATED BY XMISH (ABUSE LOGIC)    
				; b2 1 = SHIFT LIGHT ON    
				; b1 1 = DLATCH ACTIVE    
				; b0 1 = ACTUAL ENGINE RPM OVER DESIRED RPM      
;
L5246  	FDB  L3213		; 45, SAREFFNL FINAL VALUE OF SAREF (MSB)
L5248  	FDB  L3213+1		; 46, SAREFFNL+1 FINAL VALUE OF SAREF (LSB)
				;	    DEG = N /(256/90)
;
L524A  	FDB  L3226		; 47, PA2OLD, CPU PA1 KNOCK COUNTS FM LAST MINOR LP, (MSB)                     
L524C  	FDB  L3226+1		; 48, PA2OLD, CPU PA1 KNOCK COUNTS FM LAST MINOR LP, (LSB)
;  
L524E  	FDB  L0232		; 49, INT, CLS LP INTEGRATOR COUNTS                        		
				; 0 - 255    
;   
L5250  	FDB  L02BC		; 50, DESTPS, DESIRED GOVENERING TPS TO BE OUTPUT	
				;	 % * 256/100
;  
L5252  	FDB  L0298		; 51, ALDLCNTR, ALDL RICH LEAN CHAGE COUNTER     
				;    COUNTS
;   
L5254  	FDB  L006E		; 52, EGR MD WD 	                                    
				; b7 1 = EGR ON
				; b6 1 = ERG MPH HYST
				; b5 1 = EGR TPS HYST  
				; b4 1 = EGR MAP HYST  
				; b3 1 = ACELL ENR 1st TIME   
				; b2 1 = EGR HI VAC HYST
				; b1 1 = BURST KNOCK ACTIVE   
				; b0 1 = EGR DIAG INT RESET           
;   
L5256  	FDB  L0075		; 53, GOVMW, ELECTRONIC GOVENOR MD WD                              
				; b7 1 = IN MPH GOVENOR   
				; b6 1 = MPH OVERSPEED   
				; b5 1 =    
				; b4 1 = RPM LEAD TPS RETURN   
				; b3 1 = OVERSPEED LIGHT ON   
				; b2 1 = RPM OVERSPEED
				; b1 1 = GOV LEAD MODE ANTICP RPM GOV'ING
				; b0 1 = IN RPM GOV           
;
L5258  	FDB  L026A		; 54, BLM CELL NUM
				;  (0 - 16)
;
L525A  	FDB  L026B		; 55, BLM MULTIPLIER * 128	
;
L525C  	FDB  L022F		; 56, NOCKRTD, KNOCK RETARD	
				;	  DEG = 45N/256		
				;	  (E6 =  AFR RATIO * 10)	
;	
L525E  	FDB  L326F		; 57, INJ PW, MSB	
L5260  	FDB  L326F+1		; 58, INJ PW, LSB	
				;	
				; (MSB*256) + LSB))/65.536	
				; 	
				; MSEC/65.536    
;  
L5262  	FDB  L01D1		; 59, DSEGRPOS, DESIRED EGR CMD,	
				; 	   	% * 2.56
;
L5264  	FDB  L01DC		; 60, CCP DUTY CYCLE 	
				; 		%/2.56 
;
L5266  	FDB  L01D7		; 61, AEGRPOS  LINEAR EGR POS  
				;	 (A/D VAL)
;
L5268  	FDB  L01F9		; 62,  PURGEDC, PURGE D.C
				;  	  % * 2.56
;
L526A  	FDB  L0855		; 63, MATDEGA,  NON DEFAULTED MAT
				;	   	N= (DEG C - 40) * 256/192
				;  		N= (DEG C - 40) * 256/192    
;
;---------------------------------------------
; DEVICE ID $F4, MODE 1/1	
;
; 03-05-1996  Dissassemby of BDWM
;
;---------------------------------------------
;
L526C  	FDB  $0000		; NEXT ENRTY ADDRESS	
;
L526E  	FCB  $F4		; DEVICE ID	
;
L526F	FCB  $80		; MEM FLAG, FORM RPM TBL	
L5270  	FCB  64			; NUM OF BYTES TO TX	
;
L5272  	FDB  L036E		; ADDR OF OUTPUT BUFFER	
L5274  	FDB  L036E		; ADDR OF INPUT BUFFER	
;
L5275   FDB  L003D 		; 1, AFR MD WORD 0,	
				; b7 1 = DELIVER ASYNC PULSE	
  				; b6 1 = ACELLL ENR IS ACTIVE   
				; b5 1 = PWR ENR IS ACTIVE   
				; b4 1 = DECEL ENLEAN IS ACYIVE   
				; b3 1 = DELAY BLM UPDATE   
				; b2 1 = BLK LRN ADDR CHANGE 1 = CHANGED
				; b1 1 = VATS PASS/FAIL    
				; b0 1 = PWR ENR ENR FLAG    
;    
L5277   FDB L01F8		;  2, o2 VOLTS, (VDC/226)           	    
; 
L5279   FDB L003E		;  3, AFR MD WD 1                      	 	
				; b7 1 = CLOSED LOOP FLAG	
				; b6 1 = RICH/ LEAN, 1 = RICH   
				; b5 1 = CLOSED LOOP FOR O/L IDLE CONDITION 
				; b4 1 = ASYNC PULSE FLAG
				; b3 1 = A/F DECAY INT DONE FOR COLD P/D  
				; b2 1 = LOW BATTERY
				; b1 1 = BLM ENABLE    
				; b0 1 = DECELL FUEL C/O TPS ACEL ENRICH    
;  
L527B   FDB L02EB		;  4, "CHOKE" AFR VALUE
;
L527D   FDB L0315		;  5,
;
L527F   FDB L00A1		;  6, A/D batt volts (volts = val/10)                     	    
;
L5281   FDB L0262		;  7,
;
L5283   FDB L087F		;  8, Acell Enrich PW ADDER
;		  
L5285   FDB L0261		;  9,
;
L5287   FDB L02EE		; 10, SPK
;
L5289   FDB L025E 		; 11,
;
L528B   FDB L026D		; 12, AFR
;
L528D   FDB L01EF		; 13, BARO
;
L528F   FDB L01EA		; 14, OLD MAP
;
L5291   FDB L01FF		; 15, OLD TPS
;
L5293   FDB L026B		; 16, BLM	
;
L5295   FDB L026A		; 18, BLM CELL NUM
				;  (0 - 16)
;
L5297   FDB L0037		; 14, DIACMW4  IAC CNT'L MD WD 
				; b7 1 = ADD DERIV TERM TO g/SEC FLOW, (0 = SUB)
				; b6 1 = PRORP LIMITING ATHORITY BEING EXERSIZED    
				; b5 1 = PART 2 OF DIAG TEST TO BE RUN  
				; b4 1 = THROTTLE KICKER BARO DISABLE REQUESTED    
				; b3 1 = THROTTLE KICKER DISABLE REQUESTED 
				; b2 1 = THROTTLE KICKER ACTIVE    
				; b1 1 = ETC * KONST    
				; b0 1 = ETC ONCE FLAG  
; 
L5299   FDB L0006		; 20 COOL
;
L529B   FDB L0871		; 21 IAC FLOW 
;
L529D   FDB L02D0		; 22
;
L529F   FDB L0873		; 23
;
L52A1   FDB L02D2		; 24
;
L52A3   FDB L02C8		; 25 IAC FLOW Vs. COOL
;
L52A5   FDB L02C6		; 26  IAC FLOW Vs. COOL
;
L52A7   FDB L0875		; 27 IAC COLD OFFSET DELAY PERIOD
;
L52A9   FDB L0857		; 28, DESSPD, DESIRED IDLE 
				; RPM/12.5        
;
L52AB   FDB L0254		; 29, VE TABLE RESULTS
;
L52AD   FDB L01D1		; 30, DESIRED EGR POSIT 	
				; %/2.56 
;
L52AF   FDB L01D5		; 31
;
L52B1   FDB L022E		; 32. ??? SPK BIAS
;
L52B3   FDB L022D		; 33. LO OCT BASE SPK RETARD SPK
;
L52B5   FDB L02A7		; 34, FILTMPH,  MPH	 0 - 255 MPH                   	      
				;	MPH/1
				;	KPH = 1.61  
;
L52B7   FDB L0882		; 35
;
L52B9   FDB L029E		; 36
;
L52BB   FDB L0255		; 37
;
L52BD   FDB L088B		; 38
;
L52BF   FDB L02CA		; 39
;
L52C1   FDB L02C0		; 40
;
L52C3   FDB L02CC		; 41
;
L52C5   FDB L02C2		; 42
;
L52C7   FDB L02CD		; 43
;
L52C9   FDB L02C3		; 44
;
L52CB   FDB L02CF		; 45
;
L52CD   FDB L02C5		; 46
;
L52CF   FDB L086D		; 47
;
L52D1   FDB L0232		; 48, INTEGRATOR                        		
				; 0 - 255    
;
L52D3   FDB L0862		; 49
;
L52D5   FDB L0007		; 50, IAC, PRESSENT MOTOR POSIT         
				; 0 to MAX IAC (typ 145)
;
L52D7   FDB L027A		; 51, CURRENT MAP VALUE
				;      (n+28.05)/2.71
;
L52D9   FDB L022F		; 52
;
L52DB   FDB L01FC		; 53,  TPS
;
L52DD   FDB L0214		; 54
;
L52DF   FDB L029D		; 55
;
L52E1   FDB L02F1		; 56
;
L52E3   FDB L0859		; 57
;
L52E5   FDB L0217		; 58
;
L52E7   FDB L0219		; 59 SPARK
;
L52E9   FDB L00F9		; 60,                                    
				; b7 1 =    
				; b6 1 =    
				; b5 1 =    
				; b4 1 =    
				; b3 1 =    
				; b2 1 =    
				; b1 1 =    
				; b0 1 =            
;
L52EB   FDB L00B1		; 61
;
L52ED   FDB L00B2		; 62
;
L52EF   FDB L026F		; 63
;
L52F1   FDB L0270		; 64
;
;---------------------------------------------
; DEVICE ID $F4, MODE 1/2 
;
;
;
;---------------------------------------------
;
L52F3  	FDB	$0000		;
;
L52F5  	FCB	$F4		; DEVICE ID
;
L52F6  	FCB	$80		; MEM FLAG, TBL FROM ROM
;			
L52F7  	FCB	 37		; NUM BYTE TO TX
;
L52F8  	FDB  L036E		; ADDR OF OUTPUT BUFFER
L52FA  	FDB  L036E		; ADDR OF INPUT BUFFER
;
L52FC  	FDB  L00B1		;  1,
;
L52FE  	FDB  L00B2		;  2,
;
L5300  	FDB  L027A		;  3, CURRENT MAP VALUE
;
L5302  	FDB  L0214		;  4,
;
L5304  	FDB  L00A2		;  5, COOL TEMP                     	    
				;  Deg  c = n*.75 - 40
				;  (deg F = 1.35N - 40)
;
L5306  	FDB  L0007		;  6, IAC, PRESSENT MOTOR POSIT         
				; 0 to MAX IAC (typ 145)
;
L5308  	FDB  L026D		;  7, AFR
;
L530A  	FDB  L00F9		;  8,                                    
				; b7 1 =    
				; b6 1 =    
				; b5 1 =    
				; b4 1 =    
				; b3 1 =    
				; b2 1 =    
				; b1 1 =    
				; b0 1 =            
;   
L530C  	FDB  L01FC		;  9,  TPS
;
L530E  	FDB  L02A7		; 10, FILTMPH,  MPH	 0 - 255 MPH                   	      
				;	MPH/1
				;	KPH = 1.61  
;   
L5310  	FDB  L0873		; 11,
;
L5312  	FDB  L0871		; 12, IAC FLOW
;
L5314  	FDB  L0889		; 13,
;
L5316  	FDB  L02F1		; 14,
;
L5318  	FDB  L0857		; 15, DESSPD, DESIRED IDLE               
				; RPM/12.5        
;
L531A  	FDB  L088B		; 16,
;
L531C  	FDB  L0862		; 17,
;
L531E  	FDB  L0879		; 18,
;
L5320  	FDB  L0876		; 19, %IAC FLOW MAX STEP FOR A/C OFF/ON
;
L5322  	FDB  L02EB		; 20, "CHOKE" AFR VALUE
;
L5324  	FDB  L026B		; 21,, BLM
;	
L5326  	FDB  L00A1		; 22,, BATTERY A/D                   	    
				; VDC/10    
;
L5328  	FDB  L0232		; 23, INTEGRATOR                        		
				; 0 - 255    
; 
L532A  	FDB  L029E		; 24,
;
L532C  	FDB  L02EE		; 25, SPK
;
L532E  	FDB  L02A3		; 26, 
;
L5330  	FDB  L01F8		; 27, o2 VOLTS, (VDC/226)           	    
;
L5332  	FDB  L022F		; 28,
;
L5334  	FDB  L026A		; 29, BLM CELL NUM
				;  (0 - 16)
;
L5336  	FDB  L0255		; 30,
;
L5338  	FDB  L01F5		; 31,
;
L533A  	FDB  L01EC		; 32, CURRENT VACUUM VALUE
;
L533C  	FDB  L0261		; 33,
;
L533E  	FDB  L0262		; 34,
;
L5340  	FDB  L026F		; 35,
;
L5342  	FDB  L0270		; 36,
;
L5344  	FDB  L01D0		; 37,
;
;----------------------------------------------
; not used
;
;----------------------------------------------
;
L5346  	FDB  L0051		; 38,  
;
L5348  	FDB  L30F7		; 39, 
; 
L534A  	FDB  L30F8		; 40, TIMING, (MSB) ENGINE RUNNING TIME SEC
;
L534C  	FDB  L0857		; 41,  DESSPD, DESIRED IDLE
				;	RPM/12.5
;
L534E  	FDB  L01FA		; 42,
;
L5350  	FDB  L0052		; 43, MODE WD 4                            	       
				; b7 1 = QUAIS CLSD LP ENABLED	
				; b6 1 = CAT OVER TEMP
				; b5 1 = A/C ANTICIPATING TO BE ON
				; b4 1 = A/C DISABLED DUE TO HIGH RPM  
				; b3 1 = VARIABLE TUNING CNT'L
				; b2 1 = TPS CLOSED, (FOR BLM)
				; b1 1 = ZERO ACTIVITY FLAG (LO OCTANE)
				; b0 1 = HIGH KNOCK ACTIVITY     
;    
L5352  	FDB  L0053		; 44,  MODE WD 5 		
				; b7  	 (not used)	
				; b6 1 = SINGLE FIRE ALT EXIT IS DESIRED	    
				; b5 1 = A INJECTOR FIRED AT 1st DRP    
				; b4 1 = PWR STEER CRAMP STALL SAVER    
				; b3 1 = HI RPM INDICATED BY XMISH (ABUSE LOGIC)    
				; b2 1 = SHIFT LIGHT ON    
				; b1 1 = DLATCH ACTIVE    
				; b0 1 = ACTUAL ENGINE RPM OVER DESIRED RPM      
;
L5354  	FDB  L3211		; 45,
; 
L5356  	FDB  L3211+1		; 46, 
;	
L5358  	FDB  L3224		; 47, 
;
L535A  	FDB  L3224+1		; 48, 
;
L535C  	FDB  L0230		; 49, 
;
L535E  	FDB  L02BA		; 50, 
;.
L5360  	FDB  L0296		; 51, 
;
L5362  	FDB  L006E		; 52, EGR MD WD 	                              
				; b7 1 = EGR ON
				; b6 1 = ERG MPH HYST
				; b5 1 = EGR TPS HYST  
				; b4 1 = EGR MAP HYST 
				; b3 1 = ACELL ENR 1st TIME   
				; b2 1 = EGR HI VAC HYST
				; b1 1 = BURST KNOCK ACTIVE   
				; b0 1 = EGR DIAG INT RESET           
;
L5364  	FDB  L0075		; 53, ELECTRONIC GOVENOR MD WD                             
				; b7 1 = IN MPH GOVENOR   
				; b6 1 = MPH OVERSPEED   
				; b5 1 =    
				; b4 1 = RPM LEAD TPS RETURN   
				; b3 1 = OVERSPEED LIGHT ON   
				; b2 1 = RPM OVERSPEED
				; b1 1 = GOV LEAD MODE ANTICP RPM GOV'ING
				; b0 1 = IN RPM GOV           
;
L5366  	FDB  L0268		; 54, DECEL FUEL C/O EXIT TIMER
;
L5368  	FDB  L0269		; 55,
;
L536A  	FDB  L026C		; 56,
;
L536C  	FDB  L326D		; 57,
;
L536E  	FDB  L326D+1		; 58,
;
L5370  	FDB  L01CF		; 59,
;
L5372  	FDB  L01DA		; 60,
;
L5374  	FDB  L01D5		; 61,
;
L5376  	FDB  L01F7		; 62, OLD o2 VDC
;
L5378  	FDB  L0855		; 63, LINEAR MAT VAL   	
				; (DEG C + 40) * (256/192) 
;
;----------------------------------------------
;  DEVICE ID $F4, MODE 2
;  64 ADDRESS SEQUENTIAL MEM DUMP
;
;----------------------------------------------
;
L537A  	FDB  $0000 		; NEXT ENRTY ADDRESS
;
L537C  	FCB  $F4		; DEVICE ID
;
L537D  	FCB  $40		; MEM FLAG, TBL FM RAM
;
L537E  	FCB   65		; NUM BYTES TO TX
;
L537F  	FDB  L036E		; ADDR OF OUTPUT BUFFER
L5381  	FDB  L036E		; ADDR OF INPUT BUFFER
;----------------------------------------------
;		
;----------------------------------------------
;  DEVICE ID $F4, MODE 3
;
;  SEQUENTIAL 8 ADDRESS MEM DUMP
;----------------------------------------------
;
L5383  	FDB  $0000		; NEXT ENRTY ADDRESS
;
L5385  	FCB  $F4		; DEVICE ID
;
L5386  	FCB  $40		; MEM FLAG, TBL FM RAM
L5387  	FCB    9		; NUM BYTES TO TX
;
L5388  	FDB  L036E		; ADDR OF OUTPUT BUFFER
L538A  	FDB  L036E		; ADDR OF INPUT BUFFER
;---------------------------------------------
;
;---------------------------------------------
;  DEVICE ID $F4, MODE 4  
;
;  'CONTROLER MODE'
;---------------------------------------------
;
L538C  	FDB  $0000		; NEXT ENRTY ADDRESS
;
L538E  	FCB  $F4		; DEVICE ID
;
L538F  	FCB   00		; MEM FLAG, NONE
L5390  	FCB    1		; NUM BYTES TO TX
;
L5391  	FDB  L036E		; ADDR OF OUTPUT BUFFER
L5393  	FDB  L036E		; ADDR OF INPUT BUFFER
;---------------------------------------------
;
;---------------------------------------------
;  DEVICE ID $F4, MODE  10
;
;  MODE 10, ALDL ERR RESET
;---------------------------------------------
;
L5395  	FDB  $0000		; NEXT ENRTY ADDRESS
;
L5397  	FCB  $F4		; DEVICE ID
;
L5398  	FCB    0		; MEM FLAG, NONE
L5399  	FCB    1		; NUM BYTES TO TX
;
L539A  	FDB  L036E		; ADDR OF OUTPUT BUFFER
L539C  	FDB  L036E		; ADDR OF INPUT BUFFER
;---------------------------------------------
;
;**********************************************
;* $F5 TABLES
;*
;**********************************************
;---------------------------------------------
;  DEVICE ID $F5, MODE 
;
;
;---------------------------------------------
;
L539E  	FDB  L53BF		; NEXT ENRTY ADDRESS
;
L53A0  	FCB  $F5		; DEVICE ID
;
L53A1  	FCB    0		; MEM FLAG, NONE
L53A2  	FCB  128 		; NUM BYTES TO TX
;
L53A3  	FDB  L036E		; ADDR OF OUTPUT BUFFER
L53A5  	FDB  L036E		; ADDR OF INPUT BUFFER
;
L53A7  	FDB  L53BF		;  0, DEVICE ID $F5, MODE 0, RESET NORMAL COMM
L53A9  	FDB  L53CE		;  1, DEVICE ID $F5, MODE 1, SCAN TOOL DATA
L53AB  	FDB  L5549		;  2, DEVICE ID $F5, MODE 2, 64 ADDRESS MEM DUMP
L53AD  	FDB  L5552		;  3, DEVICE ID $F5, MODE 3, SEQUENTIAL 8 ADDRESS MEM DUMP
L53AF  	FDB  L555B		;  4, DEVICE ID $F5, MODE 4, CONTROLER MODE
L53B1  	FDB  $0000		;  5, not used 
L53B3  	FDB  $0000		;  6, not used 
L53B5  	FDB  $0000		;  7, not used
L53B7  	FDB  $0000		;  8, not used
L53B9  	FDB  $0000		;  9, not used
L53BB  	FDB  L5564		; 10, DEVICE ID $F5, MODE 10, ALDL ERR RESET
L53BD  	FDB  $0000		; 11, not used
;----------------------------------------------
;
;----------------------------------------------
; DEVICE ID $F5
; MODE 0, RESET NORMAL COMM
;
;    MODE 0 (RETURN TO NORMAL MODE)
;            ALDL REQUEST:
;            - MESSAGE ID     = $F5
;            - MESSAGE LENGTH = $56
;            - MODE           = $00
;            - SUM CHECK
;
;            ECM RESPONSE:
;              NONE
;----------------------------------------------
;
L53BF  	FDB  $0000		; NEXT ENRTY ADDRESS
;
L53C1  	FCB  $F5		; DEVICE ID
;
L53C2  	FCB    0		;
L53C3  	FCB    1		; NUM BYTES TO TX
;
L53C4  	FDB  L036E		; ADDR OF OUTPUT BUFFER
L53C6  	FDB  L036E		; ADDR OF INPUT BUFFER 
;----------------------------------------------
;	
;----------------------------------------------
L53C8  	FDB  L53CE		; 1, DEVICE ID $F5, MODE 1, MESSAGE 0, (66 bytes)
L53CA  	FDB  L545B		; 2, DEVICE ID $F5, MODE 1, MESSAGE 1, (30 bytes)
L53CC  	FDB  L54A0		; 3, DEVICE ID $F5, MODE 1. MESSAGE 3, (80 bytes) 
;----------------------------------------------
;
;----------------------------------------------
;  DEVICE ID $F5, MODE 1
;  PCM TYPE $0D 
;
; ALDL MODE 1, MESSAGE 0 DATA LIST
;	TRANSMISSION DATA STREAM INFORMATION MY 94/95 PCM-6 VEHICLES 
;    WITH A 4L60E TRANSMISSION.
;    MODE 1 (TRANSMIT FIXED DATA STREAM)
;            ALDL REQUEST:
;            - MESSAGE ID     = $F5
;            - MESSAGE LENGTH = $57
;            - MODE           = $01
;            - MESSAGE        = $00
;            - SUM CHECK
;
;            THE PCM WILL RESPOND WITH THE FOLLOWING MESSAGE:
;            - MESSAGE ID     = $F5
;            - MESSAGE LENGTH = $98
;            - MODE           = $01
;            - DATA BYTE 1
;              .
;              .
;            - DATA BYTE 66
;            - SUM CHECK
;
;
; (SCAN TOOL MODE)
;----------------------------------------------
;
L53CE  	FDB  $0000		; NEXT ENRTY ADDRESS
;
L53D0  	FCB  $F5		; DEVICE ID
;
L53D1  	FCB  $80 		; MEM FLAG, TBL FROM ROM
L53D2  	FCB   67		; 66 BYTES TO TX
					;
L53D3  	FDB  L036E		; ADDR OF OUTPUT BUFFER
L53D5  	FDB  L036E		; ADDR OF INPUT BUFFER
;
;-----------------------------
L53D7  	FDB  L000B		;  1, ERR MD WD 1                     	
				; b7 1 = ERR 13, o2 SENSOR     
				; b6 1 = ERR 14, COOL SENSOR, HIGH    
				; b5 1 = ERR 15, COOL SENSOR, LOW    
				; b4 1 = ERR 16, 2002 PPM VEH. SPEED SENSOR FAILURE      
				; b3 1 = ERR 17, not used, (RPM SIGNAL PROBLEM)
				; b2 1 = ERR 18, not used, (CAM CRANK ERROR)
				; b1 1 = ERR 19, not used    
				; b0 1 = ERR 21, TPS SENSOR HIGH    
;----------------------------    
L53D9  	FDB  L0016		;  2, CURRENT ERR MD 1                    		
				; b7 1 = ERR 13, o2 SENSOR     
				; b6 1 = ERR 14, COOL SENSOR, HIGH    
				; b5 1 = ERR 15, COOL SENSOR, LOW    
				; b4 1 = ERR 16, Vss BUFFER       
				; b3 1 = ERR 17, not used,  (RPM SIGNAL PROBLEM)     
				; b2 1 = ERR 18, not usedC, (CAM CRANK ERROR)
				; b1 1 = ERR 19, not used    
				; b0 1 = ERR 21, TPS SENSOR HIGH    
;----------------------------    
L53DB  	FDB  L000C		;  3, ERR WD 2                    		
				; b7 1 = ERR 22, LOW TPS     
				; b6 1 = ERR 23, LOW MAT     
				; b5 1 = ERR 24, LOW Vss    
				; b4 1 = ERR 25, MAT LOW     
				; b3 1 = ERR 26	 not used
				; b2 1 = ERR 27	 not used
				; b1 1 = ERR 28, PRESS SW MANIFOLD   
				; b0 1 = ERR 29, not used
;-----------------------------    
L53DD  	FDB  L0017		;  4,   CURRENT ERR WD 2
					;	
				; b7 1 = ERR 22, THROTTLE POSITION LOW     
				; b6 1 = ERR 23, MAT SENSOR LOW     
				; b5 1 = ERR 24, LOW Vss    
				; b4 1 = ERR 25, MAT LOW    
				; b3 1 = ERR 26, (not used)
				; b2 1 = ERR 27, (not used)
				; b1 1 = ERR 28, PRESS SW MANIFOLD   
				; b0 1 = ERR 29, (not used)
;-----------------------------    
L53DF  	FDB  L000D		;  5, ERR MD WD 3		                		
				; b7 1 = ERR 31, GOVERNOR    
				; b6 1 = ERR 32, EGR ERROR     
				; b5 1 = ERR 33, MAP SENSOR HI    
				; b4 1 = ERR 34, MAP SNENSOR LOW      
				; b3 1 = ERR 35, IAC ERROR    
				; b2 1 = ERR 36,  not used, (IAC THROTTLE KICKER FAIL)     
				; b1 1 = ERR 37, TCC BRAKE SW STUCK ON    
				; b0 1 = ERR 38, TCC BRAKE SW STUCK OFF    
;-----------------------------    
L53E1  	FDB  L0018		;  6,   CURRENT ERR WD 3	
				; b7 1 = ERR 31, GOVERNOR    
				; b6 1 = ERR 32, EGR ERROR     
				; b5 1 = ERR 33, MAP SENSOR HI    
				; b4 1 = ERR 34, MAP SENSOR LOW    
				; b3 1 = ERR 35, IAC ERROR    
				; b2 1 = ERR 36, not used, (IAC THROTTLE KICKER FAIL)
				; b1 1 = ERR 37, TCC BRAKE SW STUCK ON   
				; b0 1 = ERR 38, TCC BRAKE SW STUCK OFF  
;-----------------------------    
L53E3  	FDB  L000E		;  7, ERR MD WD 4		                 		
				; b7 1 = ERR 39, TCC STUCK OFF     
				; b6 1 = ERR 41, 1x CAM SENSOR     
				; b5 1 = ERR 42, IGN ERROR    
				; b4 1 = ERR 43, KNOCK SENSOR CKT     
				; b3 1 = ERR 44, OXYGEN SENSOR LEAN    
				; b2 1 = ERR 45, OXYGEN SENSOR RICH     
				; b1 1 = ERR 46, VATS FAIL    
				; b0 1 = ERR 47, not used    
;-----------------------------    
L53E5  	FDB  L0019		;  8,  CURRENT ERR WD #4	
				;  b7 1 = ERR 39, TCC STUCK OFF   
				;  b6 1 = ERR 41, 1x CAM SENSOR   
				;  b5 1 = ERR 42, IGN ERROR    
				;  b4 1 = ERR 43, KNOCK SENSOR CKT    
				;  b3 1 = ERR 44, OXYGEN SENSOR LEAN    
				;  b2 1 = ERR 45, OXYGEN SENSOR RICH     
				;  b1 1 = ERR 46, VATS FAIL    
				;  b0 1 = ERR 47, (not used)    
;-----------------------------    
L53E7  	FDB  L000F		; 9, ERR MD WD 5				        	
				; b7 1 = ERR 48. (not used)   
				; b6 1 = ERR 49, (not used)  	
				; b5 1 = ERR 51, EPROM ERROR    
				; b4 1 = ERR 52, SYSTEM VOLTAGE HIGH-LONG TEST    
				; b3 1 = ERR 53, HI SYS VOLTAGE      
				; b2 1 = ERR 54, FUEL PUMP RELAY MALFUNCTION    
				; b1 1 = ERR 55, FAULTY COMPUTER    
				; b0 1 = ERR 56, QUAD DRIVER B FAULT    
;-----------------------------    
L53E9  	FDB  L001A		; 10, CURRENT ERR WD 5	
				; b7 1 = ERR 48. (not used)   
				; b6 1 = ERR 49, (not used)	
				; b5 1 = ERR 51, EPROM ERROR    
				; b4 1 = ERR 52, MISSING FUEL CALPACK     
				; b3 1 = ERR 53, HI SYS VOLTAGE      
				; b2 1 = ERR 54, LOW FUEL PUMP VDC    
				; b1 1 = ERR 55, FAULTY COMPUTER    
				; b0 1 = ERR 56, not used, (QUAD DRIVER B FAULT)
;-----------------------------    
L53EB  	FDB  L0010		; 11, MALFFLG6, ERROR WD 6
				; b7 1 = ERR 57. (not used)
				; b6 1 = ERR 58, TRANSMISSION TEMPERATURE HI
				; b5 1 = ERR 59, TRANSMISSION TEMPERATURE LO
				; b4 1 = ERR 61, TURBO BOOST SENSOR HIGH
				; b3 1 = ERR 62, TURBO BOOST SENSOR LOW 
				; b2 1 = ERR 63, BAROMETRIC PRESSURE SENSOR HI
				; b1 1 = ERR 64, BAROMETRIC PRESSURE SENSOR	LO
				; b0 1 = ERR 65, not used
;-----------------------------
L53ED  	FDB  L001B		; 12,  CURMALF5, CURRENT ERROR WD 6 
				; b7 1 = ERR 57. (not used)
				; b6 1 = ERR 58, TRANSMISSION TEMPERATURE HI
				; b5 1 = ERR 59, TRANSMISSION TEMPERATURE LOW
				; b4 1 = ERR 61, TURBO BOOST SENSOR HI
				; b3 1 = ERR 62, TURBO BOOST SENSOR LOW
				; b2 1 = ERR 63, BAROMETRIC  SENSOR HI
				; b1 1 = ERR 64, BAROMETRIC  SENSOR LOW
				; b0 1 = ERR 65. (not used)
;-----------------------------    
L53EF  	FDB  L0011		; 13,  MALFFLG7, ERROR WD 7
				; b7 1 = ERR 66  3-2 DS QDM2/SOLENOID FAILURE
				; b6 1 = ERR 67  TCC EN QDM2/SOLENOID FAILURE
				; b5 1 = ERR 68  TRANSMISSION COMPONENT SLIPPING
				; b4 1 = ERR 69  TCC ON
				; b3 1 = ERR 71  ENGINE SPEED LOW
				; b2 1 = ERR 72  OUTPUT SPEED LOSS
				; b1 1 = ERR 73  FORCE MOTOR CURRENT
				; b0 1 = ERR 74  INPUT SPEED SENSOR
;-----------------------------
L53F1  	FDB  L001C		; 14, CURMALF7, CURRENT ERROR WD 7
				; b7 1 = ERR 66, 3-2 DS QDM2/SOLENOID FAIL
				; b6 1 = ERR 67, TCC EN QDM2/SOLENOID FAIL
				; b5 1 = ERR 68, XMISH COMPONENT SLIPPING
				; b4 1 = ERR 69, TCC ON
				; b3 1 = ERR 71, ENGINE SPEED LOW
				; b2 1 = ERR 72, OUTPUT SPEED LOSS
				; b1 1 = ERR 73, FORCE MOTOR CURRENT
				; b0 1 = ERR 74, INPUT SPEED SENSOR
;-----------------------------    
L53F3  	FDB  L0012		; 15, MALFFLG8, ERROR WD 8
				; b7 1 = ERR 75  SYSTEM VOLTAGE LOW
				; b6 1 = ERR 76  not used
				; b5 1 = ERR 77  MNP SWITCH
				; b4 1 = ERR 78  not used
				; b3 1 = ERR 71, ENGINE SPEED LOW
				; b2 1 = ERR 72, OUTPUT SPEED LOSS
				; b1 1 = ERR 73, FORCE MOTOR CURRENT
				; b0 1 = ERR 74, INPUT SPEED SENSOR
;-----------------------------
L53F5  	FDB  L001D		; 16, CURMALF8, CURRENT ERROR WD 8
				; b7 1 = ERR 75, SYSTEM VDC LOW
				; b6 1 = ERR 76, (not used)
				; b5 1 = ERR 77, NMP SWITCH
				; b4 1 = ERR 78, (not used) 
				; b3 1 = ERR 79, XMISH HOT
				; b2 1 = ERR 81, SHIFT SOLENOID B FAIL
				; b1 1 = ERR 82, SHIFT SOLENOID A FAIL
				; b0 1 = ERR 83, TCC SOLENOID FAIL
;-----------------------------
L53F7  	FDB  L0013		; 17,   MALFFLG9, CURRENT ERROR WD 9
				; b7 1 = ERR 84  not used
				; b6 1 = ERR 85  RATIO-UNDEFINED REGION
				; b5 1 = ERR 86  SOLENOID B STUCK ON
				; b4 1 = ERR 87  SOLENOID B STUCK OFF 
				; b3 1 = ERR 88	 not used
				; b2 1 = ERR 89  MAXIMUM ADAPT LONG SHIFT
				; b1 1 = ERR 91  not used
				; b0 1 = ERR 92  not used
;-----------------------------
L53F9  	FDB  L001E		; 18,  CURMALF9, CURRENT ERROR WD 9
				; b7 1 = ERR 84  not used
				; b6 1 = ERR 85  RATIO-UNDEFINED REGION
				; b5 1 = ERR 86  SOLENOID B STUCK ON
				; b4 1 = ERR 87  SOLENOID B STUCK OFF 
				; b3 1 = ERR 88	 not used
				; b2 1 = ERR 89  MAXIMUM ADAPT LONG SHIFT
				; b1 1 = ERR 91  not used
				; b0 1 = ERR 92  not used
;-----------------------------
L53FB  	FDB  L00A0 		; 19,  17, TPS A/D	
				; BIN = VDC * 50        
;-----------------------------    
L53FD  	FDB  L00A6		; 20, BARO, BAROMETRIC PRESSURE
				;   	   	VOLTS = N * (5/256)
				;   	   	(kpa = (N + 28.06)/2.71)
;-----------------------------
L53FF  	FDB  L319D		; 21,  NE,   ENGINE SPEED, (MSB)
L5401  	FDB  L319E		; 22,  NE+1, ENGINE SPEED, (LSB)
;		RPM/8
;-----------------------------
L5403  	FDB  L30BC		; 23, NT,   ENGINE SPEED, (MSB)
L5405  	FDB  L30BC+1		; 24, NT+1, ENGINE SPEED, (LSB)
				;		RPM/8
;-----------------------------
L5407  	FDB  L30C0		; 25, FILTERED TURBINE SPEED MSB
L5409  	FDB  L30C0+1		; 26, FILTERED TURBINE SPEED LSB
				;		RPM/8    
;-----------------------------
L540B  	FDB  L00D1		; 27, VEHSPEEN,  VEHICLE SPEED (FILTERED
				;		MPH/2
;-----------------------------
L540D  	FDB  L00DC		; 28, PRESSURE, CURRENT TQ SIG PRESS
				; 		PSI/1
;----------------------------
L540F  	FDB  L0147		; 29, CURRENT, REF CURRENT FORCE MTR CKT
				;		AMP = N/51.2
;----------------------------
L5411  	FDB  L0148		; 30, CURRENT, ACTUAL FM CURRENT FM SHUNT (A/D)
				;		AMP = N/51.2
;----------------------------
L5413  	FDB  L0146		; 31, FMDC, FORCE MOTOR D.C.
				; 		% = N/2.56
;----------------------------
L5415  	FDB  L009C		; 32, PRNDLFLG, CURRENT STATE OF XMISH RANGE SEL				
				; 		b7 1 = ILLEGAL RANGE
				; 		b6 1 = PK/NEUT
				; 		b5 1 = REVERSE
				; 		b4 1 = (ot used)
				; 
				; 		b3 1 = DRIVE 4
				; 		b2 1 = DRIVE 3
				; 		b1 1 = DRIVE 2
				; 		b0 1 = LOW GEAR
;-----------------------------
L5417  	FDB  L00A1		; 33, ADBAT, BATTERY VOLTS                   	    
				; 				VDC/10    
;-----------------------------
L5419  	FDB  L3124		; 34,  TCCDC,   TCC PWM SOLENOID DC, (MSB)
L541B  	FDB  L3124+1		; 35,  TCCDC+1, TCC PWM SOLENOID DC, (MSB)
				;	  % = (256MSB + LSB)/655.36
;-----------------------------
L541D  	FDB  L30F3		; 36,  RATIO,   XMISH (INPUT SPD/OUTPUT SPD) NI/NO, (MSB)
L541F  	FDB  L30F3+1		; 37,  RATIO+1, XMISH (INPUT SPD/OUTPUT SPD) NI/NO, (MSB)
				;				RATIO = N/16384
;-----------------------------
L5421  	FDB  L00D3		; 38, GEAR, CURRENT GEAR
				;		GEAR = N + 1
;----------------------------
L5423  	FDB  L0145		; 39, CURCELL, CURRENT ADPTIVE MOD'ER INDEX W/IN TBL
				; 		CELL = N
;----------------------------
L5425  	FDB  L0144		; 40,  CURADP, CURRENT ADAPTIVE MODIFIER 
				;		ADPT/1
;----------------------------
L5427  	FDB  L0082		; 41, TRSTATUS, 
				; b7 1 = POWERTRAIN IS IN MOTIION (SOME SPEED <> 0)
				; b6  	 ELCO INSTRUMENTATION PROM IS PRESEN
				; b5  	 (not used)
				; b4  	 NV RAM UPDATING IS ALLOWED
				; b3 1 = POWER IS ON (IGN1 > THRESH)
				; b2 1 = FORCE MOTOR PULSE IN PROGRESS
				; b1 1 = not usedT
				; b0 1 = FORCE MOTOR SHUT OFF DUE TO LOW BATTERY VOLTAGE
;----------------------------
L5429  	FDB  L012D		; 42, ADAPTBL, ADAPT WORD FOR SERVICE
				; b7 1 = TPS CHANGED TOO MUCH
				; b6 	 not used
				; b5 1 = LONG SHIFT DELAY TIME
				; b4 1 = VEHICLE SPEED CHANGED TOO MUCH 	 
				; b3 	 not used
				; b2 1 = TPS OUT OF RANGE 	 
				; b1 1 = THROTTLE OUT OF RANGE
				; b0 1 = LAST SHIFT WAS USABLE FOR ADAPTING  0 = UNUSABLE
;----------------------------
L542B  	FDB  L012C		; 43, TCCSTAT, TCC STATUS FOR SERVICE
				; b7 1 = SLIP CONTINGENCY FORCED TCC OFF 
				; b6 1 = A/C  CONTINGENCY FORCED TCC OFF  
				; b5 1 = TCC IS IN LOCK ADJ MODE	 
				; b4 1 = NEG DELTA TPS FORCED TCC OFF
				; b3 1 = BRAKE ON FORCED TCC OFF
				; b2  	 (not used) 
				; b1 1 = MIN TPS CONTINGENCY FORCED TCC OFF 
				; b0 1 = RANGE CONTINGENCY FORCED TCC OFF
;----------------------------
L542D  	FDB  L0135		; 44, ERROR12, ERR BETWEEN DESIRED & ACTUAL SHIFT TIMES 
				;			   FOR LATEST 1-2 UPSHIFT  1=1/32 SEC
				;			    SEC. = N/40
;-----------------------------
L542F  	FDB  L0136		; 45, ERR0R23, ERR BETWEEN DESIRED & ACTUAL SHIFT TIMES 
				;			   FOR LATEST 2 -> 3 1=1/32 SEC
				;					SEC = N/40
;-----------------------------
L5431  	FDB  L30E5		; 46, SLIP,   ABSOLUTE VAL OF SLIP (MSB)
L5433  	FDB  L30E5+1		; 47, SLIP+1, ABSOLUTE VAL OF SLIP (LSB)
				;	   TPM/8 (signed value)
;-----------------------------
L5435  	FDB  L0132		; 48, TIME12, TIME OF LATEST 1->2 SHIFT
				;		SEC = N/40
;-----------------------------
L5437  	FDB  L0133		; 49, TIME23, TIME OF LATEST 3->3 SHIFT
				;		SEC = N/40
				;
;-----------------------------
L5439  	FDB  L0083		; 50, TEMPMW       TEMPER STATUS WORD 
				; b7 1 = not used
				; b6 1 = not used
				; b5 1 = not used
				; b4 1 = not used   	 
				; b3 1 = not used   
				; b2 1 = not used
				; b1 1 = HOT  TEMPERATURE THRESHOLD
				; b0 1 = COLD TEMPERATURE THRESHOLD
;-----------------------------
L543B  	FDB  L0096		; 51, DIRFLAGS, BIT STATUS WORD
				; b7 1 = DIAGNOSTICS REQUESTED
				; b6 1 = CRUISE LOW ACTIVE
				; b5 1 = 4 WD LOW ACTIVE
				; b4 1 = KICK DN PATTERN ACTIVE  	 
				; b3     (not used)
				; b2 1 = PWR ENRICH ACTIVE
				; b1 1 = A/C COMPRESSOR ENGAGED
				; b0 1 = BRAKE ON
;-----------------------------
L543D  	FDB  L004B		; 52, IODPORTA,  I/O D PORT A
				; b7 1 = TCC ENABLE			0=OFF  1=ON	  
				; b6 1 = BYPASS (FROM REST)		  
				; b5 1 = BYPASS MONTR (FROM REST) 
				; b4 1 = BYPASS CTRL (FROM REST)  
				; b3 1 = 3->2 DN SHIFT FEEDFBACK 	0=ON    1=OFF
				; b2 1 = TCC R\ENABLE FEEDFBACK		0=ON    1=OFF
				; b1 1 = SHIFT B FEEDFBACK		0=ON    1=OFF
				; b0 1 = SHIFT A FEEDFBACK		0=ON    1=OFF
;---------------------------------
L543F  	FDB  L004C		; 53, IODPORTB,  I/O D PORT B
				;	  SOLENOID COMBINATION FOR DIGITAL EGR
				; b7 1 = M/CLR			SFTB   SFTA  GR
				; b6 1 = REF IRQ CLR    	  0     0    3rd
				; b5 1 = FORCE MOTOR ENABLE	  0     1    4th
				; b4 1 = IAC ENABLE   		  1     0    2nd 							  1     1    1st
				; b3 1 = IAC COIL B ON   
				; b2 1 = IAC COIL A ON  
				; b1 1 = 2 - 3 SOL ON,	SHIFT B
				; b0 1 = 1 - 2 SOL ON,	SHIFT A
;---------------------------------
L5441  	FDB  L004D		; 54, IODPORTC, I/O PORT C  (L3064)
				; b7 1 = FWD LOW SW (NO) 1=ON
				; b6     (not used)
				; b5     (not used) 		  R1  R2  R3  GR
				; b4 1 = RANGE SW 3 OFF 	  0   1   1   LOW						  1   1   1   DR2
				; b3 1 = RANGE SW 2 OFF 	  1   1   0   DR3
				; b2 1 = RANGE SW 1 OFF 	  1   0   0   DR4
				; b1 1 = BK SW ON    		  1   0   1   P/N
				; b0 1 = A/C REQUEST ON 	  0   0   1   REV
;---------------------------------
L5443  	FDB  L4000		; 55, FIRST  PROM I.D. WORD (MSB)
L5445  	FDB  L4000+1		; 56, SECOND PROM I.D. WORD (LSB)
				;	   ID = MSB*256 + LSB
;-----------------------------
L5447  	FDB  L012E		; 57,DSSDC  3-2 PWM SOL % DUTY CYCLE
				;  	% = N / 2.56
;-----------------------------
L5449  	FDB  L0098		; 58, FAULTFLG     DISCRETE FAULT FLAGS
				;	b7  not used
				;	b6  not used
				;	b5  1 = SHIFT SOLENOID B VALID
				;	b4  1 = SHIFT SOLENOID A VALID
				;	b3  1 = INJECTOR 2 FAULT DETECTED
				;	b2  1 = INJECTOR 1 FAULT DETECTED
				;	b1  1 = QDM 2 FAULT DETECTED
       	   			;	b0  1 = QDM 1 FAULT DETECTED
;---------------------------------
L544B  	FDB  L0081		; 59, SHIFTED      SHIFT WORD
				;	b7 1 = SHIFT IS COMPLETE
				;	b6 not used
				;	b5 1 = 2-4 JUMP SHIFT WHEN PRNDL = D3
				;	b4 1 = SHIFT HAS STARTED
				;	b3 not used
				;	b2 not used
				;	b1 1 = DERIVATIVE SHIFT STARTED
				;	b0 1 = SLOPE OF DERIVATIVE IS NEGATIVE
;-----------------------------    
L544D  	FDB  L0099		; 60,  PATTERN,  MNP PATTERN
				;	b7   not used
				;	b6   not used
				;	b5   not used
				;	b4   not used
				;	b3   1 = ILLEGAL PATTERN REQUESTED
				;	b2   1 = "MANUAL" PATTERN REQUESTED
				;	b1   1 = "PERFORMANCE" PATTERN REQUESTED
				;	b0   1 = "NORMAL" PATTERN REQUESTED
;-----------------------------    
L544F  	FDB  L02A7		; 61,  FILTMPH,  MPH	 0 - 255 MPH                   	      
				;		MPH/1
				;		KPH = 1.61  
;-----------------------------    
L5451  	FDB  L30C6		; 62, RAW OUTPUT SPEED(MSB)
L5453  	FDB  L30C6+1		; 63, RAW OUTPUT SPEED(LSB)
				;	  	RPM = N/8
;-----------------------------
L5455  	FDB  L0197		; 64, NDTHROPS, TPS FOR ENGINE
				; 	   (CURRENT NON-DEAULT VALUE)   
				;	   % * 2.56
;-----------------------------    
L5457  	FDB  L00A2		; 65, COOLDEGA, COOL TEMP
				;   		Deg  c = n*.75 - 40
				;   		(deg F = 1.35N - 40)
;-----------------------------    
L5459  	FDB  L00AE		; 66, TRANSDEGA, NORMALIZED TRANSMISSION FLUID TEMPERATURE
				;		NON-DEFAULTED
				;   DEGREES C = N*.75 - 40                  
				;   (deg F = 1.35N - 40)
				;
;-------------------------------------------------
;
;-------------------------------------------------
;  DEVICE ID $F5
;  MODE 1, MESSAGE 1
;
;    MODE 1 (TRANSMIT FIXED DATA STREAM)
;            ALDL REQUEST:
;            - MESSAGE ID     = $F5
;            - MESSAGE LENGTH = $57
;            - MODE           = $01
;            - MESSAGE        = $01
;            - SUM CHECK
;
;            THE PCM WILL RESPOND WITH THE FOLLOWING MESSAGE:
;            - MESSAGE ID     = $F5
;            - MESSAGE LENGTH = $74
;            - MODE           = $01
;            - DATA BYTE 1
;              .
;              .
;            - DATA BYTE 30
;            - SUM CHECK
;
;-------------------------------------------------
;
L545B  	FDB  $0000		; NEXT ENRTY ADDRESS
;
L545D  	FCB  $F5		; DEVICE ID
;
L545E  	FCB  $80		; MEM FLAG,TBL FROM ROM
;
L545F  	FCB   31		; 30 BYTES TO TX
;
L5460  	FDB  L036E 		; ADDR OF OUTPUT BUFFER
L5462  	FDB  L036E		; ADDR OF INPUT BUFFER
;
L5464  	FDB  L000B		;  1, ERR MD WD 1                     
				; b7 1 = ERR 13, o2 SENSOR     
				; b6 1 = ERR 14, COOL SENSOR, HIGH    
				; b5 1 = ERR 15, COOL SENSOR, LOW    
				; b4 1 = ERR 16, Vss BUFFER     
				; b3 1 = ERR 17, RPM SIGNAL PROBLEM     
				; b2 1 = ERR 18, CAM CRANK ERROR     
				; b1 1 = ERR 19,     
				; b0 1 = ERR 21, TPS SENSOR HIGH    
;----------------------------    
L5466  	FDB  L0016		;  2, CURRENT ERR MD 1                    		
				; b7 1 = ERR 13, o2 SENSOR     
				; b6 1 = ERR 14, COOL SENSOR, HIGH    
				; b5 1 = ERR 15, COOL SENSOR, LOW    
				; b4 1 = ERR 16, Vss BUFFER      
				; b3 1 = ERR 17, not used,  (RPM SIGNAL PROBLEM)     
				; b2 1 = ERR 18, not usedC, (CAM CRANK ERROR)
				; b1 1 = ERR 19, not used    
				; b0 1 = ERR 21, TPS SENSOR HIGH    
;----------------------------    
L5468  	FDB  L000C		;  3, ERR MD WD 2                    		
				; b7 1 = ERR 22, LOW TPS     
				; b6 1 = ERR 23, LOW MAT     
 				; b5 1 = ERR 24, LOW Vss    
				; b4 1 = ERR 25, MAT LOW        
				; b3 1 = ERR 26	 not used
				; b2 1 = ERR 27	 not used
				; b1 1 = ERR 28, PRESS SW MANIFOLD   
				; b0 1 = ERR 29	 not used
;-----------------------------    
L546A  	FDB  L0017		;  4,   CURRENT ERR WD 2
				; b7 1 = ERR 22, LOW TPS     
				; b6 1 = ERR 23, LOW MAT     
				; b5 1 = ERR 24, LOW Vss    
				; b4 1 = ERR 25, MAT LOW     
				; b3 1 = ERR 26	 not used
				; b2 1 = ERR 27	 not used
				; b1 1 = ERR 28, PRESS SW MANIFOLD   
				; b0 1 = ERR 29	 not used
;-----------------------------    
L546C  	FDB  L000D		;  5, ERR MD WD 3		                		
				; b7 1 = ERR 31, GOVENOR    
				; b6 1 = ERR 32, EGR ERROR     
				; b5 1 = ERR 33, MAP SENSOR HI    
				; b4 1 = ERR 34, MAP SENSOR LOW    
				; b3 1 = ERR 35, IAC ERROR    
				; b2 1 = ERR 36, IAC THROTTLE KICKER FAIL     
				; b1 1 = ERR 37, TCC BRAKE SW STUCK ON    
				; b0 1 = ERR 38, TCC BRAKE SW STUCK OFF    
;-----------------------------    
L546E  	FDB  L0018		;  6,   CURRENT ERR WD 3	
				; b7 1 = ERR 31, GOVENOR    
				; b6 1 = ERR 32, EGR ERROR     
				; b5 1 = ERR 33, MAP SENSOR HI    
				; b4 1 = ERR 34, MAP SNENSOR LOW      
				; b3 1 = ERR 35, IAC ERROR    
				; b2 1 = ERR 36, IAC THROTTLE KICKER FAIL
				; b1 1 = ERR 37, TCC BRAKE SW STUCK ON   
				; b0 1 = ERR 38, TCC BRAKE SW STUCK OFF  
;-----------------------------    
L5470  	FDB  L000E		; 7, ERR MD WD 4		                 		
				; b7 1 = ERR 39, TCC STUCK OFF     
				; b6 1 = ERR 41, 1x CAM SENSOR     
				; b5 1 = ERR 42, IGN ERROR    
				; b4 1 = ERR 43, KNOCK SENSOR CKT     
				; b3 1 = ERR 44, LEAN    
				; b2 1 = ERR 45, RICH     
				; b1 1 = ERR 46, VATS FAIL    
				; b0 1 = ERR 47, not used    
;-----------------------------    
L5472  	FDB  L0019		;  8,  CURRENT ERR WD #4	
				;  b7 1 = ERR 39, TCC STUCK OFF   
				;  b6 1 = ERR 41, 1x CAM SENSOR   
				;  b5 1 = ERR 42, IGN ERROR    
				;  b4 1 = ERR 43, KNOCK SENSOR CKT   
				;  b3 1 = ERR 44, LEAN    
				;  b2 1 = ERR 45, RICH     
				;  b1 1 = ERR 46, VATS FAIL    
				;  b0 1 = ERR 47, not used    
;-----------------------------    
L5474  	FDB  L000F		; 9, ERR MD WD 5				        		
				; b7     ERR 48  not used
				; b6  	 ERR 49  not used
				; b5 1 = ERR 51, EPROM ERROR    
				; b4 1 = ERR 52, SYSTEM VOLTAGE HIGH-LONG TEST        
				; b3 1 = ERR 53, HI SYS VOLTAGE      
				; b2 1 = ERR 54, LOW FUEL PUMP VDV    
				; b1 1 = ERR 55, FAULTY COMPUTER    
				; b0 1 = ERR 56, not used    
;-----------------------------    
L5476  	FDB  L001A		; 10, CURRENT ERR WD 5	
				; b7     ERR 48  not used
				; b6  	 ERR 49  not used
				; b5 1 = ERR 51, EPROM ERROR    
				; b4 1 = ERR 52, SYSTEM VOLTAGE HIGH-LONG TEST       
				; b3 1 = ERR 53, HI SYS VOLTAGE      
				; b2 1 = ERR 54, LOW FUEL PUMP VDV    
				; b1 1 = ERR 55, FAULTY COMPUTER    
				; b0 1 = ERR 56, not used    
;-----------------------------    
L5478  	FDB  L0010		; 11,	  MALFUNCTION WORD 6
				; b7 1 = ERR 57. (not used)
				; b6 1 = ERR 58, XMISH TEMP HI
				; b5 1 = ERR 59, XMISH TEMP LOW
				; b4 1 = ERR 61, TURBO BOOST SENSOR HI
				; b3 1 = ERR 62, TURBO BOOST SENSOR LOW
				; b2 1 = ERR 63, BARO SENSOR HI
				; b1 1 = ERR 64, BARO SENSOR LOW
				; b0 1 = ERR 65. (not used)
;-----------------------------
L547A  	FDB  L001B		; 12,  CURRENT ERR WD 6, ERROR WD 6
				; b7 1 = ERR 57. (not used)
				; b6 1 = ERR 58, XMISH TEMP HI
				; b5 1 = ERR 59, XMISH TEMP LOW
				; b4 1 = ERR 61, TURBO BOOST SENSOR HI
				; b3 1 = ERR 62, TURBO BOOST SENSOR LOW
				; b2 1 = ERR 63, BARO SENSOR HI
				; b1 1 = ERR 64, BARO SENSOR LOW
				; b0 1 = ERR 65. (not used)
;-----------------------------    
L547C  	FDB  L0011		; 13, MALFUNCTION WORD 7
				;	b7 1 = ERR 66  3-2 DS QDM2/SOLENOID FAILURE
				;	b6 1 = ERR 67  TCC EN QDM2/SOLENOID FAILURE
				;	b5 1 = ERR 68  TRANSMISSION COMPONENT SLIPPING
				;	b4 1 = ERR 69  TCC ON
				;	b3 1 = ERR 71  ENGINE SPEED LOW
				;	b2 1 = ERR 72  OUTPUT SPEED LOSS
				;	b1 1 = ERR 73  FORCE MOTOR CURRENT
				;	b0 1 = ERR 74  INPUT SPEED SENSOR
;-----------------------------
L547E  	FDB  L001C		; 14, CURRENT MALFUNCTION WORD 7
				;	b7 1 = ERR 66  3-2 DS QDM2/SOLENOID FAILURE
				;	b6 1 = ERR 67  TCC EN QDM2/SOLENOID FAILURE
				;	b5 1 = ERR 68  TRANSMISSION COMPONENT SLIPPING
				;	b4 1 = ERR 69  TCC ON
				;	b3 1 = ERR 71  ENGINE SPEED LOW
				;	b2 1 = ERR 72  OUTPUT SPEED LOSS
				;	b1 1 = ERR 73  FORCE MOTOR CURRENT
				;	b0 1 = ERR 74  INPUT SPEED SENSOR
;-----------------------------
L5480  	FDB  L0012		; 15, MALFUNCTION WORD 8
				;	b7 1 = ERR 75  SYSTEM VOLTAGE LOW
				;	b6 1 = ERR 76  not used
				;	b5 1 = ERR 77  MNP SWITCH
				;	b4 1 = ERR 78  not used
				;	b3 1 = ERR 79  TRANSMISSION HOT
				;	b2 1 = ERR 81  SHIFT SOLENOID B FAILURE
				;	b1 1 = ERR 82  SHIFT SOLENOID A FAILURE
				;	b0 1 = ERR 83  TCC SOLENOID FAILURE
;----------------------------
L5482  	FDB  L001D		; 16, CURRENT MALFUNCTION WORD 8
				;	b7 1 = ERR 75  SYSTEM VOLTAGE LOW
				;	b6 1 = ERR 76  not used
				;	b5 1 = ERR 77  MNP SWITCH
				;	b4 1 = ERR 78  not used
				;	b3 1 = ERR 79  TRANSMISSION HOT
				;	b2 1 = ERR 81  SHIFT SOLENOID B FAILURE
				;	b1 1 = ERR 82  SHIFT SOLENOID A FAILURE
				;	b0 1 = ERR 83  TCC SOLENOID FAILURE
;-----------------------------    
L5484  	FDB  L0013		; 17, MALFUNCTkON WORD 9
				;	b7 1 = ERR 84  not used
				;	b6 1 = ERR 85  RATIO-UNDEFINED REGION
				;	b5 1 = ERR 86  SOLENOID B STUCK ON
				;	b4 1 = ERR 87  SOLENOID B STUCK OFF
				;	b3 1 = ERR 88  not used
				;	b2 1 = ERR 89  MAXIMUM ADAPT LONG SHIFT
				;	b0 1 = ERR 92  not used
				;	b1 1 = ERR 91  not used
;-----------------------------
L5486  	FDB  L001E		; 18,  CURRENT ERR WD 9
				;	b7 1 = ERR 84  not used
				;	b6 1 = ERR 85  RATIO-UNDEFINED REGION
				;	b5 1 = ERR 86  SOLENOID B STUCK ON
				;	b4 1 = ERR 87  SOLENOID B STUCK OFF
				;	b3 1 = ERR 88  not used
				;	b2 1 = ERR 89  MAXIMUM ADAPT LONG SHIFT
				;	b0 1 = ERR 92  not used
				;	b1 1 = ERR 91  not used
;-----------------------------    
L5488  	FDB  L012D		; 19, ADAPTABL   ADAPT WORD FOR SERFICE
				;	b0  1 = LAST SHIFT WAS USABLE FOR ADAPTING  0 = UNUSABLE
				;	b1  1 = HOT TEMPERATURE THRESHOLD
				;	b2  1 = THROTTLE OUT OF RANGE
				;	b3  not used
       	   			;
				;	b4  1 = VEHICLE SPEED CHANGED TOO MUCH
				;	b5  1 = LONG SHIFT DELAY TIME
				;	b6  not used
				;	b7  1 = THROTTLE CHANGED TOO MUCH
;-----------------------------    
L548A  	FDB  L00DB		; 20,LSTADEL    LAST CHANGE TO ADAPTIVE MODIFIER
       	   			;				PSI = N/4
;-----------------------------    
L548C  	FDB  L0144		; 21, CURADPT   CURRENT ADAPTIVE MODIFIER
       	   			;				N = STATUS
;-----------------------------    
L548E  	FDB  L0145		; 22,CURCELL    CURRENT ADAPTIVE MODIFIER INDEX WITHIN TABLE
       	   			;				N = CELL
;-----------------------------    
L5490  	FDB  L031B		; 23, EADPT2    PRESSURE MODIFIER (ADAPTIVE) - 2nd GEAR
				;				PSI = N/4
;-----------------------------  
L5492  	FDB  L031C		; 24, EADPT2+1  PRESSURE MODIFIER (ADAPTIVE) - 2nd GEAR
				;				PSI = N/4
;-----------------------------  
L5494  	FDB  L031D		; 25, EADPT2+2  PRESSURE MODIFIER (ADAPTIVE) - 2nd GEAR
				;				PSI = N/4
;-----------------------------  
L5496  	FDB  L031E		; 26, EADPT2+3  PRESSURE MODIFIER (ADAPTIVE) - 2nd GEAR
				;				PSI = N/4
;--------------------------------------
L5498  	FDB  L031F		; 27, EADPT3,   PRESSURE MODIFIER (ADAPTIVE) - 3rd GEAR
				;				PSI = N/4
;-----------------------------  
L549A  	FDB  $0320		; 28, EADPT3+1, PRESSURE MODIFIER (ADAPTIVE) - 3rd GEAR
				;				PSI = N/4
;-----------------------------  
L549C  	FDB  L0321		; 29, EADPT3+2 	PRESSURE MODIFIER (ADAPTIVE) - 3rd GEAR
				;				PSI = N/4
;-----------------------------  
L549E  	FDB  L0322		; 30, EADPT3+3 	PRESSURE MODIFIER (ADAPTIVE) - 3rd GEAR
				;				PSI = N/4
;				
;---------------------------------------------
;  MSG ID $F5,
;  MODE 1, MESSAGE 3					
;
;
;---------------------------------------------
;
L54A0  	FDB  $0000		; NEXT ENRTY ADDRESS
;
L54A2  	FCB  $F5		; DEVICE ID
L54A3  	FCB  $80		; MEM FLAG, TBL FROM ROM
L54A4  	FCB   81		; 80 BYTES TO TX
;
L54A5  	FDB  L036E		; ADDR OF OUTPUT BUFFER
L54A7  	FDB  L036E		; ADDR OF INPUT BUFFER
;-----------------------------
L54A9  	FDB  L000B		;  1, ERR MD WD 1                     	
				; b7 1 = ERR 13, o2 SENSOR     
				; b6 1 = ERR 14, COOL SENSOR, HIGH    
				; b5 1 = ERR 15, COOL SENSOR, LOW    
				; b4 1 = ERR 16, Vss BUFFER     
				; b3 1 = ERR 17, RPM SIGNAL PROBLEM     
				; b2 1 = ERR 18, CAM CRANK ERROR     
				; b1 1 = ERR 19,     
				; b0 1 = ERR 21, TPS SENSOR HIGH    
;----------------------------    
L54AB  	FDB  L0016		;  2, CURRENT ERR MD 1                    		
				; b7 1 = ERR 13, o2 SENSOR     
				; b6 1 = ERR 14, COOL SENSOR, HIGH    
				; b5 1 = ERR 15, COOL SENSOR, LOW    
				; b4 1 = ERR 16, Vss BUFFER      
				; b3 1 = ERR 17, not used,  (RPM SIGNAL PROBLEM)     
				; b2 1 = ERR 18, not usedC, (CAM CRANK ERROR)
				; b1 1 = ERR 19, not used    
				; b0 1 = ERR 21, TPS SENSOR HIGH    
;----------------------------    
L54AD  FDB  L000C		;  3, ERR MD WD 2                    		
				; b7 1 = ERR 22, LOW TPS     
				; b6 1 = ERR 23, LOW MAT     
				; b5 1 = ERR 24, LOW Vss    
				; b4 1 = ERR 25, MAT LOW      
				; b3  1 = ERR 26
				; b2  1 = ERR 27
				; b1  1 = ERR 28, PRESS SW MANIFOLD   
				; b0  1 = ERR 29
;-----------------------------    
L54AF  FDB  L0017		;  4,   CURRENT ERR WD 2
				; b7 1 = ERR 22, LOW TPS     
				; b6 1 = ERR 23, LOW MAT     
				; b5 1 = ERR 24, LOW Vss    
				; b4 1 = ERR 25, MAT LOW       
				; b3 1 = ERR 26
				; b2 1 = ERR 27
				; b1 1 = ERR 28, PRESS SW MANIFOLD   
				; b0 1 = ERR 29
;-----------------------------    
L54B1  FDB  L000D		;  5, ERR MD WD 3		                		
				; b7 1 = ERR 31, GOVENOR    
				; b6 1 = ERR 32, EGR ERROR     
				; b5 1 = ERR 33, MAP SENSOR HI    
				; b4 1 = ERR 34, MAP SNENSOR LOW      
				; b3 1 = ERR 35, IAC ERROR    
				; b2 1 = ERR 36, IAC THROTTLE KICKER FAIL     
				; b1 1 = ERR 37, TCC BRAKE SW STUCK ON    
				; b0 1 = ERR 38, TCC BRAKE SW STUCK OFF  
;-----------------------------    
L54B3  FDB  L0018		;  6,   CURRENT ERR WD 3	
				; b7 1 = ERR 31, GOVENOR    
				; b6 1 = ERR 32, EGR ERROR     
				; b5 1 = ERR 33, MAP SENSOR HI    
				; b4 1 = ERR 34, MAP SNENSOR LOW     
				; b3 1 = ERR 35, IAC ERROR    
				; b2 1 = ERR 36, IAC THROTTLE KICKER FAIL
				; b1 1 = ERR 37, TCC BRAKE SW STUCK ON   
				; b0 1 = ERR 38, TCC BRAKE SW STUCK OFF 
;-----------------------------    
L54B5  FDB  L000E		;  7, ERR MD WD 4		                 		
				; b7 1 = ERR 39, TCC STUCK OFF     
				; b6 1 = ERR 41, 1x CAM SENSOR     
				; b5 1 = ERR 42, IGN ERROR    
				; b4 1 = ERR 43, KNOCK SENSOR CKT     
				; b3 1 = ERR 44, LEAN    
				; b2 1 = ERR 45, RICH     
				; b1 1 = ERR 46, VATS FAIL    
				; b0 1 = ERR 47,     
;-----------------------------    
L54B7  FDB  L0019		;  8,  CURRENT ERR WD #4	
				;  b7 1 = ERR 39, TCC STUCK OFF   
				;  b6 1 = ERR 41, 1x CAM SENSOR   
				;  b5 1 = ERR 42, IGN ERROR    
				;  b4 1 = ERR 43, KNOCK SENSOR CKT
				;  b3 1 = ERR 44, LEAN    
				;  b2 1 = ERR 45, RICH     
				;  b1 1 = ERR 46, VATS FAIL    
				;  b0 1 = ERR 47,     
;-----------------------------    
L54B9  FDB  L000F		; 9, ERR MD WD 5				        		
				; b7     
				; b6  	
				; b5 1 = ERR 51, EPROM ERROR    
				; b4 1 = ERR 52, MISSING FUEL CALPACK    
				; b3 1 = ERR 53, HI SYS VOLTAGE      
				; b2 1 = ERR 54, LOW FUEL PUMP VDV    
				; b1 1 = ERR 55, FAULTY COMPUTER    
				; b0 1 = ERR 56, QUAD DRIVER B FAULT  
;-----------------------------    
L54BB  FDB  L001A		; 10, CURRENT ERR WD 5	
				; b7    (not used)   
				; b6    (not used)	
				; b5 1 = ERR 51, EPROM ERROR    
				; b4 1 = ERR 52, MISSING FUEL CALPACK         
				; b3 1 = ERR 53, HI SYS VOLTAGE      
				; b2 1 = ERR 54, LOW FUEL PUMP VDC    
				; b1 1 = ERR 55, FAULTY COMPUTER    
				; b0 1 = ERR 56, QUAD DRIVER B FAULT  
;-----------------------------    
L54BD  FDB  L0010		; 11, MALFFLG5, ERROR WD 6
				; b7 1 = ERR 57, (not used)
				; b6 1 = ERR 58, XMISH TEMP HI
				; b5 1 = ERR 59, XMISH TEMP LOW
				; b4 1 = ERR 61, TURBO BOOST SENSOR HI
				; b3 1 = ERR 62, TURBO BOOST SENSOR LOW
				; b2 1 = ERR 63, BARO SENSOR HI
				; b1 1 = ERR 64, BARO SENSOR LOW
				; b0 1 = ERR 65. (not used)
;-----------------------------    
L54BF  FDB  L001B		; 12, CURRENT ERR WD 6
				; b7 1 = ERR 57. (not used)
				; b6 1 = ERR 58, XMISH TEMP HI
				; b5 1 = ERR 59, XMISH TEMP LOW
				; b4 1 = ERR 61, TURBO BOOST SENSOR HI
				; b3 1 = ERR 62, TURBO BOOST SENSOR LOW
				; b2 1 = ERR 63, BARO SENSOR HI
				; b1 1 = ERR 64, BARO SENSOR LOW
				; b0 1 = ERR 65. (not used)
;-----------------------------    
L54C1  FDB  L0011		; 13,  MALFFLG7, ERROR WD 7
				; b7 1 = ERR 66  3-2 DS QDM2/SOLENOID FAILURE
				; b6 1 = ERR 67  TCC EN QDM2/SOLENOID FAILURE
				; b5 1 = ERR 68  TRANSMISSION COMPONENT SLIPPING
				; b4 1 = ERR 69  TCC ON
				; b3 1 = ERR 71  ENGINE SPEED LOW
 				; b2 1 = ERR 72  OUTPUT SPEED LOSS
				; b1 1 = ERR 73  FORCE MOTOR CURRENT
				; b0 1 = ERR 74  INPUT SPEED SENSOR
;-----------------------------
L54C3  FDB  L001C		; 14, CURMALF7, CURRENT ERROR WD 7
				; b7 1 = ERR 66, 3-2 DS QDM2/SOLENOID FAIL
				; b6 1 = ERR 67, TCC EN QDM2/SOLENOID FAIL
				; b5 1 = ERR 68, XMISH COMPONENT SLIPPING
				; b4 1 = ERR 69, TCC ON
				; b3 1 = ERR 71, ENGINE SPEED LOW
				; b2 1 = ERR 72, OUTPUT SPEED LOSS
				; b1 1 = ERR 73, FORCE MOTOR CURRENT
				; b0 1 = ERR 74, INPUT SPEED SENSOR
;-----------------------------
L54C5  FDB  L0012		; 15, MALFFLG8, ERROR WD 8
				; b7 1 = ERR 75  SYSTEM VOLTAGE LOWL
				; b6 1 = ERR 76  not used
				; b5 1 = ERR 77  MNP SWITCH
				; b4 1 = ERR 78  not used
				; b3 1 = ERR 71, ENGINE SPEED LOW
				; b2 1 = ERR 72, OUTPUT SPEED LOSS
				; b1 1 = ERR 73, FORCE MOTOR CURRENT
				; b0 1 = ERR 74, INPUT SPEED SENSOR
;-----------------------------
L54C7  FDB  L001D		; 16, CURMALF8, CURRENT ERROR WD 8
				; b7 1 = ERR 75, SYSTEM VDC LOW
				; b6 1 = ERR 76, (not used)
				; b5 1 = ERR 77, NMP SWITCH
				; b4 1 = ERR 78, (not used) 
				; b3 1 = ERR 79, XMISH HOT
				; b2 1 = ERR 81, SHIFT SOLENOID B FAIL
				; b1 1 = ERR 82, SHIFT SOLENOID A FAIL
				; b0 1 = ERR 83, TCC SOLENOID FAIL
;-----------------------------    
L54C9  FDB  L0013		; 17,  MALFFLG9, CURRENT ERROR WD 9
				; b7 1 = ERR 84  not used
				; b6 1 = ERR 85  RATIO-UNDEFINED REGION
				; b5 1 = ERR 86  SOLENOID B STUCK ON
				; b4 1 = ERR 87  SOLENOID B STUCK OFF 
				; b3 1 = ERR 88	 not used
				; b2 1 = ERR 89  MAXIMUM ADAPT LONG SHIFT
				; b1 1 = ERR 91  not used
				; b0 1 = ERR 92  not used
;-----------------------------
L54CB  FDB  L001E		; 18,   CURMALF9, CURRENT ERROR WD 9
				; b7 1 = ERR 84  not used
				; b6 1 = ERR 85  RATIO-UNDEFINED REGION
				; b5 1 = ERR 86  SOLENOID B STUCK ON
				; b4 1 = ERR 87  SOLENOID B STUCK OFF 
				; b3 1 = ERR 88	 not used
				; b2 1 = ERR 89  MAXIMUM ADAPT LONG SHIFT
				; b1 1 = ERR 91  not used
				; b0 1 = ERR 92  not used
;-----------------------------
L54CD  FDB  L00A0		; 19,  TPS A/D	
				; BIN = VDC * 50        
;-----------------------------    
L54CF  FDB  L00A6		; 20, BARO, BAROMETRIC PRESSURE
				;   	   	VOLTS = N * (5/256)
				;   	   	(kpa = (N + 28.06)/2.71)
;-----------------------------
L54D1  FDB  L319D		; 21,  NE,   ENGINE SPEED, (MSB)
L54D3  FDB  L319D+1		; 22,  NE+1, ENGINE SPEED, (LSB)
				;	 	RPM/8
;------------------------------
L54D5  FDB  L30BC		; 23, NT,   ENGINE SPEED, (MSB)
L54D7  FDB  L30BC+1		; 24, NT+1, ENGINE SPEED, (LSB)
				;	 	RPM/8
;------------------------------
L54D9  FDB  L30C0		; 25, FILTERED TURBINE SPEED MSB
L54DB  FDB  L30C0+1		; 26, FILTERED TURBINE SPEED LSB
				;		  RPM/8    
;------------------------------
L54DD  FDB  L00D1		; 27, VEHSPEEN,  VEHICLE SPEED (FILTERED
				;		MPH/2
;-----------------------------
L54DF  FDB  L00DC		; 28, PRESSURE, CURRENT TQ SIG PRESS
				; 		PSI/1
;----------------------------
L54E1  FDB  L0147		; 29, CURRENT, REF CURRENT FORCE MTR CKT
				;		AMP = N/51.2
;----------------------------
L54E3  FDB  L0148		; 30, CURRENT, ACTUAL FM CURRENT FM SHUNT (A/D)
				;		AMP = N/51.2
;----------------------------
L54E5  FDB  L0146		; 31, FMDC, FORCE MOTOR D.C.
				; 		% = N/2.56
;----------------------------
L54E7  FDB  L009C		; 32, PRNDLFLG, CURRENT STATE OF XMISH RANGE SEL
				; 		b7 1 = ILLEGAL RANGE
				; 		b6 1 = PK/NEUT
				; 		b5 1 = REVERSE
				; 		b4 1 = (ot used)
				; 		b3 1 = DRIVE 4
				; 		b2 1 = DRIVE 3
				; 		b1 1 = DRIVE 2
				; 		b0 1 = LOW GEAR
;-----------------------------
L54E9  FDB  L00A1		; 33, BATTERY A/D                   	    
				; 		VDC/10    
;-----------------------------
L54EB  FDB  L3124		; 34, TCCDC,   TCC PWM SOLENOID DC, (MSB)
L54ED  FDB  L3124+1		; 35, TCCDC+1, TCC PWM SOLENOID DC, (MSB)
				;	   % = (256MSB + LSB)/655.36
;-----------------------------
L54EF  FDB  L30F3		; 36,  RATIO,   XMISH (INPUT SPD/OUTPUT SPD) NI/NO, (MSB)
L54F1  FDB  L30F3+1		; 37,  RATIO+1, XMISH (INPUT SPD/OUTPUT SPD) NI/NO, (MSB)
				;			RATIO = N/16384
;-----------------------------
L54F3  FDB  L00D3		; 38, GEAR, CURRENT GEAR
				;		GEAR = N + 1
;----------------------------
L54F5  FDB  L00D4		; 39,
;----------------------------
L54F7  FDB  L00DB		; 40,LSTADEL    LAST CHANGE TO ADAPTIVE MODIFIER
          			;				PSI = N/4
;-----------------------------    
L54F9  FDB  L0145		; 41, CURCELL, CURRENT ADPTIVE MOD'ER INDEX W/IN TBL
				; 		CELL = N
;----------------------------
L54FB  FDB  L0135		; 42, ERROR12, ERR BETWEEN DESIRED & ACTUAL SHIFT TIMES 
				;			   FOR LATEST 1-2 UPSHIFT  1=1/32 SEC
				;			    SEC. = N/40
;-----------------------------
L54FD  FDB  L0136		; 43, ERR0R23, ERR BETWEEN DESIRED & ACTUAL SHIFT TIMES 
				;			   FOR LATEST 2 -> 3 1=1/32 SEC
				;					SEC = N/40
;-----------------------------
L54FF  FDB  L0137		; 44,
;-----------------------------
L5501  FDB  L00F7		; 45,
;-----------------------------
L5503  FDB  L00DF		; 46,
;-----------------------------
L5505  FDB  L00E0		; 47,
;-----------------------------
L5507  FDB  L30E5		; 48, SLIP,   ABSOLUTE VAL OF SLIP (MSB)
L5509  FDB  L30E5+1		; 49, SLIP+1, ABSOLUTE VAL OF SLIP (LSB)
				;	    TPM/8 (signed value)
;------------------------------
L550B  FDB  L0132		; 50, TIME12, TIME OF LATEST 1->2 SHIFT
				;		SEC = N/40
;-----------------------------
L550D  FDB  L0133		; 51, TIME23, TIME OF LATEST 3->3 SHIFT
				;		SEC = N/40
;-----------------------------
L550F  FDB  L0083		; 52, TEMPMW       TEMPER STATUS WORD 
				; b7 1 = not used
				; b6 1 = not used
				; b5 1 = not used
				; b4 1 = not used  	 
				; b3 1 = not used   
				; b2 1 = not used
				; b1 1 = HOT  TEMPERATURE THRESHOLD
				; b0 1 = COLD TEMPERATURE THRESHOLD
;-----------------------------
L5511  FDB  L031B		; 53, EADPT2    PRESSURE MODIFIER (ADAPTIVE) - 2nd GEAR
				;				PSI = N/4
;-----------------------------
L5513  FDB  L031C		; 54, EADPT2+1  PRESSURE MODIFIER (ADAPTIVE) - 2nd GEAR
				;				PSI = N/4
;-----------------------------
L5515  FDB  L031D		; 55, EADPT2+2  PRESSURE MODIFIER (ADAPTIVE) - 2nd GEAR
				;				PSI = N/4
;-----------------------------
L5517  FDB  L031E		; 56, EADPT2+3  PRESSURE MODIFIER (ADAPTIVE) - 2nd GEAR
				;				PSI = N/4
;--------------------------------------
L5519  FDB  L031F		; 57, EADPT3,   PRESSURE MODIFIER (ADAPTIVE) - 3rd GEAR				PSI = N/4
;--------------------------------------
L551B  FDB  L0320		; 58, EADPT3+1  PRESSURE MODIFIER (ADAPTIVE) - 3rd GEAR
				;				PSI = N/4
;--------------------------------------
L551D  FDB  L0321		; 59, EADPT3+2  PRESSURE MODIFIER (ADAPTIVE) - 3rd GEAR
				;				PSI = N/4
;--------------------------------------
L551F  FDB  L0322		; 60, EADPT3+3, PRESSURE MODIFIER (ADAPTIVE) - 3rd GEAR
				;				PSI = N/4
;--------------------------------------
L5521  FDB  L0323		; 61,
;--------------------------------------
L5523  FDB  L0324		; 62,
;--------------------------------------
L5525  FDB  L0325		; 63,
;--------------------------------------
L5527  FDB  L0326		; 64,
;--------------------------------------
L5529  FDB  L0096		; 65, DIRFLAGS, BIT STATUS WORD
 				; b7 1 = DIAGNOSTICS REQUESTED
				; b6 1 = CRUISE LOW ACTIVE
				; b5 1 = 4 WD LOW ACTIVE
				; b4 1 = KICK DN PATTERN ACTIVE 	 
				; b3     (not used)
				; b2 1 = PWR ENRICH ACTIVE
				; b1 1 = A/C COMPRESSOR ENGAGED
				; b0 1 = BRAKE ON
;-----------------------------
L552B  FDB  L004B		; 66, IODPORTA,  I/O D PORT A
				; b7 1 = TCC ENABLE			0=OFF  1=ON	  
				; b6 1 = BYPASS (FROM REST)	  
				; b5 1 = BYPASS MONTR (FROM REST) 
				; b4 1 = BYPASS CTRL (FROM REST)    	
				; b3 1 = 3->2 DN SHIFT FEEDFBACK 	0=ON    1=OFF
				; b2 1 = TCC R\ENABLE FEEDFBACK		0=ON    1=OFF
				; b1 1 = SHIFT B FEEDFBACK		0=ON    1=OFF
				; b0 1 = SHIFT A FEEDFBACK		0=ON    1=OFF
;---------------------------------
L552C  FDB  L004C		; 67 IODPORTB,  I/O D PORT B                        
				; b7 1 = M/CLR				SFTB   SFTA  GR
				; b6 1 = REF IRQ CLR   			0     0    3rd
				; b5 1 = FORCE MOTOR ENABLE		0     1    4th
				; b4 1 = IAC ENABLE   			1     0    2nd							  1     1    1st
				; b3 1 = IAC COIL B ON   
				; b2 1 = IAC COIL A ON  
				; b1 1 = 2 - 3 SOL ON,	SHIFT B
				; b0 1 = 1 - 2 SOL ON,	SHIFT A
;-----------------------------    
L552F  FDB  L004D		; 68, IODPORTC, I/O PORT C             	  	
        		   	; b7 1 = FWD LOW SW (NO) 1=ON  
				; b6     (not used)
				; b5     (not used)		R1  R2  R3  GR
				; b4 1 = RANGE SW 3 OFF 	0   1   1   LOW 			  1   1   1   DR2
				; b3 1 = RANGE SW 2 OFF		1   1   0   DR3 
				; b2 1 = RANGE SW 1 OFF		1   0   0   DR4 
				; b1 1 = BK SW ON 		1   0   1   P/N
				; b0 1 = A/C REQUEST ON		0   0   1   REV 
;-----------------------------      
L5531  FDB  L4000		; 69, EPROM ID,MSB
L5533  FDB  L4000+1		; 70, EPROM ID,LSB
;-----------------------------
L5535  FDB  L012E		; 71,DSSDC  3-2 PWM SOL % DUTY CYCLE
				;   		% = N / 2.56
;-----------------------------
L5537  FDB  L0098		; 72, FAULTFLG     DISCRETE FAULT FLAGS
				;	b7  not used
				;	b6  not used
				;	b5  1 = SHIFT SOLENOID B VALID
				;	b4  1 = SHIFT SOLENOID A VALID
				;	b3  1 = INJECTOR 2 FAULT DETECTED
				;	b2  1 = INJECTOR 1 FAULT DETECTED
				;	b1  1 = QDM 2 FAULT DETECTED
				;	b0  1 = QDM 1 FAULT DETECTED
;---------------------------------
L5539  FDB  L0081		; 73, SHIFTED      SHIFT WORD
				;	b7 1 = SHIFT IS COMPLETE
           			;	b6 not used
           			;	b5 1 = 2-4 JUMP SHIFT WHEN PRNDL = D3
           			;	b4 1 = SHIFT HAS STARTED
           			;	b3 not used
           			;	b2 not used
           			;	b1 1 = DERIVATIVE SHIFT STARTED
           			;	b0 1 = SLOPE OF DERIVATIVE IS NEGATIVE
;-----------------------------    
L553B  FDB  L0099		; 74,  PATTERN,  MNP PATTERN
           			;	b7   not used
           			;	b6   not used
           			;	b5   not used
           			;	b4   not used
           			;	b3   1 = ILLEGAL PATTERN REQUESTED
           			;	b2   1 = "MANUAL" PATTERN REQUESTED
           			;	b1   1 = "PERFORMANCE" PATTERN REQUESTED
           			;	b0   1 = "NORMAL" PATTERN REQUESTED
          			;
;-----------------------------    
L553D  FDB  L02A7		; 75, FILTMPH,  MPH	 0 - 255 MPH                   	      
				;		MPH/1
				;		KPH = 1.61  
;-----------------------------    
L553F  FDB  L30C6		; 76, RAW OUTPUT SPEED(MSB)
L5541  FDB  L30C6+1		; 77, RAW OUTPUT SPEED(LSB)
				;	  	RPM = N/8
;-----------------------------
L5543  FDB  L0197		; 78, NDTHROPS, TPS FOR ENGINE
				; 	   (CURRENT NON-DEAULT VALUE)   
				;	   % * 2.56
;-----------------------------    
L5545  FDB  L00A2		; 79, COOL TEMP
				;   		deg c = n*.75 - 40
				;   		deg F = 1.35N - 40
;-----------------------------    
L5547  FDB  L00AE		; 80, TRANSDEGA, NORMALIZED TRANSMISSION FLUID TEMPERATURE
				;		NON-DEFAULTED
				;   	deg c = N*.75 - 40                  
				;   	deg f = 1.35N - 40
;---------------------------------------------
;				
;---------------------------------------------
;  DEVICE ID $F5, MODE 2
;				
; SEQUENTIAL MEMORY DUMP 
;---------------------------------------------
;
L5549  	FDB  $0000		; NEXT ENRTY ADDRESS
;
L554B  	FCB  $F5		; DEVICE ID
;
L554C  	FCB  $40		; MEM FLAG, TBL FROM RAM
L554D  	FCB   65		; NUM BYTES TO TX  
;
L554E  	FDB  L036E		; ADDR OF OUTPUT BUFFER
L5550  	FDB  L036E		; ADDR OF INPUT BUFFER
;---------------------------------------------
;
;---------------------------------------------
; DEVICE ID $F5, MODE 3
;
;  8 ADDRESS RAMDOM MEM DUMP
;---------------------------------------------
;
L5552  	FDB  $0000		; NEXT ENRTY ADDRESS
;
L5554  	FCB  $F5		; DEVICE ID
L5555  	FCB  $40		; MEM FLAG, TBL FROM RAM
L5556  	FCB  $9			; NUM BYTES TO TX 
;		
L5557  	FDB  L036E		; ADDR OF OUTPUT BUFFER
L5559  	FDB  L036E		; ADDR OF INPUT BUFFER
;---------------------------------------------
;
;---------------------------------------------
;  DEVICE ID $F5, MODE 4
;
; 'CONTROLLER MODE'
;---------------------------------------------
;
L555B  FDB  $0000		; NEXT ENRTY ADDRESS
;
L555D  FCB  $F5			; DEVICE ID
;
L555E  FCB  $00			; MEM FLAG, NONE
L555F  FCB    1			; NUM BYTES TO TX
;
L5560  FDB  L036E		; ADDR OF OUTPUT BUFFER
L5562  FDB  L036E		; ADDR OF INPUT BUFFER
;-------------------------------------------------
;
;-------------------------------------------------
;  DEVICE ID $F5, MODE 10
;
;    MODE 10 (CLEAR MALFUNCTION CODES)
;            ALDL REQUEST:
;            - MESSAGE ID     = $F5
;            - MESSAGE LENGTH = $56
;            - MODE           = $0A
;            - SUM CHECK
;
;            THE PCM WILL RESPOND WITH THE FOLLOWING MESSAGE:
;            - MESSAGE ID     = $F5
;            - MESSAGE LENGTH = $56
;            - MODE           = $0A
;            - SUM CHECK
;
;  RESET ERR'S
;-------------------------------------------------
;
L5564  FDB  $0000		; NEXT ENRTY ADDRESS
;
L5566  FCB  $F5			; DEVICE ID
;
L5567  FCB  $00			; MEM FLAG, NONE
L5568  FCB    1			; NUM BYTES TO TX
;
L5569  FDB  L036E		; ADDR OF OUTPUT BUFFER
L556B  FDB  L036E		; ADDR OF INPUT BUFFER
;----------------------------------------------
;
;----------------------------------------------
; INSTRUMENTION CALIB PARAM
;
;
;----------------------------------------------
;
	ORG	$55EE		
;
L55EE  	FDB   L0086		; DISP VAL SAVED TO THIS ADDR ON HI LIGHTS
L55F0  	FDB   L00D3		; DISP VAL SAVED TO THIS ADDR ON CAL POSIT NUM
;----------------------------------------------
;
;***********************************************
;* END OF ENGINE CALIBRATION
;*
;***********************************************
;
;***********************************************
;* DIAGNOSTIC MODE WORDS
;* ENGINE      
;*
;*  Dissassemby of BDWM TYPE $0D ECM
;*
;***********************************************
;----------------------------------------------
; DEFINED ENGINE ERRORS
;----------------------------------------------
;       ERR 13 o2 fail
;       ERR 14 HI COOL TEMP
;       ERR 15 LO COOL TEMP
;       ERR 16 2002 PPM Vss FAIL
;       ERR 21 HI TPS
;       ERR 22 LO TPS
;       ERR 23 MAT LOW
;       ERR 24 LO OUTPUT XMISH SPD
;       ERR 25 MAT HIGH
;       ERR 27 XMISH PRESS MANIFOLD
;       ERR 31 GOVERNER FAIL
;       ERR 32 EGR FAIL
;       ERR 33 MAP HI
;       ERR 34 MAP LOW
;       ERR 35 IAC FAIL
;       ERR 36 IAC TPS KICKER FAIL      
;       ERR 37 BRAKE ON
;       ERR 38 BRAKE OFF
;       ERR 41 TX, (CAM) PULSE SENSOR FAIL
;       ERR 42 EST MONITOR ERROR
;       ERR 43 KNOCK SYS FAIL
;       ERR 44 o2 LEAN
;       ERR 46 VATS FAIL
;       ERR 51 EPROM CKS SUM FAIL       
;       ERR 52 HI SYS VOLTAGE, LONG TEST
;       ERR 53 HI SYS VOLTAGE 
;       ERR 54 FUEL PUMP RELAY FAIL
;       ERR 58 HI XMIXH TEMP    
;       ERR 58 HI XMISH TEMP
;       ERR 59 LO XMISH TEMP
;       ERR 63 HIGH BARO PRESS
;       ERR 64 LOW BARO PRESS
;       ERR 66 3 -> 2 SHFT QUAD DVR FAIL       
;       ERR 67 TCC ENAB QUAD DVR FAIL
;       ERR 68 XMISH SLIPPING
;       ERR 69 TCC ON
;       ERR 71 LOW ENGINE SPD 
;       ERR 72 OUTPUT SPD LOSS
;       ERR 73 FORCE MOTOR CURRENT
;       ERR 74 TURBINE SPEED
;       ERR 75 LOW SYS VOLTAGE,
;       ERR 77 MNP SWITCH
;       ERR 79 HOT XMISH
;       ERR 81 QUAD DVR 1 & SHFT B ERR
;       ERR 82 QUAD DVR 1 & SHFT A ERR
;       ERR 83 QUAD DVR 1 ERR
;----------------------------------------------
;
	ORG	$5B00
;
;~~~~~~~~~~~~~~~~~~
;-Error word masks
;~~~~~~~~~~~~~~~~~~
;
; 1=flag error
;
L5B00   FCB $F1			; 1111 0001, ERR MASK 1
				; b7 1 = ERR 13, o2 fail
				; b6 1 = ERR 14, HIGH COOLANT TEMP
				; b5 1 = ERR 15, LOW COOLANT TEMP
				; b4 1 = ERR 16, 2002 PPM Vss FAIL
				; b3 1 = ERR 17
				; b2 1 = ERR 18
				; b1 1 = ERR 19
				; b0 1 = ERR 21, HIGH TPS
;
L5B01   FCB $A2			; 1010 0010, ERR MASK 2
				; b7 1 = ERR 22, TPS LOW 
				; b6 1 = ERR 23, MAT LOW
				; b5 1 = ERR 24, OUTPUT XMISH SPD LOW
				; b4 1 = ERR 25, MAT HIGH
				; b3 1 = ERR 26,  
				; b2 1 = ERR 27, XMISH PRESS MANIFOLD
				; b1 1 = ERR 28,
				; b0 1 = ERR 29,
;
L5B02   FCB $7B			; 0111 1011, ERR MASK 3
				; b7 1 = ERR 31, GOVERNER FAIL
				; b6 1 = ERR 32, EGR FAIL
				; b5 1 = ERR 33, MAP HI
				; b4 1 = ERR 34, MAP LOW
				; b3 1 = ERR 35, IAC FAIL
				; b2 1 = ERR 36, THROTTLE KICKER FAIL
				; b1 1 = ERR 37, BRAKE ON
				; b0 1 = ERR 38, BRAKE OFF
;
L5B03   FCB $3C			;0011 1100, Error mask 4
				; b7 1 = Error 39,
				; b6 1 = Error 41, TX, (CAM) pulse sensor failure
				; b5 1 = Error 42, EST mon. failure
				; b4 1 = Error 43, ESC failure
				; b3 1 = Error 44, O2 lean
				; b2 1 = Error 45, O2 rich
				; b1 1 = Error 46  VATS
				; b0 1 = Error 47,
;
L5B04   FCB $3C			; 0011 1100, ERR MASK 5
				; b7 1 = ERR 48, 
				; b6 1 = ERR 49,
				; b5 1 = ERR 51, EPROM CKS SUM FAIL
				; b4 1 = ERR 52, HI SYS VDC ERR, LONG TEST
				; b3 1 = ERR 53,, HI SYS VDC ERR 
				; b2 1 = ERR 54, FUEL PUMP RELAY FAIL
				; b1 1 = ERR 55,
				; b0 1 = ERR 56,
;
L5B05   FCB $60			; 0110 0000, ERR MASK 6
				; b7 1 = ERR 57,
				; b6 1 = ERR 58, HI XMISH TEMP
				; b5 1 = ERR 59, LO XMISH TEMP
				; b4 1 = ERR 61,
				; b3 1 = ERR 62, 
				; b2 1 = ERR 63, HIGH BARO PRESS
				; b1 1 = ERR 64, LOW BARO PRESS
				; b0 1 = ERR 65, 
;
L5B06   FCB $D6			; 1101 0110, ERR MASK 7
				; b7 1 = ERR 66, 3 -> 2 SHFT QUAD DVR FAIL
				; b6 1 = ERR 67, TCC ENAB QUAD DVR FAIL
				; b5 1 = ERR 68, TCC slipping/not engaged
				; b4 1 = ERR 69, TCC ON
				; b3 1 = ERR 71, LOW ENGINE SPD 
				; b2 1 = ERR 72, OUTPUT SPD LOSS
				; b1 1 = ERR 73, FORCE MOTOR CURRENT
				; b0 1 = ERR 74, TURBINE SPEED
;
L5B07   FCB $8E			; 1000 1110, ERR MASK 8
				; b7 1 = ERR 75, LOW SYS VOLTAGE,
				; b6 1 = ERR 76,
				; b5 1 = ERR 77, MNP SWITCH
				; b4 1 = ERR 78,
				; b3 1 = ERR 79, HOT XMISH
				; b2 1 = ERR 81, QUAD DVR 1 & SHFT B ERR
				; b1 1 = ERR 82, QUAD DVR 1 & SHFT A ERR
				; b0 1 = ERR 83, QUAD DVR 1 ERR
;
L5B08   FCB $00			; 0000 0000, ERR MASK 9
				; b7 1 = ERR 84,
				; b6 1 = ERR 85, RATIO UN-DEFINED OP REGION
				; b5 1 = ERR 86, LOW RATIO
				; b4 1 = ERR 87, HIGH RATIO
				; b3 1 = ERR 88, 
				; b2 1 = ERR 89, Excessive transmission slip detected
				; b1 1 = ERR 91,
				; b0 1 = ERR 92,
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Flagged error masks for SES lamp
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; 1=illuminate SES on error
;
L5B09   FCB $F1			; 1111 0001, ERR MASK CK XMISH
				; b7 1 = ERR 13, o2 SENSOR FAIL
				; b6 1 = ERR 14, HI COOLANT TEMP
				; b5 1 = ERR 15, LO COOLANT TEMP
				; b4 1 = ERR 16, 2002 PPM Vss FAIL
				; b3 1 = ERR 17
				; b2 1 = ERR 18
				; b1 1 = ERR 19
				; b0 1 = ERR 21,        HIGH TPS
;
L5B0A   FCB $80			; 1000 0000, ERR  MASK CK XMISH
				; b7 1 = ERR 22, TPS LOW 
				; b6 1 = ERR 23, MAT LOW
				; b5 1 = ERR 24, OUTPUT XMISH SPD LOW
				; b4 1 = ERR 25, MAT HIGH
				; b3 1 = ERR 26, 
				; b2 1 = ERR 27, XMISH PRESS MANIFOLD
				; b1 1 = ERR 28,
				; b0 1 = ERR 29,
;
L5B0B   FCB $78			; 0111 1000
				; b7 1 = ERR 31, GOVERNER FAIL
				; b6 1 = ERR 32, EGR FAIL
				; b5 1 = ERR 32, MAP HI
				; b4 1 = ERR 34, MAP LOW
				; b3 1 = ERR 35, IAC FAIL
				; b2 1 = ERR 36, IAC TPS KICKER FAIL
				; b1 1 = ERR 37, BRAKE ON
				; b0 1 = ERR 38, BRAKE OFF
;
L5B0C   FCB $3C			; 0011 1100
				; b7 1 = ERR 39,
				; b6 1 = ERR 41, TX, (CAM) PULSE SENSOR FAIL
				; b5 1 = ERR 42, EST MONITOR ERROR
				; b4 1 = ERR 43, KNOCK SYS FAIL
				; b3 1 = ERR 44, o2 LEAN
				; b2 1 = ERR 45, o2 RICH
				; b1 1 = ERR 46, VATS FAIL
				; b0 1 = ERR 47,
;
L5B0D   FCB $24			; 0010 0100
				; b7 1 = ERR 48, 
				; b6 1 = ERR 49,
				; b5 1 = ERR 51, EPROM CKS SUM FAIL
				; b4 1 = ERR 52, HI SYS VDC ERR, LONG TEST
				; b3 1 = ERR 53,, HI SYS VDC ERR 
				; b2 1 = ERR 54, FUEL PUMP RELAY FAIL
				; b1 1 = ERR 55,
				; b0 1 = ERR 56,
;
L5B0E   FCB $00			; 0000 0000
				; b7 1 = ERR 57,
				; b6 1 = ERR 58, HI XMISH TEMP
				; b5 1 = ERR 59, LO XMISH TEMP
				; b4 1 = ERR 61,
				; b3 1 = ERR 62, 
				; b2 1 = ERR 63, HIGH BARO PRESS
				; b1 1 = ERR 64, LOW BARO PRESS
				; b0 1 = ERR 65, 
;
L5B0F   FCB $00			; 0000 0000
				; b7 1 = ERR 66, 3 -> 2 SHFT QUAD DVR FAIL
				; b6 1 = ERR 67, TCC ENAB QUAD DVR FAIL
				; b5 1 = ERR 68, XMISH SLIPPING
				; b4 1 = ERR 69, TCC ON
				; b3 1 = ERR 71, LOW ENGINE SPD 
				; b2 1 = ERR 72, OUTPUT SPD LOSS
				; b1 1 = ERR 73, FORCE MOTOR CURRENT
				; b0 1 = ERR 74, TURBINE SPEED
;
L5B10   FCB $00			; 0000 0000
				; b7 1 = ERR 75, LOW SYS VOLTAGE,
				; b6 1 = ERR 76,
				; b5 1 = ERR 77, MNP SWITCH
				; b4 1 = ERR 78,
				; b3 1 = ERR 79, HOT XMISH
				; b2 1 = ERR 81, QUAD DVR 1 & SHFT B ERR
				; b1 1 = ERR 82, QUAD DVR 1 & SHFT A ERR
				; b0 1 = ERR 83, QUAD DVR 1 ERR
;
L5B11   FCB $00			; 0000 0000
				; b7 1 = ERR 84,
				; b6 1 = ERR 85, RATIO UN-DEFINED OP REGION
				; b5 1 = ERR 86, LOW RATIO
				; b4 1 = ERR 87, HIGH RATIO
				; b3 1 = ERR 88, 
				; b2 1 = ERR 89,  MAX ADPT LONG SHFT
				; b1 1 = ERR 91,
				; b0 1 = ERR 92,
;
;~~~~~~~~~~~~~~~~~~~
;-Error code params
;~~~~~~~~~~~~~~~~~~~
;
L5B12   FCB 50			;50 startups, number of error free starts to clear
				;set errors
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Error 14, high cool temp
;~~~~~~~~~~~~~~~~~~~~~~~~~~
;
L5B13   FCB 42			;97 deg C, cool temp threshold to set error
L5B14   FCB 5			;1 sec, time threshold to set/clear error
;
;~~~~~~~~~~~~~~~~~~~~~~~~~
;-Error 15, low cool temp
;~~~~~~~~~~~~~~~~~~~~~~~~~
;
L5B15   FCB 254			;-40 deg C, threshold to set error  
L5B16   FCB 10			;2 secs, time threshold to set/clear error
;
;-Default engine cool temp for transmission
;
L5B17   FCB 173			;90 deg C, default cool temp
;
;-Default engine cool temp params for errors 14/15
;
L5B18   FDB 0060    		;Engine runtime thresh to use high default cool temp
L5B1A	FCB 138     		;63.5 deg C, high default linear cool temp, used during run
L5B1B   FCB 80      		;20 deg C, low default linear cool temp, used during inital startup
;
;~~~~~~~~~~~~~~~~~~~~
;-Error 21, high TPS
;~~~~~~~~~~~~~~~~~~~~
;
L5B1C   FCB  249		;A/D TPS threshold to set error
L5B1D   FCB  160		;4 secs, time threshold to set error 21, high TPS
L5B1E	FCB  255		;255 starts/runs, threshold to permanently set error 21
;
;~~~~~~~~~~~~~~~~~~~
;-Error 22, low TPS
;~~~~~~~~~~~~~~~~~~~
;
L5B1F   FCB  8			;A/D TPS threshold to set error
L5B20   FCB  160		;4 secs, threshold to set error 22, low TPS
L5B21   FCB  255		;255 starts/runs, threshold to permanently set error 22
;
;-Default TPS params for errors 21/22
;
L5B22   FCB  90			;Default A/D trans TPS
L5B23   FCB  9			;Default A/D TPS
;
;~~~~~~~~~~~~~~~~~~~~~~
;-TPS self zero params
;~~~~~~~~~~~~~~~~~~~~~~
;
L5B24   FCB  64			;25%, max idle %TPS
L5B25   FCB  16			;Filter coeff for idle %TPS 
L5B26   FCB  92     		;1.44, scalar for %TPS              
L5B27   FCB  240    		;%TPS filter coeff.
;
;~~~~~~~~~~~~~~~~~~~~~
;-Kickdown TPS params
;~~~~~~~~~~~~~~~~~~~~~
;
L5B28   FCB  255		;Filter coeff, trans. TPS 
L5B29   FCB  1			; TPS OFFSET INCR for EACH DECEL
L5B2A   FCB  10			; 5 MPH Vss TO QULIFY DECEL for TPS INCREASE
;
;~~~~~~~~~~~~~~~~~~~
;-Diag. word  masks
;~~~~~~~~~~~~~~~~~~~
;
;-1 = Recognize error
;
L5B2B   FCB $61			;1100 0001, Error word #1 mask
                    		; b7, 1=Err 13, O2 error
                    		; b6, 1=Err 14, High cool temp
                    		; b5, 1=Err 15, Low cool temp
                    		; b4, 1=Err 16, 2002 PPM VSS error
                    		; b3, 1=Err 17
                    		; b2, 1=Err 18
                    		; b1, 1=Err 19
                    		; b0, 1=Err 21, High TPS error
;        
L5B2C   FCB $A2     		;1010 0010, Error word #2 mask
                    		; b7, 1=Err 22, TPS low 
                    		; b6, 1=Err 23, MAT low
                    		; b5, 1=Err 24, output trans. speed low
                    		; b4, 1=Err 25, MAT high
                    		; b3, 1=Err 26,  
                    		; b2, 1=Err 27, 
                    		; b1, 1=Err 28, Trans. press. manifold
                    		; b0, 1=Err 29,
;
L5B2D   FCB $33     		;0011 0011 Error word #3 mask
                    		; b7 1 = Err 31, Governer fail
                    		; b6 1 = Err 32, EGR fail
                    		; b5 1 = Err 32, MAP high
                    		; b4 1 = Err 34, MAP low
                    		; b3 1 = Err 35, IAC fail
                    		; b2 1 = Err 36, IAC TPS kicker fail
                    		; b1 1 = Err 37, Brake sw. on
                    		; b0 1 = Err 38, Brake sw. off
;
L5B2E   FCB $00			; 0000 0000 MASK XMISSH ERR WD 4
				; b7 1 = Err 39,
				; b6 1 = Err 41, TX, (CAM) PULSE SENSOR FAIL
				; b5 1 = Err 42, EST MONITOR ERROR
				; b4 1 = Err 43, KNOCK SYS FAIL
				; b3 1 = Err 44, o2 LEAN
				; b2 1 = Err 45, o2 RICH
				; b1 1 = Err 46, VATS FAIL
				; b0 1 = Err 47,
;
L5B2F   FCB $18     		;0001 1000, Diag. word #5 mask 
                    		; b7 1 = Err 48, 
                   		; b6 1 = Err 49,
                    		; b5 1 = Err 51, EPROM check sum fail
                    		; b4 1 = Err 52, Hi system voltage, long test
                    		; b3 1 = Err 53, Hi system voltage
                    		; b2 1 = Err 54, Fuel pump relay failure
                    		; b1 1 = Err 55,
                    		; b0 1 = Err 56,
;
L5B30   FCB $60			; 0110 0000, Diag. word #6 mask
				; b7 1 = Err 57,
                    		; b6 1 = Err 58, High trans temp
                    		; b5 1 = Err 59, Low trans temp
                    		; b4 1 = Err 61,
                    		; b3 1 = Err 62, 
                    		; b2 1 = Err 63, High baro pressure
                    		; b1 1 = Err 64, Low baro pressure
                    		; b0 1 = Err 65, 
;
L5B31	FCB $D6			;1101 0110, Diag. word #7 mask
                    		; b7, 1 = Err 66, 3 -> 2 shift quad driver fail
                    		; b6, 1 = Err 67, TCC enable quad driver fail
                    		; b5, 1 = Err 68, Trans. slipping
         			; b4, 1 = Err 69, TCC on
               			; b3, 1 = Err 71, Low engine speed 
               			; b2, 1 = Err 72, Output speed loss
               			; b1, 1 = Err 73, Force motor current
               			; b0, 1 = Err 74, Turbine speed
;
L5B32   FCB $8E			; 1000 1110,  ERR WD 8
				; b7 1 = Err 75, LOW SYS VOLTAGE,
				; b6 1 = Err 76,
				; b5 1 = Err 77, MNP SWITCH
				; b4 1 = Err 78,
				; b3 1 = Err 79, HOT XMISH
				; b2 1 = Err 81, QUAD DVR 1 & SHFT B ERR
				; b1 1 = Err 82, QUAD DVR 1 & SHFT A ERR
				; b0 1 = Err 83, QUAD DVR 1 ERR
;
L5B33   FCB $0			; 0000 0000, Trans. diag. word #9 mask
                    		;
                    		; b7 1 = Err 84,
                    		; b6 1 = Err 85, Ratio undefined op region
                    		; b5 1 = Err 86, Low ratio
                    		; b4 1 = Err 87, High ratio
                    		; b3 1 = Err 88, 
                    		; b2 1 = Err 89, Max adapt long shift
                    		; b1 1 = Err 91,
                    		; b0 1 = Err 92,
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Trans error filter words
;~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;-0 = dont filter error
;
L5B34   FCB $0			;0000 0000, Diag. word #1
                   	 	; b7, 0=Err 13, o2 error
                    		; b6, 0=Err 14, High cool temp
                    		; b5, 0=Err 15, Low cool temp
                    		; b4, 0=Err 16, 2002 PPM VSS error
                    		; b3, 0=Err 17
                    		; b2, 0=Err 18
                    		; b1, 0=Err 19
                    		; b0, 0=Err 21, High TPS error
;                                        
L5B35   FCB $0      		;0000 0000, Diag. word #2
                    		; b7, 0=Err 22, TPS low 
                    		; b6, 0=Err 23, MAT low
                    		; b5, 0=Err 24, output trans. speed low
                    		; b4, 0=Err 25, MAT high
                    		; b3, 0=Err 26,  
                    		; b2, 0=Err 27, trans. press. manifold
                    		; b1, 0=Err 28,
                    		; b0, 0=Err 29,
;
L5B36   FCB $0			; 0000 0000 XMISH ERR WD 3 ALT
				; b7 1 = Err 31, GOVERNER FAIL
				; b6 1 = Err 32, EGR FAIL
				; b5 1 = Err 32, MAP HI
				; b4 1 = Err 34, MAP LOW
				; b3 1 = Err 35, IAC FAIL
				; b2 1 = Err 36, IAC TPS KICKER FAIL
				; b1 1 = Err 37, BRAKE ON
				; b0 1 = Err 38, BRAKE OFF
;
L5B37   FCB $0			; 0000 0000 XMISH ERR WD 4 ALT
				; b7 1 = Err 39,
				; b6 1 = Err 41, TX, (CAM) PULSE SENSOR FAIL
				; b5 1 = Err 42, EST MONITOR ERROR
				; b4 1 = Err 43, KNOCK SYS FAIL
				; b3 1 = Err 44, o2 LEAN
				; b2 1 = Err 45, o2 RICH
				; b1 1 = Err 46, VATS FAIL
				; b0 1 = Err 47,
;
L5B38   FCB $0			; 0000 0000 XMISH ERR WD 5 ALT
				; b7 1 = Err 48, 
				; b6 1 = Err 49,
				; b5 1 = Err 51, EPROM CKS SUM FAIL
				; b4 1 = Err 52, HI SYS VDC ERR, LONG TEST
				; b3 1 = Err 53,, HI SYS VDC ERR 
				; b2 1 = Err 54, FUEL PUMP RELAY FAIL
				; b1 1 = Err 55,
				; b0 1 = Err 56,
;             
L5B39   FCB $0      		; 0000 0000 XMISH ERR WD 6 ALT
                    		; b7 1 = Err 57,
                    		; b6 1 = Err 58, HI XMISH TEMP
                    		; b5 1 = Err 59, LO XMISH TEMP
                    		; b4 1 = Err 61,
                    		; b3 1 = Err 62, 
                    		; b2 1 = Err 63, HIGH BARO PRESS
                    		; b1 1 = Err 64, LOW BARO PRESS
                    		; b0 1 = Err 65, 
;
L5B3A   FCB $0			;0000 0000, Diag. word #7
				; b7, 0 = Err 66, 3 -> 2 shift quad driver fail
				; b6, 0 = Err 67, TCC enable quad driver fail
				; b5, 0 = Err 68, Trans. slipping
				; b4, 0 = Err 69, TCC on
				; b3, 0 = Err 71, Low engine speed 
				; b2, 0 = Err 72, Output speed loss
				; b1, 0 = Err 73, Force motor current
				; b0, 0 = Err 74, Turbine speed
;
L5B3B   FCB $0			;0000 0000 XMISH ERR WD 8 ALT
				; b7 1 = Err 75, LOW SYS VOLTAGE,
				; b6 1 = Err 76,
				; b5 1 = Err 77, MNP SWITCH
				; b4 1 = Err 78,
				; b3 1 = Err 79, HOT XMISH
				; b2 1 = Err 81, QUAD DVR 1 & SHFT B ERR
				; b1 1 = Err 82, QUAD DVR 1 & SHFT A ERR
				; b0 1 = Err 83, QUAD DVR 1 ERR
;
L5B3C   FCB $0			;0000 0000 Diag. word #9 
				; b7 0 = Error 84
				; b6 0 = Err 85, Ratio undefined op region
				; b5 0 = Err 86, low ratio
				; b4 0 = Err 87, high ratio
				; b3 0 = Err 88, 
				; b2 0 = Err 89,  Max adapt long shift
				; b1 0 = Err 91,
				; b0 0 = Err 92,
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Error 24, low VSS params
;~~~~~~~~~~~~~~~~~~~~~~~~~~
;
L5B3D   FDB 2000		;250 RPM, max TOS speed to set error
L5B3F   FDB 24000		;3000 RPM min engine speed to set error
;
L5B41   FCB  120		;3 secs, time threshold to set error
L5B42   FCB  255		;Min threshold to set error 24
;
L5B43   FDB  37519		;2.289:1, ratio for reverse gear 
;
L5B45   FCB  100		;51.3 kPa, min MAP threshold to set error 
L5B46   FCB  255		;100 kPa, max MAP threshold to set error  
L5B47   FCB  26			;10%, min TPS threshold to set error   
L5B48   FCB  255		;100%, max TPS threshold to set error 
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Transmission gear ratios
;~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; Gear ratio = val/16384	;Gear #
;
L5B49   FDB  50135		;1st 
	FDB  26705		;2nd 
	FDB  16384		;3rd 
	FDB  11468		;4th 
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Error 28, pressure manifold switch params
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
L5B51   FCB  80			;2 secs, time threshold to set error 28
L5B52   FCB  255		;Threshold to set error perminently
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Error 37, brake switch stuck on params
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
L5B53   FCB  10			;5 MPH, threshold for vehicle to be stopped
L5B54   FCB  40			;20 MPH, threshold for vehicle to be in motion
L5B55   FCB  60			;6 secs, time for vehicle to be in motion to flag error
L5B56   FCB  60			;6 secs, time for vehicle to be coasting to flag error
L5B57   FCB  7			;7, number of faults to set error 37
L5B58	FCB  255		;255, number of times before error 37 remains permanently set
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Error 38, brake switch inoperative
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
L5B59   FCB  40			;20 MPH, MPH threshold for vehicle to be in motion
L5B5A   FCB  10			;5 MHP, threshold for vehicle to be stopped
L5B5B   FCB  60 		;6 secs, time threshold for vehicle to be decelerating using
				;brakes to set error
L5B5C   FCB  60			;6 secs, time threshold for vehicle in motion to set error
L5B5D   FCB  7			;7, number of brake switch faults to set error
L5B5E   FCB  255		;255, number of times before error 38 remains permanently set
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Error 52, extended high battery voltage params 
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
L5B5F   FCB  160		;16.0V, threshold to set error
L5B60   FDB  65535		;6553.5 secs, time threshold to set error
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Error 53, high battery voltage params
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~                              
;
L5B62   FCB  195		;19.5V, threshold to set error  
L5B63   FCB  20			;2 secs, time threshold to set error
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Error 58,  high trans temp params
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
L5B64   FCB  17			;130 deg C, A/D temp threshold for error 58
L5B65   FCB  5			;1 sec, time to set/clear error 58
L5B66   FCB  227		;130 deg C, default trans temp
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Error 59, low trans temp params
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
L5B67   FCB  251		;-30 deg C, A/D temp threshold for error 59
L5B68   FCB  5			;1 sec, time to set/clear error 59 
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Error 63, baro pressure high params
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
L5B69   FCB  249		;102.2 kPa, threshold to set error
L5B6A   FCB  20			;2 secs, time threshold to set error
L5B6B	FDB  32768		; DEFAULT 100 Kpa BARO IF ERR 63
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Error 64, baro pressure low params
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
L5B6D   FCB  3			;11.5 kPa, threshold to set error
L5B6E   FCB  20			;2 secs, time threshold to set error
;
L5B6F   FDB  32768		;90 kPa, default trans baro if error
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Error 66,  3 -> 2 shift quad driver failure params
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
L5B71   FCB  40			;4 sec, time threshold to set error
L5B72   FCB  255		;255, number of errors to leave error 66 permanently set
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Error 67, TCC solenoid failure params
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
L5B73   FCB  20			;2 secs, time threshold to set error
L5B74   FCB  255		;255, number of errors for error 67 to remain set
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Error 68, TCC slipping/not engaged params                                  
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
L5B75   FDB  800		;100 RPM, threshold for TCC to be slipping
L5B77   FCB  255		;25.5 secs, time threshold to set error
L5B78   FCB  10			;10, max number of logged errors to allow for a reset
L5B79   FCB  255		;# of errors for error 68 to remain set	
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Error 69, TCC failed to disengage
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;-TCC slip window to set error
;
L5B7A   FDB  0160		;20 RPM, upper TCC slip window bound
L5B7C   FDB  65376		;-20 RPM, lower TCC slip window bound
;
L5B7E   FCB  40			;4 secs, time threshold to set error  
L5B7F   FCB  64			;25% TPS, lower TPs threshold to set error
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Error 71, low engine speed params
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~                                 
;
L5B80   FDB  1600		;200 RPM, engine speed threshold to set error
L5B82   FCB  80			;2 secs, time threshold to set error
L5B83   FDB  2400		;300 RPM, lower engine RPM threshold to set error 69
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Error 72, output speed lost params
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
L5B85   FDB  8000		;1000 RPM, threshold to set error, in gear                    
L5B87   FDB  16384		;2048 RPM, threshold to set error, in P/N or invalid range                 
L5B89   FDB  1600		;200 RPM, min engine speed to set error
L5B8B   FCB  255		;# of errors for error 72 to remain perm. set
L5B8C   FCB  80			;2 secs, time threshold to set error 72  
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Error 73, force motor fault params 
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
L5B8D   FCB  8			;160 mA, sustained proportional error to set error
L5B8E   FCB  20			;2 secs, time threshold to set error 2 SECS CRITERIA FOR ERR 73 
L5B8F   FCB  255		;# of faults for error 73 to remain perminently set
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-ERR 75, low system voltage params
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
L5B90   FCB  73			;7.3 volts, lower temp range for seting error
L5B91   FCB  114		;11.4 volts, upper temp range for setting error
;
L5B92   FCB  83			;8.3 volts, lower temp range for error recovery
L5B93   FCB  124		;12.4 volts, upper temp range for error recovery
;
L5B94   FCB  40			;4 secs, time threshold to set error
;
L5B95   FDB  8000		;200 RPM, threshold for performing diags.
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Error 77, manual/normal/performance switch params
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
L5B97   FCB  40			;1 sec, time threshold to set error
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Error 79, prolonged high transmission temp params
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
L5B98   FCB  236		;137 deg C, threshold to reset error 79
L5B99   FCB  248		;146 deg C, threshold to set error 79  
L5B9A   FDB  18000		;1800 secs, time threshold to set error
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Error 81, 2 -> 3 shift sol. driver failure params
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
L5B9C   FCB  20			;2 secs, time threshold to set error
L5B9D   FCB  255    		;Number of errors for error to remain set
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Error 82, 1 -> 2 shift sol. driver failure params
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
L5B9E   FCB  20			;2 secs, time threshold to set error
L5B9F   FCB  255		;Number of errors for error to remain set
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Error 83, shift solenoid driver failure params
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
L5BA0   FCB  20			;2 secs, time threhsold to set error
L5BA1   FCB  5			;Number of errors for error to remain set
;
;~~~~~~~~~
;-Error 85 
;~~~~~~~~~
;
L5BA2   FCB  10     		;
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Error 86, engine speed to MPH ratio low params
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
L5BA3   FDB  65535		;16383.6:1  engine speed to MPH threshold to set error
L5BA5   FCB  255		;99.6% TPS, threshold to allow error to be set
L5BA6   FCB  255		;127.5 MPH, threshold to allow error to be set
L5BA7   FCB  255		;25.5 secs, time threshold to set error
L5BA8   FCB  255    		;Number of errors for error to remain set
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Error 87, engine speed to MPH ratio high params
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
L5BA9   FDB  0000		;0:1, engine speed to MPH ratio to set error
L5BAB   FCB  255		;99.6% TPS, threshold to set error
L5BAC   FCB  255		;151 deg C, trans temp threshold to set error
L5BAD   FCB  255		;25.5 sec, time threshold to set error
L5BAE   FCB  255    		;Threshold to set error
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Error 89, excessive transmission slip params
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
L5BAF   FCB  0			;-40 deg C, lower trans temp threshold for diags.
L5BB0   FCB  0			;0% TPS, lower TPS threshold for diags.
L5BB1   FCB  255		;128 MPH, lower MPG threshold for diags.
;
L5BB2   FCB  0			;Min allowed time for previous shift to set error (1 -> 2) 
L5BB3   FCB  0			;Min allowed time for previous shift to set error (2 -> 3)
L5BB4   FCB  0			;Min allowed time for previous shift to set error (3 -> 4)
;
L5BB5   FCB  0			;Number of shifts to set error (1 -> 2)
L5BB6   FCB  0			;Number of shifts to set error (2 -> 3)
L5BB7   FCB  0			;Number of shifts to set error (3 -> 4)
L5BB8   FCB  0			;Number of errors for error 89 to remain permanently set
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Base transmission params
;~~~~~~~~~~~~~~~~~~~~~~~~~~
;     
        ORG $5D00
; 
L5D00   FDB $9CB6		;PROM ID for transmission
;
L5D02	FCB $21			;0010 0001, Option byte
                    		; b7=
                    		; b6=
                    		; b5, 1 = Force 2nd gear if in OD and not in manualFORCE 2ND GR IF IN D2 AND NOT MANUAL
                    		; b4, 0 = Output speed from dig. ratio adaptor.   
                    		; b3=
                    		; b2, 1=Use MAP and baro, 0=MAP only
                    		; b1=
                    		; b0, 1 = Allow TPS hyst. buffer every 25 msecs
;
L5D03   FCB $1F			;0001 1111, Option byte
				; b7
				; b6
				; b5
				; b4, 1=Filter 4 wheel drive low input 
				; b3, 1=Filter cruise input
				; b2, 1=Filter A/C input
				; b1, 1=Filter pattern inputs
				; b0, 1=Filter range inputs  
;
L5D04   FCB 4			;val./160 = .025 sec, min time for brake switch change state  
;
L5D05   FCB 121     		;5.1 kPa/256 counts, trans baro scalar 
L5D06   FDB 3513    		;10.71 kPa, trans baro bias   
L5D08   FCB 64			;Trans baro filter coeff.
;
L5D09   FCB 101			;Transmission temperature filter coeff.
L5D0A   FCB 12     		;Engine coolant for transmission filter coeff.
L5D0B   FCB 52			;Torque converter slip filter coeff.
;
L5D0C   FCB 3			;120 msec, min time to clear powertrain in motion flag
L5D0D   FCB 0			;Non-zero value disables manual shift pattern
L5D0E   FCB 21			;21 RPM samples for rate of change in engine RPM
;
;~~~~~~~~~~~~~~~~~~~~~~~~~
;-Transmission/VSS params
;~~~~~~~~~~~~~~~~~~~~~~~~~
;
L5D0F	FCB 64			;Filter coeff. for engine RPM x 8
L5D10	FCB 252			;Filter coeff. for Trans Input Speed RPMs
L5D11   FCB 252			;Filter coeff. for transmission input speed
;
L5D12	FCB 1			;20 msec max time to wait for TOS signal before signal
				;considered no longer present
;
L5D13   FDB 43946		;Constant for TOS RPM calc, val = (# pulses/rev) x 65536 / 60
;
L5D15   FCB 252			;Transmission output speed filter coeff.
;
L5D16   FDB 44564		;2.72:1, 4WD low reduction ratio
;
L5D18   FDB 45773		;Value for converting TOS RPMs to MPH
				;
				;	     8 x num. gear ratio x 12 x 5280
				; Value = ------------------------------------- x 65536
				;          512 x 60 x tire rad in in. x 2 x pi
				;
;
L5D1A	FCB 252			;Filter coeff. for MPH
;
;~~~~~~~~~~~~~~~~~~~~~
;-Kickdown TPS params
;~~~~~~~~~~~~~~~~~~~~~
;
L5D1B	FCB 242			;95% TPS, threshold to enable kickdown
L5D1C   FCB 230			;90.2% TPS, threshold to disable kickdown
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Params for powertrain to be in motion
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
L5D1D   FDB 1600		;200 RPM, min engine RPM
L5D1F   FDB 65535		;8191.9, min TOS RPM
L5D21   FDB 65535		;8191.9, min TIS RPM
;
L5D23   FCB 160			;4 secs, delay for error 72 diags. when 4WD low has changed
				;state
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Time threshold to prevent gear shifts while TCC being applied/released
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
L5D24   FCB 0			;0 sec, time threshold to limit shifts after TCC applied/
				;released
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Time thresholds to limit nusiance downshifts into 2nd from 4th gear
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
L5D25   FCB 10			;1 sec, max time to limit 3->2 shift when limiting 4->2 shifts    
L5D26   FCB 2			;.2 sec, lower threshold for 4 -> 2 shift limiting      
L5D27   FCB 15			;1.5 sec, upper threshold for 4 -> 2 shift limiting   
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Temp window for prevending shifts into 4th 
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
L5D28   FCB 29			;-18 deg C, lower bound
L5D29   FCB 33			;-15 deg C, upper bound
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Window defining cold engine trans shifts
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
L5D2A   FCB 93			;30 deg C, lower cold shift bound
L5D2B   FCB 100			;35 deg C, upper cold shift bound 
;
;~~~~~~~~~~~~~~~~~~~~~~~
;-Baro shift point gain
;~~~~~~~~~~~~~~~~~~~~~~~
;
;	gain =(value - 128) / 128
;
L5D2C   FCB 128			;0, baro shift point gain
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Downshift threshold for second gear if output speed lost
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
L5D2D   FDB 5200		;650 RPM, upper RPM threshold to force second gear  
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Manual pattern shift params
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
L5D2F   FCB 0			;0 MPH, downshift threshold for manual 4 -> 3, 3 -> 2 shift
L5D30   FCB 255			;128 MPH, upshift threshold for manual 2 -> 3, 3 -> 4 shift
L5D31   FCB 56			;28 MPH, manual 1 -> 2 upshift threshold
L5D32   FCB 52			;26 MPH, manual 1 -> 2 downshift threshold 
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Base Kickdown MPH shift thresholds 
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~		
;
;-Upshift threshold
;		
;	MPH = val/2		;Gear shift      
;
L5D33   FCB 56			;1 -> 2
L5D34   FCB 106			;2 -> 3
L5D35   FCB 254			;3 -> 4
;
;-Downshift threshold
;
L5D36   FCB 46			;2 -> 1
L5D37   FCB 98			;3 -> 2
L5D38   FCB 250			;4 -> 3
;
;-upshift threshold for hot transmission       
;
L5D39   FCB 55			;1 -> 2  
L5D3A   FCB 101			;2 -> 3 
L5D3B   FCB 254			;3 -> 4
;
;-Downshift threshold for hot transmission
;
L5D3C   FCB 45			;2 -> 1  
L5D3D   FCB 93			;3 -> 2  
L5D3E   FCB 244			;4 -> 3  
; 
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Kickdown cold MPH upshift modifier vs. gear #
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;	MPH mod. = (val-128)/2	;Gear shift       
;
L5D3F   FCB 124			;1 -> 2  
L5D40   FCB 122			;2 -> 3  
L5D41   FCB 128			;3 -> 4 
; 
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Kickdown cold MPH downshift modifier vs. gear #
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;	MPH mod. = (val-128)/2	;Gear shift       
;
L5D42   FCB 124			;2 -> 1   
L5D43   FCB 122			;3 -> 2   
L5D44   FCB 128			;4 -> 3  
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Kickdown mode standard/cruise cont'l transmission upshift RPM vs. gear shift
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;	RPM = val/8		;Gear shift       
;
L5D45   FDB 32000		;1 -> 2   
L5D47   FDB 32000		;2 -> 3  
L5D49   FDB 65535		;3 -> 4         
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Kickdown mode hot transmission upshift RPM vs. gear shift
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;	RPM = val/8		;Gear shift       
;
L5D4B   FDB 30400		;1 -> 2  
L5D4D   FDB 30400		;2 -> 3 
L5D4F   FDB 65535		;3 -> 4 
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Kickdown cold upshift RPM modifier vs. gear shift
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;	signed RPM = val / 8	;Gear shift
;      
L5D51   FDB 63936		;1 -> 2   
L5D53   FDB 63936		;2 -> 3     
L5D55   FDB 63936		;3 -> 4 
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Manual 1 -> 2 upshift MPH vs. TPS
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;	MPH = val/2		;%TPS
;
L5D57  	FCB   0			;  0.0
	FCB   0			;  6.3
	FCB   0			; 12.5
	FCB   0			; 18.8
	FCB   0			; 25.0
	FCB   0			; 31.3
	FCB   0			; 37.5
	FCB   0			; 43.8
	FCB   0			; 50.0
	FCB   0			; 56.3
	FCB   0			; 62.5
	FCB   0			; 68.8
	FCB   0			; 75.0
	FCB   0			; 81.3
	FCB   0			; 87.5
	FCB   0			; 93.8
	FCB   0			;100.0
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Manual 2 -> 3 upshift MPH vs. TPS
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;	MPH = val/2		;%TPS
;
L5D68  	FCB   0			;  0.0
	FCB   0			;  6.3
	FCB   0			; 12.5
	FCB   0			; 18.8
	FCB   0			; 25.0
	FCB   0			; 31.3
	FCB   0			; 37.5
	FCB   0			; 43.8
	FCB   0			; 50.0
	FCB   0			; 56.3
	FCB   0			; 62.5
	FCB   0			; 68.8
	FCB   0			; 75.0
	FCB   0			; 81.3
	FCB   0			; 87.5
	FCB   0			; 93.8
	FCB   0			;100.0
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Cruise cont'l 2 -> 3 upshift MPH vs. %TPS
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;	MPH = val / 2		;%TPS
;
L5D79   FCB  38			;  0.0
	FCB  38			;  6.3
	FCB  41			; 12.5
	FCB  51			; 18.8
	FCB  61			; 25.0
	FCB  67			; 31.3
	FCB  71			; 37.5
	FCB  75			; 43.8
	FCB  79			; 50.0
	FCB  83			; 56.3
	FCB  87			; 62.5
	FCB  91			; 68.8
	FCB  94			; 75.0
	FCB  97			; 81.3
	FCB  102		; 87.5
	FCB  106		; 93.8
	FCB  106		;100.0
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Cruise cont'l 3 -> 4 upshift MPH vs. %TPS
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;	MPH = val / 2		;%TPS
;
L5D8A   FCB  55			;  0.0
	FCB  55			;  6.3
	FCB  57			; 12.5
	FCB  66			; 18.8
	FCB  76			; 25.0
	FCB  86			; 31.3
	FCB  95			; 37.5
	FCB  226		; 43.8
	FCB  226		; 50.0
	FCB  226		; 56.3
	FCB  226		; 62.5
	FCB  226		; 68.8
	FCB  226		; 75.0
	FCB  226		; 81.3
	FCB  226		; 87.5
	FCB  226		; 93.8
	FCB  226		;100.0
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Cruise cont'l/standard 3 -> 2 downshift MPH vs. %TPS
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;	MPH = val / 2		;%TPS
;
L5D9B   FCB 36			;  0.0
	FCB 36			;  6.3
	FCB 36			; 12.5
	FCB 36			; 18.8
	FCB 36			; 25.0
	FCB 36			; 31.3
	FCB 36			; 37.5
	FCB 36			; 43.8
	FCB 47			; 50.0
	FCB 58			; 56.3
	FCB 63			; 62.5
	FCB 69			; 68.8
	FCB 73			; 75.0
	FCB 79			; 81.3
	FCB 85			; 87.5
	FCB 90			; 93.8
	FCB 98			;100.0
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Cruise cont'l/standard 4 -> 3 downshift MPH vs. %TPS
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;	MPH = val / 2		;%TPS
;
L5DAC   FCB 48			;  0.0
	FCB 48			;  6.3
	FCB 48			; 12.5
	FCB 49			; 18.8
	FCB 54			; 25.0
	FCB 60			; 31.3
	FCB 66			; 37.5
	FCB 70			; 43.8
	FCB 76			; 50.0
	FCB 84			; 56.3
	FCB 91			; 62.5
	FCB 100			; 68.8
	FCB 113			; 75.0
	FCB 220			; 81.3
	FCB 220			; 87.5
	FCB 220			; 93.8
	FCB 220			;100.0
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Cruise cont'l/standard 1 -> 2 upshift MPH vs. %TPS
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;	MPH = val / 2		;%TPS
;
L5DBD   FCB 20			;  0.0
	FCB 20			;  6.3
	FCB 24			; 12.5
	FCB 28			; 18.8
	FCB 33			; 25.0
	FCB 36			; 31.3
	FCB 38			; 37.5
	FCB 40			; 43.8
	FCB 42			; 50.0
	FCB 44			; 56.3
	FCB 46			; 62.5
	FCB 48			; 68.8
	FCB 50			; 75.0
	FCB 52			; 81.3
	FCB 54			; 87.5
	FCB 56			; 93.8
	FCB 56			;100.0
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Standard 2 -> 3 upshift MPH vs. %TPS
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;	MPH = val / 2		;%TPS
;
L5DCE   FCB 38			;  0.0
	FCB 38			;  6.3
	FCB 41			; 12.5
	FCB 51			; 18.8
	FCB 61			; 25.0
	FCB 67			; 31.3
	FCB 71			; 37.5
	FCB 75			; 43.8
	FCB 79			; 50.0
	FCB 83			; 56.3
	FCB 87			; 62.5
	FCB 91			; 68.8
	FCB 94			; 75.0
	FCB 97			; 81.3
	FCB 102			; 87.5
	FCB 106			; 93.8
	FCB 106			;100.0
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Standard 3 -> 4 upshift MPH vs. %TPS
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;	MPH = val / 2		;%TPS
;
L5DDF   FCB  55			;  0.0
	FCB  55			;  6.3
	FCB  57			; 12.5
	FCB  66			; 18.8
	FCB  76			; 25.0
	FCB  86			; 31.3
	FCB  95			; 37.5
	FCB  103		; 43.8
	FCB  111		; 50.0
	FCB  121		; 56.3
	FCB  226		; 62.5
	FCB  226		; 68.8
	FCB  226		; 75.0
	FCB  226		; 81.3
	FCB  226		; 87.5
	FCB  226		; 93.8
	FCB  226		;100.0
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Cruise cont'l/standard 2 -> 1 upshift MPH vs. %TPS
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;	MPH = val / 2		;%TPS
;
L5DF0   FCB  16			;  0.0
	FCB  16			;  6.3
	FCB  16			; 12.5
	FCB  16			; 18.8
	FCB  16			; 25.0
	FCB  16 		; 31.3
	FCB  16			; 37.5
	FCB  16			; 43.8
	FCB  16			; 50.0
	FCB  16			; 56.3
	FCB  16			; 62.5
	FCB  16			; 68.8
	FCB  18			; 75.0
	FCB  23			; 81.3
	FCB  31			; 87.5
	FCB  43			; 93.8
	FCB  43			;100.0
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Manual pattern 3 -> 2 downshift MPH vs. %TPS
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;	MPH = val / 2		;%TPS
;
L5E01   FCB  36			;  0.0
	FCB  36			;  6.3
	FCB  36			; 12.5
	FCB  36			; 18.8
	FCB  36			; 25.0
	FCB  36			; 31.3
	FCB  36			; 37.5
	FCB  36			; 43.8
	FCB  47			; 50.0
	FCB  58			; 56.3
	FCB  63			; 62.5
	FCB  69			; 68.8
	FCB  73			; 75.0
	FCB  79			; 81.3
	FCB  85			; 87.5
	FCB  90			; 93.8
	FCB  98			;100.0
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Manual pattern 4 -> 3 downshift MPH vs. %TPS
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;	MPH = val / 2		;%TPS
;
L5E12	FCB  48			;  0.0
	FCB  48			;  6.3
	FCB  48			; 12.5
 	FCB  49			; 18.8
	FCB  54			; 25.0
	FCB  60			; 31.3
	FCB  66			; 37.5
	FCB  70			; 43.8
	FCB  76			; 50.0
	FCB  84			; 56.3
	FCB  91			; 62.5
	FCB  100		; 68.8
	FCB  113		; 75.0
	FCB  220		; 81.3
	FCB  220		; 87.5
	FCB  220		; 93.8
	FCB  220		;100.0
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Cold engine 1 -> 2 upshift modifier vs. %TPS
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;	MPH corr. = val - 128	;%TPS
;
L5E23  	FCB   128		;  0.0
	FCB   128		;  6.3
	FCB   124		; 12.5
	FCB   124		; 18.8
	FCB   124		; 25.0
	FCB   124		; 31.3
	FCB   124		; 37.5
	FCB   124		; 43.8
	FCB   124		; 50.0
	FCB   124		; 56.3
	FCB   124		; 62.5
	FCB   124		; 68.8
	FCB   124		; 75.0
	FCB   124		; 81.3
	FCB   124		; 87.5
	FCB   124		; 93.8
	FCB   124		;100.0
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Cold engine 2 -> 3 upshift modifier vs. %TPS
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;	MPH corr. = val - 128	;%TPS
;
L5E34  	FCB   128		;  0.0
	FCB   128		;  6.3
	FCB   124		; 12.5
	FCB   124		; 18.8
	FCB   124		; 25.0
	FCB   124		; 31.3
	FCB   124		; 37.5
	FCB   124		; 43.8
	FCB   124		; 50.0
	FCB   124		; 56.3
	FCB   124		; 62.5
	FCB   124		; 68.8
	FCB   124		; 75.0
	FCB   124		; 81.3
	FCB   124		; 87.5
	FCB   124		; 93.8
	FCB   124		;100.0
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Cold engine 3 -> 4 upshift modifier vs. %TPS
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;	MPH corr. = val - 128	;%TPS
;
L5E45  	FCB   128		;  0.0
	FCB   128		;  6.3
	FCB   128		; 12.5
	FCB   124		; 18.8
	FCB   124		; 25.0
	FCB   124		; 31.3
	FCB   124		; 37.5
	FCB   124		; 43.8
	FCB   124		; 50.0
	FCB   124		; 56.3
	FCB   124		; 62.5
	FCB   124		; 68.8
	FCB   124		; 75.0
	FCB   124		; 81.3
	FCB   124		; 87.5
	FCB   124		; 93.8
	FCB   124		;100.0
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Cold engine 2 -> 1 downshift modifier vs. %TPS
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;	MPH corr. = val - 128	;%TPS
;
L5E56	FCB   128		;  0.0
	FCB   124		;  6.3
	FCB   124		; 12.5
	FCB   124		; 18.8
	FCB   124		; 25.0
	FCB   124		; 31.3
	FCB   124		; 37.5
	FCB   124		; 43.8
	FCB   124		; 50.0
	FCB   124		; 56.3
	FCB   124		; 62.5
	FCB   124		; 68.8
	FCB   124		; 75.0
	FCB   124		; 81.3
	FCB   124		; 87.5
	FCB   124		; 93.8
	FCB   124		;100.0
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Cold engine 3 -> 2 downshift modifier vs. %TPS
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;	MPH corr. = val - 128	;%TPS
;
L5E67	FCB   128		;  0.0
	FCB   124		;  6.3
	FCB   124		; 12.5
	FCB   124		; 18.8
	FCB   124		; 25.0
	FCB   124		; 31.3
	FCB   124		; 37.5
	FCB   124		; 43.8
	FCB   124		; 50.0
	FCB   124		; 56.3
	FCB   124		; 62.5
	FCB   124		; 68.8
	FCB   124		; 75.0
	FCB   124		; 81.3
	FCB   124		; 87.5
	FCB   124		; 93.8
	FCB   124		;100.0
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Cold engine 4 -> 3 downshift modifier vs. %TPS
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;	MPH corr. = val - 128	;%TPS
;
L5E78	FCB   128		;  0.0
	FCB   124		;  6.3
	FCB   124		; 12.5
	FCB   124		; 18.8
	FCB   124		; 25.0
	FCB   124		; 31.3
	FCB   124		; 37.5
	FCB   124		; 43.8
	FCB   124		; 50.0
	FCB   124		; 56.3
	FCB   124		; 62.5
	FCB   124		; 68.8
	FCB   124		; 75.0
	FCB   124		; 81.3
	FCB   124		; 87.5
	FCB   124		; 93.8
	FCB   124		;100.0
;
;~~~~~~~~~~~~~~~~~~~~~~
; 3->2 solenoid params
;~~~~~~~~~~~~~~~~~~~~~~
;
L5E89	FCB $0F			;Transmission option byte
				; b7= 
				; b6=
				; b5= 
				; b4= 
				; b3=    
				; b2, 1=A/C  comp. enabled
				; b1, 1=Temp comp. enabled
				; b0= 
;
L5E8A	FCB 0			;0%, min allowed DC for 3->2 shift sol.
L5E8B	FCB 230 		;90%, max allowed DC for 3->2 shift sol.
;
L5E8C	FCB 0			;0%, DC for alternate apply rate, manual range select
L5E8D	FCB 0			;0%, DC for alternate apply rate, output speed lost
L5E8E	FCB 40			;4 secs., max time for 3->2 shift modulation
;
L5E8F	FCB 135			;2.7% DC, A/C on compensation
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Base 3->2 solenoid duty cycle vs. baro and MPH
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;	DC = val/256, lower number, lower apply rate; higher number, higher apply rate
;
L5E90   FCB   0    	; Min BARO Kpa Val, (128/BARO)
        FCB   0    	; Min MPH Value, (40 MPH, SCALED)
        FCB   17   	; COL'S/ROW
;
;	MPH: 10   14   18   22   26   30   34   38   42   46   50   54   58   62   66   70   74 :kPa baro
;
L5E93	FCB  25,  25,  25,  97, 172, 192, 212, 230, 230, 230, 230, 230, 230, 230, 230, 230, 230 ; 50
	FCB  25,  25,  25,  97, 172, 192, 212, 230, 230, 230, 230, 230, 230, 230, 230, 230, 230 ; 54 
	FCB  25,  25,  25,  97, 172, 192, 212, 230, 230, 230, 230, 230, 230, 230, 230, 230, 230 ; 58         
	FCB  25,  25,  25,  97, 172, 192, 212, 230, 230, 230, 230, 230, 230, 230, 230, 230, 230 ; 62          
	FCB  25,  25,  25,  89, 164, 184, 204, 224, 230, 230, 230, 230, 230, 230, 230, 230, 230 ; 66 
	FCB  25,  25,  25,  81, 156, 176, 196, 216, 230, 230, 230, 230, 230, 230, 230, 230, 230 ; 70 
	FCB  25,  25,  25,  73, 148, 168, 188, 208, 228, 230, 230, 230, 230, 230, 230, 230, 230 ; 74 
	FCB  25,  25,  25,  65, 140, 160, 180, 200, 220, 230, 230, 230, 230, 230, 230, 230, 230 ; 78 
	FCB  25,  25,  25,  57, 132, 152, 172, 192, 212, 230, 230, 230, 230, 230, 230, 230, 230 ; 82 
	FCB  25,  25,  25,  49, 124, 144, 164, 184, 204, 224, 224, 224, 224, 224, 224, 224, 224 ; 86      
	FCB  25,  25,  25,  41, 116, 136, 156, 176, 196, 216, 216, 216, 216, 216, 216, 216, 216 ; 90 
	FCB  25,  25,  25,  33, 108, 128, 148, 168, 188, 208, 208, 208, 208, 208, 208, 208, 208 ; 94 
	FCB  25,  25,  25,  25, 100, 120, 140, 160, 180, 200, 200, 200, 200, 200, 200, 200, 200 ; 98 
	FCB  25,  25,  25,  25, 100, 120, 140, 160, 180, 200, 200, 200, 200, 200, 200, 200, 200 ;102 
	FCB  25,  25,  25,  25, 100, 120, 140, 160, 180, 200, 200, 200, 200, 200, 200, 200, 200 ;106 
	FCB  25,  25,  25,  25, 100, 120, 140, 160, 180, 200, 200, 200, 200, 200, 200, 200, 200 ;110
	FCB  25,  25,  25,  25, 100, 120, 140, 160, 180, 200, 200, 200, 200, 200, 200, 200, 200 ;114
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; -3->2 sol. DC correction vs. trans temp
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;	% corr = 
;	(val - 128)/128		;Deg C
;
L5FB4   FCB   96		;-40
	FCB   96		;-28
	FCB   96		;-16
	FCB   100		;- 4
	FCB   105		;  8
	FCB   110		; 20
	FCB   120		; 32
	FCB   123		; 44
	FCB   128		; 56
	FCB   128		; 68
	FCB   128		; 80
	FCB   128		; 92
	FCB   128		;104
	FCB   128		;116
	FCB   128		;128
	FCB   128		;140
	FCB   128		;152
;
L5FC5	FCB 0			;0 secs, min elapsed shift time for adaptive modifier update
;  
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Shift duration threshold to flag excessive trans slip
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;  
;	secs = val/40		;Shift #
;                  
L5FC6   FCB 48			;1 -> 2 
	FCB 48			;2 -> 3   
	FCB 48			;3 -> 4   
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Jump shift compensation params
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
L5FC9   FCB 40			;1 sec, time to check for jump shift from 1st
L5FCA   FCB 40			;1 sec, time to check for jump shift from 2nd
;
L5FCB   FCB 80			;2 secs, time duration for 1->3 jump shift compensation
L5FCC   FCB 20			;20 PSI, added pressure for 1->3 jump shift
;
L5FCD   FCB 80			;2 secs, time duration for 1->4 jump shift compensation  
L5FCE   FCB 20			;20 PSI, added pressure for 1->4 jump shift   
;
L5FCF	FCB 80			;2 secs, time duration for 2->4 jump shift
L5FD0   FCB 10			;10 PSI, added pressure for 2->4 jump shift 
;  
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Derivative RPM threshold for shift to have started
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;	singed derivative	;Gear shift
;
L5FD1   FDB 65475		;1 -> 2        
	FDB 65416		;2 -> 3 
	FDB 65400		;3 -> 4   
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Derivative RPM threshold for shift to be complete
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;	signed derivative	;Gear shift
;
L5FD7   FDB 65475		;1 -> 2       
	FDB 65416		;2 -> 3  
	FDB 65400		;3 -> 4 
;
;-------------------------------
; GARAGE SHIFT PARAM'S
;-------------------------------
;
L5FDD   FCB 8			;0.2 secs, time to add pressure for shift into forward range
L5FDE   FCB 0			;0 PSI, added momentary line pressure for shifting into 
				;forward range 
;
L5FDF   FCB 10			;0.2 secs, time to add pressure for shift into reverse
L5FE0   FCB 0			;0 PSI, added momentary line pressure for shifting into 
				;reverse   
;
L5FE1   FCB 16			;8 MPH, upper MPH threshold for low speed line pressure 
				;handling (P/N) 
L5FE2   FCB 16			;8 MPH, upper MPH threshold for low speed line pressure 
				;handling (drive) 
;
L5FE3   FCB 5			;5 PSI, added low speed line pressure for P/N   
L5FE4   FCB 5			;5 PSI, added low speed line pressure for drive  
L5FE5   FCB 5			;5 PSI, added low speed line pressure for reverse
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Max time alloted for trans downshift
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
L5FE6	FCB 80			;2 secs, max time for 2 -> 1
        FCB 80			;2 secs, max time for 3 -> 2
        FCB 0			;0 secs, max time for 4 -> 3 
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Line pressure offset for A/C on
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;	PSI = -val.		;Gear #
;
L5FE9	FCB 0			;1st 
L5FEA   FCB 3			;2nd   
L5FEB   FCB 5			;3rd   
L5FEC	FCB 5			;4th
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Adaptive modifer cell limits to change cells
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;-Defines TPS range to change from one cell to another
;
L5FED   FCB 64			;25%, upper TPS limit for cell #1  
L5FEE	FCB 59			;23%, lower TPS limit for cell #1
;
L5FEF	FCB 102			;40%, upper TPS limit for cell #2
L5FF0	FCB 97			;38%, lower TPS limit for cell #2 
;
L5FF1	FCB 178			;69.8%, upper TPS limit for cell #3 
L5FF2	FCB 173			;68.0%, lower TPS limit for cell #3
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Temp threshold to enable adaptive modifier
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; 
L5FF3   FCB 133			;60 deg C, lower temp limit
L5FF4   FCB 200			;110 deg C, upper temp limit
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Lower TPS limit for adaptive modifier updates
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;      
;	% TPS = val x .391	;Gear #
;   
L5FF5   FCB 64			;1st 
        FCB 255			;2nd 
        FCB 255			;3rd 
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Lower TPS limit for adaptive modifier updates
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;      
;	% TPS = val x .391	;Gear #
;       
L5FF8   FCB 255			;1st 
        FCB 0			;2nd   
        FCB 0			;3rd  
;      
L5FFB   FCB 13			;5%, max change in TPS for adaptive modifier updates
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Adaptive modifier limits
;~~~~~~~~~~~~~~~~~~~~~~~~~~
;  
L5FFC   FCB 236			;-5 PSI, lower limit
L5FFD   FCB 40			;10 PSI, upper limit
  
L5FFE   FCB 80			;2 secs, delay for engine braking after shift
L5FFF   FDB 65526		;-1.25 RPM, TCC slip threshold to enable engine braking
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Break points used for temp based press. comp. tables
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; Used with tables L624B, 
; L627A, and L62A9
;
;	%TPS = val/.390		;Table offset		
;
L6001   FCB 0			; 0
L6002	FCB 38			;16
L6003   FCB 102			;32
L6004   FCB 178			;48
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Min time between shifts for adaptive modifier updates
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;	Time = val/40		;Shift #
;
L6005   FCB 80			;1 -> 2 
        FCB 80			;2 -> 3   
        FCB 80			;3 -> 4 
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Min time after change in selected range to allow adpative modifier updates
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;	Time = val/40		;Shift #
;
L6008   FCB 240			;1 -> 2
        FCB 240			;2 -> 3 
        FCB 240			;3 -> 4 
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Max change in vehicle speed for adaptive modifier updates
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;	MPH = val/2		;Gear shift
;
L600B   FCB 40			;1 -> 2
        FCB 40			;2 -> 3 
        FCB 0			;3 -> 4 
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Threshold for large shift error corrections
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;	msec = val x 25		;Shift 
;
L600E	FCB 244			;1 -> 2 
        FCB 244			;2 -> 3
        FCB 244			;3 -> 4
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Adaptive modifier proportional term for large shift error
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;	added PSI/shift = val/4 ;Shift
;
L6011	FCB 8			;1 -> 2
        FCB 8			;2 -> 3  
        FCB 8			;3 -> 4
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Flare time threshold to compensate for shift flare
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;	msec = val x 25		;Shift
;
L6014   FCB 32			;1 -> 2
        FCB 0			;2 -> 3   
        FCB 0			;3 -> 4   
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Min shift time for adaptive modifier updates
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;	msec = val x 25		;Shift
;
L6017	FCB 20			;1 -> 2  
        FCB 20			;2 -> 3       
        FCB 255			;3 -> 4
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Adaptive mod'r proportional term for compensating for shift flare
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;	added PSI/shift = val/4	;Shift
;
L601A   FCB 12			;1 -> 2     
        FCB 16			;2 -> 3
        FCB 0			;3 -> 4
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Adaptive modifier option byte
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
L601D   FCB 1			; b7=
				; b6=
				; b5=
				; b4=
				; b3=
				; b2=
				; b1=
				; b0, 1 = Shift flare compensation disabled
;
L601E   FCB 90			;90 PSI, max allowed added line pressure
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Max time to modulate line pressure for shift
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;	secs = val/40		;Shift
;
L601F   FCB 48			;1 -> 2 
	FCB 48			;2 -> 3  
	FCB 48			;3 -> 4  
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Added power enrich line pressure vs. input speed
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;	PSI = (val - 128)	;RPM
;
L6022   FCB 	128		;   0
	FCB 	128		; 512 
	FCB 	128		;1024 
	FCB 	128		;1536 
	FCB 	128		;2048 
	FCB 	128		;2560
	FCB 	128		;3072 
	FCB 	128		;3584 
	FCB 	128		;4096 
	FCB 	128		;4608 
	FCB 	128		;5120
	FCB 	128		;5632 
	FCB 	128		;6144 
	FCB 	128		;6656 
	FCB 	128		;7168 
	FCB 	128		;7680
	FCB 	128		;8192 
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Line pressure offset vs. MAT
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;	PSI = val - 128		;Deg C
;
L6033   FCB 	128		;-4
	FCB     128		; 2
	FCB     128		; 8
	FCB     128		;14
	FCB     128		;20
	FCB     128		;26
	FCB     128		;32
	FCB     128		;38
	FCB     128		;44
	FCB     128		;50
	FCB     128		;56
	FCB     128		;62
	FCB     128		;68
	FCB     128		;74
	FCB     128		;80
	FCB     128		;86
	FCB     128		;92
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Desired 1 -> 2 shift time baro gain vs. TPS
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;	gain = val/64		;%TPS	
;
L6044   FCB  0			;  0.0
	FCB  0			;  6.3
	FCB  0			; 12.5
	FCB  0			; 18.8
	FCB  0			; 25.0
	FCB  0			; 31.3
	FCB  0			; 37.5
	FCB  0			; 43.8
	FCB  0			; 50.0
	FCB  0			; 56.3
	FCB  0			; 62.5
	FCB  0			; 68.8
	FCB  0			; 75.0
	FCB  0			; 81.3
	FCB  0			; 87.5
	FCB  0			; 93.8
	FCB  0			;100.0
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Desired 2 -> 3 shift time baro gain vs. TPS
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;	gain = val/64		;%TPS	
;
L6055   FCB  0			;  0.0
	FCB  0			;  6.3
	FCB  0			; 12.5
	FCB  0			; 18.8
	FCB  0			; 25.0
	FCB  0			; 31.3
	FCB  0			; 37.5
	FCB  0			; 43.8
	FCB  0			; 50.0
	FCB  0			; 56.3
	FCB  0			; 62.5
	FCB  0			; 68.8
	FCB  0			; 75.0
	FCB  0			; 81.3
	FCB  0			; 87.5
	FCB  0			; 93.8
	FCB  0			;100.0
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Desired 3 -> 4 shift time baro gain vs. TPS
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;	gain = val/64		;%TPS	
;
L6066   FCB  0			;  0.0
	FCB  0			;  6.3
	FCB  0			; 12.5
	FCB  0			; 18.8
	FCB  0			; 25.0
	FCB  0			; 31.3
	FCB  0			; 37.5
	FCB  0			; 43.8
	FCB  0			; 50.0
	FCB  0			; 56.3
	FCB  0			; 62.5
	FCB  0			; 68.8
	FCB  0			; 75.0
	FCB  0			; 81.3
	FCB  0			; 87.5
	FCB  0			; 93.8
	FCB  0			;100.0
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Baro gain factor for line pressure baro compensation vs. TPS (2nd gear)
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;	gain = val/64		;%TPS
;
L6077   FCB 0			;  0.0
	FCB 0			;  6.3
	FCB 0			; 12.5
	FCB 30			; 18.8
	FCB 38			; 25.0
	FCB 43			; 31.3
	FCB 49			; 37.5
	FCB 54			; 43.8
	FCB 58			; 50.0
	FCB 64			; 56.3
	FCB 64			; 62.5
	FCB 64			; 68.8
	FCB 70			; 75.0
	FCB 77			; 81.3
	FCB 77			; 87.5
	FCB 77			; 93.8
	FCB 77			;100.0
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Baro gain factor for line pressure baro compensation vs. TPS (3rd gear)
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;	gain = val/64		;%TPS
;
L6088   FCB  0			;  0.0
	FCB  0			;  6.3
	FCB  0			; 12.5
	FCB  26			; 18.8
	FCB  32			; 25.0
	FCB  38			; 31.3
	FCB  45			; 37.5
	FCB  51			; 43.8
	FCB  55			; 50.0
	FCB  58			; 56.3
	FCB  68			; 62.5
	FCB  68			; 68.8
	FCB  68			; 75.0
	FCB  72			; 81.3
	FCB  77			; 87.5
	FCB  77			; 93.8
	FCB  77			;100.0
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Baro gain factor for line pressure baro compensation vs. TPS (4th gear)
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;	gain = val/64		;%TPS
;
L6099   FCB  	0		;  0.0
	FCB  	0		;  6.3
	FCB  	0		; 12.5
	FCB  	0		; 18.8
	FCB  	0		; 25.0
	FCB  	0		; 31.3
	FCB  	0		; 37.5
	FCB  	0		; 43.8
	FCB  	0		; 50.0
	FCB  	0		; 56.3
	FCB  	0		; 62.5
	FCB  	0		; 68.8
	FCB  	0		; 75.0
	FCB  	0		; 81.3
	FCB  	0		; 87.5
	FCB  	0		; 93.8
	FCB  	0		;100.0
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Engine braking line pressure vs. MPH (1st gear)
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;	PSI = val		;MPH
;
L60AA   FCB 	40		;  0
	FCB 	40		;  8
	FCB 	40		; 16
	FCB 	40		; 24
	FCB 	40		; 32
	FCB 	40		; 40
	FCB 	40		; 48
	FCB 	90		; 56
	FCB 	90		; 64
	FCB 	90		; 72
	FCB 	90		; 80
	FCB 	90		; 88
	FCB 	90		; 96
	FCB 	90		;104
	FCB 	90		;112
	FCB 	90		;120
	FCB 	90		;128
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Engine braking line pressure vs. MPH (2nd gear)
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;	PSI = val		;MPH
;
L60BB   FCB    40		;  0
	FCB    40		;  8
	FCB    40		; 16
	FCB    40		; 24
	FCB    40		; 32
	FCB    40		; 40
	FCB    50		; 48
	FCB    60		; 56
	FCB    80		; 64
	FCB    90		; 72
	FCB    90		; 80
	FCB    90		; 88
	FCB    90		; 96
	FCB    90		;104
	FCB    90		;112
	FCB    90		;120
	FCB    90		;128
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Engine braking line pressure vs. MPH (3rd gear)
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;	PSI = val		;MPH
;
L60CC   FCB    0		;  0
	FCB    0		;  8
	FCB    0		; 16
	FCB    0		; 24
	FCB    0		; 32
	FCB    0		; 40
	FCB    0		; 48
	FCB    0		; 56
	FCB    0		; 64
	FCB    0		; 72
	FCB    0		; 80
	FCB    0		; 88
	FCB    0		; 96
	FCB    0		;104
	FCB    0		;112
	FCB    0		;120
	FCB    0		;128
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Cool temp compensation line pressure for shift into forward range
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;	PSI = val
;
L60DD   FCB   0			;-40 deg C, min temp
        FCB   0			;0 RPM, min RPM
        FCB   17		;# cols/row
;
;   TOS RPM:   0  128  256  512  384  640  768  896 1024 1152 1280 1408 1536 1664 1792 1920 2048 :Deg C    
;
	FCB   25,  25,  25,  25,  25,  25,  25,  25,  25,  25,  25,  25,  25,  25,  25,  25,  25 ;-40
	FCB   20,  20,  20,  20,  20,  20,  20,  20,  20,  20,  20,  20,  20,  20,  20,  20,  20 ;-20
	FCB   10,  10,  10,  10,  10,  10,  10,  10,  10,  10,  10,  10,  10,  10,  10,  10,  10 ;  0
	FCB    0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0 ; 20
	FCB    0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0 ; 40
	FCB    0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0 ; 60
	FCB    0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0 ; 80
	FCB    0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0 ;100
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Cool temp compensation line pressure for shift into forward range
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;	PSI = val
;
L6168   FCB   0			;-40 deg C, min temp
        FCB   0			;0 RPM, min RPM
        FCB   17		;# cols/row
;
;   TOS RPM:   0  128  256  512  384  640  768  896 1024 1152 1280 1408 1536 1664 1792 1920 2048 :Deg C    
;
	FCB   25,  25,  25,  25,  25,  25,  25,  25,  25,  25,  25,  25,  25,  25,  25,  25,  25 ;-40
	FCB   25,  25,  25,  25,  25,  25,  25,  25,  25,  25,  25,  25,  25,  25,  25,  25,  25 ;-20
	FCB    5,   5,   5,   5,   5,   5,   5,   5,   5,   5,   5,   5,   5,   5,   5,   5,   5 ;  0
	FCB    0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0 ; 20
	FCB    0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0 ; 40
	FCB    0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0 ; 60
	FCB    0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0 ; 80
	FCB    0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0 ;100
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Low speed line pressure vs. MPH and TPS
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;	PSI = val
;
L61F3   FCB   0			;0%, min %TPS
        FCB   0			;0 MPH, min speed
        FCB   5			;# cols/row
;
;       MPH:   0     4     8    12    16 ;%TPS
;
	FCB  128,  128,  138,  148,  148 ;  0.00
	FCB  128,  133,  138,  138,  138 ;  6.25 
	FCB  128,  138,  138,  138,  138 ; 12.50
	FCB  128,  160,  160,  160,  160 ; 18.75 
	FCB  160,  160,  160,  160,  160 ; 25.00 
	FCB  255,  255,  255,  255,  255 ; 31.25 
	FCB  255,  255,  255,  255,  255 ; 37.50
	FCB  255,  255,  255,  255,  255 ; 43.75
	FCB  255,  255,  255,  255,  255 ; 50.00
	FCB  255,  255,  255,  255,  255 ; 56.25
	FCB  255,  255,  255,  255,  255 ; 62.50
	FCB  255,  255,  255,  255,  255 ; 68.75
	FCB  255,  255,  255,  255,  255 ; 75.00
	FCB  255,  255,  255,  255,  255 ; 81.25
	FCB  255,  255,  255,  255,  255 ; 87.50
	FCB  255,  255,  255,  255,  255 ; 93.75
	FCB  255,  255,  255,  255,  255 ;100.00 
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Trans temp based line pressure correction, 2nd gear
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; Line press corr = val - 128
;
L624B   FCB   0     	;0%, min %TPS
        FCB   0     	;-40 deg C, min trans temp 
        FCB   11    	;# cols.
;
;    Deg C: -40, -20,   0,  20,  40,  60,  80, 100, 120, 140, 160 ;%TPS
;
				;
	FCB 133, 128, 128, 128, 125, 125, 128, 128, 130, 131, 134 ; 0
	FCB 138, 133, 128, 128, 125, 125, 128, 128, 130, 131, 134 ;38
	FCB 138, 133, 133, 128, 128, 128, 128, 128, 130, 134, 140 ;40 
	FCB 138, 133, 133, 133, 133, 130, 128, 128, 134, 134, 140 ;70   
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Trans temp based line pressure correction, 3rd gear
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; Line press corr = val - 128
;
L627A   FCB   0     	;0%, min %TPS
        FCB   0     	;-40 deg C, min trans temp 
        FCB   11    	;# cols.
;
;    Deg C: -40, -20,   0,  20,  40,  60,  80, 100, 120, 140, 160 ;%TPS
;
	FCB 153, 148, 148, 143, 128, 128, 128, 128, 131, 131, 134 ; 0
	FCB 153, 148, 148, 143, 128, 128, 128, 128, 131, 131, 134 ;38
	FCB 153, 148, 148, 143, 128, 128, 128, 128, 134, 134, 140 ;40         
	FCB 153, 148, 148, 143, 128, 128, 128, 128, 134, 134, 140 ;70
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Trans temp based line pressure correction, 4th gear
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; Line press corr = val - 128
;
L62A9   FCB   0     	;0 %TPS, min %TPS
        FCB   0     	;-40 deg C, min trans temp 
        FCB   11    	;# cols.
;
;    Deg C: -40, -20,   0,  20,  40,  60,  80, 100, 120, 140, 160 ;%TPS
;
	FCB 133, 128, 128, 128, 128, 128, 128, 128, 128, 131, 134 ; 0
	FCB 138, 133, 128, 128, 128, 128, 128, 128, 128, 131, 134 ;38
	FCB 138, 133, 133, 128, 128, 128, 128, 128, 128, 131, 134 ;40
	FCB 138, 133, 133, 128, 128, 128, 128, 128, 128, 131, 134 ;70
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Added line pressure vs.load, 0-64 MPH
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; psi=val.
;						
L62D8   FCB   0		;0%, min TPS
        FCB   0		;0 mph, min mph x 4
        FCB   17	;# cols.
;
;	MPH:  0    4    8   12   16   20   24   28   32   36   40   44   48   52   56   60   64 ; %TPS	
;
	FCB   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0	;  0.00  
	FCB  12,  12,  12,  12,  12,  12,  12,  12,  12,  12,  12,  12,  12,  12,  12,  12,  12 ;  6.25     
	FCB  20,  20,  20,  20,  20,  20,  20,  20,  20,  20,  20,  20,  20,  20,  20,  20,  20 ; 12.50     
	FCB  27,  27,  27,  27,  27,  27,  27,  27,  27,  27,  27,  27,  27,  27,  27,  27,  27 ; 18.75     
	FCB  32,  32,  32,  32,  32,  32,  32,  32,  32,  32,  32,  32,  32,  32,  32,  32,  32 ; 25.00
	FCB  37,  37,  37,  37,  37,  37,  37,  37,  37,  37,  37,  37,  37,  37,  37,  37,  37 ; 31.25
	FCB  42,  42,  42,  42,  42,  42,  42,  42,  42,  42,  42,  42,  42,  42,  42,  42,  42 ; 37.50 
	FCB  46,  46,  46,  46,  46,  46,  46,  46,  46,  46,  46,  46,  46,  46,  46,  46,  46 ; 43.75  
	FCB  90,  90,  49,  49,  49,  49,  49,  49,  49,  49,  49,  49,  49,  49,  49,  49,  49 ; 50.00
	FCB  90,  90,  52,  52,  52,  52,  52,  52,  52,  52,  52,  52,  52,  52,  52,  52,  52 ; 56.25
	FCB  90,  90,  54,  54,  54,  54,  54,  54,  54,  54,  54,  54,  54,  54,  54,  54,  54 ; 62.50 
	FCB  90,  90,  56,  56,  56,  56,  56,  56,  56,  56,  56,  56,  56,  56,  56,  56,  56 ; 68.75  
	FCB  90,  90,  58,  58,  58,  58,  58,  58,  58,  58,  58,  58,  58,  58,  58,  58,  58 ; 70.00
 	FCB  90,  90,  59,  59,  59,  59,  59,  59,  59,  59,  59,  59,  59,  59,  59,  59,  59 ; 81.25  
	FCB  90,  90,  59,  59,  59,  59,  59,  59,  59,  59,  59,  59,  59,  59,  59,  59,  59 ; 87.50
	FCB  90,  90,  59,  59,  59,  59,  59,  59,  59,  59,  59,  59,  59,  59,  59,  59,  59 ; 93.75  
	FCB  90,  90,  59,  59,  59,  59,  59,  59,  59,  59,  59,  59,  59,  59,  59,  59,  59 ;100.00
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Added line pressure vs.load, 64 MPH +
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; psi=val.
;
L63FC	FCB   0		;0%, min %tps
        FCB   0		;64 mph, min (mph - 64) x 4
        FCB   17	;# cols
;
;	MPH:  64  68   72   76   80   84   88   92   96  100  104  108  112  116  120  124  128 ; %TPS
;
	FCB   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0	;  0.00
	FCB  12,  12,  12,  12,  12,  12,  12,  12,  12,  12,  12,  12,  12,  12,  12,  12,  12 ;  6.25 
	FCB  20,  20,  20,  20,  20,  20,  20,  20,  20,  20,  20,  20,  20,  20,  20,  20,  20 ; 12.50  
	FCB  27,  27,  27,  27,  27,  27,  27,  27,  27,  27,  27,  27,  27,  27,  27,  27,  27 ; 18.75 
	FCB  32,  32,  32,  32,  32,  32,  32,  32,  32,  32,  32,  32,  32,  32,  32,  32,  32 ; 25.00
	FCB  37,  37,  37,  37,  37,  37,  37,  37,  37,  37,  37,  37,  37,  37,  37,  37,  37 ; 31.25
	FCB  42,  42,  42,  42,  42,  42,  42,  42,  42,  42,  42,  42,  42,  42,  42,  42,  42 ; 37.50 
	FCB  46,  46,  46,  46,  46,  46,  46,  46,  46,  46,  46,  46,  46,  46,  46,  46,  46 ; 43.75	
	FCB  49,  49,  49,  49,  49,  49,  49,  49,  49,  49,  49,  49,  49,  49,  49,  49,  49 ; 50.00  
	FCB  52,  52,  52,  52,  52,  52,  52,  52,  52,  52,  52,  52,  52,  52,  52,  52,  52 ; 56.25
	FCB  54,  54,  54,  54,  54,  54,  54,  54,  54,  54,  54,  54,  54,  54,  54,  54,  54 ; 62.50
	FCB  56,  56,  56,  56,  56,  56,  56,  56,  56,  56,  56,  56,  56,  56,  56,  56,  56 ; 68.75 
	FCB  58,  58,  58,  58,  58,  58,  58,  58,  58,  58,  58,  58,  58,  58,  58,  58,  58 ; 75.00  
	FCB  59,  59,  59,  59,  59,  59,  59,  59,  59,  59,  59,  59,  59,  59,  59,  59,  59 ; 81.25
	FCB  59,  59,  59,  59,  59,  59,  59,  59,  59,  59,  59,  59,  59,  59,  59,  59,  59 ; 87.25
	FCB  59,  59,  59,  59,  59,  59,  59,  59,  59,  59,  59,  59,  59,  59,  59,  59,  59 ; 93.75
	FCB  59,  59,  59,  59,  59,  59,  59,  59,  59,  59,  59,  59,  59,  59,  59,  59,  59 ;100.00 
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-2nd gear load based pressure offset
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;	line pressure = val - 128	;%TPS
;
L6520   FCB  128			;  0.0
	FCB  120    			;  6.3
	FCB  116    			; 12.5
	FCB  113    			; 18.8
	FCB  113    			; 25.0
	FCB  112    			; 31.3
	FCB  112    			; 37.5
	FCB  113   			; 43.8
	FCB  115    			; 50.0
	FCB  118    			; 56.3
	FCB  119    			; 62.5
	FCB  119    			; 68.8
	FCB  119    			; 75.0
	FCB  119    			; 81.3
	FCB  119    			; 87.5
	FCB  119    			; 93.8
	FCB  119    			;100.0
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-3rd gear load based pressure offset
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;	line pressure = val - 128	;%TPS
;
L6531   FCB  128    			;  0.0
	FCB  123    			;  6.3
	FCB  119    			; 12.5
	FCB  118    			; 18.8
	FCB  117    			; 25.0
	FCB  117    			; 31.3
	FCB  117    			; 37.5
	FCB  121    			; 43.8
	FCB  124    			; 50.0
	FCB  129    			; 56.3
	FCB  129    			; 62.5
	FCB  129    			; 68.8
	FCB  129    			; 75.0
	FCB  129    			; 81.3
	FCB  129    			; 87.5
	FCB  129    			; 93.8
	FCB  129    			;100.0
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-4th gear load based pressure offset
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;	line pressure = val - 128	;%TPS
;
L6542   FCB  128			;  0.0
	FCB  128			;  6.3
	FCB  128			; 12.5
	FCB  128			; 18.8
	FCB  128			; 25.0
	FCB  128			; 31.3
	FCB  128			; 37.5
	FCB  128			; 43.8
	FCB  128			; 50.0
	FCB  128			; 56.3
	FCB  128			; 62.5
	FCB  128			; 68.8
	FCB  128			; 75.0
	FCB  128			; 81.3
	FCB  128			; 87.5
	FCB  128			; 93.8
	FCB  128			;100.0
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Line pressure modifier for downshift (2 -> 1)
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
L6553   FCB 0			;0 MPH, offset for lookup
	FCB 128			;64 MPH, upper limit for lookup
	FCB 32			;4 MPH per line
;
;	PSI = val - 128		;MPH
;
L6556   FCB  128		; 0
	FCB  128		; 4
	FCB  128		; 8
	FCB  128		;12
	FCB  128		;16
	FCB  128		;20
	FCB  128		;24
	FCB  128		;28
	FCB  128		;32
	FCB  128		;36
	FCB  128		;40
	FCB  128		;44
	FCB  128		;48
	FCB  128		;52
	FCB  128		;56
	FCB  128		;60
	FCB  128		;64
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Line pressure modifier for downshift (2 -> 1)
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
L6567   FCB 28			;14 MPH, offset for lookup
	FCB 128			;(128 + 28) x .5 = 78 MPH, upper limit for lookup
	FCB 32			;4 MPH per line
;
;	PSI = val - 128		;MPH
;
L656A   FCB  137		;14    
	FCB  137		;18
	FCB  135		;22
	FCB  123		;26
	FCB  112		;30
	FCB  106		;34
	FCB  106		;38
	FCB  106		;42
	FCB  106		;46
	FCB  106		;50
	FCB  106		;54
	FCB  106		;58
	FCB  106		;62
	FCB  128		;66
	FCB  128		;70
	FCB  128		;74
	FCB  128		;78
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Line pressure modifier for downshift (4 -> 3)
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
L657B   FCB 60			;30 MPH, offset for lookup
	FCB 128			;(128 + 60) x .5 = 94 MPH, upper limit for lookup
	FCB 32			;4 MPH per line
;
;	PSI = val - 128		;MPH
;
L657E   FCB 128			;30
	FCB 128			;34
	FCB 128 		;38
	FCB 128			;42
	FCB 128			;46
	FCB 128			;50
	FCB 128			;54
	FCB 128			;58
	FCB 128			;62
	FCB 128			;66
	FCB 128			;70
	FCB 128			;74
	FCB 128			;78
	FCB 128			;82
	FCB 128			;86
	FCB 128			;90
	FCB 128			;94
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Desired 1 -> 2 shift time vs. TPS
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;	msec = val x 25		;%TPS
;
L658F   FCB  34			;  0.0
	FCB  34			;  6.3
	FCB  29			; 12.5
	FCB  30			; 18.8
	FCB  33			; 25.0
	FCB  35			; 31.3
	FCB  35			; 37.5
	FCB  33			; 43.8
	FCB  33			; 50.0
	FCB  35			; 56.3
	FCB  35			; 62.5
	FCB  34			; 68.8
	FCB  35			; 75.0
	FCB  35			; 81.3
	FCB  36			; 87.5
	FCB  37			; 93.8
	FCB  37			;100.0
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Desired 2 -> 3 shift time vs. TPS
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;	msec = val x 25		;%TPS
;
L65A0   FCB  32			;  0.0
	FCB  32			;  6.3
	FCB  32			; 12.5
	FCB  34			; 18.8
	FCB  34			; 25.0
	FCB  34			; 31.3
	FCB  34			; 37.5
	FCB  34			; 43.8
	FCB  34			; 50.0
	FCB  38			; 56.3
	FCB  38			; 62.5
	FCB  38			; 68.8
	FCB  38			; 75.0
	FCB  40			; 81.3
	FCB  40			; 87.5
	FCB  40			; 93.8
	FCB  40			;100.0
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Desired 3 -> 4 shift time vs. TPS
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;	msec = val x 25		;%TPS
;
L65B1   FCB  32     		;  0.0
	FCB  32     		;  6.3
	FCB  32     		; 12.5
	FCB  32     		; 18.8
	FCB  32     		; 25.0
	FCB  32     		; 31.3
	FCB  32   		; 37.5
	FCB  32     		; 43.8
	FCB  32     		; 50.0
	FCB  32     		; 56.3
	FCB  32     		; 62.5
	FCB  32     		; 68.8
	FCB  32     		; 75.0
	FCB  32     		; 81.3
	FCB  32     		; 87.5
	FCB  32     		; 93.8
	FCB  32     		;100.0
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Adaptive modifier 1 -> 2 proportional term vs. shift error
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;	added PSI/shift 
;	= signed value/4	;msecs 
;
L65C2   FCB 8			;-200
	FCB 6			;-175
	FCB 5			;-150
	FCB 4			;-125
	FCB 3			;-100
	FCB 2			; -75
	FCB 1			; -50
	FCB 0			; -25
	FCB 0			;   0
	FCB 0			;  25
	FCB 255			;  50
	FCB 254			;  75
	FCB 253			; 100
	FCB 252			; 125
	FCB 251			; 150
	FCB 250			; 175
	FCB 248			; 200
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Adaptive modifier 2 -> 3 proportional term vs. shift error
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;	added PSI/shift 
;	= signed value/4	;msecs 
;
L65D3   FCB 8			;-200
	FCB 6			;-175
	FCB 5			;-150
	FCB 4			;-125
	FCB 3			;-100
	FCB 2			; -75
	FCB 1			; -50
	FCB 0			; -25
	FCB 0			;   0
	FCB 0			;  25
	FCB 255			;  50
	FCB 254			;  75
	FCB 253			; 100
	FCB 252			; 125
	FCB 251			; 150
	FCB 250			; 175
	FCB 248			; 200
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Adaptive modifier 3 -> 4 proportional term vs. shift error
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;	added PSI/shift 
;	= signed value/4	;msecs 
;
L65E4   FCB 8			;-200
	FCB 6			;-175
	FCB 5			;-150
	FCB 4			;-125
	FCB 3			;-100
	FCB 2			; -75
	FCB 1			; -50
	FCB 0			; -25
	FCB 0			;   0
	FCB 0			;  25
	FCB 255			;  50
	FCB 254			;  75
	FCB 253			; 100
	FCB 252			; 125
	FCB 251			; 150
	FCB 250			; 175
	FCB 248			; 200
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-2nd gear pressure offset vs. %TPS
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;	PSI = val - 128		;%TPS
;
L65F5   FCB  128    		;  0.0
	FCB  128    		;  6.3
	FCB  128    		; 12.5
	FCB  128    		; 18.8
	FCB  128    		; 25.0
	FCB  128    		; 31.3
	FCB  128    		; 37.5
	FCB  128    		; 43.8
	FCB  128    		; 50.0
	FCB  128    		; 56.3
	FCB  128    		; 62.5
	FCB  128    		; 68.8
	FCB  128    		; 75.0
	FCB  128    		; 81.3
	FCB  128    		; 87.5
	FCB  128    		; 93.8
	FCB  128    		;100.0
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-3rd gear pressure offset vs. %TPS
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;	PSI = val - 128		;%TPS
;
L6606   FCB  128    		;  0.0
	FCB  128    		;  6.3
	FCB  128    		; 12.5
	FCB  128    		; 18.8
	FCB  128    		; 25.0
	FCB  128    		; 31.3
	FCB  128    		; 37.5
	FCB  128    		; 43.8
	FCB  128    		; 50.0
	FCB  128    		; 56.3
	FCB  128    		; 62.5
	FCB  128    		; 68.8
	FCB  128    		; 75.0
	FCB  128    		; 81.3
	FCB  128    		; 87.5
	FCB  128    		; 93.8
	FCB  128    		;100.0
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-4th gear pressure offset vs. %TPS
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;	PSI = val - 128		;%TPS
;   			
L6617   FCB  128    		;  0.0
	FCB  128    		;  6.3
	FCB  128    		; 12.5
	FCB  128    		; 18.8
	FCB  128    		; 25.0
	FCB  128    		; 31.3
	FCB  128    		; 37.5
	FCB  128    		; 43.8
	FCB  128    		; 50.0
	FCB  128    		; 56.3
	FCB  128    		; 62.5
	FCB  128    		; 68.8
	FCB  128    		; 75.0
	FCB  128    		; 81.3
	FCB  128    		; 87.5
	FCB  128    		; 93.8
	FCB  128    		;100.0
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-1 -> 2 pressure ramp delay time vs %TPS
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;	Sec = val/40		;%TPS
;
L6628	FCB  0			;  0.0
	FCB  0			;  6.3
	FCB  0			; 12.5
	FCB  0			; 18.8
	FCB  0			; 25.0
	FCB  0			; 31.3
	FCB  18			; 37.5
	FCB  18			; 43.8
	FCB  18			; 50.0
	FCB  18			; 56.3
	FCB  18			; 62.5
	FCB  18			; 68.8
	FCB  18			; 75.0
	FCB  18			; 81.3
	FCB  18			; 87.5
	FCB  18			; 93.8
	FCB  18			;100.0
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-2 -> 3 pressure ramp delay time vs. %TPS
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;	Sec = val/40		;%TPS
;
L6639   FCB  0			;  0.0
	FCB  0			;  6.3
	FCB  0			; 12.5
	FCB  0			; 18.8
	FCB  0			; 25.0
	FCB  0			; 31.3
	FCB  0			; 37.5
	FCB  0			; 43.8
	FCB  0			; 50.0
	FCB  0			; 56.3
	FCB  0			; 62.5
	FCB  0			; 68.8
	FCB  0			; 75.0
	FCB  0			; 81.3
	FCB  0			; 87.5
	FCB  0			; 93.8
	FCB  0			;100.0
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-3 -> 4 pressure ramp delay time vs. %TPS
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;	Sec = val/40		;%TPS
;
L664A   FCB  0			;  0.0
	FCB  0			;  6.3
	FCB  0			; 12.5
	FCB  0			; 18.8
	FCB  0			; 25.0
	FCB  0			; 31.3
	FCB  0			; 37.5
	FCB  0			; 43.8
	FCB  0			; 50.0
	FCB  0			; 56.3
	FCB  0			; 62.5
	FCB  0			; 68.8
	FCB  0			; 75.0
	FCB  0			; 81.3
	FCB  0			; 87.5
	FCB  0			; 93.8
	FCB  0			;100.0
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-1 -> 2 time thresh. to use extended ramping pressure vs %TPS
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;	Sec = val/40		;%TPS
;
L665B   FCB  0			;  0.0
	FCB  0			;  6.3
	FCB  0			; 12.5
	FCB  0			; 18.8
	FCB  0			; 25.0
	FCB  0			; 31.3
	FCB  0			; 37.5
	FCB  0			; 43.8
	FCB  0			; 50.0
	FCB  0			; 56.3
	FCB  0			; 62.5
	FCB  0			; 68.8
	FCB  0			; 75.0
	FCB  0			; 81.3
	FCB  0			; 87.5
	FCB  0			; 93.8
	FCB  0			;100.0
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-2 -> 3 time thresh. to use extended ramping pressure vs %TPS
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;	Sec = val/40		;%TPS
;
L666C   FCB  0			;  0.0
	FCB  0			;  6.3
	FCB  0			; 12.5
	FCB  0			; 18.8
	FCB  0			; 25.0
	FCB  0			; 31.3
	FCB  0			; 37.5
	FCB  0			; 43.8
	FCB  0			; 50.0
	FCB  0			; 56.3
	FCB  0			; 62.5
	FCB  0			; 68.8
	FCB  0			; 75.0
	FCB  0			; 81.3
	FCB  0			; 87.5
	FCB  0			; 93.8
	FCB  0			;100.0
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-3 -> 4 time thresh. to use extended ramping pressure vs %TPS
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;	Sec = val/40		;%TPS
;
L667D   FCB  0			;  0.0
	FCB  0			;  6.3
	FCB  0			; 12.5
	FCB  0			; 18.8
	FCB  0			; 25.0
	FCB  0			; 31.3
	FCB  0			; 37.5
	FCB  0			; 43.8
	FCB  0			; 50.0
	FCB  0			; 56.3
	FCB  0			; 62.5
	FCB  0			; 68.8
	FCB  0			; 75.0
	FCB  0			; 81.3
	FCB  0			; 87.5
	FCB  0			; 93.8
	FCB  0			;100.0
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-1 -> 2 initial ramping pressure vs. TPS
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;	PSI/sec = (val - 128) 
;		  x 1.25	;%TPS
;
L668E   FCB  128    		;  0.0
	FCB  128    		;  6.3
	FCB  128    		; 12.5
	FCB  128    		; 18.8
	FCB  128    		; 25.0
	FCB  128    		; 31.3
	FCB  128    		; 37.5
	FCB  128    		; 43.8
	FCB  128    		; 50.0
	FCB  128    		; 56.3
	FCB  128    		; 62.5
	FCB  128    		; 68.8
	FCB  128    		; 75.0
	FCB  128    		; 81.3
	FCB  128    		; 87.5
	FCB  128    		; 93.8
	FCB  128    		;100.0
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-2 -> 3 initial ramping pressure vs. TPS
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;	PSI/sec = (val - 128) 
;		  x 1.25	;%TPS				
;
L669F   FCB  128		;  0.0   
	FCB  128		;  6.3
	FCB  128		; 12.5
	FCB  128		; 18.8
	FCB  128		; 25.0
	FCB  128		; 31.3
	FCB  128		; 37.5
	FCB  128		; 43.8
	FCB  128		; 50.0
	FCB  128		; 56.3
	FCB  128		; 62.5
	FCB  128		; 68.8
	FCB  128		; 75.0
	FCB  128		; 81.3
	FCB  128		; 87.5
	FCB  128		; 93.8
	FCB  128		;100.0
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-3 -> 4 initial ramping pressure vs. TPS
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;	PSI/sec = (val - 128) 
;		  x 1.25	;%TPS				
;
L66B0   FCB  128		;  0.0
	FCB  128		;  6.3
	FCB  128		; 12.5
	FCB  128		; 18.8
	FCB  128		; 25.0
	FCB  128		; 31.3
	FCB  128		; 37.5
	FCB  128		; 43.8
	FCB  128		; 50.0
	FCB  128		; 56.3
	FCB  128		; 62.5
	FCB  128		; 68.8
	FCB  128		; 75.0
	FCB  128		; 81.3
	FCB  128		; 87.5
	FCB  128		; 93.8
	FCB  128		;100.0
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-1 -> 2 ramping pressure vs. TPS
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;	PSI/sec = (val - 128) 
;		  x 1.25	;%TPS				
;
L66C1   FCB  128		;  0.0
	FCB  128		;  6.3
	FCB  128		; 12.5
	FCB  128		; 18.8
	FCB  128		; 25.0
	FCB  128		; 31.3
	FCB  128		; 37.5
	FCB  128		; 43.8
	FCB  100		; 50.0
	FCB  100		; 56.3
	FCB  90			; 62.5
	FCB  90			; 68.8
	FCB  90			; 75.0
	FCB  85			; 81.3
	FCB  85			; 87.5
	FCB  85			; 93.8
	FCB  85			;100.0
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-2 -> 3 ramping pressure vs. TPS
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;	PSI/sec = (val - 128) 
;		  x 1.25	;%TPS				
;
L66D2   FCB  128		;  0.0
	FCB  128		;  6.3
	FCB  128		; 12.5
	FCB  128		; 18.8
	FCB  128		; 25.0
	FCB  128		; 31.3
	FCB  128		; 37.5
	FCB  128		; 43.8
	FCB  128		; 50.0
	FCB  128		; 56.3
	FCB  128		; 62.5
	FCB  128		; 68.8
	FCB  128		; 75.0
	FCB  128		; 81.3
	FCB  128		; 87.5
	FCB  128		; 93.8
	FCB  128		;100.0
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-3 -> 4 ramping pressure vs. TPS
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;	PSI/sec = (val - 128) 
;		  x 1.25	;%TPS				
;
L66E3   FCB  128		;  0.0
	FCB  128		;  6.3
	FCB  128		; 12.5
	FCB  128		; 18.8
	FCB  128		; 25.0
	FCB  128		; 31.3
	FCB  128		; 37.5
	FCB  128		; 43.8
	FCB  128		; 50.0
	FCB  128		; 56.3
	FCB  128		; 62.5
	FCB  128		; 68.8
	FCB  128		; 75.0
	FCB  128		; 81.3
	FCB  128		; 87.5
	FCB  128		; 93.8
	FCB  128		;100.0
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-1 -> 2 post-shift ramp time vs. %TPS
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;	secs = val/40		;%TPS
;
L66F4   FCB  0			;  0.0
	FCB  0			;  6.3
	FCB  0			; 12.5
	FCB  0			; 18.8
	FCB  0			; 25.0
	FCB  0			; 31.3
	FCB  0			; 37.5
	FCB  0			; 43.8
	FCB  0			; 50.0
	FCB  0			; 56.3
	FCB  0			; 62.5
	FCB  0			; 68.8
	FCB  0			; 75.0
	FCB  0			; 81.3
	FCB  0			; 87.5
	FCB  0			; 93.8
	FCB  0			;100.0
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-2 -> 3 post-shift ramp time vs. %TPS
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;	secs = val/40		;%TPS
;
L6705   FCB  0			;  0.0
	FCB  0			;  6.3
	FCB  0			; 12.5
	FCB  0			; 18.8
	FCB  0			; 25.0
	FCB  0			; 31.3
	FCB  0			; 37.5
	FCB  0			; 43.8
	FCB  0			; 50.0
	FCB  0			; 56.3
	FCB  0			; 62.5
	FCB  0			; 68.8
	FCB  0			; 75.0
	FCB  0			; 81.3
	FCB  0			; 87.5
	FCB  0			; 93.8
	FCB  0			;100.0
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-3 -> 4 post-shift ramp time vs. %TPS
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;	secs = val/40		;%TPS
;
L6716   FCB  0			;  0.0
	FCB  0			;  6.3
	FCB  0			; 12.5
	FCB  0			; 18.8
	FCB  0			; 25.0
	FCB  0			; 31.3
	FCB  0			; 37.5
	FCB  0			; 43.8
	FCB  0			; 50.0
	FCB  0			; 56.3
	FCB  0			; 62.5
	FCB  0			; 68.8
	FCB  0			; 75.0
	FCB  0			; 81.3
	FCB  0			; 87.5
	FCB  0			; 93.8
	FCB  0			;100.0
;
;~~~~~~~~~~~~~~~~~~
;-TCC option flags
;~~~~~~~~~~~~~~~~~~
;

L6727	FCB $6F			; 0110 1111     TCC option byte #1
				; b7 = gradual release
				; b6 = Disable TCC when cold engine temps present
				; b5 = Disable TCC when cold trans temps present
				; b4 = not used	IN MY94, see MY95
				; b3 = Disable TCC when manual 1st/2nd selected
  				; b2 = Prevent repeated TCC locking/unlocking
				; b1 = Disable TCC during rapid decel
				; b0 = Disable TCC during downshift
;
L6728	FCB $1C			; 0001 1100     TCC option byte #2
				; b7 = not used 
				; b6 = not used
				; b5 = not used
				; b4 = Disable TCC during upshift
				; b3 = specific errors turn off TCC
				; b2 = Prevent repeated locking/unlocking during low
				;      TPS
				; b1 = A/C on disables TCC when trans not overheating
				; b0 = Flag excess negative  torque conv. slip
;
;-Cold engine temp thresholds to prevent/disable TCC lockup
;
L6729 	FCB  133		;60 deg C, threshold for TCC to remain unlocked
L672A   FCB  128		;56 deg C, threshold to unlock TCC
;
;-Cold trans temp thresholds to prevent/disable TCC lockup
;
L672B   FCB  80			;20 deg C, threshold for TCC to remain unlocked
L672C   FCB  76			;17 deg C, threshold to unlock TCC
;
;-Trans temp thresholds to flag overheating transmission
;
L672D   FCB  233		;135 deg C, threshold to set flag, hot trans.
L672E   FCB  222		;127 deg C, threshold to clear flag, hot trans.
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-MPH threshold to unlock TCC when A/C is on
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; If the MPH is less and A/C on, TCC will unlock
;
;	MPH = val/2		;Gear #
;
L672F   FCB  0			;2nd
	FCB  0			;3rd 
	FCB  0			;4th   
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-MPH threshold for TCC to remain unlocked when A/C is on
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; If the MPH is less and A/C on, TCC will remain unlocked
;
;	MPH = val/2		;Gear #
;
L6732   FCB  0			;2nd
	FCB  0			;3rd 
	FCB  0			;4th
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Params for enabling/disabling TCC
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
L6735   FCB  222		;127 deg C, trans temp for TCC to unlock when 
				;applicable errors have been set
L6736   FCB  233		;135 deg C, trams temp for TCC to be allowed to lock
				;when applicable errors have been set
;
L6737   FDB  57352		;-1023 RPM, threshold to flag excessive negative
				;TCC slip
L6739   FDB  63936		;-200 RPM, threshold to clear excessive negative TCC
				;slip
;  
L673B   FCB  76			;38 MPH, upper window bound for using low speed TPS
				;thresholds 
L673C   FCB  72			;36 MPH, lower window bound for using low speed TPS
				;thresholds        
L673D   FCB  2			;0.8% TPS, low speed threshold to prevent repeated 
				;locking/unlocking of TCC when TCC being 
				;applied/released
L673E   FCB  4			;1.6% TPS,low speed threshold to prevent repeated 
				;locking/unlocking of TCC when TCC locked/unlocked
L673F   FCB  5			;2% TPS, high speed threshold to prevent repeated
				;locking/unlocking of TCC when TCC being 
				;applied/released
L6440   FCB  7			;2.7% TPS, high speed threshold to prevent repeated
				;locking/unlocking of TCC when TCC locked/unlocked  
;
L6741   FCB  245		;-3.83%/msec, delta TPS threshold to unlock TCC during
				;decel
L6742   FCB  9			;.14 %/msec, delta TPS threshold for TCC to remain
				;locked during accel
;
L6743   FCB  0			;0 secs, time for TCC to reamin unlocked after rapid
				;accel
L6744   FCB  0			;0 secs, time for TCC to remain unlocked after rapid
				;decel
;
L6745   FCB  31			;.75 secs, time to disable TCC during downshift
L6746   FCB  31			;.75 secs, time to disable TCC during upshift 

L6747   FCB  255		;4000%/sec, PWM TCC release rate for rapid accel 
;
;~~~~~~~~~~~~~~~~~~~~~~~
;-Params for TCC timers
;~~~~~~~~~~~~~~~~~~~~~~~
;
L6748   FDB  0			;0 secs, min time for repeated TCC relocks
;
L674A   FDB  0160		;4 secs, time threshold to allow extended relock delays
;
L674C   FDB  0160		;4 secs, period to decrement accumulated TCC relock delay 
				;timer
;
L674E   FDB  0000		;0 secs, ammount to increment TCC relock delay timer
L6750   FDB  0020		;0.5 secs, ammount to decrement TCC relock delay 
				;timer each period
;
L6752   FCB  40			;1 sec, time threshold to reset rapid decel flag
;
L6753   FCB   0			;0 kPa, MAP threshold to allow TCC to lock
L6754   FCB   0			;0 kPa, MAP threshold to allow TCC to remain lock
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Kickdown pattern TCC engage/release thresholds
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;	MPH = val/2		;Gear #
;
;-Engage threshold
;
L6755   FCB  251		;2nd
L6756	FCB  150		;3rd 
L6757	FCB  150		;4th
;
;-Release threshold
;
L6758   FCB  250		;2nd
L6759	FCB  146		;3rd
L675A	FCB  146		;4th
;
;-Hot transmission engage threshold
;
L675B   FCB  251		;2nd
L675C	FCB  150		;3rd
L675D	FCB  75			;4th
;
;-Hot transmission release threshold
;
L675E   FCB  250		;2nd
L675F	FCB  146		;3rd
L6760	FCB  72			;4th
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-TCC release thresholds vs. TPS (2nd gear)
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;	MPH = val/2		;%TPS
;
L6761 	FCB  250		;  0.0
	FCB  250		;  6.3
	FCB  250		; 12.5
	FCB  250		; 18.8
	FCB  250		; 25.0
	FCB  250		; 31.3
	FCB  250		; 37.5
	FCB  250		; 43.8
	FCB  250		; 50.0
	FCB  250		; 56.3
	FCB  250		; 62.5
	FCB  250		; 68.8
	FCB  250		; 75.0
	FCB  250		; 81.3
	FCB  250		; 87.5
	FCB  250		; 93.8
	FCB  250		;100.0
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-TCC release thresholds vs. TPS (3rd gear)
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;	MPH = val/2		;%TPS
;
L6772   FCB  72			;  0.0
	FCB  72			;  6.3
	FCB  72			; 12.5
	FCB  72			; 18.8
	FCB  73			; 25.0
	FCB  76			; 31.3
	FCB  80			; 37.5
	FCB  84			; 43.8
	FCB  88			; 50.0
	FCB  96			; 56.3
	FCB  106		; 62.5
	FCB  118		; 68.8
	FCB  146		; 75.0
	FCB  146		; 81.3
	FCB  146		; 87.5
	FCB  146		; 93.8
	FCB  146		;100.0
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-TCC release thresholds vs. TPS (4rd gear)
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;	MPH = val/2		;%TPS
;
L6783   FCB  72			;  0.0
	FCB  72			;  6.3
	FCB  72			; 12.5
	FCB  72			; 18.8
	FCB  73			; 25.0
	FCB  76			; 31.3
	FCB  80			; 37.5
	FCB  84			; 43.8
	FCB  88			; 50.0
	FCB  96			; 56.3
	FCB  106		; 62.5
	FCB  118		; 68.8
	FCB  146		; 75.0
	FCB  146		; 81.3
	FCB  146		; 87.5
	FCB  146		; 93.8
	FCB  146		;100.0
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-TCC release thresholds vs. TPS (hot trans, 2nd gear)
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;	MPH = val/2		;%TPS
;
L6794  	FCB   250		;  0.0
	FCB   250		;  6.3
	FCB   250		; 12.5
	FCB   250		; 18.8
	FCB   250		; 25.0
	FCB   250		; 31.3
	FCB   250		; 37.5
	FCB   250		; 43.8
	FCB   250		; 50.0
	FCB   250		; 56.3
	FCB   250		; 62.5
	FCB   250		; 68.8
	FCB   250		; 75.0
	FCB   250		; 81.3
	FCB   250		; 87.5
	FCB   250		; 93.8
	FCB   250		;100.0
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-TCC release thresholds vs. TPS (hot trans, 3rd gear)
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;	MPH = val/2		;%TPS
;
L67A5   FCB  72			;  0.0
	FCB  72			;  6.3
	FCB  72			; 12.5
	FCB  72			; 18.8
	FCB  73			; 25.0
	FCB  77			; 31.3
	FCB  82			; 37.5
	FCB  86			; 43.8
	FCB  94			; 50.0
	FCB  106		; 56.3
	FCB  117		; 62.5
	FCB  146		; 68.8
	FCB  146		; 75.0
	FCB  146		; 81.3
	FCB  146		; 87.5
	FCB  146		; 93.8
	FCB  146		;100.0
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-TCC release thresholds vs. TPS (hot trans, 4th gear)
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;	MPH = val/2		;%TPS
;
L67B6   FCB   72		;  0.0
	FCB   72		;  6.3
	FCB   72		; 12.5
	FCB   72		; 18.8
	FCB   72		; 25.0
	FCB   72		; 31.3
	FCB   72		; 37.5
	FCB   72		; 43.8
	FCB   72		; 50.0
	FCB   72		; 56.3
	FCB   72		; 62.5
	FCB   72		; 68.8
	FCB   72		; 75.0
	FCB   72		; 81.3
	FCB   72		; 87.5
	FCB   72		; 93.8
	FCB   72		;100.0
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-TCC release thresholds vs. TPS (cruise control, 3rd gear)
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;	MPH = val/2		;%TPS
;
L67C7   FCB 72			;  0.0
	FCB 72			;  6.3
	FCB 72			; 12.5
	FCB 72			; 18.8
	FCB 73			; 25.0
	FCB 77			; 31.3
	FCB 82			; 37.5
	FCB 86			; 43.8
	FCB 94			; 50.0
	FCB 106			; 56.3
	FCB 117			; 62.5
	FCB 146			; 68.8
	FCB 146			; 75.0
	FCB 146			; 81.3
	FCB 146			; 87.5
	FCB 146			; 93.8
	FCB 146			;100.0
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-TCC release thresholds vs. TPS (cruise control, 4th gear)
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;	MPH = val/2		;%TPS
;
L67D8   FCB 72			;  0.0
	FCB 72			;  6.3
	FCB 72			; 12.5
	FCB 72			; 18.8
	FCB 73			; 25.0
	FCB 77			; 31.3
	FCB 82			; 37.5
	FCB 86			; 43.8
	FCB 94			; 50.0
	FCB 106			; 56.3
	FCB 117			; 62.5
	FCB 146			; 68.8
	FCB 146			; 75.0
	FCB 146			; 81.3
	FCB 146			; 87.5
	FCB 146			; 93.8
	FCB 146			;100.0
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-TCC engage thresholds vs. TPS (2nd gear)
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;	MPH = val/2		;%TPS
;
L67E9   FCB  250		;  0.0
	FCB  250		;  6.3
	FCB  250		; 12.5
	FCB  250		; 18.8
	FCB  250		; 25.0
	FCB  250		; 31.3
	FCB  250		; 37.5
	FCB  250		; 43.8
	FCB  250		; 50.0
	FCB  250		; 56.3
	FCB  250		; 62.5
	FCB  250		; 68.8
	FCB  250		; 75.0
	FCB  250		; 81.3
	FCB  250		; 87.5
	FCB  250		; 93.8
	FCB  250		;100.0
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-TCC engage thresholds vs. TPS (3rd gear)
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;	MPH = val/2		;%TPS
;
L67FA   FCB 75			;  0.0
	FCB 75			;  6.3
	FCB 75			; 12.5
	FCB 77			; 18.8
	FCB 84			; 25.0
	FCB 93			; 31.3
	FCB 100			; 37.5
	FCB 106			; 43.8
	FCB 114			; 50.0
	FCB 150			; 56.3
	FCB 150			; 62.5
	FCB 150			; 68.8
	FCB 150			; 75.0
	FCB 150			; 81.3
	FCB 150			; 87.5
	FCB 150			; 93.8
	FCB 150			;100.0
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-TCC engage thresholds vs. TPS (4th gear)
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;	MPH = val/2		;%TPS
;
L680B   FCB  75			;  0.0
	FCB  75			;  6.3
	FCB  75			; 12.5
	FCB  77			; 18.8
	FCB  84			; 25.0
	FCB  93			; 31.3
	FCB  100		; 37.5
	FCB  106		; 43.8
	FCB  114		; 50.0
	FCB  150		; 56.3
	FCB  150		; 62.5
	FCB  150		; 68.8
	FCB  150		; 75.0
	FCB  150		; 81.3
	FCB  150		; 87.5
	FCB  150		; 93.8
	FCB  150		;100.0
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-TCC engage thresholds vs. TPS (hot trans, 2nd gear)
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;	MPH = val/2		;%TPS
;
L681C   FCB  250		;  0.0
	FCB  250		;  6.3
	FCB  250		; 12.5
	FCB  250		; 18.8
	FCB  250		; 25.0
	FCB  250		; 31.3
	FCB  250		; 37.5
	FCB  250		; 43.8
	FCB  250		; 50.0
	FCB  250		; 56.3
	FCB  250		; 62.5
	FCB  250		; 68.8
	FCB  250		; 75.0
	FCB  250		; 81.3
	FCB  250		; 87.5
	FCB  250		; 93.8
	FCB  250		;100.0
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-TCC engage thresholds vs. TPS (hot trans, 3rd gear)
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;	MPH = val/2		;%TPS
;
L682D   FCB  75			;  0.0
	FCB  75			;  6.3
	FCB  75			; 12.5
	FCB  77			; 18.8
	FCB  84			; 25.0
	FCB  93			; 31.3
	FCB  100		; 37.5
	FCB  106		; 43.8
	FCB  114		; 50.0
	FCB  150		; 56.3
	FCB  150		; 62.5
	FCB  150		; 68.8
	FCB  150		; 75.0
	FCB  150		; 81.3
	FCB  150		; 87.5
	FCB  150		; 93.8
	FCB  150		;100.0
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-TCC engage thresholds vs. TPS (hot trans, 4th gear)
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;	MPH = val/2		;%TPS
;
L683E   FCB  75			;  0.0
	FCB  75			;  6.3
	FCB  75			; 12.5
	FCB  75			; 18.8
	FCB  75			; 25.0
	FCB  75			; 31.3
	FCB  75			; 37.5
	FCB  75			; 43.8
	FCB  75			; 50.0
	FCB  75			; 56.3
	FCB  75			; 62.5
	FCB  75			; 68.8
	FCB  75			; 75.0
	FCB  75			; 81.3
	FCB  75			; 87.5
	FCB  75			; 93.8
	FCB  75			;100.0
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-TCC engage thresholds vs. TPS (cruise control, 3rd gear)
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;	MPH = val/2		;%TPS
;
L684F   FCB  75			;  0.0
	FCB  75			;  6.3
	FCB  75			; 12.5
	FCB  77			; 18.8
	FCB  84			; 25.0
	FCB  93			; 31.3
	FCB  150		; 37.5
	FCB  150		; 43.8
	FCB  150		; 50.0
	FCB  150		; 56.3
	FCB  150		; 62.5
	FCB  150		; 68.8
	FCB  150		; 75.0
	FCB  150		; 81.3
	FCB  150		; 87.5
	FCB  150		; 93.8
	FCB  150		;100.0
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-TCC engage thresholds vs. TPS (cruise control, 4th gear)
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;	MPH = val/2		;%TPS
;
L6860   FCB  75			;  0.0
	FCB  75			;  6.3
	FCB  75			; 12.5
	FCB  77			; 18.8
	FCB  84			; 25.0
	FCB  93			; 31.3
	FCB  150		; 37.5
	FCB  150		; 43.8
	FCB  150		; 50.0
	FCB  150		; 56.3
	FCB  150		; 62.5
	FCB  150		; 68.8
	FCB  150		; 75.0
	FCB  150		; 81.3
	FCB  150		; 87.5
	FCB  150		; 93.8
	FCB  150		;100.0
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-PWM TCC release operating point DC vs. TPS 
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; 3rd gear/default
;
;	%DC = value x .391	;%TPS
;
L6871   FCB  0			;  0.0
	FCB  0			;  6.3
	FCB  0			; 12.5
	FCB  0			; 18.8
	FCB  0			; 25.0
	FCB  0			; 31.3
	FCB  0			; 37.5
	FCB  0			; 43.8
	FCB  0			; 50.0
	FCB  0			; 56.3
	FCB  0			; 62.5
	FCB  0			; 68.8
	FCB  0			; 75.0
	FCB  0			; 81.3
	FCB  0			; 87.5
	FCB  0			; 93.8
	FCB  0			;100.0
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-PWM TCC release operating point DC vs. TPS 
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; 4th gear
;
;	%DC = value x .391	;%TPS
;
L6882   FCB  0			;  0.0
	FCB  0			;  6.3
	FCB  0			; 12.5
	FCB  0			; 18.8
	FCB  0			; 25.0
	FCB  0			; 31.3
	FCB  0			; 37.5
	FCB  0			; 43.8
	FCB  0			; 50.0
	FCB  0			; 56.3
	FCB  0			; 62.5
	FCB  0			; 68.8
	FCB  0			; 75.0
	FCB  0			; 81.3
	FCB  0			; 87.5
	FCB  0			; 93.8
	FCB  0			;100.0
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-PWM TCC release rate vs. TPS 
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; 3rd gear/default
;
;	%DC/sec = value x 1.25	;%TPS
; 
L6893   FCB  255		;  0.0
	FCB  255		;  6.3
	FCB  255		; 12.5
	FCB  255		; 18.8
	FCB  255		; 25.0
	FCB  255		; 31.3
	FCB  255		; 37.5
	FCB  255		; 43.8
	FCB  255		; 50.0
	FCB  255		; 56.3
	FCB  255		; 62.5
	FCB  255		; 68.8
	FCB  255		; 75.0
	FCB  255		; 81.3
	FCB  255		; 87.5
	FCB  255		; 93.8
	FCB  255		;100.0
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-PWM TCC release rate vs. TPS 
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; 4th gear
;
;	%DC/sec = value x 1.25	;%TPS
; 
L68A4   FCB  255		;  0.0
	FCB  255		;  6.3
	FCB  255		; 12.5
	FCB  255		; 18.8
	FCB  255		; 25.0
	FCB  255		; 31.3
	FCB  255		; 37.5
	FCB  255		; 43.8
	FCB  255		; 50.0
	FCB  255		; 56.3
	FCB  255		; 62.5
	FCB  255		; 68.8
	FCB  255		; 75.0
	FCB  255		; 81.3
	FCB  255		; 87.5
	FCB  255		; 93.8
	FCB  255		;100.0
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-PWM TCC apply operating point DC vs. TPS 
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; 3rd gear/default
;
;	%DC = value x .391	;%TPS
; 
L68B5   FCB  255		;  0.0
	FCB  255		;  6.3
	FCB  255		; 12.5
	FCB  255		; 18.8
	FCB  255		; 25.0
	FCB  255		; 31.3
	FCB  255		; 37.5
	FCB  255		; 43.8
	FCB  255		; 50.0
	FCB  255		; 56.3
	FCB  255		; 62.5
	FCB  255		; 68.8
	FCB  255		; 75.0
	FCB  255		; 81.3
	FCB  255		; 87.5
	FCB  255		; 93.8
	FCB  255		;100.0
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-PWM TCC apply operating point DC vs. TPS 
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; 4th gear
;
;  	%DC = value x .391	;%TPS
;
L68C6   FCB  255		;  0.0
	FCB  255		;  6.3
	FCB  255		; 12.5
	FCB  255		; 18.8
	FCB  255		; 25.0
	FCB  255		; 31.3
	FCB  255		; 37.5
	FCB  255		; 43.8
	FCB  255		; 50.0
	FCB  255		; 56.3
	FCB  255		; 62.5
	FCB  255		; 68.8
	FCB  255		; 75.0
	FCB  255		; 81.3
	FCB  255		; 87.5
	FCB  255		; 93.8
	FCB  255		;100.0
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-PWM TCC engage rate vs. TPS
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; 3rd gear/default
;
; 	%DC/sec = value x 1.25	;%TPS
;
L68D7   FCB  255		;  0.0
	FCB  255		;  6.3
	FCB  255		; 12.5
	FCB  255		; 18.8
	FCB  255		; 25.0
	FCB  255		; 31.3
	FCB  255		; 37.5
	FCB  255		; 43.8
	FCB  255		; 50.0
	FCB  255		; 56.3
	FCB  255		; 62.5
	FCB  255		; 68.8
	FCB  255		; 75.0
	FCB  255		; 81.3
	FCB  255		; 87.5
	FCB  255		; 93.8
	FCB  255		;100.0
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-PWM TCC engage rate vs. TPS
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; 4th gear
;
; 	%DC/sec = value x 1.25	;%TPS
;
L68E8   FCB  255		;  0.0
	FCB  255		;  6.3
	FCB  255		; 12.5
	FCB  255		; 18.8
	FCB  255		; 25.0
	FCB  255		; 31.3
	FCB  255		; 37.5
	FCB  255		; 43.8
	FCB  255		; 50.0
	FCB  255		; 56.3
	FCB  255		; 62.5
	FCB  255		; 68.8
	FCB  255		; 75.0
	FCB  255		; 81.3
	FCB  255		; 87.5
	FCB  255		; 93.8
	FCB  255		;100.0
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-DC for PWM TCC to be fully locked vs. added line pressure
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;	%DC = value x .391	;PSI
;	
L68F9   FCB  230		;  0
	FCB  230		;  8
	FCB  230		; 16
	FCB  230		; 24
	FCB  230		; 32
	FCB  230		; 40
	FCB  230		; 48
	FCB  230		; 56
	FCB  230		; 64
	FCB  230		; 72
	FCB  230		; 80
	FCB  230		; 88
	FCB  230		; 96
	FCB  230		;104
	FCB  230		;112
	FCB  230		;120
	FCB  230		;128
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Mult. for min PWM TCC apply rate vs. added line pressure
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;	Mult. = val/128		;PSI
;
L690A   FCB  128		;  0
	FCB  128		;  8
	FCB  128		; 16
	FCB  128		; 24
	FCB  128		; 32
	FCB  128		; 40
	FCB  128		; 48
	FCB  128		; 56
	FCB  128		; 64
	FCB  128		; 72
	FCB  128		; 80
	FCB  128		; 88
	FCB  128		; 96
	FCB  128		;104
	FCB  128		;112
	FCB  128		;120
	FCB  128		;128
;
;~~~~~~~~~~~~~~~~
;-PWM TCC params
;~~~~~~~~~~~~~~~~
;
L691B   FDB 0000		;0%, min DC for PWM TCC to be unlocked
;
L691D   FCB 12			;6 MPH, threshold for manditory TCC 
L691E   FCB 8			;4 MPH, threshold to exit manditory TCC
;
L691F   FCB 0			;0 secs, min time for TCC to lock
L6920   FCB 0			;0 secs, min time for TCC to engage 
;
L6921   FCB 0			;Min time for TCC to remain unlocked      
L6922   FCB 0			;Min time to fully unlock TCC  
L6923   FCB 0			;0 secs, duration of TCC engagement 
L6924   FCB 0			;0 secs, duration of TCC disengagement
;
;~~~~~~~~~~~~~~~~~~~~
;-Force motor params
;~~~~~~~~~~~~~~~~~~~~
;
L6925   FCB 80			;.5 secs, time for flagging force motor fault 
;
L6926   FCB 73			;7.3 volts, lower range bound for flagging
				;low batt volts (-40 deg C)                 
L6927   FCB 114			;11.4 volts, upper range bound for flagging
				;low batt volts (150 deg C)
;                   
L6928   FCB 78			;7.8 volts, lower range bound for un-flagging
				;low batt volts (-40 deg C)
L6929   FCB 119			;11.9 volts, upper range bound for un-flagging
				;low batt volts (150 deg C)
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Force motor current vs. added line pressure and trans temp
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; 
; Used when desired line pressure is rising
;
;	mA = val x 19.53
;
L692A	FCB   0			;-40 deg C, min row val.
        FCB   0			;0 PSI, min col. val.
        FCB   17		;# of cols/row
;
;	PSI:   0    6   12   18   24   30   36   42   48   54   60   66   72   78   84   90   96 ;Deg C
;
	FCB   55,  45,  45,  45,  43,  41,  39,  37,  36,  32,  31,  28,  25,  21,  17,  11,  11 ;-40 
	FCB   55,  47,  46,  45,  43,  41,  39,  37,  35,  33,  30,  28,  24,  21,  17,  11,  11 ;-28
	FCB   55,  47,  46,  45,  43,  41,  40,  38,  36,  33,  31,  28,  25,  22,  18,  13,  13 ;-16        
	FCB   55,  47,  46,  45,  43,  41,  40,  38,  36,  33,  31,  28,  25,  22,  18,  12,  12 ; -4 
	FCB   55,  48,  46,  45,  43,  41,  39,  37,  35,  33,  30,  28,  24,  21,  17,  11,  11 ;  8
	FCB   55,  48,  46,  45,  43,  41,  39,  37,  35,  32,  30,  27,  24,  20,  16,  10,  10 ; 20 
	FCB   55,  48,  46,  44,  42,  41,  39,  37,  35,  32,  30,  27,  23,  20,  15,   9,   9 ; 32   	
	FCB   55,  47,  46,  44,  42,  40,  39,  36,  34,  32,  29,  26,  23,  19,  15,   8,   8 ; 44 
	FCB   55,  47,  46,  44,  42,  40,  38,  36,  34,  32,  29,  26,  23,  19,  15,   7,   7 ; 56 
	FCB   55,  47,  46,  44,  42,  40,  38,  36,  34,  32,  29,  26,  23,  19,  15,   7,   7 ; 68
	FCB   55,  47,  46,  44,  42,  40,  38,  36,  34,  32,  29,  26,  23,  19,  14,   7,   7 ; 80 
	FCB   55,  47,  45,  44,  42,  40,  38,  36,  34,  32,  29,  26,  23,  19,  14,   7,   7 ; 92
	FCB   55,  47,  45,  44,  42,  40,  38,  36,  34,  32,  29,  26,  23,  19,  14,   7,   7 ;104
	FCB   55,  47,  45,  44,  42,  40,  38,  36,  34,  32,  29,  26,  23,  19,  14,   7,   7 ;116
	FCB   55,  47,  45,  44,  42,  40,  38,  36,  34,  32,  29,  26,  23,  19,  14,   7,   7 ;128 
	FCB   55,  47,  45,  43,  42,  40,  38,  36,  34,  31,  29,  26,  23,  19,  14,   7,   7 ;142
	FCB   55,  47,  45,  43,  42,  40,  38,  36,  34,  31,  29,  26,  23,  19,  14,   6,   6 ;152
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Force motor current vs. line pressure and trans temp
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; 
; Used when desired line pressure is falling
;
;	mA = val x 19.53
;
L6A4E	FCB   0			;-40 deg C, min row val.
        FCB   0			;0 PSI, min col. val.
        FCB   17		;# of cols/row
;
;	PSI:   0    6   12   18   24   30   36   42   48   54   60   66   72   78   84   90   96 ;Deg C
;
	FCB   55,  45,  45,  45,  43,  41,  39,  37,  36,  32,  31,  28,  25,  21,  17,  11,  11 ;-40 
	FCB   55,  47,  46,  45,  43,  41,  39,  37,  35,  33,  30,  28,  24,  21,  17,  11,  11 ;-28
	FCB   55,  47,  46,  45,  43,  41,  40,  38,  36,  33,  31,  28,  25,  22,  18,  13,  13 ;-16 
	FCB   55,  47,  46,  45,  43,  41,  40,  38,  36,  33,  31,  28,  25,  22,  18,  12,  12 ; -4 
	FCB   55,  48,  46,  45,  43,  41,  39,  37,  35,  33,  30,  28,  24,  21,  17,  11,  11 ;  8
	FCB   55,  48,  46,  45,  43,  41,  39,  37,  35,  32,  30,  27,  24,  20,  16,  10,  10 ; 20 
	FCB   55,  48,  46,  44,  42,  41,  39,  37,  35,  32,  30,  27,  23,  20,  15,   9,   9 ; 32
	FCB   55,  47,  46,  44,  42,  40,  39,  36,  34,  32,  29,  26,  23,  19,  15,   8,   8 ; 44
	FCB   55,  47,  46,  44,  42,  40,  38,  36,  34,  32,  29,  26,  23,  19,  15,   7,   7 ; 56 
	FCB   55,  47,  46,  44,  42,  40,  38,  36,  34,  32,  29,  26,  23,  19,  15,   7,   7 ; 68
	FCB   55,  47,  46,  44,  42,  40,  38,  36,  34,  32,  29,  26,  23,  19,  14,   7,   7 ; 80 
	FCB   55,  47,  45,  44,  42,  40,  38,  36,  34,  32,  29,  26,  23,  19,  14,   7,   7 ; 92
	FCB   55,  47,  45,  44,  42,  40,  38,  36,  34,  32,  29,  26,  23,  19,  14,   7,   7 ;104
	FCB   55,  47,  45,  44,  42,  40,  38,  36,  34,  32,  29,  26,  23,  19,  14,   7,   7 ;116
	FCB   55,  47,  45,  44,  42,  40,  38,  36,  34,  32,  29,  26,  23,  19,  14,   7,   7 ;128 
	FCB   55,  47,  45,  43,  42,  40,  38,  36,  34,  31,  29,  26,  23,  19,  14,   7,   7 ;142
	FCB   55,  47,  45,  43,  42,  40,  38,  36,  34,  31,  29,  26,  23,  19,  14,   6,   6 ;152
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Line pressure hyst. vs. trans temp for force motor current lookup
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;	PSI = val		;Deg C
;
L6B72   FCB  1			;-40
	FCB  1			;-36
	FCB  1			;-32
	FCB  1			;-28
	FCB  1			;-24
	FCB  1			;-20
	FCB  1			;-16
	FCB  1			;-12
	FCB  1			; -8
	FCB  1			; -4
	FCB  1			;  0
	FCB  1			;  4
	FCB  1			;  8
	FCB  1			; 12
	FCB  1			; 16
	FCB  1			; 20
	FCB  1			; 24
;
;~~~~~~~~~~~~~~~~~~~~~~
;
; Start of main code
;
;~~~~~~~~~~~~~~~~~~~~~~
;
	ORG	$7200			;
;
L7200	LDS     #$03FF			;Load stack @ 0x3FF
	LDX     #$1000			;Base CPU address
	LDAA    #$03			;New base address for Regs.
	STAA    $3D,X			;Save it, Regs. now init. at 0x3000
	LDX     #$3000			;New base address
	LDAA    #$B8			;1011 1000
	STAA    $39,X			;Option Reg.
					; b7=ADC powerup bit
					; b6=Clock select
					; b5=Configure IRQ
					; b4=Enable startup delay
					; b3=Clock monitor
					; b2= -
					; b1=CR1, COP timer rate select 
					; b0=CR0, COP timer rate select 
;
	LDAA    #$03			;0000 0011, select E/16, or 524 kHz for 16 bit counter
	STAA    $24,X			;TMSK2(Timer Mask Register 2) Reg.
					; b7=Timer overflow interrupt enable
					; b6=Real time interrupt enable
					; b5=Pulse Accumulator Interrrupt enable 
					; b4=Pulse Accumulator Interrupt edge Interrupt enable
					; b3= -
					; b2= -
					; b1=PR1, Timer Prescale select
					; b0=PR0, Timer Prescale select
;
	BCLR    $38,X,#$20		;Clr b5=, OPT 2 REG
	LDAA    #$1B			;0001 1011
	STAA    $35,X			;BPROT (EEPROM block protect) REG
					; b7= -
					; b6= -
					; b5= -
					; b4=Protect Config Reg.
					; b3=BPRT3 (Block Protect) 0xB6E0-0xB7FF
					; b2=BPRT2 (Block Protect) 0xB660-0xB6DF
					; b1=BPRT1 (Block Protect) 0xB620-0xB65F
					; b0=BPRT0 (Block Protect) 0xB600-0xB61F
;
	JSR     LCA8A			;Reg. config routine
;
;-Perform additional clears/resets
;
	CLRA 				;Preclear A   
	STAA    0,X			;Port A
	STAA    $08,X			;Port D
	LDAA    #$7F			;
	LDAB    #$FF			;
	STD     L306E			;
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; SET UP HARDWARE FOR CPI/PFI MODE
; or TBI MODE
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
	LDX     #$B93A			;TBI mode
	LDY     #L400B			;mode word  
	BRCLR   0,Y,#$01,L723A		;Bra if b0=0, In TBI mode
;
;-In CPI/PFI mode
;											
	LDX     #$B91A			;CPI/PFI fuel mode
;
;-Here to store 
;
L723A	STX     L3FFC          		;I/O D Port 
	JSR     LF08A			;RTS, Delay
;
	LDD     L3FFC          		;I/O D Port
	JSR     LF08A          		;RTS, short delay
;
	ANDB    #$FB			;1111 1011, clear b2
	ORAB    #$08			;Set b3
	STD     L3FFC          		;I/O D PORT 
;
;-Serial Data
;
	LDX     #L3000			;Reg. base addr.
	LDD     $2E,X			;Access SCI status register
	LDAB    #$26			;0010 0110 
	STAB    $2D,X			;SCI Contol Reg. 2
					; b7=Transmit interrupt enable
					; b6=Transmit complete interrupt enable
					; b5=Recieve interrupt enable
					; b4=Idle line interrupt enable
					; b3=Transmit enable
					; b2=Recieve enable
					; b1=Reciever wake up
					; b0=Send break 
;
;-Clear varous regs.
;
	LDX     #L3FC0			;DRP
	CLRA 				;Preclear A   
	CLRB 				;Preclear B   
L725B	STD     0,X			;Clear reg.
	INX 				;+1    
	INX  				;+1   
	CPX     #L3FFA			;Last address to clear
	BNE     L725B			;Bra if !=, continue	
;			
	LDX     #L0036			;Start address of RAM to clear
	LDD     #0197			;197, #of bytes to clear
	JSR     LEF9A			;RAM clear routine
;
	LDX     #L0100			;Start address to clear
	LDD     #$01C0			;448, number of bytes to clear
	JSR     LEF9A			;Ram clear routine
;
	LDX     #L0800			;Start address to clear
	LDD     #$0100			;256, number of bytes to clear
	JSR     LEF9A			;RAM clear routine
;
;-Perform additional clears
;
	LDX     #L0362			;Start address		
L7282	CLR     0,X			;Clear address
	INX    				;+1, next addresss 
	CPX     #L03FF			;Last address to clear
	BNE     L7282			;Bra to contine if not at last address
;
	JSR	LEEAA			;A/D routine with multi-channel read
;		
	JSR     LEEE2			;A/D routine, multi-channel read
;
	JSR     LC3BA			;Read in fuel pump volts
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Check ALDL switch position
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;-Check for normal operation
;
	LDAA    L082F			;ALDL switch voltage
	CMPA    #100			;2 V
	BCC     L72C3			;Bra if >=
;
;-Check for field test mode
;
	CMPA    #40			;800 mV
	BCS     L72C3			;Bra if <
;
;-In factory test mode
;											
	BSET    L0050,#$08		;Set b3,  In factory test mode
	LDAA    L00A0          		;A/D TPS
	CMPA    #252           		;4.92 V
	BCS     L72C3          		;Bra if TPS <
;                                                           
	LDAA    L082D          		;A/D coolant
	CMPA    #252           		;149 deg C
	BCS     L72C3          		;Bra if cool temp is less
;
	LDAA    L009F          		;Transmission temp
	CMPA    #5			;-40c
	BHI     L72C3          		;Bra if trans. temp is >
;
	BSET    L0044,#$01		;Status word, set b0, factory test requested 
	CLR     L0001          		;(MAX TERM CNT MJR LP SEG)

	LDX     #$B93A			;
	STX     L3FFC          		;I/O Port D
	JMP     L7548			;
                                     
;
;~~~~~~~~~~~~~~
;-Check EPROM
;~~~~~~~~~~~~~~
;
L72C3	BCLR    L0005,#$10		;Status word, clear b4=bad EPROM CLR b4, BAD EPROM FLAG
	LDAA    L303F          		;System config reg.
	CMPA    #$0B           		;0000 1011, Disable COP, enable ROM, enable EEPROM
	BEQ     L72D2			;EPROM checksum routine
;
	BSET    L003A,#$04     		;Status word, set b2, improper hardware init.
	BRA     L72EC			;bra
;
;~~~~~~~~~~~~~~~~~~~~~~~~
;
; EPROM checksum routine
;
;~~~~~~~~~~~~~~~~~~~~~~~~
;
L72D2	LDX     #L4008			;Start of EPROM area to check
	JSR     LEF47			;Checksum routine
;
	LDAA    L4008			;Mask ID byte										
	CPY     L4006			;Checksum
	BNE     L72E5			;Bran if !=, check for test mode
;												
	CMPA    #$0D			;Mask ID 
	BEQ     L730C			;Bra if == to mask ID
;
;-Check for test mode
;												
L72E5	CMPA    #$AA			;Test mode
	BEQ     L730C			;Bra if ==
;
;~~~~~~~~~~~~~~~~~~~	
;-Here to set error
;~~~~~~~~~~~~~~~~~~~	
	BSET    L003A,#$02		;Status word, set b1, EPROM failure
L72EC	BSET    L0005,#$10		;Set b4, EPROM error
											
	LDD     L3FFC			;I/O D Port
	JSR     LF08A			;RTS, short delay
	ANDB    #$F7			;1111 1110
	STD     L3FFC          		;I/O D Port
	BSET    L001A,#$20		;Set b5 
	BSET    L000F,#$20		;Set b5,  Error 51, EPROM failure
	JSR     LEFC3			;RAM checksum routine
;
	STAA    L0015	        	;Save it, Malf. word checksum
;
;-Check ALDL switch status
;	
	LDAA    L082F			;ALDL A/D voltage
;	
L7308	CMPA    #100			;2 V
	BCC     L7308			;Bra if votage >, loop and wait to reset
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Here if not setting error
;~~~~~~~~~~~~~~~~~~~~~~~~~~~
;											
L730C	JSR     LEFC3			;RAM checksum routine
	CMPA    L0015			;Malf. word checksum 
	BEQ     L733C			;Bra if equal
;
;-Clear RAM if bad checksum
;     
	LDX     #$0000			;Start addr.
	LDD     #0011			;# of bytes to clear
	JSR     LEF9A			;RAM clear routine
;
	LDX     #L000B			;Start addr.
	LDD     #0043			;# of bytes to clear
	JSR     LEF9A			;RAM clear routine
;
	LDX     #L02C0			;Start addr.
	LDD     #0117			;# of bytes to clear 
	JSR     LEF9A			;RAM clear routine
;
	LDAA    L5B24			;Max alowed base TPS 
	LDAB    #0128			;
	STD     L0319			;Save it, Base TPS for closed throttle

	JSR     LEF1D			;BLM reset routine
;
	BSET    L0046,#$40		;Status word, set b6, BLMs reset
L733C	BSET    L0082,#$08		;Status word, set b3, ign. power is on 
;
;-Perform additional checksum
;
	LDX     #L0327			;Uppermost address
	LDAB    #$0D			;# of bytes to check
	CLRA				;    
	PSHB    			;# of bytes to check to stack

L7346	ADDA    0,X			;Add value stored at address
	DEX  				;-1, move to next address   
	DECB  				;-1, # of bytes to check  
	BNE     L7346			;Bra if !=0, continue

	PULB				;# of bytes to check    
	TSTA    			;Test checksum
	BEQ     L735A			;Bra if ==0
;
	CLRA				;
; 
;-Clear RAM if checksum !=0
;   
L7351	INX     			;+1, step back to previous address
	STAA    0,X			;Clear address
	DECB    			;-1 from # of bytes
	BNE     L7351			;Bra if !=0, continue
;
	BSET    L003A,#$40		;Set b6, bad RAM checksum

L735A	LDAA    L4133			;Base SA
	CLRB				;    
	STD     L0219			;Save it, commanded SA

;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;-Error count checking
;
;	Used to permanently set error. Above a 
;	certain # of starts/runs with errors,
;	the error will remain permanently set
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;-Check error 21, Hi TPS, counter
;
	LDAA    L001F			;Error 21 counter
	BEQ     L736D			;Bra if ==0
               
	CMPA    L5B1E			;Threshold to set error 21
	BCC     L736D			;Bra if >=  
;							                
	DECA				;-1, error 21 counter                 
	STAA    L001F			;save it
;
;-Check error 22, low TPS, error
;
L736D	LDAA    L0020			;Error 22 counter
	BEQ     L7379			;
											
	CMPA    L5B21			; 
	BCC     L7379			;
;                 
	DECA				;     
	STAA    L0020			;
;
;-Check error 24 low output trans speed, counter
;
L7379	LDAA    L0021			;Error 24 counter
	BEQ     L7385			;
;							                                
	CMPA    L5B42			;
	BCC     L7385			;
;							                              
	DECA				;       
	STAA    L0021			;
;
;-Check error 28, man. pressure switch, counter
;
L7385	LDAA    L0022			;Error 28 counter
	BEQ     L7391			;
;                
	CMPA    L5B52			;
	BCC     L7391			;
;							                                
	DECA				;    
	STAA    L0022			; 
;
;-Check error 37, brake on, counter
;
L7391	LDAA    L0023			;Error 37 counter
	BEQ     L739D			;
;							                                
	CMPA    L5B58			; 
	BCC     L739D			; 
;							                                 
	DECA				;        
	STAA    L0023			; 
;
;-Check error 38, brake off, counter
;
L739D	LDAA	L0024			;Error 38 counter
	BEQ     L73A9			;
;							                                 
	CMPA    L5B5E			;
	BCC     L73A9			; 
;							                               
	DECA				;       
	STAA    L0024			;
;
;-Check error 68, trans. slipping, counter
;
L73A9	LDAA    L0025			;Error 68 counter
	BEQ     L73B5			;
;							                                
	CMPA    L5B78			;
	BCC     L73B5			;
;							                               
	DECA				;     
	STAA    L0025			;

;-Check error 66, 3-2 shift quad dvr. fail, counter
;
L73B5	LDAA    L0026			;Error 66 counter
	BEQ     L73C1			;
;               
	CMPA    L5B72			;
	BCC     L73C1			;
;							                                 
	DECA				;    
	STAA    L0026			; 
;
;-Check error 67, TCC quad dvr. fail, counter
;
L73C1	LDAA    L0027			;Error 67 counter
	BEQ     L73CD			;
;                
	CMPA    L5B74			;
	BCC     L73CD			;
;							                              
	DECA				;       
	STAA    L0027			;
;
;-Check error 68 counter
;
L73CD	LDAA    L0028			;Error 68 counter
	BEQ     L73D9			; 
;             
	CMPA    L5B79			; 
	BCC     L73D9			; 
							                              
	DECA				;          
	STAA    L0028			; 
;
;-Check error 72, output spd loss, counter
;
L73D9	LDAA    L0029			;Error 72 counter
	BEQ     L73E5			;
;              
	CMPA    L5B8B			; 
	BCC     L73E5			;
;            
	DECA				;    
	STAA    L0029			; 
;
;-Check error 73, force motor current, error
;
L73E5	LDAA    L002A			;Error 73 counter
	BEQ     L73F1			;
;                 
	CMPA    L5B8F			; 
	BCC     L73F1			; 
;                
	DECA				;
	STAA    L002A			; 
;
;-Check error 81, quad dvr. 1 & shif B err., counter
;
L73F1	LDAA    L002B			;Error 81 counter
	BEQ     L73FD			;
;                
	CMPA    L5B9D			;
	BCC     L73FD			;
;                
	DECA				;    
	STAA    L002B			; 
;
;-Check error 82, quad dvr. 1 & shift A err., counter
;
L73FD	LDAA    L002C			;Error 82 counter
	BEQ     L7409			; 
;							                                 
	CMPA    L5B9F			;
	BCC     L7409			;
;							                                 
	DECA				;     
	STAA    L002C			;  
;
;-Check error 83, quad dvr. 1 error, counter 
;
L7409	LDAA    L002D			;Error 83 counter
	BEQ     L7415			;
;                
	CMPA    L5BA1			; 
	BCC     L7415			;
;                 
	DECA				; 
	STAA    L002D			; 
;
;-Check error 85 counter
;
L7415	LDAA    L002E			;Error 85 counter
	BEQ     L7421			;
;               
	CMPA    L5BA2			;
	BCC     L7421			;
;                 
	DECA				;
	STAA    L002E			; 
;
;-Check error 86, low ratio, counter
;
L7421	LDAA    L002F			;Error 86 counter
	BEQ     L742D			; 
;                 
	CMPA    L5BA8			;
	BCC     L742D			;
;                 
	DECA				;    
	STAA    L002F           	;
;
;-Check error 87, high ratio, counter 
;           
L742D	LDAA    L0030			;Error 87 counter
	BEQ     L7439			;
;                
	CMPA    L5BAE			;
	BCC     L7439			;
;               
	DECA				;
	STAA    L0030			;
;
;-Check error 89, Max adpt long shift, counter
;
L7439	LDAA    L0031			;Error 89 counter
	BEQ     L7445			; 
;               
	CMPA    L5BB8			;
	BCC     L7445			;
;                 
	DECA				;   
	STAA    L0031			;

L7445	BRCLR   L001E,#$04,L7457	;Error word #9, bra if b2==0, no error 89 flagged
;
;-Error 89 flagged
;
	LDAA    L0031			;Error 89 counter
	CMPA    L5BB8			;Compare to threshold to set error
	BCC     L7457			;Bra if >=, set error
;
;-Not setting error here
;											
	INCA    			;+1
	INCA  				;+1  
	STAA    L0031			;Save it, error 89 counter
	BCLR    L001E,#$04      	;Clear b2, not setting error yet
;
;-Here if perm. setting error, continue
;
L7457	BCLR    L0019,#$04		;Clear b2, error 45, O2 rich
	BCLR    L0019,#$02		;Clear b1, error 46, VATS
;
;-Check if TCC present
;
	LDX     #L400F			;Option byte, 0000 1000  
	BRCLR   0,X,#$80,L7467		;Bra if b7=0, TCC present
;
;-No TCC here, dont set VSS error
;
	BCLR    L0017,#$20         	;Clear b5, ERR 24, Low VSS if no TCC present
;
;------------------------------------
; CK ERR CNT'S
; AND ??? to B IF ERROR > THRESH 
;------------------------------------
;                  
;-----------------------
; XMISH ERR HANDER
; MAX ADP LONG SHIFT
; ERR 89 (L5BB5)
;-----------------------
;
L7467	LDX	#L5B00			;Start of diag. word masks
	LDAA    L0032			; ERR 89 CNT'S
	CMPA    $B5,X			; IF ERR CNT'R G.T. 0 SET ERR 89, 
	BCS     L7472			; (1 -> 2) 
;											
	STAB    L0032			; ERR 89 CNT'S                 
;
;-----------------------
; MAX ADP LONG SHIFT
; ERR 89 (L5BB6) 
;-----------------------
;
L7472	LDAA    L0033			; ERR 89 CNT'R, IF ERR CNT'R G.T. 0 SET ERR 89
	CMPA    $B6,X			; (2 -> 3)
	BCS     L747A			; ... else                 
;							                
	STAB    L0033			; SAVE ERR 89 CN'S
;
;-----------------------
; MAX ADP LONG SHIFT
; ERR 89 (L5BB7) 
;-----------------------
;
L747A	LDAA    L0034			; ERR 89 CNT'R 
	CMPA    $B7,X			;
	BCS     L7482			;
;
	STAB    L0034			; ERR 89 CNT'R
;									
;-----------------------
; MAX ADP LONG SHIFT
; ERR 89 (L5BB7) 
;-----------------------
;
L7482	LDAA    L082E			;A/D MAP, A/D conv.
	STAA    L01E9			;A/D MAP
;
;~~~~~~~~~~~~~~~~~~
;-Do S/D MAP conv.
;~~~~~~~~~~~~~~~~~~
;         
;S/D MAP=(151x(MAP-26))/128 
;
	STAA    L01E6			;A/D MAP, filtered
	SUBA    #26			;A/D MAP-26
	LDAB    #151			;
	MUL				;151x(A/D MAP-26)
	ADDD    #64			;+64 to round if needed
	ASLD				;x2
	BCC     L7499			;Bra if no overflow
;										
	LDD     #$FFFF			;Clear overflow
;
L7499	STAA    L01E3			;Save it, SD MAP
	STAA    L01EA			;Save it, SD MAP, filtered
;
	JSR     LC45F			;Baro init and lookup routine


	BSET    L004E,#$01		;Set b0=
	BSET    L0043,#$80		;Set b7=
;	
	BCLR    L0004,#$10		;Clear b4, hot restart in effect
;
	LDAA    L0006			;Cool temp, linear
	PSHA				;To the stack    
	JSR     LC78A			;Cool temp lookup routine		
;
	JSR     LC805			;Routine to do look ups based on cool temp.
;
	JSR     LC879			;Routine to do lookups related to SA and knock						

	LDAA    L0006			;Cool temp, linear
	STAA    L02A5			;Initial cool temp (engine off)
;
	PULB				;Get coolant back
	CMPB    L4950			;95 Deg c, previous run temp, min temp for hot restart
	BCS     L74CA			;Bra if cool temp is <
;											
	CMPA    L4951			;110 Deg c, min temp for hot restart 
	BCS     L74CA			;Bra if cool temp is <
;
	BSET    L0004,#$10		;Status word, non-vol. ram, set b4=hot restart
;
L74CA	JSR     L9341			;Power-on idle initialization routine
;
;-Init O2
;
	LDAA    L48B4			;450 mV, O2 init value 450 Mv
	STAA    L01F8			;A/D O2 volts
	STAA    L01F3			;Filtered O2
	STAA    L01F5			;Slow filtered O2 volts (based on airflow)
	JSR     LB24D			;Idle %TPS routine
;
	JSR     LB185			;Do trans. tps based params.
;
;-Init TPS
;
	LDAA    L01FC			;TPS
	STAA    L01FF			;AE TPS, filtered per cool temp			
	STAA    L01FD			;TPS, one delayed
	STAA    L01FE			;TPS, one delayed
	STAA    L0201			;Slow filtered %TPS
;
	LDAA    L489B			;Max position for closed EGR pintle
	STAA    L01D8			;Save it, closed EGR pintle pos.
	JSR     LD075			;Linear EGR routine
;
	LDAA    L0192			;Cool temp
	CLRB    			;Clear LSB
	STD     L0193			;
	JSR     L755E			;Trans I/O init. routine
;
	CLRB				;    
	LDAA    L00A2			;Load linear cool temp
	CMPA    L4071			;Threshold to init cat temp to default value
	BCS     L750C			;Bra if cool temp less then threshold
;
;-Cool temp > threshold, init cat temp to default val, otherwise, clear cat temp 
;
	LDAB    L4072			;Cat init temp when cool temp is hot
;
L750C	STAB    L0078			;Cat temp
	LDD     L3FCA			;EST circuit info
	STD     L0228			;Store it, EST circuit info
;
	LDD     L3012			;TIC 2
	STD     L0851			;TIC 2 result
;
	LDAA    L48DD			;14.7:1, stoich AFR
	STAA    L026D			;AFR
	LDAA    #$0E			;dec 14
	STAA    L0002			;Loop counter
	LDAA    #128			;
	STAA    L026B			;BLM multiplier
	LDAA    #$14			;dec 20
	STAA    L0049			;Major loop counter
	JSR     LEF2B			;BLM clamp routine
;
;-Do COP
;
	LDAA    #$55			;0101 0101
	STAA    L303A			;COP reset reg
;
	LDAA    #$AA			;1010 1010
	STAA    L303A			;COP reset reg
;
	LDAA    #$40			;0100 0000 
	STAA    L004C			;Port C
;
	BRCLR   L0004,#$08,L7548	;Status word, bra if b3==0, last shut down
;
;-Bad shutdown
;									
	LDAA    L4D8C			;Init value for DRP counter on bad shutdown
	STAA    L02F1			;Stored DRP counts.
;
;-Here if no bad shutdown
;
L7548	LDAA    #255			;1111 1111
	STAA    L3023			;Timer interrups flag register 1
;
;-Set TOC 3 value
;
	LDD     L300E			;16 bit free running CPU counter
	ADDD    #$08			;
	STD     L301A			;Save it, Timer output compare 3
;
	LDAA    #$A0			;1010 0000
	STAA    L3022			;Timer interrupt mask reg. 1
;
	JMP     L7BE6			;
;
;~~~~~~~~~~~~~~~~~~~~~~~~~
;
; Trans I/O init. routine
;
;~~~~~~~~~~~~~~~~~~~~~~~~~
;
L755E	CLR     L012E			;Clear 3->2 sol. DC
	CLRB 				;Preclear B to clear solenoid DC  
	LDAA    #$51			;
	CMPA    #$51			;
	BCS     L756A			;

	BRA     L756C			;

L756A	LDAA    #$51			;
;
L756C	STAA    L012F			;Save it, 3 -> 2 solenoid switching speed
	STD     L306C			;Save it, 3 -> 2 solenoid output
;
	LDAA    #$01			;Set b1, normal shift pattern requested
	STAA    L0099			;Trans shift pattern word
;
	LDD     #$DFFF			;
;
	LDX     #L400F			;Option byte 
	BRCLR   0,X,#$40,L7583		;Bra if b6==0, using electronic trans 
;
;-Non CC trans here
;																	; 
	LDD     #$D000			;
;
;-CC trans here
;
L7583	STD     L3FCC			;Save it, I/O port
	JSR     LF08A			;Short delay
;
	LDD     #$D000			;
	STD     L3FEA			;
	JSR     LF08A			;Short delay
;
	LDAA    #$02			;3rd gear
	STAA    L00D3			;Gear #
	RTS				;Return 
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~    
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; DRP/crank fuel/injector handling routine
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
L7597	BRCLR   L0044,#$01,L759E	;Bra if b0==0, factory test not requested
;
;~~~~~~~~~~~~~~~~~~~~~~~~		
; Factory test requested
;~~~~~~~~~~~~~~~~~~~~~~~~
;
	JMP     LF8BB			;
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; No factory test req., continue
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
L759E	LDD     L300E			;Load 16 bit free running counter
	PSHB 				;Push counter to stack   
	PSHA				;    
;
;-Look up knock window delay time vs RPM
;
	LDAB    L0062			;RPM/25
	LSRB				;/2
	ANDB    #$FE			;Clear b0, double byte offset
	LDX     #L4653			;Knock window delay time vs. RPM
	ABX				;Add offset into X     
	PULA				;Get free running counter val. back    
	PULB 				;   
	ADDD    0,X			;Add in delay to counter
	STD     L3016			;Save it Timer output compare 1
;
	LDX     #L3062			;I/O port D
	BCLR    0,X,#$40		;Clear b6
	BSET    0,X,#$40		;Set b6
;
	BRSET   L0044,#$10,L75C3	;Bra if b4==1, ignition off
;											
	BSET    L0050,#$20		;Set b5, DRP ocured 
;
L75C3	LDAB    #255			;1111 1111
	STAB    L306F			;Hardware, fuel pump relay control
	BRCLR   L0050,#$01,L75DB	;Bra if b0==0, dont read in MAP yet
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Read in A/D channel 5, MAP
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
	LDAA    #$05			;0000 0101, A/D channel 5
	STAA    L3030			;A/D control reg 
	ASLD				;33 cycle delay for A/D channel conv.    
	MUL 				;    
	MUL     			;
	MUL     			;
	LDAA    L3031			;Get A/D MAP
	STAA    L082E			;Save it, A/D MAP val.
;
L75DB	BRCLR   L004F,#$10,L75E2	;Bra if b4==0, engine not above crank to run threshold
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Engine above crank to run threshold
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
	JMP     L76FD			;Transition from crank to run
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Engine not yet above crank to run threshold here
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
L75E2	LDD     L0068			;Filtered RPM/6.26
	STD     L006A			;Delayed filtered RPM/6.25  
	BRSET   L0044,#$10,L75F5	;Status word, bra if b4==1, ignition off
;
;-Here if not shutting down
;										
	LDAB    L02F0			;DRP counter
	INCB				;+1
	BEQ     L75FF			;Bra if roll-over
;										
	STAB    L02F0			;Save it, DRP counter
	BRA     L75FF			;Bra to continue
;
;-Here if shutting down/not time for fuel yet
;
L75F5	LDX     #L3000			;Load base CPU reg addr.
;											
	LDAA    #$01			;0000 0001
	STAA    $23,X			;Reset TIC3 flag, TFLG1 reg.
	JMP     L7873			;Return from interrupt
;
;-Here if not shutting down
;
L75FF	LDAA    L0006			;Cool temp, linear
	LDX     #L4DC3			;Crank fueling delay vs. cool temp table
	JSR     LF15E			;2D lookup
;							 
	CMPA    L02F0			;DRP counter
	BCC     L75F5			;Bra if DRPs less, Reset TIC3/TFLG1 and RTI
;
;-Here if its time to run the loop
;
	LDX     #L400C			;Opt. byte 
	BRCLR   0,X,#$04,L7626		;Bra if b2==0, dont fire all inj. each DRP
;
	LDD     L02F2			;PW
	JSR     L78D4			;Cranking PW correction routine
	STD     L026F			;Sync BPW
	LDX     #L400B			;Opt. byte
	BRSET   0,X,#$10,L7629		;Bra if b4==1, async. double fire 
;
	JMP     L76D7			;Output BPW to harware and continue
;
L7626	JMP     L76DD			;Continue 
;
;-Check for err 41, cam sensor
;
L7629	BRCLR   L0019,#$40,L7634	;Err. word #4, bra if no error 41, 1x cam sens.
;
;-Here if err. 41
;										
	LSRD				;PW/2 for double fire
	STD     L026F			;Save it, BPW
	JMP     L76D7			;Output to hardware and contine
;
;-Here if err. 41 not set
;
L7634	ADDB    L0279			;Add in bias to comp. for batt volts
	ADCA    #0			;Perform carry if needed
	BPL     L763E			;Bra if < 32767
;											
	LDD     #$7FFF			;Clear overflow
L763E	STD     L0273			;Save it, TOC 5 value	 
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Handle startup injector firings
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
	LDX     #L3000			;Base CPU reg. addr.										;
	BRSET   $23,X,#$01,L76A4	;Bra if edge detected for TIC3, do TOC 4
;										
	LDAA    L02F0			;DRP counter
	CMPA    #$01			;
	BNE     L7654			;Bra if !=1
;											
	BSET    L0053,#$20		;Status word, set b5, inj. firing pending											
	BRA     L7678			;
											
L7654	BRCLR   L0053,#$20,L7678	;Bra if injector firing is not pending
;	
;-Injector firing is pending 
;										
	PSHX 				;Push CPU addr, to stack
	LDX     #L0273			;Addr. of TOC 4/5 val
	LDAA    L4956			;Crank BPW mult. for 2nd inj. firing
	JSR     LF1ED			;8x16 mult. routine
;											
	PULX				;Get base CPU addr. back
	ASLD				;BPW x 2
	BCC     L7669			;Bra if no overflow
;											
	LDD     #$FFFF			;Clear overflow
;
L7669	STD     L0273			;Save as TOC 4/5 value											
	LDAA    L02F1			;DRP counter
	DECA				;-1, maintain at zero
	CMPA    #255			;Check for underflow
	BNE     L7675			;Bra if no underflow
											
	INCA				;+1 
;   
L7675	STAA    L02F1			;DRP counter
;
;-Here if DRPs > 1/no inj. firing pending
;
L7678 	BSET    $20,X,#$03		;TCTL1, set TOC 5 output line to 1
	BSET    $0B,X,#$08		;CFORC, force output level for TOC 5
	JSR     L7907			;Check for cam sens. error
;											
	LDD     L0273			;TOC 4/5 value
	ASLD				;x2, double counts to match free running counter
	BCC     L768A			;Bra if no overflow
;											
	LDD     #$FFFF			;Clear overflow
;
L768A	ADDD    $0E,X			;Add in free running counter
	STD     L0825			;Save it, outputted val. to TOC 5											
	BCLR    $20,X,#$01		;Clear b0, set TOC 5 output line to 0										
	LDD     $0E,X			;16 bit free running timer
	ADDD    #$02			;+2
	CPD     L0825			;TOC 5 value
	BPL     L76A0			;Bra if interrupt not pending
;											
	LDD     L0825			;Load previous value instead
;
L76A0	STD     $1E,X			;Save it, TOC 5 
	BRA     L76DD			;Bra to continue			
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Here if edge detected for TIC3
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;											
L76A4	LDAA    #$01			;0000 0001
	STAA    $23,X			;Reset TIC3 flag, TFLG1 reg.
	CLR     L084E			;Err. 41 counter
;											
	BSET    $20,X,#$0C		;Set b2,b3, set TOC3 on successful compare, 
	BSET    $0B,X,#$10		;Set b4, force compare for TOC4 
;											
	BCLR    L0053,#$20		;Status word, clear b5, injector fired at first DRP 
;											
	LDD     L0273			;TOC 4/5 value
	ASLD				;x2, double counts to match free running counter
	BCC     L76BD			;Bra if no overflow
;											
	LDD     #$FFFF			;Clear overflow
;
L76BD	ADDD    $0E,X			;Add in 16 bit free running counter
	STD     L0827			;Save as TOC 4 value
;											
	BCLR    $20,X,#$04		;Clear TOC 4 line on interrupt
;											
	LDD     $0E,X			;16 bit free running counter
	ADDD    #$02			;+2 counts
	CPD     L0827			;TOC 4 value
	BPL     L76D3			;Bra if >=
;											
	LDD     L0827			;Load TOC 4 instead if still pending
;
L76D3	STD     $1C,X			;Save it, TOC 4 timer val.
	BRA     L76DD				
;											
L76D7	LDD     L026F			;Sync BPW
	JSR     L862B			;Output BPW to hardware
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Here to increment DRP counter and limit in case of FP relay failure
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;											
L76DD	LDAA    L02F1			;Load DRP counter
	INCA				;+1
	BNE     L76E5			;Bra if no roll-over
;											
	LDAA    #$F0			;Reset to 240 after roll-over
;
L76E5	STAA    L02F1			;Save it
	LDAB    L0055			;Load fuel pump volts
	CMPB    L4D8E			;Lower voltage threshold for fuel pump functioning
	BHI     L76FA			;Bra if greater then threshold
;	
	CMPA    L4D8D			;Upper counts limit when pump volts low/zero
	BLS     L76FA			;Bra if counter is less
;											
	LDAA    L4D8D			;Load max
;
	STAA    L02F1			;Save it, cranking fuel counter #2
L76FA	JMP     L7873			;Return from interrupt
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Here to transition from crank to run
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;											
L76FD	LDAA    L02EC			;AFR crank transition AFR
	BEQ     L7743			;Bra if ==0
;											
	LDAA    L028B			;Elapsed DRPs 
	INCA				;+1
	BEQ     L770B			;Bra if roll-over
;											
	STAA    L028B			;If no roll-over, save it
;
;-Look up transition decay delay DRP value 
;
L770B	LDX     #L4CB5			;Addr. of table
	LDAA    L0006			;Load cool temp, linear
	JSR     LF15E			;2D look up
;
	TAB				;Looked up val. to B   
;
;-Transition decay delay multiplier 
;
	LDX     #L4CC6			;Table addr.
	LDAA    L02F1			;Stored DRP counts
	ASLA				;x2
	ASLA				;x2   
	BCC     L7720			;Bra if no overflow
											
	LDAA    #255			;Clear overflow
;
L7720	JSR     LF15E			;2D lookup
	MUL				;
;
;-# of elapsed DRPs to delay = (DRPs x delay multipier)/256 
;				 
	CMPA    L028B			;Elapsed DRPs
	BCC     L7743			;Bra if timer <=
;
;-Run the routine on every third count of L081E, skip otherwise
;										
	INC     L081E			;+1, execution delay timer
	LDAA    L081E			;exec. delay timer
	CMPA    L495A			;# counts to run routine
	BCS     L7743			;Bra if timer less, skip transition adj.

	CLR     L081E			;Reset timer for next pass
	LDAA    L02EC			;AFR crank transition ratio
	SUBA    L028A			;Sub. AFR decay rate vs cool temp
	BCC     L7740			;Bra if no underflow
;										
	CLRA 				;Clear underflow   
;
L7740	STAA    L02EC			;AFR crank transition ratio
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Here if transition complete
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
L7743	LDAA    L400C			;Option byte  
	BITA    #$08			;Test b3, Asnyc/single fire 
	BEQ     L7786			;Bra if ==0, Async fueling instead
;	
;~~~~~~~~~~~~~~~~~~~~
; Here if CPI in use
;~~~~~~~~~~~~~~~~~~~~
;										
	BRSET   L0019,#$40,L7786	;Bra if b6==1, err 41, 1x cam sensor 
;										
	BRSET   L0051,#$01,L7767	;Status word, bra if b0==1, in single fire mode 
;											
	BRCLR   L0053,#$40,L77C9	;Status word, bra if b6==0, 1=Single fire alt. exit desired
;									
	BCLR    L0053,#$40		;Clear b4, single fire alt exit desired
;
	LDD     L0277			;Sync. BPW
	ASLD 				;x2 for single fire mode
	BCC     L7762			;Bra if no overflow
										
	LDD     #$FFFF			;Clear overflow
;
L7762	STD     L0277			;Save it BPW
	BRA     L77AF			;Bra to continue if BPW maxed out
;
;-Here if in single fire
;
L7767	INC     L0829			;+1, DRP counter
	LDAA    L0829			;DRP counter
	CMPA    #$03			;3 DRPs
	BCC     L7789			;Bra if >=
;
;-Check for async/single fire mode
;
	LDD     L0277			;Sync. BPW
	CPD     L4920			;Threshold for double to single fire mode
	BCS     L77C9			;Bra if BPW less
;
;-In sync/double fire mode here
;											
	LDAA    L0829			;Load DRP counter
	CMPA    #$01			;1 DRP
	BNE     L77BF			;Bra if !=
;
	BSET    L0053,#$40		;Set b6, single fire alt. exit desired
	BRA     L77BF			;	
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Here if tbi in use instead
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~		
;
L7786	JMP     L7812			;Bra to continue
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Here if 3 or more DRPs have occured
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
L7789	LDX     #L3000			;Load base CPU reg. addr.
					;
	LDAA    #$01			;Set b0, TIC 3 flag
	STAA    $23,X			;Save it, timer interrupts flag reg. 1
;											
	LDD     L0277			;sync BPW
	CPD     L4920			;Threshold for single to double fire mode 
	BCC     L77A5			;Bra if sync BPW is >
;		
;-Still in single fire mode
;									
	LDAA    L0829			;DRP counter
	CMPA    #$04			;4 drps to reset counter
	BNE     L7812			;Bra if !=, continue
;											
	CLR     L0829			;Clear DRP counter
	BRA     L7812			;Bra to continue
;
;-Above threshold to transition to double fire mode
;
L77A5	LDAA    L0829			;DRP counter		
	CMPA    #$04			;4 DRPs
	BNE     L77AF			;Bra if !=
;											
	BSET    L0053,#$40		;Set b6, single fire alt. exit desired

L77AF	LDD     L0277			;BPW
	ADDB    L0279			;Batt volts BPW correction
	ADCA    #$0000			;round if needed
	BPL     L77BC			;Bra if no signed overflow

	LDD     #$7FFF			;Load max BPW
;
L77BC	STD     L0273			;TOC 4/5 value
;
L77BF	BCLR    L0051,#$01		;Status word, clear b0, CPI single fire mode
	CLRA 				;   
	STAA    L082A			;Time in double fire mode
	STAA    L0829			;DRP counter
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Transition from double to single fire mode
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
L77C9	LDY     #L400B			;Load option byte
	BRSET   0,Y,#$20,L77D5		;Bra if b5==1, 180 deg offset
;											
	BSET    L0051,#$80		;Set b7, transition from double to single fire mode
;											
L77D5	LDX     #L3000			;Load base CPU reg. addr.
	BRCLR   $23,X,#$01,L77F9	;Bra if TIC 3 flag not set
;											
	LDAA    #$01			;
	STAA    $23,X			;Reset TIC 3 
	CLR     L084E			;Err. 41 counter 
	BRSET   0,Y,#$20,L7801		;Bra if b5==1. 180 deg offfset
;
;-No offset
;											
L77E8	JSR     L78A4			;Routine to set up TOC 4 interrupt
;										
	BRSET   L0051,#$80,L7812	;Bra if transition already flagged
;											
	BSET    L0051,#$80		;Set b7, transition in progress
;		
	CLRA				;Clear time based delay for next pass
	CLRB				;
	STD     L081F			;Time based delay for injector firing
	BRA     L7801			;Bra to continue				
;											
L77F9	JSR     L7907			;Routine to check for cam sens. error
;											
	BRSET   0,Y,#$20,L77E8		;Bra if b5==1, 180 deg offset
											
L7801	JSR     L7874			;Routine to set up for TOC 5 interrupt
;											
	BRSET   L0051,#$80,L7812	;Status word, bra if b7==1, transitioning from
					;double to single fire mode
;											
	BSET    L0051,#$80		;Set b7, transition in progress					
	CLRA				;
	CLRB				;
	STD     L081F			;Clear time delay for injector firing
	BRA     L77E8			;Bra to continue				
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Here to continue and return from interrupt
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;	
;-Check if its time to fire the injectors
;		
L7812	LDD     L3FFA			;CPU status
	ORAA    L0073			;CPU status word
	BCLR    L0073,#$40		;CPU status word, clear b6
;											
	BITA    #$40			;Test b6
	BEQ     L7873			;Bra if ==0, return to skip injector firing											
;
;-Check quals for limiting RPM derivative term
;
	LDAB    L0067			;RPM/6.25
	CMPB    L450C			;Upper RPM limit for derivative RPM term
	BCS     L783B			;Bra if RPMs <
;
	LDAB    L01FC			;%TPS
	CMPB    L48D0			;TPS threshold for idling
	BHI     L783B			;Bra if TPS >, limit derivative term											
;
;-Filter RPM derivative term
;
	LDD     L0068			;Filtered RPM/6.25
	LDX     #L006A			;Delayed filtered RPM/6.25
	LDY     #L450E			;Filter coeff.
	JSR     LF116			;Lag filter routine
;
	BRA     L783D			;Bra to save
;
;-Here to skip filtering
;
L783B	LDD     L0068			;Filtered RPM/6.25
;
L783D	STD     L006A			;Save it, delayed filtered RPM
	LDAA    L400B			;Opt. byte
	BITA    #$01			;Test b0
	BEQ     L7873			;Bra if not in CPI/PFI mode
;
;-In CPI/PFI mode here
;
	LDAA    L400C			;Opt byte 
	BITA    #$08			;Test b3
	BEQ     L7851			;Bra if async double fire mode
;
	BRCLR   L0019,#$40,L7873	;Err. word, bra if b6==0, no err. 41 
;											
L7851	BRCLR   L0051,#$01,L7865	;Bra if b0==0, not in single fire mode
;											
	BRCLR   L003F,#$10,L7870	;Status word, bra if b4==0, sync/async flag
;
;-Check if ok to exit single fire mode
;											
	LDD     L0277			;BPW
	CPD     L4920			;Threshold to go from single to double fire mode
	BLS     L7865			;Bra if BPW <=
;
;-Here to exit single fire
;
	BCLR    L0051,#$01		;Status word, clear b0, not in single fire mode
;
;-Fire the fuel injectors
;
L7865	BCLR    L003F,#$10		;Clear b4, sync/async flag											;
	JSR     L7874			;Routine to set up for TOC 5 interrupt to fire 
					;injector
	JSR     L78A4			;Routine to set up for TOC 4 interrupt to fire 
					;injector
;					
	BRA     L7873			;Bra to return
;
L7870	BSET    L003F,#$10		;Set b4, sync/async flag
;
;-Jump here if ign. off
;										
L7873	RTI				;Return from interrupt     
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; Routine to set up for TOC 5 interrupt for injector firing
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
L7874	LDX     #L3000			;Base CPU reg. addr.
;
L7877	LDD     L081F			;Time based delay for inj. firing
	CPD     #$0006			;11.4 usec, min delay 
	BCC     L7883			;Bra if >=
;
	LDD     #$06			;Load min allow. value
;	
L7883	ADDD    $0E,X			;Add to 16 bit counter
	STD     L0821			;Save it, TOC 5 timer value
;											
	LDAA    #$08			;Set b3, Reset TOC 5
	STAA    $23,X			;TFLG1 reg.
;
	BSET    $22,X,#$08		;Set b3, TMSK1, req. TOC 5 interrupt
	BRCLR   0,X,#$08,L789B		;Bra if b3 of port A (TOC 5) ==0
;
;-Interrupt has already occured here
;											
	LDD     L0821			;TOC 5 counter val
	CPD     $1E,X			;Timer val at TOC 5 reg.
	BPL     L78A3			;Bra if >= TOC 5 timer val 
;
;-If previous interrupt not pending, reset and continue
;
L789B	BSET    $20,X,#$03		;Set b1, b0 of Timer Control Reg. 1 (TCTL1),
					;set output line to 1 on compare 						
	LDD     L0821			;TOC 5 counter value
	STD     $1E,X			;Save it TOC 5
;										
L78A3	RTS  				;Return  
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 					
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; Routine to set up TOC 4 interrupt for injector firing
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;											
L78A4	LDX     #L3000			;Base CPU reg. addr.
	LDD     L081F			;Time based delay for inj. firing
	CPD     #$0006			;11.4 usec, min delay
	BCC     L78B3			;Bra if >=
;											
	LDD     #$06			;Load min allowed value
;
L78B3	ADDD    $0E,X			;Add in to free running counter val.
	STD     L0823			;Save it, TOC 4 counter val.
;											
	LDAA    #$10			;0001 0000
	STAA    $23,X			;Reset TOC 4, TFLG1 reg.
;											
	BSET    $22,X,#$10		;Set b4, TOC 4 interrupt req.
	BRCLR   0,X,#$10,L78CB		;Bra if b4 of port A (TOC 4) ==0
;
;-Interrupt has already occured here
;											
	LDD     L0823			;TOC 4 counter val.
	CPD     $1C,X			;Compare to timer val. @ TOC 4 reg
	BPL     L78D3			;Bra if >= TOC 4 
;
;-If previous interrupt not pending, reset and continue
;											
L78CB	BSET    $20,X,#$0C		;Set b2, b3 of Timer Control Reg. 1 (TCTL1),
					;set output line to 1 on compare 										
	LDD     L0823			;Load TOC 4 counter val 
	STD     $1C,X			;Save it, TOC 4 
;
L78D3	RTS				;Return    
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; Cranking PW correction routine
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
L78D4	PSHB				;Push pulsewidth to stack  
	PSHA    			;
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Choose one of the below tables as the multipliers to use depending on the conditions
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
	LDAB    L02F1			;Load stored DRP counts
	CMPB    #23			;23 counts
	BLS     L78E4			;Bra if counts <= 23
;
;-Counts > 23 here, use table to cycle the fueling
;											
	ANDB    #$07			;0000 0111, clear all but b0-b2, cycle crank fuel if cranking longer then this
;
;-Do cycling crank fuel mult. vs counts when counts > 23
;
	LDX     #L4E1C			;Addr. of cycling crank fuel mult. vs crank fuel counter
	BRA     L78F8			;Jump to do indexed lookup
;
;-Use hot restart crank fuel mult. if hot restart in effect
;
L78E4	LDX     #L4E04			;Addr. of hot restart crank fuel mult. vs cranking fuel counter									;
	BRSET   L0004,#$10,L78F8	;Bra if hot restart in effect, do lookup
;
;-Use cold crank mult. vs cranking counts if cool temp < 15 deg C
;
	LDX     #L4DEC			;Address of cold cranking mult. vs cool temp											;
	LDAA    L0006			;Cool temp, linear
	CMPA    L4D8F			;Lower threshold to use table
	BCC     L78F8			;Bra if cool temp is greater, do lookup
;
;-Use this table if cool temp >= 15 deg C
;
	LDX     #L4DD4			;Crank fuel mult. vs cranking counts
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Here to do indexed lookup
;~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
L78F8	ABX				;Add in cranking fuel counter #2
	LDAA    0,X			;Get address of mult. from selected table
	TSX				;Get current stack pointer into X
	JSR     LF1ED			;(A x X*)/256
;		
	PULX				;Clean up stack, get PW  back
	ASLD				;Result x 2, now (mult. x PW)/128 
	BCC     L7906			;Bra if no overflow, return
;											
	LDD     #$FFFF			;Clear overflow
;
L7906	RTS				;Return    
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; Error 41, cam pulse sensor fail
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
L7907	LDAA    L5B03			;Error mask 4
	BITA    #$40			;Test b6, error 41, cam pulse sens. fail
	BEQ     L7921			;Bra if not checking error, return
;											
	LDAA    L084E			;Error 41 counter
	CMPA    #255			;
	BEQ     L7919			;Bra if ==255, compare to threshold to set err
;											
	INCA				;+1, error 41 counter
;
	STAA    L084E			;Save it
L7919	CMPA    L4E65			;Threshold to set error
	BLS     L7921			;Bra if less dont set error yet
;
;-Error counter above threshold
;
	BSET    L0019,#$40		;Set b6, error 41, 1x cam sens. 
;
L7921	RTS				;Return 
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;~~~~~~~~~~~~~~~~~~~~~~~~~
;
; TOC 5 interrupt routine
;
;~~~~~~~~~~~~~~~~~~~~~~~~~
;
L7922	LDX     #L3000			;Base CPU adr.
	LDAA    #$08			;Set b3, clear TOC 5 flag
	STAA    $23,X			;TFLG1 reg.
;
;-TOC 5 line ==1, reset TOC
;
	BRCLR   0,X,#$08,L7972		;Reg A, bra if TOC 5 line ==0
;											
	BCLR    $22,X,#$08		;TMSK1 reg, clear b3==0, disable TOC 5 interrupt
	BCLR    $20,X,#$01		;TCTL1 reg, clear b1, clear TOC 5 line on inter.
;
	BRCLR   L0046,#$01,L7954	;Status word, bra if ==0, no sync AE this pass
;
;-Here if sync AE requested
;

	LDD     L0273			;TOC 4/5 value
	ADDD    L025D			;Add in stored async pulse time for sync output
	BCS     L7942			;Bra if overflow 
;											
	ASLD				;x2 for single injector firing
	BCC     L7945			;Bra if no overflow
;										
L7942	LDD     #$FFFF			;Clear overflow
;
L7945	ADDD    $1E,X			;Add in TOC 5 value
	STD     L0825			;TOC 5 counter value
	CLRA				; 						  
	CLRB				;
	STD     L025D			;Clear stored async pulse time, outputted to injectors
;
	BCLR    L0046,#$01		;Status word, clear b0, reset sync AE flag
	BRA     L7962			;				
;
;-No sync AE this pass
;		
L7954	LDD     L0273			;TOC 4/5 value
	ASLD				;x2, match free running counter
	BCC     L795D			;Bra if no overflow
;				
	LDD     #$FFFF			;Clear overflow
;
L795D	ADDD    $1E,X			;Add in TOC 5 reg.
	STD     L0825			;TOC 5 timer value
;
L7962	LDD     $0E,X			;Free running CPU counter
	ADDD    #$02			;+2 counts
	CPD     L0825			;TOC 5 counter value
	BPL     L7983			;Bra if interrupt not pending
;
;-Interrupt pending
;											
	LDD     L0825			;Load TOC 5 value
	BRA     L7983			;Bra to store and return				
;									
L7972	BSET    $20,X,#$01		;TCTL1 reg, set b0, set TOC 5 line on inter.
	LDD     $0E,X			;Free running CPU counter
	ADDD    #$02			;+2 counts
	CPD     L0821			;TOC 5 counter value
	BPL     L7983			;Bra if interrupt not pending
;			
	LDD     L0821			;Load TOC 5 counter value				
;
L7983	STD     $1E,X			;Save it, TOC 5
	RTI				;Return     					
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;~~~~~~~~~~~~~~~~~~~~~~~~~
;
; TOC 4 interrupt routine
;
;~~~~~~~~~~~~~~~~~~~~~~~~~
;
L7986	LDX     #L3000			;Base CPU reg.
	LDAA    #$10			;0001 0000, clear TOC 4 flag
	STAA    $23,X			;TFLG 1 reg  
	BRCLR   0,X,#$10,L79D6		;Reg A, bra if TOC 4 line ==0
;
;-TOC 4 line ==1, reset TOC
;											
	BCLR    $22,X,#$10		;TMSK1 reg, clear b4==0, disable TOC 4 interrupt
	BCLR    $20,X,#$04		;TCTL1 reg, clear b2, clear TOC 4 line on inter.
;
	BRCLR   L0046,#$01,L79B8	;Status word, bra if ==0, no sync AE this pass
;
;-Here if sync AE requested
;
	LDD     L0273			;TOC 4/5 value
	ADDD    L025D			;Add in stored async pulse time for sync output
	BCS     L79A6			;Bra if overflow
;											
	ASLD				;x2 for single injector firing
	BCC     L79A9			;Bra if no overflow
;											
L79A6	LDD     #$FFFF			;Clear overflow
;
L79A9	ADDD    $1C,X			;Add in TOC 4 timer value
	STD     L0827			;TOC 4 counter value
	CLRA				;
	CLRB				;
	STD     L025D			;Clear stored async pulse time
	BCLR    L0046,#$01		;Status word, clear b0, reset sync AE flag
	BRA     L79C6			;
;
;-No sync AE this pass
;
L79B8	LDD     L0273			;TOC 4/5 value
	ASLD				;x2, match free running counter
	BCC     L79C1			;Bra if no overflow
;											
	LDD     #$FFFF			;Clear overflow
;
L79C1	ADDD    $1C,X			;Add in TOC 4 reg.
	STD     L0827			;TOC 4 timer value
											
L79C6	LDD	$0E,X			;CPU counter
	ADDD    #02			;+2 counts
	CPD     L0827			;TOC 4 counter value
	BPL     L79E7			;Bra if interrupt not pending
;
;-Interrupt pending
;											
	LDD     L0827			;TOC 4 counter value
	BRA     L79E7			;Bra to save and return
;
L79D6	BSET    $20,X,#$04		;TCTL1 reg, set b2, set TOC 4 line on inter.
	LDD     $0E,X			;Free running CPU counter
	ADDD    #$02			;+2 counts
	CPD     L0823			;TOC 4 counter value
	BPL     L79E7			;Bra if interrupt not pending
;
;-Interrupt pending
;											
	LDD     L0823			;Load TOC 4 counter value
;
L79E7	STD     $1C,X			;Save it, TOC 4
	RTI				;Return 
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~   					
;
;~~~~~~~~~~~~~~~~~~~~~~
;
; Main vector handler
; 
;-Run every 6.25 msec
;
;~~~~~~~~~~~~~~~~~~~~~~
;
;-Set up for next interrupt
;
L79EA	LDD     L301A			;TOC 3 reg
	ADDD    #0819			;819, 6.25 msec
	STD     L301A			;Save it, time to next TOC 3 interrupt

	LDAA    #$20			;0010 0000, Reset TOC 3 flag
	STAA    L3023			;TFLG 1 reg.
	BSET    L0000,#$01		;Status word, set b0, main vector loop initiated 
;
;-Check for factory test
; 
	BRCLR   L0044,#$01,L7A05	;Status word, bra if b0==0, no factory test req.
;
;-Factory test req.
;
	INC     L0002			;Loop count			
	JMP     LF705			;Go do factory test
;
;-No factory test req., check for loop overrun
;
L7A05	BRSET   L003A,#$20,L7A17	;Status word, bra of b5==1, loop overrun already flagged  
	BRCLR   L004F,#$04,L7A14	;Status word, bra if b2==0, loop didnt overrun 
;
;-Here if loop overran
;											
	LDAA    L0002			;Loop counter
	STAA    L0003			;Loop counter, one delayed
	BSET    L003A,#$20		;Status word, set b5, loop overrun flagged 
;
L7A14	BSET    L004F,#$04		;Status word, reset b2, loop overrun flag
;
L7A17	TSX				;Get stack pointer into X
	CPX     #L03FF-8		;
	BEQ     L7A20			;
;											
	BSET    L003A,#$08		;Status word, set b3==1, data failed to be pulled
					;from stack
;
L7A20	LDS     #L03FF			;Load stack @ 0x3FF
	BRCLR   L003B,#$10,L7A3B	;Serial data status word, bra if b4==0, not in mode 4 
;	
;-In mode 4, check and enable SES light if requested
;										
	LDAB    L0390+1			;Load in data from ALDL diag. data buffer (ALDL output
					;enables)
	BITB    #$04			;Test b2
	BEQ     L7A3B			;Bra if ==0, no discrete enables
;											
	BCLR    L0072,#$80		;Status word, clear b7, turn off SES light
;
	LDAB    L0390+2			;Load in data from ALDL diag. data buffer (ALDL output
					;states)
	BITB    #$04			;Test b2
	BEQ     L7A3B			;Bra if ==0, no request to turn on SES light
;
	BSET    L0072,#$80		;Status word, set b7, turn on SES light
;
L7A3B	JSR     L9286			;IAC control routine	
;
	JSR     LF08B			;I/O routine
;
	LDAA    L082E			;A/D MAP val.
	LDAB    L006F			;Status word
	BITB    #$48			;Test b6=High MAP err, b3=Low MAP err.
	BEQ     L7A73			;Bra if ==0, no map errors set
;
	BRCLR   L004F,#$10,L7A70	;Status word, bra if b4==0, engine off/cranking
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Low/High map errors set, generate default MAP val.
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;										
	LDAA    L4E5F			;Load TPS coeff. for default MAP
	LDAB    L01FC			;TPS
	MUL				;Coeff x TPS
	ASLD				;x2
	BCS     L7A70			;Bra if overflow
											
	ASLD				;x2
	BCS     L7A70			;Bra if overflow
											
	PSHA				;Push to stack    
;
;-Get default MAP bias
;
	LDAA    L0062			;RPM/25 
	LSRA 				;/2 for lookup
	LDX     #L4E60			;Default MAP bias vs. RPM
	LDAB    #16			;800 RPM offset
	JSR     LF137			;2D lookup with UL
;
	PULB				;    
	ABA   				;Add in the bias 
	BCS     L7A70			;Bra if overflow
;
;-Default MAP = (TPS x default MAP coeff.)/64 + default MAP bias
;
	CMPA    L4E5E			;Upper limit for default MAP 
	BLS     L7A73			;Bra if less, use calculated default MAP
;
;-Here to load default MAP if overflow from prev. ops./not running
;
L7A70	LDAA    L4E5E			;90.8 Kpa default MAP, if not running/overflow from default MAP calc.
;
L7A73	LDAB    L4139			;Transient MAP filter coeff
	BRSET   L0047,#$01,L7A7D	;Bra if b0==1, load transient MAP filter coeff.
;
	LDAB    L4138			;Load standard MAP filter coeff
L7A7D	LDX     L01E6			;Load old filtered MAP
	JSR     LF0F6			;Lag filter routine
	STD     L01E6			;Save filtered MAP
	ASLB				;Shift LSB left    
	ADCA    #$0			;Round if LSB >= 128
	STAA    L01E9			;Save it, current MAP
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Read in force motor current
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
	SEI				;Disable interrupts
	LDAA    #$07			;0000 0111, 4 consec. reads of channel 7
	STAA    L3030			;A/D control reg. (ADCTL)
;
;-Wait for data
;
	LDX     #L3000			;Base CPU reg. addr.
L7A95	BRCLR   $30,X,#$80,L7A95	;ADCTL, bra if b7==0, A/D conversion not done yet 
;
;-A/D conversion complete, average readings
;
	CLRA				;Preclear A
	LDAB    L3031			;A/D result reg. 1, first A/D read
	ADDB    L3032			;Add in second result from A/D read
	ADCA    #$00			;Perform carry if needed
	ADDB    L3033			;Add in third result
	ADCA    #$00			;Carry if needed
	ADDB    L3034			;Add in fourth result
	ADCA    #$00			;Carry if needed
	LSRD				;/2
	LSRD				;/2
	ADCB    #$00			;Round up if needed
	STAB    L0148			;Save it, average of four reads 

	JSR     LEEE2			;Read in TPS, MAP, and O2 A/D vals.	
;											
	CLI				;Enable interrupts    
	JSR     L7CF6			;Force motor control routine
;											
	SEI				;Disable interrupts
	LDD     L3FFA			;CPU status
	STD     L0073			;Save it
	CLI				;Re-enable interrupts     
	BITA    #$04			;Test b2, 
	BEQ     L7AD4			;Bra if b2==0, No VSS pulses			
;
	LDD     L3FC4			;VSS counter
	CPD     L080D			;Stored VSS counter val. from previous pass
	BEQ     L7AD4			;Bra if no change
;
	INC     L080C			;+1, VSS pulse ocured
	STD     L080D			;Save VSS counter

L7AD4	LDAB    L0146			;Force motor DC
	LDAA    #$06			;
	STD     L3068			;Output to hardware
;
	JSR     L7E04			;Brake switch handling routine
;
	LDAA    L0002			;Loop counter
	INCA				;+1    
	CMPA    #160			;1 second
	BNE     L7B00			;Bra if !=, continue
;
;-Here to increment engine run time
;
	LDAB    L0043			;
	EORB    #$20			;Toggle b5,
	STAB    L0043			;Save it
	BRCLR   L004F,#$80,L7AFF 	;Status word, bra if b7==0, engine not running
;
	LDX     L00F8			;Load engine run time
	INX				;+1
	BEQ     L7AF7			;Bra if roll-over
										
	STX     L00F8			;Engine run time
L7AF7	CPX     L4953			;Min run time to clear hot restart
	BCS     L7AFF			;Bra if run time <
;
;-Clear hot restart status
;
	BCLR    L0004,#$10		;Status word, clear b4, hot restart in effect
L7AFF	CLRA				;Preclear A   
;
;-Here if 1 sec has not yet passed
; 
L7B00	STAA    L0002			;Loop counter
	SEI 				;Disable interrupts    
	LDAA    L003B			;Serial data status word 
	ANDA    #$FB			;Clear b2, diagnostics disabled
	BRSET   L0051,#$08,L7B0F	;Bra if shutdown in effect, disable diagnostics
;
	BITA    #$18			;Test b3=in mode 4, b4=in mode 4
	BEQ     L7B11			;Bra if ==0, dont disable diag.
;
L7B0F	ORAA    #$04			;Set b2, disable diagnostics
;
L7B11	STAA    L003B			;Save serial data status word
	CLI				;Re-enable interrupts    
;
	BRCLR   L001A,#$20,L7B2D	;Bra if b5==0, No err 51 set, faulty prom
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~
; Err. 51, faulty prom set
;~~~~~~~~~~~~~~~~~~~~~~~~~~
;
	BRCLR   L0002,#$01,L7B7C	;Loop counter, bra if b1==0, run every other pass
;
	BSET    L0043,#$80		;Set b7, 
	JSR     LAD0C			;RPM calc. routine
;
;-Do COP
;
	LDD     #$AA55			;1010 1010 0101 0101
	STAA    L303A			;COP
	STAB    L303A			;COP
	BRA     L7B7C			;Bra to run real time subroutines
;
;-No err 51 set.
;
L7B2D	BRSET   L0002,#$01,L7B51	;Loop counter, bra if b0 set, run every other pass
;
;~~~~~~~~~~~~~~~~~~~~~
; First 80 Hz cluster
;~~~~~~~~~~~~~~~~~~~~~
;
	JSR     L924A			;I/O control routine
;
	JSR     LA53C			;Spark minor loop
;
	JSR     LD075			;EGR pos. read in routine
;
	JSR     LD11D			;EGR pos. PID control routine
;
	JSR     LF285			;ALDL scheduled message transmission routine
;
;-Alternate passes for each set of loops
;
	BRCLR   L0002,#$02,L7B49	;Loop counter, bra if b1==0, run second cluster
;
;~~~~~~~~~~~~~~~~~~~~~
; First 40 Hz cluster
;~~~~~~~~~~~~~~~~~~~~~
;
	JSR     LB773			;Line pressure/TCC routine calling schedule
;
	BRA     L7B7C			;Bra to continue
;
;~~~~~~~~~~~~~~~~~~~~~~
; Second 40 Hz cluster
;~~~~~~~~~~~~~~~~~~~~~~
;
L7B49	JSR     LCF65			;Routine to look up/calculate EGR flowrate
;
	JSR     LAF40			;Trans diag./misc. routine schedule
;
	BRA     L7B7C			;Bra to continue
;
;~~~~~~~~~~~~~~~~~~~~~~
; Second 80 Hz cluster
;~~~~~~~~~~~~~~~~~~~~~~
;
;
L7B51	JSR     L7E2E			;Main fuel loop
;
	JSR     LCC05			;Routine to output to EGR

	BRCLR   L003B,#$10,L7B6A	;Status word, bra if b4==0, no mode 4 engine diags.
;											
	LDAA    L0393			;
	BITA    #$80			;
	BEQ     L7B6A			;
;  
	BSET    L0044,#$04		; 
	BSET    L0043,#$80		;
	BRA     L7B6D			;
;
;-Here if no mode 4, continue and schedule 40 Hz clusters 
;
L7B6A	JSR     LF081			;Hardware routine

L7B6D	BRCLR   L0002,#$02,L7B79	;Loop counter, bra if b1==0, dont run routines yet
;
;~~~~~~~~~~~~~~~~~~~~~
; Third 40 Hz cluster
;~~~~~~~~~~~~~~~~~~~~~
;
L7B71	JSR     L94B3			;Main idle routine
;
	JSR     LC307			;40 Hz entry point for TPS handling
;
	BRA     L7B7C			;Bra to execute Real Time Subroutines
;
;~~~~~~~~~~~~~~~~~~~~~
; Forth 40 Hz cluster
;~~~~~~~~~~~~~~~~~~~~~
;
L7B79	JSR     LB3B5			;Schedule of transmission routines
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; Execute Real Time Subroutines
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
L7B7C	LDAB    L0002			;Loop counter
	ANDB    #$0F			;Clear all but lower 4 bits
	LDX     #L7BEA			;Addr. of RTS table
	ASLB				;x2 for double byte values
	ABX				;Add in as offset
	LDX     0,X			;Load RTS addr.
	JSR     0,X			;Go run the routine
;
;~~~~~~~~~~~~~~~~
; Back from loop
;~~~~~~~~~~~~~~~~
;								
	BCLR    L004F,#$04		;Status word, clear b2, loop overrun flag
	BRCLR   L0082,#$40,L7BA9	;Status word, bra if b6==0, no HU (mode 4 diags.)
;
;~~~~~~~~~~~~~~~~~~
; Mode 4 in effect
;~~~~~~~~~~~~~~~~~~
;				
	LDAB    L0002			;	
	ANDB    #$0F			;
	ASLB				;
	LDX     #L02F6			;
	ABX				
;
	LDD     L300E			; 
	SUBD    L301A			; 
	ADDD    #$0333			; 
	CPD     0,X			;
	BLS     L7BA9			;
;											
	STD     0,X			;
;
;~~~~~~~~~~~~~
; Refresh RAM 
;~~~~~~~~~~~~~
;
L7BA9	LDX     L02BE			;Pending RAM block to refresh
;
L7BAC	CPX     #$0400			;
	BNE     L7BB7			;Bra if current addr != $0400
;
;-Skip to next block of addressable memory to refresh
;											
	LDX     #$0800			;Move to $0800
	STX     L02BE			;
;
L7BB7	CPX     #$0900			;
	BEQ     L7BE6			;Bra if == $0900, all addressable RAM has been 
					;refreshed
;											
	SEI				;Disable interrupts 
;
;-Refresh 16 byte block of RAM
;    
	LDD     0,X			;
	STD     0,X			;

	LDD     $02,X			;
	STD     $02,X			;

	LDD     $04,X			;
	STD     $04,X			;

	LDD     $06,X			;
	STD     $06,X			;

	LDD     $08,X			;
	STD     $08,X			;

	LDD     $0A,X			;
	STD     $0A,X			;

	LDD     $0C,X			;
	STD     $0C,X			;

	LDD     $0E,X			;
	STD     $0E,X			;
;
;-Move on to next 16 byte block on next pass
;
	LDAB    #$10			;16 bytes
	ABX				;Add it in
	STX     L02BE			;Save it, pending RAM block to refresh
	CLI				;Enable interrupts     
	BRA     L7BAC			;Bra to continue 
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; Here to wait for interrupt
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
L7BE6	CLI				;Clear interrupt bit, enable interrupts
	WAI				;Wait for interrupts
	BRA     L7BE6			;Loop				
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Real Time Subroutine table
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;	Addr.				;RTS #
;
L7BEA	FDB	L7C0B			; 0
	FDB	L7C12			; 1	
	FDB	L7C1C			; 2
	FDB	L7C22			; 3
	FDB	L7C35			; 4
	FDB	L7C6B			; 5
	FDB	L7C7C			; 6
	FDB	L7C83			; 7
	FDB	L7C9C			; 8
	FDB	L7CA0			; 9
	FDB	L7CAA			;10
	FDB	L7CAE			;11
	FDB	L7CBE			;12
	FDB	L7CC2			;13
	FDB	L7CCC			;14
	FDB	L7CDD			;15
;
L7C0A	RTS				;
;
;~~~~~~~~~~~~~~~~~~~~~~~~~
;
; Real time subroutine #0
;
;~~~~~~~~~~~~~~~~~~~~~~~~~
;
L7C0B	JSR     LC3BA			;A/D leadin to retrieve FP volts
	JSR     LCD23			;3 -> 2 DC solenoid handling routine
	RTS				;Return     
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;~~~~~~~~~~~~~~~~~~~~~~~~~
;
; Real time subroutine #1
;
;~~~~~~~~~~~~~~~~~~~~~~~~~
;
L7C12	JSR     LC3CE			;MAP/Baro handling
	JSR     LC4B3			;Trans baro handling
	JSR     LAE65			;Non CC trans TCC handling
	RTS				;Return
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~     
;
;~~~~~~~~~~~~~~~~~~~~~~~~~
;
; Real time subroutine #2
;
;~~~~~~~~~~~~~~~~~~~~~~~~~
;
L7C1C	SEI				;Disable interrupts			     
	JSR     LEEAA			;Lead-in for A/D routine, multi channel read
	CLI				;Enable interrupts
	RTS				;Return
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;~~~~~~~~~~~~~~~~~~~~~~~~~
;
; Real time subroutine #3
;
;~~~~~~~~~~~~~~~~~~~~~~~~~
;
L7C22	JSR     LF26C			;Heads up routine
	JSR     LD425			;Routine to determine desired gear #
;
;-Check if CC trans in use
;
	LDX     #L400F			;Opt. byte
	BRCLR   0,X,#$C0,L7C31		;Bra if b6, non-CC transmission, and b7, manual 
					;transmission/TCC, ==0, run routine
;									
	BRA     L7C34			;Bra to return, no CC trans			   
;										   
L7C31	JSR     LC32A			;Routine to handle various trans I/O's
;											
L7C34	RTS				;Return    					
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;~~~~~~~~~~~~~~~~~~~~~~~~~
;
; Real time subroutine #4
;
;~~~~~~~~~~~~~~~~~~~~~~~~~
;											
L7C35	JSR     LC6D1			;Schedule of trans error routines
;
	BRCLR   L0002,#$10,L7C4A	;Loop counter, bra if ==0, run second cluster
;
;~~~~~~~~~~~~~~~~~~~~
; First 5 Hz cluster
;~~~~~~~~~~~~~~~~~~~~
;
	JSR     LE860			;Routine to check for error 58, high trans temp
;
	JSR     LE893			;Routine to check for error 59, low trans temp
;
	JSR     LC6F6			;Routine to look up linear trans temp
;
	JSR     LC71D			;Routine to handle trans temp related items
;										
	BRA     L7C59			;Bra to continue				
;
;~~~~~~~~~~~~~~~~~~~~~
; Second 5 Hz cluster
;~~~~~~~~~~~~~~~~~~~~~
;								
L7C4A	JSR     LE50E			;Routine to check for error 14, high cool temp
;										
	JSR     LE542			;Routine to check for error 15, low cool temp
;											
	JSR     LC78A			;Look up linear cool temp
;											
	JSR     LC7E0			;Routine to generate cool temp for trans
;											
	JSR     LC805			;Lookup routine of various temp. based items
;
;~~~~~~~~~~~~~~~~~~~~~~
; Check for TPS errors
;~~~~~~~~~~~~~~~~~~~~~~
;											
L7C59	BCLR    L006F,#$24		;Status word, clear b2, high TPS, and b5, 
					;low TPS error flags
	BRCLR   L0016,#$01,L7C63	;Error word #2, bra if b0==0, err 21, high TPS not set
;
;-Error 21 has been set, flag error
;		
	BSET    L006F,#$04		;Set b2, high TPS
;
L7C63	BRCLR   L0017,#$80,L7C6A	;Error word #3, bra if b7==0, error 22, low TPS not set
;
;-Error 22 has been set, flag error
;
	BSET    L006F,#$20		;Set b5, low TPS
;
L7C6A	RTS				;Return    
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;~~~~~~~~~~~~~~~~~~~~~~~~~
;
; Real time subroutine #5
;
;~~~~~~~~~~~~~~~~~~~~~~~~~
;
L7C6B	JSR     LD2D7			;Routine to check for catalytic converter reaction 
					;overtemp
;
	JSR     LD399			;CPI manifold variable tune routine
;
;-Check if CC trans. TCC in use
;
	LDX     #L400F			;Opt. byte
	BRSET   0,X,#$40,L7C7B		;Bra if b6==1, standard TCC in use, return
; 												; ... else
	JSR     LC506			;TCC routine
;
L7C7B	RTS				;Return     
;
;~~~~~~~~~~~~~~~~~~~~~~~~~
;
; Real time subroutine #6
;
;~~~~~~~~~~~~~~~~~~~~~~~~~
;
L7C7C	JSR     LC923			;Adaptive A/C control routine
;
L7C7E	JSR     LCC1E			;Low octane handling routine
;
	RTS				;Return     
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;~~~~~~~~~~~~~~~~~~~~~~~~~
;
; Real time subroutine #7
;
;~~~~~~~~~~~~~~~~~~~~~~~~~
;
;-Check if CC trans in use
;
L7C83	LDX     #L400F			;Option byte
	BRCLR   0,X,#$C0,L7C8C		;Bra if b6, non-CC transmission, and b7, manual 
					;transmission, ==0, cc trans in use
;
;-CC trans not in use here
;
	BRA     L7C8F			;Bra to continue
;
;-Here if using CC transmission
;
L7C8C	JSR	LC32A			;Routine to handle various trans I/O's
;
L7C8F	JSR     LF279			;Routine to init heads up (ALDL mode 4)
;
	JSR     L9F7C			;Initial startup idle handling and 
					;idle integrator coordination routine
;
	JSR     LC879			;Routine to do SA and Knock related look ups
;
	JSR     LAD62			;Shift light routine
;
	RTS				;Return 
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~    
;
;~~~~~~~~~~~~~~~~~~~~~~~~~
;
; Real time subroutine #8
;
;~~~~~~~~~~~~~~~~~~~~~~~~~
;
L7C9C	JSR     LCCC9			;3->2 solenoid and misc. knock sensor handling routine
;
	RTS				;Return     
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;~~~~~~~~~~~~~~~~~~~~~~~~~
;
; Real time subroutine #9
;
;~~~~~~~~~~~~~~~~~~~~~~~~~
;
L7CA0	JSR	LCE3D			;Routine to check params for EGR
;
	JSR	LA386			;Routine to decay out various idle terms
;
	JSR	LD414			;MPH filtering routine
;
	RTS				;Return 		    
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; Real time subroutine #10
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~
;
L7CAA	JSR     LD1D9			;Charcoal canister purge routine
;
	RTS 				;Return    
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; Real time subroutine #11
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~
;
L7CAE	JSR     LD425			;Routine to determine desired gear #
;
;-Check if CC trans in use
;
	LDX     #L400F			;Load option byte
	BRCLR   0,X,#$C0,L7CBA		;Bra if b6, non-CC TCC, and b, manual in use ==0
;
;-Non-CC trans in use
;
	BRA     L7CBD			;Bra to return
;
L7CBA	JSR     LC32A			;Routine to handle various trans I/O's
;
L7CBD	RTS				;Return 
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; Real time subroutine #12
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~
;
L7CBE	JSR     LCA43			;Hardware config routine
;
	RTS				;Return    
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; Real time subroutine #13
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~
;
L7CC2	JSR     LCACF			;Output handling routine
;
	JSR     LD9BB			;Routine to handle MAT 
;
	JSR     LDA49			;Engine error code/diagnostics handling routine
;
	RTS				;Return   
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; Real time subroutine #14
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;-Check and reset pending RAM addr. block to refresh when complete
;
L7CCC	LDX     L02BE			;Pending RAM addr. to be refreshed
	CPX     #$0900			;Value when RAM refresh complete
	BEQ     L7CD9			;Bra if ==, reset and continue
;
;-RAM refresh was interrupted
;
	BSET    L0004,#$04		;Status word, set b2, RAM refresh incomplete 
	BRA     L7CDC			;Bra to return
;
;-RAM refresh complete
;
L7CD9	CLR     L02BE			;Clear pending RAM addr. to be refreshed
;
L7CDC	RTS				;Return
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~     
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; Real time subroutine #15
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;-Check if CC trans in use
;
L7CDD	LDX     #L400F			;Index option byte
	BRCLR   0,X,#$C0,L7CE6		;Bra if b6, non-CC TCC, and b7, manual in use ==0
;
;-Non-CC trans in use
;
	BRA     L7CE9			;Bra to continue
;
L7CE6	JSR     LC32A			;Routine to handle various trans I/O's
;
L7CE9	JSR     LC30B			;Trans error handling subroutines schedule table
;
	JSR     LE3ED			;Routine to filter out related errors
;
	JSR     LE4B9			;Not used...
;
	JSR     LE046			;Routine for handling closed loop entry and non 
					;time critical lookups
;
	RTS				;Return 
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; Force motor PID control routine
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
L7CF6	BRSET   L004C,#$20,L7D08	;I/O port, bra if b5==1, force motor enabled
;
;-Force motor has been disabled
;											
	CLRA				;Perform clears
	CLRB				;
	STD     L014A			;Force motor integrator									
	STAA    L0146			;Force motor %DC
	STAA    L0149			;Force motor current error
	JMP     L7D8B			;Bra to save and continue				
;
;-Read in previous error for velocity calculation
;
L7D08	LDAA    L0149			;Load stored force motor current error
	LDAB    L014E			;Previous proportional gain vs. cool temp
	MUL				;Gain x error
	TST     L0149			;Test stored force motor current error
	BPL     L7D17			;Bra if error is positive
;											
	SUBA    L014E			;Subtract mult, now = -gain x |error|
;
L7D17	PSHB				;Push to stack
	PSHA				;
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Get force motor current error
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
	LDAA    L0147			;Desired force motor current
	CMPA    #57			;1113 mA, max allowed
	BLS     L7D22			;Bra if <=
;											
	LDAA    #57			;Load max
;
L7D22	SUBA    L0148			;Subtract A/D force motor current
	RORA				;Check for signed overflow
	ROLA				;
	BVC     L7D2D			;Bra if overflow clear
;
;-Clear invalid result
;
	LDAA    #127			;
	ADCA	#00			; 
;
;-Check min limits on force motor current error
;
L7D2D	CMPA    #01			;19.5 mA
	BEQ     L7D35			;Bra if ==, clear error
;
	CMPA    #255			;-19.5 mA
	BNE     L7D36			;Bra if !=, continue
;
L7D35	CLRA 				;Clear force motor current error
;
L7D36	STAA    L0149			;Save it, force motor current error 
	LDAB    L014D			;Prop gain vs. trans temp
	MUL				;Gain x force motor current error 
	TST     L0149			;Test error
	BPL     L7D45			;Bra if error >= 0
;
	SUBA    L014D			;Subtract mult, now = -gain x |error|
;
;-Construct the proportional velocity term 
;
L7D45	TSX				;Get stack pointer into X
	SUBD    0,X			;Subtract previous correction to get the rate 	
	BVC     L7D51			;Bra if oveflow clear
;
;-Clear invalid result
;											
	LDD     #$8000			;
	SBCB    #00			;
	SBCA    #00			;
;
;-Add the proportional velocity term into the integrator
;
L7D51	ASRA				;/2
	RORB				;
	ASRA				;/2, now /4
	RORB				;
	ADCB    #00			;Round if needed
	ADCA    #00			;
	PULX				;Clean up the stack
	TSTA				;
	BMI     L7D6B			;Bra if negative result
;
;-Positive result
;											
	ADDD    L014A			;Add it into the force motor integrator
	CPD     #$0E00			;Max duty cycle
	BLS     L7D79			;Bra if duty cycle is <=
											
	LDD     #$0E00			;Load max duty cycle
	BRA     L7D79			;
;
;-Negative result
;											
L7D6B	ADDD    L014A			;Add in prop. term to integrator
	BCC     L7D76			;Bra if no carry, result still < 0
;											
	CPD     #$0000			;
	BCC     L7D79			;Bra if magnitude of result >= 0 
											
L7D76	LDD     #$0000			;Clear result
;
;-Here to save force motor output
;
L7D79	STD     L014A			;Save it, force motor integrator
;											
L7D7C	LDX     #$0E00			;Load max duty cycle
	FDIV				;(force motor integrator)/(max DC) x 65536
	XGDX				;Get result into D, now %DC
	ADDD    #128			;Round up
	BCC     L7D88			;Bra if no overflow
;											
	LDAA    #255			;Clear overflow
;
L7D88	STAA    L0146			;Save it, force motor DC
;
;-Check if force motor fault or low batt volts flagged previously
;
L7D8B	BRSET   L008A,#$80,L7DC6	;Status word, bra if b7==1, force motor fault
					;present
;											
	BRSET   L0082,#$01,L7DC6	;Status word, bra if b0==1, batt. volts too low to  
					;flag force motor fault
;
;-Check for integrator saturation
;
	LDD     L014A			;Read force motor DC
	CPD     #$0E00			;Full DC
	BNE     L7DC6			;Bra if !=, integrator has not reached saturation
					;threshold
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Integrator has saturated, force motor fault detected
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;											
	LDAA    L00F6			;Force motor fault timer
	INCA				;+1
	CMPA    L6925			;Timer threshold to set force motor fault
	BLS     L7DC4			;Bra if <=, dont set fault yet
;												
	CLR     L00F6			;Force motor fault timer
	LDAA    L6927			;Load upper voltage range
	SUBA    L6926			;Subtract lower voltage range, now total span
	LDAB    L00AF  			;Trans temp
	MUL				;Trans temp x voltage span 
	ADDD    #128			;Round if needed
	ADDA    L6926			;Add in lower voltage, now temp based threshold
	CMPA    L00A1			;Compare to battery volts
	BLS     L7DBF			;Bra if batt volts >= 
;
;-Battery volts are too low for diags.
;											
	BSET    L0082,#$01		;Status word, set b0, batt. volts too low to flag
					;force motor fault
	BRA     L7DC6			;Bra to continue
;
;-Force motor fault present
;
L7DBF	BSET    L008A,#$80		;Status word, set b7, force motor fault
	BRA     L7DC6			;Bra to continue
;
L7DC4	STAA    L00F6			;Save force motor fault timer value
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Here if fault flagged or low battery volts have been present
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
L7DC6	BRSET   L008A,#$80,L7DDA	;Bra if b7==1, force motor fault
;											
	BRSET   L0082,#$01,L7DDA	;Bra if b0==1, batt. volts too low for force motor
					;diags.
;
;-No errors previously set
;											
	LDD     L014A			;Force motor integrator
	CPD     #$0E00			;Maximum DC
	BCC     L7DDA			;Bra if >=, integrator has saturated
;											
	CLR     L00F6			;Clear force motor fault timer
;
L7DDA	BRCLR   L0082,#$01,L7E03	;Bra if b0==0, no low battery volts previously
					;detected, return
;
;-Fault present and/or low battery volts
; 
	LDAA    L6929			;Upper voltage threshold for clearing low batt
					;volts flag
	SUBA    L6928			;Subtract lower voltage threshold for clearing
					;low batt volts flag
	LDAB    L00AF			;Load trans temp
	MUL				;Temp x voltage span
	ADDD    #128			;Round if needed
	ADDA    L6928			;Add in lower threshold, now threshold for flagging
					;force motor fault
	CMPA    L00A1			;Compare to battery volts
	BCC     L7E03			;Bra if battery volts <=, dont reset
;											
	LDAA    L00F6			;Force motor fault timer
	INCA				;+1
	CMPA    L6925			;Timer threshold to set force motor fault
	BLS     L7E01			;Bra if timer <=
;											
	CLR     L00F6			;Clear timer
	BCLR    L0082,#$01		;Clear b0, reset low batt. volts flag 
	BRA     L7E03			;Bra to continue
;
L7E01	STAA    L00F6			;Save force motor fault timer
;
L7E03	RTS 				;Return    
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; Brake status routine 
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;-L004D, 1=brakes not applied, 0=brakes applied
; 
L7E04	BRCLR	L004D,#$02,L7E0E	;Port C I/O word, bra if brake switch on
; 
;-Brakes not applied here
;
	BRSET   L0096,#$01,L7E17	;Status word, bra if brakes where previously applied
;											
	BRA     L7E12			;Bra, no change in state 
;
;-Brakes applied here
;
L7E0E	BRCLR   L0096,#$01,L7E17	;Status word, bra if brakes whre previously not applied
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-No change in state, clear timer and return
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
L7E12	CLR     L00F5			;Clear brake status min change state timer
	BRA     L7E2D			;Bra to return
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Change in state, update if state remains the same for long enough
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
L7E17	LDAB    L00F5			;Brake status min change state timer
	CMPB    L5D04			;Min time to change state 
	BCC     L7E23			;Bra if >=
;											
	INCB				;+1
	STAB    L00F5			;Save it, brake status min change state timer
	BRA     L7E2D			;Bra to return
;
;-Here to update
;
L7E23	BCLR    L0096,#$01		;Status word, preclear b1, brake status bit
	BRSET   L004D,#$02,L7E2D	;Port C I/O word, bra if brakes not applied
;											
	BSET    L0096,#$01		;Status word, set b0, brakes applied
;
L7E2D	RTS				;Return    
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;~~~~~~~~~~~~~~~~
;
; Main fuel loop
;
;~~~~~~~~~~~~~~~~
;
L7E2E	LDX     L02A1			;Shutdown timer  
	BRCLR   L0044,#$10,L7E73	;Status word, bra if b4==0, ignition is still on	
;
;~~~~~~~~~~~~~~~~~~~~	
; Shutting down here
;~~~~~~~~~~~~~~~~~~~~
;						
	CPX     #$0008			;
	BCS     L7E90			;
;
	BCLR    L0004,#$83		;Status word, clear b7, EST error,
					;b1, Closed loop timer, and 
					;b0, O2 sensor ready
	BCLR    L0019,#$40		;Clear b6. error 41, 1x cam sens. CLR b6, ERR 41, 1x CAM SENSOR
	BCLR    L0019,#$20		;Clear b5, Err 42, EST  
	CLRA				;    
	CLRB				;    
	STD     L00F8			;Clear engine run time
	STD     L026F			;Clear sync BPW
	STAA    L0062			;Clear rpm/25
	STAA    L02F0			;Clear DRP counter
	STAA    L02F1			;Clear DRP counts
	STAA    L028B			;Clear elapsed DRP
	BSET    L0051,#$08		;Status word, set b3, shutdown in effect
	BCLR    L0082,#$08		;Status word, clear b3, ignition is off
	BCLR    L004F,#$80		;Status word, clear b7, engine running flag
	BCLR    L004F,#$10		;Clear b4, engine off
;
	BCLR    L0004,#$08		;Status word, clear b3, bad shutdown
	CPX     L4009			;Timer threshold for shutdown
	BCS     L7E90			;Bra if less, not shutting down yet	
;
;-Momentarily shut down the MCU
;										
	LDD     #$5008			;0101 0000 0000 1000
					;CCR: Allow stops, mask XIRQ, disable interrupts,
					;and set carry bit
					;Opt. reg: Enable clock monitor
	STAB    L3039			;Save B to option reg.
	TAP				;Load cond. code reg. from A
	STOP				;Stop MCU clocks
L7E71	BRA     L7E71			;Loop here if stop not allowed			
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Here if not shutting down
;~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
L7E73	BRCLR   L0051,#$08,L7E8A	;Status word, bra if b3==0, not shutting down
;											; ... else
	BCLR    L0051,#$08		;Clear b3, not shutting down yet
	BCLR    L0046,#$80		;Status word, clear b7, have been in closed lp once
					;since startup
	BCLR    L0044,#$08		;Status word, clear b3, first valid DRP recieved
	BCLR    L003E,#$08		;Status word, clear b3, A/F decay int done for cold P/D
;											
	CLRA				;
	STAA    L0246			;Clear EGR fail timer
	STAA    L02A0			;Clear cold P/N timer
;
L7E8A	BSET    L0082,#$08		;Status word, set b3, ingition is on
	LDX     #$FFFF			;Clear engine shutdown timer
;
L7E90	INX				;+1
	STX     L02A1			;Save it shutdown timer
;
;-Service COP
;
	LDAA    #$55			;0101 0101
	STAA    L303A			;COP reset reg.
;
L7E99	LDAA    L0041			;Status word
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Check quals for transient MAP
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
	ANDA    #$10			;Clear all but b4, A/C on/off flag 
	TST     L400F			;Opt. byte
	BMI     L7EA7			;Bra if b7==1, manual trans in use 
;
;-Here if TCC in use
;											
	LDAB    L0085			;Status word
	ANDB    #$20			;Clear all but b5, TCC commanded to lock/fully locked
					;flag
	ABA				;Add it into A
;
;-Here if manual tran in use
;
L7EA7	LDAB    L0047			;Status word
	ANDB    #$FE			;Mask all but b0
	CBA				;Compare A + B
	BNE     L7ECA			;Bra if !=, change in status
;
;-No changes in A/C, TCC status here
;											
	LDAB    L0063			;Load RPM/12.5
	SUBB    L0065			;RPM/12.5, one delayed
	BCC     L7EB5			;Bra if RPMs rising
;
;-RPMs falling here
;										
	NEGB				;Get abs. value of delta RPMs
;
L7EB5	CMPB    L413B			;Delta RPM theshold for transient MAP
	BHI     L7ECA			;Bra if delta RPMs greater
;											
	BRCLR   L0050,#$80,L7ECA	;Status word, bra if b7==0, not idling
;											
	BRCLR   L003E,#$A0,L7ECA	;Status word, bra if b5, b7==0
					;not in closed loop, no open loop idle condition
;											
	LDAB    L01E8			;Load transient MAP apply timer
	BEQ     L7ED2			;Bra if ==0, no transient MAP
;
;-Here to apply transient MAP for time specified
;											
	DECB				;-1,  
	BRA     L7ECD			;Bra to save and set flag				 
;
;-Here to apply transient MAP
;										
L7ECA	LDAB    L413A			;Transient MAP apply timer val.
;
L7ECD	STAB    L01E8			;Save it, transient MAP apply timer 
	ORAA    #$01			;Set b0, do transient MAP
;
;-Here for no transient MAP
;
L7ED2	STAA    L0047			;Save it, status word
	LDAA    L01FC			;Load %TPS
;
	STAA    L01FD			;TPS, one delayed
	JSR     LB270			;Routine to construct TPS
;
	BRSET   L0092,#$01,L7EE6	;Status word, bra if b0==1, trans TPS error
;											
	STAA    L01FC			;Save it, TPS
	BRA     L7EEC			;Bra to continue
;
;-Here if TPS fault
;
L7EE6	LDAA    L5B23			;Default TPS if error
	STAA    L01FC			;Save it, %TPS
;
;~~~~~~~~~~~~~~~
; Filter the O2
;~~~~~~~~~~~~~~~
;
L7EEC	LDAB    L48B3			;Slow O2 idle filter coeff.
;
L7EEF	BRSET   L0050,#$80,L7EFD	;Status word, bra if b7==1, idling
;
;-Not idling here, load slow filter coeff.
;
	LDAA    L029E			;Scaled airflow
	LDX     #L4CFB			;Slow O2 filter coeff. vs airflow table
	JSR     LF15E			;2D lookup
;											
	TAB				;O2 filter coeff. to B
;
;-Check lower O2 limit for filtering
;
L7EFD	LDAA    L48B1			;Lower O2 limit for slow filtering
	CMPA    L01F8			;O2 A/D volts
	BHI     L7F10			;Bra if O2 <
;
;-Check upper O2 limit for filtering
;											
	LDAA    L48B2			;Upper O2 limit for slow filtering
	CMPA    L01F8			;O2 A/D volts
	BCS     L7F10			;Bra if O2 >
;
;-Load O2 volts
;
	LDAA    L01F8			;O2 A/D volts
;
;-Here to limit slow O2 
;
L7F10	LDX     L01F5			;Slow filtered O2
	JSR     LF0F6			;Lag filter routine
;											
	STD     L01F5			;Save it, slow filtered O2
	LDAA    L400F			;Opt. byte
	ANDA    #$01			;Clear all but b0, governor opt.
	BEQ     L7F23			;Bra if ==0, no governor present
;									
;-Here if using governor
;		
	JSR     L91B6			;Governor actuator subroutine 	
;
;-Calculate actual kPa x 256 from A/D map
;			
L7F23	LDAB    L01E9			;A/D MAP
	CLRA 				;  					
	LDX     #0694			;256 x .369
	FDIV				;Fractional Div
	XGDX 				;Get result into D
	ADDD    #2649			;+2649, 256 x 10.34
	STD     L027A			;MAP value, kPa x 256
;											
	BRSET   L004F,#$10,L7F89	;Status word, bra if b4==1, engine running
;
;~~~~~~~~~~~~~~~~~
; Engine off here
;~~~~~~~~~~~~~~~~~
;											
	JSR     LAF17			;VATS routine
;
	CLRB				;
	LDAA    L01E3			;SD MAP
	STD     L01EA			;SD MAP, filtered per cool temp
;											
	LDAA    L01FC			;TPS
	STD     L0201			;Slow filtered %TPS
	STD     L01FF			;TPS, filtered based on cool temp
;											
	JSR     L8695			;Crank fuel lookup routine
;											
	ASLD				;BPW x2
	BCC     L7F52			;Bra if no overflow
											
	LDD     #$FFFF			;Clear overflow
;
L7F52	STD     L02F2			;Save it, cranking PW
	JSR     LC3BA			;FP volts A/D lead-in
;
;-Look up BPW correction vs. fuel pump volts
;
	LDAA    L0055			;Load fuel pump volts
	LDX     #L497E			;BPW mult. vs fuel pump volts
	JSR     LF15E			;2D lookup
;
	LDX     #L02F2			;Cranking PW
	JSR     LF1ED			;(A x X*)/256
;											
	STD     L02F2			;Save it, cranking PW
;
;-Check and perform crank fueling delay per DRPs
;
	LDX     #L400C			;Option byte 
	BRSET   0,X,#$04,L7F7D		;Bra if b2==1, no DRP adjustments to crank fueling
;
	LDAA    L0006			;Cool temp, linear
	LDX     #L4DC3			;Crank fueling delay vs. cool temp table
	JSR     LF15E			;2D lookup
;											
	CMPA    L02F0			;Compare to DRP counter
	BLS     L7F80			;Bra if <= to DRP counter
;
;-Here if not enough DRPs for crank fueling yet
;											
L7F7D	JMP     L860F			;Skip save
;
;-Enough DRPs for crank fueling here
;				
L7F80	LDD     L02F2			;Crank PW
	JSR     L78D4			;Crank PW correction routine
;											
	JMP     L85D8			;Go save and continue				
;
;~~~~~~~~~~~~~~~~~~~~~
; Engine running here
;~~~~~~~~~~~~~~~~~~~~~
;
L7F89	JSR     LAF17			;VATS routine
;
;-Check for VATS fail
;
	BRSET   L003D,#$02,L7F95	;Status word, bra if b1==1, VATS passed
;
;-Here if VATS failed
;		
	CLRA				;Clear PW
	CLRB				;
	JMP     L85D8			;Bra to save and continue				
;											
L7F95	CLRB				;
	LDAA    L01E3			;SD MAP
	SUBD    L01EA			;SD MAP, filtered per cool temp
	BLS     L7FFA			;Bra if MAP falling, no MAP AE
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; MAP rising, see about accel enrich
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;										   	
	ADDD    #128			;Round up
	STAA    L01ED			;Save it, delta MAP for AE
;
;-Check quals for MAP AE
;										   	
	LDAB    L48B6			;delta MAP threshold for AE if idling
	BRSET   L0050,#$80,L7FAE	;Status word, bra if b7==1, iding
;
	LDAB    L48B5			;Delta MAP threshold for AE
;
;-Double delta MAP threshold if TPS high enough
;										
L7FAE	LDAA    L01FC			;TPS
	CMPA    L48B7			;TPS threshold to double delta MAP threshold
;											
	BHI     L7FBB			;Bra if TPS >
;											
	ASLB				;Delta MAP threshold x2
	BCC     L7FBB			;Bra if no overflow
;											
	LDAB    #255			;Clear overflow
;											
L7FBB	LDAA    L01ED			;Delta MAP for AE
	CBA				;Compare to threshold
	BLS     L7FFD			;Bra if delta MAP AE is less
;
;~~~~~~~~~~~~~~~~~
; Do delta MAP AE
;~~~~~~~~~~~~~~~~~
;
	LDX     #L4B3F			;MAP AE anyc PW vs delta MAP
	JSR     LF153			;2D lookup
;
;-Check if enough TPS to double MAP AE
;
	LDAB    L01FC			;%TPS
	CMPB    L48B8			;TPS threshold to double MAP AE
	BLS     L7FD4			;Bra if TPS <=
;
	ASLA				;AE x 2
	BCC     L7FD4			;Bra if no overflow
;											
	LDAA    #255			;Clear overflow
;
;-Perform RPM correction
;
L7FD4	PSHA				;Push sync AE to stack   
	LDAA    L0062			;RPM/25
	LDAB    #16			;400 RPM, lower limit for lookup
	LDX     #L4B9A			;MAP AE 
	JSR     LF137			;2D lookup with UL
;
	PULB				;Get sync AE back    
	MUL				;AE x RPM correction factor     
	ASLD				;x2    
	BCC     L7FE6			;Bra if no overflow
;						
	LDAA    #255			;Clear overflow
;
;-Perform temp correction
;
L7FE6	LDAB    L0260			;AE temp correction mult.
	MUL				;AE x mult.     
	LSRD				;/2
	LSRD				;/2    
	LSRD				;/2    
	LSRD				;/2
	LSRD				;/2, now AE mult divided by 32    
	TSTA				;Test MSB    
	BEQ     L7FF4			;Bra if no overflow
;
	LDAB    #255			;Clear overflow
;
L7FF4	TBA				;Get AE into A     
	BSET    L0005,#$80		;Status word, set b7, MAP AE in effect
	BRA     L8001			;Bra to save
;
;~~~~~~~~~~~~~~~~~~~
; Here if no MAP AE
;~~~~~~~~~~~~~~~~~~~
;
L7FFA	CLR     L01ED			;Clear delta SD MAP for AE
;
L7FFD	CLRA				;clear MAP AE
	BCLR    L0005,#$80		;Clear b7, MAP AE no longer in effect
;
;-Here to continue with AE
;											
L8001	STAA    L0261			;Save it, MAP AE PW
;
;~~~~~~~~~~~~~~~~
; Perform TPS AE
;~~~~~~~~~~~~~~~~
;											
	LDAA    L01FC			;TPS
	SUBA    L01FF			;AE TPS, filtered based on cool temp
	BLS     L804F			;Bra if TPS is falling, no AE
;
;-TPS is rising
;											
	TST     L0200			;Test LSB
	BPL     L8014			;Bra if < 128
;											
	DECA				;Round down
	BEQ     L804F			;Bra if ==0, no TPS AE
;											
L8014	BRSET   L0005,#$40,L801D	;Bra if TPS AE already in effect 
;
;-Check quals for TPS AE
;											
	CMPA    L48B9			;TPS threshold for TPS AE
	BLS     L804F			;Bra if TPS <=, no TPS AE
;											
L801D	ASLA				;Delta TPS x2
	BCC     L8022			;Bra if no overflow
;											
	LDAA    #255			;Clear overflow
;
;-Look up AE
;
L8022	LDX     #L4B45			;AE vs delta TPS
	JSR     LF15E			;2D lookup
;
	LDAB    L02A7			;MPH
	CMPB    L48BB			;MPH threshold to apply multiplier
	BCC     L803D			;Bra if MPH >=
;
;-Apply off-idle TPS AE multiplier
;										    
	LDAB    L48BA			;TPS AE off-idle multiplier
	MUL				;AE x multiplier    
	ADDD    #64			;round if needed
	ASLD				;x2, now /128    
	BCC     L803D			;Vra if no overflow
											
	LDD     #$FFFF			;Clear overflow
;
L803D	LDAB    L0207			;Delta TPS AE baro factor
	MUL				;Baro factor x AE     
	ADDD    #64			;Round if needed
	ASLD				;x2. now /128    
	BCC     L804A			;Bra if no overflow
;										  	
	LDD     #$FFFF			;Clear overflow
;
L804A	BSET    L0005,#$40		;Status word, set b6, TPS AE in effect
	BRA     L8053			;Bra to continue
;
;~~~~~~~~~~~~~~~~~~~
; Here if no TPS AE
;~~~~~~~~~~~~~~~~~~~
;
L804F	CLRA				;Clear AE    
	BCLR    L0005,#$40		;Clear b6, TPS AE no longer in effect
;
L8053	STAA    L0262			;Save it, TPS AE
	BRCLR   L0002,#$02,L8069	;Loop counter, bra if b1==0, no filtering this pass
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Here to filter TPS @ 40 Hz
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
	LDAB    L0208			;AE TPS filtering coeff.
	LDAA    L01FC			;TPS
	LDX     L01FF			;AE TPS, filtered based on cool temp
	JSR     LF0F6			;Filter routine
;
	STD     L01FF			;Save it, filtered TPS
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Construct needed fueling terms
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;-Scale SD MAP for lookup, 20-40 in 5 kPa incs., 40+ in 10 kPa incs.
;
L8069	LDAB    L01E3			;SD MAP
	CMPB    #64			;40 kPa
	BLS     L8075			;
;											
	LSRB				;Div. MAP by 2 
	ADCB    #$00			;Round if needed
	ADDB    #32			;Add in 40 kPa offset
;
;-Check TPS quals for idle VE 
;
L8075	LDAA    L01FC			;TPS
	CMPA    L48D0			;TPS threshold for idling
	BCC     L808B			;Bra if TPS >=
;
;-Check MPH quals for idle VE 
;
	LDAA    L02A7			;MPH
	CMPA    L48CE			;Upper MPH limit for idle VE table
	BCC     L808B			;Bra if MPH >=
; 
;-Check RPM limits for idle VE
;
	LDAA    L0062			;RPM/25
	CMPA    #72			;1800 RPM
	BCS     L8098			;Bra if RPMs <=, use idle VE table
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Use open throttle VE table
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
L808B	LDAA    L0062			;RPM/25
	LDX     #L49CB			;Open throttle VE table
	BSET    L004F,#$08		;Status word, set b3, switch to open VE table
	JSR     LF17B			;3D lookup
;
	BRA     L80B8			;Bra to continue
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Here to use idle VE table
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
L8098	LDAA    L0063			;RPM/12.5
	CMPA    #144			;1800 rpm, max val. for lookup
	BLS     L80A0			;Bra if RPMs <=
;											
	LDAA    #144			;Load max
;
L80A0	LDX     #L4A7E			;Addr. of idle VE table
	JSR     LF17B			;3D lookup
	BRCLR   L004F,#$08,L80B8	;Bra if b3==0, not previously using open throttle VE
;
;-Reset INT if needed
;												
	LDAB    L0232			;Current INT
	CMPB    L4905			;Lower INT threshold to reset if switching tables 
	BHI     L80B5			;Bra if INT >, no reset
;											
	BSET    L004F,#$20		;Status word, set b5, reset INT on VE table change
L80B5	BCLR    L004F,#$08		;Clear b3, switch to open VE table
;
;-Here if using open throttle VE table
;
L80B8	STAA    L0254			;Save it, VE
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Calculate mass of air in cylinder
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;		           [kPa x 256] x [VE x 256] x [cyl. volume x 222]
; Mass of air term [g]  = ------------------------------------------------
;		                     [air density x 45064]
;
; Note: kPa ref pressure incorperated into air density multiplier
;
;
	LDD     L027A			;MAP kPa x 256
	LDX     L0259			;Inv. air density val.
	FDIV				;kPa / Inv. air density val.    
	PSHX				;Result to stack   
	TSX				;Get current stack pointer     
	LDAA    L4D8A			;Cyl volume
	JSR     LF1ED			;(A x X*)/256
;											
	PULX				;Get previous result back
	STD     L025B			;Save it, max mass of air in cyl.term
	LDAA    L0254			;Load VE
	LDX     #L025B			;Addr. of max. mass of air
	JSR     LF1ED			;(A x X*)/256
;											
	STD     L0257			;Save it, mass of air in cyl. x 32768
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Calculate overall airflow
;~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;		              g/cyl x 32768
; Airflow [g/s] = 256 x 2 x ---------------- x revs/sec x 4
;			          65536
;
;
;
	LDX     L020B			;Load DRP period
	IDIV				;Int. div., divide airflow by drp period    
	PSHX				;Result to stack    
	LDX     L020B			;DRP period
	FDIV				;Fract. divide, resolve remainder    
	XGDX				;Get result into D   
	TAB				;Divide result by 256     
	PULA				;Get MSB of airflow    
	TSTA				;Test MSB of result   
	PULA				;Get LSB of result    
	BNE     L80ED			;Bra if MSB !=0, result out of range
;											
	ASLD				;Result x 2, now gms/sec x 256    
	BCC     L80F0			;Bra if no overflow
											
L80ED	LDD     #$FFFF			;Clear overflow
;
L80F0	STD     L0255			;Save it, gms/sec x 256
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Divide mass air in cyl by AFR to get needed mass of fuel
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;			            g/cyl
; mass of fuel [g] = 819.2 x 256 x -------
;				     AFR
;
	LDX     L0257			;grams of air per cyl. x 32768
	LDAA    L026D			;AFR x 10
	CLRB				;Clear LSB for divide    
	XGDX				;Swap AFR and g/cyl.    
	LSRD				;/2  
	LSRD				;/2    
	FDIV				;([g/cyl x 8192]/[AFR x 2560]) x 65536    
	XGDX				;Get result into D    
;
;-Make mass fuel needed a time based PW
;
	LDX     L4D88			;Injector  flowrate 
	IDIV				;Int. divide, mass of fuel / mass flowrate    
	PSHX				;Push result to stack    
	LDX     L4D88			;Injector flowrate into X
	FDIV				;Fract. divide, resolve remainder    
	XGDX 				;Get result into D   
	TAB				;/256     
	PULA				;MSB of idiv result    
	TSTA				;Test MSB    
	PULA				;Get LSB or result into A  
	BEQ     L8112			;Bra if MSB ==0, result is within range
;											
	LDD     #65535			;Clear out of range result
;
;		  grams of fuel/cyl
; BPW = 65536 x ---------------------
;		 flowrate of injector
;
L8112	STD     L0271			;Save it, sync BPW
;
;-Adjust injector pw to compensate for EGR
;
	LDD     L01D5			;%EGR flow
	ADDD    #128			;Round if needed
	LDX     #L0271			;Sync BPW
	JSR     LF1ED			;(A x X*)/256
;											
	LSRD				;/2
	LSRD				;/2    
	STD     L0275			;Save it, EGR BPW correction
;
	LDD     L0271			;Sync BPW
	SUBD    L0275			;-EGR BPw correction
	STD     L0271			;Save it, sync BPW
;
;-Apply BLM
;
	LDAA    L026B			;BLM
	BMI     L8138			;Bra if BLM >= 128, apply correction regardless
;
;-Check for PE operation
;
	BRSET   L003D,#$20,L813B	;Status word, bra if b5==1, in PE 
;											
L8138	JSR     L81D1			;Go add blm correction to the BPW
;
;-In PE here
;
L813B	BRSET   L0016,#$10,L817F	;Error word #1, bra if b4==1, VSS error 
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~
; Trans anti-abuse routine
;~~~~~~~~~~~~~~~~~~~~~~~~~~
;
	LDX     #L400F			;Opt. byte
	BRSET   0,X,#$80,L8156		;Bra if b7==1, manual trans in use
;										
	LDAA    L514B			;Time for anti-abuse routine to be in effect
	BRSET   L0041,#$20,L8153	;Status word, bra if b5==1, in P/N
;											
	LDAA    L084D			;Trans abuse timer
	BEQ     L817F			;Bra if ==0, skip anti-abuse routine
;											
	DECA				;-1
;
;-In P/N, check quals for limiting engine speed/torque
;
L8153	STAA    L084D			;Trans abuse timer
;	
;-Check quals for limiting engine RPMs during shift
;										
L8156	LDAA    L0812			;MPH
	CMPA    L514C			;Min MPH threshold to check for abuse
	BHI     L817F			;Bra if MPH >, skip anti-abuse routine
;											
	LDAA    L01FC			;TPS
	CMPA    L514F			;Min TPS threshold to check for abuse
	BLS     L817F			;Bra if TPS <=, skip anti-abuse routine
;											
	LDAA    L514D			;RPM threshold to enter torque management
	BRCLR   L0053,#$08,L8170	;Bra if b3==0, not currently in abuse mode
;	
	LDAA    L514E			;RPM threshold to exit abuse mode
;
L8170	CMPA    L0061			;Scaled RPMs
	BCS     L8179			;Bra if RPMs >
;
;-Here to exit abuse mode
;											
	BCLR    L0053,#$08		;Clear b3, abuse mode no longer in effect
														 
	BRA     L817F			;Bra to continue
;
;-Decay out fuel to prevent trans abuse during shifts or clutch releases
;
L8179	BSET    L0053,#$08		;Set b3, trans abuse mode in effect, limiting engine
					;RPMs
	JMP     L8300			;Bra to decay out fuel
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Check and limit excessive engine RPMs/vehicle speed
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
L817F	LDX     #L400F			;Opt. byte
;
L8182	BRCLR   0,X,#$01,L818A		;Bra if b0==0, governor option not selected
;
;-Here if governor in use, use routine if errors
;		
	BRCLR   L006F,#$34,L81E9	;Bra if b2,high TPS, b4, b5, low TPS==0 
;											
L818A	LDX	#L495D			;RPM limit to enter fuel cut-off
	LDY     #L4961			;MPH limit enter fuel cut-off
;											
	BRCLR   L0046,#$10,L8198	;Status word, bra if b4==0, no fuel cut-off
;
;-Fuel cut-off in effect
;									
	INX				;+1, RPM limit to exit fuel cut-off
	INY				;+1, MPH threshold to exit fuel cut-off
;
L8198	BRSET   L0016,#$10,L81A0	;Err. word 31, bra if b4==1, VSS error
;											
	BRCLR   L0093,#$03,L81A2	;Status word, bra if b0==0, no external errors, 
					;and b1==0, no shift error
;
;-Trans errors present
;											
L81A0	INX				;+1
	INX				;+1, index alternate fuel cut-off threshold
;
;-Check for high RPMs and cut fuel 
;
L81A2	LDAB    L0062			;RPM/25
	CMPB    0,X			;Threshold to cut fuel
	BHI     L81C5			;Bra if RPMs >, cut fuel
;
;-Check for high speed and cut fuel if needed 
;											
	LDAB    L02A7			;MPH
	CMPB    0,Y			;MPH threshold to cut fuel
	BLS     L81E9			;Bra if MPH <=
;	
;-High speed present
;										
	LDAB    L0062			;RPM/25
	CMPB    L495C			;RPM threshold to re-enable fuel 	
	BLS     L81E9			;Bra if RPMs <=
;											
	LDAB    L028D			;Fuel cut-off delay timer
	CMPB    L495B			;Delay to cut fuel
;											 
	BCC     L81C5			;Bra if timer >=
;											
	INCB				;+1
	STAB    L028D			;Fuel cut-off delay timer
	BRA     L81EC			;Bra
;
;-High RPMs/speed present
;
L81C5	BSET    L0046,#$10		;Status word, set b4, fuel cut-off in effect
	LDD     #$0000			;
	STD     L0271			;Clear sync BPW
	JMP     L8300			;
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; Routine to apply BLM to BPW
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
L81D1	PSHA				;BLM to stack   
	LDD     L0271			;Sync BPW
	ASLD				;x2
	BCC     L81DB			;Bra if no overflow
;											
	LDD     #65535			;Clear overflow
;
L81DB	STD     L0271			;Save it
;
	PULA				;Get BLM back
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Entrance point to add item to BPW
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;    
L81DF	LDX     #L0271			;Sync BPW
	JSR     LF1ED			;(A x X*)/256
	STD     L0271			;Sync BPW
	RTS				;Return  
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;-Here to exit fuel cut-off
;
L81E9	CLR     L028D			;Clear fuel cut-off delay timer
;
L81EC	BCLR    L0046,#$10		;Status word, clear b4, no fuel cut-off
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Start summing up async AE
;~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
	LDAA    L087F			;Stored IAC AE
;
;-Deliver post-DFCO AE pulse if needed
;
	BRCLR   L003E,#$01,L81FE	;Status word, bra if b0==0, no post-DFCO AE pulse
;										
	BCLR    L003E,#$01		;Status word, clear b0, post-DFCO AE pulse delivered
	ADDA    L48BC			;Post DFCO AE pulse
	BCS     L8218			;Bra if overflow
;
;-Check if A/C on AE needed
;											
L81FE	BRSET   L0041,#$10,L8207	;Status word, bra if b4==1, AC has come on
;
	BCLR    L0041,#$08		;Clear b3, no A/C AE needed
	BRA     L8213			;Bra to continue
;
L8207	BRSET   L0041,#$08,L8213	;Status word, bra if b3==1, A/C AE delivered
;
	BSET    L0041,#$08		;Set b3, A/C AE has been delivered
	ADDA    L48BD			;Add in A/C on AE
	BCS     L8218			;Bra if overflow
;
;-Add in TPS AE
;											
L8213	ADDA    L0262			;Add in stored TPS AE
	BCC     L821A			;Bra if no overflow
;
;-Here to clear overflow
;										
L8218	LDAA    #255			;Clear overflow
;
;-Here if no overflow set
;
L821A	BNE     L8222			;Bra if !=0
;
;-Async AE ==0, check for MAP AE
;
	BRSET   L0005,#$80,L822F	;Status word, bra if b7==1, MAP AE in effect		
;
;-No AE needed here
;
	BRA     L8247			;Bra to continue
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Here if non-zero asnync PW
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;-Perform cool temp correction
;
L8222	LDAB    L025F			;AE temperature correction factor
	MUL				;temp correction x async AE				     
	LSRD				;/2    
	LSRD				;/2    
	LSRD				;/2    
	ADDD    L025D			;Add in stored async pulse time
	STD     L025D			;Save it, async pulse time
;             
L822F	BRSET   L003D,#$40,L8236	;Status word, bra if b6==1, AE already flagged 
;											
	BSET    L006E,#$08		;Status word, set b3, AE just became active
;          
L8236	BSET    L003D,#$C0		;Status word, set b6, AE active and
					;b7, deliver async pulse
	BSET    L006E,#$01		;Status word, set b0, INT reset/BLM cell change 
					;(EGR diag.)
	BSET    L0044,#$02		;Status word, set b1, limit closed loop 
					;correction when AE active
;
	LDAA    L48C0			;Time to lock INT
	STAA    L081D			;INT lock timer
	BRA     L8258			;Bra to continue
;
;~~~~~~~~~~~~~~~~~~~~~~
; Here if no AE needed
;~~~~~~~~~~~~~~~~~~~~~~
;
L8247	BCLR    L003D,#$40		;Status word, clear b6, AE no longer active
	LDAA    L081D			;INT lock timer
	BEQ     L8255			;Bra if ==0, unlock INT
;											
	DECA				;-1
	STAA    L081D			;Save it, INT lock timer
	BRA     L8258			;Bra, dont unlock INT yet
;
;-Here to remove closed loop correction limits after AE
;
L8255	BCLR    L0044,#$02		;Status word, clear b1, reset flag for limiting
;
;~~~~~~~~~~~~~~~~~~~~
; Decel Fuel Cut-Off
;~~~~~~~~~~~~~~~~~~~~
;
;-Check quals for DFCO
;
L8258	LDAA    L0006			;Cool temp, linear
	CMPA    L493A			;Min temp for DFCO
	BCC     L8262			;Bra if cool temp >=
;											
	JMP     L8330			;Bra, no DFCO
;
L8262	BRSET   L0041,#$20,L82CA	;Status word, bra if b5==1, in P/N
;											
	LDAB    L006F			;Status word
	BITB    #$24			;Test b2, high tps, b5, low TPS error
	BNE     L8284			;Bra if TPS errors set
;
	LDAB    #32			;800 RPM, lower limit for lookup
;
;-Check RPM threshold for DFCO
;
	LDAA    L0062			;RPM/25
	LDX     #L4C8A			;Addr of tps threshold vs. rpm
	JSR     LF137			;2D lookup with UL
;											
	CMPA    L01FC			;%TPS
	BHI     L8284			;Bra if TPS <
;	
;-Here to skip/exit DFCO
;										
	BRCLR   L0046,#$08,L82E0	;Status word, bra if b3==0, not previously in DFCO
;											
	BSET    L003E,#$01		;Status word, set b0, post-DFCO AE pulse
	BRA     L82E0			;Bra to exit DFCO
;
;-Check delta RPM threshold
;
L8284	LDAA    L0065			;RPM/12.5, one delayed
	SUBA    L0063			;Subtract RPM/12.5
	BLS     L828F			;Bra if RPMs rising
;											
	CMPA    L493B			;Delta RPM threshold for DFCO
	BHI     L82CA			;Bra if >, exit DFCO
;											
L828F	LDAB    L006F			;Status word
	BITB    #$48			;Test b6, high MAP, and b3, low MAP error
	BNE     L82B8			;Bra if !=0, MAP errors
;
;-Check delta MAP 
;											
	LDAA    L01ED			;Delta SD MAP
	CMPA    L493C			;Delta SD MAP threshold for DFCO
	BHI     L82CA			;Bra if delta MAP >
;											
	LDAA    L01F2			;MAP/baro ratio
;
	LDAB    L400E			;Opt. byte
	BITB    #$02			;Test b1, use MAP/baro ratio for DFCO
;                             				
	BNE     L82AA			;Bra if ==1
;											
	LDAA    L01E3			;Load SD MAP
;
;-Check MAP limits and perform DFCO if MAP low enough
;
L82AA	CMPA    L4933			;Lower MAP threshold for mand. DFCO
	BCS     L82B8			;Bra if MAP <, perform DFCO
;											
	BRCLR   L0046,#$08,L82E3	;Status word, bra if b3==0, not in DFCO
;											
	CMPA    L4934			;Upper MAP limit for DFCO
	BCC     L82E3			;Bra if MAP >=, no DFCO
;
;-Check RPM limits for DFCO
;											
L82B8	LDAA    L0266			;RPM threshold for DFCO
;											
	BRSET   L0046,#$08,L82C6	;Bra if b3==1, already in DFCO
;											
	ADDA    L4930			;RPM hysteresis if already in DFCO
	BCC     L82C6			;Bra if no overflow
;
	LDAA    #255			;Clear overflow
;
L82C6	CMPA    L0062			;RPM/25
	BCS     L82E8			;Bra if RPMs >
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Here for post-DFCO AE to set up for exit
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;											
L82CA	LDAA    L0268			;DFCO exit PW timer
	BEQ     L82E0			;Bra if ==0
;											
	LDD     L025D			;Async pulse time
	ADDD    L4935			;Add'l PW for exiting DFCO
	BCC     L82DA			;Bra if no overflow
											
	LDD     #32767			;Load max asnyc PW
;
L82DA	STD     L025D			;Save it, async pulse time
	BSET    L003D,#$80		;Status word, set b7, deliver async pulse
;
;~~~~~~~~~~~~~~~~~~~
; Here to exit DFCO
;~~~~~~~~~~~~~~~~~~~
;
L82E0	CLR     L0268			;Clear DFCO timer
;
L82E3	LDAA    L4937			;Min time to enter DFCO
	BRA     L8325			;Bra to sae and continue
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Here to continue with DFCO 
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
L82E8	BRSET   L0016,#$10,L82FB	;Error word #1, bra if b4==1, VSS error 
;
;-Check MPH limits for DFCO
;											
	LDAA    L4931			;Min MPH to enter DFCO DFCO
	BRCLR   L0046,#$08,L82F6	;Status word, bra if b3==0, not in DFCO 
;											
	LDAA    L4932			;Min MPH to remain in DFCO
;
L82F6	CMPA    L02A7			;MPH
	BHI     L82CA			;Bra if MPH >, exit DFCO
;
L82FB	LDAA    L0269			;DFCO delay timer
	BNE     L831E			;Bra if !=0, no DFCO yet
;
;-DFCO delay timer has elapsed, perform DFCO
;
L8300	BSET    L0046,#$08		;Status word, set b3, DFCO active
	LDAA    L4938			;Duration to sustain post-DFCO PW
	STAA    L0268			;Post-DFCO timer              
	BSET    L003D,#$10		;Status word, set b4, decel enlean underway
	LDAA    L0267			;DFCO BPW multiplier
	SUBA    L4939			;Subtract DFCO decay rate												(Decrement % per 12.5 msec LP) 
	BCC     L8315			;Bra if still decaying out fuel
;											
	CLRA				;Clear out BPW multiplier
;
L8315	STAA    L0267			;DFCO multiplier
	JSR     L81DF			;Routine to add to BPW
;											
	JMP     L83D0			;Bra to continue
;
;-Decrement DFCO delay timer if not time for DFCO yet
;
L831E	LDAB    L0002			;Loop counter
	ANDB    #$0E			;Mask all but b1-b3
	BNE     L8325			;Bra if !=0
;
;-10th of a second has elapsed
;											
	DECA				;-1 from timer   
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Here if its not time for DFCO yet
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;	
L8325	STAA    L0269			;DFCO delay timer
	LDAA    L0268			;Post-DFCO timer
	BEQ     L8330			;Bra if ==0
											
	DEC     L0268			;-1, post-DFCO timer
;
;-Here if no DFCO
;											
L8330	BCLR    L0046,#$08		;Status word, clear b3, no DFCO
	LDAA    #255			;
	STAA    L0267			;Reset DFCO decay multiplier
	LDD     L0201			;Slow filtered TPS
	ASLB				;    
	ADCA    #$0000			;Round up if needed
	BCC     L8341			;Bra if no overflow
;
	DECA				;Roll back if overflow
; 	   
L8341	SUBA    L01FC			;Subtract TPS
	BCC     L8349			;Bra if no underflow
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Here if TPS rising, no decel enlean
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
	JMP     L83CA			;
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Here to continue with decel enlean
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
L8349	STAA    L082B			;Delta TPS for decel enlean
	LDX     #L400B			;Option byte
	BRCLR   0,X,#$80,L8357		;Bra if b7==0, perorm min delta TPS test each pass
;											
	BRSET   L0053,#$02,L8362	;Status word, bra if b1==1, skip min delta TPS check
;
;-Check delta TPS threshold for decel enlean
;											
L8357	LDAA    L082B			;Delta TPS for decel enlean
	CMPA    L492C			;Threshold for decel enlean
;											
	BLS     L83CA			;Bra if delta TPS <=
;
	BSET    L0053,#$02		;Set b1, skip delta TPS check when in decel enlean 
;
L8362	LDD     L01EA			;SD MAP, filtered per cool temp
	ASLB				;    
	ADCA    #$0000			;Round if needed
	BCC     L836B			;Bra if no overflow
;
	DECA				;Roll back MSB
; 
;-Check delta MAP threshold for decel enlean
;   
L836B	SUBA    L01E3			;Subtract SD MAP
	BCS     L83CA			;Bra if MAP is rising, no decel enlean
;											
	CMPA    L492D			;Delta MAP threshold for decel enlean
	BLS     L83CA			;Bra if delta MAP <=, no decel enlean
;											
	CMPA    #160			;50 kPa, max delta MAP for lookup
	BLS     L837B			;Bra if <=
;
	LDAA    #160			;Load max for lookup
;
L837B	LDX     #L4B16			;PW reduction vs. delta MAP table
	JSR     LF15E			;2D lookup
;
	TAB				;Get result into B     
	LDX     #L4B21			;PW reduction vs. delta TPS table
	LDAA    L082B			;Delta TPS for decel enlean
	JSR     LF15E			;2D lookup
;
	ABA				;Add the reduction multipliers together     
	BCC     L8390			;Bra if no overflow
;
	LDAA    #$00FF			;Clear overflow
;
L8390	LDAB    L0265			;Decel enlean cool factor
	MUL				;Cool factor x PW multiplier     
	ASLD				;x2    
	BCS     L839D			;Bra if overflow
;
	ASLD				;x2    
	BCS     L839D			;Bra if overflow
;
	ASLD				;x2, now divided by 32   
	BCC     L839F			;Bra if no overflow

L839D	LDAA    #255			;Clear overflow
;
L839F	LDAB    L02A7			;MPH
	CMPB    L492F			;MPH threshold to scale down decel enlean 
	BCC     L83B4			;Bra if MPH >=
;
;-Scale decel enlean if vehicle not moving/moving at low speeds
;											
	LDAB    L492E			;Multiplier for low speed decel enlean
	MUL				;Multiplier x decel enlean PW factor     
	ADDD    #$40			;Round up if needed
	ASLD				;x2    
	BCC     L83B4			;Bra if no overflow
;
	LDD     #$FFFF			;Clear overflow
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Apply decel enlean to asyc and sync PW's
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
L83B4	NEGA				;256 - decel enlean factor    
	BEQ     L83CA			;Bra if no decel enlean
;
	BSET    L003D,#$10		;Status word, set b4, decel enlean underway
	PSHA				;Push decel enlean mult. to stack    
	LDX     #L025D			;Async pulse time
	JSR     LF1ED			;(A x X*)/256
;											
	STD     L025D			;Save it, async pulse time
	PULA				;Get decel enlean mult. back    
	JSR     L81DF			;Go add it to the BPW
	BRA     L83D0			;Bra to continue
;
;~~~~~~~~~~~~~~~~~~~~~~~~~
; Here if no decel enlean
;~~~~~~~~~~~~~~~~~~~~~~~~~
;
L83CA	BCLR    L003D,#$10		;Status word, clear b4, no decel enlean
	BCLR    L0053,#$02		;Status word, clear b1, check delta TPS 
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Here if DFCO active, skip decel enlean
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;-Check lower RPM limit for derivative RPM term
;
L83D0	LDAA    L0067			;RPM/6.25
	CMPA    L450C			;RPM limit for constructing derivative term
	BCS     L83EE			;Bra if RPMs <, no derivative term
;
;-Check TPS limit for derivative RPM term
;
	LDAA    L01FC			;TPS
	CMPA    L48D1			;Upper TPS limit for derivative RPM term
	BHI     L83EE			;Bra if TPS >, no derivative term
;
;-Check MPH limit for derivative RPM term
;											
	LDAA    L02A7			;MPH
	CMPA    L48D2			;Max speed for derivative RPM calc
	BHI     L83EE			;Bra if MPH >, no derivative term
;
;-Check cool limit for derivative RPM term
;											
	LDAA    L0006			;Cool temp
	CMPA    L48D3			;Lower temp limit for derivative RPM calc
	BCC     L83F4			;Bra if temp >=, perform calc
;
;-Here if quals not passed
;											
L83EE	LDAA    #128			;Load neutral value 
	STAA    L006C			;Transient RPM ratio
	BRA     L8425			;Bra to continue
;
;-Quals passed, calc. transient RPM ratio
;
L83F4	LDD     L006A			;Delayed filtered RPM/6.25
	LSRD				;/2
	LDX     L0068			;Filtered RPM/6.25
	FDIV				;Fract. divide    
	XGDX				;Get result into D    
	ADDD    #128			;Round if needed
	BCC     L8402			;Bra if no overflow
;
	LDAA    #255			;Clear overflow
;
;-Transient rpms = 128 x ([delayed RPM/6.25]/[RPM/6.25])
;
L8402	STAA    L006C			;Save it, transient RPM ratio
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Perform derivative fuel control to smooth idle
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;-Check limits for transient RPM term
;
	CMPA    L4510			;Upper limit for transient RPM ratio
	BCS     L840E			;Bra if transient ratio <
											
	LDAA    L4510			;Load limit instead
	BRA     L8416			;Bra to continue

L840E	CMPA    L450F			;Lower limit for transient RPM ratio
	BCC     L8416			;Bra if transient ratio >=
;											
	LDAA    L450F			;Use limit instead
;
;-Apply derivative action
;
L8416	LDX     #L0271			;Sync BPW addr.
	JSR     LF1ED			;(A x X*)/256
;											
	ASLD				;x2, now /128
	BCC     L8422			;Bra if no overflow
;											
	LDD     #$FFFF			;Clear overflow
;
L8422	STD     L0271			;Sync BPW
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Perform altitude correction
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; kPa MAP val for lookup = .625 x A/D MAP + 10
;
L8425	LDAA    L01E9			;A/D MAP
	CMPA    #216			;87.5 kPa, upper limit for lookup
	BLS     L842E			;Bra if MAP <=
;											
	LDAA    #216			;87.5 Kpa
;
L842E	LDAB    #151			;Multiplier to scale MAP for lookup
	MUL				;MAP x multiplier     
	ADDD    #145			;Bias, lookup starts at 10 kPa
	LDAB    L01EF			;Load baro
;
	LDX     #L49A0			;Addr. of BPW corr. table
	JSR     LF17B			;3D lookup table
;											
	LDX     #L0271			;Sync BPW
	JSR     LF1ED			;(A x X*)/256
;
	ASLD				;x2, now /128
	BCC     L8449			;Bra if no overflow
;											
	LDD     #$FFFF			;Clear overflow
;
L8449	ASLD				;x2 for batt compensation  
	BCC     L844F			;Bra if no overflow
;											
	LDD     #$FFFF			;Clera overflow
;
L844F	STD     L0271			;Sync BPW
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Perform batt. volts compensation
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
	LDAA    L0055			;A/D fuel pump volts
	LDX     #L497E			;Addr. of BPW mult. table
	JSR     LF15E			;2D lookup
;
	LDX     #L0271			;Sync BPW
	JSR     LF1ED			;(A x X*)/256
;											
	STD     L0271			;Save it, sync BPW
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Perform closed loop fuel correction to BPW
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
	CLRA				;Preclear A   
	LDAB    L0292			;closed loop fuel correction term
	BMI     L8473			;Bra if coorection <= 0
;
;-Adding fuel
;											
	ADDD    L0271			;Add in sync BPW
	BCC     L847F			;Bra if no overflow
;											
	LDD     #$FFFF			;Clear overflow
	BRA     L847F			;Bra to continue
;
;-Removing fuel
;
L8473	NEGB				;Get abs. value    
	SUBD    L0271			;Subtract sync BPW
	BCS     L847B			;Bra if sync BPW >
;
;-Sync BPW is less then corrections
;											
	CLRA				;Clear BPW    
	CLRB				;
;    
L847B	NEGA				;Switch sign, now BPW - fuel correction    
	NEGB				;    
	SBCA    #$0000			;Perform carry if needed
;
L847F	STD     L0271			;Save it, sync BPW
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Add in the MAP AE to the sync BPW
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
	CLRA				;    
	LDAB    L0261			;Stored Map AE PW
	ADDD    L0271			;Sync BPW
	BCC     L848E			;Bra if overflow
;											
	LDD     #$FFFF			;Clear overflow
;
L848E	STD     L0271			;Sync BPW
	STD     L0277			;BPW, CPI/PFI mode
;
;~~~~~~~~~~~~~~~~~~~~
; Check for shutdown
;~~~~~~~~~~~~~~~~~~~~
;   
	BRCLR   L0044,#$10,L84A0	;Status word, bra if b4==0, ignition is on 
;
;-Ignition off/shutting down here
;
	CLRA				; 	   
	CLRB				;    
	STD     L0271			;Clear sync BPW
	JMP     L859F			;Bra to continue
;
;-Engine running here
;
L84A0	LDX     #L400B			;Opt. byte
	BRCLR   0,X,#$01,L84DE		;Bra if b0==0, in TBI mode
;											
	BRCLR   L0051,#$01,L84AD	;Status word, bra if b0==0, not in single fire mode
;											
	BRA     L84D3
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; In CPI/PFI mode, check for short BPW
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
L84AD	SEI				;Disable interrupts     
	LDAA    L082A			;Time in double fire mode
	INCA				;+1
	BEQ     L84B7			;Bra if roll-over
;											
	STAA    L082A			;Save it, time in double fire mode
;
L84B7	CLI				;Re-enable interrupts
	LDY     L0277			;BPW, CPI/PFI mode
	CPY     L4922			;Threshold to transition from double to single fire
	BHI     L84DC			;Bra if BPW is >
;
;-Check time limit for transition
;											
	LDAA    L082A			;Time in double fire mode	
	CMPA    L496A			;Min time to transition back to single fire mode
	BCS     L84DC			;Bra if <, no single fire mode yet
;
;-Transition from double to single fire mode
;											
	BSET    L0051,#$01		;Status word, set b0, in single fire mode
	BSET    L003F,#$10		;Status word, set b4, sync/async flag											
	LDD     L0271			;Sync BPW
;
L84D3	ASLD				;x2 for single fire mode 
	BCC     L84D9			;Bra if no overflow
;											
	LDD     #$FFFF			;Clear overflow
;
L84D9	STD     L0271			;Save it, sync BPW
;
L84DC	BRA     L8555			;Bra to continue
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; In TBI mode, check for short BPW
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
L84DE	LDX     #L400B			;Opt. byte
	BRCLR   0,X,#$04,L84E9		;Bra if b2==0, dont force sync idle
;											
	BRSET   L0050,#$80,L854C	;Status word, bra if b7==1, engine idling
;
;-Allow async idle here
;											
L84E9	LDX     #L4920			;Addr. of sync/async thresholds
	BRSET   L003E,#$10,L84F2	;Status word, bra if b4==1, in async fire
;
;-In sync fire
;											
	INX				;+1
	INX				;+1, advance to sync to async thresholds
;
;-Check PW threshold to enter/remain in async
;											
L84F2	BSET    L003E,#$10		;Preset b4==1, async flag
	LDD     4,X			;Load MAP/RPM quals
	LDX     0,X			;PW threshold for async firing
	CPX     L0271			;Compare PW to sync BPW
	BCC     L8507			;Bra if sync BPW <= 
;
;-Check MAP threshold
;											
	CMPA    L01E3			;SD MAP
	BCC     L854C			;Bra if MAP <= threshold, dont enter async
;	
;-Check RPM threshold
;										
	CMPB    L0062			;RPM/25
	BCC     L854C			;Bra if RPMs <= threshold, dont enter async
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Here to enter asnyc fire, convert sync BPW to async firing time
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;											
L8507	CLRA				;
	CLRB				;Clear out sync BPW
	STD     L3FCE			;TBI injector driver hardware
	JSR     LF08A			;Short delay
;
;-Convert BPW to a duty cycle
;
	LDD     L3FC0			;DRP
	ASLD				;x2 for quad injector firings
	BCC     L8518			;Bra if no overflow
;											
	LDD     #$FFFF			;Clear overflow
;
L8518	XGDX				;Swap result into X    
	LDD     L0271			;Sync BPW
	FDIV				;BPW/DRP x 65536, now %DC 
;
;-%DC x (1/80 firings), or 12.5 msec per asnyc firing
;  
	LDD     #0080			;80 firings/sec when in async
	XGDX				;Swap result back into D    
	IDIV				;Int. divide, result/80    
	PSHX				;Push quotient to stack    
	LDX     #0080			;80 firings/sec when in async
	FDIV				;/80, resolve remainder    
	XGDX				;Swap remainder into D    
	PULX				;Get quotient back    
	ADDD    #0128			;Round if needed
	STAA    L0281			;Save remainder			
	XGDX				;Get quotient into D    
	BCC     L8535			;Bra if no overflow from round op
;
	ADDD    #0001			;Round if needed
;
L8535	STD     L027F			;Save it, async BPW
;
;-Save accumulated async PW
;
	LDD     L027D			;Load lower two bytes of accumulated async PW
	ADDD    L0280			;Add in async BPW
	STD     L027D			;Save it, async PW
;
	LDAA    L027C			;Uppermost byte of accumulated async PW
	ADCA    L027F			;Add in async BPW and perform carry if needed
	STAA    L027C			;Save it, async PW
	BRA     L8559			;Bra to continue
;
;~~~~~~~~~~~~~~~~~~~~~~
; Here if in sync fire
;~~~~~~~~~~~~~~~~~~~~~~
;
L854C	BCLR    L003E,#$10		;Clear b4, no async firings
;
	LDX     #$0000			;
	STX     L027C			;Clear accumulated async PW
;
;-Check for async pulse
;
L8555	BRCLR   L003D,#$80,L859F	;Status word, bra if b7==0, no async pulse needed
;
;-Check async limits
;											
L8559	LDD     L027C			;Stored async PW
	ADDD    L025D			;Add in async pulse time
	CPD     L4928			;Max async PW
	BHI     L8576			;Bra if PW >
;										
	CPD     L492A			;Min async PW to fire
	BLS     L8599			;Bra if PW <=, sync firing this pass
;
;-Empty async accumulators for injector firing
;											
	LDX     #$0000			; 
	STX     L025D			;Clear async pulse time 
	STX     L027C			;Clear stored async PW
	BRA     L858C			;Bra to continue
;
;-Here if excess PW
;
L8576	LDD     L027C			;Stored async PW
	SUBD    L4928			;- max async firing time
	BCC     L8586			;Bra if no underflow
;											
	ADDD    L025D			;Add into async pulse time, sub out PW next pass
	STD     L025D			;Save it, async pulse time											;
	CLRA				;Clear out async PW
	CLRB				;
;
L8586	STD     L027C			;Save it, stored async PW
	LDD     L4928			;Load max async firing time
;
;-Here to output async PW
;
L858C	SEI				;Disable interrupts
	JSR     L862B			;Routine to output pulsewidth
	CLI				;Re-enable interrupts
	BSET    L003D,#$80		;Status word, set b7, async pulse flag
	BCLR    L0046,#$01		;Status word, clear b0, sync AE
	BRA     L859F			;Bra to continue				
;
;-Here if short BPW, switch to sync AE instead
;											
L8599	BSET    L0046,#$01		;Status word, set b0, enable sync AE
	BCLR    L003D,#$80		;Clear b7, async pulse
;
;-Here if no async pulse needed
;											
L859F	LDD     L0271			;Sync BPW
	BRCLR   L003E,#$10,L85AE	;Status word, bra if b4==1, async firings
;
;-Async firings here
;											
	STD     L026F			;Outputted BPW
	BCLR    L003F,#$80		;Status word, clear b7, async pulse delivered 
	BRA     L860F			;Bra to continue
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Sync firings here, check if BPW bias needed
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
L85AE	BEQ     L85D8			;Bra if PW ==0
;											
	CPD     #0256			;3.9 msec 
	BCC     L85C9			;Bra if PW >=
;
;-BPW is short enough for bias
;											
	TBA				;Get BPW into A
	LDAB    #32			;488 usec bias for lookup
	LDX     #L496F			;BPW offset vs. BPW table
	JSR     LF15A			;2D lookup with SBA
;											
	TAB				;Get offset into B 
	CLRA				;
	ADDD    L0271			;Add offset into BPW
	SUBD    L4967			;Subtract bias
	BCS     L85D2			;Bra if BPW <, set flag
;
;-Clear short BPW flag
;
L85C9	BCLR    L003F,#$80		;Status word, clear b7, short BPW
	CPD     L4963			;PW threshold to set short BPW flag
	BCC     L85D8			;Bra if BPW >=
;
;-Set short BPW flag
;
L85D2	BSET    L003F,#$80		;Set b7, short BPW 
	LDD     L4965			;Load min allowed BPW
;
L85D8	STD     L026F			;Outputted BPW
	BEQ     L85EB			;Bra if BPW ==0
;
	ADDB    L0279			;Add in batt. volts compensation
	ADCA    #$00			;Perform carry if needed
	CPD     #32767			;Upper limit for PW
	BCS     L85EB			;Bra if PW <
;
	LDD     #32767			;Load max value
;
L85EB	STD     L0273			;Save PW, TOC 4/5 value
	LDX     #L400B			;Opt. byte
	BRSET   0,X,#$01,L85FA		;Bra if b0==1, in CPI/PFI mode
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; In TBI mode, output to hardware
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
	STD     L3FCE			;TBI injector driver hardware
	BRA     L860F			;Bra to continue
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Here if in CPI/PFI mode, calc. time based delay for inj. firing
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; Time based delay = 4 x (deg delay x DRP) / 256
;
;                                                
L85FA	LDAA    L4969			;Load deg delay from DRP to injector firing
	LDX     #L005F			;Load DRP period
	JSR     LF1ED			;8 x 16 multiply routine
;
;-Scale to sync with free running MCU counter
;
	ASLD				;x2
	BCS     L8609			;Bra if overflow
;
	ASLD				;x2 
	BCC     L860C			;Bra if overflow
;
L8609	LDD     #$FFFF			;Clear overflow
;
L860C	STD     L081F			;Save it, time based delay for injector firing
;
;-Here to continue with 40 hZ routines
;
L860F	BRSET   L0002,#$02,L861B	;Loop counter, bra if b1==1
;
;~~~~~~~~~~~~~~~~~~~~~
; First 40 hZ cluster
;~~~~~~~~~~~~~~~~~~~~~
;
	JSR     L8720			;O2 PID routine
	JSR     LD3CA			;VSS handling routine
	BRA     L862A			;Bra to return
;
;~~~~~~~~~~~~~~~~~~~~~~
; Second 40 Hz cluster
;~~~~~~~~~~~~~~~~~~~~~~
;
L861B	JSR     L86F6			;Check for ignition on/off
	BRCLR   L0002,#$04,L8627	;Loop counter, bra b2==0, run second cluster
;
;~~~~~~~~~~~~~~~~~~~~~
; First 20 Hz cluster
;~~~~~~~~~~~~~~~~~~~~~
;
	JSR     L8A01			;Misc. fueling and BLM handling routine
	BRA     L862A			;Bra to return
;
;~~~~~~~~~~~~~~~~~~~~~~
; Second 20 Hz cluster
;~~~~~~~~~~~~~~~~~~~~~~
;
L8627	JSR     L8F97			;Governor PID subroutine
;
L862A	RTS				;Return
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~    
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; Routine to output inj. pulsewidth
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
L862B	LDX     #L400E			;Opt. byte
	BRCLR   0,X,#$10,L864F		;Bra if b4==0, no added injector bias
;
	CPD     #$0100			;3907 usecs
	BCC     L864F			;Bra if BPW >=
;
;-Here if BPW < 3907 usecs, look up BPW inj. offset
;
	PSHB				;Push BPW to stack    
	PSHA				;    
	TBA				;Get LSB into A for lookup     
;
;-Look up injector bias based on BPW
;
	LDAB    #32			;488 usecs offset for lookup
	LDX     #L496F			;BPW offset table
	JSR     LF15A			;2D lookup with SBA
;
	TAB				;Get offset into B   	
	PULX				;Get BPW back    
	ABX				;Add in the offset     
	XGDX				;Get the result into D    
	SUBD    L4967			;Subtract offset
	BCC     L864F			;Bra if no underflow

	LDD     #$0000			;Clear underflow
;
L864F	ADDB    L0279			;Batt. volts. BPW bias correction
	ADCA    #0			;Perform carry if needed
	STD     L3FF2			;Save it, outputted BPW
;
	JSR     LF08A			;Short delay
;
	LDD     L3FFC			;I/O D port
	ANDA    #$FB			;Clear b2= 
	JSR     LF08A			;Short delay
;
	STD     L3FFC			;Save it.
;
	ORAA    #$04			;Set b2
	JSR     LF08A			;Short delay
;
	STD     L3FFC			;Save it, I/O D port
	RTS				;Return    
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~  
;
;~~~~~~~~~~~~~~~~~~~~~~~~~
;
; RPM/6.25 calc. routine
;
;~~~~~~~~~~~~~~~~~~~~~~~~~
;                                          
L866E	LDX     L3FC0			;DRP counter
	LDAA    L4142			;# of cyls.
	LDAB    #32			;32, calc DRP relative to 8 cyl.
	MUL				;(# cyls. x 32)    
	TBA				;/ result by 256    
	BEQ     L8681			;Bra if 8 cyl in use
;
;-Here if less then 8 cyl.
;
	PSHX				;DRP counts to stack   
	TSX				;Get current stack pointer				     
	JSR     LF1ED			;(A x X*)/256
;
	PULX				;Get DRP counts back    
	XGDX				;Swap result into X 
;
;-Here if 8 cyl. present.
;   
L8681	CPX     #$0000			;0 DRP counts
	BHI     L8689			;Bra if !=0
;
	CLRA				;Clear RPMs    
	BRA     L8694			;Bra to return
;
L8689	LDD     #614			;9.37 msec, 1600 RPM
	FDIV				;Fract. divide    
	XGDX				;Get result into D, now RPMs/6.25    
	CMPA    #127			;800 RPM limit
	BLS     L8694			;Bra if <=
;
	LDAA    #127			;Return with limit instead
;
L8694	RTS				;Return    
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; Crank fuel lookup routine
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
L8695	LDAA    L0006			;Cool temp, linear
	LDX     #L4D92			;Crank BPW vs. cool temp
	JSR	LF15E 			;2D lookup
;
	LDX     #L4D90			;Scalar for crank BPW
	JSR     LF1ED			;(A x X*)/256
;											
	PSHB				;Push crank BPW to stack    
	PSHA				;    
	LDAA    L02F1			;DRP counter
	CMPA    L4D8B			;DRP threshold to use RPM multipliers
	BLS     L86C4			;Bra if DRPs <=
;
	JSR     L866E			;RPM/6.25 calc. routine
;
;-Perform crank fuel corr. based on RPMs
;
	LDX     #L4DA3			;Crank fuel mult. vs RPM
	BRCLR   L0004,#$10,L86BA	;Status word, bra if b4==0, no hot restart
;
;-Hot restart in effect
;
	LDX     #L4DAC			;Hot restart crank fuel mult. vs RPM
;
L86BA	JSR     LF15E			;2D lookup
;
	TSX				;Current stack pointer into X     
	JSR     LF1ED			;(A x X*)/256
;										
	PULX				;Get old crank BPW back
	PSHB				;Push crank BPW to stack    
	PSHA				;    
;
;-Perform baro correct
;
L86C4	LDAA    L01EF			;Baro
	LDAB    #96			;Offset for lookup
	LDX     #L4DB5			;Crank fuel mult. vs baro table
	JSR     LF15A			;2D lookup with SBA
;
	TSX				;Get current stack pointer into X     
	JSR     LF1ED			;(A x X*)/256
;
	PULX				;Get crank BPW    
	ASLD				;x2, now /128    
	BCC     L86DA			;Bra if no overflow
;
	LDD     #$FFFF			;Clear overflow
;
L86DA	BCS     L86F5			;Bra if overflow, return
;
;-Perform TPS crank fuel correction
;
	PSHB				;Crank BPW to stack    
	PSHA				;    
	LDAA    L01FC			;TPS
	LDX     #L4DB9			;Crank fuel mult. vs TPS table
	JSR     LF153			;2D lookup w/ linecount
;
	TSX				;Get current stack pointer     
	JSR     LF1ED			;(A x X*)/256  
;
	PULX				;Get old BPW back off the stack   
	ASLD				;x2   
	BCS     L86F2			;Bra if overflow
;
	ASLD				;x2, now /64    
	BCC     L86F5			;Bra if no overflow
;
L86F2	LDD     #$FFFF			;Clear overflow
;
L86F5	RTS				;Return     
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; Routine to check for ignition on/off
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;-Check for low batt. volts
;
L86F6	LDAA    L00A1			;A/D battery volts
	CMPA    #90			;9 volts
	BCC     L8708			;Bra if batt volts >=
;
;-Check for ignition off
;
	CMPA    #40			;4 volts, threshold for ignition off
	BCC     L871F			;Bra if batt. volts >=, return w/o updating
;
;-Ignition is off
;    
	BSET    L0044,#$10		;Status word, set b4, ignition is off
	BCLR    L0050,#$01		;Status word, clear b0, sync. MAP reads 
	BRA     L871F			;Bra to return
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Batt volts > 9 volts and ignition is on
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;   
L8708	BRCLR   L0044,#$10,L8712	;Bra if ignition on already flagged
;
;-Update EST info
;
	LDX     L3FCA			;EST circuit info
	STX     L0228			;Save it, EST circuit info
;  
;-Update status word
;
L8712	BCLR    L0044,#$10		;Clear b4, ignition is on
;
;-Check for sync MAP reads
;
	LDAA    L400C			;Opt. byte
	BITA    #$01			;Check b0, sync MAP reads
	BEQ     L871F			;Bra if ==0, no sync MAP reads
;
	BSET    L0050,#$01		;Set b0, sync MAP reads
;
L871F	RTS				;Return    
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;~~~~~~~~~~~~~~~~~~
;
; O2 PID algorithm
;
;~~~~~~~~~~~~~~~~~~
;
L8720	BRCLR   L004F,#$10,L8745	;Status word, bra if b4==0, engine not running
;
;-Check O2 window, learn disabled if O2 dwells within window long enough
;
	LDX     #L48F5			;O2 window 
	BRCLR   L0050,#$80,L872E	;Status word, bra if b7==0, not idling
;
	LDX     #L48F9			;Load O2 window for idle
;
L872E	BRSET   L003E,#$80,L8734	;Status word, bra if b7==1, in closed loop	
;
;-Load open loop O2 window
;
	INX				;+1
	INX				;+1, open loop O2 window
;  
;-Check upper O2 window boundry
; 
L8734	LDAA    L01F8			;O2 A/D volts
	CMPA    0,X			;Upper O2 boundry
	BHI     L873F			;Bra if O2 >, O2 is functional
;
;-Check lower O2 window boundry
;
	CMPA    1,X			;Lower O2 boundty
	BCC     L8745			;Bra if O2 >, O2 still dwelling in window
;
;-O2 isnt dwelling within window, clear learn disable timer, enable learn
;
L873F	CLR     L0264			;Clear O2 dwell timer
	BSET    L0004,#$01		;Status word, set b0, O2 functional
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Scale airflow and look up airflow based parameters
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;	
L8745	LDAA    L0255			;Gms/sec airflow
	LDAB    L493F			;Scalar for airflow 
	MUL				;Gms/sec x scalar     
	ASLD				;x2    
	BCS     L8756			;Bra if overflow
;
	ASLD				;x2    
	BCS     L8756			;Bra if overflow
;
	CMPA    #128			;Upper limit for scaled airflow
	BLS     L8758			;Bra if airflow is less			
;
L8756	LDAA    #128			;Load limit
;
L8758	STAA    L029E			;Save it, scaled airflow
	LDAB    #$03			;4 table lookups
	BRCLR   L0050,#$80,L876E 	;Status word, bra if b7==0, not idling
;
;-Idling here
;
	LDD     L4940			;Rich O2 threshold if idling 
	STD     L0290			;Save it, rich and lean O2 thresholds
	LDAA    L4942			;Mean rich/lean O2 threshold if idling
	STAA    L028F			;Save it, mean rich/lean O2 threshold						
	CLRB				;Clear # of table lookups, just look up INT delay
; 
;-Loop and perform four table lookups based on airflow
; 
; 0x4CD7, integrator delay vs airflow table
; 0x4CE0, mean rich/lean O2 threshold
; 0x4CE9, rich O2 threshold vs airflow
; 0x4CF2, lean O2 threshold vs airflow 
;
; 0x028E, stored INT delay
; 0x028F, stored mean rich/lean O2 threshold 
; 0x0290, stored rich O2 threshold
; 0x0291, stored lean O2 threhsold
;
L876E	PSHB				;Push # of tables to stack    
	LDAA    #$09			;9 lines/table
	MUL				;# lines/table x # of tables     
	LDX     #L4CD7			;Start of tables
	ABX				;Add in offset     
	LDAA    L029E			;Load scaled airflow
	JSR     LF15E			;2D lookup
;
	LDX     #L028E			;Start of stored params
	PULB				;Get table # off stack    
	TSTB				;Test B    
	BEQ     L8789			;Bra if # of tables remaining is ==0
;
	ABX				;Add in offset to X     
	STAA    0,X			;Save param to memory
	DECB				;-1 from # of tables    
	BRA     L876E			;Loop to continue
;
;-Check if EGR diagnostics are in effect
;
L8789	BRCLR   L0070,#$04,L8790	;Status word, bra if b2==0, EGR diags. not underway
;
	LDAA    L4E4E			;Default INT delay if EGR diags. in effect
;
L8790	STAA    L028E			;Save it, INT delay
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Determine direction of O2
;~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
	LDAB    L003E			;Status word
	LDAA    L028F			;Mean rich/lean threshold
	ADDA    L493D			;Add rich/lean O2 mV hyst.
	BCS     L87A2			;Bra if overflow
;
;-Check rich O2 limit
;
	CMPA    L01F8			;A/D O2 volts
	BCS     L87BB			;Bra if O2 volts greater
;
;-Check lean O2 limit
;
L87A2	LDAA    L028F			;Rich/lean O2 threshold
	SUBA    L493D			;Subtract rich/lean O2 mV hyst. 
	BCS     L87AF			;Bra if underflow

	CMPA    L01F8			;A/D O2 volts
	BHI     L87B7			;Bra if >=
;
;-O2 mV neither overly high or low, check direction of O2
;
L87AF	LDAA    L01F8			;Current A/D O2
	CMPA    L01F7			;A/D O2, one delayed
	BHI     L87BB			;Bra if O2 is going richer
;
;-Going leaner here
;
L87B7	ANDB    #$BF			;Clear b6 of L003E, going leaner
	BRA     L87BD			;Bra to continue
;
;-Going richer here
;
L87BB	ORAB    #$40			;Set b6, going richer
;
L87BD	CMPB    L003E			;Status word
	BEQ     L87CA			;Bra if no change in fueling status
;
	INC     L0298			;Increment O2 cross-counts
	BSET    L0043,#$10		;Status word, set b4, change in fueling state
	CLRA				;Preclear A to reset timer
	BRA     L87D0			;Bra to save and continue				
;
;-Here if no change in fueling
;											
L87CA	LDAA    L029F			;Duration in current fueling state 
	INCA				;+1
	BEQ     L87D3			;Bra if rollover
;
L87D0	STAA    L029F			;Save timer
;
L87D3	STAB    L003E			;Save status word
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Check if INT reset needed
;~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;-Check for mode 4 operation
;
	BRCLR   L003B,#$10,L87E9	;Status word, bra of b4==0, no mode 4

	LDX     #L0393			;			
	BRCLR   0,X,#$01,L87E9		;
;
	BRCLR   1,X,#$01,L87E6		;
;
	BRA     L885C			;Bra to reset INT
;
L87E6	JMP     L8873			;
;
;-Here if normal operation
;
L87E9	BRSET   L003E,#$80,L87F1	;Status word, bra if b7==1, in closed loop 
;
	BRCLR   L0052,#$80,L885F	;Status word, bra if b7==0, no quasi-closed loop
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Check quals for quasi-closed and full closed loop
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
L87F1	BRSET   L0046,#$08,L8859	;Status word, bra if b3==1, DFCO in effect 
;
;-Check for PE
;
	BRSET   L003D,#$20,L8859	;Status word, bra if in power enrich
;
	BRSET   L0052,#$40,L8859	;Status word, bra if b6==1, cat overtemp
;
;-Check if AE has just become active
;
	LDX     #L400C			;Opt. byte
	BRCLR   0,X,#$20,L880D		;Bra if b5==0, no INT reset when AE in effect 
;
	BRCLR   L006E,#$08,L880D	;Status word, bra if b3==0, AE has not
					;just become active
;
	BCLR    L006E,#$08		;Reset AE became active flag
	BRA     L885C			;Bra to reset INT
;
;-Reset INT on BLM cell change, if specified
;
L880D	LDX     #L400C			;Opt. byte
	BRSET   0,X,#$10,L8818		;Bra if b4==1, allow INT reset on BLM cell change 
;
	BRCLR   0,X,#$40,L881C		;Bra if b6==1, reset INT on BLM cell change
;
L8818	BRSET   L003D,#$04,L885C	;Status word, bra if b2==1, BLM cell has changed 
;
;-Check for change in VE table
;
L881C	BRCLR   L004F,#$20,L8825	;Status word, bra if b5==0, no change in VE tables
;
	BCLR    L004F,#$20		;Clear b5, reset VE table change flag
	BRA     L885C			;Bra to reset INT
;
;-Check for decel enlean
;
L8825	BRCLR   L003D,#$10,L8834	;Status word, bra if b4==0, no decel enlean
	BRSET   L003E,#$40,L8873	;Status word, bra if b6==1, going richer 
;
	LDAA    L0232			;INTegrator
	BPL     L8873			;Bra if currently leaning out fueling
;
	BRA     L885C			;Bra to reset the INT
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Check MAP/RPM/MPH limits to prevent INT windup
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;-Check MAP limit
;
L8834	LDAB    L01E3			;SD MAP
	CMPB    L4901			;Lower MAP threshold to reset INT 
	BHI     L8873			;Bra if MAP >, no INT reset
;
;-Check RPM window for reset
;
	LDAB    L0062			;RPM/25
	CMPB    L4902			;Lower RPM threshold to reset INT
	BLS     L8873			;Bra if RPMs <=, no INT reset
;
	CMPB    L4903			;Upper threshold to reset INT
	BHI     L8873			;Bra if RPMs >, no INT reset
;
;-Check MPH threshold
;
	LDAB    L02A7			;MPH
	CMPB    L4904			;Lower threshold to reset INT
	BCS     L8873			;Bra if speed <=
;
;-Check direction of fueling and reset if needed
;
	BRSET   L003E,#$40,L8873	;Status word, bra if b6==1, going richer
;
;-Going leaner here
;
	LDAA    L0232			;INT
	BPL     L8873			;Bra if < 128, still leaning out fueling
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Here to reset integrator and other closed loop terms
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
L8859	BSET    L0071,#$04		;Status word, set b2, INT just reset (O2 diag.)
;
L885C	BSET    L006E,#$01		;Status word, set b0, INT just reset (EGR diag.)
;
L885F	LDAA    #128			;
	STAA    L0232			;Reset the INT
;
;-Reset the slow filtered O2 term
;
	LDAA    #102			;442 mV
	STAA    L01F5			;Save it, slow filtered O2
;
;-Clear other terms
;
	CLRA				;
	STAA    L0282			;Clear INT timer
	STAA    L029D			;Clear prop. term
	JMP     L89F4			;Bra to continue
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Here if not resetting INT, calculate slow O2 error
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;-Check direction of long term fueling
;
L8873	LDAA    L01F5			;Slow filtered O2
	CMPA    L0290			;Rich O2 threshold
	BHI     L8889			;Bra if O2 going richer
;
;-Long term fueling leaning out here
;
	SUBA    L0291			;Lean O2 threshold
	BCS     L8883			;Bra if O2 less then threshold
;
;-O2 between lean and rich threhsolds
;
	CLRA				;Clear O2 error    
	BRA     L88A1			;Bra to save O2 error
;
;-Going leaner here
;
L8883	NEGA				;Get abs. val. of O2 error   
	BCLR    L0046,#$02		;Status word, clear b1, long term fueling lean
	BRA     L8893			;Bra to continue
;
;-Long term fueling going richer here
;
L8889	SUBA    L0290			;Subtract rich O2 threshold 
	LDAB    L494B			;Factor applied to O2 error if going richer 
	MUL				;Factor x O2 error				
	BSET    L0046,#$02		;Set b1, long term fueling rich
;
;-Check for idle
;
L8893	BRCLR   L0050,#$80,L889B	;Status word bra if b7==0, not idling
	LDAB    L494C			;Factor applied to O2 error if idling 
	MUL				;Factor x O2 error
;     
L889B	CMPA    #96			;Upper limit for O2 error
	BLS     L88A1			;Bra if O2 error is less
;
	LDAA    #96			;Load max O2 error
;
L88A1	STAA    L02A3			;Save it, O2 error
	ASLA				;x2
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Look up base proportional counts
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
	LDX     #L4D04			;Base proportional counts vs. O2 error
	JSR     LF15E			;2D lookup
;
	STAA    L029D			;Save it, proportional term
;
	BRCLR   L0050,#$80,L88F6	;Status word, bra if b7==0, not idling
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Engine idling here, load idle specific params
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;										
	LDAA    L4943			;Idle prop. duration term										
	PSHA				;To stack    
	LDX     #L4945			;Start addr. of idle proportional gain terms
	LDY     #L4948			;Start addr. of idle integrator delay terms
	BRSET   L003E,#$40,L88C9	;Status word, bra if if b6==1, going richer
;
;-Going leaner here, exec. every other time
;
	SEI				;Disable interrupts
	LDAA    L003F			;Status word
	EORA    #$05			;Toggle b0, RPMs falling, and b2, High idle speed
	STAA    L003F			;Save status word
	CLI				;Re-enable interrupts
;
;-Going richer here
;
L88C9	BRSET   L003F,#$03,L88DB	;Bra if b0==1, RPMs falling, and b1==1, 
					;RPMs changing too rapidly for idle 
;
	BRSET   L003F,#$0C,L88DB	;Bra if b2==1, high idle speed, and b3==1,
					;RPMs out of desired idle range
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Stable target idle here, allow greater fueling corrections
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
	LDAA    1,Y			;Load integrator delay term
	STAA    L0285			;Save it, integrator delay
	INX				;+1,
	INX				;+1,  
	BRA     L88E6			;Bra to continue				
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Unstable idle here, lessen fueling corrections
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~						
;					
L88DB	LDAA    0,Y			;Integrator delay term
	STAA    L0285			;Save it, INT delay term
	BRSET   L003E,#$40,L88E6	;Status word, bra if b6==1, going richer
;
;-Going leaner
;
	INX				;+1, index lean params     
;	
;-Apply idle based gains
;
L88E6	LDAA    0,X			;Load prop. gain term
	LDAB    L029D			;Proportional term
	MUL     			;Proportional term x gain
	ASLD				;x2, 
	BCS     L88F2			;Bra if overflow
;
	ASLD				;x2, now /64
	BCC     L891F			;Bra if no overflow
;
L88F2	LDAA    #255			;Clear overflow
	BRA     L891F			;Bra to continue	
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Engine not idling, load standard parameters
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;-Look up prop. duration
;			
L88F6	LDAA    L029E			;Scaled airflow
	LDX     #L4D1E			;Proportional duration vs. airflow
	JSR     LF15E			;2D lookup
;											
	PSHA				;Push duration to stack
;
;-Look up proportional flow gain 
;
	LDAA    L0062			;RPM/25
	CMPA    #144			;3600 RPM, UL for lookup
	BLS     L8908			;Bra if RPMs less
;
	LDAA    #144			;3600 RPM, upper limit
;
L8908	LDAB    L01E3			;SD MAP
	LSRB				;/2 for lookup 
	LDX     #L4D27			;Proportional gain table
	JSR     LF17B			;3D lookup
;	
;-Apply gain
;		
	LDAB    L029D			;Prop. term
	MUL				;Prop. term x gain
	ADDD    #64			;Round up
	ASLD				;x2, now /128
	BCC     L891F			;Bra if no overflow
;
	LDD     #$FFFF			;Clear overflow
;
;-Here if idling
;
L891F	STAA    L029D			;Save it, prop. term
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Look up prop. duration based on O2 error
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
	LDAA    L02A3			;O2 error
	ASLA				;x2 for lookup
	LDX     #L4D11			;Prop. duration vs. O2 error table
	JSR     LF15E			;2D lookup
;
	PULB				;Pull prop. duration back off stack   
	ABA				;Add in prop. duration vs O2 error     
	BCC     L8932			;Bra if no overflow
;																		
	LDAA    #255			;Clear overflow
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Look up INT delay based on O2 errror
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
L8932	PSHA				;Push prop. duration to stack 
	LDAA    L02A3			;O2 error
	ASLA				;x2 for lookup
	LDX     #L4D7B			;INT delay mult. vs. O2 error			
	JSR     LF15E			;2D lookup
;
	LDAB    L028E			;INT delay
	MUL				;Delay x mult.     
	BRCLR   L0050,#$80,L894C	;Status word, bra if b7==0, not idling
;
;-Idling here, add in additional delay
;
	ADDA    L0285			;Add in INT delay term for idling
	BCC     L894C			;Bra if no overflow
;
	LDAA    #255			;Clear overflow
;
L894C	STAA    L028E			;Save it, INT delay
	STAA    L02A4			;Stored INT delay
;
	LDAA    L0046			;Status word
	ANDA    #$02			;Clear all but b1, long term fueling status
;
	LDAB    L003E			;Status word
	ANDB    #$40			;Clear all but b6, rich/lean flag
	ABA				;Add together     
	PULB				;Get prop. duration off stack   
	BEQ     L8962			;Bra if ind. bits==0, going leaner overall
;
	CMPA    #$42			;Test fueling indicator bits
	BNE     L89B0			;Bra if no overall trend in fueling 
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Going richer or leaner overall here, update integrator
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
L8962	LDAA    L02A3			;Slow O2 error
	ASLA				;x2    
	CMPA    L493E			;Min error threshold to update INT
	BHI     L8970			;Bra if error >
;
;-Not a large enough error to update
;
	CLR     L0282			;Clear INT timer
	BRA     L89B0			;Bra to continue
;
;-Enough error, update INT
;
L8970	LDAA    L0282			;INT delay timer
	CMPA    L028E			;INT delay
	BCS     L897B			;Bra if INT timer <, dont update
;
	CLRA				;Reset INT delay timer    
	BRA     L897C			;Bra to update INT
;
;-Not time to update INT yet
;
L897B	INCA				;+1, INT delay timer
;    
L897C	STAA    L0282			;Save INT delay timer
	BNE     L89B0			;Bra if !=0, dont update yet
;
;-Time to update the INT
;
	LDAA	L0232			;INTegrator
	BRSET   L0046,#$02,L8998	;Status word, bra if b1==1, long term fueling rich
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Long term fueling is lean
;~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
	CMPA    L4907			;Max allowed INT
	BEQ     L89AD			;Bra if INT maxed out
;										
	BRCLR   L0052,#$80,L8995	;Status word, bra if b7==0, no quasi-closed loop
;
;-Using INT alone for initial corrections, limit INT to removing fuel only
;
	CMPA    #128			;Neutral INT
	BCC     L89AD			;Bra if INT >= 128
;
;-Full closed loop here
;
L8995	INCA				;+1, increment INT
	BRA     L89AD			;Bra to save
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Long term fueling is rich
;~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
L8998	LDX     #L400B			;Opt. byte
	BRCLR   0,X,#$08,L89A3		;Bra if b3==0, dont limit INT when AE in effect
;
;-Check if INT movement needs to be limited due to AE
; 
	BRSET   L0044,#$02,L89B0	;Status word, bra if b1==1, limit closed loop 
					;correction if AE active
;
L89A3	CMPA    L4906			;Min allowed INT
	BEQ     L89AD			;Bra if INT == min INT
;
;-Check for short PW
;
	BRSET   L003F,#$80,L89AD	;Status word, bra if b7==1, short BPW
;
	DECA				;-1 from INT
;
;-Here to skip INT adjustment
;
L89AD	STAA    L0232			;Save it, INT
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Limit time prop. term can be active for
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
L89B0	LDAA    L029F			;Duration in current fueling state 
	CBA				;Compare timer to proportional duration value    
	BLS     L89C5			;Bra if timer is <= prop. duration
;
	ADDB    L4944			;Add in hyst. for prop. duration dead band
	BCC     L89BD			;Bra if no overflow
;
	LDAB    #255			;Clear overflow
;
L89BD	CBA				;Compare timer to proportional duration value				     
	BCC     L89C5			;Bra if timer >
;
;-Clear proportional term, INT correction only if within dead band
;
	CLR     L029D			;Clear proportional term			
	BRA     L89CC			;Bra to continue
;
L89C5	BRCLR   L0052,#$80,L89CC	;Status word, bra if b7==0, quasi-closed loop
;
;-Clear proportional term, INT correction only for initial corrections
;
	CLR     L029D			;Clear proportional term

L89CC	LDAA    L029D			;Load proportional term
	BRCLR   L003E,#$40,L89F4	;Status word, bra if b6==0, going leaner
;
;-Going richer here, check if AE active
;
	LDX     #L400B			;Opt. byte
	BRCLR   0,X,#$08,L89E9		;Bra if b3==0, not limiting corrections
; 
	BRCLR   L0044,#$02,L89E9	;Bra if b1==0, AE is not in effect
;
;-AE in effect, limit closed loop corrections
;
	LDAA    L4908			;Prop. term limit when AE in effect											; ACELL LMT IN EFFECT     
	CMPA    L029D			;Compare to prop. term
	BCC     L89E9			;Bra if prop. term is <=
;
	STAA    L029D			;Use limit instead
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Construct final closed loop control output
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;-Going richer
;
L89E9	LDAA    L0232			;INTegrator
	SUBA    L029D			;Sub. proportional term			
	BCC     L89FB			;Bra if no underflow
;
	CLRA				;Clear underflow    
	BRA     L89FB			;Bra to save and return
;
;-Here when going leaner
;
L89F4	ADDA    L0232			;Add prop. term to INT
	BCC     L89FB			;Bra if no overflow
;
	LDAA    #255			;Clear overflow
;
L89FB	SUBA    #128			;Subtract neutral value
	STAA    L0292			;Save it, closed loop fuel correction term
	RTS				;Return     
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; Misc. fueling and BLM handling routine
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
L8A01	LDAA    L48DD			;Stoich AFR 
	LDAB    L003E			;Status word
	BITB    #$A0			;Check b5, and b7, closed loop
	BEQ     L8A1B			;Bra if ==0
;
;-In closed loop here
;
	STAA    L026D			;Save stoich AFR as AFR 
	LDX     #L400D			;Opt byte
	BRCLR   0,X,#$01,L8A50		;Bra if b0==0, dont look up open loop AFR 
					;when in closed loop 
;
	LDAB    L0006			;Cool temp, linearized
	CMPB    L48C7			;Upper cool temp limit for AFR lookup 
					;when in closed loop
	BHI     L8A35			;Bra if cool temp greater
;
;~~~~~~~~~~~~~~~~~~~~~~~
;-Look up open loop AFR
;~~~~~~~~~~~~~~~~~~~~~~~
;
L8A1B	LDX     #L4BB0			;Start addr. of table and opt. bit
	LDAB    L01EC			;Load Vac
	LDAA    0,X			;Index opt. bit
	BEQ     L8A28			;Bra if ==0
;
	LDAB    L01E3			;SD MAP
;
L8A28	INX				;+1, move to start of table
	LSRB				;MAP,Vac/2 for lookup
	LDAA    L0006			;Cool temp, linearized
	CMPA    #192			;104 deg C, upper limit for lookup
	BLS     L8A32			;Bra if cool temp <=
;
	LDAA    #192			;Load max cool temp
;
L8A32	JSR     LF17B			;3D lookup
;
;-Subtract out choke AFR
;
L8A35	SUBA    L02EB			;Stored cranking choke AFR
	BCC     L8A3B			;Bra if no underflow
;
	CLRA				;Clear underflow
;
L8A3B	BRCLR   L0050,#$80,L8A52	;Status word, bra if b7==0, not idling 
;
;~~~~~~~~~~~~~
; Idling here
;~~~~~~~~~~~~~
;
	CMPA    L0295			;Open loop idle AFR
	BLS     L8A47			;Bra if AFR >=
;
	LDAA    L0295			;Open loop idle AFR
;
L8A47	LDAB    #60			;1.5 secs	
	STAB    L0289			;Time to filter AFR on idle to run										
	CLRB				;Clear LSB
	STD     L0287			;Save it, filtered AFR
L8A50	BRA     L8A7B			;Bra to continue
;
;~~~~~~~~~~~~~~~~~
; Not idling here
;~~~~~~~~~~~~~~~~~
;
;-Check if AFR needs to be filtered after idling
;
L8A52	LDAB    L0289			;Load time to filter AFR 
	BEQ     L8A6B			;Bra if ==0 
;
	DECB				;-1 
	STAB    L0289			;Save timer
	CMPA    L0287			;Filtered AFR
	BLS     L8A6B			;Bra if current AFR is >= 
;
;-AFR is leaner then previous AFR
;
	LDX     L0287			;Idle AFR
	LDAB    L48E2			;Filter coeff. for filtering AFR
	JSR     LF0F6			;Filter routine
;
	BRA     L8A71			;Bra to save
;
;-Here if not filtering AFR
;
L8A6B	CLRB				;Clear LSB    
	STD     L0287			;Save current AFR as filtered AFR
	BRA     L8A7B			;Bra to continue
;
;-Here to save filtered AFR
;
L8A71	STD     L0287			;Save it, filtered AFR
	ASLB				;Shift LSB left
	ADCA    #$00			;Round if needed
	BCC     L8A7B			;Bra if no overflow
;											
	LDAA    #255			;Clear overflow			
;
L8A7B	STAA    L026D			;Save it, stored AFR
;
;~~~~~~~~~~~~~~~~~~~~~~~~
; Check quals for idling
;~~~~~~~~~~~~~~~~~~~~~~~~
;
;-Check speed threshold
;
	LDAA    L02A7			;MPH
	CMPA    L48CF			;Max speed to be idling
	BHI     L8AEE			;Bra if speed is >
;
;-Check TPS threshold
;
	LDAA    L01FC			;TPS
	CMPA    L48D0			;2.7% TPS, max TPS threshold to be idling
	BHI     L8AEE			;Bra if TPS is >
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; MPH and TPS within limits to be idling, check quals for open loop idle
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
	BSET    L0050,#$80		;Status word, set b7, idling
	LDAA    L0295			;Open loop idle AFR 
	CMPA    L026D			;Compare to stored AFR
	BHI     L8AFC			;Bra if AFR < open loop idle AFR, disable 
					;open loop idle
;
;-AFR is less then lean idle AFR limit here, enter open loop idle
;
	LDAB    L0006			;Cool temp, linearized
	CMPB    L48D8			;Threshold to remain in open loop idle
	BLS     L8AE9			;Bra if less, remain in open loop idle
;
	LDAB    L48D4			;Timer threshold to enable open loop idle 
	BEQ     L8AC9			;Bra if ==0, check INT thresholds
;
	LDX     #L400B			;Opt. byte
	BRSET   0,X,#$02,L8AB0		;Bra if b1==1, AIR enabled
;
;-No open loop idle in P/N if AIR not optioned
;
	BRSET   L0041,#$20,L8AFC	;Status word, bra if b5==1,in park/neutral
;
;-Here if AIR is enabled or in drive
;
L8AB0	BRCLR   L0050,#$02,L8AC1	;Status word, bra if not in open loop idle
;
;~~~~~~~~~~~~~~~~~~~
; In open loop idle
;~~~~~~~~~~~~~~~~~~~
;
	LDAB    L0294			;Idle timer
	CMPB    L48D5			;Timer threshold to transition to open loop idle AFR
	BLS     L8AD9			;Bra if timer <=, increment timer and cont.
;
;-Here to use open loop idle AFR
;
	LDAA    L0295			;Load open loop idle AFR
	BRA     L8B05			;Bra to continue
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Not in open loop idle, check quals to enter
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
L8AC1	LDAB    L48D4			;Timer threshold to enable open loop idle
	CMPB    L0294			;Idle timer
	BCC     L8AD9			;Bra if timer is <=, dont allow open loop idle yet
;
;-Check INT thresholds for open loop idle
;
L8AC9	LDAB    #128			;Neutral INT value
	LDAA    L0232			;INT
	CMPA    #128			;Compare to neutral value
	BCC     L8AD3			;Bra if adding fuel
;
;-Fueling is leaning out
;
	NEGA				;Negate INT
;
L8AD3	ABA				;Sub out neutral INT to get diff. from neutral val.
	CMPA    L48EE			;Delta INT threshold to bar open loop idle
	BLS     L8AE4			;Bra if delta INT is less
;
;-INT threshold not yet met for open loop idle and AIR
;
L8AD9	LDAB    L0002			;Loop counter
	CMPB    #$27			;Counter val to update timer
	BNE     L8B02			;Bra if !=
;
	INC     L0294			;+1, idle timer
	BRA     L8B02			;Bra to continue
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Here to enable open loop idle
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
L8AE4	BSET    L0050,#$02		;Status word, set b1, open loop idle
	BRA     L8AFF			;Bra to continue
;
;-Alternate entry point if not resetting idle timer
;
L8AE9	BSET    L0050,#$02		;Status word, set b1, open loop idle
	BRA     L8B05			;Bra to continue
;
;~~~~~~~~~~~~~~~~~~~~
; Here if not idling
;~~~~~~~~~~~~~~~~~~~~
;
L8AEE	LDD     L00F8			;Engine run time
	CPD     L48D6			;Engine run time to clear initial idle terms
	BCS     L8AF9			;Bra if timer is <
;
	BSET    L003F,#$40		;Status word, set b6, clear initial added IAC flow
;											
L8AF9	BCLR    L0050,#$80		;Status word, clear b7, idle flag
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Here to disable open loop idle
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
L8AFC	BCLR    L0050,#$02		;Clear open loop idle flag
;											
L8AFF	CLR     L0294			;Clear idle timer
;											
L8B02	LDAA    L026D			;Stored AFR
											 
L8B05	BRSET   L003E,#$80,L8B3F	;Status word, bra if b7==1, in closed loop
;
;~~~~~~~~~~~~~~~~~~~
; In open loop here
;~~~~~~~~~~~~~~~~~~~
;
	BRCLR   L0004,#$10,L8B13	;Status word, bra if b4==0, no hot restart
;
;-Hot restart in effect
;										   	
	SUBA    L4952			;Subtract AFR bias for hot restart 
	BCC     L8B13			;Bra if no underflow
;
	CLRA				;Clear underflow
;
;-Hot restart not in effect, check temp limits for open loop AFR enrichment
;
L8B13	LDAB    L0006			;Cool temp, linear
	CMPB    L48DA			;Lower temp limit for open loop enrichment
	BLS     L8B3F			;Bra if cool temp <=
;										   	
	CMPB    L48DB			;Upper temp limit for open loop enrichment
	BCC     L8B3F			;Bra if temp >=
;										   	
	BRSET   L0041,#$20,L8B39	;Status word, bra if b5==1, in park/neutral
;	
;-In drive
;									  	
	LDAB    L082C			;Initial AFR enrichment timer
	CMPB    L48DC			;Time limit for initial AFR enrichment
	BHI     L8B3F			;Bra if timer >
;										  	
	INCB				;+1
	BEQ     L8B31			;Bra if roll over
;										  	
	STAB    L082C			;Save timer
;	
;-Apply initial AFR enrichment 
;										
L8B31	SUBA    L48E0			;Subtract initial AFR entrichment
	BCC     L8B3F			;Bra if no overflow
;										  	
	CLRA				;Clear overflow
	BRA     L8B3F			;Bra to continue				
;
;-Here if in park/neutral 
;											
L8B39	SUBA    L48E1			;Park/neutral open loop AFR enrichment
	BCC     L8B3F			;Bra if no overflow
;
	CLRA				;Clear overflow
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Here if closed loop flagged
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
L8B3F	LDAB    L4909			;Lean AFR limit
	BRCLR   L0072,#$40,L8B49	;Status word, bra if b6=0, no err 44, lean O2
;
;-Error 44 present
;
	LDAB    L4E71			;Lean AFR limit if err 44
;
L8B49	CBA				;Compare AFR to AFR limit
	BLS     L8B4D			;Bra if AFR is richer
;
	TBA				;Use AFR limit instead
;
;-Check for transition to run
;
L8B4D	LDAB    L028B			;Elapsed DRPs
	CMPB    L4955			;DRP threshold to transition from crank to run
	BCS     L8B6B			; 
;
	LDAB    L0006			;Cool temp, linearized
	CMPB    L4957			;Threshold to add AFR transition ratio in
	BHI     L8B64			;Bra if Cool temp >, add it in
;
;-Subtracting AFR, richen transition
;
	SUBA    L02EC			;Subtract AFR crank transition ratio
	BCC     L8B6B			;Bra if no underflow
;
	CLRA				;Clear underflow
	BRA     L8B6B			;Bra to continue					
;
;-Adding AFR, lean out transition
;
L8B64	ADDA    L02EC			;Add AFR crank transition ratio
	BCC     L8B6B			;Bra if no overflow
;
	LDAA    #$FF			;Clear overflow
;
L8B6B	STAA    L026D			;Save it, AFR
	BRSET   L004F,#$10,L8B75	;Status word, bra if b4==1, engine running 
;
;~~~~~~~~~~~~~~~~~~~~~~~~
; Engine not running yet
;~~~~~~~~~~~~~~~~~~~~~~~~
;
	JMP     L8D50			;				
;
;~~~~~~~~~~~~~~~~~~~~~
;-Engine running here
;~~~~~~~~~~~~~~~~~~~~~
;
;-check thresholds to bar delta speed fast entry into PE
;	
L8B75	LDAA    L02A7			;MPH
	CMPA    L4917			;Min speed threshold for delta MPH fast
					;entry into PE
	BCC     L8B8D			;Bra if MPH greater
;
	LDAA    L01FC			;TPS
	CMPA    L4918			;TPS threshold for delta MPH fast entry into PE
	BHI     L8B8D			;Bra if TPS >
;
;-Bar delta speed entry into PE here if low speed/throttle present
;
	BSET    L003D,#$01		;Set b0, no delta speed entry into PE allowed
	CLR     L0286			;Clear delta speed PE entry timer
	BRA     L8B9C			;Bra to continue
;
;-Here to bar delta speed entry into PE for period of time
;										
L8B8D	LDAA    L0286			;Delta PE entry timer
	CMPA    L4919			;Threshold to continue to bar delta speed entry
	BCC     L8B9C			;Bra if timer >=
;
	INCA				;+1
	STAA    L0286			;Save PE entry timer										
	BSET    L003D,#$01		;Set b0, delta speed fast PE entry not allowed
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Lookup TPS threshold for WOT vs. RPM
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
L8B9C	LDAB    #32			;800 RPM, offset for lookups
	LDX     #L4C50			;TPS threshold for WOT table
	LDAA    L0062			;RPM/25
	JSR     LF137			;2D lookup with UL
;
	STAA    L0203			;Save it, TPS threshold for WOT
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Lookup fast PE delay timer countdown rate vs. RPM
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;-Decreases time to enter PE if TPS > L0203
;
	LDX     #L4C75			;Fast PE delay timer countdown rate 
	LDAA    L0062			;RPM/25
	JSR     LF137			;2D lookup with UL
;											
	STAA    L0204			;Fast PE delay timer countdown rate
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Lookup TPS threshold for PE
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
	LDX     #L4C5A			;TPS threshold for PE table 
	LDAA    L0062			;RPM/25
	JSR     LF137			;2D lookup with UL
;											
	STAA    L0205			;Save it, TPS threshold for PE
	TAB				;Get threshold into B     
;  
	BRCLR   L003D,#$20,L8BCA	;Status word, bra if b5==0, not in PE
;
;~~~~~~~~~~~~~~~~~~~~~~~~
; Check threshold for PE
;~~~~~~~~~~~~~~~~~~~~~~~~
;
;-In PE here, apply hysteresis
;
	SUBB    L490D			;TPS hyst. remain in PE
	BCC     L8BCA			;Bra if no underflow
;
	CLRB				;Clear underflow  
;
;-Not in PE here
;  
L8BCA	CMPB    L01FC			;Compare to TPS
	BCS     L8BD2			;Bra if TPS >, continue with PE quals.
;
;-TPS is less then that needed to enter PE
;
	JMP     L8CDF			;Skip PE
;
L8BD2	BRSET   L003F,#$20,L8C34	;Status word, bra if b5==1, bypass PE delay 
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Check quals to bypass PE delay
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
	LDAA    L0063			;RPM/12.5
	SUBA    L0065			;RPM/12.5, one delayed
	BCC     L8BDD			;Bra if RPMs rising
;
;-RPMs falling here
;
	CLRA				;Clear delta RPM/12.5    
;
L8BDD	CMPA    L490A			;Delta RPM to bypass PE delay
	BLS     L8BE8			;Bra if delta RPMs <= 
;
;-RPMs rising fast enough to bypass PE delay
;

	BSET    L003F,#$20		;Status word, set b5, bypass PE delay 
	JMP     L8C34			;Jump to enter PE
;
L8BE8	LDAB    L0006			;Cool temp, linearized
;
;-Check upper cool temp threshold to bypass PE delay
;
	CMPB    L490C			;Upper cool temp threshold to bypass PE delay
	BHI     L8C34			;Bra if cool temp >, bypass delay
;
;-Check lower cool temp threshold to bypass PE delay
;
	CMPB    L490B			;lower cool temp threshold to bypass PE delay/
					;allow for AFR adjustments
	BLS     L8C34			;Bra if cool temp less, bypass delay
;
;-Check for high RPMs
;
	LDAB    L490E			;Upper RPM threshold for PE delay/AFR adj.
	CMPB    L0062			;RPMs/25
	BLS     L8C34			;Bra if RPMs >=
;
;-Check TPS baro factor for PE 
;
	LDAA    L0206			;TPS baro correction factor
	LDAB    L01FC			;TPS
	SUBB    L01FE			;TPS, one delayed
	BCS     L8C09			;Bra if TPS is falling
;
;-TPS rising here
;
	CBA				;Compare delta TPS baro factor to delta TPS     
	BLS     L8C31			;Bra if TPS is greater, bypass PE delay
;
L8C09	LDAA    L01F0			;Baro PE RPM threshold 
	CMPA    L0062			;RPM/25
	BLS     L8C17			;Bra if RPMs >=
;
;-RPMs less then baro threshold here
;  
	BRSET   L003D,#$20,L8C17	;Status word, bra if PE in effect
; 
	JMP     L8CDF			;Bar entry into PE if not already in effect
;
L8C17	BRSET   L003D,#$01,L8C1E	;Status word, bra if b0==1, delta speed fast PE
					;entry not allowed
;
;-Jump to check delta MPH threshold for fast PE entry
;  
	JMP     L8CD6			;Continue, check delta MPH theshold to enter PE
;
;-Here if delta MPH fast PE entry not allowed
;
L8C1E	LDAB    L029A			;PE delay timer
	CMPB    L4910			;Delay for PE to become active											; SLEW RATE
	INCB				;+1
	BCS     L8C3A			;Bra if PE delay timer is <
;
	LDAB    L029B			;PE ramping multiplier
	ADDB    L4911			;Add in ramping rate
	BCC     L8C36			;Bra if no overflow
;	
	BRA     L8C34			;
;
;~~~~~~~~~~~~~~~~~~~~~~~~~
; Here to bypass PE delay
;~~~~~~~~~~~~~~~~~~~~~~~~~
;
L8C31	BSET    L003F,#$20		;Status word, set b5, bypass PE delay next pass
;	
L8C34	LDAB    #255			;Clear overflow from previous op.
;
L8C36	STAB    L029B			;Save it, PE ramping multiplier
;
L8C39	CLRB				;Reset PE delay timer
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Here if timer less then threshold to bring in PE
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;    
L8C3A	STAB    L029A			;Save PE delay timer   
	BSET    L003D,#$20		;Status word, set b5, PE now underway
;
;-Look up PE AFR
;
	LDX     #L4C64			;PE AFR vs. RPM table
	LDAA    L0062			;RPM/25
	JSR     LF15E			;2D lookup
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Check quals for short term PE AFR enleanment
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
	TAB				;Get PE AFR into B     
	LDAA    L028C			;PE timer
	CMPA    L4913			;Threshold for extended PE
	BHI     L8CA4			;Bra if if time in PE >
;
;-Not in extended PE yet
;
	LDAA    L0002			;Loop counter
	CMPA    #$07			;Counter val, 1 sec
	BNE     L8C5A			;Bra if !=, dont inc timer
;
	INC     L028C			;PE timer
;
;-Check RPM/cool temp to allow PE AFR adjustments
;
L8C5A	LDAA    L0062			;RPM/25			
	CMPA    L490E			;Upper RPM threshold for PE delay/AFR adj.											; BYPASS PE DELAY  
	BCC     L8CA4			;Bra if RPMs greater
;
	LDAA    L0006			;Cool temp, linearized
	CMPA    L4914			;Upper cool temp threshold to allow adjustments
	BHI     L8CA4			;Bra if cool temp >
;
	CMPA    L490B			;lower cool temp threshold to bypass PE delay/
					;allow for AFR adjustments
	BLS     L8CA4			;Bra if cool temp is <=
		
	LDAA    L01FC			;TPS
	CMPA    L0203			;TPS threshold for WOT
	BHI     L8CA4			;Bra if TPS >, skip adjustments
;
;-Lean PE AFR out if needed to smooth engagement of PE
;
	SUBA    L0205			;Subtract TPS threshold for PE
	BCC     L8C7C			;Bra if no underflow
;
;			          TPS - TPS threshold for PE
;-Ratio for enleanment = ------------------------------------------
;			  WOT TPS threshold - TPS threshold for PE
;
	LDAA    #$0000			;Clear underflow
;
L8C7C	PSHB				;Push PE AFR to stack
	PSHA				;Push delta TPS to stack
	LDAA    L0203			;TPS threshold for WOT
	SUBA    L0205			;Subtract TPS threshold for PE
	BCC     L8C88			;Bra if no underflow
;
	LDAA    #$0000			;Clear underflow
;
;-Adjustment = Enleanment AFR - Enleanment AFR x ratio
;
L8C88	CLRB				;Preclear B
	XGDX				;Swap result into X
	PULA				;Get delta TPS back
	CLRB				;
	FDIV				;Divide delta TPS above PE threshold
					;by delta WOT TPS above threshold
	XGDX				;Get FDIV result into D
	LDAB    L491B			;Base PE enleanment AFR 
	MUL				;Ratio x AFR
	ADCA    #$00			;Round if needed
	TAB				;Get result into B
	LDAA    L491B			;Load enleanment AFR 
	SBA				;Subtract result
	BCC     L8C9D			;Bra if no underflow
;
	LDAA    #$0000			;Clear underflow
;
L8C9D	PULB				;Get PE AFR back
	ABA				;Add in adjustment
	BCC     L8CA3			;Bra if no overflow
;
	LDAA    #$FF			;Clear overflow
;
L8CA3	TAB				;Get result into B
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Here to skip PE AFR adjustments
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
L8CA4	LDAA    L0006			;Cool temp
	CMPA    L4912			;Threshold for cold PE
	BHI     L8CC0			;Bra if cold temp >
;
;-Cool PE in effect
;
	LDAA    L0062			;RPM/25
	CMPA    L4915			;Lower RPM limit for cold PE AFR enrichment
	BCC     L8CC0			;Bra if RPMs >
;
	LDAA    L02A7			;MPH
	CMPA    L4916			;MPH limit for cold PE AFR enrichment
	BCC     L8CC0			;Bra if MPH >=
;
;-Low speed cold PE in effect
;
	SUBB    L491A			;Subtract cold PE AFR enrichment
	BCC     L8CC0			;Bra if no underflow
;
	CLRB				;Clear underflow
;
L8CC0	LDAA    L026D			;AFR
	SBA				;Subtract modified PE AFR, now delta AFR for PE
	BLS     L8D1B			;Bra if current AFR is equal or richer
;
;-Ramp the PE AFR in for a smooth entry into PE
;
	LDAB    L029B			;PE ramping multiplier
	MUL				;Mult. x delta AFR
	ADCA    #00			;Round if needed
	TAB				;Get MSB of result into B
	LDAA    L026D			;AFR
	SBA				;Subtract delta AFR
	STAA    L026D			;Save it, AFR
	BRA     L8D1B			;Bra to continue
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Here to check delta MPH threshold for PE
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
L8CD6	LDD     L0818			;Delta MPH
	CPD     L491C			;Delta MPH threshold for PE 										; ENRICH IF L.T. 0 MPH  
	BCC     L8CF0			;Bra if delta MPH are greater, enter PE
;
;~~~~~~~~~~~~~~~~~
; Here to exit PE
;~~~~~~~~~~~~~~~~~
;
L8CDF	LDAB    L0299			;PE reset/delay timer
	CMPB    L490F			;Time out of PE to reset
	INCB				;+1
	BCS     L8D0C			;Bra if timer is less, dont reset yet
;
;-Here to reset for next power enrich
;										
	CLR     L029B			;PE AFR ramping multiplier
	CLR     L028C			;PE timer
	BRA     L8D15			;Bra to continue
;
;~~~~~~~~~~~~~~~~~~
; Here to enter PE
;~~~~~~~~~~~~~~~~~~
;
L8CF0	LDAB    L0299			;PE reset/delay timer
	BNE     L8CFB			;Bra if timer !=0, have not reset yet
;
;-PE delay timer has expired, check quals for PE
;  
	BSET    L003D,#$01		;Status word, set b0, bar delta speed fast entry
					;into PE and check quals
	JMP     L8C39			;Bra to check quals for PE
;
;-Here if recently in PE
;
L8CFB	LDAA    L01FC			;TPS
	CMPA    L0203			;TPS threshold for WOT
	BLS     L8D0B			;Bra if TPS <=, not at WOT
;
;-Wide open throttle preset
;
	SUBB    L0204			;Fast PE delay timer countdown rate
	BCC     L8D0C			;Bra if no underflow
;
	CLRB				;Clear underflow
	BRA     L8D0C			;Bra to continue
;
;-Here if intermediate throttle position
;
L8D0B	DECB				;-1, PE reset/delay timer
;
L8D0C	LDAA    L0002			;Loop counter
	CMPA    #$0017			;
	BNE     L8D15			;Bra if timer !=, dont decrement/increment counter
;
	STAB    L0299			;PE reset/delay timer
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Here to exiting or delaying entry into PE
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;    
L8D15	BCLR    L003D,#$21		;Status word, clear b0, b5, PE flags
	BCLR    L003F,#$20		;Status word, clear b5, bypass PE flag
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Here if PE already underway
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;											
L8D1B	LDAA    L01FC			;TPS
	STAA    L01FE			;Save it, TPS, one delayed
	BRCLR   L0052,#$40,L8D36	;Status word, bra if b6==0, no cat overtemp
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Cat overtemp present, limit AFRs
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
	LDAA    L0255			;Gms/sec airflow
	LDX     #L498F			;AFR limit for cat overtemp table
	JSR     LF15E			;2D lookup
	CMPA    L026D			;Compare to current AFR
	BHI     L8D36			;Bra if current AFR is richer
;
	STAA    L026D			;Save AFR limit instead
;
;~~~~~~~~~~~~~~~~~~~~~
; Check ALDL status
;~~~~~~~~~~~~~~~~~~~~~
;
L8D36	BRCLR   L003B,#$10,L8D49	;Status word, bra if b4==0, not in mode 4 
;
;-In mode 4
;
	LDAA    L0397			;				
	BITA    #$04			;
	BEQ     L8D49			;
;
	LDAA    L0399			;
	STAA    L026D			; 
	BRA     L8D50			;
;
;~~~~~~~~~~~~~~~~~~~~ 
; Check for heads up
;~~~~~~~~~~~~~~~~~~~~
;
L8D49	BRCLR   L0082,#$40,L8D50	;Status word, bra if b6==0, no HU
;
	JSR     L1800			;
;
L8D50	LDAA    L026D			;AFR
	STAA    L026E			;Stored AFR
;
;-Check if its time to update delta MPH
;
	LDAA    L0002			;Loop counter
	CMPA    #$9F			;
	BNE     L8D71			;Bra if counter !=, dont update yet
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Compute delta MPH term for PE
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
	LDD     L02A7			;MPH
	SUBD    L081A			;MPH, one delayed
	BCC     L8D68			;Bra if no underflow
;
	NEGA				;Negate, get abs. value 		   
	NEGB				;		    
	SBCA    #$00			;Perform carry if needed
;
L8D68	STD     L0818			;Save it, delta MPH for PE entry
	LDD     L02A7			;MPH
	STD     L081A			;Save it, MPH, one delayed
;
;~~~~~~~~~~~~~~~~~~~
; BLM cell handling
;~~~~~~~~~~~~~~~~~~~
;    
L8D71	BCLR	L003D,#$04		;Status word, clear b2, BLM cell change
;									
	LDAA    L48EC			;Opt. byte
	BITA    #$01			;Test b0, use seperate idle cells
	BEQ     L8DAE			;Bra if ==0. dont use seperate cells
;
;-Check TPS threshold for idling
;
	LDAB    L01FC			;TPS
	CMPB    L48D0			;TPS threshold to be idling
	BCC     L8DAE			;Bra if TPS >=, not idling
;
;-Check MPH threshold for idling
;
	LDAB    L02A7			;MPH
	CMPB    L48CF			;Max speed to be idling
	BCC     L8DAE			;Bra if VSS >=
;
;-Check if using seperate cells for A/C 
;
	LDAB    #16			;Start of idle BLM cells 
	BITA    #$02			;Test b1, seperate idle BLM cell when A/C on
	BEQ     L8D96			;Bra if ==0, dont use seperate cells
;
	BRCLR   L0041,#$10,L8D96	;Status word, bra if b4==0, A/C not on 
;
	INCB				;+1, advance to A/C idle cells
;
;-Check if using seperate cell for P/N
;
L8D96	BITA    #$04			;Test b2, seperate park/neutral cell
	BEQ     L8DA0			;Bra if ==0
;
	BRCLR   L0041,#$20,L8DA0	;Status word, bra if not in P/N
;
	ADDB    #2			;+2, advance to park/neutral cell
;
;-Check for single fire mode
;	
L8DA0	BRCLR   L0051,#$01,L8DA6	;Bra if b0==0, not in single fire mode
;
	LDAB    #20			;Use cell #20 if in single fire
;
L8DA6	CMPB    L026A			;Current BLM cell #
	BEQ     L8E1D			;Bra if still using the same cell
;
;-BLM cell has changed
;	
	JMP     L8E56			;
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Here if not idling, check BLM cell bounds
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
L8DAE	LDAB    L026A			;Load current BLM cell #
	CMPB    #16			;Start of idle cells
	BCC     L8E1F			;Bra if currently using an idle BLM cell
;
;-Not currently in an idle cell, check RPM bounds
;
	ANDB    #$03			;Clear all but lower 2 bits
	LDX     #L48E4			;Index start of BLM boundries
	ABX				;add in current BLM cell #    
	DEX				;-1, move to previous boundry  
;
;-Check lower RPM bound
;   
	TSTB				;Test BLM cell #    
	BEQ     L8DCE			;Bra if currently at the lowest RPM bound
;
	LDAA    0,X			;Load RPM boundry
	SUBA    L48EA  			;Subtract RPM hyst. for cell change
	BCS     L8DCA			;Bra if underflow
;
	CMPA    L0062			;RPM/25
	BHI     L8E1F			;Bra if RPMs <, bra to change cells
;
;-Check current RPM boundry
;
L8DCA	CMPB    #3			;Upper RPM boundry
	BEQ     L8DD9			;Bra if already at upper RPM boundry
;
L8DCE	LDAA	1,X			;Get current RPM boundry
	ADDA    L48EA 			;Add in RPM hyst. for cell change
	BCS     L8DD9			;Bra if overflow	
;
	CMPA    L0062			;Compare to RPM/25
	BCS     L8E1F			;Bra if RPMs >change cells
;
;-Check MAP boundries
;
L8DD9	LDAB    L026A			;Get current BLM
	ANDB    #$0C			;Clear all but bits 2 and 3
	LSRB				;Shift right to get BLM boundry #
	LSRB				;Shift right again  
	LDX     #L48E7			;Get start addr of BLM MAP boundries
	ABX				;Add in BLM cell # as offset
	DEX				;-1, move to previous boundry     
	TSTB				;Test boundry #    
	BEQ     L8E04			;Bra if already at lowermost boundry
;
;-Check lower MAP boundry
;
	LDAA    0,X			;Load lower MAP boundry
	SUBA    L48EB			;Subtract MAP hyst. for cell change 
	BCS     L8E00			;Bra if underflow
;
	LDY     #L400E			;Opt. byte
	CMPA    L01F2			;MAP/baro ratio
	BRSET   0,Y,#$01,L8DFE		;Opt. byte, bra if b0==1, use MAP/baro ratio 
					;for BLM map boundries
;
	CMPA    L01E3			;Compare to SD MAP instead
;
L8DFE	BHI     L8E1F			;Bra if MAP <, change cells
;
L8E00	CMPB    #3			;Uppermost BLM cell #
	BEQ     L8E1A			;Bra if already at uppermost cell
;
;-Check current MAP boundry
;
L8E04	LDAA    1,X			;Load in current MAP boundry
	ADDA    L48EB			;Add in hyst. for cell change
	BCS     L8E1A			;Bra overflow
;
	CMPA    L01F2			;MAP/baro ratio
;
	LDX     #L400E			;Opt. byte
	BRSET   0,X,#$01,L8E18		;Opt. byte, bra if b0==1, use MAP/baro ratio 
					;for BLM map boundries
;											 
	CMPA    L01E3			;Compare to SD MAP instead
;
L8E18	BCS     L8E1F			;Bra if MAP is >, change cells
;
L8E1A	LDAB    L026A			;Load in current BLM cell #
;
L8E1D	BRA     L8E5C			;
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Here to change cells based on RPM/MAP
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
L8E1F	CLRB				;Clear cell # 
	LDAA    L0062			;Load RPM/25
	CMPA    L48E4			;Lowermost RPM boundry
	BCS     L8E34			;Bra if RPMs <, use current boundry
;
	INCB				;+1, adv. BLM to next set of cells
	CMPA    L48E5			;Middle RPM boundry
	BCS     L8E34			;Bra if RPMs <, use current boundry
;
	INCB				;+1, adv. BLM to next set of cells 
	CMPA    L48E6			;Uppermost RPM boundry
	BCS     L8E34			;Bra if RPMs <, use current boundry
;
	INCB				;+1, adv. BLM to next set of cells
;
L8E34	LDAA    L01F2			;MAP/baro
;
	LDX     #L400E			;Opt. byte, use MAP/baro ratio
	BRSET   0,X,#$01,L8E41		;Bra if b0==1, use ratio for cell change
;
	LDAA    L01E3			;Load SD MAP instead
;
L8E41	CMPA    L48E7			;Lower BLM MAP boundry
	BCS     L8E56			;Bra if MAP >, use current cell row
;
	ADDB    #$04			;+4, advance to next row 
	CMPA    L48E8			;Middle BLM MAP boundry
	BCS     L8E56			;Bra if MAP >, use current cell row
;
	ADDB    #$04			;+4, adv. to next cell row
	CMPA    L48E9			;Uppermost BLM cell boundry
	BCS     L8E56			;Bra if MAP >, use current cell row
;
	ADDB    #$04			;+4, advance to uppermost cell row
;
;-Here if idling
;   
L8E56	BSET    L003D,#$0C		;Status word, set b2, BLM cell change, and 
					;b3, delay BLM update
	BSET    L006E,#$01		;Status word, set b0, BLM cell change (EGR diags.)
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Get current BLM and check limits
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
L8E5C	LDX     #L02D6			;Start of BLM cells
	ABX				;Add in offset to get current BLM cell    
	LDAA    0,X			;Load BLM
	CMPA    L48F2			;Upper limit to adjust BLM
	BHI     L8E75			;Bra if BLM >, no update
;
	CMPB    #16			;Start of BLM idle cells
	BCS     L8E70			;Bra if in an idle cell 
;
	CMPA    L48F0			;Min BLM if not idling
	BRA     L8E73			;
;
;-Here if in an idle cell, check alternate limit for BLM
;
L8E70	CMPA    L48EF			;Min BLM if idling
;
L8E73	BCC     L8E7B			;Bra if BLM is >= threshold
;
;-Reset BLM cells if BLM falls below lower bound
;
L8E75	BSET    L0046,#$40		;Status word, set b6, BLM cells reset
	JSR     LEF1D			;BLM cell reset routine
;
L8E7B	STAA    L026B			;Save BLM
	STAB    L026A			;Save BLM cell #
	LDX     #L400C			;Opt. byte
	BRSET   0,X,#$10,L8ED2		;Bra if b4==1, always flag cell change
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Check quals to flag BLM cell change
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
	LDAA    L01FC			;TPS
	CMPA    L48D0			;TPS threshold to be idling
	BCS     L8E99			;Bra if TPS <=, idling
;
;-Not idling here
;
	BRCLR   L0052,#$04,L8EA2	;Status word, bra if b2==0, idle bit cleared already
;	
	BCLR    L0052,#$04		;Clear b2, engine idling (BLM routine)
	BRA     L8ECC			;Bra to continue
;
;-Here if idling
;
L8E99	BRSET   L0052,#$04,L8EA2	;Bra if idle already flagged
;
	BSET    L0052,#$04		;Status word, set b2, engine idling (BLM routine)
	BRA     L8ECC			;Bra to continue
;
;-Here if no change in status (idle/open throttle)
;
L8EA2	LDAA    L026C			;BLM, one delayed
	SUBA    L026B			;Current BLM
	BEQ     L8EC9			;Bra if no change
;
;-BLM has changed
;
	BMI     L8EBB			;Bra if BLM falling
;
;-BLM is rising here, check delta limit to flag BLM cell change
;
	CMPA    L48FF			;Rising delta BLM threshold to clear BLM cell change 
	BCS     L8EC9			;Bra if BLM change < no dont flag change
;
	LDAA    L0232			;INT
	CMPA    L48FD			;Lower INT threshold to flag change in BLMs
	BHI     L8EC9			;Bra if INT higher, dont flag change if going
					;leaner overall
;
	BRA     L8ECC			;Bra, will be flagging BLM cell change
;
;-Here if BLM falling
;
L8EBB	NEGA				;Get abs. value    
	CMPA    L4900			;Falling delta BLM threshold to clear BLM cell
					;change
	BCS     L8EC9			;Bra if BLM cell change <, dont flag change
;
	LDAA    L0232			;Current INT
	CMPA    L48FE			;Upper INT threshold to flag change
	BCC     L8ECC			;Bra if INT >=
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Little or no change in BLM, dont flag BLM cell change
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
L8EC9	BCLR    L003D,#$04		;Status word, clear b2, BLM cell change
;
;-Here if flagging cell change
;
L8ECC	LDAA    L026B			;Load BLM
	STAA    L026C			;Save it, BLM, one delayed
;
L8ED2	LDAA    L0263			;Filtering execution timer
	BNE     L8EFD			;Bra if !=0, dont perform filtering yet
;
	LDAA    L48BE			;Execution interval
	STAA    L0263			;Save it, timer
;
;-Filter the TPS
;
	LDAB    L48BF			;Slow filtered TPS filter coeff.
	LDX     L0201			;Slow filtered TPS
	LDAA    L01FC			;TPS
	JSR     LF0F6			;Filter routine
;
	STD     L0201			;Save it, slow filtered TPS
;
;-Filter the MAP
;
	LDAB    L01EE			;Cool temp based filter coeff.
	LDX     L01EA			;SD MAP, filtered per cool tmp
	LDAA    L01E3			;SD MAP
	JSR     LF0F6			;Filter routine
;
	STD     L01EA			;Save it, S/D MAP, filt'd per cool tmp
	BRA     L8F00			;
;
;-Here if not executing filtering yet
;
L8EFD	DEC     L0263			;-1, Exec. timer
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Update BLMs based on INT and direction of fueling
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
L8F00	BRCLR   L003E,#$02,L8F4B	;Status word, bra if b1==0, not enabling BLM update
;
	BRSET   L003D,#$08,L8F4B	;Status word, bra if b3==1, delaying update
;
	LDAA    L0232			;INTegrator
	CMPA    #128			;Compare to neutral value
	BEQ     L8F4B			;Bra if INT is neutral
;
	LDAB    L0297			;BLM loop timer 
	INCB				;+1
	BMI     L8F1E			;Bra if >= 127, continue
;
;-Check to see if INT has updated yet
;
	STAB    L0297			;Save it
	ASLB				;x2 to match loop execution speed of parent loop   
	CMPB    L028E			;Compare to INT delay timer
	BCS     L8F89			;Bra if <, dont update BLM yet until INT
					;has updated
;
;-Check to see if enough time has passed to update the BLM
;
L8F1E	LDAB    L0296			;BLM update delay timer
	INCB				;+1   
	BEQ     L8F2C			;Bra if rollover
;
	STAB    L0296			;Save timer
	CMPB    L48E3			;BLM update frequency
	BCS     L8F89			;Bra if timer is <, no update
;
;-Update the BLMs
;
L8F2C	LDX     #L48ED			;Delta INT to update BLMs (also used in EGR test)
	LDY     #L48EF			;Lower BLM limit

	LDAB    L026A			;Load BLM cell #
	CMPB    #16			;Start of idle cells
	BCS     L8F3D			;Bra if not in an idle cell
;
;-In an idle cell
;
	INX				;Delta INT to update idle BLMs    
	INY				;Lower idle BLM limit
;     
L8F3D	SUBA    #128			;Subtract neutral value from INT
	BCS     L8F4E			;Bra if INT is < 128
;
;-Adding fuel here
;
	CMPA    0,X			;Delta INT needed to update BLM
	BLS     L8F89			;Bra if delta INT <=, no update
;
	BRSET   L003E,#$40,L8F89	;Status word, bra if b6==1, going richer, no update
;
;-Going leaner overall
;
	BRA     L8F57			;Bra to continue
;
;-Here if neutral INT
;
L8F4B	JMP     L8F82			;Exit routine
;
;-Removing fuel here
;
L8F4E	NEGA				;Get abs. value    
	CMPA    0,X			;Delta INT to update BLMs
	BLS     L8F89			;Bra if <=, no update
;
	BRCLR   L003E,#$40,L8F89	;Bra if b6==0, going leaner, no update
;
L8F57	LDX     #L02D6			;Start addr. of BLMs
	ABX				;Add in offset of current BLM
	LDAA    0,X			;Load current BLM
;
	BRCLR   L003E,#$40,L8F70	;Bra if going leaner
;
;-Decrement BLM
;
	SUBA    L48F1			;Subtract BLM step
	BCS     L8F6B			;Bra if underflow
;
	CMPA    0,Y			;Lower limit for BLM
	BCC     L8F7D			;Bra if BLMs >=, bra to save
;
L8F6B	LDAA    0,Y			;Load limit instead
	BRA     L8F7D			;Bra to save 
;
;-Increment BLM
;
L8F70	ADDA    L48F1			;Add in BLM step
	BCS     L8F7A			;Bra if overflow
;
	CMPA    L48F2			;Max BLM
	BLS     L8F7D			;Bra if BLMs <=
;
L8F7A	LDAA    L48F2			;Load limit instead
;
L8F7D	STAA    0,X			;Save BLM
	STAA    L026B			;Save as current BLM
;
;-Here to exit routine and reset timers
;
L8F82	CLRB				;   
	STAB    L0297			;Clear BLM loop timer
	STAB    L0296			;Clear BLM update timer
;
;-Here to exit routine
;
L8F89	BCLR    L003D,#$08		;Status word, clear b3, reset delay BLM update flag
	LDAA    L0063			;Load RPM/12.5
	STAA    L0065			;Save it, RPM/12.5, one delayed
;
	LDAA    L01F8			;A/D O2
	STAA    L01F7			;A/D O2, one delayed
	RTS				;Return     
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; Engine governor PID subroutine
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
L8F97	LDX     #L400F			;Option byte
	BRCLR   0,X,#$01,L8FA6		;Bra if b0==0, no governor
;	
	BRCLR   L004F,#$80,L8FA6	;Status word, bra if b7==0, engine not running
;
;-Check for errors
;
	BRCLR   L006F,#$34,L8FAC	;Bra if b2, high TPS, b4, error 31, gov'r previously
					;set, and b5, low TPS error flags not set
;
;-Here to disable governor
;
L8FA6	LDAA    #255			;Load max TPS
;
L8FA8	STAA    L02BC			;Save desired TPS output
	RTS				;Return     
;
;~~~~~~~~~~~~~~~~~~~~~~~
; Here for RPM governor
;~~~~~~~~~~~~~~~~~~~~~~~
;
L8FAC	BCLR    L0075,#$04		;Status word, clear b2, RPM overspeed
;
	LDAA    L50CE			;Load upper RPM threshold for RPM overspeed
	SUBA    L02B1			;Subtract governor RPM/25
	BCC     L8FBB			;Bra if delta rpms >=0
;
;-Engine speed greater then upper threshold
;
	NEGA				;Get abs. value    
	BSET    L0075,#$04		;Status word, set b2, RPM overspeed
;
;-Engine speed less then upper threshold
;											
L8FBB	STAA    L02B6			;Save it, RPM error
	BRCLR   L0082,#$40,L8FC7	;Status word, bra if b6==0, no HU
;
	JSR     L1809			;Do HU
;
	BCS     L8FA8			;Bra if carry set, save and return
;
;-Check MPH limit
;
L8FC7	LDAA    L50D0			;Upper MPH threshold
	BEQ     L901B			;Bra if ==0, skip speed governing 
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Here to check for MPH overspeed
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
	LDD     L02A7			;MPH
	SUBA    L50D0			;Subtract upper MPH limit
	BCS     L8FFA			;Bra if MPH less
;
	ASLD				;MPH error x 2
	BCS     L8FDA			;Bra if overflow
;
	ASLD				;MPH error x 2
	BCC     L8FDD			;Bra if overflow
;
L8FDA	LDD     #$FFFF			;Clear overflow
;
L8FDD	STAA    L02B7			;Save it, MPH error											;
	BSET    L0075,#$40		;Status word, set b6, MPH overspeed
;
	BRSET   L0075,#$80,L9016	;Status word, bra if b7==1, in MPH govern 
;
;-Limit and update TPS if entering MPH govern
;
	LDAA    L50D7			;Upper limit for governor TPS
	CMPA    L01FC			;%TPS
	BCS     L8FF2			;Bra if TPS >, save limit instead
;
	LDAA    L01FC			;Load TPS
;
L8FF2	STAA    L02A9			;Save it, governor TPS
;											
	BSET    L0075,#$80		;Set b7, in MPH govern
	BRA     L9016			;Bra to continue
;
;-Here if MPH less then threshold
;
L8FFA	NEGA				;Get abs. value    
	NEGB				;    
	SBCA    #$00			;Perform carry if needed
	ASLD				;x2
	BCS     L9004			;Bra if overflow
;
	ASLD				;x2
	BCC     L9007			;Bra if overflow
;
L9004	LDD     #$FFFF			;Clear overflow
;
L9007	STAA    L02B7			;Save it, MPH error
	BCLR    L0075,#$40		;Clear b6, no MPH overspeed
	BRCLR   L0075,#$80,L901B	;Bra if b7==0, not in MPH govern  

	CMPA    L50D1			;Governor MPH error threshold to exit MPH govern
	BCC     L901B			;Bra if error >= 
;
;-Here to exit RPM governor if governing MPH
;
L9016	BCLR    L0075,#$13		;Status word, clear b4, post-derivative TPS return,
					;b1, derivative action anticipated, 
					;and b0, in RPM govern
	BRA     L901E			;Bra to continue
;
;-Here if not governing MPH
;
L901B	BCLR    L0075,#$80		;Clear b7, in MPH govern
;
L901E	BRCLR   L0075,#$04,L9039	;Bra if b2==0, no RPM overspeed 
;						
	LDAB    L0075			;Status word
	BITB    #$81			;Test b7, gov. MPH, and b0, gov. RPM
	BNE     L9033			;Bra if performing RPM or MPH governing
;
;-Update governor TPS
;
	LDAA    L01FC			;%TPS
	CMPA    L02A9			;Governor TPS
	BHI     L9033			;Bra if TPS >=, no update
;
	STAA    L02A9			;Save TPS as governor TPS
;
L9033	BSET    L0075,#$01		;Set b0, in RPM govern
	BCLR    L0075,#$12		;Clear b4, TPS post derivative return, and 
					;b1, derivative action anticipated
L9039	LDAB    L0075			;Status word
	BITB    #$81			;Test b7, MPH govern, and b0, RPM govern
	BNE     L9042			;Bra if !=0
;
	JMP     L90F3			;
;
L9042	LDX     #L50DE			;Proportional gain table vs. desired TPS output
	LDAA    L02BC			;Desired TPS output
	LSRA				;/2     
	JSR     LF136			;2D lookup with UL
;
	LDAB    L02B7			;Load MPH error
	LDX     #L50D5			;Start of governor params
	BRCLR   L0075,#$01,L905A	;Bra if b0==0, not in RPM govern
;
;-Load RPM error and terms instead if in RPM govern
;
	INX				;+1, in RPM govern     
	LDAB    L02B6			;Load RPM error
;
L905A	MUL				;Gain term x error     
	ASLD				;x2
	BCS     L9064			;Bra if overflow
;
	ASLD				;x2    
	BCS     L9064			;Bra if overflow
;
	ASLD				;x2, prescale to div. by 32    
	BCC     L9067			;Bra if no overflow
;
L9064	LDD	#$FFFF			;Clear overflow
;
L9067	STD     L02AD			;Save it, governor integral term 
	LDAA    0,X			;Load prop. multiplier term
	LDX     #L02AD			;Addr. of integral term
	JSR     LF1ED			;(A x X*)/256
;										
	STD     L02AD			;Save it, governor integral term
	LDAA    L50D3			;RPM prop. gain mult.
	LDAB    L02B6			;RPM error
	MUL				;RPM error x gain     
	ASLD				;x2    
	BCS     L9082			;Bra if no overflow
;
	ASLD				;x2   
	BCC     L9085			;Bra if no overflow
;
L9082	LDD     #$FFFF			;Clear overflow
;
L9085	STD     L02B3			;RPM prop. term
	CLR     L02B5			;Clear MPH prop. term
	BRCLR   L0075,#$80,L90AF 	;Bra if b7==0, not in MPH govern
;
;-Here if in MPH govern
;
	LDAA    L02B7			;Load MPH error
	LDAB    L50D4			;MPH proportioning gain term
	MUL				;MPH error x gain term
	ASLD				;x2
	BCS     L90A2			;Bra if overflow
;
	ASLD				;x2
	BCS     L90A2			;Bra if overflow
;
	ASLD				;x2
	BCS     L90A2			;Bra if overflow
;		
	ASLD				;x2
	BCC     L90A5			;Bra if no overflow

L90A2	LDD     #$FFFF			;Clear overflow
;
L90A5	STAA    L02B5			;Save it, MPH prop. term
	BRSET   L0075,#$04,L90AF	;Status word, bra if b2==1,  RPM overspeed 
;
	STD     L02B3			;RPM prop. term
;
L90AF	LDAB    L0075			;Status word
	BITB    #$44			;Test b2, RPM overspeed, b6, MPH overspeed
	BEQ     L90DB			;Bra if no overspeed
;
;~~~~~~~~~~~~~~~~~~~~~~~~
; RPM/MPH overspeed here
;~~~~~~~~~~~~~~~~~~~~~~~~
;
	LDD     L02A9			;Load governor TPS
	SUBD    L02AD			;Gov'r integral term
	BCS     L90C2			;Bra if underflow
;
	CMPA    L50D8			;Min desired TPS for overspeed
	BCC     L90C8			;Bra if governor TPS >=
;
L90C2	LDAA    L50D8			;Load min desired TPS
	JMP     L9176			;
;
L90C8	STD     L02A9			;Save it, governor TPS 
	SUBD    L02B3			;Subtract RPM/MPH prop. term
	BCS     L90D5			;Bra if underflow
;
	CMPA    L50D8			;Min desired TPS
	BCC     L90D8			;Bra if TPS >=
;
L90D5	LDAA    L50D8			;Load min desired TPS
;
L90D8	JMP     L917A			;Bra to exit
;
;~~~~~~~~~~~~~~~~~~~~~~
; Here if no overspeed
;~~~~~~~~~~~~~~~~~~~~~~
;
L90DB	LDD     L02A9			;Load governor TPS
	ADDD    L02AD			;Governor integral term
	BCC     L90E6			;Bra if no overflow
;
	LDD     #$FFFF			;Clear overflow
;
L90E6	STD     L02A9			;Save it, governor TPS
	ADDD    L02B3			;RPM/MPH governing prop. term
	BCC     L90F0			;Bra if overflow
;
	LDAA    #255			;Clear overflow
;
L90F0	JMP     L917A			;Bra to exit
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Check if derivative action is needed
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
L90F3	CLR     L02B5			;Clear MPH prop. term
	BRSET   L0075,#$02,L9120 	;Bra if b1==1, derivative action anticipated
;
	LDAA    L02B0			;Delta governor RPMs/25
	BMI     L9144			;Bra if RPMs falling
;
;-RPMs rising here
;
	CMPA    #32			;800 rpm, allowed upper limit for delta RPM
	BLS     L9105			;Bra if <=			
;
	LDAA    #32			;Load max rpms
;
L9105	LDX     #L50E4			;Derivative RPM thresholds vs delta RPMs table
	JSR     LF153			;2D lookup with line count
;
	CMPA    L02B1			;Governor RPM/25			
	BCC     L9144			;Bra if  gov'r RPM/25 <=
;
;-RPMs high enough for derivative action
;
	BSET    L0075,#$02		;Set b1, derivative action anticipated
	LDAA    L50D7			; 0% MAX DESIRED TPS% AT LEAD MODE START
	CMPA    L01FC			; TPS
	BLS     L916E			;Bra if TPS >=
;
	LDAA    L01FC			;Load TPS
;
	BRA     L916E			;Bra to continue
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Apply derivative action if needed
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
L9120	BRSET   L0075,#$10,L912A	;Bra if b4==1, post-derivative TPS return
;
	CLRA				;Preclear A    
	SUBA    L02B0			;Delta gov'r RPM/25
	BLT     L9148			;Bra if gov'r RPMs rising
;
;-RPMs falling here, allow throttle to return
;
L912A	BSET    L0075,#$10		;Set b4, post-derivative TPS return 
	LDAA    L02BC			;Desired governing TPS to be outputted
	LDAB    L50DC			;High desired TPS return gain
	CMPA    L50DB			;Threshold for high desired TPS 
	BCC     L913B			;Bra if >=
;
	LDAB    L50DD			;Low desired TPS return gain
;
L913B	LDAA    L02B6			;RPM error
	MUL				;RPM error x gain factor     
	ADDD    L02A9			;Add in governor TPS
	BCC     L9177			;Bra if no overflow
;
L9144	LDAA    #255			;Load max TPS
	BRA     L9176			;
;
;-RPMs rising here, apply derivative action
;
L9148	LDX     #L50F0			;TPS derivative gain factor vs. cool temp
	LDAA    L0006			;Cool temp, linear
	CMPA    #128			;56 deg C, upper limit for lookup
	BLS     L9153			;Bra if cool temp <=
;
	LDAA    #128			;Load max, 56 deg C
;
L9153	LSRA				;cool temp/2 for lookup
	JSR     LF15E			;2D lookup
;
	LDAB    L02B0			;Load delta gov'r RPMs/25
	MUL				;delta RPMs x gain    
	ASLD				;x2    
	BCS     L9161			;Bra if overflow
;
	ASLD				;x2    
	BCC     L9164			;Bra if no overflow 
;
L9161	LDD     #$FFFF			;Clear overflow
;
L9164	TAB				;Get result into B     
	LDAA    L02A9			;Governor TPS
	SBA				;Subtract derivative term      
	LDAB    L02AA			;Load LSB of gov'r tps
	BCS     L9173			;Bra if underflow
;
L916E	CMPA    L50D9			;Min desired TPS for derivative action
	BCC     L9176			;Bra if TPS >=
;
L9173	LDAA    L50D9			;Load  min TPS
;
L9176	CLRB				;Clear LSB
;
L9177	STD     L02A9			;Save it, governor TPS
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Here to save desired outputted TPS and return
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
L917A	LDAB	L0075			;Status word
	BITB    #$84			;Test b7, in MPH govern, and b2, RPM overspeed 
	BEQ     L919A			;Bra if ==0
;
	BRSET   L0075,#$40,L918D	;Bra if b6==1, MPH overspeed
;
;-No MPH overspeed, add in MPH prop. term
;
	ADDA    L02B5			;Add in MPH prop. term
	BCC     L919A			;Bra if no overflow
;
	LDAA    #255			;Clear overflow
	BRA     L919A			;
;
;-Here if MPH overspeed, subtract MPH prop. term
;
L918D	SUBA    L02B5			;Subtract MPH prop. term
	BCS     L9197			;Bra if undrflow
;
	CMPA    L50D8			;Min desired TPS
	BCC     L919A			;Bra if outputted TPS is >=

L9197	LDAA    L50D8			;Load min desired TPS
;
L919A	STAA    L02BC			;Save it, desired outputted TPS
	BITB    #$11			;Test b4, RPM TPS lead return, and b0, 
;					;RPM govern in effect
	BEQ     L91B5			;Bra if ==0
;
	LDAB    L02AB			;Actuator integral term
	BNE     L91B5			;
;
	BRCLR   L0075,#$01,L91B2	;Bra if b0==1, not in RPM govern
;
	LDAA    L02B1			;Governor RPM/25
	CMPA    L50CF			;Min governor RPM
	BCC     L91B5			;Bra if RPMs >=
;
L91B2	BCLR    L0075,#$13		;Clear b4, RPM TPS lead, b1, 
;
L91B5	RTS				;Return  
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~  
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; Governor actuator PID control routine
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
L91B6	LDAB    L02BC			;Desired TPS to be outputted
	SUBB    L01FC			;Subtract current TPS
	BCC     L91BF			;Bra if TPS error >=0
;
;-TPS error < 0
;
	NEGB				;Get abs val
;    
L91BF	STAB    L02BD			;Save it, TPS error
;
;-Compute actuator prop. gain
;
	LDX     #L50F5			;Actuator %DC prop. gain vs. %TPS error table
	LDAA    L02BD			;TPS error
	JSR     LF153			;2D lookup with line count
;
	LDAB    L02BD			;Load TPS error
	MUL				;TPS error x prop. gain      
	ASLD				;x2
	BCS     L91D5			;Bra if overflow
;
	ASLD				;x2
	BCC     L91D8			;Bra if no overflow
;
L91D5	LDD     #$FFFF			;Clear overflow
;
L91D8	STD     L02B8			;Save it, actuator prop. term
;
;-Compute actuator integral gain
;
	LDX     #L50FF			;Actuator integral gain vs. %TPS error table
	LDAA    L02BD			;TPS error
	JSR     LF153			;2D lookup with line count
;
	LDAB    L02BD			;TPS error
	MUL				;TPS error x integral gain     
	STD     L02BA			;Save it, integral factor
;
	XGDX				;Swap result into X    
	LDAA    L02BC			;Desired TPS to be outputted
	CMPA    L01FC			;Compare to current TPS
	XGDX				;Swap result back in    
	BCC     L920E			;Bra if TPS <=
;
;-Here to add in the integral and prop. term
;
	ADDD    L02AB			;Add in integral factor to actuator integral term
	BCC     L91FD			;Bra if no overflow

	LDD     #$FF00			;Clear overflow
;
L91FD	STD     L02AB			;Save it, actuator integral term
;
	ADDD    L02B8			;Add in act. prop. term
	BCS     L920A			;Bra if overflow

L9205	ADDD    #0128			;Round up if needed
	BCC     L9221			;Bra if no overflow
;
L920A	LDAA    #255			;Clear overflow
	BRA     L9221			;
;
;-Here to subract the integral and prop. term
;
L920E	LDD     L02AB			;Load actuator integral term
	SUBD    L02BA			;Subtract integral factor			
	BCC     L9218			;Bra if no underflow
;
	CLRA				;Clear underflow    
	CLRB				;    	
;
L9218	STD     L02AB			;Save it, integral term
	SUBD    L02B8			;Subtract prop. term
	BCC     L9205			;Bra if no underflow

	CLRA				;Clear underflow
;    
L9221	LDAB    L02BC			;Desired governing TPS to be outputted
	CMPB    #$FF			;255 = No governing
	BNE     L922E			;Bra if != 255			
;
	LDX     #$0000			;
	STX     L02AB			;Clear actuator integral term
;
L922E	TAB				;Governing TPS to B     
	BRCLR   L0051,#$10,L9236	;Status word, bra if b4==0, no high batt. volts 
;
;-High batt volts, clear desired TPS
;
	CLRB				;    
	BRA     L9244			;
;
L9236	BRCLR   L003B,#$10,L9244	;Status word, bra if b4==0, not in mode 4
;
	LDAA    L0395			;
	BITA    #$01			;Test b0
	BEQ     L9244			;Bra if ==0
;
	LDAB    L0396			;
;
L9244	LDAA    #$1F			;
	STD     L306C			;Hardware
	RTS				;Return     
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;~~~~~~~~~~~~~~~~~~~~~~~~
;
; I/O control routine
;
;~~~~~~~~~~~~~~~~~~~~~~~~
;
L924A	LDAB    #$AA			;1010 1010
	STAB    L303A			;COP
;
;-Check opt. byte to see if A/C is present
;
	LDAA    L400F			;Opt. byte
	BITA    #$04			;Test b2, using adaptive A/C control
	BNE     L9268			;Bra if ==1, skip A/C routine
;
;-Using standard on/off control, check A/C status
;
	BRSET   L004D,#$01,L9262	;Bra if b0==1, A/C requested 
;
;-No A/C requested here
;
	BCLR    L0052,#$20		;Clear b5, A/C not requested
	BCLR    L0041,#$10		;Clear b4, A/C off
	BRA     L9268			;Bra to continue
;
;-A/C requested
;
L9262	BSET    L0052,#$20		;Set b5, A/C requested
	BSET    L0041,#$10		;Set b4, A/C on
;
;-Check to see if in P/N
;
L9268	BCLR    L0041,#$20		;Preclear b5, park/neutral 
;
	LDX     #L400F			;Opt. byte
	BRSET   0,X,#$40,L927C		;Bra if b6==1, non-electronic trans present
;
;-Electronic trans in use here
;
	LDAA    L004D			;I/O port C word		
	ANDA    #$1C			;Clear all but b2-b4
	CMPA    #$14			;0001 0100
	BNE     L9285			;Bra if b2,b4 !=0, not in P/N 
;
	BRA     L9282			;Bra, in park/neutral
;
;-Non elec. trans in use here
;
L927C	LDAA    L004D			;I/O port C word
	BITA    #$10			;Test b4, P/N switch
	BNE     L9285			;Bra if !=0
;
;-Here if in park/neutral
;
L9282	BSET    L0041,#$20		;Set b5, in P/N
;
L9285	RTS				;Return
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~     
;
;~~~~~~~~~~~~~~~~~~~~~~
;
; IAC control routine
;
;~~~~~~~~~~~~~~~~~~~~~~
;
L9286	LDAB    L000A			;IAC control word
;
	LDAA    L00A1			;A/D battery voltage
	CMPA    #169			;16.9 V, max batt volts for IAC operation
	BLS     L9292			;Bra if batt volts <=
;
	ANDB    #$EF			;Clear b4, disable IAC
	BRA     L929B			;Bra				
;
L9292	BRSET   L0044,#$10,L92A0	;Bra if b4==1, ignition off Skip low volts check

	CMPA    L4EAC			;Min batt volts for IAC operation
	BHI     L92A0			;Bra to continue

L929B	BSET    L003E,#$04		;Status word, set b2, batt volts out of range for IAC operation
	BRA     L92E3			;Bra to skip IAC adj.

L92A0	BCLR    L003E,#$04		;Status word, clr b2, batt volts within range for IAC operation
;
	ORAB    #$10			;Set b4, enable IAC
	LDAA    L0007			;Current IAC steps
	CMPA    L0008			;Desired IAC steps
	BNE     L92AF			;Bra if !=, continue
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Here if delta IAC counts ==0
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
	ANDB    #$FE			;Clr b1 of IAC control word, set up to retract IAC next pass
	BRA     L92E3			;Bra to save and skip IAC adj.
;  
L92AF	BRCLR   L0009,#$01,L92B7	;Bra if resetting IAC, force IAC to step if needed 
;
	BRSET   L0002,#$03,L92E3	;Loop counter, bra if b1,b2 set, skip IAC adj.
;
L92B7	BCC     L92C2			;Bra if current IAC counts >= IAC counts, extend IAC
;
;~~~~~~~~~~~~~~~~~~~~~
; Here to retract IAC
;~~~~~~~~~~~~~~~~~~~~~
;
	ANDB    #$FE			;Clr b1, retract IAC
	BRSET   L000A,#$01,L92E3	;Status word, bra if b==1, previously exteding IAC, skip routine this time around
;
	INCA 				;+1 to current IAC steps   
	BRA     L92C9			;Bra to save it
;
;~~~~~~~~~~~~~~~~~~~~
; Here to extend IAC
;~~~~~~~~~~~~~~~~~~~~
;
L92C2	ORAB    #$01			;Set b1, extend IAC
	BRCLR   L000A,#$01,L92E3	;Bra if b0==0, previously where retracting IAC, skip routine this time around
;
	DECA				;-1 from current IAC counts 
;
;-Save IAC control word
;   
L92C9	STAA    L0007			;Current IAC counts 
;
;~~~~~~~~~~~~~~~~
;-Test IAC state
;~~~~~~~~~~~~~~~~
;
; b3, 1=Coil B state on
; b2, 1=Coil A state on
;
					;IAC control word
	BRSET   L000A,#$0C,L92D9	;Bra if b2,b3==1, coil A,B on
;
	BRCLR   L000A,#$0C,L92D9	;Bra if b2,b3==0, coil A,B off
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Here if neither coil simultaniously on/off
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
	BITB    #$01			;Test b1, extend IAC
	BNE     L92DD			;Bra if ==1, extend IAC
;											
	BRA     L92E1			;Bra if retracting IAC
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Here if both coils simultaniously on/off
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
L92D9	BITB    #$01			;Test b1, extend IAC
	BNE     L92E1			;Bra if extending IAC
;
;-Here to toggle coil A
;
L92DD	EORB    #$04			;Toggle coil A state	
;
	BRA     L92E3			;Bra to save
;
;-Here to toggle coil B 
;
L92E1	EORB    #$08			;Toggle coil B state
;
;-Here to skip IAC adj./Output to hardware
;
L92E3	STAB    L000A			;Save status word
	SEI 				;Momentarily disable interrupts
	LDAA    L004C			;I/O port D
	ANDA    #$E3			;Mask, 1110 0011,
	ANDB    #$1C			;Mask, 0001 1100
	ABA 				;Update port D val.    
	STAA    L004C			;					
	CLI				;Re-enable interrupts
	RTS 				;Return  
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; Routine to disable throttle kicker
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
L92F1	BCLR    L0037,#$18		;Status word, clear b3, throttle kicker disable req., 
					;b4, throttle kicker disable request
	BRCLR	L0037,#$04,L92FE	;Status word, bra if b2==0, throttle kicker 
					;not previously disabled
;
;-Throttle kicker previously disabled
;
	CLR     L088D			;Clear throttle kicker time
	BCLR    L0037,#$04		;Clear b2, throttle kicker no longer active 
;
L92FE	BRSET   L0036,#$40,L9305	;Bra if b6==1, T/K has been previously active					;already
;
	BSET    L0036,#$40		;Set b6, throttle kicker was just active
;
L9305	BSET    L004B,#$80		;I/O word, set b7, de-energize the throttle kicker
	RTS				;Return    
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;Routine to enable the throttle kicker
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
L9309	BRSET   L0037,#$04,L931F	;Status word, bra if b2==1, throttle kicker 
					;already enabled
;
;-Load quals for throttle kicker being active
;
	LDAA    L02D4			;Stored base closed throttle tps
	ADDA    L50CB			;Add in active T/K closed throttle offset 
	STAA    L0319			;Save it, base closed throttle TPS
	LDAA    L50C9			;Min time to disable throttle kicker
	STAA    L088D			;Throttle kicker time
	BSET    L0037,#$04		;Status word, set b2, throttle kicker
					;enabled
;
L931F	BCLR    L004B,#$80		;I/O word, clear b7, energize throttle kicker
	RTS     			;Return		
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~    
;
;~~~~~~~~~~~~~~~~~~~~~~~~
;
; Routine to enable IAC
;
;~~~~~~~~~~~~~~~~~~~~~~~~
;											
L9323	LDX	#L004C			;I/O port D control word
	BSET    0,X,#$10		;Set b4, enable IAC
	RTS				;Return     
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; Routine to check if 4WD low is engaged
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
L932A	BRSET   L004D,#$80,L9331	;I/O word, bra if b7==1, 4WD low not engaged
;
	SEC				;Set carry, 4WD low engaged   
	BRA     L9332			;Bra to return
;
L9331	CLC				;Clear carry, 4WD low not engaged 
;    
L9332	RTS				;Return
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~     
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; Routine to check status of A/C high pressure switch
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
L9333	PSHA				;Push desired idle to stack
	LDAA    L3008			;Port D
	BITA    #$04			;Test b2, A/C high pressure switch status
	PULA				;Get desired idle back 
	BEQ     L933F			;Bra if switch is open
;
;-Switch closed
;
	SEC				;Set carry, high pressure indicated
	BRA     L9340			;Return
;
;-Switch open
;			
L933F	CLC				;Clear carry high pressure not indicated
;
L9340	RTS				;Return 
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~    					
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; Power-on idle initialization routine
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
L9341	LDAA    L4EE8			;1.56 %TPS, upper limit for closed throttle	
	ASLA				;x2
	ASLA				;x2
	STAA    L0885			;Save it, upper TPS limit for closed throttle
;
	BRCLR   L0046,#$40,L9355	;Bra if b6=0, non vol. mem. intact
;
;-Non-vol. memory lost, perform resets
;
	LDAA    L4EA6			;IAC park position
	STAA    L0007			;Save it, IAC steps
	JSR     L9387			;Routine to reset idle integral cells
;
;-Non-vol. memory intact
;
L9355	BRSET   L0004,#$08,L935E	;Bra if b3==1, bad shutdown 
;
	JSR     L93D4			;Routine to init. idle integral cells
	BRA     L9361			;Bra to continue
;				
;-Here if bad shutdown
;						
L935E	JSR     L94A8			;Routine to reset IAC status word
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Check if throttle kicker is present
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
L9361	LDAA    L4E85			;IAC option word #1
	BITA    #$04			;Test b2, throttle kicker option bit
	BEQ     L937F			;Bra if ==0, no throttle kicker
;
;-Check Baro quals for throttle kicker
;											
	LDAA    L0317			;Baro 
	CMPA    L50C4			;Baro for enabling the throttle kicker
	BLS     L9377			;Bra if baro <=, enable throttle kicker
;
;-Check cool quals for throttle kicker
;											
	LDAA    L0006			;Cool temp, linear
	CMPA    L50BD			;Upper thresh. for throttle kicker
	BCC     L937C			;Bra if cool temp is greater
;
;-Here to disable throttle kicker
; 
L9377	BSET    L0037,#$04		;Set b2, throttle kicker enabled
	BRA     L937F			;Bra to save and return															
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Here to disable the throttle kicker
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;							
L937C	JSR     L92F1			;Routine to disable the throttle kicker
;	
;-Here if no throttle kicker
;										
L937F	LDAA    L0007			;Current IAC steps
	STAA    L0008			;Desired IAC steps
	JSR     L9323			;Routine to enable IAC
;											
	RTS				;Return
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;			
; Routine to reset idle integral cells
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;	
;-Reset integral cells for drive
;					
L9387	LDAA    L0006			;Cool temp, linear
	CMPA    L4EAB			;Temp threshold for hot engine		
	TPA				;Cond. Code Reg. to A   						;
  										
	LDAB    L4EA8			;Default flow value for idle integrator term            						
	STAB    L02CA			;Save it, current idle integrator, drive, no A/C
	STAB    L02CC			;Steady state idle integraotr, drive, no A/C

	TAP				;Load CCR from A
	BCS     L93A0			;Bra if stored cool temp < 110 deg C
;
;-Cool temp > 110 deg C
;
	ADDB    L4EA9			;Added flow for A/C on idle integral cells,
					;high engine temp
	BCS     L93A5			;Bra if overflow	
;
	BRA     L93A7			;Bra to store
;
;-Cool temp <= 110 deg C
;
L93A0	ADDB    L4EAA			;Added flow for A/C on idle integral cells,
					;normal engine temp
	BCC     L93A7			;Bra if no overflow

L93A5	LDAB    #$FF			;Clear overflow
;
;-Here to save flows
;
L93A7   STAB    L02CD			;Current idle integral term, drive, A/C on 
	STAB    L02CF			;Steady state idle integral term, drive, A/C on
;
;-Reset P/N integral cells
;
	LDAB    L4EA7			;Default flow for P/N idle integral cells
	STAB    L02C0			;Current idle integral term, P/N, A/C off	
	STAB    L02C2			;Steady state idle integral term, P/N, A/C off
	TAP				;Load CCR from A     
	BCS     L93C0			;Bra if cool temp < 110 deg C
;
;-Cool temp >= 110 deg C
;

	ADDB    L4EA9			;Added flow for A/C on idle integral cells, 
					;high engine temp
	BCS     L93C5			;Bra if overflow
;
	BRA     L93C7			;Bra to store
;
;-Cool temp <= 110 deg C
;
L93C0	ADDB    L4EAA  			;Added flow for A/C on idle integral cells,
					;normal engine temp
	BCC     L93C7			;Bra if no overflow
;	
L93C5	LDAB    #$FF			;Clear overflow
;
;-Here to save the flow
;
L93C7	STAB    L02C3			;Current idle integral term, P/N, A/C on
	STAB    L02C5			;Steady state idle integral term, P/N, A/C off
;	
;-Reset base throttle kicker TPS
;
	LDAA    L50CA			;Default T/K base TPS
	STAA    L02D4			;T/K base TPS
	RTS     			;Return
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; Routine to initialize idle integral cells
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Check if stable idle was attained with A/C off
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
L93D4	BRCLR   L0009,#$10,L93F2	;Status word, bra if b4==0, stable idle not 
					;previously attained on last run
;
;-Stable idle was attained on last run
;
	LDAA    L02CC			;Steady state idle integral term, drive, A/C off
	CLRB				;Preclear B
	ADDA    L508E			;Added flow when ignition is off
	BCC     L93E4			;Bra if no overflow

	LDAA    L50B9			;Load max allowed flow for integral, A/C off
;
L93E4	STAA    L02CC			;Save it, steady state integral term, drive, A/C off
	CMPA    L02CA			;Compare, current idle integral term, drive, no A/C
	BLS     L93EF			;Bra if current integral term wasl <=
;
;-Current integral term was <, clamp integral term to steady state integral term
;
	STD     L02CA			;Save steady state integral as current integral term
;
;-Current integral term was >=, dont clamp
;
L93EF	BCLR    L0009,#$10		;Clear b4, steady idle flag
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Check if stable idle was attained with A/C on
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;				
L93F2	BRCLR   L0009,#$20,L9410	;Bra if b5==0, stable idle not attained on last run 
;
;-Stable idle was attained on last run
;
	CLRB				;Preclear B
	LDAA    L02CF			;Steady state integral term, drive, A/C on
	ADDA    L508F			;Added flow when ignition is off
	BCC     L9402			;Bra if no overflow
;
	LDAA    L50BA			;Load max allowed flow for integral, A/C on
;
L9402	STAA    L02CF			;Save it, steady state integral, drive, A/C on
	CMPA    L02CD			;Compare, current integral term, drive, A/C on
	BLS     L940D			;Bra if current integral was <=
;
;-Current integral term was <, clamp integral term to steady state integral term
;
	STD     L02CD			;Save steady state integral as current integral
;
;-Current integral term was >=, dont clamp
;
L940D	BCLR    L0009,#$20		;Clear b5, steady idle flag, A/C on
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Reset current non-A/C integral cells using SS cells if optioned 
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
L9410	LDAB    L4E86			;Opt. byte
	BITB    #$01			;Test b0, reset current integral cells
					;to steady state values
	BEQ     L9423			;Bra if ==0, no reset
;
;-Reset current integral to steady state values
;
	LDAA    L02C2			;Steady state integral term, P/N, no A/C
	STAA    L02C0			;Save as current integral term, P/N, no A/C										;
	LDAA    L02CC			;Steady state integral term, drive, no A/C
	STAA    L02CA			;Save as current integral term, drive, no A/C
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Use non-A/C cells to reset A/C on cells if optioned
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;	
L9423	LDAA    L4E85			;Option byte
	BITA    #$08			;Test b3, use A/C off integral cells to reset A/C
					;on integral cells
	BEQ     L9494			;Bra if ==0, not using this method to reset cells
;
;-Reset park/neutral A/C on cells
;
	LDAA    L0006			;Cool temp, linearized
	CMPA    L4EAB			;Temp threshold for hot engine
	TPA				;CCR to A
	LDAB    L02C0			;Idle integral term, P/N, no A/C
	TAP				;Restore CCR     
	BCS     L943D			;Bra if engine temp was <, cool engine
;
;-High engine temps present
;
	ADDB    L4EA9			;Added flow for A/C idle cell init, high engine 
					;temps
	BCS     L9442			;Bra if overflow
;
	BRA     L9447			;No overflow, bra to save
;
;-Normal engine temps preset
;
L943D	ADDB    L4EAA			;Added flow for A/C idle cell init, normal 
					;engine temps
	BCC     L9447			;Bra if no overflow
; 
L9442	LDAB    #255			;Load max for idle cell
	STAB    L02C4			;LSB. idle integral term, P/N, A/C on
;
;-Reset steady state A/C idle integral cell
;
L9447	STAB    L02C3			;MSB, idle integral term, P/N, A/C on
;
	LDAB    L02C2			;Steady state idle integral term, P/N, no A/C			
	TAP				;Restore CCR to previous state     
	BCS     L9457			;Bra if engine temp < threshold for hot engine
;
;-High engine temps present
;
	ADDB    L4EA9			;Added flow for A/C idle cell init, high engine 
					;temps
;
	BCS     L945C			;Bra if overflow
;
	BRA     L945E			;Bra to save
;
;-Normal engine temps preset
;
L9457	ADDB    L4EAA			;Added flow for A/C idle cell init, normal 
					;engine temps
;
	BCC     L945E			;Bra to save			
;
L945C	LDAB    #$00FF			;Clear overflow
;
L945E	STAB    L02C5			;Save it, steady state integral cell, P/N, A/C on
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Reset A/C on integral cells for drive
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
	LDAB    L02CA			;Current idle integral term, no A/C, drive 
	TAP				;Restore CCR to previous state     
	BCS     L946E			;Bra if engine temp was < temp threshold
;
;-High engine temps present
;
	ADDB    L4EA9			;Added flow for A/C idle cell init, high engine 
					;temps
;
	BCS     L9473			;Bra if overflow
;
	BRA     L9478			;Bra to save
;
;-Normal engine temps preset
;
L946E	ADDB    L4EAA			;Added flow for A/C idle cell init, normal 
					;engine temps
;
	BCC     L9478			;Bra if no overflow
;
L9473	LDAB    #$00FF			;Load max for idle cell
	STAB    L02CE			;LSB, current idle integral term, A/C, drive
;
L9478	STAB    L02CD			;MSB, current idle integral term, A/C, drive
;
;-Reset steady state A/C on integral cell
;
	LDAB    L02CC			;Steady state integral term, A/C on, drive			
	TAP				;Restore CCR to previous state    
	BCS     L9488			;Bra if engine temp was < temp threshold
;
	ADDB    L4EA9			;Added flow for A/C idle cell init, high engine 
					;temps
;
	BCS     L948D			;Bra if overflow
;
	BRA     L948F			;Bra to return

L9488	ADDB    L4EAA			;Added flow for A/C idle cell init, normal 
					;engine temps
	BCC     L948F			;Bra if no overflow

L948D	LDAB    #$FF			;Clear overflow
;
L948F	STAB    L02CF			;Save it, steady state idle integral term, drive
					;A/C on
	BRA     L94A7			;Bra to return
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Here if not using A/C off integral cells to reset A/C on integral cells
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;-Use steady state cells to reset current A/C on integral cells if optioned
;
L9494	LDAB    L4E86			;Opt byte
	BITB    #$01			;Test b0, reset current integral cells to
					;steady state values
	BEQ     L94A7			;Bra if ==0, dont reset using this method
;
	LDAA    L02C5			;Load steady state integral term, A/C on, P/N
	STAA    L02C3			;Save it, current integral term, A/C on, P/N
;
	LDAA    L02CF			;Load steady state integral term, A/C on, drive
	STAA    L02CD			;Save it, current integral term, A/C on, drive
;
L94A7	RTS				;Return     
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; Routine to reset IAC status word
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;  
; b3==1, coil B on
; b2==1, coil A on
;
L94A8	BRSET   L0009,#$01,L94B2	;Bra if b0==1, IAC reset in progress
;
	LDAA    L000A			;
	ANDA    #$0C			;0000 1100, clear all but b2, b3
	STAA    L000A			;
;
L94B2	RTS				;Return 
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~    
;
;~~~~~~~~~~~~~~~~~~~
;
; Main idle routine
;
;~~~~~~~~~~~~~~~~~~~
;
L94B3	BRCLR   L0082,#$40,L94BA	;Status word, bra if b6==0, no heads up
;
	JSR     L1803			;HU vector table
;						
L94BA	BCLR    L0038,#$40		;
	BRCLR   L003B,#$10,L94C4	;Status word, bra if b4==0, no mode 4 (ALDL)
;
	JSR     LA4DD			;ALDL IAC routine
;    
L94C4	BRCLR   L0009,#$01,L94DC	;Status word, bra if b0==0, IAC reset not in 
					;progress
;
	CLR     L0008			;Clear desired IAC steps
	TST     L0007			;Test current IAC steps
	BEQ     L94D3			;Bra if ==0, IAC closed
;											
	JMP     L997E			;Jump to return
;  
L94D3	BCLR    L0009,#$01		;Clear b0, IAC reset in progress
	BSET    L0009,#$04		;Set b2, IAC reset not requested
	JSR     L9387			;Routine to reset idle integral cells
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Here if not resetting IAC
;~~~~~~~~~~~~~~~~~~~~~~~~~~~
; 
L94DC	BRCLR   L0044,#$10,L94ED	;Status word, bra if b4==0, ignition is on
;
;~~~~~~~~~~~~~~~~~~~~~~
; Ignition is off here
;~~~~~~~~~~~~~~~~~~~~~~
;
	BRCLR   L0009,#$04,L94EA	;Status word, bra if b2==0, IAC reset requested
;
	LDAA    L4EA6			;IAC park steps
	JMP     L997C			;Save and return
;
;-IAC reset was requested
;
L94EA	JMP     LA4D0			;Go close and seat IAC
;
;~~~~~~~~~~~~~~~~~~~~~
; Ignition is on here
;~~~~~~~~~~~~~~~~~~~~~
;
L94ED	BRCLR   L004F,#$80,L94F4	;Bra if b0==0, engine not running
;
;-Engine running
;
	JMP     L961A			;
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Engine is off here, look up various parameters for startup
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
L94F4	LDAA    L02A6			;Range limited cool temp
	LDX     #L4EEB			;Closed loop delay for high RPMs vs cool temp
	JSR     LF15E			;2D lookup
;
	STAA    L0866			;Save it, IAC closed loop delay (high rpms)
;
	LDAA    L02A6			;Range limited cool temp 
	LDX     #L4EF8			;Closed loop delay for low RPMs vs cool temp
	JSR     LF15E			;2D lookup
;
	STAA    L0867			;IAC closed loop delay (high rpms)
;
;-Load initial IAC flow
;
	LDAA    L4EAE			;Initial added IAC flow for startup
	STAA    L086B			;Initial added IAC flow
;
	LDAA    L4EAF  			;Decay period for initial added IAC flow
	STAA    L086C			;Save it, initial IAC flow decay period
;
;-Scale coolant temp for lookup
;
	LDAA    L02A6			;Range limited cool temp
	CMPA    #160			;92 deg C
	BLS     L9524			;Bra if cool temp <=
;
	SUBA    #160			;-160 for scaling 
	ASLA				;x2
	ADDA    #160			;+160
;
; Cool temp = .75 x val - 28 when <= 92 deg C
;
; Cool temp = .375 x val + 32 when > 92 deg C
;
L9524	PSHA				;Push term to stack
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Look up desired idle speed based on cool temp
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
	LDX     #L4F28			;Desired idle speed vs. cool temp
	LDAB    L4E85			;IAC option byte
	BITB    #$01			;Test b0
	BEQ     L9532			;Bra if b0==0, auto in use
;
;-Here if manual in use, use A/C on table instead
;
	LDX     #L4F46			;Desired idle speed vs. cool temp
;
L9532	JSR     LF15E			;2D lookup
	PULB				;Get cool temp back    
	PSHA				;Push desired idle speed to stack    
	TBA				;Get cool temp into B
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Look up added idle speed vs. cool temp
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
	LDX     #L4F64			;Added initial desired idle speed vs. cool temp
	JSR     LF15E			;2D lookup
;
	STAA    L085C			;Save it, added desired idle speed
;
;-Check to see if manual in use
;                     
	LDAB    L4E85			;IAC option byte
	BITB    #$01			;Test b0
	BNE     L9551			;Bra if ==1, manual trans in use
;
;-Auto is in use here, apply multiplier for added idle speed
;
	LDAB    L4EB2			;Load multiplier 
	MUL				;Added initial desired idle speed x multiplier     
	ASLD				;x2, now /128    
	BCC     L9551			;Bra if no overflow
;
	LDAA    #$FF			;Clear overflow
;
;-Manual in use here
;
L9551	PULB				;Get desired idle speed off stack				    
	ABA				;Add in added initial desired idle sped     
	STAA    L0857			;Save it, desired idle speed
;
;~~~~~~~~~~~~~~~~
; Perform resets
;~~~~~~~~~~~~~~~~
;
	LDAA    L4EB1			;Decay period for added initial idle speed
	STAA    L085D			;Added initial idle speed decay period
;
	LDAA    L4EB0			;Time untill init. idle speed is decayed 
	STAA    L085E			;Timer for decaying out init. added idle speed
;
;-Perform clears
;
	CLRA				;Preclear A    
	STAA    L086D			;Proportional idle term
	STAA    L0879			;Added IAC flow for trans. shift
	STAA    L087A			;Transmission shift transition period
	STAA    L0876			;Added flow for A/C transition 
	STAA    L0877			;Transition period for A/C on to off 
	STAA    L0887			;IAC flow when in DFCO
	STAA    L087B			;P/S compensation flow
	STAA    L0864			;IAC closed loop enable delay timer
	LDAA    #$80			;Load neutral value
	STAA    L086E			;Stored idle derivative term
;
	BCLR    L0036,#$BD		;Status word, clear the following:
					; b7=High idle speed flag
					; b5=
					; b4=Stall saver in effect
					; b3=Closed loop idle in effect
					; b2=Closed loop idle quals met
					; b0, 0=A/C off, 1=A/C on
; 
	BCLR    L0037,#$9B		;Status word, clear the following:
					; b7, 1=positive derivative idle term
					; b4=Throttle kicker disable req. flag
					; b3=Throttle kicker disable req. flag
					; b1=reduce thtottle follower action
					; b0=warm engine flag, throttle follower
;

	BCLR    L0038,#$0F		;Status word, clear the following:
					; b3=Update idle int. flag, drive, A/C on
					; b2=Update idle int. flag, drive, no A/C
					; b1=Update idle int. flag, P/N, A/C on
					; b0=Update idle int. flag, P/N, no A/C
;	
	BCLR    L0038,#$70		;Status word, clear the following
					; b6=
					; b5=Sign of added DFCO IAC airflow
					; b4=Initial stored IAC flow updated
;
	BCLR    L0039,#$C7		;Status word, clear the following
					; b7=Throttle open flag
					; b6=Manditory post-DFCO T/F added flow update
					; b2=Throttle kicker diags. performed
					; b1=Post throttle kicker base TPS updated
					; b0=Throttle kicker base TPS updated
;
	LDAA    L4FB4			;Upper RPM limit for updating SS idle cells
	STAA    L0859			;Filtered RPM error
	BCLR    L0009,#$02		;Status word, clear b1, First run flag for
					;IAC flow kickdown logic 
;
;~~~~~~~~~~~~~~~~~~~~~~~~
; Look up IAC %flowrates
;~~~~~~~~~~~~~~~~~~~~~~~~
;
	LDAA    L02A6			;Range limited cool temp
	LDX     #L4EC5			;IAC flow vs. cool temp table (drive)
	JSR     LF15E			;2D lookup
;
	CLRB				;Clear LSB    
	STD     L02D0			;Coupled IAC flow vs. cool temp (drive)
	STD     L02D2			;IAC flow vs. cool temp (drive)
;
	LDAA    L02A6			;Range limited cool temp
	LDX     #L4EB8			;IAC flow vs. cool temp table (park/neutral)
	JSR     LF15E			;2D lookup
;
	CLRB				;Clear LSB  
	STD     L02C6			;Coupled IAC flow vs. cool temp, (P/N)
	STD     L02C8			;IAC flow vs. cool temp, (P/N)
;
	LDAA    L02A6			;Range limited cool temp
	LDX     #L4ED2			;IAC cold offset delay period vs. cool temp table
	JSR     LF15E			;2D lookup
;
	STAA    L0875			;IAC flow decay delay timer
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Init idle integrator flow using either default values or idle integral terms
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
	LDAA    L4E85			;IAC opt. byte
	BITA    #$20			;Test b5, init INT with calib const., not running 
	BEQ     L95DA			;Bra if ==0, 
;
	CLRB				;Preclear B    
	BITA    #$01			;Test b0, manual/auto trans flag
	BNE     L95D5			;Bra if ==1, manual trans in use
;
;-Auto in use here
;
	LDAA    L4EA7			;Default flow for P/N idle cells
	BRA     L95EB			;Bra to save
;
;-Manual in use here
;
L95D5	LDAA    L4EA8			;Default flow for drive idle cells
	BRA     L95E6			;Bra to save and continue
;
;-Here if init. using idle integral terms
;
L95DA	BITA    #$01			;Test b0, manual/auto trans flag
	BNE     L95E3			;Bra if ==1, manual in use
;
;-Auto in use
;
	LDD     L02C0			;Load current idle integral term, P/N
	BRA     L95EB			;Bra to save and continue
;
;-Manual in use
;
L95E3	LDD     L02CA			;Load current idle integral term, drive
;
L95E6	LDX     L02D0			;IAC flow vs. cool temp, drive
	BRA     L95EE			;Bra to save
;
;-Continue here if auto in use
;
L95EB	LDX     L02C6			;Load coupled IAC flow vs. cool temp, P/N
;
L95EE	STD     L0862			;Save idle term as idle integrator idle flow
	STX     L0871			;Save IAC flow vs. cool temp as added IAC flow
					;vs. cool temp
;
	LDX     #$FFFF			;Max flow
	STX     L087D			;stored IAC flow, used in AE routine
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Move IAC prior to start if cool temp is great enough
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
	LDAA    L0006			;Cool temp, linearized
	CMPA    L4E87  			;Temp threshold to move IAC prior to start
	BCS     L9614			;Bra if cool temp <, park IAC and return
;
;-Cool temp high engouh to move IAC
;
	LDD     L0862			;Load idle integrator flow
	ADDA    L086B			;Add in initial added IAC flow
	BCS     L960E			;Bra if overflow
;
	ADDD    L0871			;Add in IAC flow vs. cool temp
	BCC     L9611			;Bra if no overflow
;
L960E	LDD     #$FFFF			;Clear overflow
;
L9611	JMP     L9920			;Go save as desired flow
;
;-Temp not high enough to move IAC
;
L9614	LDAA    L4EA6			;IAC park steps
	JMP     L997C			;Go park IAC
;
;~~~~~~~~~~~~~~~~~~~~~~~~
; Here if engine running
;~~~~~~~~~~~~~~~~~~~~~~~~
;
L961A	LDAB    L0036			;Status word
	PSHB				;Push to stack
;
	LDAA    L400F			;Opt. byte
	BITA    #$40			;Test b6
	BEQ     L962A			;Bra if ==0, not using TCC alone
;
;-TCC only in use
;
	BRSET   L0041,#$20,L9654	;Status word, Bra if b5==1, in P/N
;
	BRA     L9632			;Bra to continue
;
;-Electronic trans in use here
;
L962A	BRSET   L009C,#$40,L9654	;Trans range word, bra if b6==1, in P/N
;
;-Not in P/N
;
	BRSET   L009C,#$10,L9654	;Trans range word, bra if b4==1, invalid range
;
;-Here to flag drive being set
;
L9632	BSET    L0036,#$02		;Status word, set b2, in drive
	BITB    #$02			;Test previous state 
	BNE     L9684			;Bra if already in drive
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Vehicle has shifted from park to drive
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
	LDAA    L4F19			;Added IAC %flow for park to drive
	PSHA    
	LDAA    L4F1B			;Load closed loop idle delay
	LDAB    L4F1A			;Period to transition from park to drive
	XGDX				;Swap vals into X 	   
	LDAA    L0006			;Cool temp
	CMPA    L4F1F			;Threshold for cold idle added delays
	XGDX				;Get looked up vals back    
	BCC     L9674			;Bra if cool temp is >= threshold
;
;-Cold engine temp present
;
	ADDA    L4F20			;Additional delay for closed loop when cold
	ADDB    L4F21			;Additional transition delay when cold
	BRA     L9674			;Bra to save and continue
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Vehicle in park/neutral here
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
L9654	BCLR    L0036,#$02		;Status word clear b1, now in P/N 
	BITB    #$02			;Test previous state
	BEQ     L9684			;Bra if already in P/N
;
;-Have shifted from drive to park/neut.
;
	LDAA    L4F1C			;Add'l IAC flow for drive to park/neutral			
	PSHA				;Push to stack    
	LDAA    L4F1E			;Load closed loop delay
	LDAB    L4F1D			;Load drive to park transition period
	XGDX				;Swap into X    
	LDAA    L0006			;Load cool temp
	CMPA    L4F1F			;Threshold for cold idle added delays
	XGDX				;Get values back     
	BCC     L9674			;Bra if cool temp >= threshold 
;
;-Cold engine temp present
;
	ADDA    L4F22			;Add in addition closed loop delay
	ADDB    L4F23			;Add in addition transition time
;
L9674	STAB    L087A			;Save trans shift transition period
	ABA				;Add in transition time to closed loop delay    
	PULB				;Get added flow off stack    
	STAB    L0879			;Save it, added IAC flow for trans shift			
;
	CMPA    L0864			;IAC closed loop enable delay timer
	BLS     L9684			;Bra if calculated delay is <=
;
;-Calculated delay was >
;
	STAA    L0864			;Save it, closed loop enable delay timer
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Here if no change in P/N or drive status
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;-Generate delta flow term for A/C transition
;
L9684	BRSET   L0036,#$02,L9690  	;Status word, bra if b1==1, in drive
;
;-In P/N
;
	LDAA    L02C3			;Current idle integrator, P/N, A/C on
	SUBA    L02C0			;-Current idle integrator, P/N, A/C off
	BRA     L9696			;Bra to continue
;
;-In drive
;
L9690	LDAA    L02CD			;Current idle integrator, drive, A/C on
	SUBA    L02CA			;-Current idle integrator, drive, A/C off
;
;~~~~~~~~~~~~~~~~~~~~~
; Check status of A/C
;~~~~~~~~~~~~~~~~~~~~~
;
L9696	PULB				;Pull status word back off stack and into B  
	PSHB				;Push it back on   
	BRSET   L0052,#$20,L96C5	;Status word, bra if b5==1, A/C anticipat'g to be on 
;
;-A/C currently off here
;
	BCLR    L0036,#$01		;Clear b0, A/C off
	BITB    #$01			;Test b0, previous state			
	BEQ     L970F			;Bra if ==0, A/C already off
;
;-A/C has just turned off
;
	TST     L0877			;IAC period for A/C on to off 
	BNE     L970F			;Bra if !=0, recently changed state
;
;-Enough time has passed for A/C to be off, transition from A/C running to off IAC flow
;
	LDAB    L4F05			;Delta flow multiplier for on to off transition
	MUL				;Delta flow x multiplier, now sub'd flow to 
					;to transition from A/C on to off     
	CMPA    L4F06			;Min A/C on to off transition flow
	BCC     L96B4			;Bra if >=
;
	LDAA    L4F06			;Load min flow
;
L96B4	CMPA    L4F07			;Max A/C on to off transition flow
	BLS     L96BC			;Bra if <=
;
	LDAA    L4F07			;Load max flow
;
L96BC	LDAB    L4F09			;Additional added delay for closed loop idle enable
	PSHB				;Push to stack    
	LDAB    L4F08			;A/C on to off transition period
	BRA     L96FF			;Bra to save and continue
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Here if A/C is going to be on
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
L96C5	BSET    L0036,#$01		;Set b0, A/C active
	BITB    #$01			;Test b0 of B, previous state
	BNE     L970F			;Bra if A/C was already active
;
;-A/C has just become active
;
	LDAB    L4F18			;Decay period for added A/C transition flow
	STAB    L0878			;Save it, A/C transition flow decay period
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; A/C has just changed state
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
	LDAB    L4E85			;IAC opt. byte
	BITB    #$40			;Test b6, use MAT based table for A/C on off trans.
	BEQ     L96E4			;Bra if ==0, use constants instead
;
;-Using MAT based table for delta A/C transition flow
;
	LDX     #L4F0A			;A/C transition delta flow vs. MAT temp
	LDAA    L0252			;Load MAT temp
	JSR     LF141			;2D lookup with offset and upper limit
;
	BRA     L96F8			;Bra to continue
;
;-Here if not using MAT based table for added IAC flow
;
L96E4	LDAB    L4F12			;Flow transition multiplier for A/C change state
	MUL				;Airflow x multiplier
	CMPA    L4F13			;Min allowed change in airflow for A/C transition
	BCC     L96F0			;Bra if change in airflow was >=
;
	LDAA    L4F13			;Load min change in flow
;
L96F0	CMPA    L4F14			;Max allowed change in airflow for A/C transition
	BLS     L96F8			;Bra if change in flow was <= 
;
	LDAA    L4F14			;Load max change in flow
;
L96F8 	LDAB    L4F16			;Delay for closed loop after A/C transition
	PSHB 				;Push to stack
;									
	LDAB    L4F15			;Idle transition period for A/C change state
;									
L96FF	STAA    L0876			;Save added flow for A/C transition 
	STAB    L0877			;Save delay, idle transition period for A/C
					;change state
	PULA				;Pull closed loop delay    
	ABA				;Add in idle delay   
	CMPA    L0864			;Compare delay to closed loop enable delay timer
	BLS     L970F			;Bra if enable delay timer is >=
;
	STAA    L0864			;Save delay, closed loop enable delay timer
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Here if no change in A/C status, check if integral terms need updating
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
L970F	PULA				;Get status word off stack    
	PSHA				;Push it back on   
;
	EORA    L0036			;EOR with current status word
	ANDA    #$03			;Clear all but b0, A/C status, and b1, drive/PN
	BNE     L971A			;Bra if change in A/C status or drive/PN
;
;-No change in status
;
	PULA				;Retrieve status word    
	BRA     L9781			;Bra to continue
;
;-Change in status has ocured, check for updates
;
L971A	LDAB    L0036			;Status word
	BITB    #$02			;Test b1, drive/PN
	BNE     L973C			;Bra if ==1, in drive
;
;-In P/N 
;
	BITB    #$01			;Test b1, A/C status
	BNE     L9730			;Bra if ==1, A/C is on
;
;-In P/N, no A/C
;
	BRCLR   L0038,#$01,L9756	;Bra if b0==0, update integral flag, P/N, no A/C,
					;has not been set
;
;-Here to update P/N, no A/C integral term
;
	LDX     L088F			;Updated integral term, P/N, no A/C
	STX     L02C0			;Save it, integral term, P/N, no A/C
	BRA     L9756			;Bra to continue
;
;-In P/N, A/C on
;
L9730	BRCLR   L0038,#$02,L9756	;Bra if b1==1, update integral flag, P/N, A/C on, 
					;has not been set
; 
;-Here to update P/N, A/C on idle cell
;
	LDX     L0891			;Updated integral term, P/N, A/C
	STX     L02C3			;Save it, integral term, P/N, A/C
	BRA     L9756			;Bra to continue
;
;-Here if in drive
;
L973C	BITB    #$01			;Test b0, A/C status 
	BNE     L974C			;Bra if ==1, A/C on
;
;-In drive, A/C off
;
	BRCLR   L0038,#$04,L9756	;Bra if b2==0, update integral flag, drive, A/C off,
					;has not been set
;
;-Here to update drive, A/C off idle cells
;
	LDX     L0895			;Updated integral term, drive, no A/C
	STX     L02CA			;Save it, integral term, drive, no A/C
	BRA     L9756			;Bra to continue
;
;-In drive, A/C on
;
L974C	BRCLR   L0038,#$08,L9756	;Bra if b2==0, update integral flag, drive, A/C on,
					;has not been set
;											
	LDX     L0897			;Updated integral term, drive, A/C on
	STX     L02CD			;Save it, integral term, drive, A/C on
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Here if not updating int. terms, check if initial flows need updating
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
L9756	PULB				;Get status word back
	BRCLR   L0038,#$10,L977E	;Bra if b4==0, initial IAC flows not updated
;											
	LDAA    L02C8			;IAC flow vs. cool temp P/N
	ORAA    L02D2			;IAC flow vs. cool temp, drive
	BEQ     L977B			;Bra if flows decayed out
;
;-Flows not yet decayed out, check for change in state
;
	EORB    L0036			;Ex. or with stord status word
	BITB    #$02			;Test b1, P/N, drive status 
	BEQ     L977E			;Bra if no chang in state
;
;-Have shifted between P/N, drive
;
	BRSET   L0036,#$02,L9775	;Bra if b1==1, in drive
;
;-In P/N
;
	LDD     L0893			;Updated initial IAC flow vs. cool temp (P/N)
	STD     L02C6			;Save it, coupled IAC flow vs. cool temp (P/N)
	BRA     L977B			;Bra to continue
;
;-Here if in drive
;
L9775	LDD     L0899			;Updated initial IAC flow vs. cool temp (drive)
	STD     L02D0			;Save it, coupled IAC flow vs. cool temp (drive)
;
L977B	BCLR    L0038,#$10		;Status word, clear b4, update initial IAC flows
					;flag
;
L977E	BCLR    L0038,#$0F		;Clear lower four update integral flags
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Check if power steering switch is in use
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
L9781	LDAA    L4E86			;Opt. byte
	BITA    #$02			;Test b1, power steering press. switch in use
	BEQ     L97A5			;Bra if ==0, not using switch
;
;-Switch in use here
;
	JSR     L932A			;Go read in state of switch
;
	BCC     L97A2			;Bra if carry clear, P/S not active
;
	BSET    L0036,#$20		;Status word, set b5, power steering compensation
					;in progress
;
	LDAA    L4F24			;Added air for P/S compensation
	STAA    L087B			;Save it, P/S compensation flow
;	
	LDAA    L4F26			;P/S compensation flow decay period
	STAA    L087C			;Save it 
;
	LDAA    L4F25			;Closed loop idle delay if P/S comp. enabled
	STAA    L0864			;Closed loop idle enable delay timer
;
L97A2	BCLR    L0036,#$20		;Clear b5, P/S compensation in progress
;
;~~~~~~~~~~~~~~~~~~
; Decrement timers
;~~~~~~~~~~~~~~~~~~
;
L97A5	LDAA    L087A			;Trans shift transition timer			
	BNE     L97AF			;Bra if !=0
;
;-Trans shift complete
;
	STAA    L0879			;Added IAC flow for trans shift
	BRA     L97B3			;Bra to continue
;
;-Still in transition
;
L97AF	DECA				;-1, trans shift transition timer    
	STAA    L087A			;Save timer
;
;-Decrement A/C transition timer
;
L97B3	LDAA    L0877			;A/C on/off transition timer
	BEQ     L97BC			;Bra if ==0
;
	DECA				;-1 from timer   
	STAA    L0877			;Save it
;
;-Decrement stall saver duration timer
;
L97BC	LDAA    L0869			;Stall saver duration timer
	BNE     L97C6			;Bra if ==0
;
	BCLR    L0036,#$10		;Status word, clear b4, stall saver in 
					;progress flag
	BRA     L97CA			;
;
;-Stall saver still in effect
;
L97C6	DECA				;-1
	STAA    L0869			;Save it, stall saver duration timer
;
;-Decrement TPS filtering delay timer
;
L97CA	LDAA    L0884			;TPS filtering period
	BEQ     L97D3			;Bra if ==0
;
	DECA				;-1    
	STAA    L0884			;Save it, TPS filtering period
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Check engine speed against desired idle speed 
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
L97D3	LDAA    L0063			;RPM/12.5
	SUBA    L0857			;Subtract desired RPMs
	BSET    L0036,#$80		;Status word, set b7, high idle speed
	BCC     L97E1			;Bra if RPMs where greater then desired idle speed
;
;-Idle speed is less then desired idle 
;
	NEGA				;Get abs. value    
	BCLR    L0036,#$80		;Clear b7, low idle speed
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Check if IAC compensation is in effect and use default errors if needed
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; 
;
;-Check for manual park/drive trans. shift
;									
L97E1	STAA    L0858			;Save it, idle speed RPM error
	CLRB				;   
	TST     L087A			;Trans shift transition period 
	BEQ     L97ED			;Bra if ==0
;
	LDAB    L4F76			;Default RRM error when shifting from park to drive
					;and vice versa
;
;-Check for A/C on off transition
;			
L97ED	TST     L0877			;A/C on/off transition period
	BEQ     L97FA			;Bra if ==0
;
	CMPB    L4F77			;Compare to default A/C transition RPM error
	BCC     L97FA			;Bra if current RPM error >=

	LDAB    L4F77			;Use default RPM error instead
;
;-Check if power steering pump under load
;
L97FA	TST     L087C			;Decay period for power steering active
	BEQ     L9807			;Bra if ==0			
;
	CMPB    L4F78			;Default error if power steering under load
	BCC     L9807			;Bra if idle RPM error is >=
;
	LDAB    L4F78			;Use default RPM error instead
;
L9807	PSHB				;Push error to stack    
	LDAB    L0882			;Filtered TPS 
	CMPB    L0885			;Upper limit for closed TPS
	PULB				;Get error back    
	BLS     L9819			;Bra if TPS is less
;
;-Throttle is open
;
	CMPB    L4F75			;Upper default RPM error limit for open throttle
	BCC     L9819			;Bra if error is >
;
	LDAB    L4F75			;Load limit instead
;
;-Throttle is closed
;
L9819	TSTB				;Test default RPM error    
	BEQ     L9820			;Bra if ==0
;
	TBA				;Get default RPM error instead     
	CLRB				;    
	BRA     L9829			;Bra to save and continue

L9820	LDX     L0859			;Filtered RPM error
	LDAB    L4F74			;Filter coeff. for RPM error
	JSR     LF0F6			;Filter routine
;
L9829	STD     L0859			;Save it, filtered RPM error
	JSR     L9AA8			;
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Start constructing the desired IAC airflow 
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
	JSR     L9D03			;Routine to look up proportional idle term
;
	JSR     L9D89			;Routine to generate derivative idle term
;
	LDD     L0862			;Load stored idle integrator
	ADDD    L0871			;added coupled IAC flow vs. cool temp
	BCC     L9840			;Bra if no overflow
;
	LDD     #$FFFF			;Clear overflow
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Add in proportional idle term
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
L9840	BRSET   L0036,#$80,L984E	;Status word, bra if b7==1, high idle
;
;-Low idle
;
	ADDA    L086D			;Add in proportional idle term
	BCC     L9855			;Bra if no overflow
;
	LDD     #$FFFF			;Clear overflow
	BRA     L9855			;Bra to continue
;
;-High idle
;
L984E	SUBA    L086D			;Subtract out proportional idle term
	BCC     L9855			;Bra if no underflow
;
	CLRA				;Clear underflow 	   
	CLRB				;    	
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Add in derivative idle term
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
L9855	BRCLR   L0037,#$80,L9863	;Status word, bra if b7==0, negative derivative
					;term
;
;-Positive derivative term
;
	ADDA    L0870			;Add in derivative idle term
	BCC     L986A			;Bra if no overflow
;
	LDD     #$FFFF			;Clear overflow
	BRA     L986A			;Bra to continue
;
;-Negative derivative term
;
L9863	SUBA    L0870			;Subtract out derivative idle term
	BCC     L986A			;Bra if no underflow
;
	CLRA				;Clear underflow    
	CLRB				;   
; 
;-Add in initial flow after startup
;
L986A	ADDA    L086B			;Initial added IAC flow
	BCS     L9874			;Bra if overflow

	ADDA    L087B			;Add in P/S compensation flow
	BCC     L9877			;Bra if no overflow
;
L9874	LDD     #$FFFF			;Clear overflow
;
;-Add in A/C transition flow if needed
;
L9877	TST     L0876			;Added flow for A/C on/off transition
	BEQ     L9891			;Bra if added flow was ==0, transition 
					;complete
;
	BRCLR   L0036,#$01,L988A	;Status word, bra if b0==0, no A/C
;
;-A/C has come on, add in the flow
;										   
	ADDA    L0876			;Add in added flow
	BCC     L9891			;Bra if no overflow
;
	LDD     #$FFFF			;clear overflow										;
	BRA     L9891			;Bra to continue				
;
;-Here if A/C has gone off, subtract out the flow
;																						
L988A	SUBA    L0876			;Subtract out added flow
	BCC     L9891			;Bra if no underflow
;											
	CLRA				;Clear underflow
	CLRB				;
;
L9891	ADDA    L0879			;Added IAC flow for trans shift
	BCC     L9899			;Bra if no overflow
;
	LDD     #$FFFF			;Clear overflow
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Check if IAC accel enrich is needed
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;										
L9899	LDX     L087D			;Load stored IAC flow
	STD     L087D			;Update stored IAC flow
;
;-Check TPS threshold for IAC AE
;											
	PSHA				;MSB to stack
	LDAA    L01FC			;TPS
	CMPA    L509F			;TPS threshold for IAC AE
	PULA				;Get MSB back
	BCC     L98B5			;Bra if TPS >=, no AE
;											
	PSHX				;Push stored IAC flow to stack
	TSX				;Get stack pointer into X
	SUBD    0,X			;Subtract stored flow from current flow
	PULX				;Get stored flow back
	BCS     L98B5			;Bra if stored flow was greater			
;
;-IAC flow is rising, check flow threshold for IAC AE
;
	CMPA    L509E			;Flow threshold for IAC AE
	BCC     L98B8			;Bra if flow >=, enough for AE 
;
;-No IAC AE here
;
L98B5	CLRA				;Clear out AE
	BRA     L98C6			;Bra to save				
;
;-Continue with AE calcs
;											
L98B8	ASLD				;Delta flow x2
	BCS     L98BE			;Bra if overflow
;
	ASLD				;x2
	BCC     L98C0			;Bra if no overflow
;
L98BE	LDAA    #$FF			;Clear overflow
;
;-Look up needed AE
;
L98C0	LDX     #L5098			;Accel enrich vs. delta IAC flow table
	JSR     LF153			;2D lookup with line count
;	
;-Here if no AE
;
L98C6	STAA    L087F			;Save it, IAC flow AE
	LDD     L087D			;Get IAC flow back
	ADDA    L0880			;Add in throttle follower flow
	BCC     L98D6			;Bra if no overflow
;
	LDD     #$FFFF			;Clear overflow
	BRA     L9913			;Bra to continue
;
L98D6	BRCLR   L0046,#$08,L98FD	;Status word, bra if b3==0, no DFCO
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-DFCO in effect, look up added IAC flow
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
	PSHB				;Push flow to stack    
	PSHA				;   
;
	LDX     #L50A3			;Additional IAC flow vs. RPM for DFCO
	LDAA    L0062			;RPM/25
	JSR     LF15E 			;2D lookup
;	
	SUBA    L50A2			;Subtract flow bias for table
	BCS     L98EE			;Bra if underflow
;
;-Added flow positive here
;
	BCLR    L0038,#$20		;Status word, clear b5, added DFCO IAC flow 
					;is positive 			
	BRA     L98F2			;				
;
;-Added flow negative here
;
L98EE	BSET    L0038,#$20		;Set b5, added DFCO IAC flow negative
	NEGA				;Get absolute value
;
L98F2	STAA    L0887			;Save it, added IAC flow when in DFCO
	LDAA    L50B5			;DFCO decay rate
	STAA    L0888			;Save it, DFCO flow decay rate
;
	PULA				;Get IAC flow back
	PULB				;
;
L98FD	BRSET   L0038,#$20,L990B	;Status word, bra if added DFCO flow negative
;
;-Adding flow
;
	ADDA    L0887			;Add in DFCO flow
	BCC     L9913			;Bra if no overflow
;
	LDD     #$FFFF			;Clear overflow
	BRA     L9913			;Bra to continue
;
;-Subtracting flow
;
L990B	SUBA    L0887			;Subtract DFCO flow
	BCC     L9913			;Bra if no underflow
;
	LDD     #$0000			;Clear underflow
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Adjust IAC if PE in effect
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
L9913	BRCLR   L003D,#$20,L9920	;Status word, bra if b5==0, no PE 
;
;-In PE
;
	CPD     L50A0			;Compare flow to IAC flow limit when in PE
	BLS     L9920			;Bra if IAC flow is <=
;
	LDD     L50A0			;Load limit instead
;
;-Here to save IAC flow
;
L9920	STD     L0889			;Save it, finalized IAC flow 
;
;~~~~~~~~~~~~~~~~~~~~~~~~~
; Apply baro compensation
;~~~~~~~~~~~~~~~~~~~~~~~~~
;
	LDX     #L4E99			;IAC altitude compensation vs. baro
	LDAA    L01EF			;Baro value for lookup
	JSR     LF141			;2D lookup with additional quals
;
	LDX     #L0889			;finalized IAC air flow
	JSR     LF1ED			;(A x X*)/256
;	
	ASLD				;Result x 2, now /128    
	BCC     L9938			;Bra if no overflow
;
	LDD     #$FFFF			;Clear overflow
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Check and compensate for throttle kicker if active
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
L9938	LDX     #L4E85			;IAC option byte
	BRCLR   0,X,#$04,L9963		;Bra if b2==0, no throttle kicker 
;
;-Throttle kicker in use
;
	BRCLR   L0037,#$04,L994F	;Status word, bra if b2==0, throttle kicker
					;not active
;
	SUBA    L50BE			;Subtracted % airflow bias when throttle 
					;kicker is active
	BCC     L994F			;Bra if no underflow
;
;-Excess throttle kicker action, request T/K disable
;
	BSET    L0037,#$08		;Status word, set b3, disable throttle 
					;kicker request
	CLRA				;Clear desired flow    
	CLRB				;    
	BRA     L9963			;Bra to save and continue				
;
;-Throttle kicker action within limits, apply % flow multiplier
;
L994F	XGDX				;Swap airflow into X    
	PSHX				;To stack   
	PSHX				;To stack    
	TSX				;Get current stack pointer into X    
;
	LDAA    L50BF			;% airflow multiplier for throttle kicker
	JSR     LF1ED			;(A x X*)/256
;	
	PULX				;Clean up stack   
	TSX				;Get current stack pointer into X     
	ADDD    0,X			;Add in old airflow
	PULX				;Clean up stack    
	BCC     L9963			;Bra if no overflow
;		
	LDD     #$FFFF			;Clear overflow
;
L9963	STD     L088B			;Save it, desired airflow
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Look up IAC counts from flowrate
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
	LDX     #L4E88			;IAC steps vs. airflow
	LSRD				;/2    
	LSRD				;/2    
	LSRD				;/2    
	LSRD				;/2, airflow /16 for lookup
	ADCB    #$00			;perform carry if needed
	ADCA    #$00			;perform carry if needed				
	PSHB				;Push LSB to stack   
	TAB				;Get airflow into B, use as offset	     
	ABX				;Add in offset     
	LDD     0,X			;Load in IAC counts  
;
;-Interpolate result
;
	SBA				;Subtract IAC counts in next cell from 
					;current to get slope    
	PULB				;Get LSB back    
	NEGA				;Flip sign of slope    
	MUL				;LSB of airflow x IAC counts     
	ADCA    0,X			;Add in to looked up IAC counts to 
					;interpolate
;
;~~~~~~~~~~~~~~~~~~~~~~~~~
; Entry point to park IAC
;~~~~~~~~~~~~~~~~~~~~~~~~~
;
L997C	STAA    L0008			;Save as desired IAC steps
;
L997E	RTS				;Return     
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; Routine to construct desired RPM term
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
L997F	BRCLR   L0038,#$40,L9986	; 
;
	JMP     L9AA7			;Return
;
;-Scale cool temp for lookup
;
L9986	LDAA    L02A6			;Range limited cool temp
	CMPA    #160			;92 deg C			
	BLS     L9992			;Bra if cool temp is greater
;
; Cool temp = .75 x val - 28 when <= 92 deg C
;
; Cool temp = .375 x val + 32 when > 92 deg C
;
	SUBA    #160			;-160
	ASLA				;x2    
	ADDA    #160			;+160
;
;-Check if A/C is on
;
L9992	BRCLR   L0036,#$01,L99A2	;Status word, bra if b0==0, no A/C 
;
;-A/C requested here
;
	LDX     #L4F37			;Desired idle speed table for P/N and A/C on
;
;-Check if in park/neutral
;
	BRCLR   L0036,#$02,L99AC	;Status word, bra if b1==0, in P/N 
;
;-In drive and A/C is on
;
	LDX     #L4F55			;Desired idle speed table for drive and A/C is on
	BRA     L99AC			;Go do lookup
;
;-No A/C here
;				
L99A2	LDX     #L4F28			;Desired idle speed table for P/N and A/C off
;
;-Check if in park/neutral
;
	BRCLR   L0036,#$02,L99AC	;Status word, bra if b1==0, in P/N
;
;-In drive and A/C is off
;
	LDX     #L4F46			;Desired idle speed table for drive and A/C is off
;
L99AC	JSR     LF15E			;2D lookup
	PSHA				;Push looked up value to stack
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Check and correct for low batt volts
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
	LDAB    L4F7A			;Lower threshold for batt volt correction
	SUBB    L00A1			;A/D batt volts
	BLS     L99C6			;Bra if batt volts >=
;
;-Batt volts lower then threshold
;
	LDAA    L4F7B			;Prop. gain for batt volts
	MUL				;Gain x delta batt volts     
	CMPA    L4F7C			;Upper limit for added RPM
	BCS     L99C3			;Bra if RPM corrections less
;
	LDAA    L4F7C			;Load max
;
L99C3	PULB				;Get desired RPMs back off of stack    
	ABA				;Add in corrections     
	PSHA				;Push desired RPM back to stack   
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Appy gain to RPM error to generate added desired idle RPMs based on RPM error
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
L99C6	LDAB    L0859			;Load filtered RPM error
	LDAA    L4F73			;Gain factor for RPM error
	MUL				;Gain x RPM error     
	CMPA    L4F79			;Upper added RPM limit for rough idle
	BCS     L99D5			;Bra if RPM error is <=
;
	LDAA    L4F79			;Load upper limit instead
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Check if CCP purge compensation is necessary
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; 
L99D5	PULB				;Get desired RPM back    
	ABA				;Add in the additional RPMs    
	LDAB    L026A			;Get current BLM cell #
	CMPB    #16			;Start of idle cells
	BCS     L99ED			;Bra if not in idle cell
;
;-In an idle cell
;
	LDAB    L026B			;Current BLM
	CMPB    L4F7D              	;BLM threshold to add hot idle purge compensation
	BCC     L99ED			;Bra if BLM >=
;
	LDAB    L0006			;Cool temp, linearized
	CMPB    L4F7F			;Min cool temp for hot idle purge compensation
	BHI     L99F2			;Bra if cool temp is >
;
;-Here to skip purge compensation
;
L99ED	CLR     L085B			;Clear idle purge timer
	BRA     L9A20			;Bra to continue
;
;-Here if purge compensation ok'd
;
L99F2	LDAB    L085B			;Load idle purge timer
	CMPB    L4F7E			;Timer threshold to begin compensation
	BCC     L9A05			;Bra if timer >=, continue with compensation
;
;-Not time for idle purge compensation yet
;
	BRCLR   L0002,#$F0,L9A00	;Loop counter, bra if b0-b3 are clear, inc. timer
;
	BRA     L9A20			;Skip compensation
;
L9A00	INC     L085B			;+1, idle purge timer
	BRA     L9A20			;Bra to skip compensation
;
;-Here to continue with idle purge compensation
;
L9A05	BRSET   L0036,#$02,L9A15	;Bra if b1==1, in drive
;
;-In P/N
;
	LDAB    L4F80			;Added RPM for P/N purge comp.
	BRCLR   L0036,#$01,L9A1F	;Status word, bra if b0==0, no A/C
;
;-In P/N and A/C is on
;
	LDAB    L4F81			;Added RPM for P/N + A/C purge comp.
	BRA     L9A1F			;Bra to continue
;
;-In drive
;
L9A15	LDAB    L4F82			;Added RPM for drive purge compensation
	BRCLR   L0036,#$01,L9A1F	;Status word, bra if b0==0, A/C off
;
;-In drive and A/C is on
;
	LDAB    L4F83			;Added RPm for drive + A/C purge compensation
;
L9A1F	ABA 				;Add it into the desired RPMs
;
;-Here to skip purge compensation
;   
L9A20	PSHA				;Push desired RPM term to stack    
	LDAA    L085C			;Added desired idle speed
	BEQ     L9A31			;Bra if ==0
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Here to add in additional desired idle speed
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
	BRSET   L0036,#$02,L9A31	;Status word, bra if b1==1, in drive
;
;-In neutral
;
	ASLA				;Added desired idle x2    
	LDAB    L4EB2			;Multiplier for P/N
	MUL				;added desired idle speed x multiplier     
	ADCA    #$0000			;Round if needed
;
L9A31	PULB				;Get desired idle speed term back    
	ABA				;Add in the additional desired idle speed     
	CMPA    L085F			;Filtered desired idle speed term
	BCC     L9A3C			;Bra if desired RPMs are rising
;
;-Desired idle speed falling, gradually bring down stored idle speed term down to match
;
	LDAA    L085F			;Filtered desired idle speed term
	DECA				;-1 
;   
L9A3C	STAA    L085F			;Save it, filtered desired idle speed term
	LDX     #L4E85			;IAC option word
	BRCLR   0,X,#$10,L9A8B		;Bra if b4==0, no A/C high pressure switch in use
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; A/C high pressure switch in use, increase idle speed if needed to help
; improve airflow through condensor
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;

	LDAB    L4F84			;Desired RPMs for high A/C press. (P/N)
	BRCLR   L0036,#$02,L9A50	;Status word, bra if b1==0, in P/N
;
	LDAB    L4F85			;Desired RPMs for high A/C press (drive)		
;
L9A50	CBA				;Compare to filtered desired RPM term     
	BCS     L9A56			;Bra if stored desired RPMs are <
;
;-Desired RPMs are greater then those for high A/C pressure 
;
	CLRA				;Clear added A/C high pressure RPMs    
	BRA     L9A7F			;Bra to save and continue
;
;-Desired RPMs are less
;
L9A56	JSR     L9333			;Routine to test status of high pressure switch
;
	BCS     L9A6B			;Bra if A/C high pressure switch is closed
;
;-Excessive pressure on high side of A/C not indicated
;
	LDAA    L0860			;Added desired RPMs for high A/C pressure
	BEQ     L9A66			;Bra if ==0
;
	LDAB    L0861			;High A/C pressure delay timer
	BNE     L9A7B			;Bra if !=0, added RPMs still in effect
;
	DECA				;-1, decay out added RPMs
;    
L9A66	LDAB    L4F87			;Load time delay for added RPMs for high A/C press.
	BRA     L9A82			;Bra to save and continue
;
;-Here if excessive high side pressure is indicated
;
L9A6B	CMPB    L0857			;Compare desired A/C RPMs to desired RPMs
	BHI     L9A73			;Bra if desired RPMs are less
;
	TBA				;Get desired A/C RPMs into A instead     
	BRA     L9A8B			;Bra to save and continue
;
;-Stored desired RPMs are less then desired RPMs for high A/C press.
;
L9A73	LDAA    L0860			;Added desired RPMs
	LDAB    L0861			;High A/C pressure delay timer
	BEQ     L9A7E			;Bra if timer ==0, begin add
;
L9A7B	DECB				;-1 from timer    
	BRA     L9A82			;Bra
;
;-Timer has run out, add in additional desired RPM to help improve airflow through
; condensor
;
L9A7E	INCA				;+1, added A/C high pressure desired RPMs
;    
L9A7F	LDAB    L4F86			;Delay to add in desired RPMs if high load
					;previously present
;
L9A82	STAA    L0860			;Save it, A/C high pressure added RPMs
	ADDA    L085F			;Add stored filtered desired RPM term in
	STAB    L0861			;Save high A/C pressure delay timer
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Here if A/C high pressure switch isnt in use
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
L9A8B	STAA    L0857			;Save as desired idle RPM
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Calculate stall saver activation threshold
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
	LDX     #L5043			;Start of stall saver multipliers
	LDAB    #$03			;Mask b0 and b1
;
; L0036
;  b1, 1=Drive, 0=P/N
;  b0, 1=A/C on, 0= A/C off
;
	ANDB    L0036			;Mask lower two bits
	ABX				;Add in as offset     	
	LDAB    L4E85			;IAC option byte
	BITB    #$80			;Test b7, use default RPM threshold for stall saver
	BEQ     L9AA1			;Bra if ==0, use desired RPM and multiplier instead
;
	LDAA    0,X			;Load RPM threshold
	BRA     L9AA4			;Bra to save
;
;-Use multiplier here
;
L9AA1	LDAB    0,X			;Load multiplier
	MUL				;Desired RPM x multiplier     
L9AA4	STAA    L0868			;Save it, stall saver threshold
L9AA7	RTS				;Return    
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; Routine to update the integrator 
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; Items on stack:
; 
; 0: MSB, Delta integrator term
; 1: LSB, Delta integrator term
; 2: MSB, Coupled IAC flow vs. cool temp
; 3: LSB, Coupled IAC flow vs. cool temp
; 4: MSB, Current integrator idle airflow term
; 5: LSB, Current integrator idle airflow term
;
L9AA8	LDX     #L02CA			;start of idle integrator cells, drive
	BRSET   L0036,#$02,L9AB2	;Status word, bra if b1==1, in drive
;
;-In park/neutral
;
	LDX     #L02C0			;Start of idle integrator cells, park/neutral
;
;-Here if in drive
;
L9AB2	LDD     3,X			;Load current idle integrator flow term, A/C
	BRSET   L0036,#$01,L9ABA	;Bra if b0==1, A/C is on
;
;-A/C is off
;
	LDD     0,X			;Load current idle integrator flow term, no A/C
;
;-Here if A/C is on
;
L9ABA	PSHB				;Push current integrator term to stack    
	PSHA 				;   
	LDD     6,X			;Load coupled IAC flow vs. cool temp
	PSHB				;Push to stack    
	PSHA				;    
	CLRB				;Pre-clear two bytes for delta integrator term    
	PSHB				;    
	PSHB				;    
	TSY				;Get current stack pointer into Y     
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Check quals for closed loop idle
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
	LDAA    L01FC			;TPS
	CMPA    L4EE8			;1.56 %TPS, upper threshold for closed throttle
	BCS     L9AD2			;Bra if TPS <
;
	BSET    L0039,#$80		;Status word, set b7, throttle is open
	BRA     L9ADD			;Bra to continue
;
L9AD2	BCLR    L0039,#$80		;Clear b7, throttle closed									
	LDAA    L02A7			;MPH
	CMPA    L4EE9			;Upper MPH limit for closed loop idle
	BCS     L9AE8			;Bra if MPH less
;
;-Closed loop quals not met
;
L9ADD	LDAB    L4EEA			;Closed loop enable delay if quals not met
	STAB    L0864			;Closed loop idle enable delay timer
	BCLR    L0036,#$04		;Status word, clear b2, closed loop idle quals 
					;not met
	BRA     L9B10			;Bra to continue
;
;-Closed loop idle quals met
;
L9AE8	BSET    L0036,#$04		;Status word, set b2, closed loop quals met
	LDAA    L0882			;High res filtered TPS
	CMPA    L0885			;Upper limit for closed TPS, hi res term
	BHI     L9B10			;Bra if throttle is open
;
;-Closed loop quals met and throttle is closed
;
	LDAA    L0864			;Closed loop idle delay timer
	BEQ     L9AFE			;Bra if ==0, continue
;
;-Closed loop idle timer has not yet run out
;
	DECA				;-1, 
	STAA    L0864			;Save it, closed loop idle delay timer
	BRA     L9B10			;Bra to continue
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; closed loop quals and delay have been met, check quals for enabling closed loop
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;-Check delay for entering closed loop
;
L9AFE	BRSET   L0036,#$80,L9B09	;Status word, bra if b7==1, idle RPM high
;
;-Low idle
;
	LDAA    L0866			;Closed loop idle delay, low idle
	BEQ     L9B2A			;Bra if ==0, continue
;
	BRA     L9B3B			;Bra, no closed loop
;
;-Here if idle high
;
L9B09	LDAA    L0867			;Closed loop idle delay, high idle
	BEQ     L9B2A			;Bra if ==0, continue
;
	BRA     L9B3B			;Bra, no closed loop
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Timer to ok closed loop entry not yet met or throttle is open
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
L9B10	BRSET   L0036,#$80,L9B3B	;Status word, bra if b7==1, high idle, bar entry 
					;into closed loop
;
;-Check for a transition or timer that has not yet been met
;
	LDAA    L0866			;Closed loop idle delay, low idle
	ORAA    L087A			;Trans shift transition timer
	ORAA    L0877			;A/C on/off transition timer
	BNE     L9B3B			;Bra if in transition or still delaying closed loop
;
;-Check if closed loop quals already met
;
	BRSET   L0036,#$04,L9B2A	;Status word, bra if b2==1, closed loop quals met
;
;-Check if manual is in use, and bar entry into closed loop if delays not yet met 
;
	LDAA    L4E85			;Option byte
	BITA    #$01			;Test b0
	BNE     L9B3B			;Bra if ==1, manual trans in use
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Check if ok to enter closed loop
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
L9B2A	BRSET   L003E,#$04,L9B3B	;Status word, bra if b2==1, batt volts too low 
					;for IAC, no closed loop
;
;-Check if throttle kicker is active
;
	BRCLR   L0037,#$04,L9B41	;Status word, bra if b2==0, throttle kicker not 
					;active, enter closed loop idle
;
;-Throttle kicker is active
;
	LDD     L088B			;Desired IAC airflow
	BNE     L9B41			;Bra if !=, allow closed loop idle
;
;-Desired airflow is ==0, check for low idle
;
	BRCLR   L0036,#$80,L9B41	;Bra if b7==0, low idle, allow closed loop
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Here if closed loop idle not ok'd
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
L9B3B	BCLR    L0036,#$08		;Clear b3, no closed loop idle
	JMP     L9CE9			;Bra to continue				
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Here to ok closed loop idle
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
L9B41	BSET    L0036,#$08		;Set b3, now in closed loop idle
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Check if stall is pending
;~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
	BRCLR   L0036,#$10,L9B50	;Bra if b4==0, stall saver not active
;
;-Stall saver active, check if engine has recovered
;
	BRCLR   L0036,#$80,L9B82	;Bra if b7==0, low idle, engine has not yet
					;recovered
;
;-Engine recovered, clear stall saver timer and added flow
;
	CLRA				;
	CLRB				;
	BRA     L9B70			;Bra to save and continue
;
;-No stall saver, check if engine below threshold for stall saver
;										
L9B50	LDAA    L0868			;Stall saver activation threshold
	CMPA    L0063			;RPM/12.5
	BLS     L9B82			;Bra if RPMs >= to threshold
;
;-RPMs below threshold for stall saver
;
	BSET    L0036,#$10		;Set b4, stall saver now active
	PSHX				;    
;
;-Look up added airflow
;				 
	LDX     #L5047			;Start of added stall saver flow
	LDAB    #$03			;Mask out lower two bytes 
	ANDB    L0036			;And with status word 
	ABX				;Add in as 
	LDAA    0,X			;Load in added flow 
	PULX				;
	LDAB    L504B			;Time for stall saver to be active, P/N
	BRCLR   L0036,#$02,L9B70	;Status word, bra if b1==1, in P/N
;
;-In drive
;
	LDAB    L504C			;Time for stall saver to be active, drive
;			
L9B70	STAB    L0869			;Save it, stall saver duration timer			
	CLRB				;
	ADDD    4,Y			;Add in the stall saver flow to the current 
					;idle integrator
	BCC     L9B7C			;Bra if no overflow

	LDD     #$FFFF			;Clear overflow
;
L9B7C	STD     4,Y			;Save the integrator to stack
	JMP     L9CBD			;Bra to continue			
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Scale RPM error for lookup
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; 200 RPM < RPM error 
;  value for lookup = (RPM error/6.25) + 80
;
; 50 RPM < RPM error <= 200 RPM
;  value for lookup = (RPM error/3.125) + 48
;
; RPM error <= 50 RPM
;  value for lookup = (RPM error/.78125) 
;
L9B82	LDAA    L0858			;Idle speed RPM error
	CMPA    #40			;500 RPM
	BLS     L9B8D			;Bra if RPM error <=

	LDAA    #160			;500 RPM, max scaled RPM for lookup 
	BRA     L9BA1			;Bra to continue
;
;-RPM error <= 500 RPM
;
L9B8D	ASLA				;x2
	CMPA    #32			;200 RPM
	BLS     L9B96			;Bra if RPMs <=
;
;-RPM error > 200 RPM
;
	ADDA    #80			;Add in  offset
	BRA     L9BA1			;Bra to continue
;
;-RPM error <= 200 RPM
;
L9B96	ASLA				;x2
	CMPA    #16			;50 RPM
	BLS     L9B9F			;Bra if RPMs <=
;
;-RPM error > 50 RPM
;
	ADDA    #48			;Add in offset
	BRA     L9BA1			;Bra to continue
;
;-RPM error <= 50 RPM
;
L9B9F	ASLA				;x2    
	ASLA				;x2
;
L9BA1	PSHX				;Push addr. to stack   
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Look up idle integral gain 
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
	BRSET   L0036,#$80,L9BB2	;Status word, bra if b7==1, idle too high
;
;-Idle speed low and in drive
;
	LDX     #L4F9E			;Idle integral gain vs. RPM error (low RPM & drive)
	BRSET   L0036,#$02,L9BBC	;Bra if b1==1 in drive
;
;-Idle speed low and in P/N
;
	LDX     #L4F88			;Idle integral gain vs. RPM error (low RPM and P/N)
	BRA     L9BBC			;Bra to continue
;
;-Idle speed high and in drive
;
L9BB2	LDX     #L4FA9			;Idle integral gain vs. RPM error (hi RPM and drive)
	BRSET   L0036,#$02,L9BBC	;Bra if b1==1, in drive
;
;-Idle speed high and in P/N
;
	LDX     #L4F93			;Idle integral gain vs. RPM error (hi RPM and P/N)
;
L9BBC	JSR     LF15E			;2D lookup
	PULX				;Restore addr.
	LDAB    L0858			;RPM error 
	MUL				;RPM error x integral gain
	STD     0,Y			;Save to stack, delta integral term
;
;-Check condition of idle
;
	BRSET   L0036,#$80,L9C31	;Status word, bra if b7==1, high idle
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Low idle here, check if IAC reset is needed
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
	LDAA    L0006			;Cool temp, linearized
	CMPA    L50B8			;Lower cool temp limit to check integrator 
	BLS     L9BF9			;Bra if cool temp <=
;
;-Cool temp threshold met, check for pending integrator wind up
;
	LDAA    L50B9			;Upper limit for integral idle term if no A/C
	BRCLR   L0036,#$01,L9BDC	;Bra if b0==0, on A/C
;
;-A/C on
;
	LDAA    L50BA			;Upper limit for integral idle term if A/C on
;
L9BDC	CMPA    4,Y			;Compare to integrator
	BCC     L9BF9			;Bra if integrator <=, within limits
;
	BRSET   L003B,#$10,L9BF9	;Status word, bra if b4==1, in mode 4
;
	BRSET   L0050,#$10,L9BF9	;Status word, bra if b4==1, in diagnostics
;
;-Excessive integrator detected, actual and stored positions differ, reposition IAC
;
	BCLR    L0009,#$04		;Status word, clear b2, IAC reset requested
	LDAA    L0002			;Loop counter
	CMPA    L50BC			;Counter val., time threshold to begine removing
					;steps and force IAC to retract
	BCC     L9BF6			;Bra if counter is <=, dont pull steps yet
;
	DEC     L0007			;-1 from stored current IAC steps
;
L9BF6	JMP     L9CE9			;Bra to exit															
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Here if integrator is within limits or temp threshold not yet met
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
L9BF9	LDD     L088B			;Desired IAC airflow
	CPD     #65535			;100% airflow
	BEQ     L9C5F			;Bra if ==, airflow maxed out
;
	LDAA    2,X			;Steady state integrator term, A/C off
	BRCLR   L0036,#$01,L9C0A	;Status word, bra if b0==0, no A/C
;
;-A/C on
;
	LDAA    5,X			;Steady state integrator term, A/C on
;
L9C0A	CMPA    4,Y			;Compare to integrator term on stack
	BHI     L9C23			;Bra if current integrator has fallen
;
;-Here if current integrator term is rising with respect to steady state value
;
	LDD     6,X			;Coupled IAC flow vs. cool temp
	CPD     8,X			;IAC flow vs. cool temp
	BCC     L9C23			;Bra if coupled flow is >=
;
;-Add delta integrator term to IAC flow vs. cool temp instead
;
	ADDD    0,Y			;Add in delta integrator term
	BCC     L9C1E			;Bra if no overflow
;
	LDD     #$FFFF			;Clear overflow
;
L9C1E	STD     2,Y			;Save it, coupled IAC flow vs cool temp on stack
	BRA     L9C5F			;Bra to continue
;
;-Current integrator term has fallen with respect to steady state value
;
L9C23	LDD     4,Y			;Current integrator
	ADDD    0,Y			;Add in delta integrator term
	BCC     L9C2E			;Bra if no overflow

	LDD     #$FFFF			;Clear overflow
;
L9C2E	JMP     L9CBD			;Bra to save
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Here if high idle is present
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;-Provide additional decay for added flows if needed
;
L9C31	LDAA    L087B			;Added P/S compensation flow
	BEQ     L9C49			;Bra if ==0
;
	SUBA    0,Y			;Subtract delta integrator term
	BCC     L9C3E			;Bra if no underflow
;
	CLRA				;Clear flow    
	BRA     L9C44			;Bra to sace

L9C3E	CMPA    L087B			;Compare to stored P/S compensation flow
	BNE     L9C44			;Bra if non-zero difference
;
;-No change from subtraction of delta integrator term
;
	DECA				;-1 if no change from delta integrator term 
;   
L9C44	STAA    L087B			;Save it, P/S compensation flow
	BRA     L9C5F			;Bra to sve and exit
;
;-Check added initial flow after startup
;
L9C49	LDAA    L086B			;Added initial flow after startup
	BEQ     L9C62			;Bra if ==0
;
	SUBA    0,Y			;Subtract delta integrator term
	BCC     L9C56			;Bra if no underflow
;
	CLRA				;Clear underflow    
	BRA     L9C5C			;Bra to save and exit
;
L9C56	CMPA    L086B			;Compare to added initial flow after startup
	BNE     L9C5C			;Bra if delta integrator was non-zero
;
;-No change from subtraction of delta integrator term
;
	DECA				;-1 from flow
;
L9C5C	STAA    L086B			;Initial flow added after startup
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Here if added flow terms are !=0, dont compute integrator terms yet
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
L9C5F	JMP     L9CE9			;Bra to save and return
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Here if flow terms have been decayed out
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
L9C62	LDD     4,Y			;Current idle integrator airflow term
	SUBD    0,Y			;Subtract delta integral term
	BCC     L9C6D			;Bra if no underflow
;
	LDD     #$0000			;Clear underflow
;
;-Check for VSS error
;
L9C6D	BRSET   L0016,#$10,L9C94	;Error word #1, bra if b4==1, err 16, VSS 
;
;-No VSS error
;
	PSHX				;Push addr. in X to stack		    
	LDX     2,Y			;Load coupled IAC flow vs. cool temp 
	PULX				;Restore X    
	BEQ     L9C94			;Bra if cool temp decayed out
;
	PSHB				;Push LSB of current integral term to stack				    
	LDAB    5,X			;Steady state idle integral term, A/C on
	BRSET   L0036,#$01,L9C81	;Status word, bra if b0==1, A/C on
;
;-A/C is off
;
	LDAB    2,X			;Steady state idle integral term, A/C off
;
;-A/C is on
;
L9C81	CBA				;Compare to current integral term     
	PULB				;Get LSB back    
	BCC     L9CBD			;Bra if current integral has risen
;
;-Current integral has fallen below steady state val, reduce cool temp based flow
;
	LDD     2,Y			;Coupled IAC flow vs. cool temp
	SUBD    0,Y			;Subtract delta integral term
	BCC     L9C8F			;Bra if no underflow
;
	CLRA				;Clear flow    
	CLRB				;
;
L9C8F	STD     2,Y			;Save IAC flow vs. cool temp
	BRA     L9CE9			;Bra to continue
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Here to check if low integrator is present
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
L9C94	CPD     #$0000			;Compare integrator to zero
	BNE     L9CBD			;Bra if non-zero integrator is present
;
;-Check for throttle kicker failure
;   
	BRSET   L0009,#$40,L9CBD	;Status word, bra if b6==1, T/K has failed diags.
;
;-Integrator has zero'd out, check RPM error
;
	PSHA				;Push MSB of integrator to stack    
	LDAA    L0858			;RPM error
	CMPA    L50BB			;RPM error threshold to request reset and reposition
					;IAC
	BLS     L9CBC			;Bra if RPM error is <=
;
	BRSET   L003B,#$10,L9CBC	;Status word, bra if b4==1, in mode 4
;
	BRSET   L0050,#$10,L9CBC	;Status word, bra if b4==1, in diags.
;   
;-Integrator has zero'd out and RPM error present, reposition IAC and req. reset
;
	BCLR    L0009,#$04		;Status word, clear b2, request reset
	LDAA    L0002			;Loop counter
	CMPA    L50BC			;Counter val., time threshold to begine adding
					;steps and force IAC to extend
	BCC     L9CBC			;Bra if counter >=
;
	INC     L0007			;+1, stored current IAC steps
;
L9CBC	PULA				;Get MSB of integrator back   
;
;-Check for VSS fault
;
L9CBD	BRCLR   L0016,#$10,L9CDC	;Error word #1, bra if b4==1, err 16, VSS  
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; VSS fault present, make sure integrator doesnt fall below defualt value
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
	PSHB				;Push LSB to stack    
	LDAB    L4EA8			;Default value for integrator term, drive
	BRSET   L0036,#$02,L9CCC	;Status word, bra if b1==1, in drive
;
;-In park/neutral
;
	LDAB    L4EA7			;Default value for integrator term, park/neutral
;
L9CCC	BRCLR   L0036,#$01,L9CD3	;Bra if b0==0, A/C off
;
;-A/C is on
;
	ADDB    L4EA9			;Added flow if A/C is on
;
L9CD3	CBA				;Compare MSB of integrator to default value     
	BCS     L9CD9			;Bra if integrator was <
;
;-Integrator is >= to default value, save and return
;
	PULB				;Get LSB back    
	BRA     L9CDC			;Bra to save and return
;
;-Integrator is < then default value, load default integrator value instead
;
L9CD9	TBA				;Get default value into A     
	PULB				;Pull LSB    
	CLRB				;Clear LSB 
; 
;-Here if no VSS fault
;
L9CDC	STD     4,Y			;Save current integrator to stack
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Save integrator terms and return
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; 
	BRCLR   L0036,#$01,L9CE7	;Status word, bra if b1==0, no A/C
;
;-A/C is on
;
	STD     3,X			;Save current integrator to A/C on idle cell
	BRA     L9CE9			;
;
;-Here if no A/C
;
L9CE7	STD     0,X			;Save current integrator to A/C off idle cell
;
L9CE9	LDD     2,Y			;Coupled IAC flow vs. cool temp
	STD     6,X			;Save it to idle cell
	STD     L0871			;Save it, added coupled IAC flow vs. cool temp
	LDD     8,X			;IAC flow vs. cool temp
	STD     L0873			;Save it, stored IAC flow vs. cool temp
	LDD     4,Y			;Current idle integrator
	STD     L0862			;Save it, stored idle integrator
;
;-Restore stack
;
	LDAB    #$06			;+6
	ABY				;Add in as offset     
	TYS				;Transfer to stack pointer
	RTS				;Return
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; Routine to look up idle proportional term
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
L9D03	BRSET   L0036,#$02,L9D13	;Status word, bra if b1==1, in drive
;
;~~~~~~~~~~~~~~~~~
; In neutral here
;~~~~~~~~~~~~~~~~~
;
	LDX     #L4FC2			;Idle proportional term, low RPM and P/N
	BRCLR   L0036,#$80,L9D53	;Bra if b7==0, low idle RPM
;
;-In neutral and high idle present
;
	LDX     #L4FCD			;Idle proportional term, high RPM and P/N
	BRA     L9D53
;
;~~~~~~~~~~~~~~~
; In drive here
;~~~~~~~~~~~~~~~
;
L9D13	CLRB				;Clear table index    
	BRCLR   L0036,#$01,L9D1A	;Bra if b0==0, A/C is off
;
;-A/C is on
;
	ADDB    #$08			;Advance ahead four tables if A/C is on
;
;-A/C is off
;
L9D1A	BRCLR   L0036,#$08,L9D20	;Bra if b3==0, closed loop idle not enabled
;
;-Closed loop idle
;
	ADDB    #$04			;Advance ahead two tables if in closed loop
;
;-Here if open loop idle, check for high/low idle
;
L9D20	BRCLR   L0036,#$80,L9D26	;Status word, bra if b7==0, low idle
;
;-High idle present
;
	ADDB    #$02			;Advance one table if high idle
;
;-Low idle present
;
L9D26	LDX     #L9D79			;Start of proportional table indexes
	ABX 				;Add in offset for lookup
	LDX     0,X			;Load table
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Check quals for limiting proportional action if high idle and in motion
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
	BRCLR   L0036,#$80,L9D53	;Bra if b7==0, low idle
;
;-High idle present, if in motion, limit proportional action
;
	LDAA    L02A7			;MPH
	CMPA    L4FBF			;Lower threshold to limit prop. action
	BCS     L9D3D			;Bra if current MPH >
;
;-In motion, limit proportional action
;
	BSET    L0037,#$40		;Status word, set b6, limiting idle proportional 
					;action  
	BRA     L9D4B			;Bra to continue
;
;-Check if speed low enough to restore full proportional control
;
L9D3D	CMPA    L4FC0			;Upper threshold to allow full prop. control 
	BCC     L9D47			;Bra if speed >=, dont restore proportional action
;
;-Not in motion, allow full proportional control
;
	BCLR    L0037,#$40		;Clear b6, not limiting proportional action
	BRA     L9D53			;Bra to continue
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Limit proportional action if needed
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
L9D47	BRCLR   L0037,#$40,L9D53	;Bra if b6==0, not limiting proportional action
;
;-Limiting proportional action
;
L9D4B	LDAA    L4FC1			;Upper limit for RPM error if limiting action
	CMPA    L0858			;RPM error
	BLS     L9D56			;Bra if RPM error is >= to threshold
;
;-Here if not limiting proportional action
;
L9D53	LDAA    L0858			;RPM error
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Scale RPM error for lookup
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; 200 < RPM error 
;  value for lookup = (RPM error/6.25) + 80
;
; 50 < RPM error <= 200 RPM
;  value for lookup = (RPM error/3.125) + 48
;
; RPM error <= 50
;  value for lookup = (RPM error/.78125) 
;
L9D56	CMPA    #40			;500 RPM
L9D58	BLS     L9D5E			;Bra if RPM error <=
;
	LDAA    #160			;500 RPM, max scaled RPM for lookup 
	BRA     L9D72			;Bra to continue
;
;-RPM error <= 500 RPM
;
L9D5E	ASLA				;x2    
	CMPA    #32			;200 RPM				
	BLS     L9D67			;Bra if RPMs <=					
;
;-RPM error > 200 RPM
;
	ADDA    #80			;Add in  offset			
	BRA     L9D72			;Bra to continue				
;
;-RPM error <= 200 RPM
;
L9D67	ASLA				;x2    					
	CMPA    #16			;50 RPM				
	BLS     L9D70			;Bra if RPMs <=				
;
;-RPM error > 50 RPM
;
	ADDA    #48			;Add in offset				
	BRA     L9D72			;Bra to continue				
;
;-RPM error <= 50 RPM
;
L9D70	ASLA				;x2     
	ASLA				;x2    
; 
L9D72	JSR     LF15E			;2D lookup 				
;
	STAA    L086D			;Save it, proportional idle term
	RTS				;Return
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~     
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Idle proportional flow tables vs. RPM error
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;-A/C off, in drive, and open loop idle
;                   
L9D79   FDB L4FD8			;Low idle
	FDB L4FE3			;High idle
;
;-A/C off, in drive, and closed loop idle
;
	FDB L4FEE			;Low idle
	FDB L4FF9			;High idle
;
;-A/C on, in drive, and open loop idle
;				
	FDB L5004			;Low idle
	FDB L500F			;high idle
;
;-A/C on, in drive, and closed loop idle
;
	FDB L501A			;Low idle
	FDB L5025			;High idle
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; Routine to construct derivative idle term
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
L9D89	LDX	#L5032			;IAC derivative term table vs. transient RPM ratio 	  
	LDAA    L006C			;Transient RPM ratio
	SUBA    #96			;.75, offset for lookup
	BCC     L9D95			;Bra if no underflow
;
	CLRA				;Clear underflow    
	BRA     L9D9F			;Bra to do lookup
;
L9D95	CMPA    #64			;1.25, upper limit for lookup
	BCS     L9D9D			;Bra if less
;
	LDAA    #255			;Load max value
	BRA     L9D9F			;Bra to do lookup
;
;-Scale ratio for lookup
;
L9D9D	ASLA				;x2    
	ASLA				;x2
L9D9F	JSR     LF15E			;2D lookup
;
;-Check if derivative term is changing
;
	LDX     L086E			;Stored idle derivative term
	CLRB				;Clear LSB    
	CMPA    L086E			;Compare term from lookup to stored term
	BEQ     L9DDF			;Bra if ==, return
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Check quals for limiting derivative action
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;-Needed derivative term has changed
;
	BRSET   L0037,#$80,L9DB7	;Status word, bra if b7==1, positive derivative term
;
;-Derivative term was negative
;
	BHI     L9DC2			;Bra if derivative action is decreasing
;
;-Negative derivative action is increasing, check for low idle
;
	BRCLR   L0036,#$80,L9DC2	;Status word, bra if b7==0, low idle, limit 
					;derivative action
;
;-High idle present
;
	BRA     L9DC8			;Bra, dont limit derivative action
;
;-Here if derivative term was positive
;
L9DB7	BCS     L9DBD			;Bra if derivative action is decreasing
;
;-Positive derivative action is increasing, check for low idle
;
	BRCLR   L0036,#$80,L9DC8	;Bra if b7==1, low idle, dont limit action
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Here to limit positive derivative action
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
L9DBD	LDAB    L5030			;Filter coeff. for limiting positive derivative
					;action
	BRA     L9DC5			;Bra to go filter derivative term
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Here to limit negative derivative action 
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
L9DC2	LDAB    L5031			;Filter coeff. for limiting negative derivative
					;action
L9DC5	JSR     LF0F6			;Filter routine
;
L9DC8	STD     L086E			;Save it, stored derivative idle term
	SUBA    #128			;Subtract 128, make signed value
	BCC     L9DD5			;Bra if derivative term is positive
;
;-Derivative term is negative
;			
	NEGA				;Get abs. value 	   
	BCLR    L0037,#$80		;Clear b7, negative derivative term
	BRA     L9DD8			;Bra to save
;
;-Derivative term is positive
;
L9DD5	BSET    L0037,#$80		;Set b7, derivative term is positive
;
L9DD8	LSRA				;/2    
	LSRA    			;/2
	ADCA    #$00			;Round up if needed
	STAA    L0870			;Save it, derivative idle flow term
L9DDF	RTS				;Return     
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; Throttle follower routine
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
L9DE0	LDAA    L01FC			;TPS
	ASLA				;x2
	BCS     L9DE9			;Bra if overflow
;
	ASLA				;x2   
	BCC     L9DEB			;Bra if overflow
;										
L9DE9	LDAA    #255			;Clear overflow
;
L9DEB	CMPA    L0882			;Hi res filtered TPS
	BLS     L9E16			;Bra if current TPS is <=
;
;-TPS is rising
;											
	LDAB    L5064			;Max TPS for P/N throttle follower
;
	BRCLR   L0036,#$02,L9E0F	;Status word, bra if b1==0, in P/N
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Look up TPS filtering delay timer 
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
	PSHA				;Push current TPS to stack    
;
	LDAA    L02A7			;MPH
	CMPA    #80			;80 MPH, upper limit for lookup
	BLS     L9E01			;Bra if MPH are less
;
	LDAA    #80			;Load upper limit instead
;				
L9E01	ASLA				;MPH x 2 for lookup
	LDX     #L5067			;TPS filtering period vs. MPH 
	JSR     LF15E			;2D lookup
;
	STAA    L0884			;Save it, TPS filtering period
;
	PULA				;Get current TPS back				    
	LDAB    L5065			;Max TPS for drive throttle follower
;
;-Here if TPS is rising and in P/N
;
L9E0F	CBA				;Compare TPS to limit     
	BLS     L9E13			;Bra if TPS is less

	TBA				;Get limit in instead
;     
L9E13	CLRB				;Clear LSB   
	BRA     L9E51			;Bra to continue
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Here if TPS is falling, filter TPS back down
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
L9E16	LDAB    L0884			;Load TPS filtering period
	BEQ     L9E20			;Bra if ==0
;
;-Timer !=0
;
	LDD     L0882			;Stored hi res filtered TPS
	BRA     L9E51			;Bra to continue
;
;-Here if timer has expired, time to filter TPS
;
L9E20	PSHA				;Push TPS to stack    
	BRSET   L0036,#$02,L9E2A	;Status word, bra if b1==1, in drive 
;
;-In park/neutral
;
	LDAA    L5066			;P/N TPS filter coeff.
	BRA     L9E3A			;Bra to filter and continue
;
;-Here if in drive
;
L9E2A	LDAA    L02A7			;MPH
	CMPA    #80			;80 MPH, upper limit for lookup
	BLS     L9E33			;Bra if MPH <=
;
	LDAA    #80			;Load upper limit instead
;
L9E33	ASLA				;MPH x2   
	LDX     #L5072			;TPS filtering coeff. vs. MPH
	JSR     LF15E			;2D lookup
;
L9E3A	LDAB    L01E3			;SD MAP
	CMPB    L507E			;Low MAP threshold to scale down filter coeff.
	BCC     L9E46			;Bra if MAP >=
;
;-Low MAP present, scale down filter coeff.	
;
	LDAB    L507F			;Low MAP multiplier for filter coeff.
	MUL				;TPS filter coeff. x multiplier
;     
L9E46	TAB				;Get filter coeff. into B     
	BNE     L9E4A			;Bra if !=0
;
	INCB				;+1, lowest allowable filter coeff.  
;  
L9E4A	PULA				;Get TPS back off stack   
	LDX     L0882			;Load high res filtered TPS
	JSR     LF0F6			;Filter routine
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Throttle was rising, skip filtering
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;-Check for cool engine temp
;
L9E51	PSHB				;Push filtered TPS to stack    
	PSHA				;   
;
	BRSET   L0037,#$01,L9E70	;Status word, bra if b0==1 warm engine
					;already flagged
;
	LDAB    L02A5			;Initial cool temp (engine off)
	CMPB    L5090			;Lower temp limit for cool engine T/F operation
	BCS     L9E6D			;Bra if cool temp was <
;											
	CMPB    L5091			;Temp threshold for cool engine
	BCC     L9E6D			;Bra if cool temp >=, warm engine temps present
;
;-Check time limit for cool engine status to be set
;
	LDX     L00F8			;Engine run time
	CPX     L5092			;Upper time limit to flag cool engine temps										
	BHI     L9E6D			;Bra if run time was greater
;
	BRA     L9E70			;Bra to continue
;
;-Warm engine temps
;
L9E6D	BSET    L0037,#$01		;Status word, set b0, engine is warm
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Check if throttle is above threshold for open throttle
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~							
;				
L9E70	BRSET   L0039,#$80,L9EB2	;Status word, bra if b7==1, throttle is open
;
;-Throttle is closed here
;
	TST     L0884			;TPS filtering delay period
	BNE     L9EB2			;Bra if !=0
;
;-Filtered TPS has been updated
;
	BRSET   L0036,#$04,L9EB2	;Bra if b2==1, closed loop idle quals met
;
;-Closed loop idle not pending
;							
	BRCLR   L0046,#$08,L9E8D	;Bra if b3==0, DFCO not in effect
;
;-DFCO in effect
;
	LDAB    L4E85			;IAC option byte
	BITB    #$02			;Test b1, maintain T/F action during DFCO
	BNE     L9E8D			;Bra if ==1, perform lower limit for ensuring 
					;added T/F flow
;
;-Here for limited or no T/F action during DFCO
;
	BSET    L0039,#$40		;Status word, set b6, manditory post-DFCO  
					;T/F flow update
;
	BRA     L9ECF			;Bra to skip and continue			
;
;-Check lower limit for filtered TPS during closed throttle to ensure some added flow
;
L9E8D	LDAA    L02A6			;Cool temp
	LDX     #L5080			;TPS limit for minimum added T/F flow 
	JSR     LF15E			;2D lookup
;
	CLRB				;Clear LSB    
;
	BRSET   L0037,#$01,L9EA8	;Status word, bra if b0==1, engine is warm
;
;-Cool engine temps present, scale up lower limit to add aditional flow
;
	LDAB    L5096			;Multiplier for limit
	MUL				;Limit x multiplier     
	ASLD				;x2    
	BCS     L9EA5			;Bra if no overflow
;
	ASLD				;x2, now result /64    
	BCC     L9EA8			;Bra if no overflow
;
L9EA5	LDD     #$FFFF			;Clear overflow
;
L9EA8	TSX				;Get current stack pointer into X    
	CPD     0,X			;Compare to lower limit for TPS
	BLS     L9EBE			;Bra if TPS is greater
;
	STD     0,X			;Save limit instead
	BRA     L9EBE			;Bra to continue
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Here if throttle open/not updating filtered TPS this pass/closed loop idle pending
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
L9EB2	LDAB    #$FF			;Max value for timer if closed loop pending
	BRSET   L0036,#$04,L9ECC	;Status word, bra if closed loop quals met
;
;-Closed loop idle not pending
;										
	CLRB				;Reset timer    
;
	BCLR    L0037,#$02		;Status word, clear b1, reduce throttle follower
					;action
	BRA     L9ECC			;Bra to save and continue
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Closed throttle present, limit T/F action after period of time
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
L9EBE	LDAB    L0886			;Timer to limit T/F action during closed throttle
	CMPB    L508D			;Compare to threshold to reduce T/F action
	BCS     L9ECB			;Bra if timer is <
;
;-Enough time has passed to limit T/F action
;
	BSET    L0037,#$02		;Status word, set b1, limit T/F action
	BRA     L9ECF			;Bra to continue
;
;-Not enough time, inc. timer 
;
L9ECB	INCB				;+1
;
L9ECC	STAB    L0886			;Save timer
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Check if throttle follower action sill required
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
L9ECF	PULA				;Get filtered TPS back    
	PULB				;    	
	STAA    L0882			;Save it, MSB of filtered TPS
;
;-Check TPS limits for save
;
	BNE     L9EDB			;Bra if MSB !=0, save and continue
;
	CMPB    #$0028			;Lower limit for filtered TPS
	BCC     L9EDB			;Bra if TPS >
;
	CLRB				;Clear TPS, throttle closed  
;  
L9EDB	STAB    L0883			;Save it, LSB of filtered TPS
	SUBA    L0885			;Subtract closed throttle TPS
	BCC     L9EE9			;Bra if TPS >=, throttle open
;
;-TPS below limit, clear throttle follower flow and exit
;
	LDD     #$0000			;Clear flow
	JMP     L9F78			;Bra to save and exit
;
;-Reduce throttle follwer action if needed
;
L9EE9	BRCLR   L0037,#$02,L9EFE	;Bra if b1==0, not limiting action
;
;-Reduced action ok'd
;
	PSHB				;Push %TPS to stack    
	PSHA    			;
	TSX				;Get current stack pointer into X     
	LDAA    L5094			;Multiplier for TPS, cool engine
	BRCLR   L0037,#$01,L9EFA	;Bra if b0==0, cool engine temps
;
;-Warm engine temps present
;
	LDAA    L5095			;Multiplier for TPS, warm engine
;
L9EFA	JSR     LF1ED			;(A x X*)/256
;
	PULX				;Clean up stack
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Begin applying proportional T/F gains
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;   
;-Apply RPM based gain
;
L9EFE	PSHB				;Push %TPS to stack    
	PSHA				;    
	LDAA    L0063			;RPM/12.5
	LDX     #L505A			;Throttle follower gain vs. RPMs
	JSR     LF153			;2D lookup w/ line count
;
	TSX				;Get current stack pointer into X     
	JSR     LF1ED			;(A x X*)/256
;
	PULX				;Clean up stack 	   
	ASLD				;Result x2    
	BCS     L9F13			;bra if overflow
;
	ASLD				;Result x2, now result /64			    
	BCC     L9F18			;Bra if no overflow
;
L9F13	LDD     #$FFFF			;Clear overflow
	BRA     L9F31			;Bra to continue
;
;-Apply cool temp based gain
;
L9F18	PSHB				;Push result to stack    
	PSHA				;    
	LDX     #L504D			;Throttle follower gain vs. cool temp
	LDAA    L02A6			;Cool temp
	JSR     LF15E			;2D lookup
;
	TSX				;Get current stack pointer     
	JSR     LF1ED			;(A x X*)/256

	PULX				;Clean up stack   
	ASLD				;Result x 2    
	BCS     L9F2E			;Bra if overflow
;
	ASLD				;Result x2, now result /64    
	BCC     L9F31			;Bra if no overflow
;
L9F2E	LDD     #$FFFF			;Clear overflow
;
L9F31	BRSET   L0037,#$01,L9F48	;Bra if b0==1, engine is warm
;
;-Cool engine temps present, apply additional throttle follower gain
;
	PSHB				;Push result to stack   
	PSHA				;    
	TSX				;Get current stack pointer into X     
	LDAA    L5097			;Load added gain
	JSR     LF1ED			;(A x X*)/256
;
	PULX				;Clean up stack   
	ASLD				;x2    
	BCS     L9F45			;Bra if overflow
;
	ASLD				;x2, now /64    
	BCC     L9F48			;Bra if no overflow
;
L9F45	LDD     #$FFFF			;Clear overflow
;
;-Here if warm engine temps present
;
L9F48	BRCLR   L0039,#$80,L9F60	;Status word, br aif b7==0, throttle closed
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Throttle is open, check limits for updating T/F flow
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
	PSHB				;Push throttle follower airflow to stack    
	PSHA				;    	
	PULX				;Get flow into X    
	SUBD    L0880			;Subtract previous flow
	BCC     L9F58			;Bra if added flow increasing
;
	NEGB				;Get abs. value    
	ADCA    #$0000			;
	NEGA				;
;    
L9F58	CMPA    L507D			;Min allowed change in flow to update
	BCS     L9F7B			;Bra if current delta flow is <, no update

;
;-Enough flow to update
;
	XGDX				;Get updated flow back    
	BRA     L9F78			;Bra to save and return			
;
;~~~~~~~~~~~~~~~~~~~~
; Throttle is closed
;~~~~~~~~~~~~~~~~~~~~
;
L9F60	BRSET   L0046,#$08,L9F6D	;Bra if b3==1. DFCO in effect
;
;-Check if manditory post DFCO update in effect
;
	BRCLR   L0039,#$40,L9F6D	;Bra if b6==0, no manditory update
;
;-Manditory update in effect
;
	BCLR    L0039,#$40		;Clear b6, manditory update flag
	BRA     L9F78			;Bra to save and exit
;
L9F6D	LDX     L0880			;Load stored T/F flow
	BEQ     L9F78			;Bra if ==0, save and return
;
	CPD     L0880			;Compare updated flow to stored flow
	BCC     L9F7B			;Bra if flow was rising
;
;-Here to save flow
;
L9F78	STD     L0880			;Added T/F IAC flow
;
;-Here to return w/o updating
;
L9F7B	RTS				;Return   
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~  
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; Initial startup idle handling and idle integrator coordination routine
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
L9F7C	BRSET   L004F,#$80,L9F83	;Status word, bra if b7==1, engine is running
;
;-Engine is off
;
	JMP     LA370			;Bra to skip routine
;
;-Here if engine is running
;
L9F83	JSR     L997F			;Routine to construct desired RPMs
;   
	BRSET   L0009,#$02,L9FAD	;Status word, bra if b1, first run flag for 
					;kickdown logic, ==1 
;
;-First run here
;
	LDAA    L02A7			;MPH
	CMPA    L4EB6			;Upper MPH limit to decay out initial IAC flow
					;(kickdown)
	BLS     L9FAD			;Bra if MPH <=
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Time to start decaying out initial flows (kickdown)
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
	LDAA    L4EB7			;IAC flow vs. cool temp kickdown multiplier
	LDX     #L02C6			;Coupled IAC flow vs. cool temp, P/N
	JSR     LF1ED			;(A x X*)/256
;
	STD     L02C6			;Coupled IAC flow vs. cool temp, P/N
;
	LDAA    L4EB7			;IAC flow vs. cool temp kickdown multiplier
	LDX     #L02D0			;Coupled IAC flow vs. cool temp, drive 
	JSR     LF1ED			;(A x X*)/256
;
	STD     L02D0			;Save it, Coupled IAC flow vs. cool temp, drive  
	BSET    L0009,#$02		;Status word, set b1, first run flag for kickdown
					;logic
;
;-Have been running here
;
L9FAD	LDAA    L0887			;Added IAC flow when in DFCO
	BEQ     L9FCD			;Bra if ==0
;
	BRSET   L0046,#$08,L9FCD	;Status word, bra if b3==1, DFCO in effect
;
;-No DFCO here, decay out the added flow
;
	DEC     L0888			;Load DFCO flow decay rate
	BNE     L9FCD			;Bra if !=, dont decay flow down yet
;
	LDAB    L50B5			;Load decay rate
	STAB    L0888			;Save it, DFCO decay rate
	SUBA    L50B4			;Subtract DFCO flow decay ammount
	BCC     L9FCA			;Bra if no underflow
;
	CLRA				;    
	BCLR    L0038,#$20		;Status word, clear b5, DFCO added flow sign
;
L9FCA	STAA    L0887			;Save DFCO added flow
;
L9FCD	LDAA    L4E85			;IAC option byte
	BITA    #$04			;Test b2, throttle kicker enabled
	BNE     L9FD6			;Bra if ==1, throttle kicker is enabled
;
;-No throttle kicker 
;
	BRA     LA01C			;Bra to continue			
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Throttle kicker option selected
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; 004B, b7==0, T/K enabled
; 0037, b4==1, T/K enabled
;
L9FD6     BRSET   L0037,#$04,L9FDD  	;Status word, bra if b2==1, throttle kicker 
					;is active
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Throttle kicker is inactive
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
	JMP     LA05F			;Go check quals for re-enabling kicker
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Throttle kicker is active here
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
L9FDD	BRSET   L0036,#$0C,L9FE3	;Status word, bra if b2, closed loop quals met, and
					;b3, closed loop idle enabled, ==1
;
;-Still in open loop idle here, no throttle kicker diags.
;
	BRA     L9FEC			;				
;
;-In closed loop idle, update base TPS for diagnostics
;											
L9FE3	LDAA    L0319			;Base TPS for closed throttle
	STAA    L02D4			;Save it, base throttle kicker TPS
	BSET    L0039,#$01		;Status word, set b0, throttle kicker TPS updated
;
;-Check if disable request has been set
;
L9FEC	BRSET   L0037,#$08,LA031	;Status word, bra if b3==1, throttle kicker
					;disable request set 
;
	LDAA    L0006			;Cool temp, linearized
	CMPA    L50BD			;Temp threshold to enable throttle kicker
	BCC     LA000			;Bra if cool temp is >=, throttle kicker will be 
					;active for limited time only
;
;-Cool temp low, allow throttle kicker at all times untill warm
;
	LDAA    L50C2			;Time for throttle kicker to be active
	STAA    L088D			;Save it, throttle kicker timer
	JMP     LA0B1			;Continue with throttle kicker enabled
;
;-Check if disable request has been set
;
LA000	BRSET   L0037,#$10,LA031	;Status word, bra if b4==1, throttle kicker disable
					;request has been set
;
;-Check baro threshold
;
	LDAA    L0317			;Baro
	CMPA    L50C3			;Upper baro threshold for throttle kicker
	BCC     LA031			;Bra if baro >=, check quals for disabling the
					;Throttle kicker
;
;-Check idle RPMs
;
	BRCLR   L0036,#$80,LA01C	;Status word, bra if b7==0, low idle RPMs present
;
;-High idle speed present, check integrator
;
	LDAA    L02CA			;Idle integrator term, A/C off
	CMPA    L50C5			;Integrator threshold for disabling throttle kicker
	BCC     LA01C			;Bra if integrator >=, dont disable the kicker
;
;-Low integrator is present
;
	BRSET   L0036,#$08,LA01F	;Status word, bra if b3==1, closed loop idle enabled
;
;-Here if in open loop idle or low idle is present
;
LA01C	JMP     LA0B4			;Continue with throttle kicker still enabled
;
;-Here if in closed loop idle
;
LA01F	LDAA    L50C7			;Idle RPM error threshold for disabling throttle
					;kicker (drive) 
	BRSET   L0036,#$02,LA029	;Status word, bra if b1==1, in drive
;
;-In P/N
;
	LDAA    L50C8			;Idle RPM error threshold for disabling throttle
					;kicker (park/neutral)
;											
LA029	CMPA    L0858			;Idle RPM error
	BLS     LA031			;Bra if RPM error is less, continue checking quals
;
;-Excessive RPM error is present
;
	BSET    L0037,#$10		;Set b4, throttle kicker disable request
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Here to continue checking quals for disabling throttle kicker
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LA031	BRCLR   L0036,#$40,LA03A	;Status word, bra if b6==0, throttle kicker 
					;has not yet been active
;
;-Throttle kicker active at least once, check timer
;
	LDAA    L088D			;Throttle kicker timer
	BNE     LA053			;Bra if !=0, continue with kicker endabled
;
;-Timer has run out, check quals for keeping kicker enabled
;
LA03A	LDAA    L01FC			;TPS
	CMPA    L50C1			;TPS threshold for throttle kicker (low baro)
	BHI     LA089			;Bra if >, go check baro
;
	LDAA    L02A7			;MPH
	CMPA    L50C0			;MPH threshold for throttle kicker (low baro)
	BHI     LA089			;Bra if >, go check baro
;
;-Low speed/TPS present with nominal baro pressure, keep kicker enabled 
;
	BRSET   L0036,#$40,LA0B1	;Bra if b6==1, kicker has alreay been active
;
	LDAA    L088D			;Throttle kicker timer
	BEQ     LA089			;Bra if ==0, continue checking quals to
					;disable the throttle kicker for diags.
;
;-Here if timer has not expired yet
;
LA053	LDAB    L0002			;Loop counter
	ANDB    #$F0			;Mask upper four bits
	BNE     LA0B1			;Bra if !=0
;
	DECA				;-1
	STAA    L088D			;Save timer
	BRA     LA0B1			;Bra to enable kicker
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Here if throttle kicker isnt active
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LA05F	LDAA    L088D			;Throttle kicker timer
	CMPA    #15			;15 seconds, min time for re-enabling the T/K			
	BLS     LA07D			;Bra if time <=
;
;-Throttle kicker has been inactive for more then 15 seconds
;
	BRSET   L0036,#$0C,LA06C	;Bra if b2, closed loop idle quals met, and
					;b3, closed loop idle enabled, ==1
;
;-In open loop, dont update base TPS for diags.
;
	BRA     LA075			;
; 
;-In closed loop, update base TPS for diags.
;
LA06C	LDAA    L0319			;Base TPS for closed throttle
	STAA    L02D5			;Save it, post throttle kicker base TPS
	BSET    L0039,#$02		;Status word, set b1, post T/K base TPS updated
;
;-Here if in open loop
;
LA075	LDAA    L088D			;Throttle kicker timer
	CMPA    L50CD			;Min time before re-enabling the throttle kicker 
	BEQ     LA089			;Bra if ==, check quals for throttle kicker

LA07D	LDAB    L0002			;Loop counter
	ANDB    #$F0			;Mask upper four bits
	BNE     LA09C			;Bra if !=0 
;
	INCA				;+1, throttle kicker timer
	STAA    L088D			;Throttle kicker time
	BRA     LA09C			;Bra to disable throttle kicker
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Check quals for disabling throttle kicker
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;-Check for low baro
;
LA089	LDAA    L0317			;Filtered baro
	CMPA    L50C4			;Baro threshold for throttle kicker
	BLS     LA0A1			;Bra if baro is <=, continue
;
;-Nominal baro present here, check integrator
;
	LDD     L02CA			;Idle integrator, A/C off
	ADDD    L02D0			;Coupled IAC flow vs. cool temp
	CMPA    L50C6			;Lower IAC flow threshold for enabling kicker
	BHI     LA0A1			;Bra if flow is >, enable T/K
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Here to disable the throttle kicker
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LA09C	JSR     L92F1			;Routine to disable throttle kicker
	BRA     LA0B4			;Bra to continue
;
;-Continue checking quals for throttle kicker, check TPS
;
LA0A1	LDAA    L01FC			;TPS
	CMPA    L50C1			;TPS threshold to enable throttle kicker (low baro)
	BHI     LA0B1			;Bra if TPS >, enable kicker
;
;-Check MPH threshold
;											
	LDAA    L02A7			;MPH
	CMPA    L50C0			;Min speed to enable throttle kicker (low baro)
	BLS     LA0B4			;Bra if MPH <=, dont enable throttle kicker
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Here to enable throttle kicker
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;											
LA0B1	JSR     L9309			;Routine to enable throttle kicker
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Here if throttle kicker not optioned
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LA0B4	LDAA    L02C8			;IAC flow vs. cool temp (park)
	ORAA    L02D2			;IAC flow vs. cool temp (drive)
	BNE     LA0BF			;Bra if both !=0 yet
;
	JMP     LA1AE			;Jump to continue
;
LA0BF	LDAA    L0875			;IAC flow decay delay timer
	BEQ     LA0C7			;Bra if ==0
;                
	DECA				;-1 from timer
	BRA     LA13F			;Bra to save and continue
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Time to start decaying out initial IAC flow
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LA0C7	LDAA    L4EB3			;Decay multiplier for inital IAC flows  
	LDX     #L02C8			;Added IAC flow vs. cool temp (P/N)
	JSR     LF1ED			;(A x X*)/256
;
	TSTA				;Test MSB    
	BNE     LA0D5			;Bra if !=0 yet
;
	CLRA				;Clear flow    
	CLRB				;
;    
LA0D5	STD     L02C8			;Save it, IAC flow vs. cool temp (P/N)
	BEQ     LA0EC			;Bra if ==0, save
;
;-Flow vs. cool temp is not decayed out yet
;
	LDAA    L4EB4			;Decay multiplier (P/N)
	LDX     #L02C6			;Coupled IAC flow vs. cool temp (P/N)
	JSR     LF1ED			;(A x X*)/256
;
	CPD     L02C8			;IAC flow vs. cool temp (P/N)
	BLS     LA0EC			;Bra if flow is <=
;
	LDD     L02C8			;Use IAC flow vs. cool temp instead
;
LA0EC	STD     L02C6			;Save it, IAC flow vs. cool temp (P/N)
;
	LDAA    L4EB3			;Decay multiplier for inital IAC flows
	LDX     #L02D2			;IAC flow vs. cool temp (drive)
	JSR     LF1ED			;(A x X*)/256
;
	TSTA				;Test MSB    
	BNE     LA0FD			;Bra if !=0
;
	CLRA				;Clear flow    
	CLRB				;
;    
LA0FD	STD     L02D2			;Save it, IAC flow vs. cool temp (drive)
	BEQ     LA114			;Bra if ==0
;
	LDAA    L4EB5			;IAC flow decay multiplier (drive)
	LDX     #L02D0			;Coupled IAC flow vs. cool temp (drive)
	JSR     LF1ED			;(A x X*)/256
;
	CPD     L02D2			;IAC flow vs. cool temp (drive)
	BLS     LA114			;Bra if <=
;
	LDD     L02D2			;Use IAC flow vs. cool temp instead
;
LA114	STD     L02D0			;Save it, IAC flow vs. cool temp (drive)
;
	LDAA    L02C8			;IAC flow vs. cool temp (P/N)
	ORAA    L02D2			;OR with IAC flow vs. cool temp (drive)
	BEQ     LA13F			;Bra if ==0, initial flow decayed out
;
;-Initial IAC flows have not yet been decayed out
;
	LDAA    L02A6			;Range limited cool temp
	LDX     #L4ED2			;IAC decay delay period vs. cool temp			
	JSR     LF15E			;2D lookup
;
	PSHA				;Push to stack    
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Construct decay delay period from airflow and multiplier
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
	LDX     #L4EDF			;Decay delay mult. vs. gms/sec airflow
	LDAA    L0255			;Gms/sec airflow
	CMPA    #64			;UL for lookup
	BLS     LA135			;Bra if airflow is less
;
	LDAA    #64			;64 gms/sec, UL for lookup
;
LA135	JSR     LF15E			;2D lookup
;
	PULB				;Get decay delay period    
	MUL				;Decay delay period x multiplier     
	ASLD				;x2, now /128   
	BCC     LA13F			;Bra if no overflow
;
	LDAA    #255			;Clear overflow
;
LA13F	STAA    L0875			;Save it, added IAC flow decay delay period
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Check to make sure that initial IAC flows in P/N, drive are not too far apart
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;-Used during transition from P/N, drive to prevent sudden change in airflow
;
	BRSET   L0036,#$02,LA176	;Status word, bra if b1==1, in drive
;
;-In park/neutral here
;
	LDAA    L02C6			;Coupled IAC flow vs. cool temp (P/N)
	BEQ     LA1AC			;Bra if ==0, flow decayed out
;
;-Check bounds for saving flow, no save if:
;  
;  L02D0 <= L02C6 + allowable percentage
;  L02D0 > L02C6 - allowable percentage
;
	LDAB    L4EE6			;Max allowable percentage difference
	BEQ     LA15B			;Bra if allowable difference ==0
;
	MUL				;Coupled IAC flow vs. cool temp (P/N) x multiplier     
	ADCA    L02C6			;Add in IAC flow, now IAC flow + delta flow
	BCS     LA15B			;Bra if overflow
;
	CMPA    L02D0			;Compare to coupl'd IAC flow vs. cool temp (drive)			
	BCS     LA170			;Bra if <, save flow
;
LA15B	LDAA    L02C6			;IAC flow (P/N)
	LDAB    L4EE7			;Min allowable percentage difference
	BEQ     LA1AC			;Bra if allowable difference ==0
;
	MUL				;IAC flow x multiplier     
	ADCA    #$0000			;round if needed
	TAB				;Get result into B     
	LDAA    L02C6			;IAC flow vs. cool temp (P/N)
	SBA				;Subtract the delta      
	CMPA    L02D0			;Compare to IAC flow in drive
	BCS     LA1A9			;Bra if flow <, no update
;
;-Save here if IAC flow in park is within a +/- percentage of IAC flow in drive   
;
LA170	CLRB				;   
	STD     L0899			;Save it, updated initial IAC flow in drive			
	BRA     LA1A4			;Bra to continue
;
;-Here if in drive
;
LA176	LDAA    L02D0			;IAC flow vs. cool temp (drive)
	BEQ     LA1AC			;Bra if flow is already decayed out
;
;-Check bounds for saving flow, no save if:
;  
;  L02C6 <= L02D0 + allowable percentage
;  L02C6 > L02D0 - allowable percentage
;
	LDAB    L4EE4			;Load max allowable difference
	BEQ     LA18B			;Bra if allowable difference is ==0
;
	MUL				;Coupled IAC flow (drive) x multiplier     
	ADCA    L02D0			;Add in IAC flow to delta
	BCS     LA18B			;Bra if overflow
;
	CMPA    L02C6			;Coupled IAC flow vs. cool temp (P/N)
	BCS     LA1A0			;Bra if flow is less, save
;
LA18B	LDAA    L02D0			;Coupl'd IAC flow (drive)
	LDAB    L4EE5			;Load min allowable difference
	BEQ     LA1AC			;Bra if allowable difference is ==0
;
	MUL				;IAC flow x allowable difference     
	ADCA    #$0000			;Round if needed
	TAB				;Get result into B     
	LDAA    L02D0			;Load coupled IAC flow (drive)
	SBA				;Subtract delta from IAC flow     
	CMPA    L02C6			;Compare to IAC flow vs. cool temp (P/N)
	BCS     LA1A9			;Bra if initial flow in drive is less
;
;-Save here if IAC flow in drive is within a +/- percentage of IAC flow in park  
;
LA1A0	CLRB				;    
	STD     L0893			;Save it, updated initial IAC flow in P/N
;
LA1A4	BSET    L0038,#$10		;Status word, set b4, initial stored IAC flow
					;has been updated		
	BRA     LA203			;Bra to continue
;
LA1A9	BCLR    L0038,#$10		;Clear b4, IAC intial flow not updated
LA1AC	BRA     LA203			;Bra to continue
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Here if initial IAC flow has been decayed out, update steady state integrator cells
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LA1AE	BRSET   L0036,#$0C,LA1B4	;Status word, bra if b2==1, closed loop quals met, 
					;and b3==1, closed loop idle
;
;-Not in full closed loop idle, skip integral update
;
	BRA     LA203
;
;-In closed loop idle
;
LA1B4	LDAA    L0006			;Cool temp, linearized
	CMPA    L4FB5			;Lower temp threshold for steady state idle
					;check
	BCS     LA203			;Bra if cool temp is <
;
	CMPA    L4FB6			;Upper cool temp threshold for steady state idle
					;check
	BHI     LA203			;Bra if cool temp >
;
;-Temp within range, check for steady state idle
;
	LDAA    L0858			;Idle speed RPM error
	CMPA    L4FB4			;Upper RPM error for updating integral cells
	BHI     LA203			;Bra if error is >
;
	LDAA    L0859			;Filtered RPM error
	CMPA    L4FB4 			;Upper RPM error for updating integral cells
	BHI     LA203			;Bra if rpm error >
;
;-RPM error within limits
;
	BCLR    L0018,#$08		;Error word #3, clear b3, err 35, IAC error 
	BRSET   L0036,#$02,LA1EB	;Status word, bra if b2==1, in drive
;
;-In park/nutral
;
	BRSET   L0036,#$01,LA1E3	;Status word, bra if b0==1, A/C is on
;
;-A/C off
;
	LDAA    L02C0			;Current idle integral, P/N, A/C off
	STAA    L02C2			;Save it, steady state idle integral, P/N, A/C off
	BRA     LA203			;Bra to continue
;
;-A/C is on
;
LA1E3	LDAA    L02C3			;Current idle integral, P/N, A/C on
	STAA    L02C5			;Save it, steady state idle integral, P/N, A/C on
	BRA     LA203			;Bra to continue
;
;-In drive here
;
LA1EB	BRSET   L0036,#$01,LA1FA	;Status word, bra if b0==1, A/C is on
;
;-A/C is off
;
	LDAA    L02CA			;Current idle integral, drive, A/C off
	STAA    L02CC			;Save it, steady state idle integral, drive,
					;A/C off 
	BSET    L0009,#$10		;Status word, set b4, stable idle attained, A/C off
	BRA     LA203			;Bra to continue
;
;-A/C is on here
;
LA1FA	LDAA    L02CD			;Current idle integral, drive, A/C on
	STAA    L02CF			;Save it, steady state idle integral, drive,
					;A/C on 
	BSET    L0009,#$20		;Status word, set b4, stable idle attained, A/C on
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Here if initial IAC flows not updated, check for A/C transition
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LA203	LDAA    L0876			;Added IAC flow for A/C transition
	BEQ     LA22A			;Bra if ==0
;
	LDAB    L0877			;A/C on/off transition timer
	BNE     LA22A			;Bra if !=0, still in transition
;
;-A/C on/off transition complete
;
	BRCLR   L0036,#$01,LA226	;Status word, bra if b0==0, A/C is off
;
	LDAB    L0878			;A/C transition flow decay period
	BEQ     LA21B			;Bra if ==0, decay out flow this pass
;
;-Not time to decay out flow yet
;
	DEC     L0878			;-1 from decay period
	BNE     LA22A			;Bra if !=0, dave and continue
;
;-A/C has been off, time to decay out the flow
;
LA21B	LDAB    L4F18			;Decay period
	STAB    L0878			;Save it, A/C transition decay period
	SUBA    L4F17			;IAC flow decay rate for A/C transition
	BCC     LA227			;Bra if no underflow
;
;-A/C is off, clear out flow immediatly
;
LA226	CLRA				;Clear out added flow 
;  
LA227	STAA    L0876			;Added IAC flow for A/C transition				
;
;-Here if no delta IAC flow or not decaying out flow yet
;
LA22A	LDAB    L0036			;Status word
	BITB    #$0008			;Test b3
	BEQ     LA238			;Bra if ==0, closed loop idle not enabled
;
;-Closed loop idle has been enabled
;
	BITB    #$10			;Test b4, stall saver
	BNE     LA238			;Bra if stall saver it active
;
	BRCLR   L003E,#$04,LA23B	;Bra if ==0, batt volts within range
;
;-In open loop, stall saver active, or battery volts are out of range for IAC operation
;
LA238	JMP     LA35E
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Check to make sure P/N and drive integrater terms arnt getting too far apart
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LA23B	CLRB    
	BRSET   L0036,#$01,LA249	;Status word, bra if b0==1, A/C on
;
;-A/C off
;
	LDX     #L02C0			;Idle integrator term, P/N, no A/C
	LDY     #L088F			;Start of updated integrator flows, no A/C
	BRA     LA250			;Bra to continue
;
;-A/C on
;
LA249	LDX     #L02C3			;Idle integrator term, P/N, A/C
	LDY     #L0891			;Start of updated integrator flows, A/C			
;
LA250	LDAA    $00,X			;Integrator term P/N
	SUBA    $0A,X			;Subtract integrator term for drive
	BCS     LA273			;Bra if term for P/N was <
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Integrator term for P/N is larger then drive
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
	CMPA    L4FBC			;Min flow difference to update integrators
	BLS     LA291			;Bra if delta flow is <=
;
;-Enough difference, close the gap between the integrator terms
;
	BRSET   L0036,#$02,LA269	;Bra if b1==1, in drive
;
;-In park/neutral
;
	LDAA    0,X			;P/N integrator term
	SUBA    L4FBC			;Subtract allowable difference
	STD     6,Y			;Save it, updated integral flow for drive 
	BRA     LA2B1			;Bra to contiue
;
;-In drive
;
LA269	LDAA    $0A,X			;Drive integrator term
	ADDA    L4FBC			;Add in the allowable difference
	STD     0,Y			;Save it, updated integral flow for drive
	BRA     LA2B1			;Bra to continue
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Integrator term for drive is larger then P/N
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LA273	NEGA				;Get abs. value of result    
	CMPA    L4FBB			;Max allowable difference
	BLS     LA291			;Bra if <=
;
;-Enough difference, close the gap between the integrator terms
;
	BRSET   L0036,#$02,LA287	;Bra if b2==1, in drive
;
;-In park/neutral
;
	LDAA    0,X			;Load idle integrator term, P/N
	ADDA    L4FBB			;Add in allowable flow difference
	STD     6,Y			;Save it, updated integral flow for drive
	BRA     LA2B1			;Bra to continue
;
;-In drive
;
LA287	LDAA    $0A,X			;Load idle integral term, drive
	SUBA    L4FBB			;Subtract allowable difference
	STD     0,Y			;Save it, updated integral flow for drive
	BRA     LA2B1			;Bra to continue
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Integral terms are within limits here, clear flags
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LA291	BRSET   L0036,#$02,LA2A3	;Bra if b1==1, in drive
;
	BRSET   L0036,#$01,LA29E	;Bra if b0==1, A/C on
;
;-In P/N, no A/C
;
	BCLR    L0038,#$04		;Clear update integral term flag, drive, no A/C
	BRA     LA2CF			;
;
;-In P/N, A/C on
;
LA29E	BCLR    L0038,#$08		;Clear update integral term flag, drive, A/C on
	BRA     LA2CF			;
;
;-Here if in drive
;
LA2A3	BRSET   L0036,#$01,LA2AC	;Bra if b0==1, A/C on
;
;-In drive, A/C off
;
	BCLR    L0038,#$01		;Clear update integral term flag, P/N, no A/C		
	BRA     LA2CF			;
;
;-In drive, A/C on
;
LA2AC	BCLR    L0038,#$02		;Clear update integral term flag, P/N, A/C on
	BRA     LA2CF			;
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Here if idle integral terms where out of limits and adjusted, set flags
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LA2B1	BRSET   L0036,#$02,LA2C3	;Bra if b1==1, in drive
;
	BRSET   L0036,#$01,LA2BE	;Bra if b0==1, A/C on
;
;-In P/N, no A/C
;
	BSET    L0038,#$04		;Set update integral term flag, drive, no A/C		
	BRA     LA2CF			;
;
;-In P/N, A/C on
;
LA2BE	BSET    L0038,#$08		;Set update integral term flag, drive, A/C on
	BRA     LA2CF			;
;
;-Here if in drive
;
LA2C3	BRSET   L0036,#$01,LA2CC	;Bra if b0==1, A/C on
;
;-In drive, A/C off
;
	BSET    L0038,#$01		;Set update integral term flag, P/N, no A/C
	BRA     LA2CF			;
;
;-In drive, A/C on
;
LA2CC	BSET    L0038,#$02		;Clear update integral term flag, P/N, A/C on
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Check to make sure A/C and non-A/C integral terms arnt getting too far aprart
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LA2CF	BRSET   L0036,#$02,LA2E0	;Status word, bra if b1==1, in drive
;
;-In park/neutral 
;
	LDX     #L088F			;Updated integral term, P/N, no A/C
	PSHX				;To stack
	LDX     #L02C0			;Idle integral term, P/N, no A/C
	LDY     #L4FB7			;Load min allowable difference, P/N
	BRA     LA2EB			;			
;
;-In drive here
;
LA2E0	LDX     #L0895			;Updated integral term, drive no A/C
	PSHX				;
	LDX     #L02CA			;Idle integrator term, drive, no A/C
	LDY     #L4FB9			;Load min allowable flow difference
											
LA2EB	LDAB    0,Y			;Load allowable flow difference
	PSHB				;To stack
	LDD     3,X			;Load A/C integral term
	SUBD    0,X			;Subtract integral term, no A/C
	PULB				;Get allowable difference back
	BLS     LA2FF			;Bra if non-A/C integral was >=, update flow
;
;-Non-A/C integral term <, check lower limit for difference
;
	CBA				;Compare delta integral flow to difference
	BCS     LA2FF			;Bra if difference is less, update 
;
;-Check upper limit for integral flow difference
;
	LDAB    1,Y			;Load max allowed flow difference
	CBA				;Compare delta flow to difference
	BLS     LA31F			;Bra if delta flow was <= allowable difference
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; A/C and non-A/C integral terms have become too far apart, update flows
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LA2FF	BRSET   L0036,#$01,LA313	;Status word, bra if b0==1, A/C on
;
;-A/C off
;
	LDAA    0,X			;Load integrator term, no A/C
	ABA				;Add in allowable difference
	BCC     LA30D			;Bra if no overflow
;										
	LDD     #$FFFF			;Clear overflow
	BRA     LA30E			;				
;											
LA30D	CLRB				;Clear MSB
;	
LA30E	PULX				;Get addr. of updated integral terms back
	STD     2,X			;Save it, updated integral term, A/C
	BRA     LA340			;Bra to continue
;
;-A/C is on
;
LA313	LDAA    3,X			;Load integral term, A/C
	SBA				;Subtract out the allowable difference     
	BCC     LA319			;Bra if no underflow
;
	CLRA				;Clear underflow
;
LA319	CLRB				;Clear LSB    
	PULX				;Get addr. of updated integral terms back   
	STD     0,X			;Save it, updated integral term, no A/C
	BRA     LA340			;Bra to continue
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Here if A/C and non-A/C integral terms where not too far apart
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LA31F	PULX				;Get addr. back off stack
	BRSET   L0036,#$02,LA332	;Bra if b1==1, in drive
;
	BRSET   L0036,#$01,LA32D	;Bra if b0==1, A/C is on
;
;-In P/N, no A/C
;
	BCLR    L0038,#$02		;Clear update integral term flag, P/N, A/C on 
	BRA     LA35E			;
;
;-In P/N, A/C on
;
LA32D	BCLR    L0038,#$01		;Clear update integral term flag, P/N, no A/C
	BRA     LA35E			;
;
;-In drive here
;
LA332	BRSET   L0036,#$01,LA33B	;Bra if b0==1, A/C on 
;
;-In drive, no A/C
;											
	BCLR    L0038,#$08		;Clear update integral term flag, drive, A/C
	BRA     LA35E			;
;
;-In drive, A/C on
;
LA33B	BCLR    L0038,#$04		;Clear update integral term flag, drive, no A/C
	BRA     LA35E			;
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Here if A/C and non-A/C integral terms where too far apart
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LA340	BRSET   L0036,#$02,LA352	;Status word, bra if b1==1, in drive
;
	BRSET   L0036,#$01,LA34D	;Bra if b0==1, A/C on
;
;-In P/N, no A/C
;
	BSET    L0038,#$02		;Set b1, update integral term flag, P/N, no A/C
	BRA     LA35E			;
;
;-In P/N, A/C on
;
LA34D	BSET    L0038,#$01		;Set set b0, update integral term flag, P/N, A/C on
	BRA     LA35E			;
;
;-In drive
;
LA352	BRSET   L0036,#$01,LA35B	;Bra if b0==1, A/C is on 
;
;-In drive, A/C off
;
	BSET    L0038,#$08		;Set b3, update integral term flag, drive, A/C
	BRA     LA35E			;
;
;-In drive, A/C on
;
LA35B	BSET    L0038,#$04		;Set b2, update integral term flag, drive, no A/C
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Decrement closed loop delay terms
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LA35E	LDAA    L0866			;Closed loop idle delay, low idle
	BEQ     LA367			;Bra if already ==0
;
	DECA				;-1 from timer    
	STAA    L0866			;Save it
;
LA367	LDAA    L0867			;Closed loop idle delay, high idle
	BEQ     LA370			;Bra if already ==0
;
	DECA				;-1 from timer 
	STAA    L0867			;Save it
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Update stored steady state integral term
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LA370	LDX     #L02CA			;Start of idle integrator terms, drive
	BRSET   L0036,#$02,LA37A	;Status word, bra if b1==1, in drive
;
;-In park/neutral
;
	LDX     #L02C0			;Start of idle integrator terms, P/N
;
LA37A	LDAA    2,X			;Load steady state integral term, no A/C
;
	BRCLR   L0036,#$01,LA382	;Bra if b0==0, A/C not currently active
;
;-A/C on 
;
	LDAA    5,X			;Load steady state integral term, A/C on
;
LA382	STAA    L086A			;Stored steady state idle integral term
	RTS				;Return     
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; Routine to decay out various idle terms
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LA386	BRSET   L004F,#$80,LA38D	;Status word, bra if b7==1, engine is running
;
;-Engine not running
;
	JMP     LA4DC			;Return
;
;-Engine is running here
;
LA38D	JSR     L9DE0			;Throttle follower routine
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Decay out added P/S IAC flow
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
	LDAA    L087B			;Load P/S compensation IAC flow
	BEQ     LA3B0			;Bra if ==0, flow already decayed out
;
;-Flow not yet decayed out
;
	LDAB    L087C			;P/S compensation flow decay period
	BEQ     LA39F			;Bra if ==0, decay out the flow
;
	DEC     L087C			;Decr. decay period
	BNE     LA3B0			;Bra if !=0, dont decay flow yet
;
;-Time to decay the flow
;
LA39F	SUBA    L4F27			;Subtract P/S compensation flow decay rate
	BCC     LA3A5			;Bra if no underflow
;
	CLRA				;Clear underflow
;   
LA3A5	STAA    L087B			;Save it, P/S compensation flow
	BEQ     LA3AD			;Bra if flow ==0
;
;-Reset decay period timer
;
	LDAA    L4F26			;Decay period for P/S compensation flow
;
LA3AD	STAA    L087C			;Save decay period
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Decay out initial IAC flow
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LA3B0	LDAA    L086B			;Initial flow added after startup
	BEQ     LA3D0			;Bra if ==0
;
	BRCLR   L003F,#$40,LA3BC	;Status word, bra if b6==0, not clearing initial
					;idle terms
;
;-Here to clear intial added IAC flow
;
	CLRA				;Clear flow    	
	BRA     LA3CD			;Bra to save	 
;
;-Check if time to decay out flow
;
LA3BC	LDAB    L086C			;Decay period for initial added IAC airflow
	BEQ     LA3C6			;Bra if ==0, dec flow
;
;-Decay period not yet ==0
;
	DEC     L086C			;-1, IAC flow decay period
	BNE     LA3D0			;Bra if !=0
;
;-Here to decay the flow
;
LA3C6	LDAB    L4EAF  			;Initial added flow decay period
	STAB    L086C			;Save it
;
	DECA				;-1, initial added IAC flow
;
LA3CD	STAA    L086B			;Save initial IAC flow
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Decay out added desired idle speed
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LA3D0	LDAA    L085C			;Load added desired idle speed
	BEQ     LA400			;Bra if already decayed out
;
;-Still additional idle speed to be decayed out
;
	BRCLR   L003F,#$40,LA3DC	;Bra if b6==0, not clearing initial idle terms
;
;-Here to clear added desired idle speed
;
	CLRA				;Clear added desired idle speed   
	BRA     LA3FD			;Bra to save
;
LA3DC	LDAB    L085E			;Timer to begin decaying out init. added idle speed
	BEQ     LA3EC			;Bra if ==0, decay out idle speed
;
	LDAB    L0002			;Loop counter
	BITB    #$F0			;Test upper four bits
	BNE     LA400			;Bra if !=, not time to decrement timer
;
	DEC     L085E			;Dec timer
	BRA     LA400			;Bra to continue
;
;-Enough time has passed to begin decaying out added desired idle speed
;
LA3EC	LDAB    L085D			;Decay period for added idle speed
	BEQ     LA3F6			;Bra if ==0, decay out added desired idle speed
;
;-Not decaying out added idle speed yet
;
	DEC     L085D			;Added desired idle speed decay period
	BNE     LA400			;Bra if !=0, dont decay idle speed yet
;
;-Decay out added desired idle speed
;
LA3F6	LDAB    L4EB1			;Decay period for added desired idle speed
	STAB    L085D			;Save it
;
	DECA				;Decr. added initial idle speed
;
LA3FD	STAA    L085C			;Save it
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Limit current integral if too far away from steady state integral
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;-Helps limit integrator wind-up if selected
;
LA400	LDAA    L0036			;Status word
	BITA    #$02			;Test b1, P/N, drive flag 
	BEQ     LA432			;Bra if ==0, in P/N 
;
;-In drive
;
	BITA    #$0C			;Test b2, closed loop idle quals met, and b3, 
;					;in closed loop idle
	BNE     LA432			;Bra if in closed loop or closed loop idle pending
;
;-No closed loop idle pending, check integral terms
;
	BITA    #$01			;Test b0, A/C on flag 
	BNE     LA421			;Bra if b0==1, A/C is on
;
;-A/C off
;			
	LDAA    L02CC			;Steady state idle integral, drive, A/C off
	ADDA    L4FBD			;Add in difference in flow limit for current
					;integral
	BCS     LA432			;Bra if overflow
;											
	CMPA    L02CA			;Compare to current idle integral, drive, no A/C
	BCC     LA432			;Bra if integral is <=
;
;-Current integral is greater then allowed
;										
	CLRB				;Clear LSB    
	STD     L02CA			;Save upper limit instead
	BRA     LA432			;Continue
;
;-A/C is on here
;
LA421	LDAA    L02CF			;Steady state idle integral, drive, A/C on
	ADDA    L4FBE			;Add in difference in flow limit for current 
					;integral
	BCS     LA432			;Bra if overflow
;											
	CMPA    L02CD			;Compare to current idle integral, drive, A/C on			
	BCC     LA432			;Bra if integral is <=
;
;-Current integral is greater then allowed
;										
	CLRB				;Clear LSB    
	STD     L02CD			;Save upper limit instead
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Perform throttle kicker diagnostics
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LA432	LDAA    L4E85			;IAC option byte
	BITA    #$04			;Test b2, throttle kicker enabled flag
	BEQ     LA46E			;Bra if ==0, no throttle kicker in use
;
;-Throttle kicker is in use
;
	LDAA    L5B02			;Error mask 3
	BITA    #$04			;Test b2, will be flagging err 36,
					;throttle kicker actuator faulure
	BEQ     LA46E			;Bra if ==0, not flagging error
;
;-Erorr 36 enabled
;
	BRSET   L0039,#$04,LA46E	;Bra if thottle kicker diags. performed already
;
	BRSET   L0039,#$03,LA44A	;Bra if stored base closed throttle TPS
					;readings have been updated
;
	BRA     LA46E			;No update yet, skip diagnostics
;
;-Check base closed throttle positions before and after kicker to see if actuator ok
;
LA44A	LDAA    L02D4			;Throttle kicker base TPS
	SUBA    L02D5			;-initial base TPS
	BCS     LA457			;Bra if initial base TPS <
;
;-Check if kicker exteding far enough
;
	CMPA    L50CC			;Min TPS difference between readings
	BCC     LA465			;Bra if >=, dont set error
;
;-Kicker isnt functional or not extending far enough, set error
;
LA457	BRSET   L0009,#$40,LA460	;Status word, bra if b6==1 kicker has already failed

	BSET    L0009,#$40		;Status word, set b6, throttle kicker failed diags.
	BRA     LA46B			;Bra to continue
;
;-Here to set error
;
LA460	BSET    L0018,#$04		;Error word #3, set b2, error 36, throttle kicker
	BRA     LA46B			;Bra to coninue
;
;-Here if diags passed
;
LA465	BCLR    L0018,#$04		;Clear b2, throttle kicker ok
	BCLR    L0009,#$40		;Clear b6, throttle kicker ok
;
LA46B	BSET    L0039,#$04		;Set b2, throttle kicker actuator diags. performed
;
;-Alternate passes for next portion of code
;
LA46E	BRSET   L0002,#$20,LA4DC	;Loop counter, bra if b5==1, dont run this pass
;
;~~~~~~~~~~~~~~~~~~~~
; Perform IAC diags.
;~~~~~~~~~~~~~~~~~~~~
;
	LDAA    L5B02			;Error mask #3
	BITA    #$08			;Test b3, error 35, IAC fault
	BEQ     LA4BC			;Bra if ==0, not setting err. 35
;
;-Error 35 being recognized, check quals for setting error
;
	BRSET   L0016,#$10,LA4B6	;Error word #1, bra if b4==1, error 16, VSS error
;
	BRSET   L0016,#$01,LA4B6	;Bra if b0==1, error 21, high TPS error
;
	BRSET   L0017,#$80,LA4B6	;Error word #2, bra if b7==1, err 22, low TPS
;
	BRSET   L003E,#$04,LA4B6	;Status word, bra if b2==1, low batt volts
;
;-Check for closed loop idle
;
	BRCLR   L0036,#$08,LA4B6	;Status word, bra if b3==0, no closed loop idle
;
;-In closed loop idle
;
	BRSET   L0009,#$40,LA4B6	;Status word, bra if b6==1, throttle kicker
					;failed diags. 
;
	LDX     L02D0			;Load coupled IAC flow vs. cool temp, drive
	BNE     LA4B6			;Bra if !=0, dont perform diags.
;	
	LDX     L02C6			;Load coupled IAC flow vs. cool temp, P/N
	BNE     LA4B6			;Bra if !=0, dont perform diags.
;
;-All quals met, perform IAC diags.
;
	LDAB    L0859			;Filtered idle RPM error
	CMPB    L50B6			;Sustained RPM error threshold to set err 35, 
					;IAC fault
;							
	BLS     LA4B6			;Bra if RPM error less, dont set error
;
;-Large RPM error present, check if enough time has passed to set error
;
	LDAB    L088E			;Error 35 timer
	CMPB    L50B7			;Error 35 timer threshold	
	BHI     LA4B1			;Bra if >, set error 35
;
;-Not enough time has passed yet
;
	INCB				;+1
	STAB    L088E			;Save err 35 timer
	BRA     LA4BC			;Bra to continue
;
;-RPM error has been sustained for long enough to set error
;
LA4B1	BSET    L0018,#$08		;Error word #3, set b3, error 35, IAC fault
	BRA     LA4BC			;Bra to continue
;
;-Here if quals not met/not setting error
;
LA4B6	CLR     L088E			;Clar err 35 timer
	BCLR    L0018,#$08		;Clear b3, error 35, IAC fault		
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Check if IAC reset needed
;~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LA4BC	BRSET   L0009,#$01,LA4DC	;Status word, bra if b0==1, IAC reset in progress
;
	BRSET   L0009,#$04,LA4DC	;Bra if b2==1, IAC reset not requested
;
;-IAC reset has been requested, check quals for reset
;
	BRSET   L0044,#$10,LA4D0	;Status word, bra if b4==1, ign. off, perform reset
;
;-Check VSS limit for reset
;
	LDAA    L02A7			;MPH
	CMPA    L4EAD			;Lower MPH limit to perform IAC reset
	BCS     LA4DC			;Bra if MPH <, no reset this pass
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Entry point to close IAC for reset
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LA4D0	BSET    L0009,#$01		;Status word, set b0, IAC reset in progress
	BCLR    L0036,#$1C		;Status word, clear b2, closed loop idle quals met,
					;b3, closed loop on RPM enabled, and b4, stall 
					;saver in effect
	CLRA				;Preclear A   
	STAA    L0008			;Clear desired IAC steps
	DECA				;-1, now 255    
	STAA    L0007			;Save as current IAC position, force IAC to
					;rapidly close and seat
;
;-Here if engine off, return
;
LA4DC	RTS				;Return     
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;~~~~~~~~~~~~~~~~~~
;
; ALDL IAC routine
;
;~~~~~~~~~~~~~~~~~~
;
LA4DD	BRSET   L0044,#$10,LA53B	;Status word, bra if b4==1, ignition is off
;
;-Ignition is on here
;
	BRSET   L0009,#$01,LA53B	;Status word, bra if b0==1, IAC reset in progress
;
	LDAB    L0393			;ALDL status word word (diag. data buffer)
	BITB    #$20			;Test b5, reset IAC
	BEQ     LA4F0			;Bra if ==0, not resetting IAC
;
	PULX				;    
	JMP     LA4D0			;Continue with reset
;
;
;
;
;                                  
LA4F0	LDAB    L0397			; ALDL CNT'L WD 7
	ANDB    #$01			; 0000 0001
;
	LDAA    L0390+1			; $F4, ALDL MODE 4, DISRETE ENABLES
	ANDA    #$40			; b6, ??? 
	ABA				;     
	BEQ     LA53B			;
;
	CMPA    #$41			;
	BEQ     LA53B			;
;
	BITA    #$01			;
	BEQ     LA521			;
;
	LDAA    L0397			; ALDL CNT'L WD 7
	BITA    #$02			; b1
	BNE     LA516			; BR IF b1
;
; DIACMW2, NON-VOL IDLE CNT'L MD WD
;
	BCLR    L0036,#$0C		; CLR $0C, DIAG MD WD #3
;
	PULX				;    
	LDAA    L0398			; ALDL CNT'L WD 8
	JMP     L997C			;
;
LA516	LDAA    L0398			; ALDL CNT'L WD 8
	STAA    L0857			; DESIRED IDLE RPM/12.5
;
	BSET    L0038,#$40		;
	BRA     LA53B			;
;
; CK ALDL CW 3
;
LA521	PULX				;    
	LDAA    L0392			; (ALDL CNT'L WD 2)
	BITA    #$40			; b6,
	BEQ     LA532			;
;
	JSR     L9305			;
;
;---------------------------
; IAC DIAG MD WD 3	
; 	b2 1 = THROTTLE KICKER ACTIVE    
;----------------------------- 
	BSET    L0037,#$04		; SET b2, THROTTLE KICKER ACTIVE
;
	JMP     L997E			;
;
LA532	JSR     L931F			;Energize throttle kicker
;---------------------------
; IAC DIAG MD WD 3	
; 	b2 1 = THROTTLE KICKER ACTIVE    
;----------------------------- 
;
	BCLR    L0037,#$04		;Status word, clear b2, throttle kicker active
	JMP     L997E			;Jump to continue
;
LA53B	RTS				;Return     
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; Spark minor loop 
;
; Run 80x a second
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;-Calculate vacuum
;
LA53C	LDAB    #151			;
;
	LDAA    L0317			;Load filtered baro
	SUBA    L01E9			;Subtract A/D MAP
	BHI     LA548			;Bra if Baro > MAP
;
;-MAP >= Baro
;
	LDAA    #$01			;Clear underflow
;
;-MAP < Baro
;
LA548	MUL				;(A/D Baro - A/D MAP) x 151     
	ADDD    #64			;+64 to round up if needed
	ASLD 				;x2   
	BCC     LA552			;Bra if overflow
;
	LDD     #$FFFF			;Clear overflow
;
LA552	NEGA				;change sign  
	STAA    L01EC			;Store it, vacuum = 256 - 1.18(Delta A/D counts) 
;
;-Calc. S/D MAP
;
	LDAB    #151			;Multiplier for S/D MAP
	LDAA    L01E9			;A/D MAP
	SUBA    #26			;
	BCC     LA562			;Bra if underflow
;
	CLRA 				;Clear underflow  
	BRA     LA56C			;Bra to save
;
LA562	MUL				;151 x (A/D MAP - 64)     
	ADDD    #64			;+64 to round up if needed
	ASLD 				;x2   
	BCC     LA56C			;Bra if overflow

	LDD     #$FFFF			;Clear overflow

LA56C	STAA    L01E3			;S/D map
;
;-Calculate ratio of MAP kPa to baro kPa
;
	LDAA    L01F1			;baro factor x 256 for MAP/baro ratio
	CLRB				;Preclear B   
	LDX     L027A			;MAP kPa x 256
	XGDX				;Get baro factor into X and MAP into D  
	FDIV				;Fract. divide   
	XGDX				;Get result into D    
	SUBD    #8192			;-8192d
	BCC     LA581             	;Bra if no underflow
;
	LDD     #$0000			;Clear underflow
;
LA581	ASLD				;x2    
	BCC     LA586			;Bra if overflow
;
	LDAA    #255			;Clear overflow
;
;-Now = 256 x 1.25 x [MAP kPa/(baro kPa - 5)] - 64
;
LA586	STAA    L01F2			;MAP/baro ratio
;
;~~~~~~~~~~~~
; MAP diags.
;~~~~~~~~~~~~
;
;-Perform low MAP test
;
	BCLR    L0018,#$10		;Error word #3, clear b4, err 34, MAP sensor low
;
	BCLR    L006F,#$08		;Status word, clear b3, low MAP 
;
	BRSET   L006F,#$04,LA5B7	;
;
	LDAA    L082E              	;Load A/D MAP
	CMPA    L4E5A              	;Lower MAP limit to set Err. 34
	BCC     LA5B7			;Bra if MAP >=
;
;-Low MAP present, check quals. to set err 34
;
	LDAA    L024C              	;Err. 34 timer
	CMPA    L4E5C			;Timer threshold to set err
	BHI     LA5BC			;Bra if >
;
	LDAA    L0061			;Scaled RPMs
	CMPA    L4E5B              	;1200 RPM, upper limit for err 34
	BCS     LA5B2			;Bra if RPMs less
;
	LDAA    L01FC			;%TPS
	CMPA    L4E5D			;Lower %TPS limit for err 34 
	BLS     LA5B7			;Bra if TPS <=, no err 34
;
;-Quals. met for err 34, increment timer
;
LA5B2	INC     L024C			;+1, err 34 timer
	BRA     LA5C2			;Bra
;
;-Quals. not met for err 34, bra to continue
;
LA5B7	CLR     L024C			;
	BRA     LA5C2			;
;
;-Err 34 timer elapsed, set err 34, MAP sensor low
;
LA5BC	BSET    L0018,#$10		;Err. word #3, set b4, err 34, MAP low
	BSET    L006F,#$08		;Status word, set b3, MAP low 
;
;-Here if not setting error
;
LA5C2	BRCLR   L0050,#$20,LA613	;Status word, bra if b5==0, no DRP yet 
;
;-DRP occured here
;
	BCLR    L0050,#$20		;Clear b5 in prep. for next DRP
	BSET    L0050,#$04		;Set b2, DRP occured, 6.25 msec test
	CLRA				;    
	CLRB				;
	STD     L0283			;Time since last DRP
;
;-Decay out dynamic dwell
;
	LDD     L020D			;Dwell
	LSRD				;/2    
	LSRD				;/2   
	LSRD				;/2    
	COMA				;1's comp.    
	COMB				;
	ADDD    L020D			;Add in negative dwell/8
	BPL     LA5E0			;Bra if result is +
;
	CLRA				;Clear out dwell   
	CLRB				;
;
LA5E0	STD     L020D			;Save dwell
	CLR     L0225			;Clear counter, DRP occured
	BRCLR   L004F,#$80,LA616	;Status word, bra if b7==0, engine not running
;
;~~~~~~~~~~~~~~~~~~~~~
; Engine running here
;~~~~~~~~~~~~~~~~~~~~~
;
	BRSET   L004F,#$10,LA5F1	;Status word, bra if engine above threshold for cranking
;
	JSR     LA657			;Engine running/Cranking status routine
;
LA5F1	BRCLR   L004F,#$40,LA655	;Bra if b6==0, EST bypass diag. not yet passed 
;
	BRSET   L0051,#$40,LA655	;Status word, bra if b6==1, all SA is ramped in 
;
;-Bring in SA after startup
;
	LDAB    L414D			;Filter coeff. for SA
	LDX     L0215			;SA ramping multiplier
	CPX     #65264			;Upper value for multipluer
	BHI     LA60E			;Bra if higher, all SA is ramped in
;
	LDAA    #255			;.996, multipler to filter to
	JSR     LF0F6			;Lag filter
	STD     L0215			;Save it
	BRA     LA655			;Bra to continue


LA60E	BSET    L0051,#$40		;Status word, set b6, all SA ramped in 
	BRA     LA655			;
;
LA613	JMP     LA671			;
;
;~~~~~~~~~~~~~~~~~~~~~~~~~
;Engine off/cranking here
;~~~~~~~~~~~~~~~~~~~~~~~~~
;   
LA616	BRSET   L0044,#$08,LA620	;Bra if first valid DRP already recieved
;
	BSET    L0044,#$08		;Set b3, first valid DRP revieved
	JMP     LAD0C			;RPM calculation routine
;
LA620	BRSET   L004F,#$10,LA627  	;Status word, bra if b4==1, engine transitioning to run
;
	JSR     LA657			;Engine running/cranking status routine
;
;-See if RPMs high enough for engine running
;
LA627	LDD     L3FC0			;DRP
	CPD     L4134			;DRP threshold for engine to be running 
	BCS     LA636			;Bra if DRP <, rpms higher then threshold for run
;
;-Engine still cranking/off here
;
	CLR     L0233			;Clear DRP counter, used for setting engine running flag
	JMP     LAD0C			;
;
;-Engine running here
;
LA636	INC     L0233			;+1, 
	LDAA    L0233			;DRP counter, used for setting engine running flag
	CMPA    L4143			;# of counts for engine to be running 
	BCC     LA644			;Bra if >=, set engine running flag and continue
;
;-Not enough DRPs for engine to be running yet
;
	JMP     LAD0C			;RPM calc. routine
;
;-Engine is now running, perform resets
;
LA644	BSET    L004F,#$80		;Status word, set b7, engine is now running
	CLR     L0233			;Clear DRP counter
	BSET    L0004,#$08		;Set b3, bad shutdown
	BCLR    L0051,#$40		;Status word, clear b6, ramp in SA
	CLRA				;
	CLRB				;
	STD     L0215			;SA ramping multiplier
;
LA655	BRA     LA6BB			;Bra to calc rpms
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; Engine running/Cranking status routine
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LA657	LDD     L3FC0			;DRP counter
	CPD     L4136			;DRP threshold for engine to be running
	BCC     LA66C			;Bra if DRP >=
;
;-Engine RPMs above cranking threshold
;
	LDAA    L0234			;Transition from crank to run delay DRP counter
	INCA				;+1
	CMPA    L4144			;Threshold to transition from crank to run
	BCS     LA66D			;Bra if # of DRPs is less
;
;-Engine now running...
;
	BSET    L004F,#$10		;Set b4, engine now transitioning to run
;
LA66C	CLRA				;Clear transition from crank to run DRP counter
LA66D	STAA    L0234			;Save it
	RTS				;Return
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Here if no valid DRPs recieved
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LA671	LDX     L0283			;Time since last DRP
	CPX     L4958			;Max time to run pump with no DRPs
	BCC     LA67F			;Bra if >=, shut down the pump

	INX				;+1
	STX     L0283			;Save the timer
	BRA     LA683			;Bra to continue
;
;-Timer elapsed, shut down the pump
;
LA67F	CLRA 				;   
	STAA    L306F			;Hardware, fuel pump relay control
;
;-Here if not shutting down fuel pump
;
LA683	CPX     #0160			;Max time window for next DRP
	BCS     LA694			;Bra if timer <
;
;-Assume engine has stalled/failed to start 
;
	SEI				;Disable interrupts
	LDAA    L4D8C			;Value to init cranking DRP counter to
	CMPA    L02F1			;Compare to cranking DRP counter
	BCC     LA694			;Bra if <=
;
	STAA    L02F1			;Save value as cranking DRP counter
;
LA694	CLI				;Re-enable interrups
	LDAA    L0225			;DRP period counter
	CMPA    #23			;.287 sec., max period for engine to be running
	BCS     LA6B0			;Bra if <=
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;DRPs outside of window here
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
	BCLR    L004F,#$90		;Clear b7=engine running, b4=now transitioning to run
	BCLR    L0044,#$08		;Clear b3, 1st DRP valid 
;
	LDD     #65535			;Reset DRP to max period
	STD     L005F			;Stored DRP
;											
	LDX     #L3060	            	;
	BCLR    0,X,#$10		;Clear b4,  
;
;~~~~~~~~~~~~~~~~~~~~~~~
; Do software interrupt
;~~~~~~~~~~~~~~~~~~~~~~~
;
LA6AD	SWI				;Software interrupt    
	BRA     LA6AD			;Loop 
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;~~~~~~~~~~~~~~~~~~~~~~~~~
; DRPs within window here
;~~~~~~~~~~~~~~~~~~~~~~~~~
;
;
LA6B0	BRSET   L004F,#$80,LA6B7	;Bra if engine is running
;
;-Here if engine isnt running
;
	JMP     LAD0C			;RPM calc. routine
;
LA6B7	INCA				;+1, DRP period counter    
	STAA    L0225			;Sace it
;
;~~~~~~~~~~~~~~~~~
; Calculate RPMs
;~~~~~~~~~~~~~~~~~
;
;-RPM = (25 x 256 x 307)/(2 x (# cyl/8) x DRP)
;
;-DRP = (65536 x 120) / (RPM x # cyl)
;
LA6BB	LDD     L3FC0			;DRP counter
	STD     L005F             	;Stored DRP value
	STD     L020B			;Stored DRP value
;											
	LDX     #L005F			;Stored DRP
	LDAA    L4142			;# of cylinders
	LDAB    #32			;mult.
	MUL				;32 x # of cylinders     
	TBA				;Get LSB into A (/256), now # of cylinders relative to V8
	BEQ     LA6D4			;Bra if ==0, no cylindr count set
;
	JSR     LF1ED			;(A x X*)/256, # of cylinders x DRP
;
	STD     L005F			;Save result
;
LA6D4	LDD     L005F			;Load result,DRPs
	ASLD				;x2
	XGDX				;Get result into X, denominator
	LDD     #307			;Numerator
	FDIV				;Fract. divide    
	XGDX				;Get result into D
	STD     L083D			;Save calculated RPMs
;
;~~~~~~~~~~~~~~~~
; Do scaled RPMs
;~~~~~~~~~~~~~~~~
;
	CMPA    #96			;2400 rpm
	BCS     LA6ED			;Bra if <, do 12.5 rpm/count portion
;
;-2400 - 4800 rpm
;
	ADDD    #$4080			;MSB = 64, offset, LSB = 128, round up if needed  
	BCC     LA6F4			;Bra if no overflow
; 
	LDAA    #255			;Clear overflow, 
	BRA     LA6F4			;Bra to save
;
;-400 - 2400
;
LA6ED	ASLD				;x2, RPM/12.5    
	SUBD    #$1F80			;MSB = 31, 400 rpm offser, LSB = 128, round down if needed
	BCC     LA6F4             	;Bra if no underflow
;
	CLRA				;Clear underflow
;    
LA6F4	STAA    L0061			;Scaled RPMs
;
;~~~~~~~~~~~~~~~~~~~~
; Do RPM/25
;~~~~~~~~~~~~~~~~~~~~
;
	LDD     L083D			;Stored RPMs from calc.
	ADDD    #128			;Round up if needed
	SBCA    #$00			;-1 if overflow
	STAA    L0062			;Save it, RPM/25
;
;~~~~~~~~~~~~~~~~~~~~~~~~~
; Filter RPM for governor
;~~~~~~~~~~~~~~~~~~~~~~~~~
;
	LDX     L02B1			;Governor RPM/25
	LDAB    L50DA			;Governor RPM filter coeff
	JSR     LF0F6			;Lag filter routine
;
	STD     L02B1			;RPM/25
	ADDD    #128			;Round if needed
	LDAB    L0002			;Loop counter
	ANDB    #$1F			;Clear all but lower 5 bits
	CMPB    #$04			;0000 0100
	BNE     LA721			;Bra if b2==1
;
	TAB				;Get rpm/25 into B     
	SUBB    L02AF			;Previous governor rpm/25 term
	STAB    L02B0			;Delta governor RPM/25
	STAA    L02AF			;Save current RPM/25 as previous term
;
;-Do rpm/6.25
;
LA721	LDD     L083D			;Stored RPM/25
	ASLD				;x2    
	BCS     LA72F			;Bra if overflow
;
	ASLD				;x2    
	BCS     LA72F			;Bra if overflow
;
	ADDD    #128			;Round if needed
	BCC     LA731			;Bra if no overflow
;
LA72F	LDAA    #255			;Clear overflow
;
LA731	STAA    L0067			;RPM/6.25
;
	CLRB				;Clear LSB    
	LDX     L0068			;
	BEQ     LA742			;Bra if filtered RPMs are ==0, save new value instead
;                
;-Filter RPMs/6.25
;
	LDX     #L0068			;Addr. of stored RPMs
	LDY     #L450D			;Addr. of filter coeff.
	JSR     LF116			;RPM filter routine
LA742	STD     L0068			;Save filtered RPMs/6.25
;
	LDD     L083D			;Stored RPMs/25
	ASLD				;x2
	BCC     LA74D			;Bra if no overflow
;
	LDD     #$FFFF			;Clear overflow
;										
LA74D	STAA    L0837			;Save it, stored RPMs/12.5
	LDX     L0063			;Filtered RPM/12.5
	BEQ     LA75B			;Bra if ==0, save new value instead

	LDY     #L4152			;Addr. of filter coeff
	JSR     LF0D3			;Lag filter routine
LA75B	STD     L0063			;Filtered RPMs/12.5
;
;~~~~~~~~~~~~~~~~~~~~~~~~~
;-Calculate needed dwell
;~~~~~~~~~~~~~~~~~~~~~~~~~
;
;-Do dynamic dwell
;
	LDAA    #255			;3.89 msec, max dyn. dwell
	LDAB    L01E3			;S/D MAP
	SUBB    L01E5			;S/D MAP, two delayed
	BCS     LA76B			;Bra if MAP falling
;
;-S/D MAP steady/rising
;
	CMPB    #32			;Delta S/D MAP, 30 kPa
	BCC     LA77A			;Bra if delta SD MAP >=, set max dwell
;
LA76B	LDD     L0209			;DRP, one delayed
	SUBD    L020B			;Subtract Stored DRP 
	ASLD				;x2
	SUBD    L020D			;Subtract previous dynamic dwell
	BMI     LA77D			;Bra if result negative, bra if RPMs falling/dynamic dwell was greater
;
;-RPMs rising, add in additional dwell = (DRP, one delayed - DRP) 
;
	ADDD    L020D			;Get delta DRPs x 2 back
;
;-Here to set max dwell
;
LA77A	STD     L020D			;Save it, dynamic dwell
;
;-Limit dynamic dwell to DRP/8, or 11.25 degrees
;
LA77D	LDD     L020B			;Stored DRP
	LSRD				;/2  
	LSRD				;/2
	LSRD				;/2, now DRP/8, total time available for dynamic dwell
	SUBD    L020D			;subtract dyn. dwell
	BCC     LA78E			;Bra if dwell is <= DRP/8
;
	ADDD    L020D			;Get DRP/8 back
	STD     L020D			;Save DRP counts/8 instead
;
;~~~~~~~~~~~~~~~~~~~~~~~~
;-Calculate static dwell 
;~~~~~~~~~~~~~~~~~~~~~~~~
;
LA78E	LDD     L020B			;DRP
	STD     L0209			;DRP, one delayed
	LSRD				;/2, now .5 x DRP    
	SUBD    #229			;
	BCC     LA79F			;Bra if DRP/2 is >=
;
;-DRP < 7 msec, dwell = .5 x DRP + 1.2 msec
;
	ADDD    #308			;
	BRA     LA7AF			;Bra to save
;
;-DRP >= 7 msec
;
LA79F	LSRD				;/2, now .25 x DRP - 1.74 msec
	SUBD    #295			;
	BCS     LA7AA			;Bra if DRPs < 1636
;
;-DRP >= 25 msec, dwell = .25 x DRP - .41 msec 
;
	ADDD    #382			;
	BRA     LA7AF			;Bra to save
;
;-7 <= DRP < 25, dwell = .0625 DRP + 4.7 msec
;
LA7AA	ADDD    #1527			;
	LSRD				;/2    
	LSRD				;/2	
; 
LA7AF	STD     L083F			;Save it, static dwell
;
;-Compensate for low batt. voltage
;
	LDAA    #120			;12V, batt volts
	SUBA    L00A1			;Subtract A/D batt volts, now delta batt volts
	BCC     LA7B9			;Bra if batt volts are <=
;
	CLRA				;Clear delta batt volts 
;   
LA7B9	LDAB    #04			;Delta batt volts multiplier, .6 msec per volt below 12 volts
	MUL				;4 x Batt volts
	ADDD    L083F			;Add in static dwell
	ADDD    L020D			;Add in dynamic dwell
	STD     L020F			;Save it, outputted dwell
;
;-Limit dwell to ensure at least 595 usecs of off time between coil firings
;
	LDD     L3FC0			;DRP counter
	SUBD    #39			;595 usecs
	SUBD    L020F			;Outputed dwell
	BCC     LA7D6			;Bra if off time > 595 usec
;
	ADDD    L020F			;Get DRPs - 595 usecs back
	STD     L020F			;Save as outputted dwell
;
;-Shuffle the MAP around
;
LA7D6	LDAB    L01E4			;SD MAP, one delayed
	STAB    L01E5			;Save it, SD MAP two delayed
	LDAB    L01E3			;SD MAP
	STAB    L01E4			;Save it, SD MAP, one delayed
;
;~~~~~~~~~~~~
; Look up SA
;~~~~~~~~~~~~
;
	LDX     #L4165			;Main SA table
	BRCLR   L0050,#$40,LA7EC	;Status word, bra if b6==0, idle spark not enabled
;
	LDX     #L4289			;Closed throttle SA table 
;
;-Load in RPM and MAP for lookup
;
LA7EC	LDAB    L01E3			;SD MAP
	LDAA    L0061			;Scaled RPMs
	CMPA    #64			;1200 RPM, upper limit to use scaled RPMs
	BLS     LA7FD			;Bra if RPMs < 1200 RPM, use scaled RPMs
;
;-Use RPM/25 if RPMs high enough
;
	LDAA    L0062			;RPM/25
	ADDA    #16			;400 RPM offset for crossing point
	BCC     LA7FD			;Bra if no overflow
;
	LDAA    #255			;Clear overflow
;
;-RPMs now 400-1200 in 12.5 RPM inc., 1200-5975 in 25 rpm inc.
; 
LA7FD	JSR     LF17B			;3D lookup
;
	STAA    L021B			;Save it, looked up SA
;
;-Look up cool temp SA correction
;
	LDX     #L43AD			;Addr. of MAP/VAC option byte
	LDAB    L01EC			;Load vacuum
	LDAA    0,X			;Load it in
	BEQ     LA810			;Bra if ==0, use vac
;
	LDAB    L01E3			;Load SD MAP
;
LA810	INX				;+1, advance to start of table
	LSRB				;Vac,MAP/2
	LDAA    L0006			;Cool temp, linear
	CMPA    #224			;128 deg C, max temp for lookup
	BLS     LA81A			;Bra if cool temp <=
;
	LDAA    #224			;Load max
;
LA81A	JSR     LF17B			;3D lookup
;
	LDAB    L0050			;Status word
	BPL     LA830			;Bra if b7==0, not idling
;
;-Idling here, check and limit negative SA
;
	LDAB    L0006			;Cool temp
	CMPB    L4141			;Cool temp threshold for 
	BCC     LA830			;Bra if cool temp is >=, skip check

	CMPA    L413D			;Cool temp SA bias
	BCC     LA830			;Bra if cool temp SA corrections greater
;
	LDAA    L413D			;Use bias if SA is less
;
;-Here if not idling
;
LA830	STAA    L021C			;Save it, cool temp spark correction
	LDAA    L400D			;Option byte 
	BITA    #$04			;Test b2, backpressure EGR
	BEQ     LA849			;Bra if not using back pressure EGR
;	
;-Lookup alt. SA correction
;
	LDAA    L01EF			;Baro val. for lookup
	LDAB    L01EC			;Vacuum
	LDX     #L44CB			;Addr. of alt. SA correction table
	JSR     LF17B			;3D lookup routine
;
	STAA    L021E			;Save it, alt. SA correction
;
;-Look up low octane SA retart
;
LA849	LDAB    L0841			;Low octane RPM mult.
	LDAA    L01E3			;SD MAP
	LDX     #L45BD			;Addr. of low octane MAP mult.  
	JSR     LF15E			;2D lookup
;
;-low octane SA mult. = (MAP mult x rpm mult) / 256
;
	MUL				;Low octane map mult. x low octane rpm mult.
	ADCA    #0			;Round if needed
	LDAB    L022D			;Low octane base SA retard
	MUL				;Low octane SA mult. x base SA retard
	ADCA    #0			;Round if needed
	STAA    L022E			;Save it, low octane SA retard
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Look up EGR sA correction
;~~~~~~~~~~~~~~~~~~~~~~~~~~~
;											
	LDAA    L413F			;Load EGR bias SA
	CLRB				;Clear B
	BRCLR   L006E,#$80,LA8A3	;Status word, bra if b7==0, EGR not active
;
;-EGR is active
;
	LDX     #L4855			;Load EGR spark correction table
	LDAB    L01D5			;Filt. % EGR flow
	LDAA    #192			;Scalar for %EGR flow
	MUL				;Scalar x EGR flow
	TAB				;MSB to B
	LDAA    0,X			;Get load selection byte
	BEQ     LA88D			;Bra if ==0, use % EGR flow
;
;-Scale vacuum for lookup
;
	LDAB    L01EC			;Vacuum
	NEGB				;Neg., now 0-80 kPa
	CMPB    #64			;20 kPa, lower limit to scale vac 
	BHI     LA882			;Bra if vacuum kPa >
;
;-Vac kPa < 20, val for lookup = vac x 2
;
	ASLB				;x2
	BRA     LA88D			;Bra to continue				
;
;-Vac kPa >= 20, val for lookup = (vac - 64)/2 + 128
;											
LA882	SUBB    #64			;Subtract offset
	LSRB				;/2    
	ADDB    #128			;+128
	CMPB    #192			;60 kPa, upper limit for lookup
	BLS     LA88D			;Bra if kPa <=
;
	LDAB    #192			;Load max
;
;-Check RPM for lookup
;											
LA88D	LDAA    L0062			;RPM/25
	CMPA    #160			;4000 RPM, UL for lookup
	BLS     LA895			;Bra if RPMs <=
;
	LDAA    #160			;Load max
;
LA895	LSRA				;/2 for lookup  
	INX				;+1, index start of table
	JSR     LF17B			;3D lookup
;					
	LDAB    L4854			;EGR SA correction filt. coeff
	LDX     L0217			;EGR correction SA
	JSR     LF0F6			;Filter routine
;
;-Here if EGR not on
;											
LA8A3	STD     L0217              	;Filtered EGR SA
;
;~~~~~~~~~~~~~~~~~~
;-Sum up SA so far
;~~~~~~~~~~~~~~~~~~
;
	ADDD    #128			;Round if needed
	TAB				;Get result into B
	LDX     #$0000			;Clear X
	ABX				;Add in EGR SA
	LDAB    L021E			;Load in alt. SA corr.
	ABX				;Add it in
	LDAB    L021B			;Load in main SA
	ABX				;Add it in
	LDAB    L021F			;PE SA
	ABX				;Add it in
	LDAB    L021C			;Cool temp SA corr.
	ABX     			;Add it in
	LDAB    L02EE			;Startup SA
	ABX				;Add it in     
	XGDX				;Get result into D    
;
;-Sub out bias' and retard
;
	SUBB    L413C          		;Main SA bias
	SBCA    #$00			;Perform carry if needed
	SUBB    L413D			;Cool correction SA bias
	SBCA    #$00			;
	SUBB    L413E          		;Alt. SA corr. bias
	SBCA    #$00			;
	SUBB    L413F			;EGR SA bias
	SBCA    #$00			;
	SUBB    L022E			;Low octane SA retard
	SBCA    #$00			;
	SUBB    L0154			;
	SBCA    #$00			;
	STD     L0220			;Save it, final SA
;
;~~~~~~~~~~~~~~~~~~~~~~
;-Do MAT SA correction
;~~~~~~~~~~~~~~~~~~~~~~
;
;
;-Look up MAT SA correction mult for negative SA correction
;
	LDAA    L01E3			;SD MAP
	LDX     #L4493			;Addr of negative MAT SA correction mult. vs. SD MAP
	BRSET   L0041,#$02,LA8F1   	;Status word, bra if b1==1, SA correction is negative                  
;
;-Look up MAT SA correction mult for positive SA correction
;
	LDX     #L44A4             	;Addr. of positive MAT SA correction mult. vs. SD MAP
LA8F1	JSR     LF15E			;2D lookup
;
	LDAB    L021D			;MAT SA correction
	MUL				;MAT SA corr. x MAT SA mult.
	STAA    L083C			;Save it, Final MAT SA correction

	BRCLR   L0041,#$02,LA90C	;Status word, bra if b1==0, MAT SA corr. >=0
;
;-Subtract MAT SA correction 
;
	LDD     L0220			;Final SA 
	SUBB    L083C			;-Final MAT SA 
	SBCA    #0			;Round if needed
	STD     L0220			;Save it, Final SA 
	BRA     LA914
;
;-Add in MAT SA correction
;
LA90C	TAB				;Get MAT correction SA into B     
	LDX     L0220			;Load final SA 
	ABX				;Add in MAT SA
	STX     L0220			;Save final SA 

LA914	LDX     L00F8			;Engine run time
	CPX     L415D			;Min run time for idle SA adj.
	BCC     LA91D			;Bra if runtime is >=
;
	BRA     LA977			;Bra to skip idle SA adj.			
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Idle under/overspeed SA proportional control
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;-Check quals for idle SA adj.
;
LA91D	LDX     #L44F8			;Addr. of engine idle underspeed SA adj. table
	LDAA    L02A7			;MPH
	CMPA    L4159			;Upper MPH limit for idle SA adj.
	BCC     LA977			;Bra if VSS >=
;
	LDAA    L415A			;Upper %TPS limit for idle SA adj., idle SA ok'd
	BRCLR   L0050,#$40,LA932	;Status word, bra if idle SA not ok'd 
;
	LDAA    L415B			;Upper TPS limit for idle SA adj., idle SA not ok'd
;
LA932	CMPA    L01FC			;%TPS
	BHI     LA939			;Bra if %TPS is less, continue
;
	BRA     LA977			;Bra, TPS exceeds limits
;
LA939	LDAB    L0006			;Cool temp, linear
	CMPB    L415C			;Min temp for SA adj.
	BLS     LA977			;Bra if cool temp less
;
	CLRB				;
	LDAA    L0857			;Desired idle speed										;
	BRSET   L0050,#$80,LA954	;Status word, bra if b7==1, engine idling
;
;-Check desired idle speed
;
	ADDA    L415F			;RPM offset if not idling
	BCC     LA954			;Bra if no overflow
;
;-Here if overflow from addition of bias
; 
	BCLR    L0053,#$01		;Status word, clear b0, engine underspeed
;
	SUBD    L0063			;Subtract RPM/12.5, now delta RPMs										;
	BRA     LA977			;Bra, quals not passed for idle SA control
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Here if quals passed for proportional idle SA control
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LA954	SUBD    L0063			;Subtract RPM/12.5
	BCS     LA95D			;Bra if eng. speed > then desired idle speed
;
	BCLR    L0053,#$01		;Status word, clear b0, engine speed <= desired idle speed
	BRA     LA967			;Bra to continue
;
;-Here if engine speed > desired idle speed
;
LA95D	BSET    L0053,#$01		;Status word, set b0, engine speed > then desired idle speed
	NEGA 				;Make error term positive
	NEGB    			;
	SBCA    #$00			;Perform carry if needed	
	LDX     #L44E6			;Addr. of engine overspeed SA correction
;
;-Here if engine speed <= desired idle speed
;
LA967	PSHX				;Table addr. to stack
	ASLD				;RPM error x 2, now RPM / 6.25
	BCS     LA973			;Bra if overflow
;
;-Scale the RPM error for lookup, 0 - 50 rpm in .78 rpm inc., 50 - 1250 rpm in 6.25 rpm inc.
;
	CMPA    #08			;50 rpm, RPM error limit for RPM x 1.28
	BCS     LA97D			;Bra if RPM error is < 

	ADDA    #56			;Offset for RPM x 1.28
	BCC     LA980			;Bra 
;
LA973	LDAA    #255			;Load max
	BRA     LA980			;Bra 
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Here if quals not passed for idle SA control
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LA977	BCLR    L0050,#$40		;Status word, clear b6, idle SA adj. not active
	JMP     LA9CC			;Bra to skip idle SA adj.
;
;-Here to continue with idle SA adj.
;
LA97D	ASLD				;x2
	ASLD				;x2
	ASLD				;x2, now RPM x 1.28  
;
LA980	JSR     LF136			;2D lookup with UL
;
	PULX				;Get start addr. back into X    
	PSHA				;Push SA corr. to stack  
;
;-Look up S/D MAP SA corr. multiplier
;
	LDAB    #$08			;Addr. offset for SD MAP SA corr. mult.
	ABX				;Add in the offset 
	LDAA    L01E3			;S/D MAP
	JSR     LF136			;2D lookup with UL
;
	PULB				;Get idle SA correction
	MUL				;Idle SA x idle SA mult.
	BRCLR   L0053,#$01,LA995  	;Status word, bra if b0==0, engine idle underspeed

	NEGA				;Make negative signed val. if SA corr. negative
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Perform idle SA derivative control
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LA995	PSHA				;Push proportional idle SA corr. to stack
	LDX     #L4511			;Addr. of transient RPM idle SA corr. table
	LDAA    L006C			;Transient RPM ratio
	SUBA    #96			;limit to .75 RPM ratio
	BCC     LA9A2			;Bra if no underflow

	CLRA				;Clear underflow   
	BRA     LA9A8			;Bra to continue
;
;-Check upper limit for lookup
;
LA9A2	CMPA    #63			;1.25 RPM ratio
	BLS     LA9A8			;Bra if <=
;
	LDAA    #63			;Load max
;
LA9A8	ASLA				;x2
	ASLA				;x2, now spans from .75-1.25 in .00195 increments
	JSR     LF15E			;2D lookup
;
	PULB				;Get idle SA corr. off the stack
	ABA				;Add it in
	SUBA    L450A			;20 deg., SA bias for transient RPM SA corr.
	BMI     LA9C0			;Bra if SA corr is negative
;
	CMPA    L450B			;Max positive SA corr.
	BLS     LA9BC			;Bra if SA corr. <=

	LDAA    L450B			;Load max SA corr.
;
LA9BC	TAB     			;Get SA corr. into B
	CLRA				;Preclear A
	BRA     LA9C3
;
;-Here if SA corr. is negative
;
LA9C0	TAB				;Get SA corr. into B
	LDAA    #$FF			;Preload with 0xFF, negative SA corr.
;
;-Here if SA corr. is positive
;
LA9C3	ADDD    L0220			;Add in SA corr.
	STD     L0220			;Save it, final SA
	BSET    L0050,#$40		;Status word, set b6 for next pass, idle SA enabled
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-SA derivative engine control routine, used with manual transmission
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LA9CC	LDAA    L0006			;Cool temp, linear
	CMPA    L46E8			;Lower limit for SA adj.
	BCS     LAA40			;Bra if cool temp less, no SA adj.
;
	CMPA    L46E9			;upper limit for SA adj.
	BHI     LAA40			;Bra if cool temp is higher, no SA adj.

;
;-Check to see if manual trans present
;
	LDAA    L400F			;Opt. byte
	BITA    #$80			;Test b7, manual trans/TCC
	BEQ     LAA40			;Bra if ==0, TCC present, skip routine
;
;-Check to make sure engine not idling
;
	BRCLR   L0050,#$80,LA9EC	;Status word, bra if b7==0, not idling
;
	LDAA    L0837			;Stored RPMs/12.5
	CLRB				;    
	STD     L0838			;Stored RPMs/12.5, filtered
	BRA     LAA40			;Bra to skip routine
;
;-All quals met here
;
LA9EC	LDAA    L0063			;RPM/12.5
	SUBA    L0838			;Subtract stored RPMs/12.5, filtered
	STAA    L083A			;Save it, RPM error
;
;-Look up filter coeff for filtered RPMs/12.5 using TPS
;
	LDX     #L46EA			;Addr. of filter coeff. vs. TPS table
	LDAA    L01FC			;TPS
	LSRA				;/2
	LSRA				;/2, now TPS/4 for lookup    
	JSR     LF15E			;2D lookup
;
	TAB				;Get filt. coef. into B     
	LDAA    L0837			;RPM/12.5
	LDX     L0838			;RPM/12.5, filtered
	JSR     LF0F6			;Filter routine
;
	STD     L0838			;Save filtered RPMs
;
	LDX     #L46E2			;Addr. of gain terms
	LDAA    L083A			;RPM error
	BPL     LAA18			;Bra if RPMs rising

;
;-RPMs falling here
;
	INX				;curr. addr. +1    
	INX				;+1
	INX				;+1, now at params for falling RPMs
	NEGA				;Make absolute value of RPM error
;
;-RPMs rising here
;
LAA18	LDAB    0,X			;Load RPM gain
	MUL				;RPM error x gain    
	ADDA    1,X			;Add in der. term bias
	CMPA    2,X			;Upper limit for der. term
	BLS     LAA23			;Bra if less
;
	LDAA    2,X			;Load max SA der. term
;
LAA23	STAA    L083B			;SA derivative term
	LDAA    L083A			;RPM error
	BPL     LAA35			;Bra if RPMs rising
;
;-RPMs falling here, add in SA der. term
;
	LDD     L0220			;Final SA
	ADDB    L083B			;Add in SA der. term
	ADCA    #$00			;Perform carry if needed
	BRA     LAA3D			;Bra to save
;
;-RPMs rising here, subtract out the SA der. term
;
LAA35	LDD     L0220			;Final SA
	SUBB    L083B			;Subtract out the SA der. term 
	SBCA    #$00			;Perform carry if needed
LAA3D	STD     L0220			;Final SA adv.
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Perform Decel Fuel Cut Off SA corrections
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LAA40	LDD     L0220			;Load final SA
	BRCLR   L0046,#$08,LAA69	;Status word, bra if b3==0, DFCO not enabled
;
;-Subtract out DFCO SA correction if entering DFCO
;
	BSET    L0051,#$20		;Status word, set b5, DFCO SA correction enabled
;
	CLRA				;
	CLRB				;
	STD     L0215			;SA ramping mult.
	LDAB    L4151			;DFCO SA 
;
	LDAA    L021C			;Cool temp corr. SA
	SUBA    L413D			;Subtract bias
	BMI     LAA5C			;Bra if SA corr. is negative
;
	ABA				;Add in DFCO SA
	TAB				;Get result into B
;
LAA5C	STAB    L022B			;Save it, DFCO SA correction											;
	LDD     L0220			;Final SA
	SUBB    L022B			;Subtract DFCO SA correction
	SBCA    #$00			;Perform carry if needed
	BRA     LAAA3			;Bra to save SA
;
;-Check to see if previously performing DFCO SA corr.
;
LAA69	BRCLR   L0051,#$20,LAAA6	;Status word, bra if b5==0, no DFCO SA corr. 

	LDAB    L414E			;Low %TPS filter coeff. for SA ramping mult.
	LDAA    L01FC			;TPS
	CMPA    L4150			;Upper TPS threshold for filter coeff.
	BLS     LAA7B			;Bra if TPS is <=
;
;-Ramp DFCO SA corr. back into main SA
;
	LDAB    L414F			;High %TPS filter coeff. for SA ramping mult.
;
LAA7B	LDX     L0215			;SA ramping multiplier
	CPX     #65264			;Upper limit to filter ramp-in mult.
	BHI     LAA8D			;Bra if higher, all SA ramped in
;
	LDAA    #255			;Value to filter ramp-in mult. to
	JSR     LF0F6			;Filter routine
;
	STD     L0215			;Save it, DFCO SA ramp-in multiplier
	BRA     LAA90			;Bra, still ramping in SA
;
;-Here if all DFCO SA ramped back in
;
LAA8D	BCLR    L0051,#$20		;Status word, clear b5, no longer performing DFCO SA corr. 
;
LAA90	LDAB    L022B			;DFCO SA corr.
	LDAA    L0215			;SA ramping multiplier
	MUL				;DFCO SA x ramping mult.     
	ADCA    #$00			;round if needed
	TAB				;Get SA into B     
	CLRA				;   
	ADDD    L0220			;Add in partial DFCO SA corr. into final SA
	SUBB    L022B			;Subtract DFCO SA corr., now only part of DFCO SA corr. is in effect
	SBCA    #$00			;Perform carry if needed
;
LAAA3	STD     L0220			;Save it, final SA
;
LAAA6	BRCLR   L0082,#$40,LAAAD	;Status word, bra if b6==0, no heads-up yet 
;
	JSR     L1812			;Heads up vector table
;
;-Check if P/S pressure switch in use
;
LAAAD	LDAA    L4E86			;Opt. byte
	BITA    #$02			;Test b1, power steering switch in use
	BEQ     LAAE0			;Bra if ==0, no PS switch in use
;    
;-Check if P/S active
;
	BRCLR   L0036,#$20,LAADD	;Bra if b5==0, P/S compensation not in effect
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Check quals for power steering SA compensation
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;-Check cool temp
;
	LDAA    L0006			;Cool temp, linearized
	CMPA    L4160			;Lower temp threshold for P/S SA compensation
	BLS     LAADD			;Bra if cool temp is <=
;
;-Check TPS
;
	LDAA    L01FC			;TPS
	BRSET   L0053,#$10,LAACD	;Status word, bra if b4==1, P/S SA compensation 
					;already underway, check threshold to disable
;
;-P/S compensation not yet underway, check TPS threshold to enable compensation
;
	CMPA    L4163			;TPS threshold for enabling P/S SA compensation 
	BHI     LAADD			;Bra if TPS >, no SA compensation
;
	BRA     LAAD2			;Bra to enable P/S SA compensation				
;
;-P/S compensation underway, check TPS threshold to disable compensation
;
LAACD	CMPA    L4164			;TPS threshold for disabling P/S SA compensation
	BHI     LAADD			;Bra if TPS > 
;
;-Enable P/S SA compensation
;
LAAD2	LDD     L4161			;Fixed SA for P/S compensation
	STD     L0220			;Save it, final SA
;									
	BSET    L0053,#$10   		;Status word, set b4, P/S SA compensation in effect
	BRA     LAAE0			;Bra to continue		
;
;-Here if no P/S compensation
;											
LAADD	BCLR    L0053,#$10   		;Clear b4, P/S compensation not in effect
;
;-Subtract base SA
;											
LAAE0	LDD     L0220			;Final SA
	SUBB    L4133			;Base SA
	SBCA    #0			;Perform carry if needed
	STD     L0211			;Save it, total SA
;
;-Check limits on SA
;											
	LDD     L4145			;Max allowable SA
	SUBD    L0211			;Subtract total SA
	BGT     LAAF9			;Bra if SA <
;
	ADDD    L0211			;Add in total SA to get max allowable SA back
	STD     L0211			;Save max allowable SA instead
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~
; Perform knock prevention 
;~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LAAF9	LDAA    L400F			;Opt. byte
	BITA    #$08			;Test b3, enable ESC
	BEQ     LAB47			;bra if b3==0, bra if no ESC
;
	BRSET   L0050,#$10,LAB47	;Bra if b4==1, in diags. mode
;
;-Check cool temp quals
;
	LDAA    L0006			;Cool temp, linear
	CMPA    L45D0			;Min cool temp for spark retard
	BLS     LAB47			;Bra if cool temp less, no retard
;
;-Check to see if knock prevention in effect
;
	BRCLR   L006E,#$02,LAB24	;Bra if knock prevention not currently in effect

	DEC     L0231			;Burst retard timer
	BNE     LAB17			;Bra if !=0, perform burst retard
;
	BCLR    L006E,#$02		;Status word, clear b1, knock prevention no longer in effect
;
;-Knock prevention in effect here
;
LAB17	LDD     L0211			;Load total SA
	SUBB    L0842			;Subtract burst retard
	SBCA    #$00			;Perform carry if needed
	STD     L0211			;Save total SA
	BRA     LAB47			;Bra to continue
;
;-Check quals for knock prevention
;				
LAB24	LDAA    L01E3			;Load SD MAP
	CMPA    L46D5			;50 kPa
	BCC     LAB47			;Bra if >=
;
	LDAB    L01FC			;%TPS
	CMPB    L46D6			;Upper limit for
	BCC     LAB47			;Bra if TPS >=
;
	SUBB    L01FD			;Subtract TPS, one delayed
	BCS     LAB47			;Bra if throttle is closing
;
	CMPB    L46D7			;Delta %TPS threshold for knock prevention
	BCS     LAB47			;Bra if delta TPS <=
;									
	BSET    L006E,#$02		;Status word, set b1, knock prevention ok'd
;
	LDAA    L46D8			;Time for burst retard to be in effect
	STAA    L0231			;Save it, burst retard timer
;
;-Here if knock prevention not ok'd
;
LAB47	SEI				;Disable interrupts
	LDAB    L0230			;Load knock counts into B
	CLR     L0230			;Clear knock counts
	CLI				;Enable interrupts
	BRSET   L006E,#$02,LAB60	;Status word, bra if knock prevention in effect
;
	BRCLR   L0016,#$60,LAB59	;Error word, bra if b5, b6==0, no cool temp sens. errors
;
	BRA     LAB63			;Bra to continue with SA retard quals if errors set.				
;
;~~~~~~~~~~~~~~~~~~~~
; Knock retard quals
;~~~~~~~~~~~~~~~~~~~~
;									
LAB59	LDAA    L0006			;Load cool temp
	CMPA    L45D0			;Min temp for knock retard 
	BCC     LAB63			;Bra if cool temp >=
;
;-Here if knock prevention already in effect
;
LAB60	CLRA				;
	BRA     LAB91			;
;
;-Check MPH 
;
LAB63	LDAA    L02A7			;MPH
	CMPA    L45CE			;Lower limit for SA retard
	BCC     LAB72			;Bra if speed >=
;
;-Check RPMs
;
	LDAA    L0063			;RPM/12.5
	CMPA    L45CF			;Lower RPM limit for SA retard
	BCS     LABA2			;Bra if RPMs <=, no retard
;
;-Check for ESC error
;
LAB72	BRCLR   L0019,#$10,LAB7B	;Error word #4, bra if no ESC failure
;
	LDAA    L4E6D			;Default SA for ESC faulure
	BRA     LAB91			;Bra to save
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; All quals passed, calc. retard
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;-Retard SA = .5 x [(knock attack rate x knock counts)/128]
;
LAB7B	LDAA    L0843			;Knock attack rate
	MUL				;Knock counts x knock attack rate     
	ASLD				;x2    
	BCS     LAB87			;Bra if no overflow
;
	ADDA    L022F			;Add in previous retard
	BCC     LAB89			;Bra if no overflow
;
LAB87	LDAA    #255			;Clear overflow
;
LAB89	TAB				;Knock retard to B     
	LDAA    L0844			;Max knock retard
	CBA				;     
	BLS     LAB91			;Bra if knock retard is less

	TBA				;Get max knock retard into A
;     
LAB91	STAA    L022F			;Save it, knock retard 
	LSRA				;/2    
	PSHA				;Push to stack	    
	TSX				;Get current stack pointer into X     
	LDD     L0211			;Load total SA
	SUBB    0,X			;Subtract retard
	SBCA    #$00			;Perform carry if needed
	STD     L0211			;Save total SA
	INS				;Restore stack pointer     
LABA2	LDAA    L0154			;				 
	BEQ     LABB2			;Bra if ==0
;
	LDD     L414B			;max retard during torque management fuel cutoff 
	CPD     L0211			;Total SA 
	BLT     LABD5			;Bra if timing is > 
;
	BRA     LABD2			;Bra to save max retard instead
;
LABB2	BRCLR   L0051,#$20,LABC9	;Bra if no longer performing DFCO SA correction
;
;-Performing DFCO SA correction here
;
	LDAA    L01FC			;TPS
	CMPA    L48D0			;Max closed throttle TPS
	BHI     LABC9			;Bra if TPS >
;
;-Throttle is closed, limit retard
;
	LDD     L4149			;Max retard during DFCO
	CPD     L0211			;Total SA
	BLT     LABD5			;Bra if timing > 

	BRA     LABD2			;Bra to save max retard instead


LABC9	LDD     L4147			;Max allowable retard
	CPD     L0211			;Total SA
	BLT     LABD5			;Bra if timing >
;
LABD2	STD     L0211			;Save total SA
;
LABD5	LDD     L0211			;Load total SA
;
;~~~~~~~~~~~~~~~~~~~~~~~
; Check for ALDL mode 4
;~~~~~~~~~~~~~~~~~~~~~~~
;
	BRCLR   L003B,#$10,LABF9	;Bra if not in mode 4
;
;-In mode 4
;
	LDX     #L0397			;
	BRCLR   0,X,#$08,LABF9		;
;
	BRSET   0,X,#$10,LABE9		;  
;
	CLRA				;
	CLRB				;
;
LABE9	BRSET   0,X,#$20,LABF4		; 
;
	ADDB    L039A			;
	ADCA    #$00			; 
;
	BRA     LABF9			;				
;
LABF4	SUBB    L039A			;
	SBCA    #$00			;
;
;-Here if no mode 4
;
LABF9	STD     L0211			;Save total SA
	STD     L0213			;
;
;-Check sign of SA and set status bit accordingly
;											
	BCLR    L004F,#$01		;Status word, clear b0, SA positive
;											
	LDD     L0211			;Load total SA
	BPL     LAC0B			;Bra if >0
;
	BSET    L004F,#$01		;Status word, set b0, SA negative
	NEGB				;Get abs. value of SA
;
LAC0B	TBA				;SA to A
;
;-Check if SA ramping still in effect
;
	BRSET   L0051,#$40,LAC1D	;Status word, bra if b6==1, all SA ramped back in
;
	LDAB    L0215			;SA ramping multiplier
	LDX     #L0216			;Index LSB for rounding
	BRCLR   0,X,#$80,LAC1B		;Bra if b7==0, LSB < 128 	
;
	INCB				;+1, round up
;
LAC1B	MUL				;SA x ramping multiplier
	INCA				;+1, compensate for the fact that multiplier only goes to 255
;
;-Check for heads up
;											
LAC1D	BRCLR   L0082,#$40,LAC24	;Status word, bra if b6==0, no heads up yet 
;				
	JSR     L180F			;To heads up vector table
;
;~~~~~~~~~~~
; Output SA
;~~~~~~~~~~~
;
;-Perform spark latency correction
;
LAC24	PSHA				;Push SA to stack    
	LDAA    L0062			;RPM/25
	LDX     #L4540			;Addr. of spark latency correction table
	JSR     LF136			;2D lookup with UL
;
	STAA    L0224			;Save it, spark latency correction
;
;-Finalize SA for output
;
	PULA				;Get SA off the stack    
	LDX     #L005F			;DRPs
	JSR     LF1ED			;(A x X*)/256
;
	BRSET   L004F,#$01,LAC3F	;Status word, bra if b0==1, SA negative
;
	NEGA				;Negate, 65536 - time based SA    
	NEGB 				;   
	SBCA    #$00			;
;
LAC3F	SUBB    L0224			;Subtract spark latency
	SBCA    #$00			;Perform carry if needed
	PSHB				;Push result to stack    
	PSHA				;    
	TSX 				;Get current stack pointer into X    
	CLRA				;Clear AB    
	CLRB				;    
	SUBD    L3FC0			;Subtract DRP counter
	LSRD				;/2    
	LSRD				;/2
	LSRD				;/2
	LSRD				;/2  
	ORAA    #$F0			;Clear lower 4 bits of MSB
	ADDD    L3FF6			;Add in EST fall counter
	SUBD    0,X			;Subtract origional result on stack
	BMI     LAC5D			;Bra if msb is ==1
;
	ADDD    0,X			;Add origional result back in
	STD     0,X			;Save 
;
LAC5D	LDD     0,X			;Finalized SA value
	SUBD    L3FF6			;Subtract EST fall counter again
	JSR     LF08A			;Short delay
;
	STD     L3FE8			;
	JSR     LF08A			;Short delay
;
	ADDD    L3FDC			;
	SUBD    L020F			;Subtract stored dwell
	NOP				;Very short delay     
	LDX     L020F			;Load dwell
	STD     L3FE6			;
	JSR     LF08A			;Short delay
;
	STX     L3FDC			;
	JSR     LF08A			;Short delay
;
	PULA				;Get SA value back off stack    
	PULB				;
	STD     L3FF6			;Save it, EST fall counter
;
;~~~~~~~~~~~~~~~~~~~
; Perform EST diag.
;~~~~~~~~~~~~~~~~~~~
;    
	BRCLR   L0044,#$08,LACEC	;Status word, bra if b3==0, no valid DRP yet 
;
	BRCLR   L0050,#$04,LACCB	;Bra if b2==0, no DRP yet
;
;-First DRP occured, perform EST check
;
	LDD     L3FCA			;EST circuit info
	PSHB				;    
	PSHA				;
	PULX				;Transfer into X
;
	LDY     #L5B03			;Error mask 4
	BRCLR   0,Y,#$20,LACD7		;Test b5, EST mon. falure, bra if ==0, not setting error
;
	BRSET   L004F,#$40,LACE0	;Bra if b6==1, EST bypass diag. already passed 
;
;-No EST diag. yet, check for open/faulty EST circuit
;  
	BRSET   L0044,#$80,LACD1	;EST failure already flagged
;
	SUBD    L0228			;Previous EST circuit info
	TSTA				;Test MSB    
	BNE     LACB9			;Bra if EST circuit info value > 255, EST open 
;
	CMPB    L4E67			;Number of counts to set error
	BHI     LACB9			;Bra if >, faulty EST, set error
;
;-EST circuit counter value within limits to be grounded
;   
	BRSET   L0044,#$40,LACD7	;Status word, bra if b6==1, EST mon. already set
;
	BSET    L0044,#$40		;Status word, set b6, EST monitor
	BRA     LACC8			;Bra to save EST info and exit.
;
LACB9	LDAB    L024F			;Load error 42 counter
	CMPB    L4E68			;# of EST errors to set code 42
	BHI     LACCE			;Bra if >, set error
;
;-Not setting code 42 yet here
;
	INCB				;Inc. error 42 counter
	STAB    L024F			;Save it
	BCLR    L0044,#$40		;Clear b6, EST monitor 											; CLR b6
LACC8	STX     L0228			;Save EST circuit info
;			
LACCB	JMP     LAD5E			;Exit				
;
;-Setting code 42 here
;    
LACCE	BSET    L0044,#$80		;Set b7, EST failure
;
LACD1	BSET    L0019,#$20		;Error word #4, set code 42, EST failure
	JMP     LAD5E			;Bra to exit
;
;-Here if EST passed diag.
;
LACD7	BSET    L004F,#$40		;Set b6, EST bypass diag. passed 
	CLR     L024F			;Clear EST error counter
	STX     L0228			;Save EST circuit info
;  
LACE0	BCLR    L0044,#$08		;Status word, clear b3, no valid DRP yet
;
	LDX     L3FEC			;
	JSR     LF08A			;Short delay
;											
	STX     L3FE4			;
;
LACEC	BSET    L004B,#$10		;I/O word, set b4
;
;-Check for EST active fault
;
	BRCLR   L0004,#$80,LACFC	;Bra if EST active fault not set
;
	BRSET   L0044,#$04,LACFC	;Status word, bra if b2==1, skip EST due to ALDL
;
;-Here to set code 42, EST 
;
	BSET    L0019,#$20		;Error word #4, set code 42, EST
	BRA     LAD5B			;Bra
;
;-Here if no EST fault/skipping EST error
;
LACFC	BRCLR   L003B,#$10,LAD5E	;Status word, bra if b4==0, not in mode 4
;
;-In mode 4
; 
	LDAA    L0393			;ALDL mode 4 request word
	BITA    #$08			;Test b3
	BEQ     LAD5E			;Bra if ==0, no request to disable EST test
;   
	BSET    L0044,#$04		;Set b2, skip EST test
	BRA     LAD5B			;Bra to continue
;
;~~~~~~~~~~~~~~~~~~~~
;
; RPM calc. routine 
;
;~~~~~~~~~~~~~~~~~~~~
;
;-RPM = (25 x 256 x 307)/(2 x (# cyl/8) x DRP)
;
;-DRP = (65536 x 120) / (RPM x # cyl)
;
LAD0C	LDX     #$FFFF			;Max DRP period    
	BRCLR   L0044,#$08,LAD59	;Bra if no valid DRPs recieved yet
;
	LDD     L3FC0			;DRP counter 
	STD     L005F             	;DRP period
	STD     L020B			;DRP period
	LDX     #L005F             	;DRP period
	LDAA    L4142              	;# cylinders
	LDAB    #32			;
	MUL				;# cylinders x 32     
	TBA				;Get result into A     
	BEQ     LAD2C			;Bra if 8 cylinders
;
	JSR     LF1ED			;(A x X*)/256
	STD     L005F             	;D = (# cyl. x DRP) / 8, now relative to 8 cyl. engine
;
LAD2C	LDD     L005F             	;Stored DRP
	ASLD				;x2
	XGDX 				;Swap DRP x 2 into X   
	LDD     #307			;
	FDIV 				;Fract. divide, X = (307/(DRPs x 2))   
	PSHX				;Push X to stack  
	PULA				;Get results into D, now RPMs/25
	PULB				; 
;   
;~~~~~~~~~~~~~~~~
; Do scaled RPMs
;~~~~~~~~~~~~~~~~
;
;-400 - 2400 in 12.5 rpm inc., 2400-4800 in 25 rpm increments
;
	CMPA    #96			;2400 rpm
	BCS     LAD44			;Bra if less
;
;-RPMs >= 2400 rpm
;
	ADDD    #$4080			;dec. 64, offset for 2400-4800 rpm, $80= round up if needed
	BCC     LAD4B			;Bra if no overflow
;
	LDAA    #255			;Clear overflow
	BRA     LAD4B			;Bra to save.
;
;-RPMs < 2400 here
;
LAD44	ASLD				;x2, now in   
	SUBD    #$1F80			;dec. 31, 400 rpm minimum, $80, round down if needed
	BCC     LAD4B			;Bra if no underflow
;
	CLRA				;Clear underflow
;    
LAD4B	STAA    L0061			;Save it, scaled rpms
;
;~~~~~~~~~~~
; Do rpm/25
;~~~~~~~~~~~
;
;-0 - 6375 in 25 rpm increments
;
	PSHX				;Push calc'd rpms to stack    
	PULA				;Get back into D
	PULB				;
	ADDD    #$80			;
	SBCA    #$00			;Round if needed
	STAA    L0062			;Save it, RPM/25
	BRA     LAD5B			;Bra 
;
LAD59	STX     L005F			;Stored DRP value

LAD5B	BCLR    L004B,#$10		;I/O word, clear b4
;
LAD5E	BCLR    L0050,#$04		;CLR b2, reset DRP test bit
	RTS				;Return
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;~~~~~~~~~~~~~~~~~~~~~
;
; Shift light routine
;
;~~~~~~~~~~~~~~~~~~~~~
;
LAD62	LDX     #L400E			;Opt. byte
	BRSET   0,X,#$04,LAD6C		;Bra if b2==1, shift light enabled
;
	JMP     LAE64			;No shift light, exit routine
;
;-Here of using shift light
;
LAD6C	BRCLR   L0050,#$10,LAD76	;Status word, bra if b4==1, in diag. mode
;
;-In diag. mode
;
	BRSET   L004F,#$80,LAD76	;Status word, bra if b7==1, engine running
;
;-Engine is off, turn on S/L 
;
	BRA     LAD8B			;Bra to return
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Check for mode 4 operation
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LAD76	BRCLR   L003B,#$10,LAD8E	;Status word, bra if b4==0, not in mode 4
;	
;-Here for mode 4 operation
;							
	LDX     #L0391			; $F4, ALDL MODE 4, DISRETE ENABLES
	BRCLR   0,X,#$01,LAD8E		; BR IF NOT b0, 1 = OVERSPEED LIGHT ENABLE 
;
	LDX     #L0392			; $F4, ALDL MODE 4, DISRETE STATES
	BRSET   0,X,#$01,LAD8B		; BR IF b0, 1 = OVERSPEED LIGHT ON 
;
	JMP     LAE03			;
;
LAD8B	JMP     LAE08			;
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Read in accumulated TOS pulse counts
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LAD8E	LDD     L3FE0			;TOS pulse accumulator
	PSHB    			;Push new value to stack
	PSHA				;    
	SUBD    L0847			;Subtract previous pulse counts
	STD     L0849			;Save it, delta TOS pulse counts
;
	PULA				;Get TOS pulse counts back    
	PULB    			;
	STD     L0847			;Save it, TOS pulse counts, one delayed
;
;~~~~~~~~~~~~~~~~~~~~
; Check quals for SL
;~~~~~~~~~~~~~~~~~~~~
;
;-Check cool temp threshold
;
	LDX     #L5109			;Start addr. of tables
	LDAA    L0006			;Load cool temp
	CMPA    $0C,X			;Compare to lower cool temp threshold for SL
	BLS     LADF3			;Bra if cool temp is <=, no SL
;
;-Check lower TPS threshold
;
	LDAA    $0D,X			;Load lower TPS threshold for SL in low gears
	LDAB    L0845			;Load gear #
	CMPB    #$02			;2nd
	BLS     LADB2			;Bra if in 1st, 2nd gear
;
	LDAA    $0E,X			;Load lower TPS threshold for SL in high gears
;
LADB2	CMPA    L01FC			;Compare threshold to TPS
	BHI     LADF3			;Bra if TPS <, no SL
;
;-Check MPH threshold
;
	LDAA    L02A7			;MPH
	CMPA    #$05			;5 MPH
	BCS     LADF3			;Bra if MPH <, no SL
;
;-Check upper RPM threshold
;
	LDAA    L0062			;RPM/25
	CMPA    $0F,X			;Upper RPM threshold for mand. SL
	BHI     LAE08			;Bra if RPMs >, turn on SL
;
;-Check lower RPM threshold
;
	CMPA    #40			;1000 RPM
	BCS     LADF3			;Bra if RPMs <, turn off SL
;
	LDAB    #$01			;1st gear
	STAB    L0845			;Save it, gear #
	PSHX				;Push addr. to stack
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Calculate ratio of engine speed to trans. output shaft speed
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; Ratio = (Engine RPS) / (40/(delta TOS pulse counts)) = Engine RPS / (TOS RPS / 10)
;
	LDD     #$0F			;15, constant
	LDX     L005F			;Stored DRPs
	FDIV				;(15/DRP val.) x 65536 = 15/DRP
	LDD     L0849			;Load stored delta TOS pulse counts
	LSRD				;/2
	ADDD    L0849			;Add TOS pulse counts back in, now TOS pulse counts x
					;(3/2)
	XGDX				;Get TOS pulse counts into X
	IDIV				;(15/DRP) / ((3/2) x TOS pulse counts)
	PSHX				;Push result to stack
	PULA				;Get result into A, B
	PULB				;
	STAB    L084B			;Save B, engine/trans RPS ratio
	PULX				;Get addr. back
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Determine what gear transmission is in
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LADE4	CMPB    0,X			;Compare current ratio to threshold
	BCC     LAE0D			;Bra if ratio is >=, current gear determined
;
;-Test next ratio
;
	INC     L0845			;+1, gear #
	INX				;+1, move to next addr.
	LDAA    L0845			;Load gear #
	CMPA    #$05			;Uppermost value for lookup, either in fourth or 
					;gearing indeterminate
	BCS     LADE4			;Bra if gear # is <, continue
;
;-Here to prepare to turn S/L off and exit
;
LADF3	LDAA    L5119			;Timer threshold to turn on S/L, low gears
	LDAB    L0845			;Gear #
	CMPB    #$02			;2nd gear
	BLS     LAE00			;Bra if in 1st/2nd
;
	LDAA    L511A			;Timer threshold to turn on S/L, high gears
;
LAE00	STAA    L0846			;Save it, S/L activation delay timer
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Here to turn shift light off
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LAE03	BCLR    L0053,#$04		;Status word, clear b2, turn off S/L
	BRA     LAE64			;Bra to return				
;
;-Here to turn shift light on
;											
LAE08	BSET    L0053,#$04		;Status word, set b2, turn on S/L
	BRA     LAE64			;Bra to return				
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Examine thresholds to determine if S/L is to be on or off
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;											
LAE0D	LDAA    L0062			;RPM/25
	CMPA    4,X			;Min RPM threshold for S/L vs. gear #
	BLS     LADF3			;Bra if RPMs <=, no S/L
;
	LDAA    8,X			;TPS load scalar vs. gear #										;
	LDAB    L01FC			;Load TPS
	MUL				;Scalar x TPS
	ASLD				;x2, now /128
	BCS     LAE21			;Bra if overflow
;
	ADDD    #128			;Round up if needed
	BCC     LAE23			;Bra if no overflow
;
LAE21	LDAA    #255			;Clear overflow
;
LAE23	PSHA				;Push TPS load val to stack   
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Look up TPS load threshold for S/L
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
	LDX     #L511B			;TPS upper hyst. threshold for S/L vs. RPM (low gears)
	LDAB    L0845			;Gear #
	CMPB    #$02			;2nd gear
	BLS     LAE31			;Bra if if in 1st or 2nd
;
;-In 3rd, 4th
;
	LDX     #L5131			;TPS upper hyst. threshold for S/L vs. RPM 
					;(high gears)
;										
LAE31	LDAB    #11			;Offset to skip to lower hyst. threshold for S/L on
	BRSET   L0053,#$04,LAE38	;Status word, bra if b2==1, S/L on
;
	ABX 				;Add in offset, move to lower threshold
;
LAE38	LDAB    #40			;1000 RPM offset for lookup
	LDAA    L0062			;RPM/25
	CMPA    #200			;5000 RPM, max for lookup
	BCS     LAE42			;Bra if RPMs <
;
	LDAA    #200			;Load max val. for lookup
;
LAE42	JSR     LF15A			;2D lookup with SBA
;	
	PSHA				;Push load threshold to stack
;
;~~~~~~~~~~~~~~~~~~~~~~~~~
; Perform baro correction
;~~~~~~~~~~~~~~~~~~~~~~~~~
;
	LDAA    L01EF			;Load baro
	LDAB    #96			;35.4 kPa, offset for lookup
;
	LDX     #L5147			;S/L load correction table vs. baro
	JSR     LF15A			;2D lookup with SBA
;										
	PULB				;Get load threshold back
	MUL				;Load threshold x baro correction mult.
	ASLD				;x2, now /128
	BCC     LAE58			;Bra if no overflow
;
	LDAA    #255			;Clear overflow
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Check if time to illuminate shift light
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LAE58	PULB				;Get TPS load value back
	CBA				;Compare threshold to load value
	BCS     LADF3			;Bra if load is >, no SL yet
;
;-Time to upshift
;
	LDAA    L0846			;SL activation delay timer
	BEQ     LAE08			;Bra if ==0, turn on shiftlight
;
	DECA				;-1
	BRA     LAE00			;Bra to save timer and exit with SL off
;
LAE64	RTS				;Return
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; Non CC trans. TCC handling routine
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;							
LAE65	LDAA    L400F			;Opt. byte
	BITA    #$40			;Test b6, TCC (non CC trans)
	BNE     LAE6F			;Bra if ==1, using TCC 
;
	JMP     LAF16			;Skip routine				
;
;-Here if using TCC
;
LAE6F	LDAA    L02A7			;MPH
	LDAB    #205			;Scalar for lookup
	MUL				;MPH x scalar
	ASLA				;x2, now scaled to 10 mph increments for lookup
	LDX     #L515E			;Lower load limit for TCC to lock
	JSR     LF15E			;2D lookup
											
	TAB				;Load limit to lock TC into B
	LDX     #L5156			;Addr. of MPH limit for mand. TCC lockup
	BRCLR   L0085,#$20,LAE8B	;Status word, bra if b5==0, TCC unlocked
;
;-TCC locked here
;
	INX				;+1, MPH limit for TCC to remain in mand. lockup
	SUBB    L5151			;TPS hyst. for TCC locked
	BCC     LAE8B			;Bra if no underflow
;
	CLRB				;Clear underflow	
;
;-Here if TCC unlocked
;	
LAE8B	LDAA    L01FC			;%TPS
	CBA				;Compare %TPS to load limit to lock TC
	BCC     LAE99			;Bra if TPS >=
;
;-Here if TPS less then limit for TCC to lock
;
	LDAA    L5155			;Coast relock delay
	STAA    L0853			;Save it 
;
	BRA     LAF0D			;Bra to unlock TCC and return				
;
;-Check for manditory lockup speed
;
LAE99	LDAA    L02A7			;MPH
	CMPA    0,X			;Manditory TCC lockup speed
	BCS     LAEA2			;Bra if MPH less, no manditory lockup
;											
	BRA     LAF02			;Bra to lock the TCC and return				
;
;-Check cool temp quals
;											
LAEA2	LDAA    L0006			;Cool temp, linearized
	CMPA    L5150			;Lower temp limit for TCC lockup
	BCS     LAF07			;Bra if cool temp less, no TCC lockup
;
;-Check throttle opening/closing rate
;
	LDAA    L01FC			;%TPS
	SUBA    L0854			;Sub. %TPS, one delayed
	BCC     LAEB7			;Bra if TPS increasing/steady
;
;-Here if throttle closing
;
	NEGA				;Get abs. val. of delta TPS
	LDAB    L5152			;Delta throttle closing rate to force TCC to unlock
	BRA     LAEBA			;Bra to contine															
;
;-Here if throttle is opening
;
LAEB7	LDAB    L5153			;Delta throttle opening rate to force TCC to unlock
;
LAEBA	CBA				;Compare to delta TPS
	BHI     LAF07			;Bra if delta TPS >, unlock TCC
;
;-Check MPH thesholds
;
	LDX     #L515A			;TCC lock/unlock MPH thresholds
	CLRB				;
;
;-Check if A/C active
;
	BRCLR   L004D,#$01,LAEC7	;I/O port C, bra if no A/C 
;
;-A/C on, adv. to next set of thresholds
;
	INX				;+1
	INX				;+1, move to next set of thresholds
;
LAEC7	BRCLR   L0085,#$20,LAECC	;Bra if b5==0, TCC not commanded to lock
;
	INX				;+1, MPH threshold to unlock TCC
;
LAECC	LDAA    L02A7			;MPH
	CMPA    0,X			;Check threshold
	BLS     LAF07			;Bra if MPH <=, unlock TCC
;
;-Check TCC load limits to lock/remain locked
;
	LDY     #L5158			;Lower RPM limit for TCC to remain locked
	LDX     #L5167			;Addr. of load limit to remain locked vs. mph
	BRSET   L0085,#$20,LAEE3	;Bra if b5==1, TCC commanded to lock
;
;-TCC unlocked here
;
	LDX     #L5174			;Addr. of load limit to lock vs. mph
	INY				;+1, rpm limit for TCC lockup
;
LAEE3	ABX				;Add in offset in B, if present
	LDAB    #205			;Scalar for MPH
	MUL				;Scalar x MPH
	ASLA				;x2
	ASLA				;x2, MPH now in 5 mph increments
	LDAB    #64			;20 MPH offset
	JSR     LF15A			;2D lookup with SBA
;											
	CMPA    L01FC			;Compare to %TPS
	BCS     LAF07			;Bra if TPS >, no lockup
;
;-Check RPM limit
;
	LDAA    L0063			;RPM/12.5
	CMPA    0,Y			;Compare to limit
	BCS     LAF07			;Bra if RPMs are less, unlock TCC
;
;-Check time delay to lock
;											
	LDAA    L0853			;TCC relock delay timer
	BEQ     LAF02			;Bra if ==0, allow TCC to lock
;
	DECA				;-1 from timer
	BRA     LAF0A			;Bra to save and return				
;
;~~~~~~~~~~~~~~~~~~~~~
; Here if lockup ok'd
;~~~~~~~~~~~~~~~~~~~~~
;
LAF02	BSET    L0085,#$20		;Status word, set b5, TCC commanded to lock
	BRA     LAF10			;bra to return															
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~
; HHere if lockup not ok'd
;~~~~~~~~~~~~~~~~~~~~~~~~~~
;											
LAF07	LDAA    L5154			;Time delay to relock TCC
;
LAF0A	STAA    L0853			;TCC relock delay timer
;
LAF0D	BCLR    L0085,#$20		;Clear b5, No TCC lockup
;	
LAF10	LDAA    L01FC			;%TPS
	STAA    L0854			;Save it, %TPS, one delayed
;
LAF16	RTS				;Return
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~     				   			  
;
;~~~~~~~~~~~~~~~~~~~~~~~
;
; Routine to check VATS
;
;~~~~~~~~~~~~~~~~~~~~~~~
;
LAF17	LDAA    L400D			;Opt. byte
	BITA    #$40			;Test b6
	BEQ     LAF3C			;Bra if not using VATS
;
;-Vats in use here
;
	BRSET   L003D,#$02,LAF3F	;Status word, bra if b1==1, VATS passed
;
	LDX     L3012			;Load TIC 2 timer val
	PSHX				;Push to stack
	PULA				;Get into AB
	PULB				;
;
	SUBD    L0851			;Previous stored TIC 2 timer val
	BEQ     LAF3F			;Bra if result ==0, VATS freq. =0
;
	STX     L0851			;Save as previous TIC 2 result
	CPD     L5181			;Upper period limit for VATS freq.
	BHI     LAF3F			;Bra if period greater, VATS fail
;
	CPD     L5183			;Lower period limit for VATS freq.
	BCS     LAF3F			;Bra if less, VATS fail
;
;-VATS passed
;
LAF3C	BSET    L003D,#$02		;Set b1, VATS passed
;
;-Here if VATS failed
;
LAF3F	RTS				;Return
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;    
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; Trans diag./misc. routine schedule
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LAF40	JSR     LAF5C			;Routine to update current gear ratio
;
	JSR     LAF62			;Transmission I/O handling routine
;
	JSR     LB106			;Routine to check for manual 2nd gear select
;
	JSR     LB12A			;Trans diagnostic/error handling routine
;
	JSR     LB291			;Routine to generate and filter needed 
					;transmission terms
;
	JSR     LB2DD			;VSS handling routine
;
	JSR     LB375			;Look up force motor gains 
;
	JSR     LF42B			;ALDL mode 4/inactivity handling routine
;
	JSR     LF2EB			;Routine to handle mode 4 engine/trans. diagnostics
;
	RTS				;Return     
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; Routine to update current gear ratio
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LAF5C	JSR     LB35E			;Routine to get gear ratio
;
	STX     L00F3			;Save it, gear ratio x 16384
	RTS				;Return
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~     
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; Transmission I/O handling routine
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LAF62	BCLR    L0076,#$80		;Status word, preclear b7, cruise control active
	LDAA    L3000			;Load port A
	BITA    #$40			;Test b6
	BEQ     LAF6F			;Bra if ==0, cruise control active
;
	BSET    L0076,#$80		;Set b7, no cruise control
;
;~~~~~~~~~~~~~~~~~~~~~~~~
; Update I/O status word
;~~~~~~~~~~~~~~~~~~~~~~~~
;
LAF6F	LDAB    L3002			;Load parallel I/O control reg.
	COMB				;Invert bits    
	ANDB    #$0F			;Clear all but lower 4 bits 
	LDAA    L3060			;Load I/O port (output feedback status)
	EORA    L3062			;EOR with I/O port controlling outputs, proper output
					;and feedback will result in bits set to 1
	ANDA    #$03			;Clear all but lower two bits corresponding to 1->2 
					;and 2->3 shift solenoid outputs and feedback
	ASLA				;Shift left...   
	ASLA				;
	ASLA				;
	ASLA				;Now status of shift solenoid outputs in b4, b5 
					;positions    
	ABA				;Sum all status bits     
	STAA    L0098			;Save it, I/O status word
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Update I/O feedback status
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
	LDX     #L004B			;I/O word
	LDAB    L00D3			;Load gear #
	CMPB    #0			;1st gear
	BNE     LAF9B			;Bra if in other gears
;
;-In first gear, check status of 3->2 downshift solenoid/output
;
	BRCLR   0,X,#$08,LAF96		;Bra if b3==0, 3->2 downshift feedback reporting 
					;circuit is open
;
;-Solenoid is at 0 DC and has power
;
	BSET    L0084,#$10		;Status word, set b4, 3->2 downshift sol. operational
	BRA     LAFB5			;Bra to continue
;
;-3->2 downshift solenoid circuit open 
;
LAF96	BCLR    L0084,#$10		;Status word, clear b4, 3->2 circuit open 
	BRA     LAFB5			;Bra to continue
;
;-Here if in other gears, should have near full DC 
;
LAF9B	BRSET   0,X,#$08,LAFAB		;Bra if b3==1, feedback reporting 0 DC 
;
;-At full DC here, output operational
;
	BRCLR   L0084,#$08,LAFA6	;Status word, output already flagged as operational 
;
	BSET    L0084,#$10		;Status word, 3-2 downshift sol. operational
;
LAFA6	BCLR    L0084,#$08		;Clear b3, output operational on this pass
	BRA     LAFB5			;Bra to continue
;
;-3->2 downshift quad driver has failed
;
LAFAB	BRSET   L0084,#$08,LAFB2	;Bra if b3==1, quad driver failure flagged last pass
;
	BSET    L0084,#$10		;Set b4 (???)
;
LAFB2	BSET    L0084,#$08		;Set b3, output has failed
;
;-Check status of TCC 
;
LAFB5	LDX     #L004B			;Index I/O word
	BRSET   0,X,#$80,LAFC5		;Bra if b7==1, TCC enabled
;
;-TCC not enabled
;
	BRSET   0,X,#$04,LAFCE		;Status word, bra if b2==1, TCC output not energized 
;
	BCLR    L0085,#$02		;Status word, clear b1, TCC output is shorted or 
					;solenoid does not have power
	BRA     LAFD1			;Bra to continue
;
;-TCC enabled here
;
LAFC5	BRCLR   0,X,#$04,LAFCE		;Bra if TCC output is energized
;
	BCLR    L0085,#$02		;Clear b1, TCC output failure or open ckt.
	BRA     LAFD1			;Bra to continue
;
;-Here if valid TCC output state
;
LAFCE	BSET    L0085,#$02		;Set b1, valid TCC output state
;
LAFD1	LDAA    L0096			;Load status word
	STAA    L0097			;Save it, status word, one delayed
	BCLR    L0096,#$04		;Preclear b2 
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Update trans. status word
;~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
	BRCLR   L003D,#$20,LAFDF	;Status word, bra if b5==0, not in PE
;
	BSET    L0096,#$04		;Set b2, PE active 
;
LAFDF	BCLR    L0096,#$80		;Preclear b7
	BRCLR   L0050,#$10,LAFED	;Status word, bra if b4==0, not in diag. mode
;
	BRSET   L0082,#$80,LAFED	;Status word, bra if b7==1, powertrain in motion, dont
					;flag diag. mode
;
	BSET    L0096,#$80		;Set b7, diags. in effect 
;
LAFED	LDY     #L5D02			;Index option bytes	
;
;~~~~~~~~~~~~~~~~~~~~~
; Update input status 
;~~~~~~~~~~~~~~~~~~~~~
;  
	LDAB    L004D			;Load I/O port C word
	TBA				;Transfer to A as well
	EORA    L0095			;XOR with stored I/O port C status 
	STAB    L0095			;Update I/O port C status
	BITA    #$01			;Test b0
	BEQ     LB001			;Bra if ==0, no change in A/C state
;
;-A/C has changed state
;
	BRSET   1,Y,#$04,LB00B		;Bra if b2==1, filtering A/C status 
;													
LB001	BCLR    L0096,#$02		;Status word, preclear b1
	BITB    #$01			;Test b0, A/C requested
	BEQ     LB00B			;Bra if ==0, no A/C
;
	BSET    L0096,#$02		;Set b1, A/C has been requested
;			
LB00B	BITA    #$80			;Test b7
	BEQ     LB01C			;Bra if ==0, no change in status of 4WD low
;
;-4WD low has changed state
;
	XGDX				;Swap current data in A,B 
	LDAA    L5D23			;Time delay for error 72 diags. when 4WD low has
					;changed state
	STAA    L019C			;Save it, error 72 diag. delay timer
	XGDX				;Get data back into A,B  
	BRSET   1,Y,#$10,LB026		;Opt. byte, bra if b4==1, filtering 4WD low input  
;
LB01C	BSET    L0096,#$20		;Status word, set b5, 4WD low active 
;									
	BITB    #$80			;Test b7 of I/O word
	BEQ     LB026			;Bra if ==0, 4WD is active
;
	BCLR    L0096,#$20		;Clear b5, no 4WD low
;								
LB026	TST     L019C			;Error 72 diag. delay timer
	BEQ     LB02E			;Bra if ==0
;
	DEC     L019C			;Decrement timer
;										
LB02E	BITA    #$1C			;Test range select state bits b2-b4
	BEQ     LB037			;Bra if ==0, no change in state
;
;-Range inputs have changed state
;
	BRSET   1,Y,#$02,LB054		;Opt. byte, bra if b1==1, filtering range select
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Look up range select value based on range inputs
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LB037	ANDB    #$1C			;Clear all but range indicator bits
	LSRB				;Shift right
	LSRB				;Shift right, now in lower three bits for indexing
	LDX     #LB0FE			;Range select value vs. range input
	ABX				;Add in status bits as index for lookup
	LDAB    0,X			;Load range select value	
	LDAA    L009B			;Range select timer
	INCA				;+1
	BEQ     LB048			;Bra if no roll-over
;
	STAA    L009B			;Save timer
;											
LB048	LDAA    L009C			;Load range select word
	STAA    L009D			;Save it, range select word, one delayed
	CBA				;Compare current selected range to previous
	BEQ     LB054			;Bra if no change in selected gear range
;
;-New gear range selected
;
	CLR     L009B			;Clear range select timer, new gear range has been 
					;selected
	STAB    L009C			;Update range select word
;	
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Check state of cruise cont'l
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;									
LB054	LDAB    L0076			;Load status word
	TBA				;Transfer to A as well
	EORA    L0077			;XOR with stored status word state
	STAB    L0077			;Update stored status word
	BITA    #$80			;Test b7, cruise control state
	BEQ     LB064			;Bra if ==0, no change in cruise cont'l state
;
;-Cruise control has changed state
;
	BRSET   1,Y,#$08,LB06E		;L5D03, bra if b3==1, filtering cruise input
;
LB064	BSET    L0096,#$40		;Status word, set b6, cruise cont'l active
	BITB    #$80			;Test b7 of L0076
	BEQ     LB06E			;Bra if ==0, cruise control active
; 
	BCLR    L0096,#$40		;Clear b6, cruise control not active
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Calc. high res engine RPM
;~~~~~~~~~~~~~~~~~~~~~~~~~~~
;											            
LB06E	LDD     #0120			;Constant
	LDX     L005F			;Stored DRP
	FDIV				;120/DRP x 65536 = 120/(60x65536/(RPM x 4)) x 65536 =
					;RPM x 8   
	XGDX				;Get result into D    
	CPD     #0120			;15 RPM, lower allowed value
	BHI     LB07D			;Bra if >
;
	CLRA				;Clear RPMs 	   
	CLRB				;
;    
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Calculate trans input speed from output speed
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LB07D	STD     L00B1			;Save it, engine RPM x 8
;
	LDD     L00F3			;Load gear ratio x 16384
	LDX     L00CB			;TOS RPM x 8
	JSR     LF23F			;16 x 16 mult. routine, now trans input speed x 120
;
	ASLD				;x2    
	BCS     LB096			;Bra if overflow
;
	ASLD				;x2, now TIS RPM x 8
	BCS     LB096			;Bra if overflow
;
;-Shift the lower two bytes of result as well to preserve resolution
;
	XGDX				;Get lower two bytes of result into D    
	ROLA				;Rotate left, get two upper bits into lower portion
					;of A    
	ROLA				;...    
	ROLA				;...    
	ANDA    #$0003			;Clear all but two lower bits
	TAB				;Get result into B     
	ABX				;Add in two shifted bits into result     
	BRA     LB099			;Bra to save and continue
;
LB096	LDX     #$FFFF			;Clear overflow
;
LB099	STX     L00B7			;
	STX     L00BE			;
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Determine transmission output shaft speed
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
	LDX     #L3FE0			;TOS pulse accumulator
	LDY     #L3FC2			;TOS 65,536 Hz free running counter, captured
					;on each edge detected			
;
	LDAA    #$10			;0001 0000
	BITA    L5D02			;Opt. byte, test b4 
	BEQ     LB0B2			;Bra if ==0, speed from dig. ratio adaptor
;
	LDX     #L3FC6			;Alternate TOS PA
	LDY     #L3FF8			;Alternate TOS counter
;
LB0B2	SEI				;Disable interrupts     
	LDD     0,X			;Load accumulated pulse counts
	JSR     LF08A			;Short delay
;
	PSHY				;Push counter value to stack    
	LDY     0,Y			;Load counter value
	JSR     LF08A			;Short delay
;
;-Check to see if another pulse has been recieved since accumulated TOS counts loaded
;
	CPD     0,X			;Compare acc. counts to current 
	BEQ     LB0D0			;Bra if accumulated counts have not changed
;
;-Another edge has been detected
;
	JSR     LF08A			;Short delay
;
	PULY				;Get counter addr. back off stack    
	LDY     0,Y			;Update free running counter value
	INCB				;+1, accumulated counts
	BRA     LB0D2			;Bra to continue
;
;-Here if no new edge detected
;
LB0D0	INS				;+1    
	INS				;+1, clean up stack   
;
;-Save the counter values
;
LB0D2	LDX     #L00C2			;Start addr. of stored TOS params
	STAB    8,X			;Store accumulated pulse counts at L00CA
	STY     6,X			;Store captured free running counter val. 
					;at L00C8,C9
	CLI				;Re-enable interrupts     
	LDY     #L00C2			;Addr. of accumulated pulse counts
	LDX     L5D13			;Constant for TOS RPM calc
	LDAB    L5D12			;Max time to wait for TOS signal
;
	JSR     LE34C			;Routine to calculate transmission output speed
;
;-Check if 4WD low is in use
;
	BRCLR   L0096,#$20,LB0FB	;Status word, bra if b5==0, no 4WD low
;
;-4WD low is in use
;
	LDX     L5D16			;Load 4WD low ratio reduction
	JSR     LF23F			;16 x 16 mult. routine
;
	ASLD				;x2
	BCS     LB0F8			;Bra if no overflow
;
	ASLD				;x2, now result/16384
	BCC     LB0FB			;Bra if no overflow
;
LB0F8	LDD     #$FFFF			;Clear overflow
;
;-Here to save trans output rotational speed
;
LB0FB	STD     L00C6			;Save it, TOS RPM x 8
	RTS				;Return
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~  
;-Range select value vs. range input 
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;	Value used		;Selected gear range
;
LB0FE	FCB	$88	 	;Invalid gear (all grounded)
	FCB	$08	 	;Forward D4
	FCB	$88	 	;Invalid gear
	FCB	$04	 	;Forward D3
	FCB	$20	 	;Reverse
	FCB	$40	 	;Park/neutral
	FCB	$01	 	;Forward D1
	FCB	$02	 	;Forward D2
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; Routine to check for manual 2nd gear select
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LB106	LDAA    L0099			;Trans shift pattern word
	STAA    L009A			;Previoud trans shift pattern 
;
	CLR     L0099			;Preclear trans shift pattern
;
	LDX     #L5D02			;Opt byte
	BRCLR   0,X,#$20,LB121		;Bra if b5==0, not forcing 2nd gear when in manual D2
;
	BRSET   L0092,#$04,LB121	;Bra if b2==0, trans. press. switch. error
;
;-Here if no errors, check for manual second
;
	BRCLR   L009C,#$02,LB121	;Bra if not in second gear
;
	BSET    L0099,#$04		;Set b2, manual pattern requested
	BRA     LB124			;Bra to return
;
LB121	BSET    L0099,#$01		;Set b0, normal pattern requested 
;
LB124	RTS				;Return  
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;~~~~~~~~~~~~~~~~~~~~~
;
;~~~~~~~~~~~~~~~~~~~~~
;   
LB125	FCB	$09		;
LB126	FCB	$01		;
LB127	FCB	$02		;
LB128	FCB	$04		;
LB129	FCB	$09		;
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; Trans diagnostic/error handling routine
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LB12A	JSR     LE677			;Routine to check for error 28, pressure manifold 
					;switch
;
	JSR     LEA7B			;Error 77, manual/normal/performance switch routine
;
	JSR     LE4BA			;Routine to check if error 28 present
;
	BRCLR   L0092,#$04,LB13B	;Status word, bra if b4==0, no error 28 present
;
;-Error 28 present, clear range word
;
	LDAA    #$88			;Default, set invalid range
	STAA    L009C			;Save it, trans range word
;									
LB13B	LDX     #L5B00			;Index diag. masks
	LDAA    L0099			;Load trans shift pattern word 
	BRCLR   $32,X,#$20,LB14E	;L5B32, bra if b5==0, not flagging error 77
;								
LB144	BRCLR   L001D,#$20,LB14E	;Error word #8, bra if b5==0, no error 77 is set
;
;-Error 77 present, force normal shift pattern
;
	ANDA    #$F0			;Clear b0-b3
	ORAA    #$09			;Set b3, illegal pattern, and b1, normal shift pattern
	BRA     LB15D			;Bra to continue				
;
;-Check if manual pattern set, and force normal pattern if error 28 present 
;
LB14E	BITA    #$04			;Test b2, manual pattern flag
	BEQ     LB15F			;Bra if ==0, manual pattern not requested
;										 
	BRSET   L0092,#$04,LB15B	;Bra if b2==1, error 28 present, force normal pattern
;
;-Error 28 not present, check if option to disable manual pattern set
;
	LDAB    L5D0D			;Load option word
	BEQ     LB15D			;Bra if ==0, dont disable manual pattern				
;
;-Here to disable manual pattern
;
LB15B	EORA    #$05			;Clear b2, manual pattern, and set b0, normal pattern			
;
LB15D	STAA    L0099			;Save shift pattern word
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Call transmission diagnostics routines
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LB15F	JSR     LE576			;Routine to check for error 21, high TPS 
;
	JSR     LE5B7			;Routine to check for error 22, low TPS
;
	JSR     LE38F			;Routine to check if TPS errors flagged
;
	JSR     LE96F			;Routine to check for error 72, output speed lost
;
	LDX     #L400F			;Opt. byte
	BRSET   0,X,#$80,LB175		;Bra if b7==1, manual trans in use
;
	JSR     LE5F6			;Routine to check for error 24, low VSS 
;
LB175	JSR     LEDD4			;Not used
;
	JSR     LE930			;Routine to check for error 71, low engine speed
;
	JSR     LE3CB			;Routine to flag error 24/72
;
	JSR     LE4A4			;Routine to flag error 72
;
	JSR     LE3B1			;Routine to flag high system voltage
;
	RTS				;Return  
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~   
;
;~~~~~~~~~~~~~~~~~~~~~~
;
; TPS handling routine
;
;~~~~~~~~~~~~~~~~~~~~~~
;
;-Check for TPS errors
;
;-Check for error 21, high TPS
;
LB185	LDX     #L5B00			;Start addr. of diag masks	
	BRSET   L0092,#$01,LB1B5	;Status word, bra if b0==1, Trans. TPS error flagged
;
	BRCLR   $2B,X,#$01,LB199	;Error mask, bra if b0==0, not setting error 21, High TPS
;
	BRSET   $34,X,#$01,LB199	;Alt. error mask, bra if b0==1, not setting error 21. 
;
	LDAA    L015B			;Error 21 timer
	BNE     LB1A6			;Bra if !=0
;
;-Check for error 22, low TPS
;
LB199	BRCLR   $2C,X,#$80,LB1B7	;Error mask, bra if b7==0, not setting error 22, TPS high
;
	BRSET   $35,X,#$80,LB1B7	;Alt. mask, bra if b7==1, not setting error 22

	LDAA    L015C			;Error 22 timer
	BEQ     LB1B7			;Bra if ==0, continue normally 
;
;~~~~~~~~~~~~~~~~~~~
; Here if TPS error 
;~~~~~~~~~~~~~~~~~~~
;
LB1A6	LDX     #L5D02			;Option byte
	BRCLR   0,X,#$01,LB1B2		;Bra if b0==0, dont allow TPS hyst. buffer every 25 msecs
;
;-Use four delayed TPS here
; 
	LDAA    L019B			;%TPS, four delayed
	BRA     LB219			;Bra
;
LB1B2	JMP     LB223			;
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Here if trans TPS error set 
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LB1B5	BRA     LB213			;Bra to load defaults
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;Here if no TPS error flagged, check and filter closed TPS down to lowest A/D reading
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LB1B7	LDAA    L00A0			;A/D TPS
	CMPA    L0319			;Base TPS for closed throttle
	BHI     LB1CD			;Bra if A/D TPS is
;
;-Filter base TPS down to A/D TPS
;
	LDAB    #128			;Set LSB
	LDX     L0319			;Base TPS for closed throttle
	LDY     #L5B25			;%TPS filter coeff.
	JSR     LF0D3			;Lag filter routine
;
	STD     L0319			;Save it, base TPS
;
;-Here if throttle kicker TPS was less
;
LB1CD	LDAA    L00F1			;MPH x 2, one delayed
	SUBA    L00D1			;MPH x 2, filtered
	BCC     LB1D8			;Bra if speed decreasing
;
;-Here if speed increasing
;
	BCLR    L0094,#$04		;Status word, clear b4, will be adding TPS on decel
	BRA     LB201			;Bra
;
;-Here if speed decreasing
;
LB1D8	BRSET   L0094,#$04,LB201	;Bra if b2==1, already added in decel TPS
;
	BRSET   L0018,#$02,LB201	;Error word #5, bra if b1==1, error 37, TCC brake switch stuck on.
;
	BRSET   L0018,#$01,LB201	;Error word #5, bra if b0==1, error 38, TCC brake switch stuck off
;
	BRCLR   L0096,#$01,LB201	;Bra if b0==1, brakes applied

	CMPA    L5B2A			;Threshold for bumping idle TPS
	BLS     LB205			;Bra if MPH less
;
;-Speed greater then 5 mph here and speed decreasing, increase throttle kicker TPS
;
	LDAA    L0319			;Base TPS for closed throttle
	ADDA    L5B29			;Additional TPS when in decel		
	CMPA    L5B24			;Max value for idle TPS
	BLS     LB1FB			;Bra if less
;
	LDAA    L5B24			;Load max
;
LB1FB	STAA    L0319			;Idle TPS
	BSET    L0094,#$04		;Set b1, additional TPS added in for decel
;
;-Here if speed increasing or brakes have been applied
;
LB201	LDAA    L00D1			;Filtered MPH x 2
	STAA    L00F1			;Save it, MPH x 2, one delayed
;
;-Shuffle %TPS values
;
; %TPS -> %TPS, one delayed -> %TPS, two delayed -> %TPS, three delayed -> %TPS, four delayed
;
LB205	LDD     L0199			;%TPS, two delayed/three delayed
	STD     L019A			;Save it, %TPS, three delayed/four delayed
	LDD     L0197 			;Current TPS/TPS, one delayed
	STD     L0198			;Save it, TPS, one delayed/two delayed
	BRA     LB219			;Bra
;
;-Here to load default TPS value if error
;
LB213	LDAA    L5B22			;Default TPS
	CLRB    			;Clear LSB
	BRA     LB221			;Bra to save
;
;-Here if no error
;
LB219	LDX     L00AA			;Trans. TPS
	LDAB    L5B28			;Filter coeff for trans. TPS
	JSR     LF0F6			;Filter routine
;
LB221	STD     L00AA			;Store it, trans. TPS

LB223	LDAA    L00AA			;Trans %TPS
	SUBA    #128			;Subtract 50% TPS
	ADDA    L0101			; 
	BVC     LB230			;Bra if no 2's comp. overflow
;
;-Clear 2's overflow
;
	LDAA    #$7F			;Load max signed %TPS
	ADCA    #$00			;Perform carry if needed
;
LB230	ADDA    #128			;Add 50% tps back in
	STAA    L00AC			;Save trans %TPS
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Check quals for enabling kickdown shift pattern
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
	LDAB    L5D1C			;90.2% TPS, threshold to disable kickdown 
	BRSET   L0096,#$10,LB23E	;Bra if kickdown enabled
;
	LDAB    L5D1B			;90.2% TPS, threshold for kickdown 
;
LB23E	BCLR    L0096,#$10		;Clear b4, kickdown enable
;											
	CMPB    L00AA			;Trans. TPS
	BCC     LB24C			;Bra if TPS <=
;
;-TPS above threshold, enable kickdown if no manual pattern requested
;											
	BRSET   L0099,#$04,LB24C	;Bra if b2==1, manual pattern requested 
;
	BSET    L0096,#$10		;Set b4, enable kickdown
;											
LB24C	RTS				;Return 
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~   
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; TPS self calibration routine
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LB24D	LDAA    L00A0			;A/D TPS
	CMPA    L0319			;Base TPS for closed throttle
	BLS     LB26F			;Bra if A/D TPS <= base %TPS, return
;
;-Filter idle TPS if A/D TPS >
;
	LDAB    #128			;Use 128 as LSB of filtered value
	LDX     L0319			;Old %TPS
	LDY     #L5B27			;TPS filter coeff.
	JSR     LF0D3			;Lag filter routine
;
	STD     L0319			;Save it, filtered base TPS
	CMPA    L5B24			;Max val. for %TPS
	BCS     LB26F			;Bra if <
;
	LDAA    L5B24			;Load max
	CLRB    			;
	STD     L0319			;Save it, Base TPS for closed throttle
;
LB26F	RTS				;Return
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~    
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; Routine to construct TPS
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LB270	LDD     L0319			;Throttle kicker TPS
	ADDD    #128			;Round if needed
	TAB				;Throttle kicker TPS to B     
	LDAA    L00A0			;Load A/D TPS
	SBA				;Subtract idle TPS from A/D TPS    
	BCC     LB27D			;Bra if no underflow

	CLRA				;Clear underflow 
;  
;-Scale TPS for save
;
LB27D	LDAB    L5B26			;Scalar for TPS
	MUL				;TPS x scalar 
	ADDD    #32			;Round if needed
	ASLD				;x2
	BCS     LB28A			;Bra if overflow
;
	ASLD				;x2   
	BCC     LB28D			;Bra if overflow
;
LB28A	LDD     #$FFFF			;Clear overflow
;
;-TPS = (L5B26 x (A/D TPS-base TPS)/64
;
LB28D	STAA    L0197			;Current TPS
	RTS 				;Return    
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; Routine to generate and filter needed transmission terms
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;-Filter engine RPM
;
LB291	LDD     L00B1			;Engine RPM x 8
	LDX     L019D			;Engine RPM x 8, filtered
	LDY     #L5D0F 			;Load addr. of filter coeff.
	JSR     LF0D3			;Filter routine
;
	STD     L019D			;Save it
;
;-Filter Transmisison input speed RPMs
;
	LDD     L00B7			;(T)rans (I)nput (S)peed RPM x 8
	LDX     L00BC			;TIS RPM x 8, filtered
	LDY     #L5D10			;Filter coeff. for TIS
	JSR     LF0D3			;Filter routine
;
	STD     L00BC			;Save it
;
	LDD     L00BE			;(T)rans (I)nput (S)peed RPM x 8
	LDX     L00C0			;TIS RPM x 8, filtered
;
LB2B1	LDY     #L5D11			;Filter coefficient for TIS RPM
	JSR     LF0D3			;Filter routine
;
	STD     L00C0			;Save it
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Calculate torque converter slip 
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
	LDD     L00B1			;Engine RPM x 8
	SUBD    L00BE			;Subtract TIS RPM x 8
	RORA				;Test for possible twos-compliment overflow    
	ROLA				; 	   
	BVC     LB2C9			;Bra if overflow clear
;
	LDD     #32767			;clear overflow
;
	ADCB    #$0000			;Perform carry if overflow from subtraction
	ADCA    #$0000			;
;
;-Filter and save the TCC slip ratio
;
LB2C9	LDX     L00E5			;Load filtered torque converter slip 
	LDY     #L5D0B			;Addr. of filter coeff.
	JSR     LF0BD			;Signed filter routine
;
	STD     L00E5			;Save it, filtered torque converter slip
	BPL     LB2DA			;Bra if result is >= 0 
;
;-Negative result, get absolute value
;
	NEGA				;Negate   
	NEGB				;    	
	SBCA    #$00			;Perform carry if needed
;
LB2DA	STD     L00E7			;Save it, magnitude of torque converter slip 
	RTS				;Return 
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~    	
;
;~~~~~~~~~~~~~~~~~~~~~~
;
; VSS handling routine
;
;~~~~~~~~~~~~~~~~~~~~~~
;
LB2DD	BRCLR   L0093,#$04,LB2FF	;Status word, bra if b2==0, no loss of output speed
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Calculate approximate trans output speed from engine RPMs if output speed lost
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;
;-Error, generate default output speed
;
	JSR     LB35E			;Read in gear ratio
;
;-Output speed = (RPM x 4 / ratio) x 65536
;
	LDD     L00B1			;RPM x 8
	LSRD				;/2    
	LSRD				;/2
	FDIV				;frac. divide
	XGDX				;Get result into D
	BRCLR   L0096,#$20,LB318	;Status word, bra if b5==0, not in 4WD
;
;-In 4WD here
;
	LDX     L5D16			;Additional reduction ratio for xfer case
	JSR     LF23F			;16 x 16 multiply routine
;
	ASLD				;x2    
	BCS     LB2FA			;Bra if overflow
;
	ASLD				;x2    
	BCC     LB318			;Bra if no overflow
;
LB2FA	LDD     #$FFFF			;Clear overflow
	BRA     LB318			;Bra to save and continue
;
;~~~~~~~~~~~~~~~~~~~
; Here if no errors
;~~~~~~~~~~~~~~~~~~~
;
LB2FF	LDAA    L015D			;Error 24 timer, VSS
	ORAA    L016E			;Error 72 timer, output speed loss
	BNE     LB316			;Bra if !=0
;
;-No errors pending here
;
	LDD     L00C6			;TOS RPM x 8
	LDX     L00CD			;TOS RPM x 8, filtered
	BRSET   L0082,#$08,LB312	;Status word, bra if b3==1, ignition is on
;
	LDX     #$0000			;Ignition off, clear TOS RPM
;
LB312	STX     L00CF			;Stored TOS RPMs
	BRA     LB318			;Bra to continue
;
;-Errors pending here
;
LB316	LDD     L00CF			;Load last known good TOS RPMs
;
LB318	STD     L00CB			;TOS RPM x 8
;
;-Transmission output speed filter
;
	LDX     L00CD			;Load TOS RPM x 8, filtered
	LDY     #L5D15			;Trans. output speed filter coeff.
	JSR     LF0D3			;Filter routine
;
	STD     L00CD			;Save it, filtered TOS RPMs
;
;~~~~~~~~~~~~~~~~~~~~~~~~
; Calculate MPH from TOS 
;~~~~~~~~~~~~~~~~~~~~~~~~
;
	LDD     L00CD			;Filtered TOS RPMs x 8
	LDX     L5D18			;Conversion factor for converting TOS to MPH x 256
	FDIV				;(TOS RPMs x 8)/conversion factor    
	XGDX				;Get result into D    	
	SUBA    #128			;
	ADDA    L0102			;
	BVC     LB33A			;Bra if signed overflow clear
;
	LDD     #32767			;Clear overflow
	ADCB    #0			;
	ADCA    #0			;
;
LB33A	ADDA    #128			;
	ADDD    #128			;
	BCC     LB344			;Bra if no overflow
;
	LDD     #$FFFF			;Clear overflow
;
LB344	STD     L0814			;Save it, MPH x 256
;
;-Filter the MPH
;
	LDX     L00D1			;Load filtered MPH x 2
	LDY     #L5D1A			;Filter coeff. for MPH
	JSR     LF0D3			;Filter routine
;
	STD     L00D1			;Save it, filtered MPH x 2
	LDAB    L00D1			;Load MPH x 2
	CLRA				;    
	XGDX				;Get MPH into X
	LDD     L019D			;Engine RPM x 8
	IDIV				;(engine RPM x 8 / MPH x 2)    
	STX     L0190			;Save it, RPM/MPH ratio x 4
;
	RTS				;Return     
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; Routine to fetch trans gear ratios
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LB35E	LDX	L5B43			;Load ratio for reverse gear 
	BRSET   L009C,#$20,LB374	;Bra if b5==1, in reverse
;
	LDX     #L5B49			;Trans ratios for gears 1st-4th
	LDAB    L00D3			;Load gear number 
	BRCLR   L0092,#$04,LB370	;Status word, bra if b2==0, no press. switch. man. error
;
;-Error, load default gear #
;
	LDAB    #2			;Error, load 1st as default
;
LB370	ASLB				;x2 for 16 bit addr. 
	ABX				;Add in offset     
	LDX     0,X			;Load gear ratio
;
;-Here if in reverse
;
LB374	RTS				;Return     
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; Look up force motor gains 
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; Trans temp value for lookup = (trans temp x 8)/256, limit value to <8 for indexed lookup
;
LB375	LDAA    L00AF			;Trans temp, filtered
	LDAB    #08			;8, scaler for table lookup
	MUL				;Trans temp x 8     
	SUBD    #01			;-1
	BLT     LB384			;Bra if < 0
;
	CMPA    L014C			;Previously stored temp
	BGE     LB38C			;Bra if >= previously stored result, save
;
;-Here if trans temp = -40 deg C or < previously stored result 
;
LB384	ADDD    #17			;+17, 1.5 deg C hyst.
	CMPA    L014C			;Previously stored result		
	BGE     LB38F			;Bra if >= previously stored result, dont save
;
;-Here if trans temp > stored value or temp difference less then 1.5 deg C
;
LB38C	STAA    L014C			;Save it, trans temp value used for lookup
;
LB38F	LDAB    L014C			;Load in stored temp value
	LDX     #LB3A5			;Get table addr.
	ABX				;Add in offset     
	LDAA    0,X			;Load table value
	STAA    L014D			;Save it, proportional gain
	LDAA    8,X			;Load value 8 bytes ahead
	STAA    L014E			;Save it, proportional gain, one delayed
	RTS 				;Return    
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Shift solenoid state table
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; b0==1, 1-2 shift sol. on
; b1==1, 2-3 shift sol. on
;				;Gear #
;
LB3A1	FCB   3			;1st
	FCB   2			;2nd
	FCB   0			;3rd
	FCB   1			;4th
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Force motor proportional velocity gains
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;	% gain = val x .0977	;Deg C			
;
;-Gain for current error
;
LB3A5	FCB   70		;-40
	FCB   64     		;-16
	FCB   60    		;  8
	FCB   58     		; 32
	FCB   54     		; 55
	FCB   52     		; 80
	FCB   52     		;103
	FCB   52     		;127
;
;-Gain for error, one delayed
;
	FCB   41     		;-40
	FCB   35     		;-16
	FCB   31     		;  8
	FCB   28     		; 32
	FCB   24     		; 55
	FCB   21     		; 80
	FCB   19     		;103
	FCB   17    		;127
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; Schedule of various transmission routines
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LB3B5	LDX	#L400F			;Index option byte	  
	BRCLR   0,X,#$C0,LB3BE		;Bra if b6, TCC only, and b7, manual trans in use ==0
;
;-TCC only or manual trans in use, return
;
	BRA     LB3C7			;Bra to return
;
;-CC trans in use
;
LB3BE	JSR     LB3C8			;Routine to check if powertrain is in motion
;
	JSR     LB3FB			;Routine to compute the average sum of the change 
					;in engine RPM
;
	JSR     LB486			;Main transmission line pressure modulation routine
;
LB3C7	RTS				;Return     
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; Routine to check if powertrain is in motion
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;-Check engine RPMs
;
LB3C8	LDD     L019D			;Engine RPM x 8	
	CPD     L5D1D			;Min engine RPM for powertrain to be in motion
	BCC     LB3F4			;Bra if engine RPM >=, powertrain in motion
;
;-Check transmission output speed RPMs
;
	LDD     L00CD			;Filtered TOS RPM x 8
	CPD     L5D1F			;Min TOS RPM for powertrain to be in motion
	BCC     LB3F4			;Bra if TOS RPM >= 
;
;-Check transmission input speed RPMs
;
	LDD     L00C0			;Filtered TIS RPM x 8
	CPD     L5D21			;Min TIS RPM for powertrain to be in motion
	BCC     LB3F4			;Bra if TIS RPM >=
;
	BRCLR   L0082,#$80,LB3FA	;Status word, bra if b7==0, powertrain in motion flag 
					;cleared already
;
;-Powertrain no longer in motion, check if enough time has elapsed to clear flag
;
	INC     L00FA			;+1, timer for powertrain in motion flag
	LDAA    L5D0C			;Min time to clear power train in motion flag
	CMPA    L00FA			;Compare to timer
	BCC     LB3FA			;Bra if timer <=
;
	BCLR    L0082,#$80		;Clear b7, powertrain no longer in motion
;
	BRA     LB3FA			;Bra to return
;
;-Powertrain is in motion here
;
LB3F4	CLR     L00FA			;Clear timer for powertrain in motion flag
	BSET    L0082,#$80		;Set b7, powertrain in motion
;
LB3FA	RTS				;Return    
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; Routine to compute the average sum of the change in engine RPM
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; Formula:
;
; deltaRPMsum_n = deltaRPMsum_n-1 + (n-1)/2 x (RPM_n + RPM_o) - sum(RPM_n,0,n-1)
;
LB3FB	CLR     L01C9			;Clear sample counter
	LDAA    L01CA			;Load uppermost byte of sum
	STAA    L01CD			;Save it, sum from previous pass
	LDD     L01CB			;Load two lower bytes of sum
	STD     L01CE			;Save it, sum from previous pass
	LDX     #L019D			;Index engine RPM x 8
;
;-Subtract the sum of all previous samples 
;
LB40D	INX				;+1,     
	INX				;+1, move to previously sampled engine PRM     	
	PSHB				;LSB to stack    
	LDAB    L01C9			;Load sample counter
	INCB				;+1, move to next sample    
	CMPB    L5D0E			;# of total collected samples
	BEQ     LB426			;Bra if ==, at end of sampled set
;
	STAB    L01C9			;Save counter
	PULB				;Get LSB back    
	SUBD    0,X			;Subtract previous value from current sum
	BCC     LB424			;Bra if no underflow
;
	DEC     L01CA			;Perform borrow
;
LB424	BRA     LB40D			;Bra to continue loop
;
;-Add in current sample and initial sample times # of delta RPM samples
;
LB426	PULB				;Get MSB back    
	STD     L01CB			;Save lower two bytes of result
	LDD     L019D			;Load engine RPM x 8
	ADDD    0,X			;Add in initial sample
	PSHB				;Push LSB to stack    
	BCC     LB43D			;Bra if no overflow
;
;-Overflow, perform carry
;
	LDAB    L5D0E			;Load # of samples
	DECB				;-1    
	LSRB				;/2, now (n_samples-1)/2    
	ADDB    L01CA			;Add in the uppermost byte
	STAB    L01CA			;Save it, uppermost byte of sum
;
;-Here if no overflow, add in current values to sum
;
LB43D	LDAB    L5D0E			;Load in # of samples
	DECB				;-1   
	LSRB				;/2, now (n_samples-1)/2    
	PSHB				;Push LSB to stack   
	MUL				;(n_samples-1)/2 x MSB (RPM_n + RPM_o)     
	ADDD    L01CA			;Add it into the delta RPM sum
	STD     L01CA			;Save the sum
	PULA				;Get multiplier back    
	PULB				;Get mult'd back    
	MUL				;(n_samples-1)/2 x LSB (RPM_n + RPM_o)     
	ADDD    L01CB			;Add in previous sum
	STD     L01CB			;Save it, sum of delta engine RPMs
	BCC     LB458			;Bra if no overflow
;
	INC     L01CA			;Perform carry
;
LB458	LDD     L01CA			;Load sum of delta engine RPMs x 8
	LSRD				;/2    
	LSRD				;/2  
	LSRD				;/2    
	ADCB    #$0080			;Make value centered about 128
	STAB    L01D0			;Save it, delta engine RPMs for ALDL diags.
;
;-Flag direction of rate of change in engine RPMs
;
	BSET    L0081,#$01		;Status word, set b0, slope of derivative of engine
					;RPMs is negative
	LDD     L01CA			;Load current sum of change in engine RPMs
	CPD     L01CD			;Compare to previous sum
	BLS     LB472			;Bra if current unsigned sum is less
;
	BCLR    L0081,#$01		;Clear b0, slope of derivative of engine RPMs is 
					;positive
;    
;-Update sampled engine RPMs
;
LB472	LDX     #L019B			;Index start addr.
	LDAB    L5D0E			;# of samples to use
	ASLB				;x2 for double byte values    
	ABX				;Add in offset
; 
LB47A	LDD     0,X			;Load sampled engine RPMs
	STD     2,X			;Save it, previously sampled RPM
	DEX				;-1     
	DEX				;-1, move to next sampled RPM     
	CPX     #L019D			;Current engine RPM x 8
	BCC     LB47A			;Bra if indexed address is >=
;
	RTS				;Return  
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;	
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; Main transmission line pressure modulation routine
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;  
LB486	LDAB    L00D5			;Load shift timer
	INCB				;+1
	BEQ     LB48D			;Bra if roll-over
;								
	STAB    L00D5			;Save it, shift timer
;
LB48D	BRSET   L0081,#$02,LB4DD	;Status word, bra if b1==1, trans shift status
					;flaged already
;										
	BRCLR   L007F,#$09,LB4E8	;Status word, bra if b0, upshift underway, and 
					;b3, downshift underway flags==0
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Shift pending, check and flag status 
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;							
	BSET    L0081,#$02		;Set b1, trans shift status flagged
	BCLR    L0081,#$91		;Clear b0, b4, and b7
	BCLR    L007F,#$12		;Preclear shift status flags
	BRCLR   L007F,#$01,LB4A5	;Bra if b0==0, upshift not in progress
;							
	BSET    L007F,#$02		;Set b1, flag trans upshift
;
LB4A5	BRCLR   L007F,#$08,LB4AC	;Bra if b3==0, downshift not in progress
;							
	BSET    L007F,#$10		;Set b4, flag downshift
;
LB4AC	BCLR    L008A,#$08		;Status word, clear b3, range select status flag
;										
	LDAA    L009B			;Load range select timer
	BNE     LB4B6			;Bra if !=0, no recent change in range status
;	
;-Selected range has changed
;									
LB4B3	BSET    L008A,#$08		;Set b3, selected range has changed
;
;-Perform resets, shift pending
;									
LB4B6	LDAA    L00D5			;Load shift timer
	STAA    L0181			;Save it, shift duration time
	CLRA				;Preclear A
	STAA    L00D4			;Clear flare timer
	STAA    L00D5			;Clear shift timer
	STAA    L00F7			;
	LDAA    #$007F			;Preload 127
	STAA    L00D9			;Reset stored underpressure term
	STAA    L00DA			;Reset stored overpressure term
	CLRA				;Preclear A
	LDAB    L009A			;Shift pattern word, one delayed
	EORB    L0099			;Shift pattern word
	ANDB    #$0007			;Clear all but lower three bytes
	BEQ     LB4D3			;Bra if ==0, no change in status
;
;-Shift pattern has changed
;
	LDAA    #$0001			;Set b0, shift pattern has changed
;
LB4D3	STAA    L008B			;Save it, status word
	CLRA				;
	STAA    L008C			;Clear status word
	LDAA    L00AC			;Load trans TPS
	STAA    L018B			;Save it, trans TPS at start of shift
;
;-Check and reset trans shift status flag if no shifts in progress
;
LB4DD	BRSET   L007F,#$01,LB4E8	;Status word, bra if b0==1, upshift in progress
;									
	BRSET   L007F,#$08,LB4E8	;Bra if b3==1, downshift in progress 
;									
	BCLR    L0081,#$02		;Clear b1, reset trans status flag
;								
LB4E8	BRSET   L0081,#$80,LB552	;Status word, bra if b7==1, time threshold for shift
					;to be complete has been met
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Check time limits for timer updates and resetting shift status
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;									
	LDAA    L00D5			;Load shift timer
	SUBA    L00D4			;Subtract shift flare timer
	STAA    L00D6			;Save it, shift duration timer 
	LDAB    L00D3			;Load gear #
	BRSET   L007F,#$02,LB504	;Bra if b1==1, trans upshift flagged
;
;-Trans upshift not flagged
;
	LDAA    L00D5			;Load shift timer
	LDX     #L5FE6			;Max alloted time for shift vs. gear #  
	ABX				;Add gear # in as offset
	CMPA    0,X			;Compare shift timer to time limit
	BCS     LB552			;Bra if time <, return
;										
	BRA     LB549			;Bra to flag shift as complete and reset
;	
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Upshift in progress determine start of shift
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;								
LB504	LDX     #L5FCF			;Index shift params
	ASLB				;Gear # x2 for double byte addr.
	ABX				;Add in gear # as offset
	LDD     L01CA			;Load sum of change in engine RPM
	BRSET   L0081,#$10,LB524	;Status word, bra if b4==1, shift has already started
;	
;-Check if derivative of engine speed is increasing
;					
	BRCLR   L0081,#$01,LB51E	;Status word, bra if b0==0, slope of derivative of
					;engine RPMs is positive, shift is flaring or has just 
					;started
;
;-Derivative of engine speed is negative, check if shift is underway
;
	CPD     0,X			;Compare to threshold for shift to be underway
	BGE     LB51E			;Bra current derivative is >=, start of shift not yet 
					;underway
;
;-Shift has started
;
	BSET    L0081,#$10		;Status word, set b4, shift has started
	BRA     LB52D			;Bra to continue				
;
;-Start of shift still in progress or shift is flaring, keep updating stored shift time
;										
LB51E	LDAA    L00D5			;Load shift timer
	STAA    L00D4			;Save it, shift flare timer 
	BRA     LB52D			;Bra to continue				
;	
;~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Here if shift is underway
;~~~~~~~~~~~~~~~~~~~~~~~~~~~
;										
LB524	BRSET   L0081,#$01,LB52D	;Status word, bra if b0==1, slope of derivative of
					;engine RPMs is negative
;
;-Positive slope, check threshold for shift to be complete
;
	CPD     6,X			;Compare to threshold for shift to be complete
	BGT     LB543			;Bra if slope of RPMs >, shift is complete
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Here if shift has just started or is still in progress
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;   
LB52D	LDAA    L008C			;Load status word
	BRSET   L0085,#$20,LB53B	;Status word, bra if b5==1, TCC commanded to lock
;
;-TCC lockup not commanded, check if releasing or being applied
;
	BRCLR   L0085,#$50,LB53B	;Bra if b4, TCC being released, and b6, TCC being
					;applied, ==0 
;
	ORAA    #$20			;Set b5, TCC being applied/released
	STAA    L008C			;Save it, status word
;
LB53B	JSR     LB553			;Routine to check for conditions applicable to 
					;adaptive modifier cell change/update
;										
	LDAA    L00D5			;Load shift duration timer
	INCA				;+1
	BNE     LB552			;Bra if no roll-over
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Here if shift is completed
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LB543	JSR     LB5FE			;Routine to analyze shift and update adaptive modifier
;											
	JSR     LEBB1			;Routine to check for error 89 after each shift
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~	
; Here to reset after shift
;~~~~~~~~~~~~~~~~~~~~~~~~~~~			
;							
LB549	BSET    L0081,#$80		;Set b7, shift now completed
	BCLR    L0081,#$02		;Clear b1, trans shift status word
	BCLR    L007F,#$12		;Clear b1, upshift in progress flag, and b4, downshift
					;in progress flag 
;										
LB552	RTS				;Return
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; Routine to check quals for adaptive modifier updates
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;										
LB553	LDAA    L008B			;Load status word
	LDX     #L5FC5			;Index adaptive modifier params										
	LDAB    L00D3			;Gear number 
	DECB				;-1
	ABX				;Add into X as offset
;
;-Check for engine braking
;
	BRCLR   L008A,#$04,LB562	;Status word, bra if b2==0, engine braking not in 
					;effect
;										
	ORAA    #$0008			;Set b3, engine braking in effect
;
;-Check for shift flare
;
LB562	LDY     #L601D			;Load option byte
	BRCLR   0,Y,#$01,LB574		;Bra if b0==0, shift flare compensation enabled
;											
	LDAB    L00D4			;Load shift flare timer
	CMPB    $004F,X			;Threshold to flag shift as flaring 
	BLS     LB574			;Bra if shift flare timer <=
;										
	BSET    L008C,#$80		;Set b7, shift is flaring
;
;-Check for long shift times indicating excess transmission slip
;											
LB574	LDAB    L00D6			;Load shift duration timer
	CMPB    1,X			;Compare to time threshold to flag excessive shift 
					;slip 
	BLS     LB57C			;Bra if shift duration timer <=
;										
	ORAA    #$20			;Set b5, excess transmission slip present 
;
;-Check TPS limits for adaptive modifier updates
;
LB57C	LDAB    L00AC			;Load trans TPS	
	CMPB    $0030,X			;Lower TPS limit for adaptive modifier update 
	BLS     LB586			;Bra if TPS <=, dont flag TPS as within range
;											
	CMPB    $33,X			;Upper TPS limit for adaptive modifier update
	BLS     LB588			;Bra if TPS <=
;											
LB586	ORAA    #$04			;Set b2, TPS out of range for adaptive modifier update
;
;-Check rate that TPS is changing
;
LB588	LDX     #L5FC5			;Index adaptive modifier params
	SUBB    L018B			;Subtract trans TPS at start of shift from current
					;trans TPS
	BCC     LB591			;Bra if no underflow
;										
	NEGB				;Neg, get abs. value
;
LB591	CMPB    $36,X			;Upper delta TPS limit for adaptive modifier 
					;update
	BCS     LB597			;Bra if delta change in TPS <
;										
	ORAA    #$0080			;Set b7, excessive change in TPS
;
;-Check for trans overtemp
;
LB597	BRSET   L0083,#$02,LB59F	;Status word, bra if b1==1, trans over-temp
;										
	BRCLR   L0099,#$04,LB5A1	;Status word, bra if b2==0, manual pattern not in
					;effect
;										
LB59F	ORAA    #$01			;Set b0, trans over-temp, manual pattern selected,
					;or shift pending
;
;-Check to make sure trans temp is within range for updates
;
LB5A1	LDAB    L00AF			;Load transmission temp
	CMPB    $2E,X			;Lower trans temp limit for adaptive modifer
					;updates
	BLS     LB5AB			;Bra if trans temp <=
;											
	CMPB    $2F,X			;Upper tran temp limit for adaptive modifer
					;updates
	BCS     LB5AD			;Bra if trans temp <=
;											
LB5AB	ORAA    #$02			;Set b1, trans temp out of range for adaptive modifier
					;updates
;
;-Check if manual first or second are selected
;
LB5AD	BRCLR   L009C,#$03,LB5B3	;Trans range word, bra if b0, manual first, and b1, 
					;manual second ==0
;											
	ORAA    #$10			;Set b4, manual first or manual second selected
;
;-Check elapsed shift time and time flaring against min time required for updates
;
LB5B3	LDAB    0,X			;Min elapsed shift time for adaptive modifier 
					;updates
	CMPB    L00D5			;Compare to shift timer
	BNE     LB5BF			;Bra if !=, (?)
;											
	CMPB    L00D4			;Compare to shift flare timer
	BLS     LB5BF			;Bra if <=, enough flare time to allow updates
;										
	ORAA    #$0040			;Set b6, too little elapsed time or flare for updates
;
;-Check if errors applicable to adaptive modifier have been set
;
LB5BF	STAA    L008B			;Save it, status word
	PSHX				;Push index address to stack
	JSR     LE414			;Routine to check for errors applicable to adaptive
					;modifier
;											
	PULX				;Get index address back
	LDAA    L008C			;Load status word
	BRCLR   L0092,#$10,LB5CE	;Status word, bra if b4==0, no errors applicable to
					;adaptive modifier set
;											
	ORAA    #$0008			;Set b3, errors applicable to adaptive modifier have
					;been set
;	
;-Check for back to back shifts
;										
LB5CE	LDAB    L00D3			;Load gear # 
	DECB				;-1
	ABX				;Add in to index as offset
	LDAB    L0181			;Load shift duration time
	CMPB    $0040,X			;Minimum time between shifts for shift modifier 
					;updates
	BCC     LB5DB			;Bra if time <=
;											
	ORAA    #$01			;Set b0, time between shifts too short for adaptive
					;modifier updates
;	
;-Check for recent changes in selected gear range
;										
LB5DB	LDAB    L009B			;Load range select timer
	CMPB    $43,X			;Min time after range select to allow adaptive
					;modifier updates
	BCC     LB5E3			;Bra if time >=
;											
	ORAA    #$02			;Set b1, selected range has recently changed
;	
;-Check for excessive changes in vehicle speed
;										
LB5E3	LDAB    L00D1			;Load MPH x 2
	SUBB    L0180			;Subtract MPH at start of shift
	BPL     LB5EB			;Bra if no underflow
;											
	NEGB				;Neg, get absolute value
;
LB5EB	CMPB    $46,X			;Max allowed change in vehicle speed for 
					;adaptive modifier updates
	BLS     LB5F1			;Bra if change in MPH <=
;											
	ORAA    #$04			;Set b2, change in MPH too great for adaptive modifer
					;updates
;
LB5F1	STAA    L008C			;Save it, status word
;
;-Check if 4WD low is active
;
	BCLR    L008D,#$01		;Status word, preclear b0, in 4WD low flag
;											
	BRCLR   L0096,#$20,LB5FD	;Status word, bra if b5==0, not in 4WD low
;											
	BSET    L008D,#$01		;Set b0, in 4WD low, adaptive modifier updates not
					;allowed
;											
LB5FD	RTS				;Return
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; Routine to analyze upshift and update adaptive modifier
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LB5FE	LDY     #LB74F			;Start of adaptive mod. cell addr. index
	LDAB    L0099			;Load trans shift pattern word
	ANDB    #$06			;Clear all but b1, performance pattern, and b2,
					;manual pattern requested
	ABY				;Add in as offset
	LDY     0,Y			;Load addr. of adaptive mod. cell index
	LDAB    L00D3			;Load gear # 
	DECB				;-1
	ASLB				;x2 for double byte addr.
	ABY				;Add in as offset
	LDAA    L0114			;
	BNE     LB64D			;
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Look up the desired shift time
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;											
	LDX     6,Y			;Load desired shift time vs. TPS table
	LDAA    L00AC			;Load trans TPS
	JSR     LF15E			;2D lookup
;											
	PSHA				;Push desired shift time to stack
	CLRA				;Preclear A
	LDAB    L00D3			;Load gear #
	DECB				;-1 for lookup
	ASLB				;x2 for double byte addr.
	LDX     #LB76D			;Desired shift time baro gain vs. TPS table index
	ABX				;Add in gear # as offset
	LDX     0,X			;Load table addr.
	BEQ     LB631			;Bra if ==0, 
;											
	LDAA    L00AC			;Load trans TPS
	JSR     LF15E			;2D lookup
;											
LB631	PSHA				;Push desired shift time baro gain to stack
	TSX				;Get current stack pointer
	JSR     LD96D			;Routine to generate baro gain
;	
;-Apply baro gain
;						
	INS				;Inc. stack pointer to clean up stack
	ADDD    #$0040			;Round up if needed
	ASLB				;x2
	TAB				;Get MSB into B
	ANDA    #$0000			;Clear A w/o effecting C bit status
	ROLB				;Rotate B, rounding if needed
	ROLA				;Rotate A, mult. baro gain by 2 for division op., 
					;neutral value will be 256
	XGDX				;Swap into X
	PULA				;Get desired shift time back
	CLRB				;Clear LSB
	IDIV				;Integer divide, desired shift time/baro gain
	XGDX				;Get result into AB
	TSTA				;Test MSB of result
	BEQ     LB64C			;Bra if ==0, result in range
;										
	LDAA    #$00FF			;Clear overflow
	BRA     LB64D			;Bra to continue
;										
LB64C	TBA				;Get result into A
;
;-Update stored shift times
;
LB64D	LDX     #L0132			;Index stored shift times
	LDAB    L00D3			;Load gear #
	DECB				;-1 for indexed lookup
	ABX				;Add gear # in as offset
	LDAB    L00D5			;Load shift timer
	SUBB    L00D4			;Subtract shift flare timer
	STAB    L00D6			;Save it, current shift duration timer
	STAB    0,X			;Save it, stored shift duration timer vs. shift #
	SBA				;Subtract from desired shift time
	RORA				;
	ROLA				;
	BVC     LB665			;Bra if no 2's overflow
;											
	LDAA    #$007F			;
	ADCA    #$0000			;
;
LB665	STAA    L00D7			;Save it, shift error time
	STAA    3,X			;Save it, shift erorr time vs. shift #
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Check time thresholds for adaptive modifier updates
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
	LDX     #L600E			;Large shift error threshold vs. shift 											
	LDAB    L00D3			;Load gear #
	DECB				;-1 for indexed lookup
	ABX				;Add in gear # as offset
	LDAA    L008B			;Load status word
	LDAB    L5FC5			;Load min shift time for adaptive mod. update
	CMPB    L00D5			;Compare to shift timer
	BCS     LB67D			;Bra if time >
;	
;-Short upshift times present
;									
	ORAA    #$0040			;Set b6, short shift times present
	STAA    L008B			;Save it, status word
;
LB67D	LDAA    L008C			;Load status word
	PSHX				;Push addr. to stack
	LDX     #L6017			;Index min shift time threshold for adaptive modifier
					;table
	LDAB    L00D3			;Load gear # 
	DECB				;-1 for lookup
	ABX				;Add it in as offset
	LDAB    L00D6			;Load current shift duration timer
	CMPB    0,X			;Compare to min time threshold for adaptive modifier
	BCC     LB691			;Bra if time >
;	
;-Short shift times present
;									
	ORAA    #$0040			;Set b6, shift duration too short for adaptive 
					;modifier updates
	STAA    L008C			;Save status word
;
;-Check for other conditions that bar adaptive modifier updates
;
LB691	PULX				;Get index back
	ORAA    L008B			;Or with status word
	ORAA    L008D			;Or with status word
	BEQ     LB69B			;Bra if ==0, no special conditions set, adaptive mod.
					;updates ok'd
;							
;-Special conditions set, no updates
;			
	JMP     LB74E			;Jump to return
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Compensate for shift flare if optioned
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;									
LB69B	LDAA    #$0001			;Load mask
	ANDA    L601D			;And with option byte
	BNE     LB6AC			;Bra if b0==1, dont compensate for flare
;										
;-Compensating for shift flare
;	
	LDAA    L00D4			;Load shift flare timer
	CMPA    6,X			;L6014, flare time threshold to compensate for shift
					;flare
	BCS     LB6AC			;Bra if time <
;										
	LDAA    $0C,X			;Pressure compensation for shift flare
	BRA     LB6CE			;Bra to continue
				
LB6AC	LDAA    #$FF			;
	STAA    L00F7			;
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Look up proportional pressure modifier based on shift time error							
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
	LDAA    L00D7			;Load shift error
	CMPA    0,X			;Compare to threshold for large shift error
	BGE     LB6BA			;Bra if  shift error is <
;
;-Excessive shift error present
;											
LB6B6	LDAA    3,X			;Load proportional term for large error
	BRA     LB6CE			;Bra to continue
;
;-Shift error within limits for closed loop corrections
;											
LB6BA	LDAA    L00D7			;Load shift error
	ADDA    #08			;Add in 200 msec offset for a span from -200 to 
					;200 msecs
	BGE     LB6C1			;Bra if result >= 0
;											
	CLRA				;Clear negative result
;
LB6C1	CMPA    #16			;200 msecs, max limit for lookup
	BLS     LB6C7			;Bra if reference time <=
;											
	LDAA    #16			;Load max
;
LB6C7	LDX     $0C,Y			;Index adaptive modifier proportional term vs. shift 
					;error time
	TAB				;Get shift error time into B
	ABX				;Add it in as offset
	LDAA    0,X			;Load adaptive modifier proportional pressure
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Ensure that the added corrections wont push the line pressure out of bounds
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LB6CE	STAA    L00DB			;Save it, current change to adaptive modifier cell
	LDX     0,Y			;Load adaptive modifier cell addr.
	LDAB    L0145			;Load current adaptive modifier cell #
	ABX				;Add it in as an offset
	LDAB    0,X			;Load adaptive modifier cell
;
;-Generate clamping pressure
;
	EORB    #$0005			;XOR lower 3 bits of adaptive modifier pressure
	ANDB    #$0003			;Clear all but b0, b1
	STAB    L00D8			;Save it, clamping pressure for adaptive mod'r
;
;-Check direction of line pressure
;
	TSTA				;Test added line pressure
	BPL     LB6FA			;Bra if adding pressure
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Here if reduction in line pressure, check to make sure pressure hasnt fallen too low
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;										
	LDAB    L00D9			;Stored underpressure for current shift
	BGT     LB6E9			;Bra if current line pressure is still > 0
;	
;-Line pressure has fallen to zero or below
;									
	CLRA				;Clear added line pressure corrections
	BRA     LB6F8			;Bra to continue
;											
LB6E9	NEGB				;Neg. underpressure term, now ammount needed to push
					;the line pressure to zero
	ASLB				;x2
	ASLB				;x2, now x4 to match adaptive modifier pressure
	BCC     LB6F8			;Bra if no overflow, pressure already in excess
;											
	ADDB    L00D8			;Add in clamping pressure
	SUBB    #$0003			;-3 to invert term
	BCS     LB6F8			;Bra if underflow, pressure already in excess
;											
	CBA				;Compare added pressure to delta underpressure term
					;to ensure that the corrections will not push the 
					;line pressure to or below zero
	BCC     LB6F8			;Bra if the abs. value of the line pressure 
					;corrections are <= the span needed to zero out
					;the line pressure
;
;-Subtracted pressure will push the line pressure negative
;											
	TBA				;Get span instead
;
LB6F8	BRA     LB70B			;Bra to continue
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Here if increase in line pressure, check and make sure line pressure still within limits
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~											
;
LB6FA	LDAB    L00DA			;Stored overpressure for current shift
	BGT     LB701			;Bra if > 0, pressure still below max allowed
;											
;-Excess line pressure present
;
	CLRA				;Clear added line pressure corrections
	BRA     LB70B			;Bra to continue
;										
LB701	ASLB				;x2
	ASLB				;x2, now x4 to match adaptive modifier pressure
	BCS     LB70B			;Bra if overflow
;											
	ADDB    L00D8			;Add in clamping pressure
	CBA				;Compare added pressure to delta overpressure term to
					;ensure that the added corrections will not exceed the 
					;max allowed pressure
	BLS     LB70B			;Bra if added pressure is still <= to the span to the
					;maximum line pressure
;
;-Added pressure will exceed max allowed line pressure
;											
	TBA				;Get span instead
;
;-Here if line pressure will still be within limits
;
LB70B	ADDA    0,X			;Add line pressure corrections to adaptive modifier
	BVC     LB713			;Bra if no 2's overflow
;										
	LDAA    #$007F			;Clear overflow
	ADCA    #$0000			;
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Update the adaptive modifier
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LB713	LDY     #LB74F			;Load adaptive mod. cell index 
	LDAB    L0099			;Load shift pattern word
	ANDB    #$0006			;Clear all but b1, performance pattern, and b2,
					;manual pattern
	ABY				;Add in as offset
	LDY     0,Y			;Load address
	PSHX				;Push index to stack
	LDX     $0012,Y			;L5FFC, lower allowed limit for adaptive modifier
					;correction
	CMPA    0,X			;Compare to adaptive modifier
	BGE     LB72A			;Bra if >=
;											
	LDAA    0,X			;Load limit instead
;
LB72A	LDX     $14,Y			;L5FFD, upper allowed limit for adaptive modifier
					;correction
	CMPA    0,X			;Compare to adaptive modifier
	BLE     LB733			;Bra if <=
;											
	LDAA    0,X			;Load limit instead
;
LB733	PULX				;Get addr. index back
	TST     L0113			;
	BNE     LB74E			;
;										
	BRCLR   L0082,#$08,LB74E	;Bra if b3==0, ignition is off, return
;
;-Save updated terms
;									
	LDY     $16,Y			;Load index for net change in adaptive modifier
	TAB				;Get current adaptive mod'r pressure correction into B
	NEGB				;Negate current adaptive modifier
	ADDB    0,X			;Add in old adaptive modifier, now net change in mod'r
	ADDB    0,Y			;Add in previous change in adaptive modifier
	SEI				;Disable interrupts
	STAA    0,X			;Save updated adaptive modifier
	STAB    0,Y			;Save updated net change in adaptive modifier
	CLI				;Enable interrupts
;
LB74E	RTS				;Return    					
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;											
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Addr of set of adaptive modifier cells to use
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;	Cell RAM addr. table	;Shift pattern
;
LB74F   FDB LB755		;Normal pattern
	FDB LB755		;Performance pattern
	FDB LB755		;Manual Pattern
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Addr. of adaptive modifier cells for each gear
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;	Start addr.		;Shift
;
LB755	FDB L031B		;1 -> 2
	FDB L031F		;2 -> 3
	FDB L0323		;3 -> 4
; 
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Addr. of desired shift time for each shift
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;     
;	Table addr.		;Shift 
;
LB75B   FDB L658F		;1 -> 2 
	FDB L65A0		;2 -> 3 
	FDB L65B1		;3 -> 4 
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Adaptive modifier proportional term for each shift
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;	Table addr.		;Shift
;
LB761   FDB L65C2		;1 -> 2 
	FDB L65D3		;2 -> 3 
	FDB L65E4		;3 -> 4 
;
LB767   FDB L5FFC		;Lower allowed limit for adaptive modifier
	FDB L5FFD		;Upper allowed limit for adaptive modifier
;
LB76B	FDB L0327		;Addr. of net change in adaptive modifier term
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Baro gain for shift time for each shift
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;	Table addr.		;Shift
;
LB76D   FDB L6044		;1 -> 2
	FDB L6055 		;2 -> 3
	FDB L6066		;3 -> 4
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; Line pressure/TCC routine calling schedule
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LB773	LDX     #L400F			;Opt. byte
	BRCLR   0,X,#$C0,LB77C		;Bra if b7==0, auto trans and b6==0, CC transmission 
;
;-Non CC transmission here
;
	BRA     LB78B			;Bra to return
;
;-CC transmission here
;
LB77C	JSR     LB78C			;Routine to construct transmission line pressure
;
	JSR     LBCED			;Routine to look up desired force motor current
;
	JSR     LBDF3			;Routine to handle TCC related timers
;
	JSR     LBD4D			;Routine to check quals for disabling TCC
;
	JSR     LBE8F			;PWM TCC control routine
;
LB78B	RTS				;Return    
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; Routine to construct transmission line pressure
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LB78C	LDAA    L0111			;Load diag. line pressure
	BEQ     LB794			;Bra if ==0, no diag. in effect
;
;-Diag. override in effect
;
	JMP     LBACB			;Bra to save diag. line pressure
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Look up desired added line pressure 
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LB794	LDAB    L0099			;Transmission shift pattern word
	ANDB    #$06			;Clear all but b1=performance pattern req, 
					;b2=manaul pattern requested
	LDX     #LBB48			;Addr. of added line pressure table(s) for 0-64 mph
	CLRA				;Preclear A   
	BRSET   L009C,#$20,LB7A2	;Status word, bra if b5==1, trans in reverse
;
;-Here to load MPH
;
	LDAA    L00D1			;Load MPH x 2
;
LB7A2	ASLA				;x 2
	BCC     LB7A8			;Bra if <=

	LDX     #LBB4E			;Addr. of line pressure table(s) for 64-128 mph
;
LB7A8	ABX				;Add in the offset
	LDX     0,X			;Load addr. of line pressure table
	TAB				;MPH to B
	LDAA    L00AC			;Trans %TPS
	JSR     LF17B			;3D lookup
;											
	LDAB    L00D3			;Gear # 
	DECB				;-1
	BRCLR   L007F,#$10,LB7B9	;Status word, bra if b4==0, trans has not downshifted 
;
;-Trans has downshifted
;
	INCB				;+1  
; 
LB7B9	BMI     LB7E8			;Bra if in first
;	
	PSHA				;Push line pressure to stack
	LDX     #L0115			;
	ABX				;Add in gear # as offset
	LDAA    0,X			;
	BNE     LB7E0			;
;
	BRCLR   L0099,#$06,LB7D0	;Trans shift pattern word, bra if b2==0,
					;manual pattern not selected, and b1==0,
					;performance pattern not requested.
;											
	ADDB    #03			;Skip ahead to second set of offset tables
;
	BRSET   L0099,#$02,LB7D0	;Bra if b1==1, performance pattern requested
;											
LB7CE	ADDB    #$03			;Skip ahead again to third set of tables
;
;-Look up pressure offset vs. %TPS table addr
;
LB7D0	LDX     #LBB54			;Pressure offset table index
	ASLB				;x2 
	ABX				;Add in gear number as offset
	LDX     0,X			;Load table addr 
	BEQ     LB7E0			;Bra if addr. ==0
;
	LDAA    L00AC			;Trans TPS
	JSR     LF15E			;2D lookup
									
	SUBA    #128			;Make pressure offset signed
;
;-Here if address ==0, table not in use
;
LB7E0	PULB				;Get looked up line pressure back
	ABA				;Add in offset
	BVC     LB7E8			;Bra if no 2's overflow
;											
	LDAA    #127			;Clear overflow
	ADCA    #$00			;Perform carry if needed
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Look up and apply adaptive modifier
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LB7E8	PSHA				;Push added line pressure to stack
	JSR     LBACE			;Routine to look up adaptive modifer
;
	PULB				;Get line pressure back
	ABA				;Add in adaptive pressure modifier
	BVC     LB7F4			;Bra if no 2's overflow
;										
	LDAA    #127			;Clear overflow
	ADCA    #$00			;Perform carry if needed
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Load in temp corr. table addr. based on gear #
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LB7F4	LDX     #LBB78			;Temp correction table address index
	LDAB    L00D3			;Gear # 
	ASLB				;x2 for lookup
	ABX				;Add in as offset
	LDX     0,X			;Load in table address
	BNE     LB802			;Bra if !=0,
;
;-Here if table not in use
;
	JMP     LB882			;
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Look up break points and prepare to perform look up
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LB802	PSHA				;Line pressure to stack
	PSHX				;Push table addr. to stack
	LDAA    L00AF			;Load trans temp
	LDAB    #$A0			;multiplier for lookup, 11 cols. in table
	MUL				;(Trans temp x multiplier)
	PSHA				;Push formatted trans temp to stack
;
;-Put some fluff on stack in case following scaling op. is skipped
;
	PSHX				;Table addr to stack
	PSHX				;
	PSHX				;											;
	LDAA    #00			;Zero offset											;
	LDAB    L00AC			;Trans. TPS
	CMPB    L6001			;Lower TPS limit for lookup
	BLS     LB870			;Bra if TPS <=, use zero offset
;
	LDAA    #$30			;4 line offset											;
	CMPB    L6004			;Upper TPS limit for lookup
	BCC     LB870			;Bra if >=, use 4 line offset
;
	LDAA    #$10			;16 counts/table line											;
	LDX     #L6002			;Start addr. of break points for lookup
;
;-Calculate offset to current break point
;
LB822	CMPB    0,X			;Current TPS break point
	BCS     LB82B			;Bra if TPS <
;
	INX				;Advance to next break point
	ADDA    #$10			;+16 for next break point
	BRA     LB822			;Bra back to check next break point
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Scale %TPS to break point table for lookup
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LB82B	TSY				;Get current stack pointer     
	STAB	5,Y			;Save trans TPS
	STAA	4,Y			;Save break point offset
;
;-Calc delta %TPS between each set of break points
;
	LDAA	0,X			;Load break point from table
	DEX				;-1     
	LDAB	0,X			;Load previous break point
	SBA				;Get delta TPS between points     
	STAA   	3,Y			;Save delta TPS between break points
;
;-Delta %TPS x offset
;
	LDAB   	4,Y			;Load break point offset
	MUL				;delta TPS x offset    
	STD    	1,Y			;Save result, now %TPS offset
;
;-Trans %TPS x 16
;
	LDAB   	5,Y			;Load trans TPS
	LDAA	#$10			;16			
	MUL				;%TPS x 16     
	STD	4,Y			;Save result
;
	LDAA	1,X			;Load current %TPS break point from table
	LDAB	#$10			;16
	MUL				;Break point x 16     
	COMA				;Flip sign    
	COMB				;    
	ADDD	#$01			;
	ADDD	1,Y			;[(Delta %TPS x offset) - (%TPS BP x 16)],
					;Now %TPS above current break point
	ADDD	4,Y			;Add result to Trans TPS x 16
	STAA	4,Y			;Save result
;
;-Finish scaling it for look up
;			
	CLRA				;    
	STAA	2,Y			;Preclear addr.
	LDX	2,Y			;Get delta TPS between break points into X
	LDAA	4,Y			;Load previous result
	IDIV				;Int. divide     
	STX	0,Y			;Store it, X = Total BP %TPS / delta BP %TPS
	LDAA	1,Y			;Load in result for lookup
;
;~~~~~~~~~~~~~~~~~~~~
; Here to do look up
;~~~~~~~~~~~~~~~~~~~~
;
;-Clean up the stack
;
LB870	PULX				;    
	PULX				;
	PULX				;
	PULB				;Get trans temp back
	PULX				;Get table addr. back    
	JSR     LF17B			;3D lookup
;
	SUBA    #$0080			;Make pressure correction signed
	PULB				;Get line pressure back    
	ABA				;Add in temp. compensation     
	BVC     LB882			;Bra if no 2's overflow
;
	LDAA    #$007F			;Clear overflow
	ADCA    #$0000			;Perform carry if needed
;
LB882	PSHA				;Push line pressure to stack    
	JSR     LBC0C			;Shift pressure mod. routine
;
	PULB				;Get line pressure back    
	ABA				;Add in the pressure modification				     
	BVC     LB88E			;Bra if no 2's overflow
;
	LDAA    #$007F			;Clear overflow
	ADCA    #$0000			;Perform carry if needed
;
LB88E	JSR     LBCEC			;Not used
;
	JSR     LBB86			;Routine for jump shift pressure compensation
;
	BRCLR   L007F,#$10,LB8BE	;Status word, bra if b4==0, trans has not downshifted
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Trans has downshifted, see if special pattern has been requested
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
	LDAB    L00D3			;Load gear #
	BRCLR   L0099,#$06,LB8A6	;Bra if b1, performance pattern, and b2, manual 
					;pattern requested flags ==0
;
;-Performance and/or manual pattern requested
;

	ADDB    #$0003			;+3, advance to performance pettern tables
	BRSET   L0099,#$02,LB8A6	;Bra if b1==1, performance pattern requested
;
	ADDB    #$0003			;+3, advance to manual pattern tables
;
;-Apply line pressure modification for downshift
;
LB8A6	LDX     #LBB66			;Line pressure mod. for downshift table addr. index
	ASLB				;x2 for double byte addr.
	ABX				;Add in offset
	LDX     0,X			;Load in addr. of table
	BEQ     LB8BE			;Bra if ==0, table not in use
;
	TAB				;Get line pressure into B
	LDAA    L00D1			;Load MPH x 2
	JSR     LF141			;Go do 2D lookup with additional quals
;										
	SUBA    #128			;Subtract offset
	ABA				;Add into line pressure 
	BVC     LB8BE			;Bra if no 2's comp. overflow
;
	LDAA    #127			;Clear overflow
	ADCA    #$00			; 
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Here if no downshift, continue with line pressure compensation
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LB8BE	BRCLR   L0096,#$04,LB8D5	;Status word, bra if b2==0, PE not active
;
;-PE in effect
;
	TAB				;Get line pressure into B     
	LDAA    L019D			;Load RPM x 8
	LDX     #L6022			;Added line pressure for PE vs. RPM
	JSR     LF15E			;2D lookup
;
	SUBA    #$0080			;Subtract offset
	ABA				;Add result into line pressure
	BVC     LB8D5			;Bra if no 2's comp. overflow
;
	LDAA    #$007F			;Clear overflow
	ADCA    #$0000			;
;
LB8D5	TSTA				;Test line pressure   
	BPL     LB8D9			;Bra if >0
;
	CLRA				;Clear negative result
;
;-Look up pressure offset vs. MAT
;
LB8D9	TAB				;Get line pressure into B
	LDAA    L0252			;Load MAT
	SUBA    #48			;36 deg C offset for lookup
	BCS     LB8E8			;Bra if underflow
;
	ASLA				;MAT x2
	BCC     LB8E9			;Bra if no overflow
;
	LDAA    #$FF			;Clear overflow
	BRA     LB8E9			;Bra to continue with lookup
;
LB8E8	CLRA 				;Here to clear underflow for lookup
;
LB8E9	LDX     #L6033			;Line pressure offset vs. MAT table
	JSR     LF15E			;2D lookup
;										
	SUBA    #128			;Subtract offset
	ABA				;Add into line pressure
	BVS     LB8F8			;Bra if 2's comp. overflow set, result out of range
;
;-No 2's overflow
;										
	BMI     LB8FC			;Bra if negative result, clear overflow
;
;-Result > 0
;										
	BRA     LB8FD			;Bra to continue with compensations
;										
LB8F8	LDAA    #127			;Load max, clear out of range result
	BRA     LB8FD			;Bra to continue
;										
LB8FC	CLRA				;Clear negative result
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Look up and apply baro compensation to line pressure
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LB8FD	PSHA				;Push line pressure to stack    
	LDAB    L00D3			;Load gear # 
	BRCLR   L007F,#$10,LB905	;Status word, bra if b4==0, trans has not downshifted
;
;-Transmission has downshifted
;
	INCB				;+1, use previous gear # for table lookup after 
					;downshift
;										
LB905	CLRA				;Preclear A
	DECB				;-1 for lookup   
	BMI     LB917			;Bra if <0, in first gear
;											
	ASLB				;x2 for double byte lookup
	LDX     #LBB80			;Table addr. index table
	ABX				;Add in offset
	LDX     0,X			;Load line pressure baro comensation table addr. index
	BEQ     LB917			;Bra if ==0, table not in use
;
	LDAA    L00AC			;Load trans TPS
	JSR     LF15E			;2D lookup
;	
;-Here if in first or table not in use
;										
LB917	PSHA				;Push baro gain factor to stack
	TSX				;Get current stack pointer
	JSR     LD96D			;Routine to generate baro gain 
;											
	INS				;+1, clean up stack
	ADDD    #128			;Round if needed
	PULB				;Get line pressure back off stack
	MUL				;Baro gain x line pressure
	ASLD				;x2, now /128
	BVS     LB927			;Bra if 2's comp. overflow set
;											
	BPL     LB929			;Bra if result < 128, still within range
;											
LB927	LDAA    #$007F			;Clear overflow/out of range result
;
LB929	STAA    L013E			;Save it, finalized base desired line pressure
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Look up and make sure adiquate line pressure present during engine braking
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
	BCLR    L008A,#$04		;Status word, pre-clear b2, engine braking in effect
					;flag
	BRCLR   L007F,#$09,LB942	;Status word, bra if b0, upshift in progress, and
					;b3, downshift in progress flags ==0
;	
;-Shift in progress
;										
	CLR     L00E2			;Clear engine braking delay timer
	LDX     L00E9			;Load stored torque converter slip
	CPX     L5FFF			;Compare torque converter slip to threshold for engine
					;braking
	BLT     LB94B			;Bra if torque converter slip <, engine braking in
					;effect
;											
	LDAB    L5FFE			;Load delay for engine braking after shift
	STAB    L00E2			;Save it, engine braking delay timer
;
;-Here if no shifts in progress
;
LB942	LDAB    L00E2			;Load engine braking delay timer
	BEQ     LB94B			;Bra if ==0, look up engine braking line pressure
;											
	DECB				;-1
	STAB    L00E2			;Save it
	BRA     LB988			;Bra to skip engine braking line pressure lookup
;	
;-Here if no shifts where present or engine braking in progress
;										
LB94B	BRCLR   L0092,#$04,LB954	;Status word, bra if b2==0, no press. switch. man.
					;error
;
;-Pressure switch manifold error present 
;						
	LDX     #L60BB			;Load 2nd gear engine braking line pressure as default
	BRA     LB976			;Bra to continue
;	
;-No errors, check if overrun clutch engaged when coasting
;										
LB954	LDAB    L00D3			;Load gear #
	CMPB    #$0003			;4th gear
	BEQ     LB988			;Bra if in 4th, no engine braking
;										
	BRCLR   L009C,#$07,LB988	;Trans range word, bra if b0, b1, and b2==0, not
					;in manual first, manual second, or drive range
;
;-Drive/manual gear range selected, engine braking possible
;											
	LDX     #L60CC			;Engine braking line pressure vs. MPH, 3rd gear 
	CMPB    #$02			;3rd gear
	BEQ     LB976			;Bra if gear # ==, in 3rd gear
;											
	BRCLR   L009C,#$03,LB988	;Bra if b0 and b1==0, not in manual first or second,
					;no engine braking line pressure
;	
;-Manual first or second selected
;										
	LDX     #L60BB			;Engine braking line pressure vs. MPH, 2nd gear
	TSTB				;Test gear #
	BNE     LB976			;Bra if != 0, in 2nd gear
;											
	BRCLR   L009C,#$01,LB988	;Bra if  b0==0, in manual second range while in 1st,
					;skip engine braking line pressure lookup
;
;-Manual first selected
;										
	LDX     #L60AA			;Engine braking line pressure vs. MPH, 1st gear
;
LB976	TAB				;Get line pressure into B
	LDAA    L00D1			;Load MPH x 2
	JSR     LF15E			;2D lookup
;											
	STAA    L013F			;Save it, engine braking line pressure
	CBA				;Compare engine braking line pressure to line pressure
	BCS     LB987			;Bra if line pressure >, use line pressure instead
;
;-Engine braking line pressure is >=, engine braking in effect
;											
	BSET    L008A,#$04		;Status word, set b2, engine braking in effect
	BRA     LB988			;Bra to continue
;
;-Main line pressure is >, engine braking not in effect
;											
LB987	TBA				;Get line pressure into A
;
LB988	LDX     L00E5			;Load torque converter slip
	STX     L00E9			;Save it, stored torque converter slip										
	STAA    L0141			;Save it, final transmission line pressre
	LDAB    L00D1			;Load MPH x 2
	BRCLR   L009C,#$50,LB9AB	;Bra if b4, invalid range, and b6, in P/N flags ==0
;
;~~~~~~~~~~~~~~~~~~~~~~~~~
; Invalid range or in P/N
;~~~~~~~~~~~~~~~~~~~~~~~~~
;
	BCLR    L008A,#$30		;Clear b4, in reverse flag, and b5, in forward gear 
					;flag
	CMPB    L5FE1			;Compare MPH x 2 to threshold for low speed line 
					;pressure handling (P/N)
	BCS     LB9A0			;Bra if MPH <
;											
LB99D	JMP     LBA76			;MPH not met for shift params, jump to continue
;
;-Vehicle nearly stopped or stopped, continue with low speed line pressure handling
;					
LB9A0	BCLR    L008A,#$02		;Status word, clear b1, no forward gears selected
					;flag
	LDX     #L60DD			;Index cool temp compensation line pressure for shift
					;into forward range 
	LDAB    L5FE3			;Added line pressure for (P/N)
	BRA     LB9F7			;Bra to continue
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Here if not in P/N, check quals for low speed line pressure handling
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;											
LB9AB	CMPB    L5FE2			;Threshold for low speed line pressure handling
	BCC     LB9BA			;Bra if MPH >=, skip low speed line pressure hand'g
;											
	BRCLR   L009C,#$0F,LB9BF	;Range select word, bra if b0-b3==0 no forward gears
					;have been selected, in reverse
;										
;-Forward gears have been selected, check if forward gear recently selected
;	
	BRCLR   L009D,#$0F,LB9CA	;Range select word, one delayed, bra if no forward
					;gears selected previously
;	
;-A forward gear has been selected already
;																				
LB9B8	BRA     LB9C3			;Bra to continue
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Here if thresholds not met for shift into drive range
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;											
LB9BA	BCLR    L008A,#$30		;Clear b4, in reverse flag, and b5, in forward gear 
					;flag
	BRA     LB99D			;Bra to continue
;
;-Check if vehicle previously in reverse
;											
LB9BF	BRCLR   L009D,#$20,LB9CA	;Bra if b5==0, not previously in reverse
;
;-Vehicle was reverse or forward gears selected previous
;											
LB9C3	BRSET   L008A,#$02,LBA21	;Bra if b1==1, quals met for low speed line pressure
					;handling
;
;-Quals where not met previously for low speed line pressure handling
;											
	JMP     LBA7C			;Jump to skip low speed line pressure handling
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Forward range/reverse recently selected, apply cool temp compensation for shift into gear
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;											
LB9CA	BSET    L008A,#$02		;Status word, set b1, quals met for low speed line 
					;pressure handling
;											
	BRSET   L009C,#$20,LB9E5	;Status word, bra if b5==1, in reverse
;
;-In forward gear
;											
	BCLR    L008A,#$10		;Status word, clear b4, in reverse flag
	BSET    L008A,#$20		;Set b5, in forward gear flag
	LDAA    L5FDD			;Time to add line pressure for shift into forward
					;range
	STAA    L013D			;Save it, momentary added line pressure timer
	LDX     #L60DD			;Index cool temp compensation line pressure for shift
					;into forward range
	LDAB    L5FE4			;Added line pressure for forward range
	BRA     LB9F7			;Bra to continue
;	
;-In reverse here
;										
LB9E5	BSET    L008A,#$10		;Set b4, in reverse flag
	BCLR    L008A,#$20		;Clear b5, in forward gear flag
	LDAA    L5FDF			;Time to add line pressure for shift into reverse
	STAA    L013D			;Save it, momentary added line pressure timer
					;gear range
	LDX     #L6168			;Index cool temp compensation line pressure for shift
					;into reverse
	LDAB    L5FE5			;Added line pressure for reverse
;
LB9F7	STAB    L013C			;Save it, added low speed line pressure 
					
	LDD     L019D			;Load RPM x 8
	ASLD				;x2
	BCS     LBA03			;Bra if overflow
;											
	ASLD				;x2
	BCC     LBA05			;Bra if no overflow
;											
LBA03	LDAA    #$00FF			;Clear overflow
;
LBA05	ADDD    #$0080			;Round up if needed
	SBCA    #$0000			;Subtract carry, roll back if overflow
	PSHA				;Push RPM to stack
	LDAA    L00AF  			;Load trans temp
	LDAB    #0154			;Load scalar
	MUL				;Trans temp x scalar, trans temp x .6
	ADCA    #$0000			;Add with carry to round if needed
	CMPA    #$0070			;Max 8 rows for lookup
	BLS     LBA18			;Bra if formatted temp is less
;											
	LDAA    #$0070			;Load max
;
LBA18	PULB				;Get RPM back
	JSR     LF17B			;Go do 3D lookup
;											
	SUBA    #$0080			;-128, make looked up value signed
	STAA    L0142			;Save it, added cool temp compensated line pressure
					;for shift into gear range
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Here if in gear to continue with low speed line pressure handling
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LBA21	LDX     #L61F3			;Low speed line pressure vs. MPH and TPS
	LDAA    L00AC			;Load trans TPS
	LDAB    L00D1			;MPH x 2
	CMPB    #0032			;16 MPH, max for lookup
	BLS     LBA2E			;Bra if MPH <=
;
	LDAB    #$20			;Load max

LBA2E	ASLB				;x2, now MPH x 4    
	JSR     LF17B			;3D lookup
;
	SUBA    #$0080			;-128, line pressure now signed
	ADDA    L0142			;Load added cool temp compensated line pressure for
					;shift into gear range
	BVC     LBA3D			;Bra if no 2's overflow
;
	LDAA    #$007F			;Clear 2's overflow
	ADCA    #$0000			;

LBA3D	ADDA    #$0080			;Add 128 back in, now (line pressure + added line 
					;pressure for shift into gear range)
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Momentarily add in line pressure for shift into gear range
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
	LDAB    L013D			;Load momentary added line pressure timer
	BEQ     LBA5D			;Bra if ==0, no longer adding pressure
;
	BRCLR   L008A,#$20,LBA4D	;Bra if b5==0, not in a forward gear
;
;-In forward gear
;
	ADDA    L5FDE			;Added momentary line pressure for shifting into 
					;forward range
	BRA     LBA54			;Bra to continue
;
LBA4D	BRCLR   L008A,#$10,LBA54	;Bra if b4==0, not in reverse
;
;-In reverse
;
	ADDA    L5FE0			;Added momentary line pressure for shifting into 
					;reverse
;
LBA54	DECB				;-1    
	STAB    L013D			;Save timer
	STAA    L0140			;Save final low speed line pressure
	BRA     LBA63			;Bra to continue
;
;-Here if no longer adding in line pressure for shift
;
LBA5D	STAA    L0140			;Save final low speed line pressure
	BCLR    L008A,#$30		;Clear b4, in reverse flag, and b5, in forward gear
					;flag				
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Finalize transmission line pressure
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LBA63	LDAB    L0140			;Load final low speed transmission line pressure
	LDAA    L0141			;Load final transmission line pressure
	ADDB    L013C			;Add in added low speed line pressure for shift into
					;gear range
	BCS     LBA71			;Bra if overflow
;
;-Compare low speed line pressure to line pressure and load whichever is greater
;
	CBA				;Compare low speed transmission line pressure to
					;transmission line pressure     
	BGE     LBA76			;Bra if transmission line pressure >=
;
LBA71	LDAA    L0140			;Load low speed line pressure instead
	BRA     LBA7C			;Bra to continue
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Here if quals no longer met for low speed transmission handling 
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LBA76	BCLR    L008A,#$02		;Status word, clear b1, quals met for low speed 
					;transmission line pressure handling flag
	CLR     L013D			;Clear timer for momentary added line pressure for 
					;shift into gear range
;
LBA7C	BRCLR   L0096,#$02,LBA91	;Status word, bra if b1==0, A/C not requested
;
;-A/C has been requested
;
	LDAB    L00D3			;Load gear # 
	LDX     #L5FE9			;Index pressure offset for A/C on vs. gear #
	ABX				;Add in gear # as offset
	SUBA    0,X			;Subtract offset from line pressure
	BVC     LBA8E			;Bra if no 2's comp. overflow
;
	LDAA    #$007F			;Clear 2's comp. overflow
	ADCA    #$0000			;
;
LBA8E	BPL     LBA91			;Bra if pressure >= 0
;
	CLRA				;Clear negative result
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Keep track of line pressure
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; 
LBA91	TAB				;Get line pressure into B     
	LDAA    L601E			;Load max allowed line pressure
	PSHA				;Push to stack    
	SBA				;Subtract desired line pressure from max allowed
	RORA				;Rotate right, get C bit into result, get LSB into C   	
	ROLA				;Rotate left, restore result and move carry bit back
					;into C    
	BVC     LBA9F			;Bra if negative result with carry or positive result
					;with no carry, no 2's overflow
;
	LDAA    #$007F			;Clear 2's overflow
	ADCA    #$0000			;

;
;-Update stored overpressure term
;
LBA9F	CMPA    L00DA			;Compare to stored overpressure term
	BGE     LBAA5			;Bra if desired line pressure is still not exceeding
					;the max allowed or not exceeding the peak stored line 
					;pressure above the max allowed for this shift
;
	STAA    L00DA			;Save it, stored overpressure term
;
LBAA5	PULA				;Pull max allowed line pressure from stack  
;
;-Update stored underpressure term
;  
	CMPB    L00D9			;Compare desired line pressure to underpressure term
	BGE     LBAAC			;Bra if current line pressure is higher, dont update
;
	STAB    L00D9			;Save it, stored underpressure term
;
LBAAC	BRSET   L0092,#$08,LBABF
;
	PSHB				;Push line pressure to stack 	   
	PSHA				;Push max allowed line pressure to stack   
	JSR     LE424			;Routine to check for external errors
;
	PULA				;Get max allowed line pressure back   
	PULB				;Get line pressure back    
	BRSET   L0092,#$20,LBABF	;Status word, bra if b6==1, external errors applicable
					;to line pressure/force motor set
;
	CBA				;Compare max allowed line pressure to desired line 
					;pressure    
	BLS     LBABF			;Bra if desired line pressure >= max allowed
;
;-Line pressure within limits
;
	TBA				;Get desired line pressure into A 
;    
LBABF	ADDA    L0112			;Add in ALDL line pressure
	BVC     LBAC8			;Bra if no 2's comp overflow
;
	LDAA    #$007F			;Clear overflow
	ADCA    #$0000			;
;
LBAC8	BPL     LBACB			;Bra if line pressure >=0
;
	CLRA				;Clear invalid negative result
; 	   
LBACB	STAA    L00DC			;Save it, desired line pressure
	RTS				;Return
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~     
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; Routine to change/load adaptive modifier cell
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LBACE	LDAB    L00D3			;Load gear #
	BRSET   L007F,#$02,LBADB	;Status word, bra if b1==1, trans upshift flagged
;
	BRCLR   L007F,#$10,LBAE7	;Status word, bra if b4==0, downshift not flagged
;
;-Trans has downshifted
;
	INCB				;+1, use current gear #    
	BRA     LBAE7			;Bra to continue
;
;-Here if upshift flagged
;
LBADB	BRSET   L007F,#$01,LBAE7	;Bra if b0==1, upshift in progress
;
;-Upshift no longer in progress, check condition flags
;
	LDAA    L008B			;
	ORAA    L008C			;			
	ORAA    L008D			;Or status words together
	BEQ     LBB44			;Bra if no special conditions flagged
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Shift in progress here or special conditions flagged
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LBAE7	CLRA				;Preclear A    
	DECB				;-1, gear #    
	BMI     LBAFD			;Bra if already in first
;
;-In other gears
;
	LDX     #LB74F			;Addr of set of cells to use based on shift pattern
	PSHB				;Gear number to stack    
	LDAB    L0099			;Shift pattern word
	ANDB    #$06			;Clear all but b1=performance pattern req.,
					;b2=manual pattern requested
	ABX				;Add in as offset     
	PULB				;Get gear number back    
	LDX     0,X			;Load next table addr. based on shift pattern req.
	ASLB				;Gear #, x2 for indexed lookup    
	ABX				;Add in the offset    
	LDX     0,X			;Load in start addr of cells
	BNE     LBB02			;Bra if !=0
;
;-Here if not using adaptive modifier cells or in 1st gear
;
LBAFD	STAA    L0145			;Clear adaptive modifier cell #
	BRA     LBB41			;Bra to clear current mod. and continue
;
;-Here if using adaptive modifier cells
;
LBB02	PSHX				;Push RAM addr. to stack    
	LDX     #L5FEE			;Base addr. for cell TPS thresholds
	LDY     #L5FEC			;Base addr. for cell TPS thresholds
	LDAA    L00AC			;Trans. TPS
	LDAB    L0145			;Current adaptive modifier cell #
	BEQ     LBB1D			;Bra if in cell #0
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Here to check/change adaptive modifier cell #
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
	ASLB				;x2 for indxed lookup
	ABY				;Add in offset
	CMPA    0,Y			;Lower TPS limit for current cell
	BCS     LBB23			;Bra if TPS is less, change cells
;
	CMPB    #6			;Cell 3, uppermost cell
	BEQ     LBB36			;Bra if already in uppermost cell
;
LBB1D	CMPA    1,Y			;Upper TPS limit for current cell
	BCS     LBB36			;Bra if TPS is <=, remain in current cell

	DEX				;-1, point to upper TPS limit to remain in cell
;
;-Here to change cells
;
LBB23	CLRB				;Clear cell #
	CMPA    0,X			;TPS limit to remain in cell #0
	BCS     LBB33			;Bra if <, use cell #0
;
	INCB				;Advance to cell #1 
	CMPA    2,X			;TPS limit to remain in cell #1
	BCS     LBB33			;Bra if <, use cell #1
;
	INCB				;Advance to cell #2
	CMPA    4,X			;TPS limit to remain in cell #2
	BCS     LBB33			;Bra if TPS <, use cell #2

	INCB				;Advance to cell #3 
;
LBB33	STAB    L0145			;Save current adaptive modifier cell 
;
LBB36	PULX				;Get address of current set of cells
	LDAB    L0145			;Load up adaptive modifier cell # 
	ABX				;Add in as offset
	LDAA    0,X			;Load contents of cell
	ASRA				;/2
	ASRA				;/2
	ADCA    #$0000			;Round if needed
;
LBB41	STAA    L0144			;Save it, current adaptive pressure modifier 
;
LBB44	LDAA    L0144			;Current adaptive modifer
	RTS				;Return 
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~    
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Addresses of transission related tables
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Load based line pressure (0-64 MPH)
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;	Addr.			;Shift pattern
;
LBB48	FDB L62D8   		;Normal  
	FDB L62D8   		;Perf.
	FDB L62D8   		;Manual 
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Load based line pressure (64-128 MPH)
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;	Addr.			;Shift pattern
;
LBB4E   FDB L63FC		;Normal  
	FDB L63FC		;Perf. 
	FDB L63FC		;Manual 
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Pressure offset vs %TPS, normal pattern
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;	Addr.			;Gear #
;
LBB54   FDB L6520		;2nd 
	FDB L6531   		;3rd 
	FDB L6542   		;4th 
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Pressure offset vs %TPS, performance pattern
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;	Addr.			;Gear #
;
LBB5A	FDB L6520   		;2nd 
	FDB L6531   		;3rd 
	FDB L6542		;4th 
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Pressure offset vs %TPS, manual pattern
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;	Addr.			;Gear #
;
LBB60   FDB L6520   		;2nd
	FDB L6531   		;3rd
	FDB L6542		;4th
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Line pressure modifier for downshift table index vs. shift
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;-Standard pattern		
;
;	Index			;Shift #
;
LBB66   FDB L6553		;2 -> 1
	FDB L6567		;3 -> 2
	FDB L657B		;4 -> 3
;
;-Performance pattern
;
LBB6C   FDB L6553		;2 -> 1
	FDB L6567		;3 -> 2
	FDB L657B		;4 -> 3
;
;-Manual pattern
;
LBB72   FDB L6553		;2 -> 1
	FDB L6567		;3 -> 2
	FDB L657B		;4 -> 3
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Addr. of transmission dependant line pressure vs TPS and trans temp
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;	Table addr.		;Gear #
;
LBB78	FDB $0000		;1st 
	FDB L624B		;2nd 
	FDB L627A		;3rd
	FDB L62A9		;4th
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Baro gain factor for line pressure baro compensation index
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;	Table addr.		;Gear #
;
LBB80   FDB L6077		;2nd 
	FDB L6088		;3rd 
	FDB L6099		;4th 
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; Routine to prevent excess slip during jump shifts
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LBB86	PSHA				;Line pressure to stack    
	BRSET   L0081,#$20,LBBCA	;Status word, bra if b5==1, drive range has
					;just been selected during 2->4 shift, prevent repeat
					;2->4 jump shift check
;
	BRSET   L007F,#$08,LBBFD	;Status word, bra if b3==1, downshift in progress
;
	BRCLR   L007F,#$01,LBBD6	;Status word, bra if b0==0, upshift not in progress,
					;continue with compensation if needed
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Upshift in progress, check for jump shift
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
	LDAB    L00D3			;Load gear #
	CMPB    #$01			;2nd gear
	BNE     LBBA1			;Bra if in other gears
;
;-In second gear
;
	LDAA    L5FC9			;Time duration to check for jump shift
	STAA    L018C			;Save it, jump shift timer
	BRA     LBBD6			;Bra to continue
;
;-In other gears
;
LBBA1	CMPB    #$02			;3rd gear
	BNE     LBBB8			;Bra if not in 3rd gear
;
;-In 3rd gear
;
	LDAA    L5FCA			;Time threshold to check for 2->4 jump shifts
	STAA    L018D			;Save it, jump shift 2->4 timer
	LDAA    L018C			;Load jump shift timer
	BEQ     LBBD6			;Bra if ==0, no 1->3 jump shift
;
;-1st to 3rd shift in progress
;
	LDAA    L5FCB			;Time for 1->3 shift compensation
	LDAB    L5FCC			;Added pressure for 1->3 shift
	BRA     LBBD0			;Bra to continue				
;
;-In 4th gear
;											
LBBB8	LDAA    L018C			;Load jump shift timer
	BEQ     LBBC5			;Bra if ==0, no 1->4 jump shift
;
;-1st to 4th shift in progress
;
	LDAA    L5FCD			;Time for 1->4 shift compensation
	LDAB    L5FCE			;Added pressure for 1->4 shift					
	BRA     LBBD0			;Bra to apply added pressure and continue				
;
;-Check if recent 2nd to 4th shift
;											
LBBC5	LDAA    L018D			;Jump shift 2->4 timer
	BEQ     LBBD6			;Bra if ==0, no recent 2->4 shifts
;
;-2nd to 4th shift in progress
;
LBBCA	LDAA    L5FCF			;Time for 2->4 shift compensation
	LDAB    L5FD0			;Added pressure for 2->4 shift
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Here to apply jump shift pressure compensation
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LBBD0	STAA    L018E			;Jump shift compensation apply timer
	STAB    L018F			;Jump shift pressure compensation
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Here to continue with jump shift compensation if needed
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;										
LBBD6	PULA				;Get line pressure back
	LDAB    L018E			;Jump shift compensation duration timer
	BEQ     LBBE9			;Bra if ==0, time threshold to apply compensation 
					;has expired
;
	DECB				;-1
	STAB    L018E			;Save it, compensation duration timer
	ADDA    L018F			;Add in the pressure compensation to the line pressure
	BVC     LBBE9			;Bra if no 2's comp. overflow
;
	LDAA    #$7F			;Clear overflow
	ADCA    #$00			;
;
;-Decay out timers
;
LBBE9	LDAB    L018C			;Load jump shift timer
	BEQ     LBBF2			;Bra if ==0
;
	DECB				;-1
	STAB    L018C			;Save it
;
LBBF2	LDAB    L018D			;Jump shift 2->4 timer
	BEQ     LBBFB			;Bra if ==0
;											
	DECB				;-1
	STAB    L018D			;Save it
;
LBBFB	BRA     LBC0B			;Bra to return
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~	
; Here to perform resets if downshift in progress
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;									
LBBFD	CLRA				;Preclear A
	STAA    L018E			;Jump shift compensation duration timer
	STAA    L018C			;Jump shift timer
	STAA    L018D			;Jump shift 2->4 timer
	STAA    L018F			;Jump shift pressure compensation
	PULA				;Get time pressure back
;
LBC0B	RTS				;Return
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; Line pressure shift modification routine
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;			
LBC0C	BRSET   L0092,#$04,LBC14	;Status word, bra if b2==1, press. switch. man. error
;
	BRCLR   L009C,#$70,LBC1C	;Trans range sel. word, bra if b4-b6==0, not in P/N, 
					;reverse, or invalid range 
;
;-Here to skip routine if errors
;
LBC14	BCLR    L008A,#$01		;Status word, clear b0, upshift flag
	CLRA				;Clear accumulated pressure
	CLRB				;
	JMP     LBCE8			;Jump to save and return
;	
;-Here if no errors, check status of transmission shifts
;										
LBC1C	BRSET   L007F,#$08,LBC5A	;Status word, bra if b3==1, downshift underway
;											
	BRCLR   L007F,#$01,LBC33	;Status word, bra if b0==0, upshift not in progress
;
;-Upshift has just commenced
;											
	BSET    L008A,#$01		;Set b0, upshift in progress
	LDD     #$0000			;Pre-clear D, perform clears
	STAA    L0139			;Post shift timer
	STAA    L0138			;Start of shift timer
	JMP     LBCE8			;Jump to return
;
LBC33	BRCLR   L008A,#$01,LBC5A	;Status word, bra if b0==0, upshift not in progress
;
;~~~~~~~~~~~~~~~~~~~~~
; Upshift in progress 
;~~~~~~~~~~~~~~~~~~~~~
;											
	LDAB    L00D3			;Load gear #
	DECB				;-1
	LDX     #L601F			;Max time to modulate line pressure for shift
	ABX				;Add in gear # as offset
	LDAA    L00D5			;Load shift duration timer
	CMPA    0,X			;Compare to time limit
	BCC     LBC5A			;Bra if shift time >=
;
;-Look up ramp delay
;		
	LDAA    #$0011			;17 line tables
	MUL				;# lines/table x gear #
	LDX     #L6628			;Start of pressure ramp delay time tables
	ABX				;Add in calc'd offset
	LDAA    L00AC			;Load trans TPS
	JSR     LF15E			;2D lookup
;										
	CMPA    L00D5			;Shift duration timer 
	BLS     LBC5C			;Bra if timer is >=
;
;-Not enough elapsed time to begin ramping pressure
;											
	LDD     #$0000			;Clear D to clear accumulated pressure
	JMP     LBCE8			;Bra to save and return
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Here to look up pressure offset and return if downshift in progress
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;											
LBC5A	BRA     LBCCB			;Bra to continue
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Here if enough time has elapsed to begin ramping shift pressure
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;											
LBC5C	BRSET   L008C,#$10,LBCCB	;
;					
	BRCLR   L0081,#$10,LBC84	;Trans shift stat. word, bra if b4==0, shift has not 
					;yet started
;
;-Shift is now in progress
;											
	LDAA    L0138			;Start of shift time
	BNE     LBC6E			;Bra if !=0, shift time already updated
;											
	LDAA    L00D5			;Shift duration timer 
	STAA    L0138			;Save it as start of shift time
;
;-Shift already underway here
;
LBC6E	LDX     #L665B			;Start addr. of pressure ramp delay tables
	ABX				;Add in calc'd offset
	LDAA    L00AC			;Trans %TPS
	JSR     LF15E			;2D lookup
;
	ADDA    L0138			;Add in time that shift started
	BCC     LBC7E			;Bra if no overflow
;
	LDAA    #$00FF			;Clear overflow
;
LBC7E	CMPA    L00D5			;Shift duration timer
	BLS     LBC8E			;Bra if <=, look up shift ramping pressure based on 
					;TPS
;	
;-Time threshold not yet met, use initial ramping pressure 
;										
	BRA     LBC89			;
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Here if shift has not yet started
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;											
LBC84	TST     L0138			;Start of shift time
	BNE     LBC6E			;Bra if !=0, shift previously in progress
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Here if shift has just started
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;											
LBC89	LDX     #L668E			;start addr. initial shift ramping pressure vs. TPS
					;tables
	BRA     LBCAE			;Bra to do lookup
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Here if shift has been in effect for a specified ammount of time
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;		
LBC8E	BRSET   L0081,#$80,LBC9A	;Trans shift status word, bra if b7==1, shift complete
;
;-Shift not yet complete here
;
	CLR     L0139			;Clear post shift timer
	LDX     #L66C1			;Start addr. of shift ramping pressure vs. TPS tables
	BRA     LBCAE			;Bra to go do lookup
;
;~~~~~~~~~~~~~~~~~~~~~~~~
; Here if shift complete
;~~~~~~~~~~~~~~~~~~~~~~~~
;
LBC9A	LDX     #L66F4			;Start addr of post-shift ramp time vs %TPS tables
	ABX				;Add in cal'd offset based on gear #
	LDAA    L00AC			;Load trans TPS
	JSR     LF15E 			;2D lookup
;											
	CMPA    L0139			;Post shift timer
	BLS     LBCCB			;Bra if timer >=, no longer ramping pressure
;
;-Here if still ramping pressure after shift
;											
	INC     L0139			;+1, post shift timer
	LDX     #L66C1			;Start addr. of post-shift ramping pressure vs %TPS
;
;-Look up and add in ramping pressure
;
LBCAE	ABX				;Add in cal'd offset based on gear #
	LDAA    L00AC			;Load trans TPS
	JSR     LF15E			;2D look up
;											
	TAB				;Looked up value to B
	CLRA				;Preclear A
	SUBB    #$0080			;Make signed value
	SBCA    #$0000			;Perform carry if needed
	ASLD				;x2
	ASLD				;x2
	ASLD				;x2
	ADDD    L013A			;Accumulated pressure
	BVC     LBCC9			;Bra if overflow
;											
	LDD     #$7FFF			;Clear overflow
	ADCB    #$0000			;Perform carry if needed
	ADCA    #$0000			;
;
LBCC9	BRA     LBCE8			;Bra to save and exit
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Here if no longer ramping pressure or downshift in progress
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;											
LBCCB	CLR     L0138			;Clear start of shift timer
	BCLR    L008A,#$01		;Clear upshift in progress flag										
	LDAB    L00D3			;Gear #
	BNE     LBCD8			;Bra if in 2-4th
;
;-Clear accumulated pressure if in 1st
;											
	CLRA				;Preclear A
	BRA     LBCE8			;Bra to clear accumulated pressure
;
;-Look up pressure offset 
;											
LBCD8	DECB				;-1, from gear #
	LDAA    #$0011			;17 lines/table
	MUL				;Gear # x lines per table
	LDX     #L65F5			;Start addr of gear pressure offset vs %TPS
	ABX				;Add in calc'd value for lookup
	LDAA    L00AC			;Trans TPS
	JSR     LF15E			;2D lookup
;											
	SUBA    #$0080			;Make signed value
	CLRB				;Clear LSB
;
;-Here to save accumulated pressure and return
;
LBCE8	STD     L013A			;Save it, accumulated pressure
	RTS				;Return
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;~~~~~~~~~~~~~~~~~~~~~~
;
; Not used...
;
;~~~~~~~~~~~~~~~~~~~~~~
;											
LBCEC	RTS				;Return
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; Routine to look up desired force motor current
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LBCED	BRCLR   L0093,#$08,LBCF5	;Status word, bra if b3==0, no high system voltage 
					;fault
;	
;-Excessive voltage present, clear desired current and exit
;									
	CLRA				;
	JMP     LBD44			;
;	
;-Scale trans temp for line pressure hyst. lookup
;										
LBCF5	LDAA    L00AF  			;Load trans temp
	ASLA				;x2
	BCS     LBCFE			;Bra if overflow
;
	ADDA    L00AF  			;Add trans temp back in, now x3
	BCC     LBD00			;Bra if no overflow
;
LBCFE	LDAA    #$FF			;Clear overflow
;
;-Look up hysteresis for determining direction of line pressure
;
LBD00	LDX     #L6B72			;Line pressure hyst. vs trans temp
	JSR     LF15E			;2D lookup
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Determine direction of change in line pressure
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;		
	TAB				;Get result into B
	LDAA    L0143			;Load previous desired line pressure
	SBA				;Subtract pressure hyst.
	BCS     LBD11			;Bra if underflow
;											
	CMPA    L00DC			;Compare to desired line pressure
	BHI     LBD29			;Bra if the desired line pressure is falling
;
;-Line pressure steady or rising
;										
LBD11	LDAA    L0143			;Load previous line pressure
	ABA				;Add in hyst.
	BCS     LBD1B			;Bra if overflow
;											
	CMPA    L00DC			;Compare to desired line pressure
	BCS     LBD21			;Bra if line pressure <, desired pressure rising
;	
;-Line pressure is steady, maintain current direction of change
;										
LBD1B	BRSET   L008A,#$40,LBD21	;Status word, bra if b6==1, line pressure was rising
;											
	BRA     LBD29			;Bra, line pressure was falling previously
;
;-Here if line pressure is rising
;											
LBD21	BSET    L008A,#$40		;Set b6, line pressure is rising
	LDX     #L692A			;Desired force motor current if line pressure rising
	BRA     LBD2F			;Bra to do lookup
;
;-Here if line pressure is falling
;											
LBD29	BCLR    L008A,#$40		;Clear b6, line pressure is falling										
	LDX     #L6A4E			;Desired force motor current if line pressure falling
;
;-Scale desired line pressure for lookup
;
LBD2F	LDAA    L00DC			;Desired line pressure
	PSHA				;Push line pressure to stack
	LDAB    #170			;Scalar for lookup
	MUL				;(170 x line pressure)/256
	PULB				;Get line pressure back
	ASLB				;x2
	BCS     LBD3C			;Bra if overflow
;
	ABA				;Add it in, now desired line pressure x 2.66
	BCC     LBD3E			;Bra if no overflow
;
LBD3C	LDAA    #255			;Clear overflow
;
;-Perform desired force motor current lookup
;
LBD3E	TAB				;Get line pressure into B
	LDAA    L00AF  			;Load trans temp
	JSR     LF17B			;3D lookup
;
LBD44	STAA    L0147			;Save it, desired force motor current
	LDAA    L00DC			;Load desired added line pressure
	STAA    L0143			;Save it, desired line pressure, one delayed
	RTS				;Return
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;			
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; Routine to check quals for disabling TCC
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LBD4D	LDX     #L6727			;Index constants
	LDAB    L0184			;Load TCC accel unlock timer
	BEQ     LBD59			;Bra if ==0
;
	DECB				;-1
	STAB    L0184			;Save it
;
LBD59	LDD     L00AC			;Load trans TPS
	STAA    L00AD			;Save it, one delayed TPS
	SBA				;Subtract one delayed TPS from current
	BCC     LBD68			;Bra if TPS is rising
;
;-TPS falling here
;
	CMPA    $1A,X			;L6741, rate of decrease for TPS to disable TCC
	BHI     LBD71			;Bra if rate of decrease is <
;
	LDAA    $1D,X			;L6744, time for TCC to remain unlocked after rapid 
					;decel
	BRA     LBD77			;Bra to save				
;	
;-Here if TPS rising
;										
LBD68	CMPA    $1B,X			;L6742, compare TPS rate to threshold for TCC to 
					;remain locked
	BCS     LBD71			;Bra if TPS rate is <
;
	LDAB    $1C,X			;L6743, load time for TCC to remain unlocked after
					;rapid accel
	STAB    L0184			;Save it TCC accel unlock timer
;
LBD71	LDAA    L0183			;Load rapid decel timer
	BEQ     LBD7A			;Bra if ==0, continue
;
	DECA				;-1
;
LBD77	STAA    L0183			;Save it
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Check for excessive TCC slip
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LBD7A	LDD     L00E5			;Load torque converter slip
	CPD     $10,X			;L6737, threshold to flag excessive slip
	BGE     LBD86			;Bra if slip is <=
;
	BSET    L0085,#$80		;Status word, set b7, excessive negative TCC slip
	BRA     LBD8D			;
;											
LBD86	SUBD    $12,X			;L6739, threshold to clear excessive negative slip 
					;flag
	BLE     LBD8D			;Bra if slip >=
;
	BCLR    L0085,#$80		;Clear excessive negative slip flag
;	
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Check lower TPS thresholds for preventing repeated locking/unlocking of TCC
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LBD8D	LDAB    $16,X			;L673D, low speed TPS threshold for disabling TCC when 
					;being applied/released
	BRSET   L0085,#$40,LBD99	;Status word, bra if b6==1, TCC is being applied
;
	BRSET   L0085,#$10,LBD99	;Bra if b4==1, TCC being released
	LDAB    $17,X			;L673E, low speed TPS threshold for disabling TCC when
					;locked/unlocked
;
;-Check whether using low or high speed TPS thresholds
;
LBD99	LDAA    $14,X			;L673B, MPH threshold for using high speed thresholds
	BRCLR   L0085,#$08,LBDA1	;Bra if b3==0, not using high speed TPS TCC thresholds 
					;
;
	LDAA    $15,X			;L673C, MPH thresholds for reverting back to low speed
					;thresholds
;
LBDA1	BCLR    L0085,#$08		;Preclear using high speed thresholds flag
	CMPA    L00D1			;Compare to MPH x 2
	BHI     LBDB7			;Bra if MPH <
;
;-Using high speed thresholds here
;
	LDAB    $18,X			;L673F, high speed TPS threshold for disabling TCC
					;when being applied/released
	BRSET   L0085,#$40,LBDB4	;Bra if b6==1, TCC is being applied
;
	BRSET   L0085,#$10,LBDB4	;Bra if b4==1, TCC being released
;
	LDAB    $19,X			;L6740, high speed TPS threshold for disabling TCC 
					;when locked/unlocked
;
LBDB4	BSET    L0085,#$08		;Set b3, using high speed TPS thresholds
;
;-Check if repeat unlock/relock prevention optioned
;
LBDB7	BRCLR   0,X,#$04,LBDF2		;L6727, opt. byte, bra if b2==0, not disabling TCC
					;to prevent repeat locking/unlocking
;
;-Check TPS threshold
;
	CMPB    L00AC			;Compare to trans TPS
;
LBDBD	BLS     LBDF2			;Bra of trans TPS >=, return
;
;-Low TPS, flag and check if preventing repeated locking/unlocking
;
	BSET    L0087,#$08		;Status word, set b3, repeat unlock/relock prevention
					;in effect
	BRSET   L0085,#$40,LBDCC	;Bra if b6==1, TCC fully applied
;
	BRSET   L0087,#$20,LBDCC	;Bra if b5==1, TCC disabled due to rapid decel 
;
;-Not preventing repeated locking/unlocking
;
	BRA     LBDF2			;Bra to return
;				
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Prevent repeated TCC locking and unlocking 
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LBDCC	BRCLR   L0087,#$10,LBDF2	;Status word, bra if b4==0, PWM TCC engagement not 
					;pending
;
;-PWM TCC to be locked
;
	BSET    L0087,#$80		;Status word, set b7, TCC relock to be delayed
	BCLR    L0087,#$10		;Clear b4, clear PWM TCC to be locked flag
	LDD     L0119			;TCC coast timer
	CPD     L674A			;Compare to time threshold to allow extended relock 
					;delays
	BHI     LBDED			;Bra if higher, allow TCC to relock without delay
;
;-Time threshold between relocks met to accumulate TCC delay
;
	LDD     L011F			;Accumulated TCC relock delay
	ADDD    L674E			;Ammount to increment accumulated TCC relock delay
	BCC     LBDEA			;Bra if no overflow
;
	LDD     #$FFFF			;Clear overflow
;
LBDEA	STD     L011F			;Save it, TCC
;
LBDED	CLRA				;
	CLRB				;
	STD     L0119			;Reset TCC coast timer
;
LBDF2	RTS				;Return
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~     					
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; Routine to handle TCC related timers
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Increment TCC coast release timer
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;																						
LBDF3	LDX     L0119			;TCC coast release timer
	INX				;+1
	BEQ     LBDFC			;Bra if no roll-over
;
	STX     L0119			;Save it
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Decrement accumulated TCC relock delay timer
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LBDFC	LDX     L011B			;Accumulated TCC relock period timer
	INX				;+1
	BNE     LBE03			;Bra if no overflow
;
	DEX				;-1, roll back timer
;
LBE03	STX     L011B			;Save it
	CPX     L674C			;Period to decrement accumulated relock delay timer
	BCS     LBE1D			;Bra if period timer <, dont decrement accumulated
					;relock yet
;
;-Threshold met to decrement the timer
;
	CLRA				;Reset period timer
	CLRB				;
	STD     L011B			;Save timer
	LDD     L011F			;Load TCC accumulated relock delay timer
	SUBD    L6750			;Sumbtract ammount to reduce delay by each period
	BCC     LBE1A			;Bra if no underflow
;
	CLRA				;Clear underflow
	CLRB				;
;
LBE1A	STD     L011F			;Save it
;
;-Here if not yet time to decement the timer
;
LBE1D	LDD     L011F			;Load timer
	CPD     L6748			;Min time for repeated TCC relocks
	BCC     LBE2C			;Bra if time >=
;
	LDD     L6748			;Load min time for repeated relocks
	STD     L011F			;Save it
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Handle reset of TCC rapid decel disable flag 
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LBE2C	LDX     #L6727			;Index TCC constants
	BRCLR   L0087,#$20,LBE46	;Status word, bra if b5==0, TCC not disabled due to 
					;rapid decel
;										
;-TCC currently disabled
;
	LDAA    L0121			;Load rapid decel flag reset timer
	CMPA    L6752			;Time threshold to reset flag
	BCC     LBE43			;Bra if >=, reset rapid decel disable flag
;
	INCA				;+1
	BEQ     LBE49			;Bra if roll-over
;
	STAA    L0121			;Save timer
	BRA     LBE49			;Continue			
;
;-Here to reset TCC rapid decel disable flag
;											
LBE43	BCLR    L0087,#$20		;Clear b5, reset flag
;										
LBE46	CLR     L0121			;Reset timer
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Inrement TCC engage/release timer
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LBE49	LDAB    L0129			;Load TCC engage/release timer
	INCB				;+1
	BEQ     LBE52			;Bra if roll-over
;
	STAB    L0129			;Save timer
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Increment TCC engage/release timer
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LBE52	LDAB    L012A			;TCC engage/release delay timer
	INCB				;+1
	BEQ     LBE5B			;Bra if roll-over
;
	STAB    L012A			;Save timer
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Increment TCC engage/release shift delay timer if TCC is not being applied or released
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LBE5B	BRSET   L0085,#$40,LBE6C	;Status word, bra if b6==1, TCC being applied, no
					;gear shifts
;
	BRSET   L0085,#$10,LBE6C	;Bra if b4==1, TCC being released, no gear shifts
;
;-TCC fully locked/unlocked, inc timer to allow the trans to shift
;
	LDAB    L0123			;Load TCC engage/release shift delay timer
	INCB				;+1
	BEQ     LBE6C			;Bra if roll-over
;
	STAB    L0123			;Save it
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Decrement/reset TCC downshift disable timer
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LBE6C	LDAA    L0182			;Load TCC dowshift disable timer
	BEQ     LBE74			;Bra if ==0, update timer if downshift underway
;
	DECA				;-1
	BRA     LBE7A			;Bra to save and continue				
;											
LBE74	BRCLR   L007F,#$08,LBE7D	;Status word, bra if b3==0, downshift not underway
;
;-Downshift underway, reset timer
;
	LDAA    $1E,X			;L6745, time to disable TCC during downshift
;
LBE7A	STAA    L0182			;Save timer
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Decrement/reset TCC upshift disable timer
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LBE7D	LDAA    L0122			;Load TCC upshift disable timer
	BEQ     LBE85			;Bra if ==0, update timer if upshift underway
;
	DECA				;-1
	BRA     LBE8B			;Bra to save and continue
;											
LBE85	BRCLR   L007F,#$01,LBE8E	;Status word, bra if b0==0, upshift not underway
;
;-Upshift underway, reset timer
;											
	LDAA    $1F,X			;L6746, time to disable TCC during upshift
;
LBE8B	STAA    L0122			;Save timer
;
LBE8E	RTS				;Return
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;~~~~~~~~~~~~~~~~~~~~~~~~~
;
; PWM TCC control routine
;
;~~~~~~~~~~~~~~~~~~~~~~~~~
;											
LBE8F	BRSET   L0086,#$10,LBE9E	;Status word, bra if b4==1, TCC diag. override in
					;effect, skip routine
;
	BRSET   L0087,#$04,LBEA4	;Status word, bra if b2==1, TCC commanded to lock
;
	BRSET   L0087,#$02,LBEA1	;Bra if b1==1, TCC commanded to unlock
;
;-Here if no command set
;
	JMP     LC065			;Perform resets				
;	
;-Here to exit routine
;										
LBE9E	JMP     LC0B7			;Jump to return
;	
;-Here to unlock TCC
;				
LBEA1	JMP     LBF79			;Jump to disengage TCC
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Check quals for engaging TCC
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;				
LBEA4	BRSET   L0083,#$02,LBEC9	;Status word, bra if b1==1, trans over-temp
;
	BRCLR   L0087,#$80,LBEC9	;Status word, bra if b7==0, not delaying TCC lockup
;
;-Delaying TCC relock
;
	LDD     L011D			;Load TCC relock delay timer
	CPD     L011F			;Compare to accumulated TCC relock delay
	BCC     LBEC6			;Bra if time >=, allow TCC to be applied
;		
;-Not enough time to relock TCC
;									
	BSET    L0087,#$40		;Status word, set b6, insufficient time to relock 
					;TCC
	ADDD    #$01			;Increment TCC relock delay timer
	BCC     LBEC0			;Bra if no overflow
;
	LDD     #$FFFF			;Clear overflow
;
LBEC0	STD     L011D			;Save it
	JMP     LC06A			;Jump to perform resets, not locking TCC yet			
;
;~~~~~~~~~~~~~~~~~~~~
; Here to engage TCC
;~~~~~~~~~~~~~~~~~~~~
;											
LBEC6	BCLR    L0087,#$80		;Clear b7, delay TCC relock flag
;	
;-Here if trans overtemp, lock TCC
;										
LBEC9	CLRA				;
	CLRB				;
	STD     L011D			;Clear TCC relock delay timer
	BCLR    L0087,#$40		;Clear b6, TCC relock delayed flag
;											
	BRSET   L0085,#$40,LBEE5	;Bra if b6==1, TCC already applied
;
	BSET    L0085,#$40		;Status word, set b6, TCC applied
	BCLR    L0085,#$10		;Clear b4, TCC is being released
	CLRA				;
	STAA    L0123			;Clear TCC apply/release shift delay timer
	STAA    L0129			;Clear TCC engage/release timer
	STAA    L012A			;Clear TCC engage/release delay timer
;
LBEE5	BRSET   L0086,#$04,LBEFF	;Status word, bra if b2==1, TCC locked 
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Here if TCC currently unlocked
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
	BSET    L0086,#$08		;Status word, preset b3, insufficient time 
					;has passed to lock TCC 
	LDAA    L012A			;TCC engage timer 
	CMPA    L691F			;Min time for TCC to lock
	BCC     LBEF6			;Bra if enough time has passed
;
;-Not enough time has passed
;
	BRA     LBF0B			;Bra to continue with TCC unlocked				
;											
LBEF6	BSET    L0086,#$06		;Status word, set b1 TCC locked/being applied, and b2,
					;lock TCC
	CLR     L012A			;Clear TCC engage timer
	BCLR    L0086,#$08		;Clear b3, enough time has passed to lock TCC
;	
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Here if TCC currently locked, check if engagement of PWM TCC ok
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;										
LBEFF	BRSET   L0086,#$01,LBF1D	;Status word, bra if b0==1, enough time has
					;passed to lock PWM TCC
;
;-Check if enough time has passed to lock PWM TCC
;
	LDAA    L012A			;TCC engagement delay timer
	CMPA    L6920			;Min time for PWM TCC to engage
	BCC     LBF14			;Bra if enough time has passed
;											
LBF0B	LDD     L691B			;Load DC for PWM TCC to be unlocked
	STD     L0124			;Save it, PWM TCC
	JMP     LC0B7			;Continue with TCC unlocked			   
;	
;-Allow PWM TCC to lock
;								
LBF14	BSET    L0086,#$01		;Set b0, time quals met to lock PWM TCC
	CLR     L012A			;Reset PWM TCC engage/release delay timer
	CLR     L0129			;Reset PWM TCC engage/release timer
;
LBF1D	BSET    L0087,#$10		;Status word, set b4, PWM TCC to be locked
	LDAB    L0129			;PWM TCC engage/release timer
	CMPB    L6923			;Duration of PWM TCC engagement
	BCC     LBF76			;Bra if time >=, PWM TCC engaged
;
;~~~~~~~~~~~~~~~~
; Engage PWM TCC
;~~~~~~~~~~~~~~~~
;
;-Look up TCC operating point
;
	LDX     #LC1E1			;Start of apply PWM TCC operating point vs. TPS table 
					;index 
	LDAB    L00D3			;Load gear #
	JSR     LD90A			;Routine to look up table addr.
;											
	BCC     LBF35			;Bra if address was valid
;
	LDX     #L68B5			;Null address returned, load default
;
LBF35	LDY     #L010B			;Load min ALDL addr.
	JSR     LC0B8			;Routine to look up min apply rate
;											
	LDAA    L0110			;Load ALDL addr.
	BNE     LBF53			;Bra if !=0, dont look up apply rate
;			
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Look up desired apply rate based on TPS
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
	LDX     #LC243			;Load start of table indexes
	LDAB    L00D3			;Load gear #
	JSR     LD90A			;Routine to look up table addr.
;											
	BCC     LBF4E			;Bra if valid addr. returned
;
	LDX     #L68D7			;Null addr., load default
;
LBF4E	LDAA    L00AC			;Load trans TPS
	JSR     LF15E			;2D lookup
;
;-New DC = (257 x engage rate)/32 + old DC
;											
LBF53	LDAB    #$08			;Scalar for engage rate
	MUL				;Scalar x engage rate
	STAA    L0185			;Temp store it to ram
	ADDB    L0185			;Add in MSB of result to LSB of result
	ADDB    L0125			;Add in LSB of stored DC
	BCC     LBF62			;Bra if no overflow
;
	INCB				;Perform carry
;
LBF62	ADCA    L0124			;Add in stored DC and perform carry if needed
	BCC     LBF6A			;Bra if no overflow
;
	LDD     #$FFFF			;Clear overflow
;
LBF6A	CPD     L00EB			;Compare to operating point
	BCC     LBF71			;Bra if >=
;
	LDD     L00EB			;Load PWM TCC operating point
;
LBF71	STD     L00ED			;Save it, accumulated PWM TCC DC
	JMP     LC046			;Bra to output apply rate				
;	
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Here to fully apply PWM TCC
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;										
LBF76	JMP     LC054			;Jump to lock TCC and exit				
;	
;~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Here to disengage PWM TCC
;~~~~~~~~~~~~~~~~~~~~~~~~~~~
;										
LBF79	BRSET   L0085,#$40,LBF84	;Status word, bra if b6==1, TCC currently applied
;
	BRSET   L0085,#$10,LBF95	;Bra if b4==1, TCC is being released
;
;-Here if TCC released, perform resets
;
	JMP     LC065			;Jump to reset				
;	
;-Here if TCC was previously applied
;										
LBF84	BSET    L0085,#$10		;Set b4, TCC is being released
	BCLR    L0085,#$60		;Clear b5, TCC fully locked and b6, TCC applied
	CLRA				;
	CLRB				;
	STD     L011D			;Clear
	STAA    L0129			;Clear PWM TCC engage/release timer
	STAA    L012A			;Clear PWM TCC engage/release delay timer
;
LBF95	BRCLR   L0086,#$01,LBFAE	;Status word, bra if b0==0, TCC now fully unlocked
;
	BRCLR   L0086,#$04,LBFAE	;Status word, bra if b2==0, TCC currently unlocked
;
;-TCC is not fully unlocked
;
	LDAA    L0129			;Load engage/release timer
	CMPA    L6924			;Time for TCC to release
	BLS     LBFE6			;Bra if time <=, TCC still releasing
;	
;-Enough time has passed to fully unlock TCC
;									
	BCLR    L0086,#$01		;Clear b1, TCC now fully unlocked
	CLR     L012A			;Clear engage/release delay timer
	CLR     L0129			;Clear engage/release timer
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~
; Here to fully unlock TCC
;~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LBFAE	LDD     L691B			;DC for PWM TCC to be unlocked
	STD     L0124			;Save it, PWM TCC DC
	BRSET   L0086,#$01,LBFD2	;Bra if b0==1, time quals still have not yet
					;been met to fully unlock TCC
;
	BRCLR   L0086,#$04,LBFD2	;Status word, bra if b2==0, TCC unlocked
;
	LDAA    L012A			;Load PWM TCC engage/release delay timer
	CMPA    L6921			;Min time for TCC to remain unlocked
	BHI     LBFC6			;Bra if time >, continue with resets
;
	BRA     LBFDA			;Bra to return
;			
LBFC6	BCLR    L0086,#$04		;Status word, clear b2, unlock TCC
	CLR     L012A			;Clear engage/release delay timer
	CLR     L0129			;Clear engage/release timer
	BCLR    L0086,#$20		;
;	
;-Check time quals to fully unlock TCC and reset
;									
LBFD2	LDAA    L012A			;Load engage/release timer
	CMPA    L6922			;Time threshold to fully unlock TCC
	BHI     LBFE0			;Bra if time is >
;
;-Here if unlocking TCC but not performing resets
;
LBFDA	BSET    L0086,#$20		;Set b5, TCC to remain unlocked
	JMP     LC0B7			;Jump to return				
;	
;-Here to fully disengage TCC and reset
;										
LBFE0	BCLR    L0086,#$22		;Clear b1, TCC locked/being applied, and b5, TCC to
					;remain unlocked
	JMP     LC065			;Jump to disengage TCC and perform resets				
;
;~~~~~~~~~~~~~~~~~~~~~
; Here to release TCC
;~~~~~~~~~~~~~~~~~~~~~
;
;-Look up/construct release operating point
;
LBFE6	LDX     #LC212			;Load index of release operating point table addresses
	LDAB    L00D3			;Load gear #
	JSR     LD90A			;Routine to fetch table address
;										
	BCC     LBFF3			;Bra if valid address returned
;
	LDX     #L6871			;Null address, load default release operating point 
					;vs. TPS
;
LBFF3	LDY     #L010C			;
	JSR     LC0B8			;Routine to construct release operating point
;										
	LDAA    L0110			;
	BNE     LC01C			;Bra if !=0
;
	LDAA    L0184			;Load TCC accel release timer
	BEQ     LC00A			;Bra if =0, not unlocking TCC
;
;-Rapid accel, unlock TCC
;
	LDAA    L6747			;Release rate for rapid accel
	CLRB				;Clear LSB
	BRA     LC025			;Bra to continue				
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~
; Look up TCC release rate 
;~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LC00A	LDX     #LC274			;Load index of release table addresses
	LDAB    L00D3			;Load gear #
	JSR     LD90A			;Look up address of table
;
	BCC     LC017			;Bra if address valid
;
	LDX     #L6893			;Null address, load default
;
LC017	LDAA    L00AC			;Load trans TPS
	JSR     LF15E			;2D lookup
;
;-New DC =ABS[(257 x engage rate)/32 - old DC]
;										
LC01C	LDAB    #$08			;Scalar for release rate
	MUL				;Scalar x release rate
	STAA    L0185			;Save MSB
	ADDB    L0185			;Add into LSB
;
LC025	SUBB    L0125			;Subtract LSB of PWM TCC DC
	BCC     LC02B			;Bra if no underflow
;
	DECB				;-1
;
LC02B	SBCA    L0124			;Subtract MSB of PWM TCC DC
	BCS     LC034			;Bra if underflow
;	
;-Positive result, clear value
;									
	CLRA				;Clear result
	CLRB				;
	BRA     LC03F			;Bra to save and continue
;	
;-DC remaining, get absolute value
;									
LC034	NEGA				;Get abs. value of result
	NEGB				;
	SBCA    #$00			;
	CPD     L00EB			;Compare to PWM TCC operating point
	BLS     LC03F			;Bra if current DC is <=
;								
	LDD     L00EB			;Load operating point DC instead
;
LC03F	STD     L00ED			;Save it, desired PWM TCC
	BRA     LC046			;Bra to output DC	
;											
	JMP     LC065			;?				
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Here to output DC to TCC sol.
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;											
LC046	LDD     L00ED			;Load desired PWM TCC DC
	CPD     L691B			;DC for PWM TCC to be unlocked
	BCS     LC05C			;Bra if desired DC is <
;
	CPD     L0127			;DC for PWM TCC to be fully locked
	BCS     LC05F			;Bra if DC is <, TCC still locking
;
LC054	LDD     L0127			;Load max PWM TCC DC to fully lock TCC
	BSET    L0085,#$20		;Status word, set b5, TCC fully locked
	BRA     LC05F			;Bra to save and exit
;
;-Here to fully unlock TCC
;											
LC05C	LDD     L691B			;Load DC for PWM TCC to be unlocked
;
LC05F	STD     L0124			;Save it, PWM TCC DC
	JMP     LC0B7			;Jump to return				
;	
;~~~~~~~~~~~~~~~~~~~~~~~~
; Here to perform resets
;~~~~~~~~~~~~~~~~~~~~~~~~
;										
LC065	CLRA				;
	CLRB				;
	STD     L011D			;Clear TCC relock delay timer 
; 
;-Here if not resetting TCC relock delay timer
;
LC06A	BCLR    L0085,#$70		;Status word, clear b6, is being applied, 
					;b5, TCC fully locked, and b4, TCC is being released
	BCLR    L0086,#$05		;Clear b0, time quals met to lock TCC, and b2, TCC
					;locked 
	BCLR    L0086,#$28		;Clear b3, insufficient time to lock TCC, and b5, 
					;TCC to remain unlocked 
	CLR     L012A			;Clear apply/release delay timer
	BRCLR   L0086,#$02,LC083	;Status word, bra if b1==0, TCC not locked/being 
					;applied
;
	LDAA    L6921			;Min time for TCC to remain unlocked
	STAA    L012B			;Save it, manditory TCC timer
	BCLR    L0086,#$02		;Clear TCC locked/being applied flag
;	
;-Check time quals for manditory TCC
;										
LC083	LDAA    L012B			;Load manditory TCC timer
	BEQ     LC08E			;Bra if ==0, check for manditory TCC
;
	DECA				;-1
	STAA    L012B			;Save timer
	BRA     LC0A8			;Bra to unlock TCC			
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Check MPH thresholds for manditory TCC
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LC08E	LDAA    L00D1			;Load MPH x 2
	CMPA    L691D			;MPH threshold for manditory TCC
	BCC     LC0A3			;Bra if speed is >=, threshold met, lock TCC
;
	CMPA    L691E			;MPH threshold to exit manditory TCC
	BLS     LC0A8			;Bra if speed is <=, unlock TCC
;
;-Locking TCC, check for system fault
;
	LDD     L0124			;Load PWM TCC DC
	CPD     L0127			;Compare to DC for PWM TCC to be fully locked
	BNE     LC0A8			;Bra if !=, TCC lockup not allowed due to system fault
;
LC0A3	LDD     L0127			;Load max DC for TCC to be locked
	BRA     LC0AB			;Bra to continue
;
;-Here to disable the TCC
;											
LC0A8	LDD     L691B			;DC for PWM TCC to be unlocked
;
LC0AB	TST     L010A			;PWM TCC diag. DC
	BNE     LC0B4			;Bra if !=0, save as PWM TCC DC
;
	BRSET   L0093,#$08,LC0B7	;Status word, bra if b3==1, high system voltage
;
LC0B4	STD     L0124			;Save it, PWM TCC DC
;
LC0B7	RTS				;Return
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;Routine to generate PWM TCC appy/release operating point
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;											
LC0B8	LDAA    0,Y			;Load ALDL addr.
	TAB				;Get into B
	BNE     LC10A			;Bra if !=0, use addr. for apply rate
;
	PSHX				;Push index addr. to stack
	CLR     L0188			;Preclear overflow tracking accumulator
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Look up line pressure mult. for PWM DC
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
	LDX     #L690A			;PWM TCC DC mult. vs. line pressure table
	LDAA    L00DC			;Load desired added line pressure
	ASLA				;x2 for lookup
	BCC     LC0CC			;Bra if no overflow
;											
	LDAA    #$FF			;Clear overflow
;
LC0CC	JSR     LF15E			;2D lookup
	PULX				;Pull table address back off stack
	PSHA				;Push PWM TCC DC mult. to stack
	LDAB    0,X			;Load base value from table
	MUL				;PWM TCC DC mult. x base DC value
	STD     L0186			;Save it
	PULB				;Get DC mult. back
	LDAA    L00AC			;Load trans TPS
	JSR     LF15E			;2D lookup
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Base DC + DC slope x line pressure mult.
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;											
	MUL				;PWM TCC DC mult. x PWM TCC DC
	SUBD    L0186			;Subtract base value
	ROL     L0188			;Rotate C bit into overflow tracking accumulator
	NEG     L0188			;Invert value
	ASLD				;(PWM TCC DC mult. x PWM TCC DC) x 2, now/128
	ROL     L0188			;Rotate C bit into tracking accumulator
	ADDA    0,X			;Add in base PWM TCC DC value
	PSHB				;Push LSB to stack
	LDAB    L0188			;Load tracking accumulator
	ADCB    #$0000			;Add in C bit from last op.
	PULB				;Get LSB back
	BGE     LC0F8			;Bra if >= 0, no underflow occured
;	
;-Clear underflow
;									
	CLRA				;
	CLRB				;
;
LC0F8	BNE     LC107			;Bra if !=0, overflow occured
;
;-Format to match desired PWM DC ramping rate
;									
	STAA    L0185			;Temp store
	ADDB    L0185			;Add in to 
	BCC     LC10A			;
;											
	ADDD    #$0101			;Perform carry
	BCC     LC10A			;Bra if no overflow
;
;-Clear overflow
;											
LC107	LDD     #$FFFF			;
;
;-Here if no overflow
;
LC10A	SUBA    #128			;Subtract 128, make signed value
	ADDA    1,Y			;Add in addr. to increase/decrease DC
	BVC     LC118			;Bra if no signed overflow
;											
	LDD     #$7FFF			;Clear overflow
	ADCB    #$00			;Perform carry to roll result over if needed
	ADCA    #$00			;
;
LC118	ADDA    #128			;Add 128 back in, result no longer signed
	STD     L00EB			;Save it, PWM TCC operating point
;
	RTS				;Return
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~     
;              
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Index of MPH threshold for engaging TCC vs. TPS tables (normal pattern)
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;    
LC11D   FCB $8			;8 bytes/block
;
;	Addr. index		;Gear #
;
;-High trans temp			
;
LC11E   FDB $0000		;1st 
	FDB L681C		;2nd 
	FDB L682D		;3rd 
	FDB L683E		;4th 
;
;-Manual pattern
;
LC126   FDB $0000		;1st 
	FDB $0000		;2nd 
	FDB L67FA		;3rd
	FDB L680B		;4th 
;
;-Cruise control mode
;   >Performance pattern 
;
LC12E   FDB $0000		;1st
	FDB $0000		;2nd
	FDB L67FA		;3rd
	FDB L680B		;4th
;
;-Cruise control mode
;   >Standard pattern
;
LC136   FDB $0000		;1st 
	FDB $0000		;2nd 
	FDB L684F		;3rd 
	FDB L6860		;4th 
;
;-Standard mode
;   >Performance pattern
;
LC13E   FDB $0000		;1st
	FDB $0000		;2nd
	FDB L67FA		;3rd
	FDB L680B		;4th
;
;-Standard mode
;   >Standard pattern
;
LC146   FDB $0000		;1st
	FDB L67E9		;2nd 
	FDB L67FA		;3rd
	FDB L680B		;4th
;              
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Index of MPH thresholds for engaging TCC (kickdown pattern)
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; 
LC14E   FCB $8			;8 bytes/block
;
;	Addr. index		;Gear #
;
;-High trans temp			
;
LC14F   FDB $0000		;1st 
	FDB L675B		;2nd
	FDB L675C		;3rd 
	FDB L675D		;4th
;
;-Manual pattern
;
LC157   FDB $0000		;1st
	FDB $0000		;2nd
	FDB $0000		;3rd
	FDB $0000		;4th
;
;-Cruise control mode
;   >Performance pattern 
;
LC15F   FDB $0000		;1st
	FDB $0000		;2nd
	FDB $0000		;3rd
	FDB $0000		;4th
;
;-Cruise control mode
;   >Standard pattern
;
LC167   FDB $0000		;1st
	FDB $0000		;2nd
	FDB $0000		;3rd
	FDB $0000		;4th
;
;-Standard mode
;   >Performance pattern
;
LC16F   FDB $0000		;1st
	FDB $0000		;2nd
	FDB $0000		;3rd
	FDB $0000		;4th
;
;-Standard mode
;   >Standard pattern
;
LC177   FDB $0000		;1st
	FDB L6755		;2nd
	FDB L6756		;3rd
	FDB L6757		;4th 
;              
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Index of MPH threshold for releasing TCC vs. TPS tables (normal patterns)
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LC17F   FCB $8			;8 bytes/block
;
;	Addr. index		;Gear #
;
;-High trans temp		
;
LC180   FDB $0000		;1st
	FDB L6794		;2nd 
	FDB L67A5		;3rd 
	FDB L67B6		;4th
;
;-Manual pattern
;
LC188   FDB $0000		;1st
	FDB $0000		;2nd
	FDB L6772		;3rd 
	FDB L6783		;4th
;
;-Cruise control mode
;   >Performance pattern 
;
LC190   FDB $0000		;1st
	FDB $0000		;2nd
	FDB L6772		;3rd 
	FDB L6783		;4th 
;
;-Cruise control mode
;   >Standard pattern
;
LC198   FDB $0000		;1st
	FDB $0000		;2nd
	FDB L67C7		;3rd
	FDB L67D8		;4th
;
;-Standard mode
;   >Performance pattern
;
LC1A0   FDB $0000		;1st
	FDB $0000		;2nd
	FDB L6772		;3rd
	FDB L6783		;4th
;
;-Standard mode
;   >Standard pattern
;
LC1A8   FDB $0000		;1st
	FDB L6761		;2nd
	FDB L6772		;3rd
	FDB L6783		;4th
;              
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Index of MPH thresholds for releasing TCC (kickdown pattern)
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LC1B0   FCB $8			;8 bytes/block
;
;	Addr. index		;Gear #
;
;-High trans temp		
;
LC1B1   FDB $0000		;1st
	FDB L675E		;2nd
	FDB L675F		;3rd
	FDB L6760		;4th
;
;-Manual pattern
;
LC1B9   FDB $0000		;1st
	FDB $0000		;2nd
	FDB $0000		;3rd
	FDB $0000		;4th
;
;-Cruise control mode
;   >Performance pattern 
;
LC1C1   FDB $0000		;1st
	FDB $0000		;2nd
	FDB $0000		;3rd
	FDB $0000		;4th
;
;-Cruise control mode
;   >Standard pattern
;
LC1C9   FDB $0000		;1st
	FDB $0000		;2nd
	FDB $0000		;3rd
	FDB $0000		;4th
;
;-Standard mode
;   >Performance pattern
;
LC1D1   FDB $0000		;1st
	FDB $0000		;2nd
	FDB $0000		;3rd
	FDB $0000		;4th
;
;-Standard mode
;   >Standard pattern
;
LC1D9   FDB $0000		;1st
	FDB L6758		;2nd
	FDB L6759		;3rd
	FDB L675A		;4tH
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Index of PWM TCC apply operating point vs. gear #
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LC1E1   FCB $8			;8 bytes per block
;
;-High trans temp		;Gear #
;
LC1E2   FDB $0000		;1st 
	FDB $0000		;2nd
	FDB L68B5		;3rd
	FDB L68C6		;4th
;
;-Manual pattern
;
LC1EA   FDB $0000		;1st 
	FDB $0000		;2nd
	FDB L68B5		;3rd
	FDB L68C6		;4th
;
;-Cruise control mode 
;   >performance pattern
;
LC1F2   FDB $0000		;1st 
	FDB $0000		;2nd
	FDB L68B5		;3rd
	FDB L68C6		;4th
;
;-Cruise control mode 
;   >standard pattern
;
LC1FA   FDB $0000		;1st 
	FDB $0000		;2nd
	FDB L68B5		;3rd
	FDB L68C6		;4th
;
;-Standard mode 
;   >performance pattern
;
LC202   FDB $0000		;1st 
	FDB $0000		;2nd
	FDB L68B5		;3rd
	FDB L68C6		;4th
;
;-Standard mode 
;   >standard pattern
;
LC20A   FDB $0000		;1st 
	FDB $0000		;2nd
	FDB L68B5		;3rd
	FDB L68C6		;4th
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-PWM TCC release operating point table index vs. gear #
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LC212   FCB $8			;8 bytes per block
;
;-High trans temp		;Gear #
;
LC213   FDB $0000		;1st
	FDB $0000		;2nd
	FDB L6871		;3rd 
	FDB L6882		;4th  
;
;-Manual pattern
;
LC21B   FDB $0000		;1st
	FDB $0000		;2nd
	FDB L6871		;3rd 
	FDB L6882		;4th 
;
;-Cruise control mode 
;   >performance pattern
;
LC223   FDB $0000		;1st
	FDB $0000		;2nd
	FDB L6871		;3rd 
	FDB L6882		;4th 
;
;-Cruise control mode 
;   >standard pattern
;
LC22B   FDB $0000		;1st
	FDB $0000		;2nd
	FDB L6871		;3rd 
	FDB L6882		;4th 
;
;-Standard mode 
;   >performance pattern
;
LC233   FDB $0000		;1st
	FDB $0000		;2nd
	FDB L6871		;3rd 
	FDB L6882		;4th 
;
;-Standard mode 
;   >standard pattern
;
LC23B   FDB $0000		;1st
	FDB $0000		;2nd
	FDB L6871		;3rd 
	FDB L6882		;4th 
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-PWM TCC apply rate table index vs. gear #
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LC243   FCB $8			;8 bytes per block
;
;-High trans temp		;Gear #
;
LC244   FDB $0000		;1st
	FDB $0000		;2nd
	FDB L68D7		;3rd
	FDB L68E8		;4th
;
;-Manual pattern
;
LC24C   FDB $0000		;1st
	FDB $0000		;2nd
	FDB L68D7		;3rd
	FDB L68E8		;4th
;
;-Cruise control mode 
;   >performance pattern
;
LC254   FDB $0000		;1st
	FDB $0000		;2nd
	FDB L68D7		;3rd
	FDB L68E8		;4th
;
;-Cruise control mode 
;   >standard pattern
;
LC25C   FDB $0000		;1st
	FDB $0000		;2nd
	FDB L68D7		;3rd
	FDB L68E8		;4th
;
;-Standard mode 
;   >performance pattern
;
LC264   FDB $0000		;1st
	FDB $0000		;2nd
	FDB L68D7		;3rd
	FDB L68E8		;4th
;
;-Standard mode 
;   >standard pattern
;
LC26C   FDB $0000		;1st
	FDB $0000		;2nd
	FDB L68D7		;3rd
	FDB L68E8		;4th
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-PWM TCC release rate table index vs. gear #
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LC274   FCB $8			;8 bytes per block
;
;-High trans temp		;Gear #
;
LC275   FDB $0000		;1st
	FDB $0000		;2nd
	FDB L6893		;3rd
	FDB L68A4		;4th
;
;-Manual pattern
;
LC27D   FDB $0000		;1st
	FDB $0000		;2nd
	FDB L6893		;3rd
	FDB L68A4		;4th
;
;-Cruise control mode 
;   >performance pattern
;
LC285   FDB $0000		;1st
	FDB $0000		;2nd
	FDB L6893		;3rd
	FDB L68A4		;4th
;
;-Cruise control mode 
;   >standard pattern
;
LC28D   FDB $0000		;1st
	FDB $0000		;2nd
	FDB L6893		;3rd
	FDB L68A4		;4th
;
;-Standard mode 
;   >performance pattern
;
LC295   FDB $0000		;1st
	FDB $0000		;2nd
	FDB L6893		;3rd
	FDB L68A4		;4th
;
;-Standard mode 
;   >standard pattern
;
LC29D   FDB $0000		;1st
	FDB $0000		;2nd
	FDB L6893		;3rd
	FDB L68A4		;4th
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Baro gain for TCC MPH thresholds index vs. gear #
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; Normal shift pattern
;
LC2A5   FCB $8			;8 bytes per block
;
;-High trans temp		;Gear #
;
LC2A6   FDB L5D2C		;1st
	FDB L5D2C		;2nd
	FDB L5D2C		;3rd
	FDB L5D2C		;4th
;
;-Manual pattern
;
LC2AE   FDB $0000		;1st
	FDB $0000		;2nd
	FDB $0000		;3rd
	FDB $0000		;4th
;
;-Cruise control mode 
;   >performance pattern
;
LC2B6   FDB L5D2C		;1st
	FDB L5D2C		;2nd
	FDB L5D2C		;3rd
	FDB L5D2C		;4th
;
;-Cruise control mode 
;   >standard pattern
;
LC2BE   FDB L5D2C		;1st
	FDB L5D2C		;2nd
	FDB L5D2C		;3rd
	FDB L5D2C		;4th
;
;-Standard mode 
;   >performance pattern
;
LC2C6   FDB L5D2C		;1st
	FDB L5D2C		;2nd
	FDB L5D2C		;3rd
	FDB L5D2C		;4th
;
;-Standard mode 
;   >standard pattern
;
LC2CE   FDB L5D2C		;1st
	FDB L5D2C		;2nd
	FDB L5D2C		;3rd
	FDB L5D2C		;4th
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Baro gain for TCC MPH thresholds index vs. gear #
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; Kickdown shift pattern
;
LC2D6   FCB $8			;8 bytes per block
;
;-High trans temp		;Gear #
;
LC2D7   FDB L5D2C		;1st
	FDB L5D2C		;2nd
	FDB L5D2C		;3rd
	FDB L5D2C		;4th
;
;-Manual pattern
;
LC2DF   FDB $0000		;1st
	FDB $0000		;2nd
	FDB $0000		;3rd
	FDB $0000		;4th
;
;-Cruise control mode 
;   >performance pattern
;
LC2E7   FDB L5D2C		;1st
	FDB L5D2C		;2nd
	FDB L5D2C		;3rd
	FDB L5D2C		;4th
;
;-Cruise control mode 
;   >standard pattern
;
LC2EF   FDB L5D2C		;1st
	FDB L5D2C		;2nd
	FDB L5D2C		;3rd
	FDB L5D2C		;4th
;
;-Standard mode 
;   >performance pattern
;
LC2F7   FDB L5D2C		;1st
	FDB L5D2C		;2nd
	FDB L5D2C		;3rd
	FDB L5D2C		;4th
;
;-Standard mode 
;   >standard pattern
;
LC2FF   FDB L5D2C		;1st
	FDB L5D2C		;2nd
	FDB L5D2C		;3rd
	FDB L5D2C		;4th
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; 40 Hz entry point for TPS handling
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LC307	JSR	LB185			;TPS handling routine
	RTS				;Return
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; Trans error handling subroutines schedule table
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LC30B	JSR	LE6B4			;Routine to check for error 37, brake switch stuck on	
;
	JSR	LE74D			;Routine to check for error 38, brake switch inop.
;
	JSR	LEC7F			;Not used
;
	JSR	LED0A			;Routine to check for error 68, TCC slipping/not 
					;engaged
;
	JSR	LED63			;Routine to check for error 69, TCC failed to 
					;disengage
;
	JSR	LE9DC			;Routine for flagging error 73, force motor
;
	JSR	LEAAE			;Routine to check for error 79, prolonged high 
					;trans temp
;
	JSR	LEDD5			;Not used...
;
	JSR	LEDD6			;Routine to check for error 86, ratio of engine 
					;to vehicle speed is low
;
	JSR	LEE44			;Routine to check for error 87, ratio of engine 
					;speed to MPH is high
;
	RTS    				;Return
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~             
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~            
;
; Routine to handle various trans I/O's
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;-Look up shift solenoid states
;
LC32A	LDAB    L00D3			;Gear #
;
LC32C	LDX     #LB3A1			;Index shift solenoid state table
	ABX				;Add in gear # as offset
;
LC330	LDAA    0,X			;Load desired state
;											
LC332	LDAB    L004C			;Load I/O port D word 
	ANDB    #$FC			;Mask all but lower two bytes
	ABA				;Update shift solenoid states
	STAA    L004C			;Save it
;
;-Check trans. ALDL diag. words
;
LC33B	LDX     #L039B			;
	BRCLR   1,X,#$40,LC348		;
;
	BRSET   2,X,#$40,LC35C		;
;
	BRA     LC361			;
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Check quals for enabling force motor
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LC348	BRSET   L0093,#$08,LC361	;Status word, bra if b3==1, high system voltage 
					;fault
;
	BRSET   L001C,#$02,LC361	;Error word #7, bra if b1==1, force motor current error
;										 	
	BRSET   L001D,#$80,LC361	;Error word #8, bra if b7==1, low system volts
;										 	
	BRSET   L0082,#$01,LC361	;Status word, bra if b0==1, low battery volts
;										 	
	BRSET   L008A,#$80,LC361	;Status word, bra if b7==1, force motor fault
;	
;-Here if no errors detected
;									 	
LC35C	BSET    L004C,#$20		;I/O port D word, set b5, enable force motor
;
LC35F	BRA     LC364			;Bra to continue
;
;-Here if errors detected
;
LC361	BCLR    L004C,#$20		;Clear b5, disable force motor
;
LC364	LDAA    L004B			;Load I/O word
	LDX     #L039B			;Load start of trans diag. data buffer
;
;-Check if in mode 4 diags.
;
	BRCLR   L003B,#$08,LC37B	;
;
	BRCLR   1,X,#$20,LC37B		;
;
	ORAA    #$80			;
	BRSET   2,X,#$20,LC383		;
;
	ANDA    #$7F			;
	BRA     LC383			;				
;
;-Check if TCC is to be enabled
;										
LC37B	ORAA    #$80			;Preset b7, enable TCC
	BRSET   L0086,#$04,LC383	;Status word, bra if b2==1, lock TCC 
;
	ANDA    #$7F			;Clear b2, disable TCC
;
LC383	STAA    L004B			;Save it, I/O word
;
	LDX     #L400F			;Opt. byte
	BRSET   0,X,#$40,LC3A5		;Bra if b6==1, non-CC transmission in use
;
;-Output to PWM TCC
;
	BCLR    L0087,#$01		;Status word, preclear b0, PWM TCC unlocked
	LDD     L0124			;Load PWM TCC DC
	CPD     L691B			;DC for PWM TCC to be unlocked
	BLS     LC39F			;Bra if DC is <=
;
	BRCLR   L0086,#$04,LC39F	;Status word, bra if b2==0, TCC unlocked
;
	BSET    L0087,#$01		;Status word, set b0, PWM TCC locked
;											
LC39F	TAB				;Get DC into B
	LDAA    #$7F			;32 Hz switching speed
	STD     L306A			;Save it, PWM TCC
;	
;-Output to 3->2 shift control solenoid
;										
LC3A5	BRSET   0,X,#$01,LC3B9		;Bra if b0 of status word ==1, rev limiter in use
;
	LDX     #L400E			;Opt. byte 
	BRSET   0,X,#$04,LC3B9		;Bra if b2==1, shift light in use 
;
	LDAB    L012E			;Load duty cycle
	LDAA    L012F			;Load switching speed
	STD     L306C			;Save it, 3->2 shift control sol.
;
LC3B9	RTS				;Return     
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; A/D lead-in, get fuel pump volts
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LC3BA   LDX     #L3000			;CPU reg's base
	LDAA    #$02			;
	JSR     LEF12			;Port D config routine
;
	LDAA    #$01			;
	SEI     			;Disable interrupts
	JSR     LEEFC			;A/D routine
;
	LDAB    $34,X			;Get A/D result from channel 4
	CLI				;Re-enable interrupts
	STAB    L0055			;Fuel pump volts x 10
	RTS 				;Return
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~			    
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; Routine for MAP/Baro handling
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LC3CE	LDX     #L5D02			;Opt. byte
	BRCLR   0,X,#$04,LC3F1		;Bra if b2==0, use MAP only
;
;-Here if using MAP and Baro
;
	LDX     #L3000			;Base CPU reg. addr.
	SEI				;Disable interrupts     
	LDAA    #$0005			;Select MAP channel
	JSR     LEF12			;Port D config routine
;
	LDAA    #$0001			;Select channel
	JSR     LEEFC			;A/D routine
;
	CLI				;Enable interrupts     
	LDAA    $34,X			;Get in A/D value
	STAA    L0316			;Save it, A/D baro
	ADDA    #11			;14.5 kPa
	BCC     LC454			;Bra if no overflow
;
	LDAA    #255			;Clear overflow
;
	BRA     LC454			;Bra to continue with baro calcs.
;
;-Here to use MAP only
;
LC3F1	CLRA				;  
	BRSET   L0044,#$10,LC44A	;Status word, bra if b4==1, ign. off

	LDAB    L01E9			;A/D MAP
	CMPB    L0317			;
	BHI     LC44A			;Bra if A/D MAP is >

	BRSET   L006F,#$40,LC464	;Status word, Bra if b6=1, high MAP error
;
	LDAA    L006F			;Status word
	ANDA    #$0C			;Clear all but b2=TPS high error, b3=Low MAP error
	BEQ     LC40D			;Bra if b2, b3==0
;
	CMPB    L4E5E			;Default MAP val
;											
	BCS     LC464			;Bra if MAP is less
;
LC40D	LDAA    L0061			;Load scaled rpms
	CMPA    L4153			;Upper rpm limit for pseudo-baro
	BHI     LC45F			;Bra if RPMs are >
;
	CMPA    L4154			;Lower limit for pseudo-baro
	BCS     LC45F			;Bra if rpms are <
;
	LDAA    L0006			;Cool temp, linear
	CMPA    L4158			;Min cool temp for pseudo-baro
;
	BCS     LC45F			;Bra if cool temp is less
;
	LDAA    L01FC			;%TPS
	CMPA    L4155			;Min %TPS for pseudo-baro
	BLS     LC45F			;Bra if %TPS is <=
																								
	SUBA    L0201			;Subtract slow filtered %TPS
	BCC     LC42E			;Bra if %TPS >=
;
	NEGA 				;Make delta tps positive   
;
LC42E	CMPA    L4156			;2% delta tps, max delta TPS for pseudo-baro
	BHI     LC45F			;Bra if delta TPS is >, no pseudo-baro
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-All quals passed, do pseudo-baro
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
	LDAA    L0062			;RPM/25
	LDAB    #171			;RPM multiplier
	MUL				;RPMs x Mult. 
	ADCA    #0			;Round if needed
	CMPA    #128			;Upper limit
	BLS     LC440			;Bra if <=
;
	LDAA    #128			;Load max

LC440	LDAB    L01FC			;%TPS
	LSRB				;/2 for lookup
	LDX     #L454E			;Baro correction table
	JSR     LF17B			;3D look up

LC44A	ADDA    L01E9			;Add it into current MAP value
	BCC     LC451			;Bra if no overflow

	LDAA    #255			;Clear overflow	
;
LC451	STAA    L0316			;Save it, pseudo-baro
;
LC454	LDAB    L4157			;Baro filter coeff.
	LDX     L0317			;Load old baro value
	JSR     LF0F6			;Filter routine
;
        BRA     LC46B			;Bra to save
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; Baro Init and Lookup Routine
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LC45F	LDAA    L0317			;Filtered baro
	BNE     LC46E			;Bra if !=0
;
LC464	LDAA    L4E5E			;90.8 Kpa default MAP 
	CLRB    			;Preclear B
	STAA    L0316			;Save it, Baro
LC46B	STD     L0317			;Save it, filtered Baro
;
;-Here if Baro already updated
;
; Baro val. for lookup = .590 x Baro - 7.26
; kPa = .625 x baro val. for lookup + 14.9
; 
LC46E	TAB 				;Baro into B
	LDAA    #143			;Use max baro if baro zero'd out
	SUBB    #13			;A/D baro-13
	BCS     LC47A			;Bra if underflow, baro zero'd out
;
	LDAA    #151			;Multiplier for baro val.
;
	MUL				;[143 x (A/D baro - 13)]/256     
	ADCA    #0			;Round if needed
;
;-Look up delta TPS AE mult. factor vs. baro
;
LC47A	STAA    L01EF			;Save it, baro val. for lookup
	LDAB    #96			;offset for lookup
	LDX     #L4B56			;TPS AE mult. factor vs. baro
	JSR     LF15A			;2D lookup with SBA
;
	STAA    L0207			;Save it, delta TPS AE baro factor

;
;-Look up PE baro TPS factor
;
	LDAA    L01EF			;Baro val. for lookup
	LDAB    #96			;offset for lookup
	LDX     #L4C7F			;PE baro TPS factor
	JSR     LF15A			;2D lookup with SBA
;
	STAA    L0206			;Save it 
;
;-Lookup PE RPM qual. threshold vs. Baro
;
	LDAA    L01EF			;Load baro
	LDAB    #96			;Offset for lookup
	LDX     #L496B			;PE RPM qual. vs baro table
	JSR     LF15A			;2D lookup with SBA
	STAA    L01F0			;Save it

;
;-Lookup alt. comp. value for MAP
;
	LDAA    L01EF			;Load baro for lookup
	LDAB    #96			;offset
	LDX     #L49C7			;Baro val vs. baro table
	JSR     LF15A			;2D lookup with SBA
;	
	STAA    L01F1			;Baro val for MAP/baro ratio calc
	RTS     			;Return
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; Filtered trans baro routine
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LC4B3	LDX	#L5B00			;Start addr. of diag. words
	LDY     #L5D02			;Opt. byte
	BRCLR   $30,X,#$04,LC4C4	;Err. word #6 mask, bra if b2==0, not recognising
					;err. 63, high baro press.
;
	BRCLR   L001B,#$04,LC4C4	;Err word #6, bra if b2==0, no err. 63 set
;
	BRA     LC4D1			;Bra if error flagged
;
;-Here if no high baro sens. error flagged, check MAP
;	
LC4C4	BRCLR   $2D,X,#$20,LC4D5	;Err. word #3 mask, bra if b5==0, err. 32, 
					;high MAP sens. not being recognised
;
	BRCLR	L0018,#$20,LC4D5	;Err. word #3, bra if b5==0, err. 32,
					;high MAP sens. not flagged
;
	BRSET   0,Y,#$04,LC4D5		;Bra if b2==1, bra if using baro/MAP
;
;-Here if baro sens./MAP high error set
;
LC4D1	LDD     $006B,X			;Load default baro
	BRA     LC503			;Bra to continue
;
;-Here if no error flagged, check for low MAP/baro
;											
LC4D5	BRCLR   $0030,X,#$02,LC4DF	;Err. word #6 mask, bra if b1==0, not recognising
					;err. 64, low baro
;
	BRCLR   L001B,#$02,LC4DF	;Err word #6, bra if b1==0, no err 64, 
					;low baro set
;
	BRA     LC4EC			;Bra if error flagged
;											
LC4DF	BRCLR   $2D,X,#$10,LC4F0	;Error word #3 mask, bra if b4==0, not recognising
					;err. 33, low MAP sens.

	BRCLR   L0018,#$10,LC4F0	;Error word #3, bra if b4==0. no err 33 flagged 
;
	BRSET   0,Y,#$04,LC4F0		;Bra if b2==1, bra if using baro/MAP
;
LC4EC	LDD     $6F,X			;Load default baro
	BRA     LC503			;Bra to save and return		
;
;~~~~~~~~~~~~~~~~~~~~~~~~~
; Filter baro if no error
;~~~~~~~~~~~~~~~~~~~~~~~~~
;
LC4F0	LDAA    L0316			;Load baro
	LDAB    L5D05			;Baro scalar
	MUL				;Baro x baro scalar
	ADDD    L5D06			;Add in bias  
	LDX     L00A6			;Load trans baro press
	LDY     #L5D08			;Load filter coeff.
	JSR     LF0D3			;Filter routine
;
;-Trans baro kPa = Trans baro value/328
;
LC503	STD     L00A6			;Stored trans. baro
	RTS				;Return     					
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;													
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; Routine to command the TCC to lock/unlock
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LC506	LDX     #L6727			;TCC option bytes											;
	LDD     #$00			;
	STD     L0088			;Preclear status word
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Check if disabling TCC when manual range selected
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;											
	BRCLR   0,X,#$08,LC51D		;Bra if b3==0, not disabling TCC when manual range 
					;selected
;
	BRCLR   L009C,#$03,LC51D	;Status word, bra if b0 and b1 ==0, not in manual
					;1st or 2nd
;
	BRSET   L0083,#$02,LC51D	;Status word, bra if b1==1, excessive trans temp, 
					;allow TCC to lock when too hot
;
	BSET    L0088,#$01		;Set b0, in manual range
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~	
; Check if disabling TCC when trans is cold
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;										
LC51D	BRCLR   0,X,#$20,LC530		;Bra if b5==0, not disabling TCC for low trans temps
;
	LDAA    4,X			;L672B, cool temp threshold for TCC to remain unlocked
	BRCLR   L0087,#$01,LC529	;Status word, bra if b0==0, PWM TCC not locked
;
	LDAA    5,X			;L672C, cool temp threshold for unlocking TCC
;
LC529	CMPA    L00AF			;Compare to trans temp
	BLS     LC530			;Bra if trans temp >
;
	BSET    L0088,#$02		;Set b1, trans temp too cool for TCC
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Check if disabling TCC when engine temp is cold
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;											
LC530	BRCLR   0,X,#$40,LC543		;Bra if b6==0, not disabling TCC for cool engine
;
	LDAA    2,X			;L6729, engine cool temp for TCC to remain unlocked
	BRCLR   L0087,#$01,LC53C	;Status word, bra if b0==0, PWM'd TCC not locked
;
	LDAA    3,X			;L672A, engine cool temp for unlocking TCC
; 
LC53C	CMPA    L00A3			;Compare to engine cool temp
	BLS     LC543			;Bra if cool temp is >
;
	BSET    L0088,#$04		;Set b2, engine cool temp too low for TCC
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~					
; Check if brakes have been applied
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;						
LC543	BRCLR   L0096,#$01,LC54A	;Status word, bra if b0==0, brakes not currently 
					;applied  
;
	BSET    L0088,#$08		;Set b3, brakes have been applied, TCC also 
					;automatically disabled at hardware level when brakes 
					;applied 
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Check if rapid deceleration is present
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LC54A	BRCLR   0,X,#$02,LC55D		;Bra if b1==0, dont disable TCC during hard decel
;
	LDAA    L0183			;Load rapid decel timer
	BEQ     LC55D			;Bra if ==0, no rapid decel present
;
;-Rapid decel present
;
	BRCLR   L0085,#$40,LC55A	;Status word, bra if b6==0, TCC not being applied
;
;-TCC currently being applied
;
	BSET    L0087,#$20		;Set b5, TCC disabled due to rapid decel
;
LC55A	BSET    L0088,#$10		;Set b4, rapid decel present
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Check if downshift is underway
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;										
LC55D	BRCLR   0,X,#$01,LC569		;Bra if b0==0, not disabling TCC during downshift
;
	LDAA    L0182			;TCC downshift disable timer
	BEQ     LC569			;Bra if ==0, no recent downshift
;
	BSET    L0088,#$20		;Set b5, transmission recently downshifted
;	
;~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Check if upshift underway
;~~~~~~~~~~~~~~~~~~~~~~~~~~~
;										
LC569	BRCLR   1,X,#$10,LC575		;L6728, bra if b4==0, not disabling TCC during
					;upshift
;
	LDAA    L0122			;TCC upshift disable timer
	BEQ     LC575			;Bra if ==0, no recent upshifts
;
	BSET    L0089,#$01		;Status word, set b0, transmission recently upshifted	
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Check if allowing TCC to unlock when A/C is requested
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;										
LC575	BRCLR   1,X,#$02,LC595		;L6728, bra if b1==0, not unlocking TCC when A/C
					;requested
;
	BRSET   L0083,#$02,LC595	;Status word, bra if b1==1, excessive trans temps 
					;present
;
	BRCLR   L0096,#$02,LC595	;Status word, bra if b1==0, A/C has not been requested

	LDAB    L00D3			;Load gear #
	BEQ     LC595			;Bra if ==0, in first gear
;
	ABX				;Add in gear # as offset
	LDAA    7,X			;Load in MPH threshold for TCC to unlock
	BRSET   L0087,#$01,LC58E	;Bra if ==1, PWM TCC locked
;
	LDAA    $0A,X			;Load MPH threshold for TCC to remain unlocked
;
LC58E	CMPA    L00D1			;Compare to MPH x 2
	BLS     LC595			;Bra if speed is >=
;
	BSET    L0088,#$40		;Set b6, unlock TCC while A/C is requested
;											
LC595	LDX     #L6727			;Index TCC params
	BRCLR   1,X,#$01,LC5A3		;L6728, bra if b0==0, not flagging excess neg. slip
;
	BRCLR   L0085,#$80,LC5A3	;status word, bra if b7==0, no excessive negative
					;torque converter slip 
;															 .. TCC APPLY)
	BSET    L0088,#$80		;Set b7, excessive negative torque converter slip
;	
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Check if disabling TCC to prevent repeat locking/unlocking
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;										
LC5A3	BRCLR   1,X,#$04,LC5B1		;L6728, bra if b2==0, not preventing repeated locking/
					;unlocking of TCC during low TPS
;
	BRCLR   L0087,#$08,LC5B1	;Status word, bra if b3==0, repeat unlock/relock 
					;prevention not in effect
;
;-Repeat unlock/relock prevention in effect
;
	BCLR    L0087,#$08		;Status word, reset repeated lock/unlocking prevention
					;flag
	BSET    L0089,#$02		;Status word, set b1, TCC disabled to prevent repeat
					;locking/unlocking
;
;-Check for appicable external errors if optioned
;											
LC5B1	BRCLR  1,X,#$04,LC5E2		;Bra if b2==0, not checking for errors applicable to
					;TCC
;
	JSR     LE434			;Routine to check for applicable external errors		
;		
	BRCLR   L0092,#$40,LC5D5	;Status word, bra if b6==0, no applicable errors set
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Errors have been set, bar TCC lockup if trans not overheating
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
	LDX     #L6735			;Index trans temp hyst. pair
	BRSET   L0085,#$04,LC5C4	;Status word, bra if b2==1, transmission overheating,
					;allow TCC lockup
;
	INX				;+1 advance to lower bound to allow TCC to lock up
					;when the transmission is too hot
;
LC5C4	LDAA    L00AF			;Load transmission temp
	CMPA    0,X			;Threshold to allow TCC lockup
	BHI     LC5D2			;Bra if trans temp is higher, allow TCC lockup
;
;-Errors present, no TCC lockup
;
	BCLR    L0085,#$04		;Clear b2, transmission not overheating
	BSET    L0089,#$04		;Set b2, errors applicable to the TCC have been 
					;set											;
	BRA     LC5D5			;Bra to continue
;
;-Transmission overheating, allow TCC lockup
;																					;
LC5D2	BSET    L0085,#$04		;Set b2, transmission overheating
;											
LC5D5	BRCLR   L0093,#$04,LC5E2	;Status word, bra if b2==0, output speed not lost
;
;-Output speed has been lost
;										
	LDAA    L00D3			;Load gear #
	CMPA    #$03			;4th gear
	BEQ     LC5E2			;Bra if in 3rd
;
;-In other gears, flag output speed loss
;
	BSET    L0089,#$08		;Status word, set b3, output speed lost
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Look up %DC for TCC to be locked
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LC5E2	LDX     #L68F9			;Addr. for max allowed PWM TCC DC vs. line pressure
	LDAA    L00DC			;Desired added line pressure
	ASLA				;x2 for lookup
	BCC     LC5EC			;Bra if no overflow
;
	LDAA    #$FF			;Clear overflow
;
LC5EC	JSR     LF15E			;2D lookup
;										
	TAB				;Get result into B as well
	STD     L0127			;Save it, DC for PWM TCC to be fully locked 
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Check if PWM TCC diag. in effect
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
	LDAA    L010A			;PWM TCC diag. DC
	BEQ     LC622			;Bra if ==0, no diags.
;
	CMPA    #200			;200%, threshold for diag. DC to command TCC to unlock
	BLS     LC606			;Bra if diag DC <=
;
;-Command TCC to unlock and reset
;
	BRCLR   L0086,#$10,LC603	;Status word, bra if b4==0, PWM TCC diag. override
					;not in effect
;
	BCLR    L0086,#$14		;Clear b4, PWM TCC diag. override flag, and b2, TCC
					;locked flag
;			
LC603	JMP     LC6CA			;Bra to command PWM TCC to unlock				
;																				
LC606	CMPA    #100			;100%, max allowed diag DC
	BLS     LC614			;Bra if <=, format for output to PWM TCC sol.
;
;-TCC DC > 100%, command TCC to lock and reset
;
	BRCLR   L0086,#$10,LC611	;Bra if b4==0, PWM TCC diag. override not in effect
;							
	BCLR    L0086,#$14		;Clear b4, PWM TCC diag. override flag, and b2, TCC
					;locked flag 
;										
LC611	JMP     LC6C2			;Bra to command PWM TCC to lock				
;
;-Format DC percentage for output to hardware
;											
LC614	LDAB    #0163			;Mult. for formatting diag. DC for hardware
	MUL				;(163 x diag. DC)
	ASLD				;x2
	ASLD				;x2, now x4
	STD     L0124			;Save it, PWM TCC DC
	BSET    L0086,#$14		;Set b4, PWM TCC diag. override in effect, and b2, TCC 
					;locked 
	JMP     LC6D0			;Bra to return				
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Check for high system voltage
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;											
LC622	BRCLR   L0093,#$08,LC62E	;Status word, bra if b3==0, no high system fault
;
;-High system voltage present, clear PWM TCC duty cycle
;
	CLRA				;
	CLRB				;
	STD     L0124			;Clear PWM TCC PW
	JMP     LC6BD			;Continue				
;									
LC62E	BRCLR   L0086,#$10,LC635	; 
;
	BCLR    L0086,#$14		; 
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~	
; Check quals to lock TCC in OD if output speed lost
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;				 
LC635	BRSET   L0088,#$08,LC64A	;Status word, bra if b3==1, brakes have been applied 
;
	BRSET   L0088,#$20,LC64A	;Status word, bra if b5==1, transmission has recently
					;downshifted
;
;-Quals met for lockup, check if output speed still present
;
	BRCLR   L0093,#$04,LC64A	;Status word, bra if b2==0, output speed not lost
;
;-Here if output speed has been lost
;
	LDAA    L00D3			;Load gear #
	CMPA    #$03			;4th gear
	BNE     LC64A			;Bra if not in 4th gear
;
;-In OD, lock TCC
;
	JMP     LC6C2			;Bra to lock TCC				
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~	
; Check MAP thresholds for TCC to lock
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 
;										
LC64A	LDD     L0088			;Load status words
	BNE     LC6BD			;Bra if !=0, TCC lockup is not permitted
;
	LDAA    L0184			;TCC rapid accel unlock timer
	BNE     LC6CA			;Bra if !=0, rapid accel present
;
	LDX     #L5B2B			;Index error masks
	BRCLR   L0018,#$30,LC65E	;Error word #3, bra if b4==0, no MAP low, and b5==0,
					;no MAP hi errors
;
	BRSET   2,X,#$30,LC66D		;Error mask #3, bra if b4, MAP low and b5, MAP high 
					;errors are optioned
;
;-No errors or not flagging errors here
;
LC65E	LDAB    L6753			;MAP threshold to allow TCC to remain locked
;
	BRSET   L0085,#$40,LC668	;Status word, bra if b6==1, TCC being applied 
;
	LDAB    L6754			;MAP threshold to allow TCC to lock
;
LC668	CMPB    L01E3			;Compare threshold to S/D MAP
	BHI     LC6CA			;Bra if MAP is <, unlock TCC
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Look up MPH thresholds to lock/unlock TCC
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LC66D	LDX     #LC11D			;Index of TCC MPH thresholds
;											 
	BRCLR   L0085,#$40,LC677	;Status word, bra if b6==0, TCC not being applied
;
;-TCC is currently being applied
;
	LDAB    #$0062			;Load offset
	ABX				;Add in offset to X
;
;-Here if TCC not being applied
;
LC677	PSHX				;Push addr. to stack
	BRCLR   L0096,#$10,LC68F	;Status word, bra if b4==0, kickdown pattern not
					;active
;	
;-Kickdown pattern is active
;										
	LDAB    #$31			;Load offset 
	ABX				;Add in offset to X
	LDAB    L00D3			;Load gear #
	JSR     LD90A			;Routine to look up indexed address based on 
					;various params
;	
	BCS     LC68F			;Bra if null address
;				
	PULY				;Clean up the stack
	LDAA    0,X			;Load MPH threshold
	STAA    L0126			;Save it, TCC MPH threshold										
	BRA     LC6A1			;Bra to continue				
;
;-Here if no kickdown pattern
;											
LC68F	PULX				;Get address back
	LDAB    L00D3			;Load gear #
	JSR     LD90A			;Routine to look up indexed address based on 
					;various params
;
	BCS     LC6BD			;Bra if no address specified
;
	LDAA    0,X			;Load MPH threshold 
	STAA    L0126			;Save it, TCC MPH threshold
;					
	LDAA    L00AC			;Load trans TPS
	JSR     LF15E			;2D lookup
;											
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Look up baro adjusted MPH thresholds
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LC6A1	LDX     #LC2A5			;INdex of baro gain for TCC MPH thresholds
;										
	BRCLR   L0096,#$10,LC6AB	;Bra if b4==0, no kickdown pattern active
;
;-Kickdown pattern active
;					
	LDAB    #$0031			;Load offset
	ABX				;Add it into the index
;    
LC6AB	CLRB				;Preclear B    
	JSR     LD934			;Routine to apply baro shift point gain
;
	CMPA    L0126			;Compare baro adjusted TCC MPH threshold to TCC MPH
					;threshold
	BCC     LC6B7			;Bra if baro adjusted threshold is greater
;
;-Baro adjusted threshold was less
;					;Load TCC MPH threshold 	
	LDAA    L0126			;
;
LC6B7	CMPA    L00D1			;Compare to MPH x 2
	BHI     LC6CA			;Bra if MPH <, unlock TCC
;
;-MPH threshold met to lock the TCC
;
	BRA     LC6C2			;Bra, lock TCC
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Here to immediatly unlock TCC
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LC6BD	BCLR    L0087,#$06		;Status word, clear b1, command TCC to lock, and
					;b2, command TCC to unlock
	BRA     LC6D0			;Bra to return
;
;~~~~~~~~~~~~~~~~~~
; Here to lock TCC
;~~~~~~~~~~~~~~~~~~
;
LC6C2	BCLR    L0087,#$02		;Status word, clear b1, command TCC to unlock
	BSET    L0087,#$04		;Status word, set b2, command TCC to lock
	BRA     LC6D0			;
;
;~~~~~~~~~~~~~~~~~~~~
; Here to unlock TCC
;~~~~~~~~~~~~~~~~~~~~
;
LC6CA	BSET    L0087,#$02		;Set b1, command TCC to unlock
	BCLR    L0087,#$04		;Clear b2, command TCC to lock
;
LC6D0	RTS				;Return     
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; Schedule of trans error routines
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LC6D1	JSR     LEC80			;Error 66,  3 -> 2 shift solenoid quad driver failure
;
	JSR     LECCD			;Routine to check for error 67, TCC solenoid failure
;
	JSR     LEAEE			;Routine to check for error 81, 2 -> 3 shift sol. 
					;failure
;
	JSR     LEB2F			;Routine to check for error 82, 1 -> 2 shift sol.
					;failure
;
	JSR     LEB70			;Routine to check for error 83, shift solenoid driver 
					;failure
;
	JSR     LE7EF			;Routine to check for error 52, extended high battery
					;voltage
;
	JSR     LE82B			;Routine to check for error 53, high battery voltage
;
	JSR     LEA28			;Routine to check for error 75, low system voltage
;
	JSR     LE8C6			;Not used...
;
	JSR     LE8C7			;Not used...
;
	JSR     LE8C8			;Routine to check for error 63, high baro pressure
;
	JSR     LE8FC			;Routine to check for error 64, baro low
;
	RTS				;Return     
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; Routine to look up linear trans temp
; 
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LC6F6	LDAA    L009F			;Transmission temp
	LDX     #LF8EF			;Index cool linear temp table
	BRSET   L0043,#$80,LC706	;
;
	BRSET   L004E,#$02,LC706	;Status word, bra if b1==1, use cool linear temp table
;
	LDX     #LF9EF			;Index warm linear temp table
;
LC706	TAB				;Get into B, use as offset for lookup     
	ABX				;Add in offsert 
	LDAA    0,X			;Load linear temp
;
;-Check if cool temps present
;
	CMPA    #120			;50 deg C, upper hyst. threshold
	BHI     LC717			;Bra if temp is > use warm linear temp table
;
	CMPA    #106			;40 deg C, lower hyst. threshold
	BHI     LC71A			;Bra if temp is >, dont change temp status
;
;-Cold trans temps present
;
	BSET    L004E,#$02		;Set b1, use cool linear table for trans temp lookup
	BRA     LC71A			;Bra to save and return
;
;-Here if warm trans temps present
;
LC717	BCLR    L004E,#$02		;Clear b1, flag to use cool linear temp table 
;
LC71A	STAA    L00AE			;Save linearized trans temp
	RTS				;Return     
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; Routine to handle trans related temp items
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LC71D	LDX     #L5B00			;Diag. masks.
	LDAA    L00AE			;Linearized trans temp
	BRCLR   $30,X,#$40,LC72A	;Diag. word #6 mask, bra if b6==0,
					;not flagging code 58, high trans temp
;
;-Check for error 58
;
	BRSET   L001B,#$40,LC732	;Err. word #6, bra if b6==1, high trans temp err.
;
LC72A	BRCLR   $30,X,#$20,LC734	;Bra if b5==0, not flagging code 59, low trans temp
;
;-Check for error 59
;
	BRCLR   L001B,#$20,LC734	;Bra if b5==0, no low trans temp err. present 
;
;-Load default here
;
LC732	LDAA    $66,X			;Default trans temp
;
LC734	LDAB    #128			;Load as LSB for unfiltered temp
	LDX     L00AF			;Transmission temp, filtered
	LDY     #L5D09			;Trans temp lag filter
	JSR     LF0D3			;Lag filter routine
;
	STD     L00AF			;Trans temp, filtered
;
;-Check trans temp and set status bits accordingly
;
	LDAB    L672E			;Threshold to clear hot transmission status bit
	BRSET   L0083,#$02,LC74B	;Trans status word, bra if hot trans flagged
;
	LDAB    L672D			;Threshold to set hot transmission status bit
;
LC74B	BCLR    L0083,#$02		;Preclear b1
;	
	CBA				;Compare trans temp to threshold
	BCS     LC754			;Bra if trans temp <
;
	BSET    L0083,#$02		;Set b1, hot transmission
;
;-Check 4th gear temp limits
;								
LC754	LDAB    L00A3			;Load engine cool temp for trans
	CMPB    L5D28			;Lower temp threshold to set status bit
	BCC     LC75F			;Bra if temp >
;
	BSET    L0083,#$01		;Set b0, disallow shift into OD
											
LC75F	CMPB    L5D29			;Lower temp threshold to clear status bit
	BLS     LC767			;Bra if cool temp is less
;
	BCLR    L0083,#$01		;Clear b0, allow shift into OD
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Generate trans cold shift term
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;											
LC767	CLRA				;
	CMPB    L5D2A			;Lower threshold for cold shift
	BHI     LC770			;Bra if engine cool temp >
;
	COMA				;255
	BRA     LC787			;Bra to save and return
;
LC770	CMPB    L5D2B			;Upper threshold for cold shifr
	BCC     LC787			;Bra if cool temp >
;
;-Between boundries here
;
; Cold shift temp term = 255 - (engine temp - lower temp thresh)/(upper - lower thresh)
;
	LDAA    L5D2B			;Upper cool temp threshold
	SUBA    L5D2A			;Subtract lower threshold
	CLRB				;Clear LSB
	XGDX				;Swap result into X
	LDAA    L00A3			;Load engine cool for transmission
	SUBA    L5D2A			;Subtract lower threshold
	CLRB				;Clear LSB
	FDIV				;Fract divide
	XGDX				;Get result into D
	COMA				;Ones comp.
;
;-Here if not between bounds, 255 for cold engine, 0 for warm engine
;
LC787	STAA    L00A5			;Save it, cold trans shift temp term
	RTS				;Return
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~   
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; Lookup linear cool temp routine
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LC78A	LDAA    L082D			;Load A/D cool temp
	LDX     #LF8EF			;Addr. of 'cool' linear temp table
	BRSET   L0043,#$80,LC79B	;Bra if b7==1
;
	BRSET   L004E,#$01,LC79B	;Bra if b0==1, CTS <= 40 deg C
;
	LDX     #LF9EF			;Addr. of 'warm' linear temp table table
;
LC79B	TAB				;Get A/D cool temp into B     
	ABX				;Add in offset to X    
	LDAA    0,X			;Load linear cool temp val. 
	STAA    L00A2			;Store it, linear cool temp
;
;-Figure out the crossing points of the two tables
;
	CMPA    #120			;Compare to 50 deg C, upper hyst. threshold 
	BHI     LC7AE			;Bra if cool temp >

	CMPA    #106			;Compare to 40 deg C, lower hyst. threshold
	BHI     LC7B1			;Bra if cool temp >
;
;-Here if cool temp <= 40 deg C
;
	BSET    L004E,#$01		;Set b0 of L004E, cool temp <= 40 deg C
	BRA     LC7B1
;
;-Here if cool temp > 50 deg C
;
LC7AE	BCLR    L004E,#$01		;Clear b0 of L004E, cool temp > 50 deg C
;
LC7B1	BRCLR   L0016,#$60,LC7C6	;Bra if b5 and b6==0, no CTS errors flagged
;
;-If error flagged, use default CTS
;
	LDAA    L5B1A			;63.5 deg C, high defualt cool temp
	LDX     L00F8			;Load engine run time
	CPX     L5B18			;Runtime threshold to use high defualt CTS 
	BHI     LC7C2			;Bra if >, use high defualt cool temp
;
;-Engine still 'cold', use low default CTS val
;
	LDAA    L5B1B			;20 deg C, low default CTS
;
LC7C2   STAA    L0006			;Save it, cool temp, linear
	BRA     LC7CA			;
;
;-Here if no CTS error
;
LC7C6   LDAA    L00A2			;Linear cool temp			
	STAA    L0006			;Save it, cool temp, linear
;
LC7CA   BCLR    L0043,#$80		;Clear b7
	LDAA    L0006			;Cool temp, linear
;
;-Check lower bound on cool temp
;
	SUBA    #16			;subtract 12 deg C
	BCC     LC7D6			;Bra if cool temp >=
;
	CLRA				;Clear cool temp    
	BRA     LC7DC			;Bra to save
;
LC7D6	CMPA    #192			;114 deg C
	BLS     LC7DC			;Bra if cool temp < 
;
	LDAA    #192			;114 deg C, upper bound on cool temp
;
LC7DC	STAA    L02A6			;Save it, range limited cool temp
	RTS     			;Return
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; Routine to generate cool temp for trans.
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LC7E0	LDX     #L5B00			;Start addr. of diag. masks.
	LDAA    L00A2			;Linear cool temp
	BRCLR   $2B,X,#$40,LC7ED	;Err word #1 mask, bra if b6==0, not
					;flagging high cool temp
;
;-Check for error 14, high cool temp
;
	BRSET   L0016,#$40,LC7F5	;Error word #1, bra if b6==1, high cool temp

LC7ED	BRCLR   $2B,X,#$20,LC7F7	;Bra if b5==0, not flagging low cool temp
;
;-Check for error 15, low cool temp
;
	BRCLR   L0016,#$20,LC7F7	;Bra if b5==0, no low cool temp flagged
;
LC7F5	LDAA    $17,X			;Load default cool temp
;
LC7F7	LDAB    #128			;Load val for LSB
	LDX     L00A3			;Engine cool for transmission
	LDY     #L5D0A			;Filter coeff. for engine coolant, transmission
	JSR     LF0D3			;Lag filter routine
;
	STD     L00A3			;Save it
	RTS				;Return  
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~   					
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; Lookup routine of various temp. based items
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;-Look up AE TPS filter coeff.
;
LC805	LDAA    L02A6			;Range limited cool temp
	LDX	#L4B5A			;delta TPS AE cool temp correction factor vs temp
	JSR     LF15E			;2D lookup 
	STAA    L0208			;Store it, delta TPS AE cool temp corr. factor
;
;-Look up min decel fuel cut-off rpm 
;
	LDAA    L02A6			;Range limited cool temp
	LSRA 				;/2 for lookup
	LDX     #L4C83			;DFCO rpm vs cool temp
	JSR     LF15E 			;2D lookup
	STAA    L0266			;Save it, min DFCO rpm
;
;-Look up min idle open loop AFR 
;
	LDX     #L4BA3			;Min idle open loop AFR vs cool temp
	LDAA    L0006			;Cool temp, linear
	CMPA    #192			;104 deg C, upper limit for lookup
	BLS     LC829			;Bra if cool temp is less 
;
	LDAA    #192			;Load max
;	
LC829	JSR     LF15E			;2D lookup											;
	BRSET   L003F,#$40,LC83E	;Bra if b6==1 

	LDAB    L0006			;Cool temp, linear
	CMPB    L48D9 			;Temp threshold for open loop idle AFR bias 
	BHI     LC83E			;Bra if CTS higher 
;
	SUBA    L48DE			;1.6:1 AFR ratio bias for open loop idle
	BCC     LC83E			;Bra if no underflow
;										
	LDAA    #0			;Clear underflow
;
LC83E	STAA    L0295			;Save it, open loop idle AFR
;
;-Look up slo filtered MAP filter coeff. 
;
	LDX     #L4B32			;MAP filter coeff. vs cool temp table
	LDAA    L02A6			;Range limited cool temp
	JSR     LF15E 			;2D lookup
;
	STAA    L01EE			;Save it, slo filtered MAP filter coeff	
;
;-Look up desired egr correction factor
;
	LDX     #L4755			;EGR correction factor vs. cool temp
	LDAA    L0006			;Cool temp, linear
	LDAB    #80			;offset
	JSR     LF137			;2D lookup with UL
;
	STAA    L01DE			;Save it, EGR correction factor
;
;-Scale and offset cool temp for lookup
;
	LDAA    L0006			;Cool temp, linear
	SUBA    #153			;Sub out 75 deg C
	BCC     LC862			;Bra if temp >=
;
	LDAA    #0			;
;				
LC862	ASLA				;x2
	ASLA 				;x2
	BCC     LC868			;Bra if no overflow
;
	LDAA    #255			;Clear overflow
;
;-Look up burst knock retard
;
LC868	LDAB    #153			;
	MUL				;[4 x 153 x (cool temp - 153)]/256
	ADCA    #0			;Round if needed
	LDX     #L46D9			;Burst knock retard table
	JSR     LF136			;2D lookup with UL
;
	STAA    L0842			;Save it, burst knock retard
	JMP     LC922			;Jump to return
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; Routine to do SA and Knock related look ups
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;-See if MAT present
;
LC879	CLRA				;Preclear A			
	LDX     #L400B			;Opt. byte
	BRCLR	0,X,#$40,LC8BF		;Bra if b6==0, not using MAT sensor
;
;-Check upper bound
;
	LDAA    L0252			;Load MAT
	CMPA    #192			;104 deg C
	BCS     LC88C			;Bra if MAT less

	LDAA    #192			;Load max
	BRA     LC893			;Bra past min temp check				
;
;-Check lower bound
;											
LC88C	CMPA    #80			;19 deg C
	BCC     LC893			;Bra if MAT >=
;
	CLRA				;Clear MAT
;
	BRA     LC8BF			;Bra, continue without corrections				
;
;-Look up MAT SA correction, based on Vac and MAT
;
LC893	LDX     #L442F			;Base MAT SA corr. INDEX BASE MAT SPK ADV CORR
	LDAB    0,X			;Load flag
	BEQ     LC8A4			;Bra if ==0. use vac
;
;-Use RPM
;
	LDAB    L0062			;RPM/25
	CMPB    #176			;4400 RPM
	BLS     LC8AD			;Bra if RPMs are <=
;
	LDAB    #176			;Load Max

	BRA     LC8AD			;Bra to continue
;
;-Using vacuum
;
LC8A4	LDAB    L01EC			;Load vacuum	
	SUBB    #80			;55 kPa offset
	BCC     LC8AD			;Bra if vacuum >

	LDAB    #$0000			;Clear Vac
;
LC8AD	INX				;+1, advance to table
	JSR     LF17B			;Bra to do 3D lookup
;										
	SUBA    L4140			;Spark bias for MAT SA correction
	BMI     LC8BB			;Bra if SA negative
;
;-SA corr >= 0
;
	BCLR    L0041,#$02		;Clear b1, MAT SA correction is positive 
	BRA     LC8BF			;Bra to save
;
;-SA corr < 0
;

LC8BB	BSET    L0041,#$02		;Set b1, MAT AS correction is negaive
	NEGA				;Flip sign 
;   
LC8BF	STAA    L021D			;Save it, MAT SA correction
	CLRA 				;Preclear A
	LDAB    L0006			;Cool temp, linear
	CMPB    L45D0			;75 deg C, min cool temp for spark retard
	BLS     LC8E0			;Bra if cool temp is less
;
;-Look up PE spark SA corr. 
;
	BRSET   L0019,#$10,LC8E0	;Error word #4, bra if b4==1, knock circuit	
;
	BRCLR   L003D,#$20,LC8E0	;Status word, bra if b5=0, not in PE
;
;-In PE here
;
	LDX     #L44B5			;PE SA vs rpm table
	LDAA    L0062			;RPM/25
	JSR     LF15E			;2D look up
;
	LDAB    L029B			;PE AFR ramping multiplier
	MUL				;PE SA x ramping multiplier			     
	ADCA    #00			;Round if needed
;
;-Store PE SA
;
LC8E0     STAA    L021F			;PE SA adder
;
;-Look up low octane mult.
;
	LDAA    L0062			;RPM/25
	LSRA				;/2 for lookup
	LDX     #L45B4			;Low octane mult. vs RPM
	JSR     LF15E			;2D lookup
;
	STAA    L0841			;Save it, low octane RPM mult.
;
;-Look up knock attack rate
;

	LDX     #L458A			;Knock attack rate vs. RPM
	LDAA    L0062			;RPM/25
	JSR     LF15E			;2D lookup 
;
	STAA    L0843			;Store it, knock attack rate
;
;-Look up max PE knock retard
;
	BRCLR   L003D,#$20,LC90B	;Status word, bra if b5==0, not in PE
;
	LDX     #L457B			;Max PE knock retard vs. RPM
	LDAA    L0062			;RPM/25
	LSRA 				;/2 for lookup   
	LDAB    #16			;800 rpm offset 
	JSR     LF15A			;2D look up with SBA
	BRA     LC91F			;Bra to store
;
;-Look up max non-PE knock retard
;

LC90B	LDAA	#255			;Zero vacuum, use if MAP error
	LDAB    L006F			;Status word
	BITB    #$48			;Test b6=High MAP err, b3=Low MAP err
	BNE     LC916			;Bra if error set
;
	LDAA    L01EC			;Load vac
;	
LC916	LSRA				;/2 for lookup    
	LDAB    #32			;offset, 20 kPa of vac
	LDX     #L4583			;Max non-PE retard vs. vac
	JSR     LF15A			;2D look up with SBA
;
LC91F	STAA    L0844			;Store it, max SA retard
;
LC922	RTS				;Return   
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~		
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; Adaptive A/C control routine
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;-Do ALDL
;
LC923	SEI				;Disable interrupts     
	BCLR    L0050,#$18		;Status word, clear b3=ALDL mode, b4=diags. mode
;
	LDAA    L082F			;ALDL volts
	CMPA    #40			;781 mV
	BCS     LC93B			;Bra if ALDL less, in diags. mode
;
	CMPA    #100			;2 V
	BCC     LC93E			;Bra if ALDL volts greater, ALDL open
;
	BRCLR   L0044,#$01,LC93E	;Bra if b0==0, factory test requested
;
;-ALDL volts > 2 volts, in ALDL mode
;
	BSET    L0050,#$08 		;Set b3, ALDL mode
	BRA     LC93E			;Bra to continue
;
;-In diags. mode
;
LC93B     BSET    L0050,#$10		;Set b4, in diags. mode
;
LC93E     CLI				;Re-enable interrupts
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Check option byte for A/C control method
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
	LDX     #L400F			;Option byte
	BRSET   0,X,#$04,LC949		;Bra if adaptive A/C compressor control selected
;
;-Here if no adaptive A/C control
;
	JMP     LCA42			;Bra to return				
;
;-Here if adaptive A/C control
;
LC949	LDD     L00F8			;Engine run time
	CPD     L4022			;Min time for A/C to be enabled
	BCS     LC955			;Bra if <, not time for A/C yet
;
	BRSET   L004D,#$01,LC963	;I/O port C word, bra if b0==1, A/C requested 
;  
;-No A/C requested here
;
LC955	LDAA    L4012			;.1 sec. delay 
	STAA    L0830			;Save it, min time to enable A/C timer
;	
;-Here if one or more limits exceeded for A/C
;										
LC95B	LDAA    L4013			;A/C re-enable delay timer
	STAA    L0831			;Save it, A/C renable delay timer
	BRA     LC994			;Bra to continue				
;
;-A/C requested here
;
LC963	LDAA    L0830			;.1 sec delay to activate A/C
	BEQ     LC96D			;Bra if ==0
;											
	DEC     L0830			;-1,
	BRA     LC994			;Bra
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~
; Check TPS limits for A/C
;~~~~~~~~~~~~~~~~~~~~~~~~~~
;										
LC96D	LDAA    L4011			;Upper %TPS threshold for A/C to remain on
	BRSET   L0041,#$10,LC977	;Bra if A/C active
;
;-A/C off here
;
	LDAA    L4010			;Upper %TPS threshold for A/C to come on
;
;-A/C on here
;
LC977	CMPA    L01FC			;%TPS
	BCS     LC988			;Bra if TPS is greater
;
;-Here if TPS doesnt exceed limit
;
	LDX     #$0000			;
	STX     L0834			;Reset High TPS A/C disable timer
;
LC982	BRCLR   L0036,#$10,LC9C8	;Status word, bra if b4==0, no stall saver, dont disable A/C
;
;-Stall saver in effect, disable A/C
;
	BRA     LC9C3			;Bra to disable the A/C				
;
;-TPS exceeds limits
;										
LC988	LDX     L0834			;High TPS disable timer
	CPX     L4020			;Threshold to re-enable A/C
	BHI     LC982			;Bra if greater,  re-enable A/C if rpms where within limits
;
	INX				;+1
;
	STX     L0834			;Save it
LC994	BRCLR   L004F,#$80,LC9C0	;Status word, bra if b7==0, engine off
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Check RPM limits if high tps present
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
	LDAA    L0062			;RPM/25
	CMPA    L4019			;Upper RPM limit for A/C to remain on
	BLS     LC9AD			;Bra if less
;
;-RPMs exceeded threshold for A/C to remain on
;
	BSET    L0052,#$10		;Status word, set b4, A/C disabled due to high rpm

	CLR     L0833			;Clear high rpm A/C re-enable delay timer
	LDAA    L4013			;A/C re-enable timer val
	STAA    L0831			;Save it
	BRA     LC9C0			;Bra to disable A/C
;
;-Here if rpms less then threshold
;
LC9AD	LDAA    L0833			;High RPM A/C re-enable delay timer
	CMPA    L4016			;Threshold to re-enable A/C
	BCS     LC9BA			;Bra if less
;
	BCLR    L0052,#$10		;Status word, clear b4, A/C disabled due to high rpms
	BRA     LC9C0			;Bra 
;
LC9BA	INCA				;+1    
	BEQ     LC9C0			;Bra if roll-over

	STAA    L0833			;Save it
;
LC9C0	BCLR    L0052,#$20		;CLR b5, A/C not ok'd
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Here to disable the A/C and return
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LC9C3	BCLR    L0041,#$10		;Clear b4, disable A/C
	BRA     LCA42			;Bra to return
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Here if not disabling the A/C, check cool temp
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LC9C8	LDAA    L401B			;Upper cool temp thresh for A/C to remain on
	BRSET   L0041,#$10,LC9D2	;Bra if A/C on
;
	LDAA    L401A			;Upper CTS threshold for A/C to come on
;
LC9D2	CMPA    L0006			;Cool temp, linear
	BCS     LC95B			;Bra if cool temp greater
;
;~~~~~~~~~~~~~~~~~~~~~~~
; Check for launch mode
;~~~~~~~~~~~~~~~~~~~~~~~
;
	LDAA    L401C			;Upper mph threshold if A/C on
	BRSET   L0041,#$10,LC9E0	;Status word, bra if b4==1, A/C on
;
	LDAA    L401D			;Upper mph threshold if A/C off
;
LC9E0	BRSET   L0016,#$10,LCA0A	;Error word #1, bra if b4==1, error 16, VSS
;
	CMPA    L02A7			;filtered MPH
	BLS     LCA0A			;Bra if speed <=, continue
;
;-MPH higher then threshold here
;
	LDAA    L401E			;Upper tps threshold for A/C to come on
	BRCLR   L0041,#$10,LC9F9	;Status word, bra if b4==0, A/C off
;
;-A/C on here
;
	LDAA    L401F			;Upper tps threshold for A/C to remain on
	LDAB    L4014			;timer val
	STAB    L0836			;Launch mode A/C re-enable delay timer
;
;-A/C off here
;
LC9F9	CMPA    L01FC			;Compare to TPS
	BCC     LCA0A			;Bra if TPS <, continue
;
	LDAA    L0836			;A/C launch disable timer		
	BEQ     LCA0A			;Bra if ==0, continue
;
	DECA 				;-1   
	STAA    L0836			;Save it, A/C launch mode disable timer
	JMP     LC95B			;Bra to check rpm limits and exit
;
;~~~~~~~~~~~~~~~~~~~~~~
; Check max rpm limits
;~~~~~~~~~~~~~~~~~~~~~~
;
LCA0A	LDAA    L4017			;Upper rpm threshold for A/C to come on
	BRSET   L0041,#$10,LCA14	;Status word, bra if b4==1, A/C on
;
	LDAA    L4018			;Upper rpm threshold for A/C to remain on
;
LCA14	CMPA    L0062			;RPM/25
	BCS     LCA1D			;Bra if rpms > threshold
;
;-RPMs less then threshold here
;
	CLR     L0832			;Clear max rpm limit A/C disable timer
	BRA     LCA2B			;Bra to continue
;
;-RPMs higher or same as threshold here 
;
LCA1D	LDAA    L0832			;High rpm A/C disable timer
	CMPA    L4015			;Threshold to disable A/C
	BCC     LCA2F			;Bra if timer is >=
;
	INCA				;+1   
	BEQ     LCA2B			;Bra if roll-over
;
	STAA    L0832			;Save timer
;
LCA2B	BRCLR   L0052,#$10,LCA32	;Status word, bra if b4==0, A/C not disabled due to high rpms
;
LCA2F	JMP     LC994			;Bra to check rpm limits for A/C
;
LCA32	BSET    L0052,#$20		;Status word, set b5, A/C ok'd
;
	LDAA    L0831			;A/C re-enable delay timer
	BEQ     LCA3F			;Bra if ==0, activate A/C
;
	DEC     L0831			;-1
	BRA     LCA42			;Bra to return
;
LCA3F	BSET    L0041,#$10		;Status word, set b4, A/C on
;
LCA42	RTS				;Return
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;~~~~~~~~~~~~~~~~~~~~~~~~~
;
; Hardware config routine
;
;~~~~~~~~~~~~~~~~~~~~~~~~~
;
LCA43	BRSET   L004F,#$80,LCA56	;Status word, bra if b7==1, engine running
;
	LDAA    L0006			;Cool temp, linearized
	STAA    L02A5			;Initial cool temp (engine off)
	CLRB				;Pre-clear B   
	LDAA    L48B4			;Intitial value for O2
	STD     L01F3			;Save it, flitered O2
	STAA    L01F8			;Save it, A/D O2
;
LCA56	JMP     LCA59			;[likely work-around]
;
LCA59	LDAA    L03B2			;End of stack
	BEQ     LCA61			;Bra if ==0
;
	BSET    L003A,#$10		;Status word, set b4, stack overflow
;
LCA61	LDX     #L3000			;Base CPU addr.
	BSET    $22,X,#$A0		;TMSK1, set b5, enable 
	SEI				;Disable interrupts     
	LDD     L3FFC           	;I/O port
	ANDB    #$FE			;Clear b0
;
	LDY     #L400B			;Opt byte
	BRSET   0,Y,#$01,LCA7C		;Bra if b0==1, in CPI/PFI mode
;
;-In TBI mode
;
	ORAB    #$32			;
	ORAA    #$F9			;Set 1111 1001 0011 0010
	BRA     LCA80			;Bra to continue
;
;-In CPI/PFI mode
;
LCA7C	ORAB    #$12			;
	ORAA    #$79			;Set 0111 1001 0001 0010
;
LCA80	JSR     LF08A			;Delay
;
	STD     L3FFC			;Save it, I/O port
;
	CLI				;Enable interrupts     
	BCLR    L004C,#$80		;I/O port status word, clear b7
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Entry point for MPU Register config 
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LCA8A	CLRA				;Preclear A    
	STAA    $0C,X			;OC1 action mask Reg.
	STAA    $0D,X			;OC1 action data Reg.	
	STAA    $2C,X			;SCI control Reg. 1
	STAA    $0B,X			;Compare Force Reg.
	STAA    $28,X			;SPI control reg
					; b7=SPI interrupt enable
					; b6=SPI system enable
					; b5=Port D wired OR mode select
					; b4=Master/Slave mode select
					; b3=Clock polarity select
					; b2=Clock phase select
					; b1=SPI bit rate select
					; b0=SPI bit rate select
;
	STAA    $3E,X			;Factory Test Control Reg.
	STAA    $5C,X			;
	STAA    $03,X			;PRTC (I/O Port C)
	LDAA    #$0026			;0010 0110
	STAA    $0021,X			;TCTL2 (Timer Control Reg. 2)
					; b7= -
					; b6= -
					; b5=EDG1B (Input caputure Edge Control Bit)
					; b4=EDG1A (Input caputure Edge Control Bit)
					; b3=EDG2B (Input caputure Edge Control Bit)
					; b2=EDG2A (Input caputure Edge Control Bit)
					; b1=EDG3B (Input caputure Edge Control Bit)
					; b0=EDG3A (Input caputure Edge Control Bit)		
;
	LDAA    #$0040			;0100 0000
	STAA    $0026,X			;PACTL (Pulse accumulator control) Reg.
					; b7=DDRA7 (Data Direction Control for Port A)
					; b6=PAEN (Pulse accumulator Enable)
					; b5=PAMOD (Pulse Accumulator Mode Select)
					; b4=PEDGE (Pulse Accumulator Edge Select)
					; b3= -
					; b2= -
					; b1=RTR1 (Real Time Interrupt Rate select)
					; b0=RTR0 (Real Time Interrupt Rate select)
;
	LDAA    #$0038			;0011 1000		
	STAA    1,X			;
	LDAA    #$0038			;0011 1000
	STAA    9,X			;DDRD (Data Direction for Port D)
					; b7= -
					; b6= -
					; b5=DDRD5 (Data Direction Control for Port D b5)
					; b4=DDRD4 (Data Direction Control for Port D b4)
					; b3=DDRD3 (Data Direction Control for Port D b3)
					; b2=DDRD2 (Data Direction Control for Port D b2)
					; b1=DDRD1 (Data Direction Control for Port D b1)
					; b0=DDRD0 (Data Direction Control for Port D b0)
;
	LDAA    #$04			;0000 0100
	STAA    $2B,X			;BAUD (SCI baud rate control)
					; b7=Clear baudrate timing chain
					; b6= -
					; b5=SCI baudrate prescale select
					; b4=SCI baudrate prescale select
					; b3=SCI baudrate clock test
					; b2=SCI baudrate select
					; b1=SCI baudrate select
					; b0=SCI baudrate select
;
	LDAA    #$15			;0001 0101
	STAA    $3C,X			;HYPI0 (Highest Priority I Bit and Misc.)
					; b7=Read bootstrap ROM
					; b6=Special mode
					; b5=Mode A select
					; b4=Interval read visibility
					; b3=Priority select
					; b2=Priority select
					; b1=Priority select
					; b0=Priority select
;
	LDAA    #$AC			;1010 1100		
	STAA    $5D,X			;
	LDAA    #$CB			;1100 1011
	STAA    $5F,X			;
	LDAA    #$00			;0000 0000
	STAA    $5E,X			;
	LDAA    #$90			;
	STAA    L3061			;
	LDAA    #$FF			;
	STAA    L3063			;
	LDAA    #$00			;
	STAA    L3065			;
	RTS   				;Return  
;
;~~~~~~~~~~~~~~~~~~~~~~~~~
;
; Output handling routine
;
;~~~~~~~~~~~~~~~~~~~~~~~~~
;
;-Filter O2 for RTS loops
;
LCACF	LDX     L01F3			;Filtered O2 
	LDAB    L4E7F			;Load filter coeff.
	LDAA    L01F8			;A/D O2 volts
	JSR     LF0F6			;Filter routine
;
	STD     L01F3			;Save it, filtered O2
	LDX     #L400B			;Opt. byte
	BRSET   0,X,#$02,LCAE8		;Bra if b1==1, perform AIR injection handling
;
	JMP     LCB3A			;Skip AIR			
;
;-Check for ALDL 
;
LCAE8	CLRB				;Preclear B    
;
;~~~~~~~~~~~~~~~~~~~~~~~~~
; Check if in ALDL mode 4
;~~~~~~~~~~~~~~~~~~~~~~~~~
;
	BRCLR   L003B,#$10,LCAFE	;Status word, bra if b4==1
;
	LDX     #L0390+1		;ALDL word, descrete enables
	BRCLR   0,X,#$02,LCAFE		;Bra if b1==0, no AIR diag.
;
	LDX     #L0392			;ALDL word, descrete states
	BRSET   0,X,#$02,LCB2F		;Bra if b1==1, enable AIR system
;
	JMP     LCB34			;Bra to disable AIR
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Here if normal operation, direct air to manifolds if needed
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LCAFE	LDAA    L0006			;Cool temp, linearized
	CMPA    L4E82			;Min temp for AIR
	BCS     LCB34			;Bra if temp <=, no AIR
;
	BRSET   L0071,#$01,LCB34	;Status word, bra if b0==1, SES commanded to be on
;
;-Check for PE
;											
	BRCLR   L003D,#$20,LCB18	;Status word, bra if b5==0, no PE
;
;-In PE, check if in PE long enough to disallow AIR
;
	LDAB    L029C			;Time in PE
	CMPB    L4E83			;Threshold to disable AIR
	BHI     LCB34			;Bra if >, disable air
;
	INCB				;+1
	BRA     LCB2F			;Bra to continue
;
;-PE not active here, check MAP/RPM thresholds for AIR
;
LCB18	LDAA    L0061			;Scaled RPMs
	CMPA    L4E80			;RPM threshold to disable AIR
	BHI     LCB34			;Bra if RPMs greater, disable AIR
;											
	LDAA    L01E9			;A/D MAP
	CMPA    L4E81			;MAP threshold to disable AIR
	BCS     LCB34			;Bra if MAP <, disable air
;
;-Check if in open loop idle, enable AIR
;
	BRSET   L0050,#$02,LCB2F	;Status word, bra if b1==1, open loop idle enabled
;
;-Check if closed loop enabled
;
	BRSET   L003E,#$80,LCB34	;Status word, bra if b7==1, in closed loop, no AIR
;
;~~~~~~~~~~~~~~~~~~~~
; Here to enable AIR
;~~~~~~~~~~~~~~~~~~~~
;											
LCB2F	BSET    L0046,#$04		;Status word, set b2, enable AIR
	BRA     LCB37			;Bra to continue
;
;~~~~~~~~~~~~~~~~~~~~~
; Here to disable air
;~~~~~~~~~~~~~~~~~~~~~
;
LCB34	BCLR    L0046,#$04		;Clear b2, disable AIR
;
LCB37	STAB    L029C			;Save time in PE
;
LCB3A	BRA     LCB3C			;Bra to continue
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Check if excessive batt. volts
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LCB3C	LDAA    L00A1			;Batt volts
	CMPA    #171			;17.1 volts, threshold for high batt. volts
	BCS     LCB4B			;Bra if batt volts less
;
;-High batt voltage present
;
	BRSET   L0051,#$10,LCB66	;Status word, bra if b4==1, high batt volts already
					;flagged
;
	BSET    L0051,#$10		;Set b1, high batt volts present
	BRA     LCB4E			;Bra to continue
;
;-Here if batt volts within limits
;
LCB4B	BCLR    L0051,#$10		;Clear b4, batt volts within limits

LCB4E	BRSET   L004F,#$80,LCB75	;Status word, bra if b7==1, engine running
;
;-Engine off here
;
	BRCLR   L0071,#$80,LCB66	;Status word, bra if b7==0, diags. not in effect
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Diagnostics in effect, enable outputs
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
	LDAA    #255			;Load max CCP DC
	STAA    L01F9			;Save it, CCP DC
	BSET    L0046,#$04		;Status word, set b2, enable AIR
	BSET    L0041,#$10		;Status word, set b4, A/C active
	BSET    L0052,#$08		;Status word, set b3, enable CPI tuning valve
	BRA     LCB75			;Bra to continue			
;
;-No diags. and engine off, disable outputs
;
LCB66	CLR     L01F9			;Clear CCP DC
	BCLR    L0046,#$04		;Clear b2, disable AIR
	BCLR    L0053,#$04		;Clear b2, turn off shift light
	BCLR    L0041,#$10		;Clear b4, disable A/C
	BCLR    L0052,#$08		;Cleear b3, disable tuning valve
;
;~~~~~~~~~~~~~~~~~~~~~
; Engine running here
;~~~~~~~~~~~~~~~~~~~~~
;
LCB75	LDD     #$DFFF			;
	BRSET   L0046,#$04,LCB7F	;Status word, bra if b2==1, AIR enabled
;											
	LDD     #$D000			;
;
LCB7F	STD     L3FDA			;Output to AIR bypass solenoid
;
;~~~~~~~~~~~~~~~~~~~~
; Check if in mode 4
;~~~~~~~~~~~~~~~~~~~~
;
	BRCLR   L003B,#$10,LCB92	;Status word, bra if b4==0, not in mode 4
;
;-In mode 4
;
	LDX     #L0395			;
	BRCLR   0,X,#$04,LCB92		;
;
	LDAB    L0396			;
	BRA     LCB95			;
;
;~~~~~~~~~~~~~~~~~~~~~~~
; Here to output CCP DC
;~~~~~~~~~~~~~~~~~~~~~~~
;
LCB92	LDAB    L01F9			;Load CCP DC
;
LCB95	CLRA				;Preclear A    
	ASLD				;x2    
	ASLD				;x2    
	ASLD				;x2    
	ORAA    #$B0			;
	STD     L3FD6			;Save it, output to CCP 
;
	LDX     #$0395			;
	BRCLR   0,X,#$38,LCBB2		;
;
	CLRA				;    
	LDAB    L0396			;
	ASLD				;    
	ASLD				;    
	ASLD				;    
	ASLD				;    
	ORAA    #$0070			;
	XGDX				;    
	BRA     LCBD9			;
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Here to output overspeed light/ A/C clutch cont'l / CPI variable tuning valve
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LCBB2	LDX     #$7FFF			;Preload full DC for output
;
;-Check which opt
;
	LDY     #L400F			;Opt. byte
	BRCLR   0,Y,#$01,LCBC7		;Bra if b0==0, governor not optioned
;
;-Governor in use, output to overspeed light
;
	LDAA    L0062			;RPM /25
	CMPA    L50D2			;RPM threshold to enable overspeed light
	BCC     LCBD9			;Bra if RPMs >, enable light 
;
	BRA     LCBD6			;RPMs < threshold, no overspeed light
;
;-Check if CPI variable tuning valve enabled
;
LCBC7	BRSET   L0052,#$08,LCBD9  	;Status word, bra if b3==1, tuning valve enabled
;
;-Check if A/C clutch control enabled
;
	LDAA    L400F			;Opt. byte
	BITA    #$04			;Test b2
	BEQ     LCBD6			;Bra if ==0, CPI tuning valve in use instead 
;
;-Controlling A/C clutch
;
	BRCLR   L0041,#$10,LCBD9	;Status word, bra if b4==0, A/C not enabled
;
;-Here to disable output
;							
LCBD6	LDX     #$7000			;Load min DC
;
;-Here to enable output
;
LCBD9	STX     L3FD4			;Save to output A/C / overspeed lgt / tuning valve
;
;~~~~~~~~~~~~~~~~~~~~~~
; Output to S/L or TCC
;~~~~~~~~~~~~~~~~~~~~~~
;
	LDX     #L400E			;Opt. byte
	BRCLR   0,X,#$04,LCBF0		;Bra if b2==0, TCC in use instead
;
;-Here to output to shiftlight
;
	LDD     #$1FFF			;Preload max DC
;
	BRSET   L0053,#$04,LCBED	;Status word, bra if b2==1, shiftlight enabled
;
	LDD     #$1F00			;Load min DC
;
LCBED	STD     L306C			;Save it, SL/3->2 sol.
;
;~~~~~~~~~~~~~~~
; Output to TCC
;~~~~~~~~~~~~~~~
;
LCBF0	LDX     #L400F			;Opt byte			
	BRCLR   0,X,#$40,LCC04		;Bra if b6==0, CC trans in use

	LDD     #$7FFF			;Preload max DC
	BRSET   L0085,#$20,LCC01	;Status word, bra if b5==1, TCC locked
;
	LDD     #$7F00			;Load min DC 
;
LCC01	STD     L306A			;Save it, PWM TCC ouput
;
LCC04	RTS				;Return 
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~    
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; Routine to output to EGR
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LCC05	CLRB				;Preclear B    
	BRSET   L0051,#$10,LCC18	;Status word, bra if b4==1, high batt volts 
;
	BRSET   L004F,#$80,LCC15	;Status word, bra if b7==1, engine running
;
;-Engine off, check if diags. in effect
;
	BRCLR   L0071,#$80,LCC18	;Status word, bra if b7==0, no diags. in effect
;
	DECB				;Load with 255, enable EGR with full DC    
	BRA     LCC18			;Bra ot output and return
;
;-Here to output to EGR
;
LCC15	LDAB    L01D2			;Load EGR DC
;
LCC18	LDAA    #$F0			;   
	STD     L3FD8			;Ouput EGR DC to EGR
;
	RTS				;Return    
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; Low octaine handling routine
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LCC1E	LDAA    L0002			;Loop counter
	CMPA    #$06			;
	BNE     LCC89			;Bra if counter !=, return 
;
;-Time to run the loop
;
	LDAB    L022C			;Low octaine SA ramping multiplier
	BRSET   L0019,#$10,LCC72	;Error word #4, bra if b4==1, knock sensor error
;
;-Check cool temp threshold
;
	LDAA    L0006			;Cool temp, linearized
	CMPA    L45AC			;Min temp threshold for low octain/high knock control
	BLS     LCC86			;Bra if cool temp is <=
;
;-Check MAP threshold
;
	LDAA    L01E3			;Load SD MAP
	CMPA    L45AD			;Lower MAP threshold for low octain/high knock control
	BCS     LCC86			;Bra if SD MAP < 
;
;-Check rate that MAP is changing
;
	SUBA    L01E5			;Subtract delayed SD MAP
	BCS     LCC44			;Bra if MAP is falling
;
	CMPA    L45AE			;Max rate of increase for low octain/high knock control
	BHI     LCC86			;Bra if >, MAP is rising quickly
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Check if there is low knock activity
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LCC44	LDAA    L022F			;Load knock retard
	CMPA    L45B0			;Threshold to flag low knock activity  
	BLS     LCC66			;Bra if retard is <=
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Check if there is high knock activity
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
	CMPA    L45AF			;Knock retard threshold to flag high knock activity 
	BLS     LCC86			;Bra if retard is <
;
;-Flag high knock activity
;
	BRSET   L0052,#$01,LCC5D	;Status word, bra if b0==1, high knock activity already
					;flagged 
;
	BSET    L0052,#$01		;Set b0, high knock activity
	BCLR    L0052,#$02		;Clear b1, low knock activity
	BRA     LCC78			;Bra to continue
;
;-High knock activity has been flagged
;
LCC5D	ADDB    L45B1			;Ramp up low octaine SA multiplier
	BCC     LCC78			;Bra if no overflow
;
	LDAB    #255			;Clear overflow
	BRA     LCC78			;Bra to continue
;
;~~~~~~~~~~~~~~~~~~~~~~~~~
; Flag low knock activity
;~~~~~~~~~~~~~~~~~~~~~~~~~
;
LCC66	BRSET   L0052,#$02,LCC72	;Status word, bra if b1==1, low knock activity already
					;flagged
;
	BSET    L0052,#$02		;Set b1, low knock activity
	BCLR    L0052,#$01		;Clear b0, high knock activity
	BRA     LCC78			;Bra to continue
;
;-Here if low knock activity has been flagged
;
LCC72	SUBB    L45B2			;Ramp down low octain SA multiplier
	BCC     LCC78			;Bra if no underflow

	CLRB 				;Clear underflow
;   
LCC78	STAB    L022C			;Save low octaine SA ramping multiplier
	LDAA    L45B3			;Load low octaine retard SA 
	MUL				;Low octaine SA x ramping multiplier
	ADCA    #$00			;Round if needed
	STAA    L022D			;Save it, low octaine SA retard
	BRA     LCC89			;Bra to return
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Here if quals not met for low octaine/high knock activity check
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LCC86	BCLR    L0052,#$03		;Status word, clear b0, high knock, and b1, low knock
					;activity
;
LCC89	RTS				;Return
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; Knock counter interrupt routine
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LCC8A	LDAA    #$80			;1000 0000, clear b7, TOC1
	STAA    L3023			;Timer flag 1 reg
	LDD     L3FC8			;Knock counter
	PSHB				;Push to stack    
	PSHA				;    
	SUBD    L0226			;subtract knock counts, one delayed, now delta knock counts
	TSTA				;Test MSB    
	BEQ     LCC9C			;Bra if knock counts >255
;
	LDAB    #255			;Load max counts
LCC9C	PULX				;Get knock counts    
	STX     L0226			;Save it, knock counts, one delayed
	TBA				;Transfer knock counts to A
;
;-Look up min knock counts vs. RPMs
;
	LDX     #L45D1			;Addr. of min knock counts vs. RPM table
	LDAB    L0062			;RPM/25
	LSRB				;/2
	LSRB				;/2
	ABX				;Add in RPM offset to X
	SUBA    0,X			;Subtract knock counts
	BLS     LCCC8			;Bra if underflow, dont update knock counts
;
;-Look up added knock count bias vs RPM
;
	LDX     #L4612			;Addr. of added knock counts vs RPM table
	LDAB    L0062			;RPM/25
	LSRB				;/2
	LSRB				;/2
	ABX				;Add in the RPM offset
	ADDA    0,X			;Add knock counts bias in
	BCC     LCCBD			;Bra if no overflow
;
	LDAB    #255			;Clear overflow
	BRA     LCCC5			;Bra to save
;
LCCBD	TAB				;Knock counts to B
	ADDB    L0230			;Add in to accumulated knock counts
	BCC     LCCC5			;Bra if no overflow
;
	LDAB    #255			;Clear overflow
;
LCCC5	STAB    L0230			;Save it, knock counts
;
LCCC8	RTI				;Return    
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; 3->2 solenoid and misc. knock sensor handling routine
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LCCC9	BRSET   L0002,#$10,LCCE9	;Loop counter, bra if b5==1, dont run the routine yet
;
;-5 Hz portion, handle knock retard decay
;
	LDX     #L459B			;Knock recovery rate
	LDAA    L0062			;RPM/25
	JSR     LF15E			;2D lookup
;											
	LDAB    L022F			;Load knock retard
	MUL				;Knock retard x recovery rate
	ADCA    #$00			;Perform carry if needed
	NEGA				;Negate, make negative
	BNE     LCCE0			;Bra if !=0
;
	LDAA    #255			;Load maximum ammount of decay
;
LCCE0	ADDA    L022F			;Knock retard - decay
	BCS     LCCE6			;Bra if no underflow
;
	CLRA				;Clear out knock retard   
;
LCCE6	STAA    L022F			;Save it, knock retard
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Knock sensor diag. handling
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LCCE9	LDAA    L5B03			;Error mask 4
	ANDA    #$10			;Clear all but b4, err 43, knock sensor
	BEQ     LCD23			;Bra if ==0, not flagging error
;
;-Flagging code 43 here
;
	LDX     #L3000			;Base CPU addr.
	LDAA    #$00			;Select A/D knock diag.
	JSR     LEF12			;Port D config routine
;
;-Read in knock sensor diag. voltage
;
	SEI				;Disable interrupts    
	LDAA    #$01			;Select channel B
	JSR     LEEFC			;A/D routine
;
	CLI				;Enable interrupts     
	LDAA    $34,X			;Load A/D result
	CMPA    L4E6B			;Upper voltage threshold to set error
	BHI     LCD0B			;Bra if voltage >, set error
;
	CMPA    L4E6C			;Lower voltage limit
	BCC     LCD1D			;Bra if voltage >=
;
;-Here to set code 43
;
LCD0B	LDAA    L022A			;Error 43 timer
	CMPA    L4E6A			;Time threshold to set error
	BHI     LCD18			;Bra if error timer >, set error 43
;
;-Not setting error yet
;
	INC     L022A			;Inc error timer
	BRA     LCD23			;Bra to continue
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Here to set code 43, knock sensor
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LCD18	BSET    L0019,#$10 		;Error word #4, set b4, code 43, knock sensor
	BRA     LCD23			;Bra to continue
;
;-Here if not setting error
;
LCD1D	BCLR    L0019,#$10		;Clear b4, knock sensor ok
	CLR     L022A			;Clear error 43 timer
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; 3->2 solenoid DC handling (also executed as RTS routine)
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LCD23	LDAA    L0130			;Load 3-2 shift modulation timer
	INCA				;+1    
	BEQ     LCD2C			;Bra if roll-over
;
	STAA    L0130			;Save timer
;
;-Check if diag. in effect
;
LCD2C	LDAA    L0118			;Load 3->2 shift solenoid diag. DC
	BEQ     LCD41			;Bra if ==0, no diag. override in effect
;
;-Format diag. DC for output to hardware
;
	LDAB    #0163			;Mult. for formatting diag. DC for output to hardware
	MUL				;(163 x diag. DC)     
	ASLD				;x2    
	ASLD				;x2, now x4    
	STAA    L012E			;Save it, 3->2 shift sol. DC
	CLR     L0130			;Clear 3->2 shift modulation timer for diag.
	BCLR    L007F,#$80		;Status word, clear b7, 3->2 shift in progress flag
	BRA     LCD78			;Bra to return
;
;-Check for high system voltage
;
LCD41	BRCLR   L0093,#$08,LCD4A	;Status word, bra if b3==0, no high system voltage
					;fault
;
;-High voltage present
;
	CLR     L012E			;Clear 3->2 solenoid DC
	BRA     LCD78			;Bra to return
;
;-Check if 3->2 shift still in progress
;
LCD4A	BRSET   L007F,#$80,LCD58	;Bra if b7==1, 3->2 shift still in progress
;
;-Check if 3->2 shift occurred
;
	BRCLR   L007F,#$40,LCD60	;Bra if b6==0, 3->2 shift not underway
;
;-3->2 shift underway
;
	BSET    L007F,#$80		;Set b7, 3->2 shift in progress
	CLR     L0130			;Reset 3-2 shift modulation timer
;
;-Check status of shift
;
LCD58	LDAA    L0130			;3-2 shift modulation timer
	CMPA    L5E8E			;Max duration for 3->2 shift modulation
	BCS     LCD7B			;Bra if time <
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~
; 3->2 shift complete here
;~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LCD60	BCLR    L007F,#$80		;Clear b7, 3->2 shift in progress flag
	BCLR    L0084,#$07		;Clear special state flags
	CLR     L0130			;Clear 3-2 shift modulation timer
	LDAA    L5E8B			;Load max 3->2 solenoid DC, fully open valve 
	LDAB    L00D3			;Load gear #
	CMPB    #$00			;1st gear
	BNE     LCD75			;Bra if in other gears
;
	LDAA    L5E8A			;In 1st, load min DC, close valve
;
LCD75	STAA    L012E			;Save it, 3->2 solenoid DC
;
LCD78	JMP     LCE3C			;Jump to return
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; 3->2 shift still in progress
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LCD7B	BRCLR   L0084,#$07,LCD81	;Status word, bra if b0, b1, and b2==0, no special
					;conditions flagged
;
;-Special conditions already flagged for this shift
;
	BRA     LCD98			;Bra to continue with current state
;
LCD81	BRCLR   L009C,#$03,LCD8C	;Status word, bra if b0, b1==0, not in manual 1st
					;or 2nd
;
;-Forward D2 or forward D3 selected, flag alternate apply rate
;
	BRSET   L0092,#$04,LCD8C	;Status word, bra if b2==1, trans press. switch man.
					;error
;
	BSET    L0084,#$01		;Status word, set b0, manual range select
;
LCD8C	BRCLR   L0093,#$04,LCD95	;Status word, bra if b2==0, output speed not lost
;
;-Output speed lost
;
	BSET    L0084,#$02		;Set b1, output speed lost
	BRA     LCD98			;Bra to continue
;
LCD95	BSET    L0084,#$04		;Set b2, output speed still present
;											
LCD98	BRCLR   L0084,#$01,LCDA5	;Bra if b0==0, manual range select not flagged
;
;-Manual range selected
;
	LDAA    L5E8C			;Load alternate apply rate for manual range select
	STAA    L012E			;Save it, 3->2 solenoid DC
	JMP     LCE29			;Jump to exit
;
LCDA5	BRCLR   L0084,#$02,LCDB2	;Bra if b1==0, output speed not lost
;
;-Output speed lost
;
	LDAA    L5E8D			;Load alternate apply rate if output speed lost 
	STAA    L012E			;Save it
	JMP     LCE29			;Jump to exit				
;	
;~~~~~~~~~~~~~~~~~~~~~~~~~~
; Look up 3->2 solenoid DC
;~~~~~~~~~~~~~~~~~~~~~~~~~~
;																					
LCDB2	LDD     L00A6			;Load trans baro
	SUBD    #$4000			;-50 kPa
	BCC     LCDBC			;Bra if no underflow
;
	LDD     #$0000			;Clear underflow
;
LCDBC	CPD     #$51EB			;114 kPa, upper limit for baro 
	BLS     LCDC5			;Bra if <=
;
	LDD     #$51EB			;Load upper limit
;
LCDC5	LDX     #$51EB			;114 kPa
	FDIV				;Trans baro value/max trans baro value, now value
					;ranging from 0-255 for lookup
	XGDX				;Get result into D
;
	LDAB    L00D1			;Load MPH x 2
	SUBB    #20			;-10 MPH, lower limit for lookup
	BCC     LCDD1			;Bra if no underflow
;
	CLRB				;Clear underflow
;
LCDD1	ASLB				;x2 for lookup
	BCC     LCDD6			;Bra if no overflow
;
	LDAB    #255			;Clear overflow
;
LCDD6	LDX     #L5E90			;Base 3->2 solenoid DC vs. baro and MPH
	JSR     LF17B			;3D lookup
;											
	STAA    L012E			;Save it, 3->2 solenoid DC
;											
	LDY     #L5E89			;Transmission opt. byte 
	BRCLR   0,Y,#$02,LCE07		;Bra if b1==0, 3->2 sol. DC temp compensation not
					;enabled
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Apply 3->2 solenoid DC temp compensation
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
	LDAA    L00AF			;Load transmission temp
	LDX     #L5FB4			;Index 3->2 sol. DC compensation vs. trans temp table
	JSR     LF15E			;2D lookup
;
	SUBA    #128			;-128 for corrections centered around 128
	ASLA				;x2 for DC ranging from 0-100%
	BCS     LCDFE			;Bra if result <0
;
;-Adding DC
;
	ADDA    L012E			;Add result into DC
	BCC     LCE04			;Bra if no overflow
;
	LDAA    #255			;Clear overflow
	BRA     LCE04			;Bra to save and continue
;
;-Reducing DC here
;
LCDFE	ADDA    L012E			;Add result into DC
	BCS     LCE04			;Bra if roll-over, valid result
;
	CLRA				;Clear invalid result 
;
LCE04	STAA    L012E			;Save it, 3->2 sol. DC
;
;-Here if temp compensation not optioned
;
LCE07	BRCLR   0,Y,#$04,LCE29		;Bra if b2==0, not compensating for A/C being active
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Apply 3->2 solenoid DC correction if A/C active
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
	BRCLR   L0096,#$02,LCE29	;Status word, bra if b1==0, A/C has not been requested
;
	LDAA    L5E8F			;3->2 DC compensation for A/C being active
	SUBA    #128			;-128
	BCS     LCE20			;Bra if negative corrections
;
;-Positive correction
;
	ADDA    L012E			;Add it into the 3->2 sol. DC
	BCC     LCE26			;Bra if no overflow
;
	LDAA    #255			;Clear overflow
	BRA     LCE26			;Save and continue
;
;-Here if negative correction
;
LCE20	ADDA    L012E			;Add into 3->2 sol. DC
	BCS     LCE26			;Bra if valid result
;
	CLRA				;Clear invalid result
;
LCE26	STAA    L012E			;Save it, 3->2 solenoid DC
;	
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Check limits on 3->2 solenoid DC
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;			
LCE29	LDAB    L5E8A			;Load min allowed DC
	CMPB    L012E			;Compare to stored DC
	BCC     LCE39			;Bra if stored DC <=, save limit instead
;
	LDAB    L5E8B			;Load max allowed DC
	CMPB    L012E			;Compare to stored DC
	BCC     LCE3C			;Bra if stored DC <=
;
LCE39	STAB    L012E			;Save limit instead
;
LCE3C	RTS				;Return
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~     
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; Routine to check params for EGR
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LCE3D	LDX     #L46EF			;Index quals for EGR
	LDAA    L400B			;Opt. byte
	BITA    #$40			;Test b6, MAT sensor in use
	BEQ     LCE55			;Bra if ==0, no MAT
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Mat sensor in use, check MAT quals
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
	LDAA    L0253			;Current MAT
	LDAB    4,X			;MAT threshold to disable EGR 									
	BRSET   L006E,#$80,LCE52	;Status word, bra if b7==1, EGR enabled
;
	LDAB    3,X			;MAT threshold to enable EGR
;
LCE52	CBA				;Compare MAT to threshold
	BLS     LCEB0			;Bra if MAT <= threshold, disable EGR
;
;-Check initial cool temp at startup
;
LCE55	LDAA    L02A5			;Initial cool temp 
	CMPA    0,X			;Threshold for warm startup
	BLS     LCE64			;Bra if initial cool temp was <=
;
;-Cool temp threshold for EGR if warm startup
;
	LDAA    2,X			;Cool temp threshold to enable EGR for warm startup
	CMPA    L0006			;Cool temp, linearized
	BHI     LCEB0			;Bra if cool temp is <, temp threshold not met for EGR
;
	BRA     LCE6A			;Bra, temp threshold met
;
;-Cool temp threshold for EGR if cool startup
;
LCE64	LDAA    1,X			;Cool temp threshold to enable EGR for cool startup
	CMPA    L0006			;Cool temp, linearized
	BHI     LCEB0			;Bra if cool temp was <, temp threshold not met for EGR
;
;~~~~~~~~~~~~~~~~~~~~~~
; Check MAP thresholds
;~~~~~~~~~~~~~~~~~~~~~~
;
LCE6A	BRCLR   L0018,#$30,LCE70	;Error word #3, bra if b4, low MAP, and b5, high MAP 
					;errors not set 
;
	BRA     LCEB0			;Bra, no EGR, MAP errors			
;
LCE70	LDAA    $0A,X			;MAP threshold to enable EGR
	BRCLR   L006E,#$10,LCE78	;Status word, bra if b4==0, MAP thresholds for EGR not
					;met
;
	LDAA    $0B,X			;MAP threshold to disable EGR
; 
LCE78	CMPA    L01E3			;SD MAP 
	BCS     LCE82			;Bra if MAP > 
;											
	BCLR    L006E,#$10		;Clear b4, MAP threshold for EGR not met
	BRA     LCEB0			;Bra, EGR not ok'd
;
;-MAP thresholds met for EGR
;
LCE82	BSET    L006E,#$10		;Set b4, MAP threshold for EGR met
;
;~~~~~~~~~~~~~~~~~~~~~~
; Check TPS thresholds
;~~~~~~~~~~~~~~~~~~~~~~
;
	LDAA    L006F			;Status word
	BITA    #$24			;Test b2, high TPS, and b5, low TPS errors
	BNE     LCEA0			;Bra if !=0, errors where set
;
	LDAA    8,X			;TPS threshold to enable EGR
	BRCLR   L006E,#$20,LCE93	;Status word, bra if b5==0. TPS thresholds not met for 
					;EGR
;
	LDAA    9,X			;TPS threshold to disable EGR
;
LCE93	CMPA    L01FC			;% TPS
	BCS     LCE9D			;Bra if TPS >
;
;-Threshold not met for EGR
;
	BCLR    L006E,#$20		;Clear b5, TPS thresholds not met for EGR
	BRA     LCEB0			;Bra, EGR not ok'd
;
;-Threshold met for EGR
;
LCE9D	BSET    L006E,#$20		;Set b5, TPS thresholds met
;
;~~~~~~~~~~~~~~~~~~~~~~
; Check RPM thresholds
;~~~~~~~~~~~~~~~~~~~~~~
;
LCEA0	LDAB    L0062			;RPM/25
	LDAA    6,X			;RPM threshold for enabling EGR
	BRCLR   L006E,#$40,LCEAA	;Bra if b6==0, RPM EGR thresholds for EGR not met
;
	LDAA    7,X			;RPM threshold for disabling EGR
;
LCEAA	CBA				;Compare threshold to RPM/25
	BLS     LCEB4			;Bra if RPMs >=
;
	BCLR    L006E,#$40		;Clear b6, RPM thresholds not met for EGR
;
;~~~~~~~~~~~~~~~~~~~~~~
; Here if EGR not ok'd
;~~~~~~~~~~~~~~~~~~~~~~
;
LCEB0	CLRA				;    
	JMP     LCF37			;Jump to continue
;
;-RPM threshold for EGR has been met
;
LCEB4	BSET    L006E,#$40		;Set b6, RPM threshold for EGR has been met
;
;~~~~~~~~~~~~~~~~~~~~~~
; Check MPH thresholds
;~~~~~~~~~~~~~~~~~~~~~~
;
	BRSET   L0016,#$10,LCEC3	;Error word #1, bra if b4==1, VSS errors
;
	LDAA    L02A7			;MPH
	CMPA    L46F4			;MPH threshold to disable EGR
	BCS     LCEB0			;Bra if MPH <, no EGR
;
;~~~~~~~~~~~~~~~~~~~~~~~~~
; Check vacuum thresholds
;~~~~~~~~~~~~~~~~~~~~~~~~~
;
LCEC3	LDAA    $0C,X			;Vac threshold for disabling EGR
	BRSET   L006E,#$04,LCECB	;Bra if b2==1, vac thresholds for EGR met
;
	LDAA    $0D,X			;Vac threshold for enabling EGR
;
LCECB	CMPA    L01EC			;Compare to current vacuum
	BCC     LCED5			;Bra if vacuum is <=
;
	BCLR    L006E,#$04		;Clear b2, vac thresholds for EGR not met
	BRA     LCEB0			;Bra, no EGR
;
LCED5	BSET    L006E,#$04		;Set b2, vac thresholds for EGR have been met
;
;~~~~~~~~~~~~~~~
; Check max TPS
;~~~~~~~~~~~~~~~
;
	LDAA    $0E,X			;Max TPS for EGR
	CMPA    L01FC			;Compare to TPS
	BCS     LCEB0			;Bra if TPS >, no EGR
;
;~~~~~~~~~~~
; Check AFR
;~~~~~~~~~~~
;
	LDAA    $0F,X			;Max AFR for EGR
	CMPA    L026D			;AFR
	BHI     LCEB0			;Bra if AFR is richer, no EGR
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Check if DFCO or in open loop
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
	BRSET   L0046,#$08,LCEB0	;Status word, bra if b3==1, DFCO in effect
;
;-Check if EGR allowed in open loop
;
	LDAA    L400D			;Opt. byte
	BITA    #$08			;Test b3, disable EGR in open loop
	BEQ     LCEF5			;Bra if ==0, allow EGR in open loop
;
	BRCLR   L003E,#$80,LCEB0	;Status word, bra if b7==0, in open loop, no EGR
;
;~~~~~~~~~~~~~~~~~~~~~
; Look up desired EGR
;~~~~~~~~~~~~~~~~~~~~~
;
LCEF5	LDX     #L4700			;Desired EGR table
;
;-Load desired load variable
;
	LDAB    L01F2			;MAP/Baro ratio
	LDAA    0,X			;Load option
	CMPA    #$02			;Option 2, use MAP/Baro ratio
	BEQ     LCF0B			;Bra if ==, use ratio
;
	LDAB    L01EC			;Vacuum
	LDAA    0,X			;Load option
	BEQ     LCF0B			;Bra if option==0, use vacuum
;
	LDAB    L01E3			;SD MAP
;
LCF0B	INX				;+1, advance to table
	LSRB				;load variable/2
;
;-Check RPM limits
;
	LDAA    L0062			;RPM/25
	CMPA    #160			;4000 RPM, upper limit for lookup 
	BLS     LCF15			;Bra if <=
;
	LDAA    #160			;Load upper limit
;
LCF15	JSR     LF17B			;3D lookup
;
	TAB				;Get result into B     
	LDAA    L01DE			;EGR correction factor vs. cool temp
	MUL				;Factor x desired EGR     
	ASLD				;x2, now /128    
	BCC     LCF22			;Bra if no overflow
;
	LDAA    #255			;Clear overflow
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Look up MAP/Baro EGR correction factor 
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LCF22	PSHA				;Push desired EGR to stack    					
	LDX     #L475D			;EGR correction factor table
	LDAB    L01EF			;Baro
;
	LDAA    L01E3			;Load SD MAP
	LSRA				;/2 for lookup
	JSR     LF17B			;3D lookup
;
	PULB				;Get desired EGR back    
	MUL				;EGR x correction factor    
	ASLD				;x2, now /128    
	BCC     LCF37			;Bra if no overflow
;
	LDAA    #255			;Clear overflow
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Check if EGR activation ok
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LCF37	CMPA    L46FF			;Min allowed desired EGR
	BCC     LCF3E			;Bra if desired EGR >=
;
;-Too little EGR requested, clear desired EGR
;
	BRA     LCF48			;Bra to clear desired EGR
;
;-Check for excessive batt volts
;
LCF3E	BRSET   L0051,#$10,LCF48	;Status word, bra if b4==1, high batt volts
;
;-Check if linear EGR failed diags.
;
	BRSET   L0072,#$20,LCF48	;Status word, bra if b5==1, linear EGR failed diags.
;
;-EGR activation ok'd
;
	BRA     LCF49			;Bra to save desired EGR
;
;-Here to clear desired EGR
;
LCF48	CLRA				;Clear desired EGR
;    
LCF49	STAA    L01D1			;Save it, desired EGR
	BRCLR   L0082,#$40,LCF53	;HU status word, bra if b6==0, no HU
;
	JSR     L1815			;To HU vector table
;
;-Check if mode 4 in effect
;
LCF53	BRCLR   L003B,#$10,LCF64	;Status word, bra if b4==0, no mode 4
;
;-Mode 4 in effect
;
	LDAA    L0395			;
	BITA    #$02			;
	BEQ     LCF64			;
;
	LDAA    L0396			;
	STAA    L01D1			;Desired EGR
;
LCF64	RTS				;Return    
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; Routine to look up/calculate EGR flowrate
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LCF65	LDX     #L400D			;Opt. byte
	BRSET   0,X,#$20,LCF70		;Bra if b5==1, use table to look up EGR flow
;
;-Conditionally calculate EGR flow if no backpressure EGR present
;
	BRCLR   0,X,#$04,LCFAF		;Bra if b2==0, no backpressure EGR present
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Using table to look up %EGR flow
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LCF70	BCLR    L006E,#$80		;Clear b7, disable EGR
;									
	BRSET   L0070,#$01,LCF80	;Status word, bra if b0==1, EGR failed diags.
;
	BRSET   L0070,#$04,LCF86	;Status word, bra if b2==1, EGR diags. underway, 
					;disable EGR for diag. purposes
;
	LDAA    L01D2			;% EGR
	BNE     LCF83			;Bra if !=0, continue
;
;-No EGR here, clear flow and continue
;
LCF80	CLRA				;Clear A    
	BRA     LCFAB			;Bra to save and exit			
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Look up EGR flow as a percentage of total airflow
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LCF83	BSET    L006E,#$80		;Set b7, EGR is active
;
;-Here if EGR diags. underway, disable EGR
;
LCF86	LDX     #L4780			;EGR flow table
;
;-Scale vacuum for lookup
;												
	LDAB    L01EC			;Vacuum
	NEGB				;Neg., now 0-80 kPa    
	CMPB    #64			;20 kPa, lower limit to scale vac
	BHI     LCF94			;Bra if vacuum kPa >
;
;-Vac kPa < 20, val for lookup = kPa x 2
;
	ASLB				;x2    
	BRA     LCF9F			;Bra to continue
;
;-Vac kPa >= 20, val for lookup = (kPa - 64)/2 + 128
;
LCF94	SUBB    #64			;Subtract offset
	LSRB				;/2    
	ADDB    #128			;+128
	CMPB    #192			;60 kPa, upper limit for lookup
	BLS     LCF9F			;Bra if kPa <=
;												
	LDAB    #192			;Load max
;
LCF9F	LDAA    L0062			;Load RPMs/25
	CMPA    #160			;4000 RPM, max RPM for lookup
	BLS     LCFA7			;Bra if RPMs <=
;
	LDAA    #160			;Load max
;
LCFA7	LSRA				;/2 for lookup
	JSR     LF17B			;3D lookup
;
LCFAB	PSHA				;Push fuel correction to stack   
	JMP     LD048			;Continue...
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Check quals for enabling EGR
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LCFAF	BCLR    L006E,#$80		;Clear b7, EGR energized
	BRSET   L0070,#$01,LD02A	;Status word, bra if b0==1, EGR faild diags.
;
	BRSET   L0072,#$20,LD02A	;Status word, bra if b5==1, linear EGR failed diags
;
	BRSET   L0070,#$04,LCFF3	;Bra if b4==1, EGR diags in progress
;
	LDX     #L400D			;Opt. byte
	BRCLR   0,X,#$02,LCFDE		;Bra if b1==0, EVRV EGR
;
;-Here if linear EGR in use
;
	BRCLR   0,X,#$10,LCFCD		;Bra if b4==0, allowing EGR at idle
;
;-Not allowing EGR during idle
;
	BRSET   L0050,#$80,LD02A	;Status word, bra if b7==1, idling, no EGR
;
;-Check to see if pintle position great enough for EGR flow
;
LCFCD	LDAB    L01DC			;% EGR
	CMPB    L4850			;Min pintle position for EGR flow
	BCS     LD02A			;Bra if pintle position is <, zero flow
;
;-Check for sufficient engine run time
;
	LDX     L00F8			;Engine run time
	CPX     L4851			;Min run time to enable EGR
	BCS     LD02A			;Bra if run time <
;
;-Quals met for EGR
;
	BRA     LCFE6			;Bra to enable EGR
;
;-Here if standard EGR is in use
;
LCFDE	LDAB    L01D2			;% EGR
	CMPB    L4853			;Min % EGR for EGR flow
	BCS     LD02A			;Bra if % EGR <, no flow
;
;-Enable EGR
;										
LCFE6	BSET    L006E,#$80		;Set b7, EGR active
;
;~~~~~~~~~~~~~~~~~~~~
; Calculate EGR flow
;~~~~~~~~~~~~~~~~~~~~
;									
	LDAA    L01DC			;% EGR
	LDX     #L400D			;Opt. byte
	BRSET   0,X,#$02,LCFF6		;Bra if b1==1, liner EGR in use 
;
;-Here if standard EGR in use or if EGR diags. underway
;
LCFF3	LDAA    L01D1			;Desired EGR
;
LCFF6	LDAB    #160			;Multiplier, scale %EGR
	MUL				;%EGR x multiplier      
	ADCA    #$00			;Round up if needed
	PSHA				;Push %EGR to stack    
;
;-Look up EGR backpressure
;
	LDAA    L0255			;gms/sec of airflow
	LDX     #L47C9			;EGR exhaust back press. vs airflow
	JSR     LF15E			;2D lookup
;
	LDAB    L01EC			;Vacuum
	NEGB				;Neg, get manifold pressure    
	ABA				;Add vacuum in, now total delta pressure				     
	BCC     LD00E			;Bra if no overflow
;
	LDAA    #255			;Clear overflow
;
LD00E	STAA    L01D3			;Save it, delta pressure across EGR
	CMPA    #64			;20 kPa
	BCS     LD020			;Bra if delt press <
;
;-Pressure across EGR >= 20 kPa, scaled kPa EGR press = .63 x (val - 64) + 10
;
	SUBA    #64			;-20 kPa offset			
	LSRA				;/2    
	ADDA    #64			;Add in 20 kPa bias
	CMPA    #144			;32 kPa			
	BCS     LD020			;Bra if <
;
	LDAA    #144			;Load max
;
LD020	TAB				;delta pressure to B     
	PULA				;Get desired EGR back  
	LDX     #L47DB			;%EGR flow vs. delta pressure
	JSR     LF17B			;3D lookup

	BRA     LD02B			;Bra to save
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Here if no EGR/no EGR flow
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LD02A	CLRA 				;Clear EGR flow 
;  
;-EGR active here
;
LD02B	STAA    L01D4			;Save it, EGR flow
	LDAB    L47DA			;Gms/sec EGR flow scalar
	MUL				;scalar x EGR flow     
	LSRD				;/2    
	BCC     LD038			;Bra if b0==0

	ADDD    #01			;Round up
;
;-Get EGR flow to total airflow
;			
LD038	LDX     L0255			;Gms/sec airflow
	FDIV				;Divide EGR flow by airflow    
	PSHX				;Result to stack    
	PULA				;Get result into D    
	PULB				;    
	CMPA    #$FF			;
	BEQ     LD047			;Bra if ==255
;
	TSTB				;Test LSB    
	BPL     LD047			;Bra if ==0
;
	INCA				;Round up
;    
LD047	PSHA				;Result to stack
; 
;-Apply baro correction to flow %
;
LD048	LDX     #L484C			;EGR baro correction
	LDAA    L01EF			;Baro
	LDAB    #96			;46 kPa, lower limit for lookup
	JSR     LF15A			;2D lookup w/ SBA
;
	PULB				;Get %EGR to airflow back    
	MUL				;%flow x baro factor     
	TSTB				;Test LSB    
	BPL     LD059			;Bra if msb ==0

	INCA				;Round up if needed 
;
;-Lookup EGR flow filter coeff. vs airflow
;
LD059	PSHA				;%EGR flow to stack
	LDAA    L0255			;Airflow, gms/sec
	CMPA    #64			;64 gms/Sec, max for lookup
	BLS     LD063			;Bra if airflow > 
;
	LDAA    #64			;Load max
;
LD063	LDX     #L47C4			;EGR flow filter coeff vs. airflow
	JSR     LF15E			;2D lookup
;
	TAB				;Filter coeff. to B     
	PULA				;Get %EGR flow back    
	LDX     L01D5			;Filtered %EGR flow
	JSR     LF0F6			;Filter routine
;
	STD     L01D5			;Save it, filtered %EGR flow
	RTS				;Return    
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 			
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; EGR pintle pos. read in routine 
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LD075	LDAA    L400D			;Option byte
	BITA    #$02			;b1, 1=linear EGR, 0=EVRV EGR 
	BNE     LD083			;Bra if using linear EGR
;
;-Here if using EVRV EGR
;
	CLRA				;Clear A   
	STAA    L01DC			;% EGR 
	JMP     LD11C			;Bra to return
;
;-Here if using linear EGR
;
LD083	LDX     #L3000			;Base Reg. addr.
	SEI  				;Disable interrupts
	LDAA    #$03			;
	BCLR    $08,X,#$38		;Port D, clear b3-b5
;
	ASLA				;shift left
	ASLA				;
	ASLA				;now 0001 1000
	ORAA    $08,X			;OR b3 with b3 in Port D
;
	STAA    8,X			;Store it, port D									;
	LDAA    #$01			;Configure A/D converter, single channel read
	BCLR    L001A,#$02		;Clear b1, error 55, faulty PCM 
	STAA    L3030			;Save it, A/D control reg
;		
	LDAB    #12			;Delay counter, loop untill 0 to wait for result
;								
LD09D	BRSET   $30,X,#$80,LD0AA	;Bra if b7==1, conversion complete
;
	DECB				;-1 from delay counter
	BNE     LD09D			;Bra if !=0, continue to wait for data
;
;-A/D conversion failed 
;
	BSET    L003A,#$01		;Error word, set b0, A/D converter failed
	BSET    L001A,#$02		;Error word #5, set b1, error 55, faulty PCM 
;
;-Here if A/D conversion successful
;
LD0AA	CLI				;Enable interrupts 
	LDAA    $34,X			;Read in A/D EGR pintle value
	STAA    L01D7			;Save it, linear EGR pos.
	CMPA    L01D8			;Closed EGR pintle pos., 
	BCC     LD0CA			;Bra if current pintle pos. >=
;
;-Filter EGR pintle position down to lowest current pintle position, serves as pos. for closed pintle
;											
	LDX     L01D8			;Lag filtered closed EGR pintle pos.
	LDAB    L489D			;Pintle closing lag filter coeff.
	JSR     LF0F6			;Lag filter routine
;											
	CMPA    L489C			;Threshold for EGR closed
	BCC     LD0C7			;Bra if >=
;
;-EGR below threshold for closed position
;
	LDAA    L489C			;Load A/D value for closed pintle position
	CLRB				;Clr LSB
;
LD0C7	STD     L01D8			;Save it, lag filtered closed EGR pintle pos.
;	
LD0CA	LDAA    L01D7			;A/D EGR pintle pos.
	LDAB    #128			;Load LSB with 128
	SUBD    L01D8			;Subtract closed EGR pintle position
	BCC     LD0D5			;Bra if current EGR pintle pos. is greater  
;
	CLRA				;Clear pintle position
;
;-Calculate % that EGR is open
;
LD0D5	LDAB    L489A			;EGR pintle position scalar
	MUL				;(EGR pintle position) x scalar 
	CPD     #$3FC0			;Upper limit for EGR pos. value
	BLS     LD0E2			;Bra if <=
;
	LDD     #$3FC0			;Load upper limit instead
;
LD0E2	ASLD				;x2
	ASLD				;x2
	LDX     L01DA			;Filtered %EGR
	LDY     #L489E			;Filter coeff for %EGR
	JSR     LF0D3			;Lag filter
;
	STD     L01DA			;Save it, filtered %EGR
	ADDD    #128			;Round if needed
	STAA    L01DC			;Save it, %EGR
;
;-See if position for closed EGR needs adjusting 
;
	CMPA    #128			;50% EGR
	BLS     LD100			;Bra if less
;
;-EGR position > 50%
;
	BSET    L0041,#$01		;Set b1, adjust position for EGR closed
	BRA     LD11C			;Bra, return
;
;-Check and adjust closed pintle position if desired EGR ==0
;
LD100	TST     L01D1			;Test desired %EGR posit.
	BNE     LD11C			;Bra if !=0, return
;
	BRCLR   L0041,#$01,LD11C  	;Status word, bra if b0==0, dont adjust EGR closed pinlte position
;
	BCLR    L0041,#$01		;Clear b0, pintle position adjusted
;
	LDAA    #02			;Add in two counts
	ADDA    L01D8			;Add in closed EGR pintle pos.
	CMPA    L489B			;Max value for closed position
	BLS     LD119			;Bra if less

	LDAA    L489B			;load max
;
LD119	STAA    L01D8			;Save it, closed EGR pintle pos.
;
LD11C	RTS				;Return
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~     
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; EGR PID position control routine
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LD11D	BRSET   L0070,#$04,LD15A	;Status word, bra if b2==1, EGR diags. in progress
;
	LDX     #L400D			;Opt. byte
	BRSET   0,X,#$02,LD12E		;Bra if b==1, using linear EGR
;
;-Using standard EGR here
;
	LDAA    L01D1			;Load desired %EGR posit.
	JMP     LD1D5			;Save and return
;
;-Using linear EGR
;
LD12E	BRCLR   L003B,#$10,LD139	;Status word, bra if b4==0, not in mode 4
;
	LDAA    L0395			;(???)
	BITA    #$02			; b1,
	BNE     LD141			;
;
LD139	BRSET   L0070,#$01,LD15A	;Bra if b0==1, EGR failed diags.
;
	BRSET   L0072,#$20,LD15A	;Status word, bra if b5==1, linear EGR failed diags. 
;
;-Calc. EGR error
;
LD141	LDAA    L01D1			;Load desired %EGR position
	BCLR    L006D,#$08		;Status word, clear b3, EGR position error sign 
;											
	SUBA    L01DC			;Subtract %EGR
	BCC     LD150			;Bra if desired EGR >=

	NEGA				;Make abs. val. of EGR pos. error
	BSET    L006D,#$08		;Status word, set b3, EGR pos. error < 0
;											
LD150	STAA    L01DD			;Save it, %EGR position error
	TST     L01D1			;Desired EGR
	BNE     LD160			;Bra if !=0
;
;-Desired EGR ==0
;
	BRA     LD1CD			;Bra to perform resets and exit
;
LD15A	CLRA				;
	STAA    L01DD			;Clear EGR position error
	BRA     LD1CE			;Bra to perform resets				
;
;-Here if desired EGR >0
;
LD160	LDD     L01DF			;Load EGR integral
	BNE     LD172			;Bra if !=0
;
;-Init EGR integral
;
	LDAA    L01D1			;Load desired %EGR position
	LDX     #L489F			;Addr. of EGR int. init vs. desired %EGR pos.
	JSR     LF153			;2D lookup with line count
;
	CLRB				;Clear LSB    
	STD     L01DF			;Save it, EGR integral
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Look up EGR integral gain term 
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LD172	LDAA	L01DD			;EGR position error
	LDX     #L48A5			;Addr. of EGR integral gain vs. EGR error
	JSR     LF153			;2D lookup with line count
;
	LDAB    L01DD			;EGR error
	MUL				;EGR error x integral gain
	STD     L084F			;Save it, delta EGR integral term
;
;Add it to the EGR integrator
;
	LDD     L01DF			;Load EGR Integrator
;
	BRSET   L006D,#$08,LD193	;Status word, bra if b3==1, error <0 
;
;-Positive EGR integral term
;
	ADDD    L084F			;Add in the integral term
	BCC     LD191			;Bra if no overflow
;
	LDD     #$FFFF			;Clear overflow
;
LD191	BRA     LD19A			;Bra to save
;
;-Negative EGR integral term
;
LD193	SUBD    L084F			;Subtract the integral term
	BCC     LD19A			;Bra if no underflow
;
	CLRA				;Clear underflow    
	CLRB				;
;   
LD19A	STD     L01DF			;Save it, EGR integrator
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Look up EGR proportional gain
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
	LDAA    L01DD			;EGR position error
	LDX     #L48AB         		;Addr. of EGR proportional gain multiplier vs.
					;EGR error 
	JSR     LF153			;2D lookup with line count
;
	LDAB    L01DD			;EGR position error
	MUL				;gain x error
	ASLD				;x2
	BCC     LD1B0			;Bra if no overflow
;
	LDAA    #255			;Clear overflow
	CLRB				;
;    
LD1B0	STD     L01E1			;Save it, EGR proportional term
	LDD     L01DF			;Load integral
	BRSET   L006D,#$08,LD1C4	;Status word, bra if negative
;
;-Add in the proportional term here
;
	ADDD    L01E1			;Add in the prop. term
	BCC     LD1C2			;Bra if no overflow
;
	LDD     #$FFFF			;Clear overflow
LD1C2	BRA     LD1D5			;Bra to save and exit		
;
;-Subtract out the proportional term here
;
LD1C4	SUBD    L01E1			;Subtract the prop. term
	BCC     LD1D5			;Bra if no underflow
;
	CLRA				;Clear underflow    
	CLRB				;
	BRA     LD1D5			;Bra to save and return
;
;-Here to perform resets
;
LD1CD	CLRA				;Clear MSB	
;			   
LD1CE	CLRB				;Clear LSB
	STD     L01DF			;Save it, EGR Integrator
	STD     L01E1			;Delta EGR integrator term
;
;-Here to update and return
;
LD1D5	STAA    L01D2			;Desired %EGR
	RTS				;Return     
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; Charcoal canister purge routine
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;-5 Hz routine
;
LD1D9	BRCLR   L0002,#$10,LD1E0	;Loop counter, bra if b4==0, run the routine  
;
;-Not time to run the routine yet
;
	JMP     LD2D6			;Bra to return
;
;-Check quals for CCP
;
LD1E0	LDAA    L400C			;Opt byte 
	BITA    #$80			;Test b7, CCP enabled
	BEQ     LD21A			;Bra if b7==0, no CCP

	LDAA    L0006			;Cool temp, linearized
	CMPA    L402C			;Cool temp threshold for full DC purge on startup
	BLS     LD1FE			;Bra if cool temp is <=
;
	LDD     L00F8			;Engine run time
	CPD     L402D			;Engine run time for full DC purge
	BHI     LD1FE			;Bra if run time is higher 
;
;-Warm engine temps and engine has just started, set full purge
;
	LDAA    #255			;Max duty cycle
	STAA    L01F9			;Save it, CCP duty cycle
	JMP     LD2CB			;Return
;
;-Check if DFCO is active
;
LD1FE	BRSET   L0046,#$08,LD21A	;Status word, bra if b3==1, DFCO in effect, no CCP
;
;-Check cool temp
;
	LDAA    L0006			;Cool temp, linearized
	CMPA    L402B			;Min temp for CCP
	BLS     LD21A			;Bra if cool temp is <=, no CCP
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Check params to enable/disable CCP
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
	LDX     #L4030			;Index params for enabling CCP
	BRCLR   L0041,#$80,LD213	;Status word, bra if CCP solenoid not active
;
;-Load params for disabling CCP
;
	LDX     #L4033			;Index params for disabling CCP
;
;-Check speed
;									
LD213	LDAB    L02A7			;MPH
	CMPB    0,X			;MPH threshold to enable/disable CCP
	BCC     LD22E			;Bra if MPH >=, not disabling CCP yet
;
;~~~~~~~~~~~~~~~~
; Here if no CCP
;~~~~~~~~~~~~~~~~
;							   				
LD21A	CLR     L01F9			;Clear CCP duty cycle
	CLR     L01FB			;Clear CCP update timer
;											
	LDAA    L02A7			;MPH
	CMPA    L402F			;Lower MPH threshold to clear CCP DC multiplier
	BCC     LD22B			;Bra if MPH >=
;
	CLR     L01FA			;Clear CCP DC multiplier
;
LD22B	JMP     LD2CB			;Bra to exit
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Here to continue checking quals
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;-Check TPS
;
LD22E	LDAB    L01FC			;TPS 
	CMPB    2,X			;TPS threshold to enable/disable CCP
	BCS     LD21A			;Bra if TPS <, disable CCP
;
;-Check MAP 
;
	LDAB    L027A			;MSB, kPa of MAP
	CMPB    1,X			;Compare to MAP threshold to enable/disable CCP
	BCS     LD21A			;Bra if MAP <, disable CCP
;
;-Check to make sure CCP is allowed in current BLM cell
;
	LDX     #L4058			;Load in table of allowed cells to purge in
	LDAB    L026A			;Load BLM cell number
	ABX				;Add in as offset
	LDAA    0,X			;Load logic value
	BEQ     LD21A			;Bra if ==0, purge not allowed
;
;-Check if closed loop is in effect
;
	BRCLR   L003E,#$80,LD28D	;Status word, bra if b7==0, not in closed loop
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; In closed loop, check if updates to CCP DC are allowed
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
	LDAB    L01EC			;Vacuum
	CMPB    L4024			;Lower vac threshold to allow update
	BHI     LD288			;Bra if vacuum is <, no update
;
	LDAA    L0005			;Status word 
	BITA    #$C0			;Test b6, TPS AE, and b7, MAP AE in effect
	BNE     LD288			;Bra if either AE is in effect, no update
;
;-Check CCP DC update timer
;
	LDAB    L01FB			;CCP update timer
	CMPB    L4025			;Min time between updates 
	BCC     LD266			;Bra if time >=, update CCP DC
;
;-Not time to update yet
;
	INC     L01FB			;+1, CCP update timer
	BRA     LD2A8			;Bra to continue
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Adjust CCP duty cycle to target desired INT range 
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LD266	LDAA    L01FA			;Closed loop CCP DC multiplier
	LDAB    L0232			;Integrator
	CMPB    L4026			;Int threshold to increase CCP DC 
	BCC     LD27E			;Bra if int >=
;
	CMPB    L4027			;Int threshold to decrease CCP
	BCC     LD288			;Bra if int >=, no adjustments
;
;-Here to decrease CCP DC
;
	SUBA    L402A			;Rate to decrease multiplier for reduced DC
	BCC     LD285			;Bra if no underflow
;
	CLRA				;Clear underflow
	BRA     LD285			;Bra to save and continue
;
;-Here to increase CCP DC
;
LD27E	ADDA    L4029			;Rate to increase multiplier for additional DC
	BCC     LD285			;Bra if no overflow
;
	LDAA    #255			;Clear overflow
;
LD285	STAA    L01FA			;Save it, closed loop CCP multiplier
;
;-Here if not updating yet
;
LD288	CLR     L01FB			;CCP update timer
	BRA     LD2A8			;Bra to continue
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Calculate open loop CCP duty cycle
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LD28D	LDX     #L4047			;Minimum CCP DC 
	LDAA    L0255			;Gms/sec airflow
	JSR     LF15E			;2D lookup
;
	LDAB    L4028			;Load open loop CCP DC multiplier
	MUL				;DC x multiplier     
	ADDD    #$0040			;round up if needed
	ASLD				;x2, now /128    
	BCC     LD2A3			;Bra if no overflow
;
	LDD     #$FFFF			;Clear overflow
;
LD2A3	STAA    L01F9			;Save it, CCP dutycycle
	BRA     LD2CB			;Bra to continue
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Calculate closed loop CCP duty cycle
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LD2A8	LDX     #L4036			;Closed loop CCP DC
	LDAA    L0255			;Gms/sec airflow
	JSR     LF15E			;2D lookup
;
	LDAB    L01FA			;Closed loop CCP DC multiplier	
	MUL				;CCP DC x multiplier
	ADCA    #0			;Round if needed
	STAA    L01F9			;CCP duty cycle
;
;-Check to make sure there is a minimum ammount of CCP DC
;
	LDX     #L4047			;Minimum CCP duty cycle
	LDAA    L0255			;Gms/sec airflow
	JSR     LF15E			;2D lookup
;
	CMPA    L01F9			;CCP DC
	BLS     LD2CB			;Bra if DC is greater
;
	STAA    L01F9			;Save minimum DC instead
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Here to update CCP status
;~~~~~~~~~~~~~~~~~~~~~~~~~~~
;										
LD2CB	BCLR    L0041,#$80		;Status word, clear b7, CCP not energized										
	LDAA    L01F9			;CCP DC
	BEQ     LD2D6			;Bra if CCP ==0
;
	BSET    L0041,#$80		;Status word, set b7, CCP energized
;
LD2D6	RTS				;Return    
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; Routine to check for catalytic converter reaction overtemp
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LD2D7	LDX     #L400F			;Opt. byte
	BRSET   0,X,#$10,LD2E1		;Bra if b4==1, check for cat overtemp
;
;-Not checking for catalytic converter reaction overtemp
;
	JMP     LD398			;Jump to clear overtemp flag and return
;
;-Checking for cat reaction overtemp
;
LD2E1	LDAA    L0002			;Loop counter
	CMPA    #$05			;0000 1001
	BNE     LD2EB			;Bra if !=, dont run loop
;
	BRSET   L004F,#$80,LD2EE	;Status word, bra if b7==1, engine is running
;
;-Engine off/not time to run the loop yet, return
;
LD2EB	JMP     LD398			;Jump to clear overtemp flag and return

;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Check if under high load, and retain current state if high loads, low RPMs present
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LD2EE	LDAA    L01FC			;TPS
	CMPA    L4074			;TPS threshold to maintain current cat
					;reaction  temp state
	BHI     LD2FB			;Bra if TPS is >=, check for low RPMs
;
;-Skip test and continue to check for overtemps
;
	CLR     L007E			;Clear cat overtemp protection test timer
	BRA     LD313			;Bra to continue
;
;-Check for low RPM operation
;
LD2FB	LDAA    L0062			;RPM/25
	CMPA    L4075			;Threshold to check quals for maintaining 
					;current cat reaction temp state
	BCC     LD309			;Bra if RPMs >=, continue
;
;-High load, low RPM operation present
;
	LDAA    L007E			;Cat overtemp protection test timer
	INCA				;+1    
	BEQ     LD309			;Bra if ==0, dont save
;
	STAA    L007E			;Save it
;
LD309	LDAA    L007E			;Load timer
	CMPA    L4076			;Timer threshold to skip reaction temp test and 	
					;retain current cat reaction temp state if high
					;load, low RPMs present 
	BCS     LD313			;Bra if less, continue
;
;-Here to retain current cat reaction temperature state and skip update
;
	JMP     LD398			;Exit, dont perform test
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Portion of routine to monitor catalytic converter temp
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;-Look up cat temp filter coeff.
;
LD313	LDAA    L0255			;Gms/sec airflow
	TAB				;Get into B     
	LDX     #L4077			;Cat temp filter coeff. vs. airflow
	JSR     LF15E			;2D lookup
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Look up flow based reaction temp
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
	STAA    L007C			;Save it, cat temp filter coeff.
	LDAA    L0062			;RPM/25
	LSRA				;/2 for lookup    
	LDX     #L408F			;RPM/airflow reaction component table
	JSR     LF17B			;3D lookup
;
	STAA    L007B			;RPM/airflow cat reaction component
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Look up reaction temp bias vs. AFR
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
	LDAA    L026D			;AFR
	CMPA    #155			;15.5, max numerical AFR for lookup
	BLS     LD333			;Bra if <=
;
	LDAA    #155			;Load upper limit
;
LD333	SUBA    #107			;Subtract 10.7 AFR bias
	BCC     LD338			;Bra if no underflow
;
	CLRA				;Clear underflow 
;
LD338	ASLA				;AFR x2 for lookup    
	LDX     #L4088			;Cat reaction temp. bias vs. AFR
	JSR     LF15E			;2D lookup
;
	STAA    L007D			;Save cat reaction AFR temp. bias
	SUBA    #$80			;-128, make signed
	BMI     LD34D			;Bra if result was negative
;
;-Positive bias here
;
	ADDA    L007B			;Add in flow based reaction temp
	BCC     LD352			;Bra if no overflow
;
	LDAA    #255			;Clear overflow
	BRA     LD352			;Bra to continue
;
;-Negative bias here
;
LD34D	ADDA    L007B			;Add in flow based reaction temp 
	BCS     LD352			;Bra if result is now positive
;
	CLRA
;    
LD352	STAA    L007A			;Save it, cat reaction temperature
	LDAA    L007A			;Load reaction temp
	LDAB    L007C			;Load flow based filter coeff.
	LDX     L0078			;Filtered cat reaction temperature
	JSR     LF0F6			;Filter routine
;
	STD     L0078			;Save it, filtered cat reaction temp.

	BRCLR   L003E,#$80,LD380	;Status word, bra if b7==0, not in closed loop
;
;~~~~~~~~~~~~~~~~
; In closed loop
;~~~~~~~~~~~~~~~~
;
	LDAB    L0078			;Filtered cat reaction temp
	CMPB    L406D			;Upper allowed threshold for cat reaction temp
	BLS     LD37B			;Bra if reaction temp is <=
;
	LDAA    L084C			;Cat conv. overtemp timer
	INCA				;+1
	BNE     LD371			;Bra if no roll-over
;											
	DECA				;-1, roll back to 255
;
LD371	STAA    L084C			;Cat conv. overtemp timer
	CMPA    L4073			;Compare to max time allowed for overtemp
	BCC     LD395			;Bra if time >=, flag cat. conv. overtemp
;
	BRA     LD386			;Bra to continue
;
LD37B	CMPB    L406E			;Upper allowed threshold for cat reaction temp
					;to clear overtemp condition
	BHI     LD386			;Bra if temp >
;
;-Catalytic converter reaction temp is within allowed range
;
LD380	CLRA				;
	STAA    L084C			;Clear overtemp tiemr
	BRA     LD390			;Bra to clear overtemp flag and return
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Check maximum reaction temp. limits
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LD386	CMPB    L406F			;Maximum allowed cat reaction temp
	BHI     LD395			;Bra if temp greater, flag cat overtemp
;
	CMPB    L4070			;Upper allowed raction temp to clear overtemp condition
	BHI     LD398			;Bra if temp >, dont reset
;
;-Here to clear overtemp flag
;
LD390	BCLR    L0052,#$40		;Status word, clear b6, cat converter overtemp flag
	BRA     LD398			;Bra to return
;
;-Here to set overtemp flag
;
LD395	BSET    L0052,#$40		;Set b6, cat converter overtemp flag
;
LD398	RTS				;Return
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~  
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; CPI variable tuning valve control routine
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LD399	LDX     #L400E			;Opt. byte
	BRSET   0,X,#$08,LD3A2		;Bra if b3==1, CPI variable tuning enabled
;
	BRA     LD3C6			;Bra if ==0, no variable tune
;
LD3A2	LDX     #L412B			;Start of CPI variable tune params
;								
	BRSET   L0052,#$08,LD3B0  	;Status word, bra if b3==1, CPI tuning valve enabled
;
;-Check quals to enable tuning valve
;
	LDAA    L00A2			;Linear cool temp
	CMPA    6,X			;Lower cool temp threshold to enable tuning valve
	BCS     LD3C6			;Bra if cool temp <, dont enable tuning valve
;
;-Here to check quals for enabling CPI tuning valve
;											
	INX				;+1, advance to params for enabling
;
;-Here to check quals for disabling the CPI tuning valve
;
LD3B0	LDAA    L0062			;RPM/25
	CMPA    0,X			;Check lower RPM hyst. bound
	BLS     LD3C6			;Bra if RPMs <=, disable tuning valve
;
	CMPA    2,X			;Check upper RPM hyst. bound
	BHI     LD3C6			;Bra if RPMs >, disable tuning valve
;
	LDAA    L01FC			;TPS
	CMPA    4,X			;Lower TPS hyst. bound to disabling tuning valve
	BLS     LD3C6			;Bra if TPS <=, disable tuning valve
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Here to enable CPI tuning valve
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
	BSET    L0052,#$08  		;Status word, set b3, enable tuning valve
	BRA     LD3C9			;Bra to return
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Here to disable CPI tuning valve
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LD3C6	BCLR    L0052,#$08		;Clear b3, disabling tuning valve
;
LD3C9	RTS				;Return 	 
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;~~~~~~~~~~~~~~
;
; VSS handling
;
;~~~~~~~~~~~~~~
;
LD3CA	LDAB    L080C			;# of VSS pulses recieved since last update
	BNE     LD3DD			;Bra if pulse(s) recived
;
;~~~~~~~~~~~~~~~~~~~~
; No VSS pulses here
;~~~~~~~~~~~~~~~~~~~~
;
	CLRA				;Preclear A
	LDAB    L0811			;VSS pulse timer
	BEQ     LD3D8			;Bra if ==0, clear current MPH
;
	DECB				;-1
	BNE     LD410			;Bra to save and exit
;
;-VSS timed out, clear MPH and exit
;
LD3D8	STD     L0812			;Clear MPH
	BRA     LD410			;Bra to save and exit
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Here if VSS pulse recieved
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LD3DD	LDAA    L0811			;VSS timer
	BEQ     LD405			;Bra if =0, too few VSS pulses to update, perform resets
;
;-Calculate vehicle speed
;
	LDAA    #230			;Mult. for VSS
	MUL				;# VSS pulses recieved x 230
	ASLD				;x2
	XGDX				;Get result into X
	LDD     L080D			;Load VSS counter
	SUBD    L080F			;-VSS counter, one delayed
	XGDX				;Swap previous result into D and get VSS period into X
	FDIV				;fractional div., (# VSS pulses x 460) / VSS period
	LDD     L0812			;Current MPH
	CMPA    L4132			;Lower threshold for MPH
	XGDX				;Get result of FDIV into D
	BCC     LD402			;Bra if MPH where greater then threshold
;
;-MPH less then lower threshold here
;
	CMPA    L0812			;Compare calc'd mph to stored MPH
	BCS     LD402			;Bra if calc'd mph are less, save.
;
;-Calc'd mph where greater then stored mph
;
	INC     L0812			;+1, Current MPH
	BRA     LD405			;Bra to continue
;
LD402	STD     L0812			;Current MPH
;
;-Here to perform resets
;
LD405	CLR     L080C			;Clear # VSS pulses recieved counter
	LDX     L080D			;Load current VSS counter
	STX     L080F			;Save it, VSS counter, one delayed
	LDAB    #$27			;Max time to update VSS
;
LD410	STAB    L0811			;VSS pulse timer
	RTS 				;Return
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~    
;
;~~~~~~~~~~~~~~~~~~~~~~~
;
; MPH filtering routine
;
;~~~~~~~~~~~~~~~~~~~~~~~
;
LD414	LDD     L0812			;Load current MPH
	LDX     L02A7			;MPH
	LDY     #L4E84			;Filter coeff for MPH
	JSR     LF0D3			;Filt. routine
;
	STD     L02A7			;Save it, filtered MPH

	RTS				;Return   
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~  
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; Routine to determine desired gear #
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LD425	JSR     LE444			;Check for heat related transmission errors
;
	JSR     LE46C			;Check for trans related errors
;
	JSR     LE45C			;Check for external errors that can effect trans.
;
	CLRA				;
	STAA    L0080			;Clear status word
;											
	LDAB    L00D3			;Gear #
	CMPB    #03			;4th gear
	BCC     LD491			;Bra if in OD
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Check if upshift is desired
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
	LDX     #LD71E			;Idex of stored ALDL upshift points
	JSR     LD8FE			;Routine to look up value based on various params
;
	BEQ     LD444			;Bra if no value returned
;
;-ALDL shift points in use
;
	BSET    L0080,#$08		;Status word, set b3, ALDL upshift points in use
	BRA     LD478			;Bra to continue
;
;-No overdrive and no diag. mode, look up shift point based on range select
;
LD444	LDX     #LD6FA			;Range select upshift MPH threshold index tables
	JSR     LD8DC			;Do lookup
;											
	BCS     LD451			;Bra if null value returned
;
;-Range other then OD has been selected
;
	BSET    L0080,#$04		;Status word, set b2, manual upshift pattern active
	BRA     LD478			;Bra to continue				
;
LD451	BRCLR   L0096,#$10,LD46C	;Status word, bra if b4==0, no kick down pattern 
;
;-Kickdown pattern active
;
	LDY     #LD605			;Kickdown MPH upshift threshold index tables 
	JSR     LD854			;Perform shift point lookup
;
	BCS     LD46C			;Bra if valid shift point not returned 
;
;-Valid shift point returned
;
	PSHA				;Push shift point to stack    
	LDY     #LD61D			;Load kickdown upshift RPM index addr. 
	JSR     LD8B7			;Routine to generate upshift RPM threshold
;
	PULA				;Get shiftpoint back
	BSET    L0080,#$02		;Set b1, kickdown upshift pattern active
	BRA     LD47B			;Bra to continue
;
;-Kickdown pattern not active
;
LD46C	LDY     #LD605			;Index standard MPH upshift parameters
	JSR     LD7FB			;Routine to generate shift point
;
	BCS     LD491			;Bra if null value returned
;
	BSET    L0080,#$01		;Set b0, standard upshift pattern active
;
;-Here if in other gears and drive/manual pattern has been selected
;											
LD478	LDX     #$0000			;Clear X, no RPM shift limit
;
;-Here if RPM shift limit in use 
;
LD47B	LDAB    #$01			;Preload value to upshift
	CMPA    #$FF			;255
	BEQ     LD491			;Bra if no limit

	CMPA    L00D1			;MPH x 2
	BHI     LD491			;Bra if threshold for upshift > MPH
;
;-Shift threshold met
;
	BRCLR   L0093,#$20,LD48C	;
;
	LDX     #$0000			;Clear RPM shift threshold
;
;-Check RPM threshold to shift
;
LD48C	CPX     L019D			;RPM x 8
	BLS     LD4DA			;Bra if RPMs >=, threshold to upshift met
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Check if downshift is desired
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LD491	CLRA				;Preclear A    
	LDAB    L00D3			;Gear #
	DECB				;-1
	BMI     LD4D9			;Bra if in 1st
;
;-In other gears
;
	LDX     #LD743			;Index stord ALDL downshift points 
	JSR     LD8FE			;Routine to look item up based on gear #
;											
	BEQ     LD4A4			;Bra if null value returned
;
	BSET    L0080,#$80		;Status word, set b7 ALDL downshift points in use
	BRA     LD4CF			;
;
;-Look up shift point based on range select
;
LD4A4	LDX     #LD70C			;Index downshift point vs. range select
	JSR     LD8DC			;Routine to look up shift point based on range select
;											
	BCS     LD4B1			;Bra null value returned, or in OD, look up shift point
					;based on standard params. 

	BSET    L0080,#$40		;Set b6, manual pattern downshift pattern active
	BRA     LD4CF			;Bra to continue
;
;-Check if kickdown active
;
LD4B1	BRCLR   L0096,#$10,LD4C3	;Status word, bra if b4==0, no kickdown pattern active
;
;-Kickdown pattern active
;											
	LDY     #LD611			;Index kickdown MPH downshift params
	JSR     LD854			;Routine to construct shift point when kickdown 
					;pattern active
;											
	BCS     LD4C3			;Bra if null value returned
;
	BSET    L0080,#$20		;Set b5, kickdown downshift pattern active						
	BRA     LD4CF			;Bra to continue				
;
;-Here if kickdown pattern not active 
;
LD4C3	LDY     #LD611			;Index standard MPH downshift params
	JSR     LD7FB			;Routine to construct shift point
;
	BCS     LD4D9			;Bra if null value returned
;
	BSET    L0080,#$10		;Set b4, standard downshift pattern in use
;
;-Here to check threshold to downshift
;											
LD4CF	LDAB    #255			;Preload for downshift if MPH threshold met
	CMPA    #$00			;Value if threshold not in use
	BEQ     LD4D9			;Bra if ==0, MPH downshift threshold not in use
;
;-MPH downshift threshold in use
;
	CMPA    L00D1			;MPH x 2
	BHI     LD4DA			;Bra if higher
;
;-Here if in 1st or not downshifting
;
LD4D9	CLRB				;Clear value for downshift
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Here if threshold to shift has been met
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LD4DA	ADDB    L00D3			;Add in Gear #
	CMPB    L00D3			;Gear #
	BEQ     LD4E3			;Bra if no change in state
;
	STAA    L0180			;Save MPH shift threshold
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Check for heat related errors
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LD4E3	BRSET   L0083,#$01,LD4EB	;Bra if cold temperature threshold set
;
	BRCLR   L0092,#$80,LD4F1	;Bra if no heat related trans. errrors set
;
;-Heat related errors set here
;
LD4EB	CMPB    #$0002			;3rd gear
	BLS     LD4F1			;Bra if desired gear # <=
;
	LDAB    #$0002			;Load 3rd as default, no OD when heat related 
					;errors present
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Check for high system voltage
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LD4F1	BRCLR   L0093,#$08,LD4F7	;Bra if no high system voltage errors present
;
	LDAB    #$0002			;Load 3rd as default during errors
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Check for external errors here
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LD4F7	BRCLR   L0093,#$01,LD50F	;Status word, bra if b0==0, no ext. errors set
;
;-External errors set here
;
	CMPB    L00D3			;Compare desired gear # to current gear #
	BLS     LD505			;Bra if in lower or same gear
;
;-Upshift desired, prevent shifts into OD
;
	CMPB    #$0002			;3rd gear, default 
	BLS     LD50F			;Bra if desired gear # <=
;
	BRA     LD50D			;Bra to load current gear #, no OD
;
;-Downshift or current gear state desired here
;
LD505	CMPB    L00D3			;Compare desired gear # to current gear number
	BCC     LD50F			;Bra if same gear desired
;
;-Downshift desired, clamp to 3rd gear
;
	CMPB    #$0002			;Compare to 3rd gear, default
	BCC     LD50F			;Bra if desired gear # == 3rd
;
LD50D	LDAB    L00D3			;Load current gear #
;
;~~~~~~~~~~~~~~~~~~~~~~~~
; Check for shift errors 
;~~~~~~~~~~~~~~~~~~~~~~~~
;
LD50F	BRSET   L0092,#$08,LD517	;Status word, bra if b3==1, trans related errors
;
	BRCLR   L0093,#$02,LD519	;Status word, bra if b1==0, no shift errors set

LD517	LDAB    #$0002			;Load 3rd gear as default
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Check if output speed lost
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LD519	BRCLR   L0093,#$04,LD549	;Status word, bra if b2==0, output speed not lost
;
;-Output speed lost
;
	CMPB    L00D3			;Compare desired gear to current gear #
	BLS     LD527			;Bra if desired gear # <=
;
;-Upshift desired
;
	CMPB    #$0001			;Compare to 2nd gear
	BLS     LD549			;Bra if desired gear # <= 2nd gear
;
	BRA     LD547			;Bra to remain in current gear
;
;-Downshift desired or remaining in current gear here
;
LD527	CMPB    L00D3			;Compare to current gear #
	BCC     LD547			;Bra if desired gear # is == current gear #
;
	CMPB    #$01			;2nd gear
	BHI     LD549			;Bra if desired gear # is higher
;
	LDAA    #$0001			;
	CMPA    #$0001			;Place holder
	BNE     LD547			;
;
	LDAA    L00D3			;Load current gear #
	CMPA    #$02			;3rd gear
	BNE     LD547			;Bra if not in 3rd gear
;
;-Check RPM threshold to force second gear
;
	LDX     L019D			;Load RPM x 8
	CPX     L5D2D			;Upper RPM threshold to force second gear if output
					;speed lost
	BCC     LD547			;Bra if RPMs >=
;
	LDAB    #$0001			;2nd gear
	BRA     LD549			;Bra to continue
;
;-Here to remain in current gear
;
LD547	LDAB    L00D3			;Load current gear #
;
;~~~~~~~~~~~~~~~~~~~~~~
; Check for VSS errors
;~~~~~~~~~~~~~~~~~~~~~~
;
LD549	BRCLR   L0017,#$20,LD556	;Error word #2, bra if b5==0, no error 24
;
	LDAA    L5B2C			;Load error mask
	ANDA    #$20			;Clear all but b5
	BEQ     LD556			;Bra if ==0, not flagging error 24
;
	LDAB    #$01			;Load second gear as default if error 24
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Here if no errors detected, continue with shift dependant items
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LD556	BCLR    L007F,#$24		;Status word, preclear b5 and b2
	CMPB    L00D3			;Compare desired gear # to current gear #
	BEQ     LD567			;Bra if no change in state
;
	BCS     LD564			;Bra if desired gear # is <
;
	BSET    L007F,#$04		;Set b2, upshift desired
	BRA     LD567			;Bra to continue
;
LD564	BSET    L007F,#$20		;Set b5, downshift desired
;
;-Here if no change in state
;
LD567	BRCLR   L007F,#$20,LD57D	;Bra if b5==0, downshift not desired
;
;-Downshift desired here
;
	LDAA    L00D3			;Load gear #
	CMPA    #$02			;3rd gear 
	BNE     LD57D			;Bra if not in 3rd gear
;
;-Flag pending downshift into 2nd gear
;
	BRSET   L007F,#$40,LD589	;Status word, bra if b6==1, 3->2 downshift flagged
					;already
;
	BSET    L007F,#$40		;Set b6, 3->2 downshift underway
	BCLR    L007F,#$80		;Clear b7, 3->2 shift modulation in progress
	BRA     LD589			;Bra to continue
;
;-Here if upshift desired or remaining in current gear 
;
LD57D	BCLR    L007F,#$40		;Clear b6, no 3->2 shift
	LDAA    L0131			;Load 4-2 shift limiting timer
	INCA				;+1
	BEQ     LD589			;Bra if no roll-over
;
	STAA    L0131			;Save it
;
LD589	BRSET   L007F,#$04,LD5C2	;Bra if b2==1, upshift desired
;
	BRCLR   L007F,#$20,LD5C2	;Bra if b5==0, downshift not desired
;
;-Downshift desired, check and limit 4-2 shifts to prevent nusiance downshifts into 2nd
;
	LDAA    L00D3			;Load gear #
	CMPA    #$03			;4th gear
	BNE     LD59A			;Bra if not in 4th gear
;
	CLR     L0131			;In 4th gear, reset 
;
LD59A	BRCLR   L007F,#$40,LD5B8	;Bra if 3->2 shift not underway
;
;-3->2 shift underway, check if limiting needed on 4->2 shift
;
	LDAA    L0131			;4->2 shift limiting timer
	CMPA    L5D26			;Lower window bound for 4-2 shift limiting
	BLS     LD5B8			;Bra if time <=
;
	CMPA    L5D27			;Upper window bound for 4-2 shift limiting
	BCC     LD5B8			;Bra if time >
;
;-Limiting downshifts into 2nd from 4th gear
;
	LDAA    L0118			;Load 3->2 shift sol. diag. DC
	BNE     LD5B8			;Bra if !=0, diags. in effect
;
	LDAA    L0130			;3-2 shift modulation timer
	CMPA    L5D25			;Max time for 3-2 shift limiting
	BCS     LD5C0			;Bra if <, maintain current gear
;
;-Check if PWM TCC is being applied/released
;
LD5B8	LDAA    L0123			;Load TCC apply/release shift delay timer
	CMPA    L5D24			;Min time for shift after TCC fully released/applied
	BCC     LD5C2			;Bra if time threshold met 
;
;-Here to maintain current gear
;
LD5C0	LDAB    L00D3			;Load current gear number, TCC still being applied/
					;released or 3->2 shift modulation still in progress
;
LD5C2	TBA				;Get desired gear # into A     
	BCLR    L0081,#$20		;Status word, preclear b5, selected gear range 
					;has changed to drive flag
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Check if selected range has changed from OD to D, check for recent jump shift 
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;											
	BRCLR   L009C,#$04,LD5D6	;Status word, bra if b2==0, drive range not selected
;
;-Drive has been selected
;
	CMPB    #$03			;4th gear 
	BNE     LD5D6			;Bra if other gear desired
;
;-4th gear still desired
;
	TST     L018D			;Jump shift 2->4 timer
	BEQ     LD5D6			;Bra if ==0, no recent jump shifts
;
;-Recent 2->4 shift, prevent repeat 2->4 jump shift check 
;
	BSET    L0081,#$20		;Set b5, range has just changed to D
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Check desired gear # against that allowed by selected range
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LD5D6	JSR     LE322			;Check range routine
;
	BCLR    L007F,#$09		;Preclear b0, b3, shift status bits
	LDAB    L0103			;Mode 4 diag. gear #
	BEQ     LD5F4			;Bra if ==0, no diag. override in effect
;
;-Mode 4 trans diagnostics override in effect
;
	DECB				;-1  
	CMPB    #$03			;4th gear, max allowed
	BLS     LD5EC			;Bra if <=
;
;-Invalid gear indicated
;
	LDAB    #$04			;Load 4th gear
	STAB    L0103			;Save it
	DECB				;-1 for gear #
;   
LD5EC	LDAA    #255			;Load max shift threshold
	STAA    L0180			;Save it, MPH shift threshold
	TBA				;Get diag. gear # into A 
	BRA     LD602			;Bra to continue in specified gear
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~
; Check direction of shift
;~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LD5F4	CMPA    L00D3			;Compare desired gear to gear #
	BEQ     LD604			;Bra if in same gear
;
	BCS     LD5FF			;Bra if desired gear # <
;
;-Here if desired gear > current gear
;
	BSET    L007F,#$01		;Set b0, upshift underway
	BRA     LD602			;Bra to save 
;
;-Here if desired gear < current gear 
;
LD5FF	BSET    L007F,#$08		;Set b3, downshift underway
;
LD602	STAA    L00D3			;Save allowed gear as gear #
;
LD604	RTS				;Return   
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Kickdown/standard upshift params
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LD605   FDB LD6A4		;Index of base kickdown upshift MPH tables vs. various params
	FDB LD6C9		;Index of kickdown cold upshift modifier vs. gear #
	FDB LD799		;Index of kickdown baro shift point gain vs. various params 
	FDB LD623		;Index of TPS based upshift MPH tables vs. various params
	FDB LD648		;Index of cold engine TPS based upshift modifier tables vs. 
				;gear #
	FDB LD768		;Index of baro shift gain tables vs. gear #
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Kickdown/standard downshift params
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;		
LD611   FDB LD6CF		;Index of base kickdown downshift MPH tables vs. gear # and 
				;mode
	FDB LD6F4		;Index of kickdown cold downshift modifier vs. gear #
	FDB LD799		;Index of kickdown baro shift point gain vs. various params 
	FDB LD64E		;Index of TPS based downshift MPH tables vs. various params
	FDB LD673		;Index of cold engine TPS based downshift modifier tables vs.
				;gear #
	FDB LD768		;Index of baro shift gain tables vs. gear #
;		
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Kickdown pattern upshift RPM threshold params
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;		
LD61D   FDB LD679		;Upshift RPM threshold indexes
	FDB LD69E		;Cold shift modifier indexes
	FDB LD7CA		;Baro gain indexes
;		
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Upshift MPH tables vs. various params
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;	
LD623   FCB $06			;
;
;	Table addr.		;Gear shift
;
;-High trans temp		
;
	FDB $0000		;1 -> 2 
	FDB $0000		;2 -> 3 
	FDB $0000		;3 -> 4
;
;-Manual pattern
;
	FDB L5D57		;1 -> 2 
	FDB L5DCE		;2 -> 3 
	FDB L5DDF		;3 -> 4 
;
;-Cruise control mode
;  >Performance pattern
;
	FDB L5DBD		;1 -> 2
	FDB L5D79		;2 -> 3
	FDB L5D8A		;3 -> 4
;
;-Cruise control mode
;  >Standard pattern
;
	FDB L5DBD		;1 -> 2
	FDB L5D79		;2 -> 3
	FDB L5D8A		;3 -> 4
;
;-Standard mode
;  >Performance pattern
;
	FDB L5DBD		;1 -> 2
	FDB L5DCE		;2 -> 3
	FDB L5DDF		;3 -> 4
;
;-Standard mode
;  >Standard pattern
;
	FDB L5DBD		;1 -> 2
	FDB L5DCE		;2 -> 3
	FDB L5DDF		;3 -> 4		
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Cold upshift modifier vs. %TPS index
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; 	table addr.		;Shift
;
LD648   FDB L5E23   		;1 -> 2 
	FDB L5E34   		;2 -> 3 
	FDB L5E45   		;3 -> 4 
;
;		
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Downshift MPH tables vs. various params
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LD64E   FCB $06      		;6 bytes/set
;
;	Table addr.		;Gear shift
;
;-High trans temp		
;
	FDB $0000		;2 -> 1 
	FDB $0000		;3 -> 2 
	FDB $0000		;4 -> 3 
;
;-Manual pattern
;
	FDB L5D68		;2 -> 1 
	FDB L5E01		;3 -> 2 
	FDB L5E12		;4 -> 3  
;
;-Cruise control mode
;  >Performance pattern
;
	FDB L5DF0		;2 -> 1 
	FDB L5D9B		;3 -> 2  
	FDB L5DAC		;4 -> 3  
;
;-Cruise control mode
;  >Standard pattern
;
	FDB L5DF0		;2 -> 1 
	FDB L5D9B		;3 -> 2  
	FDB L5DAC		;4 -> 3  
;
;-Standard mode
;  >Performance pattern
;
	FDB L5DF0		;2 -> 1 
	FDB L5D9B		;3 -> 2  
	FDB L5DAC		;4 -> 3  
;
;-Standard mode
;  >Standard pattern
;
	FDB L5DF0		;2 -> 1 
	FDB L5D9B		;3 -> 2  
	FDB L5DAC		;4 -> 3   
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Cold downshift modifier vs. %TPS index
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; 	table addr.		;Shift
;
LD673   FDB L5E56		;  2 -> 1, COLD MOD SHIFT Vs. %TPS
	FDB L5E67		;  3 -> 2, COLD MOD SHIFT Vs. %TPS
	FDB L5E78		;  4 -> 3, COLD MOD SHIFT Vs. %TPS	
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Kickdown mode upshift RPM threshold params
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LD679   FCB $06			;6 bytes/set
;
;	Table addr.		;Gear shift
;
;-High trans temp
;
	FDB L5D4B		;1 -> 2 
	FDB L5D4D		;2 -> 3
	FDB L5D4F		;3 -> 4 
;
;-Manual pattern
;
	FDB $0000		;1 -> 2  
	FDB $0000		;2 -> 3 
	FDB $0000		;3 -> 4 
;
;-Cruise control mode
;   >Performance pattern 
;
	FDB L5D45		;1 -> 2 
	FDB L5D47		;2 -> 3 
	FDB L5D49		;3 -> 4 
;
;-Cruise control mode
;   >Standard pattern
;
	FDB L5D45		;1 -> 2
	FDB L5D47		;2 -> 3 
	FDB L5D49		;3 -> 4 
;
;-Standard mode
;   >Performance pattern
;
	FDB L5D45		;1 -> 2 
	FDB L5D47		;2 -> 3 
	FDB L5D49		;3 -> 4 
;
;-Standard mode
;   >Standard pattern
;
	FDB L5D45		;1 -> 2 
	FDB L5D47		;2 -> 3 
	FDB L5D49		;3 -> 4 
;		
;~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Cold upshift RPM modifier 
;~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;	Mod. index		;Gear shift
;		
LD69E   FDB L5D51		;1 -> 2 
	FDB L5D53		;2 -> 3 
	FDB L5D55		;3 -> 4 
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Base kickdown mode upshift MPH threshold params
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LD6A4   FCB $6			;6 bytes/set
;
;	Table addr.		;Gear shift
;
;-High trans temp		
;
	FDB L5D39		;1 -> 2 
	FDB L5D3A		;2 -> 3 
	FDB L5D3B		;3 -> 4 
;
;-Manual pattern
;
	FDB $0000		;1 -> 2 
	FDB $0000		;2 -> 3 
	FDB $0000		;3 -> 4 
;
;-Cruise control mode
;   >Performance pattern 
;
	FDB L5D33		;1 -> 2 
	FDB L5D34		;2 -> 3 
	FDB L5D35		;3 -> 4 
;
;-Cruise control mode
;   >Standard pattern
;
	FDB L5D33		;1 -> 2 
	FDB L5D34		;2 -> 3 
	FDB L5D35		;3 -> 4 
;
;-Standard mode
;   >Performance pattern
;
	FDB L5D33		;1 -> 2 
	FDB L5D34		;2 -> 3
	FDB L5D35		;3 -> 4 
;
;-Standard mode
;   >Standard pattern
;
	FDB L5D33		;1 -> 2 
	FDB L5D34		;2 -> 3 
	FDB L5D35		;3 -> 4 
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Kickdown cold upshift modifier vs. gear #
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;	MPH modifier		;Shift
;
LD6C9   FDB L5D3F		;1 -> 2 
	FDB L5D40		;2 -> 3  
	FDB L5D41		;3 -> 4  
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Base kickdown mode downshift MPH threshold params
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LD6CF   FCB $06			;6 bytes/set
;
;	Table addr.		;Gear shift
;
;-High trans temp		
;
	FDB L5D3C		;2 -> 1 
	FDB L5D3D		;3 -> 2 
	FDB L5D3E		;4 -> 3 
;
;-Manual pattern
;
	FDB $0000		;2 -> 1 
	FDB $0000		;3 -> 2 
	FDB $0000		;4 -> 3 
;
;-Cruise control mode
;   >Performance pattern 
;
	FDB L5D36		;2 -> 1
	FDB L5D37		;3 -> 2  
	FDB L5D38		;4 -> 3 
;
;-Cruise control mode
;   >Standard pattern
;
	FDB L5D36		;2 -> 1
	FDB L5D37		;3 -> 2 
	FDB L5D38		;4 -> 3 
;
;-Standard mode
;   >Performance pattern
;
	FDB L5D36		;2 -> 1 
	FDB L5D37		;3 -> 2 
	FDB L5D38		;4 -> 3 
;
;-Standard mode
;   >Standard pattern
;
	FDB L5D36		;2 -> 1 
	FDB L5D37		;3 -> 2 
	FDB L5D38		;4 -> 3 
;		
;--------------------------------------------------
; KICK DOWN MODE  THRESH TABLE
;
;--------------------------------------------------
;
LD6F4   FDB L5D42		; 2 -> 1  KICK DOWN MODE  THRESH TABLE
	FDB L5D43		; 3 -> 2  KICK DOWN MODE  Vs. GEAR TABLE, (COLD MODIFER)
	FDB L5D44		; 4 -> 3  KICK DOWN MODE  Vs. GEAR TABLE, (COLD MODIFER)
;
;--------------------------------------------------
;
; LOW PK/NEUT PARAMS MANUAL TABLE 
;
;--------------------------------------------------
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Range select upshift MPH index vs. gear #
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; Null value will default to standard/kickdown shift params instead
;
;-1st selected			;Current gear #
;
LD6FA   FDB L5D31		;1st
	FDB L5D30		;2nd
	FDB L5D30		;3rd
;
;-2nd selected
;
	FDB $0000		;1st
	FDB L5D30		;2nd
	FDB L5D30		;3rd
;
;-3rd selected
;
	FDB $0000		;1st
	FDB $0000		;2nd
	FDB $0000		;3rd
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Range select downshift MPH index vs. gear #
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; Null value will default to standard/kickdown shift params instead
;  
;-1st selected			;Current gear #
; 
LD70C   FDB L5D32		;1st 
	FDB L5D2F		;2nd
	FDB L5D2F		;3rd
;
;-2nd selected
;
	FDB $0000		;1st
	FDB L5D30		;2nd
	FDB L5D2F		;3rd 
;
;-3rd selected
;
	FDB $0000		;1st
	FDB $0000		;2nd
	FDB $0000		;3rd		
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Stored ALDL MPH upshift points
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;   
LD71E   FCB  $06		;3 address' per lookup	
;
;	Addr.			;Gear shift
;
;-High trans temp
;
	FDB  L0104		;1 -> 2
	FDB  L0105		;2 -> 3
	FDB  L0106		;3 -> 4
;
;-Manual pattern
;
	FDB  L0104		;1 -> 2
	FDB  L0105		;2 -> 3
	FDB  L0106		;3 -> 4
;
;-Cruise control mode
;   >Performance pattern 
;
	FDB  L0104		;1 -> 2
	FDB  L0105		;2 -> 3
	FDB  L0106		;3 -> 4
;
;-Cruise control mode
;   >Standard pattern
;
	FDB  L0104		;1 -> 2
	FDB  L0105		;2 -> 3
	FDB  L0106		;3 -> 4
;
;-Standard mode
;   >Performance pattern
;
	FDB  L0104		;1 -> 2
	FDB  L0105		;2 -> 3
	FDB  L0106		;3 -> 4
;
;-Standard mode
;   >Standard pattern
;
	FDB  L0104		;1 -> 2
	FDB  L0105		;2 -> 3
	FDB  L0106		;3 -> 4
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Stored ALDL MPH downshift points
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LD743   FCB $06			;6 line address offset for each lookup
;
;	Addr,			;Gear shift
;
;-High trans temp
;

LD744   FDB L0107		;2 -> 1
	FDB L0108      		;3 -> 2
	FDB L0109		;4 -> 3
;
;-Manual pattern
;
	FDB L0107     		;2 -> 1
	FDB L0108      		;3 -> 2
	FDB L0109      		;4 -> 3
;
;-Cruise control mode
;   >Performance pattern 
;
	FDB L0107     		;2 -> 1
	FDB L0108      		;3 -> 2
	FDB L0109      		;4 -> 3
;
;-Cruise control mode
;   >Standard pattern
;
	FDB L0107     		;2 -> 1
	FDB L0108      		;3 -> 2
	FDB L0109      		;4 -> 3
;
;-Standard mode
;   >Performance pattern
;
	FDB L0107     		;2 -> 1
	FDB L0108      		;3 -> 2
	FDB L0109      		;4 -> 3
;
;-Standard mode
;   >Standard pattern
;
	FDB L0107     		;2 -> 1
	FDB L0108      		;3 -> 2
	FDB L0109      		;4 -> 3
;		
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Baro MPH shift gain tables vs. various params
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LD768   FCB $08			;8 bytes/block
;
;	Gain index		;Gear #
;
;-High trans temp		
;
	FDB L5D2C		;1st
	FDB L5D2C		;2nd
	FDB L5D2C		;3rd
	FDB L5D2C		;4th
;
;-Manual pattern
;
	FDB $0000		;1st
	FDB $0000		;2nd
	FDB $0000		;3rd
	FDB $0000		;4th
;
;-Cruise control mode
;   >Performance pattern 
;
	FDB L5D2C		;1st
	FDB L5D2C		;2nd
	FDB L5D2C		;3rd
	FDB L5D2C		;4th
;
;-Cruise control mode
;   >Standard pattern
;
	FDB L5D2C		;1st
	FDB L5D2C		;2nd
	FDB L5D2C		;3rd
	FDB L5D2C		;4th
;
;-Standard mode
;   >Performance pattern
;
	FDB L5D2C		;1st
	FDB L5D2C		;2nd
	FDB L5D2C		;3rd
	FDB L5D2C		;4th
;
;-Standard mode
;   >Standard pattern
;
	FDB L5D2C		;1st
	FDB L5D2C		;2nd
	FDB L5D2C		;3rd
	FDB L5D2C		;4th
;		
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-kickdown TPS based baro shift gain vs. various params
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LD799   FCB $08			;8 bytes/block
;
;	Gain index		;Gear #
;
;-High trans temp		
;
	FDB L5D2C		;1st
	FDB L5D2C		;2nd
	FDB L5D2C		;3rd
	FDB L5D2C		;4th
;
;-Manual pattern
;
	FDB $0000		;1st
	FDB $0000		;2nd
	FDB $0000		;3rd
	FDB $0000		;4th
;
;-Cruise control mode
;   >Performance pattern 
;
	FDB L5D2C		;1st
	FDB L5D2C		;2nd
	FDB L5D2C		;3rd
	FDB L5D2C		;4th
;
;-Cruise control mode
;   >Standard pattern
;
	FDB L5D2C		;1st
	FDB L5D2C		;2nd
	FDB L5D2C		;3rd
	FDB L5D2C		;4th
;
;-Standard mode
;   >Performance pattern
;
	FDB L5D2C		;1st
	FDB L5D2C		;2nd
	FDB L5D2C		;3rd
	FDB L5D2C		;4th
;
;-Standard mode
;   >Standard pattern
;
	FDB L5D2C		;1st
	FDB L5D2C		;2nd
	FDB L5D2C		;3rd
	FDB L5D2C		;4th
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Kickdown baro shift RPM baro gain vs. various params
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LD7CA   FCB $08			;8 bytes/block
;
;	Gain index		;Gear #
;
;-High trans temp		
;
	FDB L5D2C		;1st
	FDB L5D2C		;2nd
	FDB L5D2C		;3rd
	FDB L5D2C		;4th
;
;-Manual pattern
;
	FDB $0000		;1st
	FDB $0000		;2nd
	FDB $0000		;3rd
	FDB $0000		;4th
;
;-Cruise control mode
;   >Performance pattern 
;
	FDB L5D2C		;1st
	FDB L5D2C		;2nd
	FDB L5D2C		;3rd
	FDB L5D2C		;4th
;
;-Cruise control mode
;   >Standard pattern
;
	FDB L5D2C		;1st
	FDB L5D2C		;2nd
	FDB L5D2C		;3rd
	FDB L5D2C		;4th
;
;-Standard mode
;   >Performance pattern
;
	FDB L5D2C		;1st
	FDB L5D2C		;2nd
	FDB L5D2C		;3rd
	FDB L5D2C		;4th
;
;-Standard mode
;   >Standard pattern
;
	FDB L5D2C		;1st
	FDB L5D2C		;2nd
	FDB L5D2C		;3rd
	FDB L5D2C		;4th
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; Routine to construct shift point (kickdown mode not active) 
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LD7FB	PSHB				;Push gear # to stack
	CLRA				;Preclear A
	PSHA				;Allocate space on stack
	PSHA				;
	PSHA				;
	LDX     $08,Y			;Load index for cold shift modifier vs. TPS										;
	ABX				;Add in gear # for offset
	ABX				;x2 for double byte indexing
	LDX     0,X			;Load in address from cal.
	BEQ     LD81B			;Bra if ==0, table not in use
;
	LDAA    L00AC			;Load trans TPS
	JSR     LF15E			;2D lookup
;											
	SUBA    #128			;-128 for corrections centered around 128
	LDAB    0,X			;Load base value
	SUBB    #128			;-128 for corrections centered around 128
	TSX				;Get current stack pointer
	STAB    2,X			;Save base cold shift modifier
	STAA    0,X			;Save cold shift modifier
	LDAB    3,X			;Get gear # back
;
;-Here if cold shift modifier table not in use
;
LD81B	LDX     6,Y			;Load start of shift point vs. TPS index
	JSR     LD90A			;Routine to look up indexed address based on 
					;various params
;										
	LDAA    #$00			;Preclear A
	BCC     LD82A			;Bra if carry ==0, table in use
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Shift point table not in use, clean up stack and return
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
	PULB				;Clean up stack
	PULB				;
	PULB				;    					
	BRA     LD852			;Bra to return
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Here if valid address returned for shift point calibration
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LD82A	LDAA    L00AC			;Load trans TPS
	JSR     LF15E			;2D lookup
;
	LDAB    0,X			;Load base value from shift point calibration
	TSX				;Get current stack pointer     
	STAB    1,X			;Store it to stack
	CLRB				;Preclear LSB    
	XGDX				;Swap shiftpoint into X    
	PULA				;Pull cold shift modifier from stack    
	CLRB				;Clear LSB    
	JSR     LD984			;Routine to apply temp based modifiers
;
	LDX     $0A,Y			;Baro shift gain index
	JSR     LD934			;Routine to apply baro shift point gain
;

;-Compare finalized shift point to base closed throttle shift point
;
	XGDX 				;Swap finalized shift point into X   
	PULA				;Pull base shift point value  
	CLRB				;Preclear LSB    
	XGDX				;Get finalized shift point back and base shift 
					;point into X 
	TAB				;Transfer to B     
	PULA				;Get base cold shift modifier   
	PSHB				;Push finalized shift point to stack    
	CLRB				;Preclear B 
	JSR     LD984			;Routine to apply cold engine shift modifier
;
	PULB				;Get finalized shift point back
	CBA				;Compare to base closed throttle shift point     	
	BCC     LD852			;Bra if base shift point is higher
;
;-Finalized shiftpoint was greater then base shift point
;
	TBA				;Get finalized shift point back
	CLC				;Clear status of carry bit before returning
;    
LD852	PULB				;Get gear # back  
	RTS				;Return     
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; Routine to construct shift point when kickdown pattern active
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LD854	PSHB				;Gear # to stack
	CLRA				;    
	PSHA				;Allocate space on stack
	PSHA				;    
	PSHA				;
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Look up cold engine shift modifier
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;    
	LDX     2,Y			;Load addr., points to kickdown cold modifier table
	ABX				;Add in gear #     
	ABX				;Add in gear #, x2 for double byte addr.     
	LDX     0,X			;Load addr.
	BEQ     LD869			;Bra if ==0, table not in use
;
;-Table in use here
;
	LDAA    0,X			;Load in cold shift modifer
	SUBA    #$0080			;Make signed value
	TSX				;Get current stack pointer into X
	STAA    0,X			;Save modifer to stack
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Look up cold shift modifier vs. TPS
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LD869	LDX     $0008,Y			;Load table addr., point to cold shift modifier vs %TPS
					;table
	ABX				;Add in gear # as offset     
	ABX				;Add in gear #, x2 for double byte addr.     
	LDX     0,X			;Load table addr.
	BEQ     LD879			;Bra if table not in use
;
;-Table in use here
;
	LDAA    0,X			;Load in first shift mod. value
	SUBA    #$0080			;Make value signed
	TSX				;Get current stack pointer     
	STAA    2,X			;Save to stack
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Look up shift point based on TPS
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LD879	LDX     $0006,Y			;Load addr., points to shift point vs %TPS
	JSR     LD90A			;Go do lookup
;
	BCS     LD886			;Bra if table not in use
;
	LDAA    0,X			;Load in first shift point calib. value
	TSX				;Get current stack pointer      
	STAA    1,X			;Save to stack
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Look up base MPH threshold to perform shift
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LD886	LDX     0,Y			;Load addr., base MPH shift threshold
	JSR     LD90A			;Go do lookup
;
	LDAA    #$0000			;Preclear A
	BCC     LD895			;Bra if table in use
;
;-Here to clean up stack and return if table not in use
;
	PULB				;   
	PULB				;    
	PULB 				;   
	BRA     LD8B5			;Bra to clean up stack and return
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Apply modifiers and gain to base shift MPH threshold
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LD895	LDAA    0,X			;Load base MPH shift threshold from table
	CLRB				;Clear LSB    
	XGDX				;Swap X and D    
	PULA				;Pull cold engine shift modifer from stack   
	CLRB				;Clear LSB    
	JSR     LD984			;Routine to apply cold engine shift modifier 
;
	LDX     4,Y			;Addr. pointing to baro shift point gain
	JSR     LD934			;Routine to apply baro shift point gain
;
	XGDX				;Swap baro modified shift point into X    
	PULA				;Get TPS MPH shift calibraiton off stack    
	CLRB				;Clear B    
	XGDX				;Swap into X, getting baro modified shiftpoint back    
	TAB				;Get result into B     
	PULA				;Get TPS cold engine MPH shift modifier off stack   
	PSHB				;Baro modified shift point to stack    
	CLRB				;    
	JSR     LD984			;Routine to apply cold engine shift modifier
;
	PULB				;Get baro modified shift point off of stack	 		   
	CBA				;Compare finalized shift point to baro modified 
					;shift point     
	BCC     LD8B5			;Bra if finalized shift point is higher
;
;-Baro modified shift point was higher then TPS derived shiftpoint
;
	TBA				;Get baro shift point into A    
	CLC				;Clear carry	
;    
LD8B5	PULB				;Get gear # back   
	RTS				;Return 
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~    
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; Routine to look up upshift RPM threshold 
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LD8B7	PSHB				;Push gear number to stack    
	LDX     2,Y			;Load addr. pointing to cold shift RPM modifier 
					;pointers
	ABX				;Add in gear number x 2 for double byte addressing     
	ABX 				;    
	LDX     0,X			;Load addr. pointing to table addr.
	BEQ     LD8C3			;Bra if null, table not in use
;
	LDX     0,X			;Load value from calibration section
;
LD8C3	PSHX				;Push addr. to stack    
	LDX     0,Y			;Load addr. pointing to upshift RPM threshold params
	JSR     LD90A			;Routine to perform lookup based on various params
;
	BCS     LD8CE			;Bra if null addr. returned
;
	LDX     0,X			;Load shift RPM threshold from cal. section
;
LD8CE	PULA				;Get cold shift RPM modifier into AB 	   
	PULB				;   
	JSR     LD984			;Routine to apply temp. based modifier
;
	LDX     4,Y			;Load base baro gain in
	JSR     LD934			;Rotuine to apply baro shift gain
;
	XGDX				;Swap result into X    
	PULB				;Get gear # back off stack    
	RTS				;Return
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~     
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; Routine to look up shift point based on range select
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;-Table lookup order
; 
; >1st
; >2nd
; >3rd
;
LD8DC	PSHB				;Gear # to stack    
	BRSET   L0092,#$04,LD8FA	;Status word, bra if b2==1, pressure swich man. error
;
;-No error set here
;
	LDAA    L009C			;Range select word
	ANDA    #$07			;Clear all but b0-b2
	BEQ     LD8FA			;Bra if other gears selected
;
;-Here if 1st/2nd/Drive selected
;
	LSRA				;Logical shift right, only 1st, 2nd   
	PSHB				;Push gear # to stack    
	LDAB    #$0003			;3 lines/table
	MUL				;3 x range select status     
	PULA				;Get gear number back in    
	ABA				;Add it in for lookup     
	ASLA				;x2 for double byte vals.  
	TAB				;Get result into B     
	ABX				;Add in as offset for lookup
	LDX     0,X			;Load table addr.
	BEQ     LD8FA			;Bra if addr. ==0
;
;-Table addr. in use
;
	LDAA    0,X			;Load shiftpoint from table
	CLC				;Clear C bit     
	BRA     LD8FC			;Bra to return
;
;-Here if in OD/P/N/R selected or table not in use
;
LD8FA	CLRA				;Clear A
	SEC				;Set C bit, table not in use
;
LD8FC	PULB				;Get gear number
	RTS				;Return
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~     
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; Routine to look item up based on gear #
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LD8FE	PSHB				;Gear # to stack
	BSR     LD90A			;Routine to fetch address
;											
	BCC     LD906			;Bra if looked up addr. !=0
;
;-Null address
;
	CLRA				;Clear A  
	BRA     LD908			;Bra to return
;
;-Valid address returned
;
LD906	LDAA    0,X			;Load A from addr.
;
LD908	PULB				;Get gear # back
	RTS				;Return     
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; Routine to look up indexed address based on various params
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; *TABLE ORDER*
; 
; -High trans temp
; -Manual pattern
; -Cruise control mode
;  >Performance pattern
;  >Standard pattern
; -Standard mode
;  >Performance pattern
;  >Standard pattern
;
;
LD90A	PSHB				;Gear # to stack	
	LDAA    0,X			;Load in bytes per table for lookups from table
	INX				;+1, next address
	ASLB				;gear # x 2
	ABX				;Add in gear number as offset
	TAB				;Offset into B											;
	BRCLR   L0083,#$02,LD91B	;Temp. status word, bra if b1==0, no hot 
					;trans temp flagged 
;
;-Here if hot trans temp threshold flagged
;
	LDAA    0,X			;Load in table addr.
	ORAA    1,X			;
	BNE     LD92C			;Bra if !=0, valid address specified
;
LD91B	ABX				;Add in offset
	BRSET   L0099,#$04,LD92C	;Shift pattern word,
					;bra if b2==1, manual pattern requested
;
;-Manual pattern not requested, check if cruise cont'l is active
;
	ABX				;Add in # of bytes/table as offset
	BRSET   L0096,#$40,LD927	;Status word, bra if b6==1, cruise cont'l active
;
;-Cruise control not active
;
	ABX				;Add in # of bytes/table as offset
	ABX				;
;
LD927	BRSET   L0099,#$02,LD92C	;Bra if b1==1, peformance pattern requested
;
;-Performance pattern not requested
;
	ABX				;Add in offset
;
LD92C	CLC				;Clear carry
	LDX     0,X			;Load in address values
	BNE     LD932			;Bra if !=0
;
	SEC				;Set carry, looked up addr ==0
;
LD932	PULB				;Get gear # back
;
	RTS				;Return     					
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; Routine to apply baro shift gain
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~	
;										
LD934	PSHB				;Push value to be modified to stack
	PSHA				;
	LDAB    L00D3			;Load in gear # 
	JSR     LD90A			;Routine look up indexed address 
;											
	BCS     LD94C			;Bra if null address, exit
;	
;-Valid address returned
;										
	JSR     LD94F			;Routine to generate baro gain factor
;											
	TSX				;Get current stack pointer
	JSR     LF1ED			;(A x X*)/256
;											
	ASLD				;x2, now /128
	BCC     LD94A			;Bra if no overflow
;
	LDD     #$FFFF			;Clear overflow
;
LD94A	PULX				;Clean up stack
	RTS				;Return    
;
;-Here if null value returned
;
LD94C	PULA				;Clean up stack    	
	PULB				;    
	RTS				;Return    
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; Routine to construct baro shift gain 
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LD94F	CLRA				;Preclear A    
	LDAB    0,X			;Load value from addr.
	SUBB    #$0080			;Make signed value
	PSHB				;Push signed value to stack    
	BPL     LD959			;Bra if result is positive
;
	NEGB				;Negative result, get abs. value   
	DECA				;Dec A, negative result was returned   
;
LD959	PSHA				;Push value to stack for future reference  
	LDAA    L00A6			;Load trans baro
	MUL				;Trans baro x base shift baro gain     
	ASLD				;x2, now /128    
	TSX				;Get current stack pointer     
	TST     0,X			;Test value on stack
	BEQ     LD967			;Bra if ==0. positive result was returned
;
;-Negative multiplier here
;
	NEGA				;Negate, make negative value    
	NEGB				;    
	SBCA    #$0000			;Perform carry if needed
;
;-Apply gain to baro and center final baro multiplier around 128
;
LD967	SUBA    1,X			;Subtract abs. value of multiplier
	ADDA    #$0080			;Add 128 to result
	PULX				;Clean up stack    
	RTS				;Return
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~     
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; Routine to generate baro gain 
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LD96D	LDAA    L00A6			;Load trans baro
	SUBA    #$0080			;100 kPa offset, reference corrections around sea
					;level
	LDAB    0,X			;Load baro gain factor
	MUL				;Trans baro x gain factor     
	TST     L00A6			;Test trans baro
	BMI     LD97B			;Bra if baro >= 100 kPa
;
	SUBA    0,X			;Add gain back in to make result negative
;
LD97B	ADDA    #$0040			;Make final corrections centered around 128
	BGT     LD982			;Bra if non-zero result
;
	LDD     #$0000			;Clear result
;
LD982	ASLD				;x2, now corrections centered around 128    
	RTS				;Return
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~     
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; Routine to apply temp based modifiers
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; In:
;   X = term to be modified
;   AB = modifier
;
; Out:
;    X = original term to be modified
;    AB = modified term
;
LD984	PSHX				;Term to be mod. to stack    
	PSHA				;Push modifier to stack    
	TSTA				;Test for sign    
	BPL     LD98D			;Bra if >= 0
;
;-Modifier is <= 0 here, get abs. value
;
	NEGA				;Make abs. value   
	NEGB				;    
	SBCA    #$0000			;Perform carry if needed
;
;-Modifer >= 0 here
;
LD98D	PSHB				;Push LSB of modifier to stack 			   
	PSHA				;Push MSB of modito stack    
	TSX				;Get current stack pointer     
	LDAA    L00A5			;Cold temp trans term
	JSR     LF1ED			;(A x X*)/256
;
	PULX				;Clean up stack by pulling |modifier| 
	TSX				;Get current stack pointer     
	TST     0,X			;Test sign of modifier
	BPL     LD99F			;Bra if plus
;
;-Modifier is negative here
;
	NEGA				;Make result negative				    
	NEGB				;    
	SBCA    #$0000			;Perform carry if needed
;
;-Here if positive modifier
;
LD99F	INS				;+1 to stack pointer			     
	PSHA				;Push MSB of result to stack    
	ADDD    1,X			;Add to origional 16 bit value to be modified on stack
	BCS     LD9AF			;Bra if overflow
;
;-Filter out positive values
;
	PSHA				;Push MSB of final value to stack    
	ANDA    0,X			;And final value with previous result
	PULA				;Get MSB of final value back    
	BPL     LD9B8			;Bra if both values arnt negative
;
;-Clear underflow here
;
	CLRA				;Clear result   
	CLRB				;    
	BRA     LD9B8			;Bra to return
;
;-Filter out change of sign carries
;
LD9AF	PSHA				;    
	ORAA    0,X			;
	PULA				;    
	BMI     LD9B8			;Bra if either value negative
;
;-Clear overflow
;
	LDD     #$FFFF			;Clear overflow
;
;-Clean up the stack
;
LD9B8	INS				;+1, stack pointer     
	PULX				;Pull X from stack
	RTS				;Return     
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;~~~~~~~~~~~~~~~~~~~~~~~
;
; Routine to handle MAT 
;
;~~~~~~~~~~~~~~~~~~~~~~~
;
LD9BB	LDX     #L3000			;Base CPU hardware addr.
	SEI				;Disable interrrupts     
	LDAA    #$04			;Select MAT
	JSR     LEF12			;Port D config routine
;
	LDAA    #$01			;
	JSR     LEEFC			;A/D routine
	CLI				;Re-enable interrupts     
	LDAA    $34,X			;Channel 4, get A/D MAT
	COMA				;Ones component, invert value
	STAA    L0253			;Save it, A/D MAT
;
	LDX     L00F8			;Engine run time
	CPX     L4E3B			;Min engine run time before performing MAT diags.
	BLS     LDA32			;Bra if engine run time is <=
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Perform code 23 diags., low MAT
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
	LDAB    L02A7			;MPH
	LDAA    L0253			;A/D MAT
	LDX     #L5B01			;Error mask #2
	BRCLR   0,X,#$40,LD9FE		;Bra if b6==0, not setting err 23, low MAT
;
	BCLR    L0017,#$40		;Error word #2, clear b6, error 23 flag
;
	CMPA    L4E37			;MAT threshold to set error 23 
	BCC     LD9FE			;Bra if MAT is >=, MAT ok
;
;-MAT is below threshold to set error, check time 
;
	LDAA    L0240			;Error 23 counter
	CMPA    L4E38			;Time threshold to set error
	BHI     LDA03			;Bra if counter >, set error
;
;-Check MPH
;
	CMPB    L4E3D			;MPH boundry for error 23/25
	BHI     LD9FE			;Bra if MPH >, dont set code yet
;
;-Inc counter
;
	INC     L0240			;+1, error 23 counter
	BRA     LDA06			;Bra to continue
;
;-Here if not setting error 
;
LD9FE	CLR     L0240			;Clear error 23 counter
	BRA     LDA06			;Bra to continue
;
;-Here to set error
;
LDA03	BSET    L0017,#$40		;Set b6, error 23, low MAT
;	
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Perform code 25 diags., high MAT
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;						
LDA06	LDX     #L5B01			;Error mask #2
	BRCLR   0,X,#$10,LDA2A		;Bra if b4==0, not setting error 25, high MAT
;
	BCLR    L0017,#$10		;Clear b4, error 25, high MAT
;
	LDAA    L0253			;Load A/D MAT
	CMPA    L4E3F			;Upper temp threshold to set error
	BCS     LDA2A			;Bra if MAT <=, MAT ok
;
	LDAA    L0241			;Error 25
	CMPA    L4E40			;Time threshold to set error
	BHI     LDA2F			;Bra if counter >, set error
;
	CMPB    L4E3D			;MPH boundry for error 23/25
	BLS     LDA2A			;Bra if MPH are <=, dont set error
;
;-Inc counter
; 
	INC     L0241			;+1, error 25 counter
	BRA     LDA32			;Bra to continue
;
;-Here if not setting error
;
LDA2A	CLR     L0241			;Clear error 25 counter
	BRA     LDA32			;Bra to continue
;
;-Here to set error
;
LDA2F	BSET    L0017,#$10		;Set b4, error 25, high MAT
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Perform lookup to linearize MAT
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LDA32	LDAA    L0253			;A/D MAT
	LDX     #LFAEF			;MAT temp vs. A/D MAT
	JSR     LF15E			;2D lookup
;
	STAA    L0855			;Save it, MAT, linearized
;
;-Check if errors set
;
	BRCLR   L0017,#$50,LDA45	;Bra if b4 and b6==0, no errors set 
;
;-Errors set, load default MAT
;
	LDAA    L4E3E			;Default MAT if errors set
;
LDA45	STAA    L0252			;MAT temp
	RTS				;Return 
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~    
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; Engine error code/diagnostics handling routine
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LDA49	BSET    L006F,#$80		;Status word, pre-set b7, one second has elapsed
;									
	BRCLR   L0002,#$F0,LDA53	;Loop counter, bra if upper bits clear, one second
					;has elapsed
;
;-One second has not yet passed
;
	BCLR    L006F,#$80		;Clear b7, one second has not yet elasped 
;											
LDA53	BRSET   L0050,#$10,LDA60	;Status word, bra if b4==1, in ALDL diags.
;								
	BCLR    L0071,#$80		;Status word, clear b7, diags not in effect
	BCLR    L0043,#$1C		;Status word, clear b4, fueling state change 
					;flag, b3, SES fueling diags. in effect, and b2,
					;SES in use to relay fueling information   
	JMP     LDAB5			;Bra to continue
;
;~~~~~~~~~~~~~~~~~~~~~~
; ALDL diags in effect
;~~~~~~~~~~~~~~~~~~~~~~
;
LDA60	BSET    L0071,#$80		;Status word, set b7, ALDL diags. in effect
;
	CLR     L024E			;Clear error log timer
	BRSET   L004F,#$80,LDA70	;Status word, bra if b7==1, engine is running
;
;-Engine not running 
;
	BCLR    L0043,#$1C		;Status word, clear b4, fueling state change 
					;flag, b3, SES fueling diags. in effect, and b2,
					;SES in use to relay fueling information 
	JMP     LDAC6			;Jump to continue
;
;-Engine is running here
;
LDA70	BRSET   L003E,#$80,LDA82	;Status word, bra if b7==1, in closed loop
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; In open loop toggle SES light @ 1hZ
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LDA74	LDAA    L0043			;Load status word
	ANDA    #$F7			;Clear b4, change in fueling state
	EORA    #$40			;Toggle b6, used to toggle SES on/off 
	STAA    L0043			;Save it
	BITA    #$40			;Test b6
	BNE     LDAAA			;Bra if ==1 dont toggle SES this pass
;
	BRA     LDA9E			;Toggle SES
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; In closed loop, toggle SES light when rich/lean
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LDA82	BRSET   L0043,#$08,LDA93	;Status word bra if b3==1, ALDL fueling
					;diagnostics previously active
;
;-Diags not yet active
;
	BRCLR   L006F,#$80,LDA74	;Bra if b7==0, one second has not yet elapsed,
					;start toggling SES light in mean time
;
;-One second has passed, initiate diags.
;
	BSET    L0043,#$08		;Set b3, diags now in effect

LDA8D	BRSET   L003E,#$40,LDAA2	;Status word, bra if b6==1, going richer

	BRA     LDAA7			;Bra to continue
;
;-Here if diags previously active
;
LDA93	BRCLR   L006F,#$80,LDAAA	;Bra if b7==0, one second has not passed, dont 
					;update SES light
;
	BRCLR   L0043,#$10,LDA8D	;Status word, bra if b4==0, no change in fueling
					;state
;
;-fUeling state has changed, toggle SES
;
	BCLR    L0043,#$10		;Clear b4, fueling state change flag
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Entry point to toggle SES
;~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LDA9E	BRSET   L0072,#$80,LDAA7	;Status word, set b7, turn on SES light
;
;~~~~~~~~~~~~~~~~~~~~~~~~~
;-Fueling is going richer
;~~~~~~~~~~~~~~~~~~~~~~~~~
;
LDAA2	BSET    L0072,#$80		;Status word, set b7, turn on SES
	BRA     LDAAA			;Bra to continue
;
;~~~~~~~~~~~~~~~~~~~~~~~~~
; Fueling is going leaner
;~~~~~~~~~~~~~~~~~~~~~~~~~
;
LDAA7	BCLR    L0072,#$80		;Clear b7, turn off SES
;
;-Here if one second has not yet passed and initiating SES fueling diags, perform clears 
;
LDAAA	CLRA				;Pre-clear A    
	STAA    L003C			;Clear status word, SES in use for fueling diags
	STAA    L0239			;Clear current error number, SES in use for
					;fueling diags.
	BSET    L0043,#$04		;Status word, set b2, SES in use to relay
					;state of fueling
	BRA     LDAD4			;Bra to continue
;
;-Here if no ALDL diags, perform clears
;
LDAB5	CLRA				;Pre-clear A      
	STAA    L003C			;Clear status word, SES in use for fueling diags
	STAA    L0239			;Clear current error number, SES in use for
	BRCLR   L001A,#$20,LDAC2	;Error word #5, bra if b5==0, no PROM error
;
;-Error 51 in effect
;
	JMP     LDFA8			;Turn on SES and return
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Check if engine is running
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LDAC2	BRSET   L004F,#$80,LDAD4	;Status word, bra if b7==1, engine running
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Here if engine off perform clears
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LDAC6	CLRA				;Preclear A    
	CLRB				;Preclear B    
	STAA    L024B			;Error 33 timer
	STAA    L023C			;Error 44 timer
	STD     L023D			;Error 45 timer
	JMP     LDEDA			;Continue
;
;~~~~~~~~~~~~~~~~~~~~~~~~
; Here if engine running
;~~~~~~~~~~~~~~~~~~~~~~~~
;
LDAD4	BRSET   L0043,#$02,LDB00	;Status word, bra if b1==1, engine run time > 
					;10 secs 
;
	LDAB    L00F9			;LSB, engine run time
	CMPB    #10			;10 secs 
	BCS     LDB00			;Bra if run time is less
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Engine has been running for more than 10 secs
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
	BSET    L0043,#$02		;Set b1, engine run time > 10 secs
	INC     L0014			;# of error free startups 
	DEC     L0015			;Malf. word checksum
;
;-Check if enough error free startups have passed to clear error words
;
	LDAA    L0014			;# of error free startups
	CMPA    L5B12			;# of error free startups to clear error words
	BLS     LDB00			;Bra if <= threshold to clear errors
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Enough error free starts have passed to clear error codes
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
	CLRA				;Preclear A    
	LDX     #$0009			;9 error words
;
LDAF2	STAA    $0A,X			;Clear error word
	DEX				;-1, move to next error word     
	BNE     LDAF2			;Bra if !=0, continue clearing error words
;
	STAA    L0014			;Clear # error free startups
	STAA    L003C			;Clear status word
	JSR     LEFC3			;RAM checksum routine
;
	STAA    L0015			;Malf. word checksum
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Check quals for error 13, O2 sensor diags
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LDB00	BCLR    L0016,#$80		;Error word #1, clear b7, err 13, O2 sensor
;
;-Check if TPS errors
;				
	LDAB    L006F			;Status word
	BITB    #$24			;Test b2, high TPS, and b5, low TPS errors
	BNE     LDB72			;Bra if error(s) set
;
;-Check if engine running long enough
;
	BRSET   L0071,#$40,LDB18	;Status word, bra if b6==1, run time threshold
					;met for O2 diags
;
	LDD     L00F8			;Engine run time
	LSRD				;/2
	CMPB    L4E28			;Run time for err 13, O2 sens. diags.
	BCS     LDB72			;Bra if run time is <, no diags.
;
	BSET    L0071,#$40		;Set b6, run time threshold for O2 diags. met
;
;-Check if DFCO in effect
;
LDB18	LDAA    L0158			;Time DFCO active, O2 diags
	BRCLR   L0046,#$08,LDB29	;Status word, bra if b3==0, not in DFCO  
;
	CMPA    L4E29 			;Upper limit for DFCO active timer											; --- TIME ACCUM
	BCC     LDB72			;Bra if timer >=, no O2 diags
;
	INC     L0158			;Inc timer
	BRA     LDB72			;Bra to continue
;
LDB29	TSTA				;Test timer    
	BEQ     LDB2D			;Bra if DFCO active timer ==0
;
	DECA				;-1, decay timer
; 		   
LDB2D	STAA    L0158			;Save timer
	CMPA    L4E2A			;Max time in DFCO for O2 diags.
	BCC     LDB72			;Bra if >=, no O2 diags.
;
;-Check if O2 volts dwelling around .450 volts
;
	LDAB    L01F8			;A/D O2 volts
	CMPB    L4E2C			;Upper window bound for O2 error
	BHI     LDB72			;Bra if O2 volts >, O2 functional 
;
	CMPB    L4E2B			;Lower window bound for O2 error 
	BLS     LDB72			;Bra if O2 volts <=, O2 functional
;
;-Check time threshold
;
	LDAB    L023F			;Error 13 timer
	CMPB    L4E2E			;Time threshold to set O2 error
	BHI     LDB77			;Bra if >, set O2 error
;
;-Check quals for incrementing O2 timer
;
	LDAA    L0002			;Loop counter
	ANDA    #$F0			;Clear lower bits
	BNE     LDB7A			;Bra if !=0, one second has not yet passed
;
	BRSET   L00F9,#$01,LDB7A	;LSB, engine run time, bra if b0==1, dont increment
					;timer this pass
;
;-Two seconds have elapsed since last pass
;
	LDAB    L0006			;Cool temp, linearized
	CMPB    L4E2F			;Cool temp threshold for O2 diags.
	BLS     LDB7A			;Bra if cool temp <=, no diags.
;
;-Check TPS quals for incrementing timer
;
	LDAA    L01FC			;TPS
	CMPA    L4E2D			;Lower TPS limit to inc. error 13 timer
	BHI     LDB6D			;Bra if TPS >
;
;-Decay out O2 timer if low TPS/idling
;
	TST     L023F			;Test error 13 timer
	BEQ     LDB7A			;Bra if ==0
;
	DEC     L023F			;Dec error 13 timer
	BRA     LDB7A			;Bra to continue
;
;-Increment error 13 timer
;
LDB6D	INC     L023F			;+1, error 13 timer
	BRA     LDB7A			;Bra to continue
;
;-Here to clear error 13 timer
;
LDB72	CLR     L023F			;Clear timer
	BRA     LDB7A			;Bra to continue
;
;-Here to set error 13, O2 sensor failure
;
LDB77	BSET    L0016,#$80		;Error word #1, set b7, error 13, O2 sensor
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Check quals for setting error 16, VSS buffer
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LDB7A	BCLR    L0016,#$10		;Clear b4, error 16, VSS buffer
;
;-Check if MAP errors
;
	LDAA    L006F			;Status word 
	BITA    #$48			;Test b6=High MAP err, b3=Low MAP err
	BNE     LDBAE			;Bra if MAP error set
;
;-Check MPH to see if diags. required
;
	LDAA    L02A7			;MPH
	CMPA    L4E30			;MPH limit for VSS diags
	BHI     LDBAE			;Bra if MPH >
;
;-Check timer threshold to set error
;
	LDAA    L0817			;Error 16 timer
	CMPA    L4E35			;Error timer thresold to set error
	BHI     LDBC4			;Bra if >, set error
;
;-Check MAP threshold for diags.
;
	LDAA    L01E9			;A/D MAP
	CMPA    L4E33			;Upper MAP threshold for VSS buffer diags.
	BCC     LDBAE			;Bra if MAP is >, engine under load
;
;-Check cool temp threshold for diags.
;
	LDAA    L0006			;Cool temp, linearized
	CMPA    L4E34			;Lower cool temp threshold for VSS buffer diags
	BLS     LDBAE			;Bra if cool temp <=, no diags
;
;-Check RPM limits for diags.
;
	LDAA    L0062			;RPM/25
	CMPA    L4E31			;Lower RPM limit for VSS buffer diags.
	BLS     LDBAE			;Bra if RPMs <=, no diags.
;
	CMPA    L4E32			;Upper RPM limit for VSS buffer diags.
	BLS     LDBB3			;Bra if RPMs <=, inc. timer
;
;-Quals not met for setting error
;
LDBAE	CLR     L0817			;Clear error 16 timer
	BRA     LDBC7			;Bra to continue
;
;-Check TPS quals for inc. error 16 timer
;
LDBB3	LDAA    L01FC			;TPS
	CMPA    L4E36			;Upper TPS limit to increment err. timer
	BCC     LDBAE			;Bra if TPS >=
;
;-Vehicle coasting, increment error 16 timer
;
	LDAA    L006F			;Status word
	BPL     LDBC7			;Bra if b7==0, one second has not yet passed
;
	INC     L0817			;Inc. error 16 timer
	BRA     LDBC7			;Bra to continue
;
;-Here to set error 16, 2002 PPM VSS buffer signal lost
;
LDBC4	BSET    L0016,#$10		;Error word #1, set b4, error 16, VSS buffer
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Check quals for error 24, VSS
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LDBC7	LDAA    L400F			;Opt byte
	BITA    #$80			;Test b7, transmission option
	BEQ     LDBFA			;Bra if ==0, TCC/auto in use
;
;-Manual transmission in use, perform diags.
;
	LDAA    L5B01			;Error mask #2
	BITA    #$20			;Test b5, flag for error 24, VSS
	BEQ     LDBFA			;Bra if ==0, not flagging error 24
;
	LDD     L0849			;Stored TOS pulses
	BEQ     LDBE2			;Bra if ==0, no pulses recieved
;
;-Pulses recieved
;
	BCLR    L0017,#$20		;Clear b5, err 24, VSS
	CLR     L081C			;Clear error 24 timer
	BRA     LDBFA			;Bra to continue
;
;-Check timer quals for setting error
;
LDBE2	LDAA    L081C			;Error 24 timer
	CMPA    L4E39			;Time threshold to set error 24, VSS 
	BLS     LDBEF			;Bra if <=, dont set error yet
;
;-Here to set error 24, VSS
;
	BSET    L0017,#$20		;Error word #2, set error 24, VSS
	BRA     LDBFA			;Bra to continue
;
;-Check MPH threshold to increment timer
;
LDBEF	LDAA    L02A7			;MPH
	CMPA    L4E3A			;Max speed to set error 24
	BLS     LDBFA			;Bra if MPH <=
;
	INC     L081C			;Inc. error 24 timer
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Check quals for error 31, Governor
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LDBFA	BCLR    L0018,#$80		;Error word #3, clear b7, err 31, governor
;
	LDX     #L400F			;Opt. byte
	BRCLR   0,X,#$01,LDC16		;Bra if b0==0, rev limiter not enabled
;
	BRSET   L006F,#$10,LDC10	;Status word, bra if b4==1, error 31 previously set
;
	LDAA    L02AB			;Governor actuator integral term
	CMPA    L4E41			;Threshold to set error
	BCS     LDC16			;Bra if integrator <
;
;-Here to set error
;
LDC10 	BSET    L0018,#$80		;Error word #3, ser b7, error 31, gov'r
	BSET    L006F,#$10		;Set b4, error 31 previously set
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Perform EGR diags and check quals for error 32, EGR
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LDC16	LDX     #L5B02			;Error word mask #3
	BRCLR   0,X,#$40,LDC24		;Bra if b6==0, not setting error 32
;
;-Error 32 enabled, pre-clear error word and perform diags
;
	BCLR    L0018,#$40		;Preclear b6, error 32, EGR flag
	BRCLR   L0070,#$08,LDC27 	;Status word, bra if b3==0, perform EGR diags.
;
;-Here if not setting error 32, EGR or single cycle EGR diags. optioned
;
LDC24	JMP     LDD14			;Skip EGR diags
;
;-Perform EGR diags.
;
LDC27	LDX     #L400D			;Opt. byte
	BRCLR   0,X,#$02,LDC35		;Bra if b1==0, standard EGR in use
;
;-Linear EGR in use
;
	BRCLR   L0070,#$01,LDC35	;Status word, bra if b0==0, EGR hasnt previously
					;failed diags.
;
	JMP     LDD0D			;Bra to continue, linear EGR failed diags. 
					;previously
;
;-Check quals for egr diags.
;
LDC35	BRSET   L003E,#$80,LDC3C	;Status word, bra if b7==1, in closed loop
;
LDC39	JMP     LDCFD			;Bra to suspend EGR diags.
;
;-Check if DFCO in effect
;
LDC3C	BRSET   L0046,#$08,LDC39	;Status word, bra if b3==1, DFCO in effect
;
	LDAA    L0242			;EGR diag. cycling timer
	BEQ     LDC52			;Bra if ==0, perform diags.
;
;-Cycle timer has not yet run down, skip diags.
;
	BRSET   L006F,#$80,LDC4B	;Status word, bra if b7==1, one second has passed
;
	BRA     LDC4F			;Bra to continue
;
LDC4B	DECA				;Dec EGR diags. cycling timer
	STAA    L0242			;Save it
;
LDC4F	JMP     LDD03			;Skip diags.				
;
;-Check desired EGR position
;
LDC52	LDAA    L01D1			;Load desired EGR position
	CMPA    L4E47			;Threshold to perform EGR diags.
	BCS     LDC8E			;Bra if desired EGR is <, no diags.
;
;-Check vac window for diags.
;
	LDAA    L01EC			;vacuum
	CMPA    L4E43			;Lower vac threshold to perform diags.
	BLS     LDC8E			;Bra if <=, vac too low for diags 
;
	CMPA    L4E44			;Upper vac threshold to perform diags
	BHI     LDC8E			;Bra if >, vac too high for diags 
;
;-Check TPS window for diags.
;
	LDAA    L01FC			;TPS 
	CMPA    L4E45			;Lower TPS threshold to perform diags.
	BLS     LDC8E			;Bra if TPS <=, no diags.
;
	CMPA    L4E46			;Upper TPS threshold to perform diags.
	BHI     LDC8E			;Bra if TPS >, no diags.
;
;-Check to ensure TPS is stable
;
	SUBA    L0201			;Subtract slow filtered TPS
	BCC     LDC7A			;Bra if no underflow
;
	NEGA				;Get abs. value
;
LDC7A	CMPA    L4E4A			;Upper change in TPS allowed for EGR diags.
	BHI     LDC8E			;Bra if >, changes in TPS are too large for diags.
;
;-Check for sufficient MPH
;
	LDAA    L02A7			;MPH
	CMPA    L4E4D			;Min speed for EGR diags.
	BCS     LDD03			;Bra if speed <, no diags.
;
;-Check if INT reset or BLM cell change
;
	BRCLR   L006E,#$01,LDC90	;Status word, bra if b0==0, no INT reset or BLM cell
					;change
;
;-INT reset/BLM cell change in effect
;
	BCLR    L006E,#$01		;Clear b0
LDC8E	BRA     LDD03			;Bra, no EGR diags.
;
;-Perform EGR diags. based on INTegrator
;
LDC90	LDAA    L0232			;INTegrator
	BRSET   L0070,#$04,LDCBB	;Status word, bra if b2==1, EGR diagnostics underway
;											
	SUBA    #128			;Subtract neutral INT 
	BCC     LDC9C			;Bra if INT <= 128
;
	NEGA				;Get abs. value
;
;-Check if change in fueling great enough to update BLM
;
LDC9C	CMPA    L48ED			;Delta INT to update BLMs 
	BHI     LDD03			;Bra if deviation is >, no BLM update
;
;-Steady fueling, check time to perform diags.
;
	LDAA    L0243			;Delay timer for EGR diags.
	CMPA    L4E48			;Min time to perform diags.
	BHI     LDCAC			;Bra if time >, perform EGR diags.
;
;-Not time to perform EGR diags. yet
;
	INCA				;+1
	BRA     LDCB6			;Bra to save and continue
;
;-Here to initialize EGR diags.
;
LDCAC	BSET    L0070,#$04		;Set b2, EGR diags. underway, temperarily disable
					;EGR
;								
	LDAA    L0232			;INT
	STAA    L0245			;Stored INTegrator reference for EGR diag.
	CLRA				;Clear A
;
LDCB6	STAA    L0243			;Save it, delay timer for EGR diags.
	BRA     LDD0D			;Bra to continue
;
;-Check INT to see if EGR is functional
;
LDCBB	SUBA    L0245			;Subtract reference INT
	BCS     LDCC5			;Bra if current INT was <, EGR not functional
;
	CMPA    L4E4C			;Min delta INT for EGR to be functional		
	BCC     LDCE5			;Bra if delta INT >=, EGR ok
;
;-EGR not yet functional, check duration for error to be set
;
LDCC5	LDAA	L0244			;Load EGR integrator diags. timer
	CMPA    L4E4B			;Max time before EGR fails diags.
	BCC     LDCD3			;Bra if timer >=, EGR failed diags.
; 
	INCA				;Inc. timer
	STAA    L0244			;Save it int EGR diags. timer
	BRA     LDD0D			;Bra to 
;
;-EGR not functional, set error
;
LDCD3	LDAA    L0246			;EGR fail timer
	INCA				;Inc. timer
	CMPA    L4E49			;Time threshold to set error
	BCS     LDCE0			;Bra if time <, dont set error yet
;
	BSET    L0070,#$01		;Status word, set b0, EGR failed diags.											;
	DECA				;Dec. timer to hold at current position
;
LDCE0	STAA    L0246			;Save timer
	BRA     LDCFD			;Bra to continue
;
;-Here if EGR passed diags
;
LDCE5	LDAA    L0246			;EGR fail timer
	BEQ     LDCEB			;Bra if ==0
;
	DECA				;Dec. counter toward 0
;
LDCEB	BNE     LDCE0			;Bra if !=0, dont reset EGR failed diags. flag yet
;
;-EGR passed diags. and sufficient time has passed to reset
;
	BCLR    L0070,#$01		;Clear b0, EGR failed diags. flag
	STAA    L0246			;Save EGR fail timer
;
;-Check if single EGR diag. cycle optioned
;							
	LDX     #L400D			;Opt. byte
	BRCLR   0,X,#$80,LDCFD		;Bra if b7==0, multiple cycles optioned 
;
	BSET    L0070,#$08		;Set b3, EGR diag. cycle performed
;	
LDCFD	LDAA    L4E42			;Time between EGR diag. cycles
	STAA    L0242			;Save it, EGR diag. cycle timer
;	
;-Clear timers for next pass
;									
LDD03	CLRA				;Preclear A
	STAA    L0243			;Clear delay timer for EGR diags.
	STAA    L0244			;Clear integral EGR diag. timer
	BCLR    L0070,#$04		;Status word, clear b2, EGR diags. underway flag
;
LDD0D	BRCLR   L0070,#$01,LDD14	;Bra if b0==0, EGR passed diags.
;
;-Here to set EGR error
;
	BSET    L0018,#$40		;Error word #3, set b6, error 32, EGR
;
;-Check if linear EGR in use
;									
LDD14	LDX     #L400D			;Opt byte 
	BRCLR   0,X,#$02,LDD8A		;Bra if b1==0, standard EGR in use 
;
;-Linear EGR in use, perform additional diags.
;								
	LDX     #L5B02			;Error word mask #3
	BRCLR   0,X,#$40,LDD8A		;Bra if b6==0, not setting error 32
;
	BRSET   L0070,#$01,LDD7E	;Bra if b0==1, EGR previously failed diags.
;
;-Check lower bound for A/D pintle position
;
	LDAA    L01D7			;A/D EGR pintle position
	CMPA    L4E4F			;Min allowed pintle position
	BCC     LDD41			;Bra if pintle position >=
;
;-A/D pintle position less then that allowed
;
	LDAA    L024A			;Linear EGR position low error timer
	CMPA    L4E55			;Time threshold to fail diags.
	BCS     LDD3B			;Bra if timer <, dont flag diag. failure yet
;
	BSET    L0072,#$20		;Status word, set b5, linear EGR failed diags.					
	BRA     LDD7B			;Bra to continue	
;										
LDD3B	INCA				;Inc. timer
	STAA    L024A			;Save it, linear EGR position low error timer
	BRA     LDD44			;Bra to continue							
;											
LDD41	CLR     L024A			;Clear pintle position low timer
;
;-Check if time for linear EGR diags. cycle
;
LDD44	BRCLR   L0070,#$08,LDD59	;Status word, bra if b3==0, diag. cycle not yet
					;completed 
;
	LDX     L0248			;Linear EGR diag. cycle timer
	CPX     L4E53			;Time between diag. cycles
	BCC     LDD56			;Bra if timer >=, perform diags.
;
	INX				;Inc timer
	STX     L0248			;Store it
	BRA     LDD8A			;Bra to skip diags.
;
;-Check pintle position error
;
LDD56	BCLR    L0072,#$20		;Status word, clear b5, linear EGR diag. failure 
					;flag 
;
LDD59	LDAA    L01D1			;Load desired EGR position
	CMPA    L4E52			;Upper  position limit to check pintle position 
					;error 
	BHI     LDD86			;Bra if desired position is >, skip diag.
;
	LDAA    L01DD			;EGR position error
	CMPA    L4E50			;Upper position error limit
	BLS     LDD86			;Bra if position error is <=
;
;-Excessive EGR pintle position error
;
	LDAB    L0247			;Pintle position error diag. failure timer
	CMPB    L4E51			;Time threshold to set error
	BCC     LDD74			;Bra if timer is >=, EGR failed diags.
;
	INCB				;Inc position error timer
	BRA     LDD87			;Bra to continue
;
;-Here if EGR failed diags.
;
LDD74	BSET    L0072,#$20		;Status word, set b5, linear EGR failed diags.										;
	BRCLR   L0070,#$08,LDD83	;Bra if b3==0, flag diag. cycle complete
					;before setting error
;
LDD7B	BSET    L0070,#$01		;Status word, set b0, EGR failed diags.
;
;-Here to set error 32, EGR
;
LDD7E	BSET    L0018,#$40		;Error word #3, set b6, error 32, EGR
	BRA     LDD8A			;Bra to continue
;
LDD83	BSET    L0070,#$08		;Set b3, EGR diag. cycle completed
;
LDD86	CLRB				;
;
LDD87	STAB    L0247			;Clear pintle position diag. timer
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Check quals for error 33, MAP sensor high
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LDD8A	BCLR    L0018,#$20		;Error word #3, preclear b5, error 33, MAP
					;sensor high
;
	LDAB    L006F			;Status word
	ANDB    #$BF			;Mask all but b6, MAP high error
	BITB    #$24			;Test b2, high TPS, and b5, low TPS errors
	BNE     LDDC1			;Bra if errors set 
;
;-Check baro kPa for diags.
;											
	LDAA    L4E57			;Lower baro limit for MAP high diags.
	CMPA    L0317			;Filtered baro
	BLS     LDDA0			;Bra if baro kPa <=, use lower limit
;
	LDAA    L0317			;Load baro
;
LDDA0	CMPA    L082E			;Compare to A/D MAP
	BCC     LDDC1			;Bra if A/D MAP is <=, MAP functional
;
	LDAA    L024B			;Error 33 timer
	CMPA    L4E58			;Time threshold to set error
	BHI     LDDC6			;Bra if timer >, set error
;
;-Check TPS thresholds to set error
;
	LDAA    L01FC			;Load TPS
	SUBA    L4E56			;Lower TPS threshold to set error 33
	BCC     LDDC1			;Bra if TPS >=, dont set error yet
;
;-Check RPM thresholds to set error
;
	LDAA    L0061			;Scaled RPM
	CMPA    L4E59			;Scaled RPM threshold to set error 33
	BCS     LDDC1			;Bra if RPMs <, dont set error yet
;
	INC     L024B			;Inc error 33 timer
	BRA     LDDCB			;Bra to continue
;
;-Here if not setting error yet
;
LDDC1	CLR     L024B			;Clear error 33 timer
	BRA     LDDCB			;Bra to continue
;
;-Here to set error 33
;
LDDC6	BSET    L0018,#$20		;Error word #3, set b5, error 33, MAP sens. high
	ORAB    #$40			;Set b6, MAP high
;
LDDCB	STAB    L006F			;Save status word
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Check quals for error 42, EST
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;    
	BRSET   L0044,#$04,LDE01	;Status word, bra if b2==1, skip EST test
;
	BRSET   L0050,#$08,LDE01	;Status word, bra if b3==1, in ALDL diags.,
					;skip error 42 diags.
;
	BRSET   L0002,#$10,LDE01	;Loop counter, bra if b4==1, no diags. this
					;pass (5 Hz alt. passes)
;
	BRCLR   L004F,#$40,LDE01	;Status word, bra if b6==0, EST has not yet passed
					;bypass diags.
;
	CLRB				;Preclear B    
	LDX     L3FCA			;Load EST circuit info
	LDAA    L0061			;Scaled RPMs
	CMPA    L4E66			;Lower RPM limit for EST diags.
	BLS     LDDFB			;Bra if RPMs <=, no diags.
;
	CPX     L0228			;Compare to stored EST info
	BNE     LDDFB			;Bra if !=
;
;-EST fault detected
;
	LDAB    L024F			;Error 42 timer
	CMPB    L4E69			;Time threshold to set error 42
	BHI     LDDF8			;Bra if time >, set error
;
	INCB				;Inc counter    
	BRA     LDDFB			;Bra to continue
;
;-Here to set error 42, EST
;
LDDF8	BSET    L0004,#$80		;Status word, set b7, EST failure
;
LDDFB	STAB    L024F			;Save error 42 timer
	STX     L0228			;Save EST info
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Check quals for error 44, O2 lean
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LDE01	BCLR    L0019,#$08		;Error word #4, clear b3, error 44, O2 lean
;
	LDAA    L4E6E			;Lower O2 volt threshold to set error
	CMPA    L01F3			;Compare to filtered O2
	BLS     LDE39			;Bra if O2 >=, O2 functional
;
	LDAA    L023C			;Error 44 timer
	BRCLR   L0072,#$40,LDE1E	;Status word, bra if b6==0, O2 lean flag
;
;-O2 lean has been flagged previously
;
	CMPA    L4E70			;Timer threshold to set error if previously
					;flagged
	BHI     LDE41			;Bra if timer >, set error 44
;
	BRSET   L00F9,#$01,LDE44	;LSB engine run time, bra if b0==1, alternate
					;passes each second
;
	BRA     LDE30			;Bra to skip remainder of diags.
;
;-O2 lean has not yet been flagged
;
LDE1E	CMPA    L4E6F			;Timer threshold to set error 44
	BLS     LDE28			;Bra if timer <=, dont set error yet
;
;-Enough time has passed to flag O2 error
;
	BSET    L0072,#$40		;Set b6, O2 lean
	BRA     LDE3C			;Bra to cintune
;
;-Check if error 44 allowed to be set
;
LDE28	BRCLR   L003E,#$80,LDE39	;Status word, bra if b7==0, not in closed loop
;
	BRSET   L0071,#$04,LDE39	;Status word, bra if b2==1, INT just reset
;
LDE30	BRCLR   L006F,#$80,LDE44	;Status word, bra if b7==1, one second has
					;not yet passed
;
;-.5 Hz execution rate
;
	INC     L023C			;Inc. error 44 timer
	BRA     LDE44			;Bra to continue
;
;-Here if not flagging error
;
LDE39	BCLR    L0072,#$40		;Clear b6, O2 lean
;
;-Here to reset for next pass
;
LDE3C	CLR     L023C			;Clear error 44 timer
	BRA     LDE44			;Bra to continue
;
;-Here to set error 44
;
LDE41	BSET    L0019,#$08		;Error word #4, set b3, error 44, O2 lean
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Check quals for error 45, O2 rich
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LDE44	LDX     #L400E			;Opt. byte
	BRSET   0,X,#$20,LDE4E		;Bra if b5==1, error 45 to remain set if O2					
					;has previously failed diags. 
;
	BCLR    L0019,#$04		;Error word #4, preclear b2, error 45, O2 rich

LDE4E	LDAA    L01F3			;Filtered O2 volts
	CMPA    L4E73			;Threshold to set error
	BLS     LDE96			;Bra if O2 volts <=, O2 functional
;
	LDX     L023D			;Error 45 timer
;
;-Check if high MAT temps present
;
	BRSET   L0044,#$20,LDE6F	;Status word, bra if b5==1, high MAT temps present

	LDAA    L0252			;MAT
	CMPA    L4E72			;Compare to hi MAT threshold to use alternate time
					;qual to set error 45
	BHI     LDE6C			;Bra if MAT >, high MAT temps present
;
;-Normal MAT temps present
;
	CPX     L4E74			;Check timer threshold to set error 45
	BHI     LDE9E			;Bra if timer threshold >, set error
;
	BRA     LDE74			;Bra to continue
;
;-Here if high MAT temops present
;
LDE6C	BSET    L0044,#$20		;Set b5, high MAT temps present (error 45 logic)
;
LDE6F	CPX     L4E76			;Alternate timer threshold to set error 45 if 
					;high MAT temps
	BHI     LDE9E			;Bra if timer >, set error 45
;
;-Check if error 45 allowed to be set
;
LDE74	BRCLR   L003E,#$80,LDE96	;Status word, bra if b7==0, not in closed loop

	BRSET   L0071,#$04,LDE96	;Status word, bra if b2==1, INT just reset
;
;-Check TPS quals to set error
;
	LDAA    L01FC			;TPS
	CMPA    L4E78			;Upper %TPS range to proceed with diags.
	BHI     LDE89			;Bra if TPS >, continue with diags.
;
	CMPA    L4E79			;Lower %TPS range to skip diags.
	BCC     LDE96			;Bra if TPS >=, dont set error
;
;-Here to procede with setting error 45
;
LDE89	BRCLR   L006F,#$80,LDEA1	;Status word, bra if b7==0, one second has not
					;yet passed
;
	LDX     L023D			;Error 45 timer
	INX				;Inc timer 	    
	STX     L023D			;Save it
	BRA     LDEA1			;Bra to continue
;
;-Here if not setting error
;
LDE96	LDX     #$0000			;
	STX     L023D			;Clear error 45 timer
	BRA     LDEA1			;Bra to continue
;
;-Here if setting error
;
LDE9E	BSET    L0019,#$04		;Error word #4, set b2, error 45, O2 rich
;
LDEA1	BCLR    L0071,#$04		;Status word, clear b2, high MAT temp flag
;
;-Check BLM limits if optioned as part of O2 diags
;
	LDAB    L400C			;Opt. byte
	BITB    #$02			;Test b1, check BLM limits as part of error 44/45
					;diags.
	BEQ     LDEB6			;Bra if ==0, dont check BLM limits
;
;-Check limits for BLMs if O2 errors have not been set
;
	BRSET   L0019,#$04,LDEB3	;Bra if b2==1, error 45, O2 rich
;	
	BRCLR   L0072,#$40,LDEB6	;Status word, bra if b6==0, O2 lean flag
;
;-Check limits on BLM cells
;
LDEB3	JSR     LEF2B			;BLM clamp routine
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Check quals for error 46, VATS
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LDEB6	LDAA    L5B03			;Error mask #4
	BITA    #$02			;Test b1, error 46, VATS
	BEQ     LDEDA			;Bra if ==0, not flagging error
;
	LDAA    L00A1			;A/D batt volts
	CMPA    #90			;9.0 volts
	BCS     LDEDA			;Bra if batt. volts < 9 volts, dont set VATS error
;
;-Check if VATS passed
;
	BRSET   L003D,#$02,LDEDA	;Status word, bra if b1==1, VATS passed
;
	LDAA    L0856			;Error 46 timer
	CMPA	L4E7A  			;Time threshold to set error 46, VATS
	BHI     LDED7			;Bra if timer >, set error
;
	INCA				;Inc error timer
	BEQ     LDEDA			;Bra if roll over
;
	STAA    L0856			;Save timer
	BRA     LDEDA			;Bra to continue
;
;-Here if setting error 46, VATS
;
LDED7	BSET    L0019,#$02		;Error word #4, set b1 error 46, VATS
;
LDEDA	BRCLR   L0043,#$04,LDEE1	;Status word, bra if b2==0, SES not in use to relay
					;state of fueling
;
;-SES currently being used to relay status of fueling
;
	JMP     LDFAB			;Bra to return
;
;-SES not in use here
;
LDEE1	BRCLR   L0071,#$80,LDEEB	;Status word, bra if b7==0, not in ALDL diag.
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; In ALDL diags., blink out error codes and return
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
	JSR     LDFAC			;Routine to blink out error codes through SES 
;
	JMP     LDF9F			;Jump to continue
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; No ALDL diags., check and store any new errors
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;    
LDEEB	BRCLR   L0044,#$10,LDEF2	;Status word, bra if b4==0, ignition is still on
;
;-Ignition off
;
	JMP     LDFAB			;Return
;
;-Ignition on, check if any stored errors
;
LDEF2	LDY     #L5B08			;Load diagnostics masks
	CLRA				;Preclear A    
	LDX     #$0009			;9 error words
;
LDEFA	ORAA    $55,X			;OR with stored error words
	DEX				;-1, move onto next error word     
	BNE     LDEFA			;Bra if !=0, continue checking
;
	TSTA				;Test result   
	BNE     LDF20			;Bra if !=0, errors have been set previously
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-No stored errors, update stored error words
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
	LDX     #$0009			;9 error words
;
LDF05	LDAA    0,Y			;Load error mask in
	ANDA    $15,X			;And mask in with error word, clear errors that 
					;are not flagged in mask
	STAA    $55,X			;Save it, stored error words
	DEY				;-1, move onto next mask     
	DEX				;-1, move onto next error word     
	BNE     LDF05			;Bra if !=0, not done updating 
;
	LDAA    L024E			;Error log timer
	BEQ     LDF1B			;Bra if ==0, turn off SES light
;
	DEC     L024E			;Dec error log timer, no new errors set
	BRA     LDF87			;Bra to continue
;
LDF1B	BCLR    L0071,#$01		;Turn off SES, no codes have been set
	BRA     LDF87			;Bra to continue
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Stored errors present, check if new errors set
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LDF20	LDX     #$0009			;Start addr. for error words
	CLRA				;Preclear A
;    
;-Check if new errors have been set
;
LDF24	LDAB    $15,X			;Load error word
	ANDB    $55,X			;Mask out previously set errors
	ANDB    0,Y			;Error mask, mask out only errors that will
					;be flagged
	STAB    $55,X			;Save updated error word
	ORAA    $55,X			;OR with stored error words
	DEY				;-1, move onto next error mask     
	DEX				;-1, move onto next error word     
	BNE     LDF24			;Bra if !=0, continue
;
	INC     L024E			;Inc error log timer
	LDX     #L4E24			;
	BRCLR   L0071,#$01,LDF40	;Status word, bra if b0==0, SES not commanded to be 
					;on
;
;-SES previously commanded to be on
;
	INX				;+1    
	INX				;+1, move to error log time thresholds for set errors 
;
;-SES not previously commanded to be on
;   
LDF40	TSTA				;Test if any errors where set    
	BEQ     LDF44			;Bra if ==0, no errors set
;
;-Errors have been set
;
	INX				;+1, min time for error to be logged if errors set
;     
LDF44	LDAB    0,X			;Load timer threshold to update flagged error log
	CMPB    L024E			;Compare to error log timer
	BCC     LDF87			;Bra if timer >=, dont update logged errors yet
;
;-Enough time has passed to log flagged errors
;
	STAB    L024E			;Save timer threshold if enough time has passed 
	TSTA				;Test if any error have been set   
	BEQ     LDF87			;Bra if no errors where set
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Time to update log of stored errors
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
	LDAA    L4E26			;10 secs, min time for next SES update/error log
	STAA    L024E			;Error log timer
;
	CLRB				;Preclear B   
	LDX     #$0009			;9 error words
;
LDF5B	LDAA    $0A,X			;Load flagged malfunction word
	ORAA    $55,X			;OR with stored errors
	STAA    $0A,X			;Save it, flagged malfunction words
	STAB    $55,X			;Clear out stored error words from previous pass
	DEY				;-1 [doesnt appear to be used in this loop]      
	DEX				;-1, move onto the next error word     
	BNE     LDF5B			;Bra if !=0, continue
;
	STAB    L0014			;Clear # of error free starts, errors have been set
	JSR     LEFC3			;Flagged malf. word checksum routine
;
	STAA    L0015			;Save it, flagged malfunction word checksum
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Illuminate SES light if flagged errors have been set
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
	LDX     #$0009			;9 error words			
	LDY     #L5B00-4		;Address for SES error masks
;
LDF76	LDAB    $15,X			;Load error word
	BITB    $15,Y			;Test using error mask
	BNE     LDF84			;Bra if flagged errors are set
;
;-Move to next error word/mask
;
	DEY				;-1, error word #				     
	DEX				;-1, error mask #    
	BNE     LDF76			;Bra if !=0, continue
;
;-No errors set
;
	BRA     LDF87			;Bra to continue
;
;-Errors set, turn on SES 
;
LDF84	BSET    L0071,#$01		;Turn on SES light
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Here if no transmission errors where set/stored
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LDF87	BRSET   L004F,#$80,LDF98	;Status word, bra if b7==1, engine is running
;
;-Turn on SES if engine off
;
	BRSET   L004F,#$02,LDFA8	;Status word, bra if b1==1, illuminate SES
;
	BRSET   L0004,#$08,LDF98	;Status word, bra if b3==1, bad shutdown
;
	BSET    L004F,#$02		;Set b1, illuminate SES light
	BRA     LDF9B			;Bra to continue		
;
;-Turn off SES light if running/bad shutdown
;
LDF98	BCLR    L004F,#$02		;Status word, clear b1, turn off SES if no malf.
;										
LDF9B	BRSET   L0050,#$08,LDFA8	;Status word, bra if b3==1, in ALDL diags.
;
;-Turn on SES if commanded to do so
;
LDF9F	BRSET   L0071,#$01,LDFA8	;Status word, bra if b0==1, SES light to be on
;					
	BCLR    L0072,#$80		;Status word, clear b7, turn SES off										;
	BRA     LDFAB			;Bra to return
;
LDFA8	BSET    L0072,#$80		;Set b7, turn SES light on
;
LDFAB	RTS				;Return	    
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; Routine to blink out error codes through SES 
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LDFAC	BRCLR   L003C,#$40,LE018	;Status word, bra if b6==0, at begining of codes

	DEC     L023B			;SES blink timer
	BNE     LDFCF			;Bra if timer !=0, return
;
	LDAB    #$04			;.4 sec, time for SES to remain off
					;between blinks
	BRSET   L0071,#$01,LE033	;Status word, bra if SES currently on
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Blink out 10's/1's portion of error code
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
	LDX     #L0237			;10's and 1's portions of error codes to blink out
	BRCLR   L003C,#$20,LDFC3	;Status word, bra if b5==0, blink out 10's portion
					;of code

	INX				;Move to 1's portion of error code 
;    			
LDFC3	DEC     0,X			;-1 blink
	BMI     LDFD1			;Bra if roll-over, blinkout complete
;
;-Activate SES light and blink out portion of code
;
	BSET    L0071,#$01		;Set b1, turn on SES light											;
	LDAB    #$04			;.4 sec, time for SES to remain on
	STAB    L023B			;SES blink timer
;
LDFCF	BRA     LE039			;Bra to return				
;
;-Upper portion of error code has been displayed through the SES
;											
LDFD1	BRSET   L003C,#$20,LDFDC	;Status word, bra if 1's portion completed
;
	BSET    L003C,#$20		;Set b5, move onto 1's portion of error code
;											
	LDAB    #$08			;.8 sec, time between 10's and 1's portion of 
					;error code
	BRA     LE033			;Bra to turn off SES and return				
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Here to reset for next code
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LDFDC	BCLR    L003C,#$20		;Status word, clear b5, display 1's portion flag										;
	BRCLR   L003C,#$03,LDFE8	;Status word, bra if b0==0, and b1==0, error code
					;not set this pass
;
	DEC     L003C			;Clear b0
	BRA     LE01B			;Bra to look up next code
;
;-Move onto next error code
;
LDFE8	LDAB    L0239			;Load current error # to blink through SES
	CMPB    #71			;71 codes
	BLS     LDFF7			;Bra if <=, still codes remaining to display
;
;-Here to loop to begining of error code list
;
	CLR     L0239			;Clear current error #
	BCLR    L003C,#$40		;Clear b6, at start of error code list 
	BRA     LE036			;Bra to return
;
;-Still have codes to display
;
LDFF7	BITB    #$07			;Test lower three bits of current error #
	BNE     LE010			;Bra if !=0, continue
;
;-Load in next diagnostics mask
;
	LSRB				;/2  
	LSRB				;/2    
	LSRB				;/2, now /8 for 8 bit masks    
	LDY     #L5B00			;Start of diag. masks
	LDX     #L000B			;Error word #1
	ABX				;Add current error # in as offset     
	ABY				;     
	LDAB    0,X			;Load diag. mask
	ANDB    0,Y			;Clear errors that arnt optioned to be set
	STAB    L023A			;Save set errors
;
LE010	INC     L0239			;Error # counter
	ASL     L023A			;Shift stored errors left
	BCC     LDFE8			;Bra if error not set
;
LE018	BSET    L003C,#$02		;Status word, set b1, error set this pass
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Look up numerical SES blink out
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LE01B	LDAB    L0239			;Load error # counter
	LDX     #LFB00			;SES error blink out table
	ABX				;Add in offset
	LDAA    0,X			;Load upper/lower nibble for error 
					;code blink out
	TAB				;Move results to B
;
;-Get 1's number of blinks in B
;
	ANDB    #$0F			;Mask out lower nibble
;
;-Get 10's number of blinks in A
;
	LSRA				;Shift right
	LSRA				;    	
	LSRA				;    
	LSRA				;Now upper nibble     
	STD     L0237			;Save result, number of blinks for error code
	LDAB    #28			;2.8 second delay between display of each code										;
	BSET    L003C,#$40		;Status word, set b6, still have codes remaining
					;to be displayed
;
LE033	STAB    L023B			;SES blink timer
;										
LE036	BCLR    L0071,#$01		;Status word, clear b0, turn off SES light
;											
LE039	BRSET   L0071,#$01,LE042	;Bra if b0==1, turn on SES light
;
;-Here to turn off SES light
;
	BCLR    L0072,#$80		;Status word, clear b7, turn off SES										;
	BRA     LE045			;Bra to return
;
;-Here to turn on SES light
;
LE042	BSET    L0072,#$80		;Status word, set b7, turn on SES light

LE045	RTS				;Return    
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; Routine for handling closed loop entry and non time critical lookups
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LE046	LDX     #L5B04			;Error word #5 mask
	BRCLR   0,X,#$04,LE088		;Bra if b2==0, dont flag fuel pump volts error
;
;-Flagging error 
;
	BCLR    L001A,#$04		;Error word #5, clear b2, err 54, fuel pump volts
	BRSET   L0072,#$01,LE085	;Bra if b1==1, low fuel pump volts flagged

	LDAB    L00A1			;Batt volts
	CMPB    L4E7E			;Lower volt. threshold to perform diags.
	BCS     LE088			;Bra if batt. volts less, skip diags.
;
;-Perform fuel pump diags.
;
	LDAA    L0055			;Load fuel pump volts
	BRCLR   L0072,#$02,LE06C	;Bra if diags. not in progress
;
	BRCLR   L004F,#$10,LE088	;Status word, bra if b4==0, engine not above 
					;theshold to be cranking/not running
;
;-Engine running
;
	CMPA    L4E7D			;Lower pump volts threshold to set error
	BCC     LE088			;Bra if volts are greater, dont set error
;
	BRA     LE085			;Bra to set error
;
;-Here to check quals for set error
;
LE06C	LDAB    L024D			;Error 54 timer
	INCB				;+1
	STAB    L024D			;Save it
	CMPB    L4E7B			;Timer threshold to set error
	BHI     LE082			;Bra if timer >, set error
;
	CMPA    L4E7C			;Fuel pump volts threshold for setting error
	BCS     LE088			;Bra if volts >
;
;-Low voltage present
;
	BSET    L0072,#$02		;Status word, set b1, fuel pump volts
					;diags. in progress
	BRA     LE088			;Bra to continue
;
;-Here to set error
;
LE082	BSET    L0072,#$01		;Status word, set b0, low fuel pump volts
;
LE085	BSET    L001A,#$04		;Error word #5, set b1, error 54, low FP volts
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Look up inverse air density based on blended intake air temp
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LE088	LDAA    L0006			;Cool temp, linear
	LDX     #L400B			;Opt. byte
	BRCLR   0,X,#$40,LE0AD		;Bra if b6==0, MAT not in use
;
	LDAA    L0255			;Gms/sec airflow
	LDX     #L4AD9			;MAT/CTS blend multiplier
	JSR     LF15E			;2D lookup
;											
	LDAB    L0006			;Cool temp, linear
	SUBB    L0252			;Sub MAT
	BCC     LE0A2			;Bra if >=0
;
;-MAT is greater, clear out val for lookup
;										   	
	CLRB				;Clear result
;
LE0A2	MUL				;(CTS - MAT) x multiplier
	ASLD				;x2, now /128
	BCS     LE0AB			;Bra if overflow
										   	
	ADDA    L0252			;Add in MAT
;
	BCC     LE0AD			;Bra if no overflow
;
LE0AB	LDAA    #$FF			;Clear overflow
;
LE0AD	LDX     #L4AEA			;Inv. air density vs. blended temp table
	JSR     LF15E			;2D lookup
;
	CLRB				;Clear LSB    
	LSRD				;/2, now x 128    
	ADDD    #29824			;.662 l/g
	STD     L0259			;Save it, inverse air density
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Look up injector offset vs. battery volts
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
	LDAA    L00A1			;Batt. volts
	LDX     #L4AFB			;Inj. offset bias table verses batt. bolts 
	JSR     LF15E			;2D lookup
;										
	STAA    L0279			;Save it, injector bias vs. fuel pump volts
;
;-Alternate passes of second portion
;										
	BRCLR   L0002,#$10,LE0CB	;Loop counter, bra if b5==0, run loop
;											
	RTS				;Return if not time to run loop yet   
;
;~~~~~~~~~~~~~~~~~~~~~
; Second 5 Hz section
;~~~~~~~~~~~~~~~~~~~~~
;
;-Look up decel enlean factor
;
LE0CB	LDAA    L0006			;Cool temp, linarized
	LDX     #L4B0C			;Decel enlean factor vs. cool temp
	JSR     LF153			;Bra to do lookup
;
	STAA    L0265			;Save it
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Look up async accel enrich correction factor
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;-Look up cool temp based portion
;
	LDAA    L0006			;Cool temp
	LDX     #L4B67			;Accel enrich coor. factor vs. cool temp
;
	JSR     LF15E			;Bra to do lookup
	TAB				;Get result into B  
;
;-Look up MAT based portion
;
	LDAA    L0252			;Load MAT
	LDX     #L4B78			;Accel enrich corr. factor vs. MAT
	JSR     LF15E			;2D lookup
;
	MUL				;Multiply factors together
	ASLD				;x2, now /128
	BCC     LE0EE			;Bra if no overflow
;
	LDAA    #255			;Clear overflow
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Look up sync accel enrich correction factor
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LE0EE	STAA    L025F			;Accel enrich corr. factor vs. cool temp
	LDAA    L0006			;Cool temp, linearized
	LDX     #L4B89			;AE correction fact. vs cool temp
	JSR     LF15E			;2D lookup
;
	STAA    L0260			;Save it, AE temp correction mult.
	BRSET   L004F,#$10,LE14F	;Status word, bra if b4==1, engine above threshold 
					;to be cranking
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Engine off or still cranking here
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
	BRSET   L0004,#$08,LE136	;Status word, bra if b3==1, bad shutdown
;
;~~~~~~~~~~~~~~~~~~~
; Look up choke AFR
;~~~~~~~~~~~~~~~~~~~
;
	LDAA    L0006			;Cool temp, linarized
	LDX     #L4C29			;Choke AFR vs. cool temp
	JSR     LF15E			;2D lookup
;
	STAA    L02EB			;Save it, choke AFR
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Look up added startup spark advance
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
	LDX     #L4522			;Addr. of startup SA adder table vs. cool temp
	LDAA    L0006			;Cool temp, linear
	JSR     LF153			;2D lookup with line count
;
	STAA    L02EE         		;Save it, startup SA 
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Look up startup SA decay rate
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
	LDX     #L452C        		;Startup SA decay delay timer
	LDAA    L0006			;Cool temp
	JSR     LF153			;2D lookup w/ line count
;
	STAA    L02EF			;Startup SA decay delay timer
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Look up various initial AFR parameters
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
	LDX     #L4C93			;AFR crank transition ratio vs. cool temp
	LDAA    L0006			;Cool temo, linearized 
	JSR     LF15E			;2D lookup

	STAA    L02EC			;Save it, AFR crank transition ratio
;
	LDAB    L491F           	;Time after initial startup to decay out choke AFR
	STAB    L02ED			;Choke AFR decay delay timer
;
;-Load up choke AFR decay rate
;
LE136	LDX     #L4CA4			;Choke AFR decay rate vs. cool temp
	LDAA    L0006			;Cool temp, linearized
	JSR     LF15E			;2D lookup
;
	STAA    L028A			;Save it, AFR crank transition ratio decay rate
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Look up choke SA decay multiplier
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
	LDX     #L4536			;Choke SA decay 
	LDAA    L0006			;Cool temp, linearized 
	JSR     LF153			;2D lookup w/ line count
;
	STAA    L0222			;Startup SA decay multiplier
;
	JMP     LE237			;Jump to return										
;
;~~~~~~~~~~~~~~~~~~~
; Engine is running 
;~~~~~~~~~~~~~~~~~~~
;
;-Update status bits for O2 handing at idle
;
LE14F	SEI				;Disable interrupts
	LDAB    L003F			;Status word
	ANDB    #$F0			;Clear lower four bits
	LDAA    L0066			;RPM/12.5, one delayed
	SUBA    L0063			;Subtract RPM/12.5
	BCC     LE15D			;Bra if RPMs rising
;
;-RPMs falling here
;
	ORAB    #$01			;Status word, set b1, RPMs falling 
	NEGA				;Get abs. value of result
;
LE15D	CMPA    L494A			;Delta RPM threshold for stable idle
	BCS     LE164			;Bra if delta RPMs <
;											
	ORAB    #$02			;Set b1, RPMs changing too rapildy for idle
;
LE164	LDAA    L0063			;RPM/12.5
	STAA    L0066			;Save it, RPM/12.5, one delayed
;
	LDAA    L0858			;Delta RPM from target idle speed
	CMPA    L4FB4			;12.5 RPM, upper RPM limit for steady state idle
	BLS     LE172			;Bra if delta RPMs less
;
;-Outside of desired idle range
;
	ORAB    #$08			;Set b3, outside target idle range
;
;-Check for high idle
;
LE172	BRCLR   L0036,#$80,LE178	;Status word, bra if b7==0, no high idle speed
;
	ORAB    #$04			;Set b2, high idle speed
;
LE178	STAB    L003F			;Save it
	CLI				;Re-enable interrupts
;
;~~~~~~~~~~~~~~~~~~~~~~~~~
; Perform choke functions
;~~~~~~~~~~~~~~~~~~~~~~~~~
;
	LDAA    L02ED			;Time after initial startup to decay choke AFR
	BNE     LE1F4			;Bra if !=0, dont decay out yet
;
;-Enough time has passed to begin decaying out AFR
;
	LDAA    L02EB			;Load choke AFR
	BNE     LE188			;Bra if !=0
;
;-Choke AFR has been decayed out
;
	JMP     LE1FC			;Bra to continue
;
;-Choke AFR has not been decayed out yet
;
LE188	LDAA    L02A0			;Choke AFR park to drive transition timer
	BEQ     LE1A6			;Bra if ==0, continue
;
	CMPA    L494F			;Park to drive transition period 
					;transition
	BNE     LE19D			;Bra if !=, continue to count down
;
;-Park to drive transition time met, check temp thresholds
;
	LDAB    L0006			;Cool temp
	CMPB    L494D			;Upper cool temp limit for cold choke park to
					;drive transition
	BHI     LE19D			;Bra if cool temp was >
;
	BRSET   L0041,#$20,LE1FC	;Status word, bra if b5==1, in P/N
;
;-Here to decrement park to drive transition timer
;
LE19D	DECA				;-1
	STAA    L02A0			;Choke AFR park to drive transition timer
	BNE     LE1FC			;Bra if !=0, dont decay out choke yet
;
;-Park to drive transition complete
;
	BSET    L003E,#$08		;Status word, set b3, cold choke decay delay 
					;complete 
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Time to decay out the choke
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;-Check choke decay period
;											
LE1A6	LDAA    L0293			;Choke AFR decay period
	BNE     LE1F9			;Bra if !=0, dont decay out choke this pass
;
;-Time to decay out the choke, look up decay period
;
	LDX     #L4C3A			;Choke AFR decay period vs. cool temp 
	LDAA    L0006			;Cool temp, linearized
	JSR     LF15E			;2D lookup
;
	STAA    L0293			;Choke AFR decay period
;
;-Adjust decay period based on airflow
;
	LDX     #L4C4B			;Decay period multiplier vs. airflow
	LDAA    L0255			;Gms/sec airflow
	CMPA    #64			;64 g/sec, UL for lookup
	BLS     LE1C2			;Bra if airflow <=
;
	LDAA    #64			;64 g/sec, UL for lookup
;
LE1C2	JSR     LF15E			;2D lookup
;
	TAB				;Get multiplier into B
	LDAA    L0293			;Get decay period
	MUL				;Decay period x airflow modifier 
	ASLD				;x2, now /128
	BCC     LE1CF			;Bra if no overflow

	LDAA    #255			;Load UL
;
LE1CF	STAA    L0293			;Save it, choke AFR decay period
	LDAA    L02EB			;Load choke AFR
	LDAB    L491E			;Choke AFR decay multiplier
	MUL				;Choke AFR x decay multiplier
	BRSET   L003E,#$08,LE1EF	;Bra if b3==1, cold park to drive transition 
					;complete
;
;-Choke AFR park to drive transition is still in effect
;
	CMPA    L494E			;Lower limit for choke enrichment during 
					;transition
	BHI     LE1EF			;Bra if AFR is >, save
;
;-Low/no choke enrichment present, check temp limit to enable transition
;
	LDAB    L0006			;Cool temp, linearized
	CMPB    L494D			;Upper cool temp limit to enable park to 
					;drive choke AFR transition
	BHI     LE1EF			;Bra if cool temp >, no transition
;
	LDAB    L494F			;Park to drive transition period                			
	STAB    L02A0			;Save it
;
LE1EF	STAA    L02EB			;Save it, choke AFR
	BRA     LE1FC			;Bra to continue
;
;-Here to decr. choke AFR decay delay timer
;
LE1F4	DEC     L02ED			;Choke AFR decay delay timer
	BRA     LE1FC			;Br to continue
;
;-Here to decr. choke AFR decay period
;
LE1F9	DEC     L0293  			;Choke AFR decay period
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Perform added initial spark functions
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LE1FC	LDAA    L02EF			;Added intial SA decay delay timer
	BEQ     LE20C			;Bra if ==0, decay out SA
;
	LDAA    L0002			;Loop counter
	ANDA    #$F0			;Mask out lower four bits
	BNE     LE237			;Bra if ==0, dont decrement timer yet
;
;-Here to decr. timer
;
	DEC     L02EF			;Initial SA decay delay timer
	BRA     LE237			;Bra to continue
;
;-Here to continue decaying out added SA
;
LE20C	LDAA    L02EE			;Initial added SA
	BEQ     LE237			;Bra if ==0
;
;-Decay out SA
;
	LDAA    L0223			;Initial added SA decay period
	BEQ     LE21B			;Bra if decay period ==0, decay out SA
;
	DEC     L0223			;SA decay period
	BRA     LE237			;Bra to continue
;
;-Look up SA decay period
;
LE21B	LDAA    L0255			;Gms/sec airflow
	CMPA    #64			;64 gms/sec, limit for lookup
	BLS     LE224			;Bra if airflow is less
;
	LDAA    #64			;use upper limit instead
;
LE224	LDX     #L44C6			;SA decay period vs. airflow
	JSR     LF15E			;2D lookup
;
	STAA    L0223			;Save it, SA decay period
;
	LDAA    L02EE			;Added initial SA
	LDAB    L0222			;Added initial SA decay multiplier
	MUL				;SA x multiplier
	STAA    L02EE			;Save it
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Check quals for entering closed loop
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LE237	BRCLR   L004F,#$10,LE250	;Status word, bra if b4==0, engine still off/
					;cranking
;
;-Chekc for O2 errors
;  
	BRCLR   L0019,#$0C,LE241	;Error word #4, clear b2, err 44, lean O2, and	
					;b3=err. 45, rich O2
;
;-O2 errors here, clear dwell timer and disable closed loop 
;
	BRA     LE24D			;Clear O2 timer and continue
;
;-No O2 errors here, check if errors pending
;
LE241	BRSET   L0072,#$40,LE24D	;Bra if b6==1, lean O2
;
;-Check for MAP errors
;
	BRCLR   L0018,#$30,LE253	;Error word #3, bra if b4==0, err 34, low map
					;and b5==0, err 35, high MAP
;
;-High or low MAP errors present, continue normally if not idling
;

	BRCLR   L0050,#$80,LE253	;Status word, bra if not idling
;
;-Here if idling, clear O2 dwell timer and disable closed loop
;
LE24D	CLR     L0264			;Clear timer	
;
;-Here to disable closed loop
;
LE250	JMP     LE2D6		;Jump to disable closed loop
;
;-Perform ALDL diags. if needed
;
LE253	BRCLR   L003B,#$10,LE264	;Bra if not b4, in mode 4 
;
;-In mode 4
;
	LDX     #L0393			;
	BRCLR   0,X,#$01,LE264		; 
;
	BRCLR   1,X,#$01,LE2D6		;
;									   
	BRA     LE2BC			;
;
;-Here if no diags. needed
;							
LE264	LDAA    L0264			;O2 dwell timer
	CMPA    L48C8			;Max time allowed for O2 to remain in window										
	BCC     LE272			;Bra if timer was >=, O2 is dwelling around stoich 
;
	INCA				;+1
	STAA    L0264			;Save it, O2 dwell timer
	BRA     LE275			;Bra to continue
;
;-O2 is dwelling in window
;
LE272	BCLR    L0004,#$01		;Status word, clear b0, O2 no longer functional
;
;-Here if dwell time in O2 window still within limits
;
LE275	BRCLR   L0004,#$01,LE2D6	;Status word, bra if b0==0, O2 no longer functional
;
;-O2 is still functional here
;
	BRSET   L0050,#$10,LE2AA	;Status word, bra if b4==1, in diags. mode
;
;-Not in diags. mode
;
	BRSET   L0004,#$02,LE2AA	;Status word, set b1, quals for closed loop fueling
					;met
;
	LDD     L00F8			;Engine run time
	LSRD				;/2
;
	LDAA    L02A5			;Initial cool temp (engine off)
	CMPA    L48C2			;Threshold for hot restart
	BLS     LE293			;Bra if cool temp was <
;
;-Hot restart in effect
;
	CMPB    L48C5			;Min time for full closed loop (hot restart)
	BCS     LE2CB			;Bra if time <, dont allow full closed loop
;
;-Here to enter closed loop
;
	BRA     LE2A4			;Go enter closed loop
;
;-Here if cool restart in effect
;
LE293	CMPA    L48C1			;Upper temp threshold for extended closed loop delay
	BHI     LE29F			;Bra if cool temp >
;
;-Cold engine, have extended closed loop delay
;
	CMPB    L48C4			;Min time for closed loop (cold engine)
	BCC     LE2A4			;Bra if run time is >=, enter closed loop
;
	BRA     LE2CB			;Bra if time <, no closed loop yet
;
;-Here if cool engine
;
LE29F	CMPB    L48C3			;Min time for closed loop (cool engine) 
	BCS     LE2CB			;Bra if time <, no closed loop yet
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Here to enter closed loop
;~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LE2A4	BSET    L0004,#$02		;Status word, set b1, quals for closed loop fueling
					;met
	BCLR    L0052,#$80		;Status word, clear b7, disable quasi-closed loop
					;for closed loop
;
LE2AA	LDAA    L0006			;Cool temp, 
	CMPA    L48C6			;Min temp to enter closed loop
	BLS	LE2D9			;Bra if cool temp is <=
;	
	BRCLR   L0082,#$40,LE2BC	;Status word, bra if b6==0, no HU	
;
;-Here for HU
;
	LDAA    L0801			;
	ANDA    #$03			;
	BNE     LE2D9			;
;
LE2BC	BSET    L0046,#$80		;Status word, set b7, have entered closed loop since 
					;startup
	BSET    L003E,#$A0		;Status word, set b5, and b7, closed loop enabled
	BRCLR   L0050,#$02,LE2DE	;Status word, bra if b1==0, open loop idle not enabled
;
;-Open loop idle in effect
;
	BCLR    L003E,#$82		;Clear b7, closed loop fueling, and b1, BLM update
					;enable flag
	BRA     LE321			;Bra to return
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Here for quasi-closed loop
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LE2CB	BSET    L0052,#$80		;Status word, set b7, quasi-closed loop enabled
;
	LDAA    L026D			;AFR
	CMPA    L48DF			;Lower AFR threshold for quasi-closed loop
	BCC     LE2D9			;Bra if AFR >=, allow quasi-closed loop 
;
;-Disable quasi-closed loop 
;
LE2D6	BCLR    L0052,#$80		;Clear b7, no quasi-closed loop idle if richer
					;then stoich
;
;-Clear closed loop flags
;
LE2D9	BCLR    L003E,#$A2		;Status word, clear b7, closed loop flag, b5, 
					;b1, BLM update enable flag
	BRA     LE321			;Bra to return
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; Subroutine to check quals for BLM update enable
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;-Check quals for BLM updats
;
LE2DE	LDAA    L006F			;Status word
	ANDA    #$48			;Test b6, high MAP error, and b3, low MAP error
	BNE     LE31E			;Bra if !=0, errors set
;
;-Check cool temp
;
	LDAA    L0006			;Cool temp
	CMPA    L48CC			;Min temp for BLM update enable
	BLS     LE31E			;Bra if cool temp is <=
;
;-Check AFR
;
	LDAA    L026D			;AFR
	CMPA    L48DD			;Stoich AFR
	BNE     LE31E			;Bra if AFR is something other then stoich, return
;
;-Check vacuum
;
	LDAA    L01EC			;kPa vacuum
	CMPA    L48C9			;Max vac for BLM update enable
	BHI     LE31E			;Bra if vacuum is greater, no BLM update
;
;-Check MAP
;
	LDAA    L01F2			;Load MAP/baro ratio
	LDAB    L400E			;Opt. byte
	BITB    #$01			;Test b1, use MAP/baro ratio						
	BNE     LE308			;Bra if ==1, use MAP/baro ratio
;
	LDAA    L01E9			;Load A/D MAP
;
LE308	CMPA    L48CB			;Upper MAP limit for BLM update enable
	BHI     LE31E			;Bra if >, no BLM update
;
	CMPA    L48CA			;Lower MAP limit for BLM update enable
	BCS     LE31E			;Bra if MAP >, no BLM update
;
;-Check RPMs
;
	LDAA    L0062			;RPMs/25
	CMPA    L48CD			;Upper RPM limit for BLM update enable
	BCC     LE31E			;Bra if RPMs >=
;
;-Here to enable BLM updates
;
	BSET    L003E,#$02		;Status word, set b1, BLM update enable flag
	BRA     LE321			;Bra to return
;
LE31E	BCLR    L003E,#$02		;Clear b1, dont allow BLM updates
;
LE321	RTS				;Return   
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; Rountine to check range select 
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LE322	LDAB    L009C			;Status word
	ANDB    #$07			;Clear all but b2-b0
	BNE     LE32A			;Bra if any set
;
;-In overdrive here
;
	LDAB    #$06			;Offset for last table
;
;In manual 1st/2nd, drive here
;
LE32A	LSRB				;/2, shift range
	PSHA				;Push gear number to stack
	LDAA    #$04			;4 bytes/table
	MUL				;Range select x table offset
	LDX     #LE33C			;Addr of first table
	ABX				;Add in offset, select appropriate table
	PULB				;Get gear # back off stack
	ABX				;Add it in as offset
	LDAA    0,X			;Get allowed gear off stack
	CBA				;Compare current gear # to looked up val
	BLS     LE33B			;Bra if gear # less
;	
	TBA				;Upper gear limit to A
;	
LE33B	RTS				;Return     					
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Table of upper gear limits for gear selected
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;                   
;	Allowed gear #		;Current gear
;
;-Manual 1st
;
LE33C   FCB  0			;1st
	FCB  1			;2nd 
	FCB  1			;3rd
	FCB  1			;4th
;
;-Manual second
;
	FCB  0			;1st 
	FCB  1			;2nd
	FCB  1			;3rd
	FCB  1			;4th
;
;-Drive
;
	FCB  0			;1st
	FCB  1			;2nd
	FCB  2			;3rd
	FCB  2			;4th
;
;-Overdrive
;
	FCB  0			;1st
	FCB  1			;2nd
	FCB  2			;3rd
	FCB  3			;4th
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; Routine to calculate transmission output shaft rotation speed
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LE34C	LDAA    $08,Y			;Load MSB of accumulated pulse counts
	SUBA    0,Y			;- stored pulse counts, one delayed
	BNE     LE364			;Bra if !=0, edges have been detected
;
;-No new edges have been detected, check time allowed for signal to be present
;
	LDAB    3,Y			;Load timer to wait for TOS signal
	BEQ     LE35F			;Bra if ==0, signal not present
;
	DECB				;-1 from timer
	STAB    3,Y			;Save it
	BNE     LE38E			;Bra if !=0, return without updating
;
;-Signal no longer present, clear output speed
;
LE35F	STD     4,Y			;Clear trans output speed
	BRA     LE38E			;Bra to return
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; New edges have been detected
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LE364	PSHB				;Push max allowed period to stack
	LDAB    3,Y			;Load timer to wait for TOS signal
	BEQ     LE37B			;Bra if ==0
;
	TAB				;Get delta pulse counts into B, delta pulse counts
					;= [(RPM x 40/60) x (elapsed timer/65536)]
	CLRA				;
	ASLD				;x2
	ASLD				;x2
	ASLD				;x2, now x8, 
	FDIV				;65536 x [8 x (RPM x 40/60) x (elapsed timer/65536)]/
					;43946, now = RPM x (elapsed timer/65536) x 8
	LDD     6,Y			;Load captured free running counter value
	SUBD    1,Y			;Subtract counter val., one delayed, now time between
					;calculation interval, which is secs x 65536
	XGDX				;Swap revs into D, elapsed time into X
	FDIV				;[RPM x (elapsed timer/65536) x 8] / 
					;(elapsed timer/65536) = TOS RPM x 8
	STX     4,Y			;Save result, TOS RPM x 8
;
LE37B	LDX     6,Y			;Load captured free running counter
	STX     1,Y			;Save it, counter, one delayed
;									
	LDAA    $08,Y			;Load accumulated pulses
	STAA    0,Y			;Save it, accumulated pulses, one delayed
	PULA				;Get max time to wait for TOS signal back
	STAA    3,Y			;Reset time to wait for TOS signal timer
	LDD     4,Y			;Load trans output speed
;											
LE38E	RTS				;Return     					
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; Routine to check if TPS errors flagged
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;-Check for error 21
;
LE38F	LDAA    L5B34			;Trans error filter word #1
	COMA				;Invert selection    
	ANDA    L5B2B			;And with trans error word #1 mask
	ANDA    L0016			;And with error word #1
	ANDA    #$0001			;And with $01, filter all but error 21
	BNE     LE3A9			;Bra if !=0, TPS error set
;
;-Check for error 22
;
	LDAA    L5B35			;Trans error filter word #2
	COMA				;Invert selection    
	ANDA    L5B2C			;And with trans error word #2 mask
	ANDA    L0017			;And with error word #2
	ANDA    #$80			;And with b7, error 22 flag, high TPS error
	BEQ     LE3AD			;Bra if error not set or being flagged
;
LE3A9	BSET    L0092,#$01		;Status word, set b0, TPS errors set
	RTS				;Return
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~     
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LE3AD	BCLR    L0092,#$01		;
	RTS				;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; Routine to check for high system voltages
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LE3B1	LDAA    L5B2F			;Diag. word #5 mask
	ANDA    L001A			;And with error word #5		
	ANDA    #$18			;Mask out errors 52, 53
	BNE     LE3C3			;Bra if errors set
;
	BRSET   L0082,#$80,LE3C7	;Status word, bra if b7==1, if powertrain in motion
;
;-Powertrain not in motion here here
;
	TST     L0100			;
	BNE     LE3C7			;
;
;-Here to flag error
;
LE3C3	BSET    L0093,#$08		;High system voltage present
	RTS				;Return     
;
;-No error/powertrian currently in motion, dont flag error
;
LE3C7	BCLR    L0093,#$08		;High system voltage present
	RTS				;Return     
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; Routine to filter and flag VSS/TOS errors
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LE3CB	LDAA    L5B35			;Load diag. word mask #2
	COMA				;Invert bits    
	ANDA    L5B2C			;And with error word mask #2
	ANDA    L0017			;And with error word
	ANDA    #$20			;And with b5, error 24, VSS
	BNE     LE3E5			;Bra if b5 !=0, error set
;
	LDAA    L5B3A			;Trans error filter mask
	COMA				;Invert selection    
	ANDA    L5B31			;Trans diag. mask
	ANDA    L001C			;And with error word #7
	ANDA    #$04			;Clear all but b2, error 72, trans output speed lost
	BEQ     LE3E9			;Bra if b2==0, no errors flagged
;
LE3E5	BSET    L0092,#$02		;Status word, set b1, VSS/TOS errors set
	RTS				;Return     
;
LE3E9	BCLR    L0092,#$02		;Clear b1, no VSS/TOS errors present
	RTS				;Return     
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; Routine to filter out related errors
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LE3ED	LDY     #L5B00			;Start of diag. words
	LDAB    #$09			;Last diag. word
;
;-Loop here to check for errors
;
LE3F3	LDAA    $3C,Y			;Load diag. word
	ANDA    $33,Y			;And with diag. word mask
	LDX     #L0015			;Start addr. for error words
	ABX				;Add in current offset     
	ANDA    0,X			;And with current error word
	LDX     #LE4CF-1		;Start addr. of filter masks
	ABX				;Add in offset
	ANDA    0,X			;And with filter mask
	BNE     LE410			;Bra if errors set
;
;-No errors here, continue
;
	DEY				;-1, go to previous diag. word
	DECB				;-1, previous error word/diag. mask
	BNE     LE3F3			;Bra if !=0, loop to check rest of errors
;
;-No errors found
;
	BCLR    L0092,#$08		;Status word, clear b3, 
					;no general trans related faults
	RTS     
;
;-Errors found here
;
LE410	BSET    L0092,#$08		;Set b3, general trans related faults 
	RTS				;Return     
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;   
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; 
; Routine to check for errors applicable to adaptive modifier
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LE414	LDX     #LE4D7			;Index diag. masks
	JSR     LE47C			;Routine to check and filter errors
;
	BNE     LE420			;Bra if !=0, errors set
;
	BCLR    L0092,#$10		;Status word, clear b4, no errors applicable to
					;adaptive modifier set
	RTS				;Return    
;
LE420	BSET    L0092,#$10		;Set b4, flag errors applicable to adaptive modifier
	RTS				;Return     
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;   
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; Routine to check if or errors applicable to force motor/line pressure control
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LE424	LDX     #LE4E0			;Index diag. masks
	JSR     LE47C			;
;											
	BNE     LE430			;Bra if result !=0, errors set
;											
	BCLR    L0092,#$20		; CLR b4, XMISH ERR
	RTS				;Return
;
LE430	BSET    L0092,#$20		; SET b4, XMISH ERR
	RTS				;     
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;   
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; Routine to check for errors applicable to TCC
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LE434	LDX     #LE4E9			;Index table for filtering trans errors
	JSR     LE47C			;Routine to filter trans errors
;
	BNE     LE440			;Bra if applicable errors where set
;
	BCLR    L0092,#$40		;Status word, clear b6, no errors affecing TCC present
;
	RTS				;Return    
;
;-Here if errors set
;
LE440	BSET    L0092,#$40		;Set b6, errors set 
	RTS				;Return     
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; Check for heat related trans errors
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LE444	BRSET   L001D,#$80,LE458	;Bra if b7==1, error 75, system voltage low
;
	BRCLR   L0083,#$02,LE454	;Bra if b1==0, trans not above hot temp threshold
;
;-Trans above hot temp threshold here, check for heat related errors
;
	LDX     #LE504			;Start of error word filter masks
	JSR     LE47C			;Routine to check for and filter errors
;
	BNE     LE458			;Bra if any errors set
;
LE454	BCLR    L0092,#$80		;No heat related transmission errors set
	RTS				;Return  
;
LE458	BSET    L0092,#$80		;Heat related transmission errors set
	RTS				;Return  
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; Check for external errors that effect trans
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LE45C	LDX     #LE4F2			;Start of error word filter masks
	JSR     LE47C			;Bra to filtere out related errors
;
	BNE     LE468			;Bra if errors set
;
;-No errors set here
;
	BCLR    L0093,#$01		;Status word, no external errors set
	RTS				;Return     
;
;-Errors set here
;
LE468	BSET    L0093,#$01		;External errors set
	RTS				;Return
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; Check for trans related errors
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LE46C	LDX     #LE4FB			;Start of error word filter masks
	JSR     LE47C			;Bra to filter out related errors
;
	BNE     LE478			;Bra if errors set
;
;-No errors set here
;
	BCLR    L0093,#$02		;Status word, clear b2, shift error not set 
	RTS				;Bra to return     
;
;-Errors set here
;
LE478	BSET    L0093,#$02		;Status word, clear b2, shift error set 
	RTS				;Return  
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~   
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; Routine to filter out related errors
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LE47C	PSHY				;Y to stack    
	PSHB				;B sto stack    
	LDY     #L5B00			;Start addr. of diag. masks
	LDAB    #$09			;Start with error word #10
	ABX				;Add in offset 
;
;-Loop here to check for errors
;    
LE486	LDAA    $3C,Y			;Load in diag. word 			
	COMA				;Ones comp.    
	ANDA    $33,Y			;And with diag. word mask
	PSHX				;Push X to stack    
	LDX     #L0015			;Start address for error words
	ABX				;Add in the offset     
	ANDA    0,X			;And in error word, clear out any codes not set
	PULX				;Get addr. of table of filter masks    
	ANDA    0,X			;And those in to filter out related errors
	BNE     LE49F			;Bra if result !=0, errors found
;
;-Here if no errors found yet, continue
;
	DEX				;-1, go to previous error word     
	DEY				;-1, go to previous diag. word/mask     
	DECB				;-1, go to previous error filter mask  
	BNE     LE486			;Bra if B !=0, not done checking all codes yet
;
;-Here to exit with error/return
;
LE49F	PULB				;Restore B    
	PULY				;Restore Y
	TSTA				;Test for set errors before returning
	RTS				;Return with results in cond. code reg.
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~  
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; Routine to check for error 72
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LE4A4	LDAA    L5B3A			;Transmission err. word #7
	COMA				;Ones comp.   
	ANDA    L5B31			;transmission err. word #7 mask, mask errors being recognized
	ANDA    L001C			;Err. word #7, mask errors flagged
	ANDA    #$04			;Mask out b2, error 72, output speed loss		
	BEQ     LE4B5			;Bra if error not flagged
;
;-Error 72 flagged
;
	BSET    L0093,#$04		;Status word, set b2, error, output speed loss
	RTS				;Return     
;
;-Error 72 not flagged
;
LE4B5	BCLR    L0093,#$04		;Clear b2, no error
	RTS				;Return
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~     
;
;~~~~~~~~~~~~~~~~~~~~~~~~
;
; Not used...
;
;~~~~~~~~~~~~~~~~~~~~~~~~
;
LE4B9     RTS				;Return     
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; Routine to check for error 28
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LE4BA	LDAA    L5B35			;Error word #2
	COMA				;1's comp.    
	ANDA    L5B2C			;Error word #2 mask, mask errors being recognized
	ANDA    L0017			;Error word #2, mask errors flagged
	ANDA    #$02			;Mask out error 28, trans press. manifold switch
	BEQ     LE4CB			;Bra if error not set
;
;-Error 28 here
;
	BSET    L0092,#$04		;Status word, set b2, trans press. switch man. error
	RTS				;Return     
;
;-No error 28 here
;
LE4CB	BCLR    L0092,#$04		;Clear b2, trans press. switch man. error 
	RTS				;Return
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~     
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Diag. filter masks, used to filter out general trans related faults
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;	binary 1=flag error	;Err. word #
;
LE4CF   FCB $01			;1
	FCB $A2			;2
	FCB $00			;3
	FCB $00			;4
	FCB $00			;5
	FCB $00			;6	
	FCB $3D			;7
	FCB $87			;8
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Diag. filter masks, used to filter out errors applicable to adaptive modifier
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;	binary 1=flag error	;Err. word #
;
LE4D7   FCB $44			;1
	FCB $61     		;2
	FCB $F2			;3
	FCB $70			;4
	FCB $3C			;5
	FCB $18			;6
	FCB $66			;7
	FCB $B7			;8
	FCB $8E			;9
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Diag. filter masks, used to filter out external errors for line pressure/force motor
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; 1=flagging erorr		;Associated error word
;
LE4E0   FCB $74			;
	FCB $01			;
	FCB $A0			;
	FCB $00			;
	FCB $00			;
	FCB $00			;
	FCB $00			;
	FCB $A4			;
	FCB $06			;
;				
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Diag. filter masks, used to filter out external errors for TCC
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; 1=flagging error		;Associated error word
;
LE4E9   FCB $74			;1
	FCB $01			;2
	FCB $82			;3
	FCB $01			;4
	FCB $00			;5
	FCB $08			;6
	FCB $00			;7
	FCB $69			;8
	FCB $85			;9
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Diag. filter mask, used to filter out external errors
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; binary 1=flag error		;Associated error word #
;
LE4F2   FCB $30			;1
	FCB $00			;2
	FCB $20			;3
	FCB $00			;4
	FCB $00			;5
	FCB $00			;6
	FCB $00			;7
	FCB $80			;8
	FCB $80			;9
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Diag. filter mask, used to filter out trans related errors
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; binary 1=flag error		;Associated error word #
;
LE4FB   FCB $00			;1
	FCB $00			;2
	FCB $00			;3
	FCB $00			;4
	FCB $00			;5
	FCB $08			;6
	FCB $00			;7
	FCB $00			;8
	FCB $04			;9
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Diag. filter masks, used to filter out heat related trans errors
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; binary 1=flag error		;Associated error word #
;
LE504   FCB $30     		;1
	FCB $01    		;2
	FCB $82			;3
	FCB $03			;4
	FCB $00			;5
	FCB $00			;6
	FCB $00			;7
	FCB $68			;8
	FCB $01			;9
;
LE50D	FCB $00			;
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; Routine to check for error 14, high cool temp
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LE50E	BRSET   L003B,#$04,LE541	;Status word, bra if b2==1, trans diags disabled
;
	LDX     #L5B00			;Index diag. word masks
	BRCLR   0,X,#$40,LE53D		;Bra if b6==0, not setting error 14, high cool temp
;
;-Check params for setting error 14, high cool temp
;
	LDAA    L0159			;Load error 14 timer
;
	LDAB    L082D			;Load A/D cool temp
	CMPB    $13,X			;Compare to threshold to set error
	BCS     LE531			;Bra if <, set error
;
;-Not setting error
;
	BRCLR   L0016,#$40,LE53D	;Error word #1, bra if error not previously flagged
;
	INCA				;+1, error 14 timer
	CMPA    $14,X			;Time threshold to set/clear error
	BCS     LE53E			;Bra if time is <
;
;-Clear error 14 if previously set
;
	BCLR    L0016,#$40		;Clear b6, error 14, high cool temp
	BRA     LE53D			;Bra to return
;
;-Setting error here
;
LE531	BRSET   L0016,#$40,LE53D	;Error word #1, set b6, error 14, high cool temp
;
	INCA				;+1, error 14 timer
	CMPA    $14,X			;Time threshold to set error
	BCS     LE53E			;Bra if <, dont set error
;
	BSET    L0016,#$40		;Set b6, error 14, high cool temp
;
LE53D	CLRA				;Reset timer
;
LE53E	STAA    L0159			;Save timer
;
LE541	RTS				;Return     
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; Routine to check for error 15, low cool temp
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; 
LE542	BRSET   L003B,#$04,LE575	;Status word, bra if b2==1, diagnostics disabled
;
	LDX     #L5B00			;Intex diag mask
	BRCLR   0,X,#$20,LE571		;Bra if b5==1, not flagging error 15, low cool temp
;
;-Flagging error
;
	LDAA    L015A			;Load error 15 timer
	LDAB    L082D			;Load A/D cool temp
	CMPB    $15,X			;Threshold to set error
	BHI     LE565			;Bra if >, set error
;
;-Not setting error
;
	BRCLR   L0016,#$20,LE571	;Error word #1, bra if b5==0, error not previously set
;
	INCA				;+1, error 15 timer
	CMPA    $16,X			;Time threshold to set/clear error
	BCS     LE572			;Bra if timer <, dont clear error yet
;
	BCLR    L0016,#$20		;Clear b5, error 15, low cool temp
	BRA     LE571			;Bra to return
;
;-Setting error
;
LE565	BRSET   L0016,#$20,LE571	;Status word, bra if b5==1, error 15, low cool temp
;
	INCA				;+1, error 15 timer
	CMPA    $16,X			;Time threshold to set/clear error
	BCS     LE572			;Bra if timer <
;
	BSET    L0016,#$20		;Set b5, erorr 15, low cool temp
;
LE571	CLRA				;Reset timer
;
LE572	STAA    L015A			;Save it
;
LE575	RTS				;Return     
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; Routine to check for error 21, high TPS
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LE576	BRSET   L003B,#$04,LE5B6	;Status word, bra if b2, diagnostics disabled
;
	LDX     #L5B00			;Index diagnostics mask
	BRCLR   0,X,#$01,LE5B2		;Bra if b0==0, not flagging error 21, high TPS
;	
;-Flagging error
;										
	LDAA    L015B			;Load error 21 timer
;											
	LDAB    L00A0			;Load A/D TPS
	CMPB    $1C,X			;Compare to threshold to set error
	BHI     LE5A5			;Bra if TPS >, set error
;
;-Not setting error
;
	BRSET   L008E,#$01,LE5B2	;Status word, bra if b0==1, error 21 fault counter
					;incremented already 
;
	BSET    L008E,#$01		;Set b0, error 21 fault counter incremented										;
	BRCLR   L0016,#$01,LE5B2	;Error word #1, bra if error 21 not previousy set
;
;-Error previously set
;
	LDAB    L001F			;Load error 21 fault counter
	CMPB    L5B1E			;Compare to threshold to permanently set error 21, 
					;high TPS
	BCC     LE5B2			;Bra if counter is >=, leave error set
;
	INCB				;+1
	INCB				;+1, one additional logged error on start/run
	STAB    L001F			;Save it, error 21 fault counter
;										
	BCLR    L0016,#$01 		;Clear b0, error 21, high TPS
	BRA     LE5B2			;Bra to return
;
;-Setting error here
;
LE5A5	BRSET   L0016,#$01,LE5B2	;Bra if b0==0 error already set
;
	INCA				;Inc. error timer   
	CMPA    L5B1D			;Compare to time threshold to set error
	BCS     LE5B3			;Bra if time <, dont set error yet
;
	BSET    L0016,#$01 		;Set b0, error 21, high TPS
;
LE5B2	CLRA				;Reset error timer
;
LE5B3	STAA    L015B			;Save it, error 21 timer
;
LE5B6	RTS				;Return     
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; Routine to check for error 22, low TPS
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; 
LE5B7	BRSET   L003B,#$04,LE5F5	;Status word, bra if b2==1, diags. disabled
;
	LDX     #L5B00 			;Index error word mask
	BRCLR   1,X,#$80,LE5F1		;Bra if b7==0, not flagging error 22, low TPS 
;
	LDAA    L015C			;Error 22 timer
	LDAB    L00A0			;A/D TPS
	CMPB    $1F,X			;Threshold to set error
	BCS     LE5E5			;Bra if TPS <, set error
;
;-Not setting error here
;
	BRSET   L008E,#$02,LE5F1	;Status word, bra if b1==1, error 22 fault counter
					;incremented on this start/run
;
	BSET    L008E,#$02		;Set b1, error 22 fault counter incremented
;
;-Check if error previouslt set
;
	BRCLR   L0017,#$80,LE5F1	;Error word #2, bra if b7==0, error not previously
					;set
;
	LDAB    L0020			;Error 22 fault counter
	CMPB    $21,X			;Threshold to permanently set error
	BCC     LE5F1			;Bra if counter >=, dont reset error 22
;
	INCB				;+1,
	INCB				;+1, one additional logged error on start/run    
	STAB    L0020			;Save it
;
	BCLR    L0017,#$80		;Clear error 22, low TPS
	BRA     LE5F1			;Bra to return
;
;-Here if setting error
;
LE5E5	BRSET   L0017,#$80,LE5F1	;Bra if b7==1, error 22 already set
;
	INCA				;+1, error 22 timer
	CMPA    $20,X			;Time threshold to set error
	BCS     LE5F2			;Bra if timer <, dont set error yet
;
	BSET    L0017,#$80		;Error word #2, set error 22, low TPS

LE5F1	CLRA				;Reset error 22 timer
;
LE5F2	STAA    L015C			;Save it 
;
LE5F5	RTS				;Return     
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; 
; Routine to check for error 24, VSS low
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; 
LE5F6	BRSET   L003B,#$04,LE676	;Status word, bra if b2==1, trans diags. disabled
;
	LDX     #L5B00			;Load start addr. of error word masks
	BRCLR   1,X,#$20,LE672		;L5B01, bra if b5==0, not flagging error 24
;
;-Check if error optioned
;
	LDAA    L015D			;Load error 24 timer
	BRSET   L008E,#$04,LE61E	;Status word, bra if b2==1, error 24 logged for
					;this session already
;
;~~~~~~~~~~~~~~~~~~~~~
; Log past set errors
;~~~~~~~~~~~~~~~~~~~~~
;
	BSET    L008E,#$04		;Set b2, error 24 logged
;
;-Check if error previously set
;
	BRCLR   L0017,#$20,LE672	;Error word #2, bra if b5==0, error 24 not set
;
;-Error 24 set
;		
	LDAB    L0021			;Error 24 log counter
	CMPB    $42,X			;Compare to L5B42, threshold to permanently set
					;error 24
	BCC     LE672			;Bra if # of errors >, error to remain set
;
	INCB				;+1,
	INCB				;+1, log additional error
	STAB    L0021			;Save it, error 24 log counter
;
;-Reset error
;
	BCLR    L0017,#$20		;Clear b5, reset error 24 flag
	BRA     LE672			;Bra to reset and return
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Check quals for error 24 diags.
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LE61E	BRSET   L0017,#$20,LE672	;Bra if b5==1, error 24 flagged already 
;
;-Exit if errors set on sensors used for diags.
;
	BRSET   L0018,#$20,LE672	;Error word #3, bra if b5==1, error 33, MAP high error
					;set
;
	BRSET   L0018,#$10,LE672	;Bra if b4==1, error 34, MAP low error set
;
	BRSET   L0016,#$01,LE672	;Error word #1, bra if b0, error 21, TPS high error 
					;set
;
	BRSET   L0017,#$80,LE672	;Bra if b7==1 error 22, TPS low error set
;
;-Check MAP thresholds
;
	LDAB    L01E3			;Load SD MAP
	CMPB    L5B45			;Lower MAP threshold for diags.
	BCS     LE672			;Bra if MAP <, no diags.
;
	CMPB    L5B46			;Upper MAP threshold for diags.
	BHI     LE672			;Bra if MAP >, no diags.
;
;-Check TPS thresholds
;
	LDAB    L00AC			;Load trans TPS
	CMPB    L5B47			;Lower TPS threshold for diags.
	BCS     LE672			;Bra if TPS <
;
	CMPB    L5B48			;Upper TPS threshold for diags.
	BHI     LE672			;Bra if TPS >
;
;-Check min allowed TOS speed
;
	LDY     L00CB			;Load trans output speed RPM x 8
	CPY     $3D,X			;L5B3D, RPM threshold to set error
	BCC     LE672			;Bra if TOS RPMs >=
;
	BRSET   L001C,#$01,LE672	;Error word #7, bra if b0==1, error 74, trans input 
					;speed loss, set
;
	BRSET   L0017,#$02,LE672	;Error word #2, bra if b1==1, press. sw. manifold 
					;erorr set
;
;-Check engine speed 
;
	LDY     L019D			;Engine RPM x 8
	CPY     $3F,X			;L5B3F, lower threshold to set erorr 24
	BLS     LE672			;Bra if RPMs <=
;
	BRCLR   L009C,#$50,LE66A	;Trans range word, bra if b6, in P/N, and b4, invalid
					;range, ==0
;
;-In P/N or invalid range, dont set error 24
;					
	BRA     LE672
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Vehicle in motion, set error 24, VSS low
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LE66A	INCA				;+1, error 24 timer
	CMPA    $41,X			;L5B41, time threshold to set error 24
	BCS     LE673			;Bra if time threshold <
;
;-Set error 24
;
	BSET    L0017,#$20		;Set b5, error 24, low VSS
;											
LE672	CLRA				;Preclear A to reset timer
;
LE673	STAA    L015D			;Save it, error 24 timer
;
LE676	RTS				;Return     
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; Routine to check for error 28, pressure manifold switch
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~		
; 							
LE677	BRSET   L003B,#$04,LE6B3	; BR IF b2, XMISH DIAG DISABLED

	LDX     #L5B00			;Start of diag. masks
	BRCLR   1,X,#$02,LE6AF		;Bra if b1==0, not flagging error 28, press. switch
					;manifold error
;
	LDAA    L015E			;Error 28 timer
	BRSET   L009C,#$80,LE6A3	;Range word, bra if b7==1, invalid range returned
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-No errors set currenty, check if errors previously set
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
	BRSET   L008E,#$08,LE6AF	;Status word, bra if b3==1, error status flagged
					;for this session already
;
	BSET    L008E,#$08		;Set b3, error status checked
;
;-Check status of error 28 flag
;
	BRCLR   L0017,#$02,LE6AF	;Error word #2, bra if b1==0, error 28 not set
;
	LDAB    L0022			;+1, error 28 log counter
	CMPB    $52,X			;L5B52, # of errors for error 28 to remain perm.
					;set
	BCC     LE6AF			;Bra if # of errors >=, error to remain set
;
	INCB				;+1
	INCB				;+1, log error
	STAB    L0022			;Save it
;
	BCLR    L0017,#$02		;Clear b1, reset error 28 flag											
	BRA     LE6AF			;Bra to reset timer and return			
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Here to set error 28, pressure switch manifold error
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LE6A3	BRSET   L0017,#$02,LE6AF	;Error word #2, bra if b1==1, error 28 already
					;set
;
	INCA				;+1, error 28 timer
	CMPA    $51,X			;L5B51, time threshold to set error
	BCS     LE6B0			;Bra if time <
;
;-Time threshold met, set error
;
	BSET    L0017,#$02		;Set b1, error 28, pressure switch manifold error
;							
LE6AF	CLRA				;Clear A to reset error timer
;
LE6B0	STAA    L015E			;Save it
;
LE6B3	RTS				;Return     					
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~																						
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; Routine to check for error 37, brake switch stuck on
;							
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; 
LE6B4	BRSET   L003B,#$04,LE6E5	;Status word, bra if b2==1, trans diags. disabled

	LDX     #L5B00			;Index diag. masks
	BRCLR   2,X,#$02,LE6DF		;L5B02, bra if b1==0, not flagging error 37 
;
	BRSET   L0096,#$01,LE6E8	;Status word, bra if b0==1, brakes not currently 
					;applied
;
;-Brakes currently applied
;
	BCLR    L008E,#$30		;Status word, clear b4, vehicle stopped flag, and b5, 
					;vehicle in  motion flag
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Check if error 37 set previously
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
	BRCLR   L0018,#$02,LE6DF	;Error word #3, bra if b1==0, error 37 not set 
					;previously
;
	LDAA    L015F			;Error 37 fault counter
	INCA				;+1, additional fault set
	CMPA    $57,X			;L5B57, # of faults to set error 37
	BCS     LE6E2			;Bra if # of faults <
;
;-Sufficient faults to set error 37 and permanently log error
;	
	LDAB    L0023			;Load error 37 log counter
	CMPB    $58,X			;L5B58, threshold to permanently set error 37
	BCC     LE6DF			;Bra if # of errors set >=, error 37 to remain set
;
	INCB				;+1
	INCB				;+1
	STAB    L0023			;Save it, log additional error 37
;
;-Here if not leaving error 37 set
;
	BCLR    L0018,#$02		;Clear b1, error 37, brake switch stuck on
;
LE6DF	JMP     LE748			;Jump to reset and return
;
LE6E2	JMP     LE749			;Jump to save counter and return continue
;
LE6E5	JMP     LE74C			;Jump to return
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Brake switch indicating brakes not applied, perform diags
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LE6E8	BRSET   L0018,#$02,LE748	;Bra if b1==1, error 37 already set
;
	LDAB    L00D1			;Load MPH x 2
	CMPB    $53,X			;L5B53, threshold for vehicle to be stopped
	BHI     LE6FE			;Bra if MPH >, vehicle not yet stopped
;
;-Vehicle stopped
;
	BSET    L008E,#$10		;Status word, set b4, vehicle stopped flag
	BCLR    L008E,#$20		;Clear b5, vehicle in motion flag
	CLRA				;Preclear A
	STAA    L017B			;Save it, reset vehicle coasting timer
	BRA     LE74C			;Bra to return
;
;-Vehicle not yet stopped
;
LE6FE	BRCLR   L008E,#$10,LE74C	;Status word, bra if b4==0, vehicle was not previously
					;stopped, return
;
;-Vehicle was previously stopped
;
	CMPB    $54,X			;L5B54, lower MPH threshold for vehicle to be in 
					;motion
	BCC     LE71A			;Bra if MPH >=, vehicle now in motion
;
;-Vehicle coasting here
;
	BRSET   L008E,#$20,LE72E	;Status word, bra if vehicle previously in motion
;
;-Vehicle was stopped prior and not in motion, now coasting forward
;
	LDAA    L017B			;Vehicle coasting timer
	ADDA    #$01			;+1
	BEQ     LE715			;Bra if roll-over
;
	STAA    L017B			;Save it
	CLRA				;Preclear A 
    
LE715	STAA    L017C			;Save to reset vehicle in motion timer
	BRA     LE74C			;Bra to return
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Vehicle in motion and brake switch indicating that brakes where not applied
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LE71A	BSET    L008E,#$20		;Set b5, vehicle in motion
	LDAA    L017C			;Load vehicle in motion timer
	ADDA    #$01			;+1
	BEQ     LE727			;Bra if ==0, roll-over
;
	STAA    L017C			;Save it
;
LE727	LDAA    L017B			;Load vehicle coasting timer
	CMPA    $55,X			;L5B55, time threshold to flag fault
	BHI     LE733			;Bra if timer >, flag fault
;
;-Here if vehicle coasting to a stop, reset flags
;
LE72E	BCLR    L008E,#$30		;Clear b4, vehicle stopped flag, and b5, vehicle in
					;motion flag
	BRA     LE74C			;Bra to return
;
;-Vehicle had been coasting
;
LE733	LDAA    L017C			;Load vehicle in motion timer
	CMPA    $0056,X			;L5B56, threshold to flag fault
	BLS     LE74C			;Bra if <=, not enough time to set error
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Vehicle has been in motion, stopped, and then coasting in gear, assume brakes applied
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
	BCLR    L008E,#$30		;Clear b4, vehicle stopped flag, and b5, vehicle in 
					;motion flag
	LDAA    L015F			;Error 37 fault counter
	INCA				;+1    
	CMPA    $0057,X			;L5B57, # of faults to set error 37, brake switch
					;stuck on
	BCS     LE749			;Bra if # of errors <
;
;-Enough faults to set error 37
;
	BSET    L0018,#$02		;Set b1, error 37, brake switch stuck on
;
LE748	CLRA				;Preclear A to reset error 37 fault counter 
;   
LE749	STAA    L015F			;Save it, error 37 fault counter
;
LE74C	RTS				;Return
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~     
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; Routine to check for error 38, brake switch inop.
;							
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LE74D	BRSET   L003B,#$04,LE781	;Status word, bra if b2==1, trans diags. disabled
;
	LDX     #L5B00			;Index diag. masks
	BRCLR   2,X,#$01,LE77B		;L5B02, bra if b0==0, not flagging error 38 
;									
	BRCLR   L0096,#$01,LE784	;Status word, bra if b0==0, brakes applied
;
;-Brakes not applied, flag and reset if no error
;										
	BCLR    L008E,#$80		;Status word, clear b7, vehicle in motion flag
	BSET    L008E,#$40		;Set b6, vehicle coasting flag
	BRCLR   L0018,#$01,LE77B	;Error word #3, bra if b0==0, error 38 not set
;
;-Error 38 set previously
;
	LDAA    L0160			;Error 38 counter
	INCA				;+1
	CMPA    $5D,X			;L5B5D, number of brake switch faults to set error 38
	BCS     LE77E			;Bra if # of errors <
;
	LDAB    L0024			;Error 38 log counter
	CMPB    $5E,X			;# of errors for error 38 to remain set
	BCC     LE77B			;Bra if # of errors >=
;
	INCB				;+1    
	INCB				;+1, log additonal error    
	STAB    L0024			;Save it, error 38 counter
	BCLR    L0018,#$01		;Clear b1, reset error 38
;
LE77B	JMP     LE7EA			;Jump to reset and return
;
LE77E	JMP     LE7EB			;Jump to save and return
;
LE781	JMP     LE7EE			;Jump to return
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Brake switch indicating that brakes are applied, perform diags.
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LE784	BRSET   L0018,#$01,LE7EA	;Bra if b0==1, error 38 already set
;
	LDAB    L00D1			;MPH x 2
	CMPB    $59,X			;L5B59, MPH threshold for vehicle to be in motion
	BLS     LE7AC			;Bra if MPH <, perform diags.
;
	BRCLR   L008E,#$40,LE796	;Bra if b6==0, vehicle was not previously
					;stopped or decelerating
;
;-Vehicle was previously stopped or decelerating
;
	CLRA				;Preclear A    
	STAA    L017E			;Save to reset vehicle in motion timer
;
LE796	BCLR    L008E,#$40		;Clear b6, vehicle decelerating/stopped
					;flag
	BSET    L008E,#$80		;Set b7, vehicle in motion flag
	LDAA    L017E			;Vehicle in motion timer
	ADDA    #$01			;+1
	BEQ     LE7A7			;Bra if overflow
;
	STAA    L017E			;Save it
	CLRA				;Preclear A
;    
LE7A7	STAA    L017D			;Clear vehicle stopped timer
	BRA     LE7EE			;Bra to return
;
;-Check if vehicle was previously in motion
;
LE7AC	BRCLR   L008E,#$80,LE7EE	;Bra if b7==0, vehicle was not in motion
;
	CMPB    $5A,X			;L5B5A, MPH threshold for vehicle to be stopped
	BLS     LE7C3			;Bra if MPH <=, vehicle is stopped
;
	BSET    L008E,#$40		;Set b6, vehicle is decelerating
;
	LDAA    L017D			;Load vehicle decelerating/stopped timer
	ADDA    #$01			;+1
	BEQ     LE7EE			;Bra if overflow, return
;
	STAA    L017D			;Save it
	BRA     LE7EE			;Bra to return
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Vehicle stopped and brakes inicated as being applied
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LE7C3	LDAA    L017D			;Load vehicle decelerating timer
	CMPA    $5B,X			;L5B5B, time threshold for vehicle to be decelerating
					;using brakes
	BLS     LE7D1			;Bra if <=, the brakes where applied
;
	LDAA    L017E			;Load vehicle in motion timer
	CMPA    $5C,X			;L5B5C, time threshold for vehicel to have been in
					;motion
	BHI     LE7D9			;Bra if time >, the vehicle was in motion
;
;-Quals not met or brakes where applied, set flags and return
;
LE7D1	BCLR    L008E,#$80		;Clear b7, vehicle no longer in motion
	BSET    L008E,#$40		;Set b6, vehicle decelerating/stopped
	BRA     LE7EE			;Bra to return
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Vehicle was in motion even though brake switch indicating that brakes applied, set error
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LE7D9	BCLR    L008E,#$80		;Clear b7, vehicle no longer in motion
	BSET    L008E,#$40		;Set b6, vehicle decelerating/stopped
	LDAA    L0160			;Load error 38 fault counter
	INCA				;+1
	CMPA    $5D,X			;L5B5D, # of faults to set error
	BCS     LE7EB			;Bra if # of faults <, dont set error yet
;
;-Enough faults accumulated to set error 38
;
	BSET    L0018,#$01		;Error word #3, set b0, error 38
;
LE7EA	CLRA				;Preclear A
;
LE7EB	STAA    L0160			;Reset error 38 fault counter
;
LE7EE	RTS				;Return     
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; 
; Routine to check for error 52, extended high battery voltage
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; 
LE7EF	BRSET   L003B,#$04,LE82A	;Status word, bra if b2==1, trans diags. disabled
;
	LDX     #L5B00			;Index diag. masks
	BRCLR   4,X,#$10,LE822		;L5B04, bra if b4==0, not flagging error 52
;
;-Flagging error
;
	LDY     L0162			;Error 52 timer
	LDAB    L00A1			;Battery volts
	CMPB    $5F,X			;L5B5F, batt. volts threshold to set error
	BHI     LE814			;Bra if voltage >
;
;-Battery voltage within range
;
	BRSET   L0091,#$01,LE822	;Status word, bra if b0==1, prev. errors reset

	BSET    L0091,#$01		;Set b1, previous errors reset if applicable
	BRCLR   L001A,#$10,LE822	;Error word #5, bra
;
	BCLR    L001A,#$10		;Clear b4, reset error 52
	BRA     LE822			;Bra to reset and return
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Quals met to set error 52
;~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LE814	BRSET   L001A,#$10,LE822	;Bra if b4==1, error 52 already set
;
	INY				;+1, error 52 timer     
	CPY     $60,X			;LB560, time threshold to set error
	BCS     LE826			;Bra if time <
;
;-Time threshold met to set error
;
	BSET    L001A,#$10		;Set b4, error 52, extended high battery voltage
;
LE822	LDY     #$0000			;Preclear Y to reset timer
;
LE826	STY     L0162			;Save it
;
LE82A	RTS				;Return   
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; 
; Routine to check for error 53, high battery voltage
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; 
LE82B	BRSET   L003B,#$04,LE85F	;Status word, bra if b2==1, trans diags. disabled
;
	LDX     #L5B00			;Index diag. masks											;
	BRCLR   4,X,#$08,LE85B		;L5B04, bra if b3==0, not flagging error 53
;
;-Flagging error 53, check quals for setting erro
;
	LDAA    L0164			;Load error 53 timer
	LDAB    L00A1			;Battery volts
	CMPB    $62,X			;L5B62, threshold to set error 
	BHI     LE84F			;Bra if batt. volts >
;
;-Battery voltage within range
;
	BRSET   L008F,#$02,LE85B	;Status word, bra if b1==1, prev. error reset 
					;completed
;
	BSET    L008F,#$02		;Set b1, previous errors reset if applicable
;
;-Reset error if flagged previously
;
	BRCLR   L001A,#$08,LE85B	;Error word #5, bra if b3==0, error 53 not set
;
	BCLR    L001A,#$08 		;Clear b3, error 53 flag
	BRA     LE85B			;Bra to reset and continue
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Battery volts too great, set error 53
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LE84F	BRSET   L001A,#$08,LE85B	;Bra if b3==1, error 53 already set
;
	INCA				;+1, error 53 timer
	CMPA    $63,X			;L5B53, time threshold to set error
	BCS     LE85C			;Bra if time <, dont set error yet
;
	BSET    L001A,#$08 		;Set b3, set error 53, high batt. volts
;
LE85B	CLRA				;Clear error timer
;
LE85C	STAA    L0164			;Save it
;
LE85F	RTS				;Return   
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; Routine to check for error 58, high trans temp
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; 
LE860	BRSET   L003B,#$04,LE892	;Status word, bra if b2==0, transmission diags. 
					;disabled
;
;-Transmission diags enabled
;
	LDX     #L5B00			;Load error mask
	BRCLR   5,X,#$40,LE88E		;Bra if b6 of L5B05==0, not flagging error
;
	LDAA    L0165			;Load trans temp high error timer
	LDAB    L009F			;Load A/D trans temp
	CMPB    $64,X			;Compare to L5B64, threshold to set error 58, high 
					;trans temp
	BCS     LE882			;Bra if A/D temp is <, trans temp too high
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; A/D temp within limits, not setting error
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
	BRCLR   L001B,#$40,LE88E	;Error word #6, bra if b6==0, error not previously set
;
;-Check quals for clearing error
;			
	INCA				;+1, error timer
	CMPA    $65,X			;L5B65, time threshold to set/clear error 58
	BCS     LE88F			;Bra if time <, not clearing error yet
;
	BCLR    L001B,#$40		;Clear b6, error 58, high trans temp
	BRA     LE88E			;Bra to reset and exit
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; A/D temp too high, setting error
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LE882	BRSET   L001B,#$40,LE88E	;Bra if b6==1, error already set
							
	INCA				;+1, error timer   
	CMPA    $65,X			;Time to set/clear error flag
	BCS     LE88F			;Bra if timer <, not setting error yet
;
	BSET    L001B,#$40		;Error word #6, set b6, error 58

LE88E	CLRA				;Preclear A
;
LE88F	STAA    L0165			;Save it, high trans temp error timer
;
LE892	RTS				;Return     
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; Routine to check for error 59, low trans temp
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; 
LE893	BRSET   L003B,#$04,LE8C5	;Status word, bra if b2==1, transmission diags. 
					;disabled
;
	LDX     #L5B00			;Load error mask
	BRCLR   5,X,#$20,LE8C1		;Bra if b5 of L5B05==0, not setting error
;
	LDAA    L0166			;Load low trans temp error timer
	LDAB    L009F			;Load A/D trans temp
	CMPB    $67,X			;Compare to L5B67, threshold for setting error
	BHI     LE8B5			;Bra if A/D temp higher, set error
;
;~~~~~~~~~~~~~~~~~~~~~~~~
; A/D temp within limits
;~~~~~~~~~~~~~~~~~~~~~~~~
;
	BRCLR   L001B,#$20,LE8C1	;Error word #6, bra if b5==0, error not previously set	
;
;-Error previously set
;											
	INCA				;+1, error timer    
	CMPA    $68,X			;Compare to L5B68, time threshold to set/clear error
	BCS     LE8C2			;Bra if time <, threshold not yet met
;
	BCLR    L001B,#$20		;Clear b5, error 59, low trans temp
	BRA     LE8C1			;Bra to reset and exit
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; A/D temp too low, set error
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LE8B5	BRSET   L001B,#$20,LE8C1	;Bra if b5==1, error 59 already set
;							
	INCA				;+1, error timer	   
	CMPA    $68,X			;L5B68, time threshold to set/clear error 59
	BCS     LE8C2			;Bra if time <, dont set error yet
;
	BSET    L001B,#$20		;Set b5, error 59, trans temp low
;												; .,.. else
LE8C1	CLRA				;Preclear A
;    
LE8C2	STAA    L0166			;Save it, trans temp low error timer 
;
LE8C5	RTS				;Return 
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~     
;
;~~~~~~~~~~~~~~~~~~~~~~~~~
;
; Not used...
;
;~~~~~~~~~~~~~~~~~~~~~~~~~
;
LE8C6	RTS				;Return
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~     
;
;~~~~~~~~~~~~~~~~~~~~~~~~~
;
; Not used...
;
;~~~~~~~~~~~~~~~~~~~~~~~~~
;
LE8C7	RTS				;Return    
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; Routine to check for error 63, high baro pressure
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; 
LE8C8	BRSET   L003B,#$04,LE8FB	;Status word, bra if b2==1, trans diags. disabled
;
	LDX     #L5B00			;Index diag. masks
	BRCLR   5,X,#$04,LE8F7		;L5B05, bra if b2==0, not flagging error 63
;
;-Flagging error
;
	LDAA    L0167			;Load error 63 timer
	LDAB    L0316			;Load baro
	CMPB    $0069,X			;L5B69, baro threshold to set error 63
	BHI     LE8EB			;Bra if baro is higher
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~
; Baro still within limits
;~~~~~~~~~~~~~~~~~~~~~~~~~~
;
	BRCLR   L001B,#$04,LE8F7	;Error word #6, bra if b2==0, error 63 not set 
					;not set previously
;
;-Error 63 set previously
;											
	INCA				;+1    
	CMPA    $6A,X			;L5B6A, time threshold to set/clear error
	BCS     LE8F8			;Bra if time <=, dont reset error yet
;
;-Time threshold met to reset error
;
	BCLR    L001B,#$04		;Clear b2, error 63 flag
	BRA     LE8F7			;Bra to reset and return
;
;~~~~~~~~~~~~~~~~~~~~~~
; Baro high, set error
;~~~~~~~~~~~~~~~~~~~~~~
;
LE8EB	BRSET   L001B,#$04,LE8F7	;Bra if b2==1, error 63 already set
;											
	INCA				;+1    
	CMPA    $6A,X			;L5B6A, time threshold to set/clear error
	BCS     LE8F8			;Bra it time <, dont set error yet
;
;-Time threshold met to set error
;
	BSET    L001B,#$04		;Set b2, error 63, baro high
;
LE8F7	CLRA				;Preclear A to clear error timer
;    
LE8F8	STAA    L0167			;Save it
;
LE8FB	RTS 				;Return
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~    
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; Routine to check for error 64, baro low
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; 
LE8FC	BRSET   L003B,#$04,LE92F	;Status word, bra if b2==1, transmission
;
	LDX     #L5B00			;Index diag. masks
	BRCLR   5,X,#$02,LE92B		;L5B05, bra if b1==0, not flagging error
;
;-Flagging error
;
	LDAA    L0168			;Load error 64 timer
	LDAB    L0316			;Load baro
	CMPB    $006D,X			;L5B6D, threshold to set error
	BCS     LE91F			;Bra if baro is <
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Baro within range, reset error if set previously
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
	BRCLR   L001B,#$02,LE92B	;Error word #6, bra if b1==0, error not set
;
;-Error set previously
;
	INCA				;+1, inc error timer    
	CMPA    $006E,X			;L5B6E, time threshold to set/clear error
	BCS     LE92C			;Bra if time is <
;
	BCLR    L001B,#$02		;Clear b1, error 64 flag
	BRA     LE92B			;Bra to reset and return
;
;~~~~~~~~~~~~~~~~~~~~~
; Baro low, set error 
;~~~~~~~~~~~~~~~~~~~~~
;
LE91F	BRSET   L001B,#$02,LE92B	;Bra if b1==1, error already set
;
	INCA				;+1    
	CMPA    $006E,X			;L5B6E, time threshold to set/clear error
	BCS     LE92C			;Bra if time <
;
;-Time threshold met to set error
;
	BSET    L001B,#$02		;Set b1, error 64, baro low
;
LE92B	CLRA				;Preclear A to reset timer
; 	   
LE92C	STAA    L0168			;Save it
;
LE92F	RTS				;Return
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~     
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; Routine to check for error 71, low engine speed
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; 
LE930	BRSET   L003B,#$04,LE96E	;Status word, bra if b2==1, trans diags. disabled
;
	LDX     #L5B00			;Start addr. of error word masks
;
	BRCLR   6,X,#$08,LE96A		;L5B08, Bra if b3==0, not flagging error 71
;
	LDAA    L016D			;Load error 71 timer
	LDY     L00B1			;Load engine speed x 8
	CPY     $80,X			;L5B80,  RPM threshold for error 71
	BCS     LE954			;Bra if engine speed <
;
;-Engine speed within limits
;
	BRCLR   L001C,#$08,LE96A	;Error word #7, bra if b3==0, error 71 not set
;
	INCA				;+1, error 71 timer
	CMPA    $82,X			;L5B82, time threshold to set/clear error 71
	BCS     LE96B			;Bra if timer <
;
	BCLR    L001C,#$08		;Clear b3, clear error 71
	BRA     LE96A			;Bra to reset and exit
;
;-Engine speed too low
;
LE954	BRSET   L001C,#$08,LE96A	;Bra if b3==1, error 71 set already
;
	BRSET   L0017,#$02,LE96A	;Error word #2, bra if b1==0, press. sw. manifold
					;error set 
;
	LDAB    L009C			;Load transmission range word
	ANDB    #$50			;Clear all but b6, in P/N, and b4, invalid range
	BNE     LE96A			;Bra if invalid range or in P/N
;
;-In other gears, set error
;
	INCA				;+1, increment timer
	CMPA    $82,X			;L5B82, time threshold to set error
	BCS     LE96B			;Bra if timer <, dont set error 
;	
	BSET    L001C,#$08		;Set b3, error 71, engine speed too low
;											
LE96A	CLRA				;Preclear A to reset timer
;
LE96B	STAA    L016D			;Save it
;
LE96E	RTS				;Return
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~     
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; Routine to check for error 72, output speed lost
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LE96F	BRSET   L003B,#$04,LE9DB	;Status word, br aif b2==1, trans diags. disabled
;
	LDX     #L5B00			;Start addr. of error masks
	BRCLR   6,X,#$04,LE9D7		;L5B06, bra if b2==0, not flagging error 72 
;
;-Get delta RPMs
;
	LDD     L00CF			;Last known good TOS RPM
	SUBD    L00C6			;TOS RPM x 8
	BPL     LE984			;Bra if positve result
;
	NEGB				;Neg, get abs. value of result
	NEGA				;
	SBCA    #$00			;
;											
LE984	BRCLR   L009C,#$50,LE98E	;Status word, bra if b4, invalid range, and b6, in
					;P/N ==0
;
;-In park/neutral or invalid range
;
	CPD     L5B87			;Delta RPM threshold to set error										;
	BRA     LE992			;Bra to continue
;	
;-In other gears
;										
LE98E	CPD     L5B85			;Delta RPM threshold to set error
;
LE992	BHI     LE9AF			;Bra if delta TOS RPMs >
;
;~~~~~~~~~~~~~~~~~~~~~~~~
; Log error 72 if needed
;~~~~~~~~~~~~~~~~~~~~~~~~
;
	BRSET   L008F,#$10,LE9D7	;Status word, bra if b4==1, error 72 logged for this
					;session
;
	BSET    L008F,#$10		;Set b4, error 72 logged
	BRCLR   L001C,#$04,LE9D7	;Error word #7, bra if b2==0, error 72 not set 
;
;-Error 72 set
;
	LDAB    L0029			;Error 72 log counter
	CMPB    L5B8B			;Threshold for error 72 to remain set
	BCC     LE9D7			;Bra if threshold >=, error to remain set
; 
;-Not permanently setting error yet
;
	INCB				;+1, 
	INCB				;+1, increment error 72 log counter
	STAB    L0029			;Save it
	BCLR    L001C,#$04		;Clear b2, reset error 72 flag 				
	BRA     LE9D7			;Bra to continue
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Here to check quals for setting error 72
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;											
LE9AF	BRSET   L001C,#$04,LE9D7	;Bra if b2==1, error 72 already set, bra to reset 
					;error timer and return
;
	LDAA    L019C			;Load error 72 diag. delay timer
	BNE     LE9D7			;Bra if !=0, dont set error yet
;
	LDD     L019D			;Engine RPM x 8
	CPD     L5B89			;Lower threshold for diags.
	BCS     LE9D7			;Bra if engine RPMs <
;
	LDAA    L009B			;Range select timer
	CMPA    #255			;6.38 secs
	BNE     LE9D7			;Bra if timer !=, dont flag error 72 during range 
					;select change
;
	BRSET   L0017,#$02,LE9D7	;Error word #2, bra if b2==1, press. sw. manifold 
					;error
;
;-Set error 72, output speed lost
;
	LDAA    L016E			;Error 72 timer
	INCA				;+1
	CMPA    L5B8C			;Time threshold to set error 72
	BCS     LE9D8			;Bra if time <
;
	BSET    L001C,#$04		;Set b2, error 72, output speed lost
;									
LE9D7	CLRA				;Preclear A to reset timer
;
LE9D8	STAA    L016E			;Save it, error 72 timer
;
LE9DB	RTS				;Return    
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; Routine for flagging error 73, force motor
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LE9DC	BRSET   L003B,#$04,LEA27	;Status word, bra if b2==1, trans. diags. disabled
;
	LDX     #L5B00			;Index diag. masks
	BRCLR   6,X,#$02,LEA23		;Bra if b1 of L5b06==0, not flagging error 73
;
;-Flagging error 73
;
	LDAA    L016F			;Error 73 timer
	BRSET   L008A,#$80,LEA12	;Status word, bra if b7==1, force motor fault
;
;-Fault has not been flagged yet, check prop. error term
;
	LDAB    L0149			;Force motor current prop. error term
	BPL     LE9F4			;Bra if error is positive
;
	NEGB				;Get abs. value
;
LE9F4	CMPB    $8D,X			;L5B8D, maximum allowed sustained prop. error
	BHI     LEA12			;Bra if error >
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Here to log any errors that have been set
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
	BRSET   L008F,#$20,LEA23	;Status word, bra if b5==1, error 73 log counter
					;already updated 
;
	BSET    L008F,#$20		;Set b5, error 73 log counter updated
;
	BRCLR   L001C,#$02,LEA23	;Error word #7 word, bra if b2==0, error 73 not set

	LDAB    L002A			;Error 73 log counter
	CMPB    $8F,X			;L5B8F, number of errors for code 73 to remain set 
	BCC     LEA23			;Bra if <=, perminently set code 73
;
	INCB				;+1,
	INCB				;+1, increment error 73 log counter
	STAB    L002A			;Save error log counter
	BCLR    L001C,#$02		;Clear error 73 
;									
	BRA     LEA23			;Bra to exit
;	
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Here if thresholds met for setting force motor error
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;									
LEA12	BRSET   L001D,#$80,LEA23	;Bra if b7==1, error 75, system voltage low
;
	TST     L0171			;Test error 75 timer, low system voltage
	BNE     LEA23			;Bra if low system voltage present, exit
;
	INCA				;+1, error 73 timer
	CMPA    $8E,X			;L5B8E, time threshold to set error
	BCS     LEA24			;Bra if time <, dont set error 73 yet
;
	BSET    L001C,#$02		;Set b1, error 73, force motor fault
;
;-Here to exit w/o setting error
;
LEA23	CLRA				;Clear error 73 timer
;
;-Here to exit with error pending
;
LEA24	STAA    L016F			;Save timer
;
LEA27	RTS				;Return     
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; Routine to check for error 75, low system voltage
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; 
LEA28	BRSET   L003B,#$04,LEA7A	;Status word, bra if b2==1, trans diag. disabled
;
	LDX     #L5B00			;Index diag. masks
	BRCLR   7,X,#$80,LEA76		;L5B07, bra if b7==0, not flagging error
;
;-Flagging error here
;
	LDD     L019D			;Load RPM x 8
	CPD     L5B95			;Threshold for test
	BCS     LEA76			;Bra if engine RPM <, no diags.
;
	BRSET   L001D,#$80,LEA4B	;Error word #8, bra if b7==1, error already set 
;
	LDAB    $91,X			;Upper voltage range
	SUBB    $90,X			;Subtract lower voltage range, now span from lowest
					;to highest threshold vs. temp.
	LDAA    L00AF  			;Load trans temp
	MUL				;Trans temp x voltage span
	ADCA    $90,X			;Add in base voltage, now threshold for setting error
	BRA     LEA54			;Bra to continue
;							
;-Use recovery threshold if code already set
;				
LEA4B	LDAB    $93,X			;Upper voltage range for recovery
	SUBB    $92,X			;Lower voltage range for recovery
	LDAA    L00AF  			;Trans temp
	MUL				;Temp x voltage span
	ADCA    $92,X			;Add in base voltage
;
;-Check quals to set error
;
LEA54	TAB				;Get threshold into B
	LDAA    L0171			;Load error 75 timer											
	CMPB    L00A1			;Compare threshold to battery volts
	BHI     LEA6A			;Bra if batt. volts <, set error
;
	BRCLR   L001D,#$80,LEA76	;Bra if b7==0, error 75 not previously set
;
;-Here to unset error
;
	INCA				;+1, continue to increment timer
	CMPA    $94,X			;L5B94, timer threshold to set error
	BCS     LEA77			;Bra if time threshold is <, save and exit
;
	BCLR    L001D,#$80		;Clear b7, error 75
	BRA     LEA76			;Bra to clear timer and exit
;	
;-Here to set error
;										
LEA6A	BRSET   L001D,#$80,LEA76	;Bra if b7==1, error 75 already set
;
	INCA				;+1, error timer
	CMPA    $94,X			;L5B94, time threshold to set error
	BCS     LEA77			;Bra if timer <=
;
	BSET    L001D,#$80		;Error word #8, set code 75, low system voltage
;
;-Here if not setting error
;
LEA76	CLRA				;Clear error timer
;
LEA77	STAA    L0171			;Save it, error 75 timer

LEA7A	RTS				;Return
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; Error 77, manual/normal/performance switch routine
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; 
LEA7B	BRSET   L003B,#$04,LEAAD	;Status word, bra if b2==1, trans diags. disabled
;
	LDX     #L5B00			;Index diag. masks											;
	BRCLR   7,X,#$20,LEAA9		;L5B07, bra if b5==0, not flagging error 77
;
;-Flagging error 77, check if illegal pattern set
;
	LDAA    L0172			;Load error 77 timer
	BRSET   L0099,#$08,LEA9D	;Status word, bra if b3==1, illegal pattern
;
	BRSET   L008F,#$40,LEAA9	;Status word, bra if b6==1, error 77 diag. performed
					;already 
;
	BSET    L008F,#$40		;Set b6, error 77 diag. performed
	BRCLR   L001D,#$20,LEAA9	;Error word #8, bra if b5==0, error 77 not set
;
;-Code set previously, clear error and continue
;
	BCLR    L001D,#$20		;Clear b5, error 77
	BRA     LEAA9			;Bra to reset and continue
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Here to flag error 77, MNP switch
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LEA9D	BRSET   L001D,#$20,LEAA9	;Bra if b5==1, error 77, MNP switch set previously 
;
;-Quals met to set error
;
	INCA				;+1, error 77 timer
	CMPA    $97,X			;L5B97, time threshold to set error
	BCS     LEAAA			;Bra if timer <, dont set error yet
;
;-Time threshold met to flag error 77, MNP switch
;
	BSET    L001D,#$20		;Set b5, error 77 
;
LEAA9	CLRA				;Preclear A to reset error 77 timer
;
LEAAA	STAA    L0172			;Save error 77 timer
;
LEAAD	RTS 				;Return    					
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~										
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; Routine to check for error 79, prolonged high trans temp
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LEAAE	BRSET   L003B,#$04,LEAED	;Status word, bra if b4==1, trans diags. disabled
;
	LDX     #L5B00			;Index diag. masks
;											
	BRCLR   7,X,#$08,LEAE5		;L5B07, bra if b3==1, not flagging error 79
;
;-Flagging error
;											
	LDY     L0173			;Load error 79 timer
	LDAA    L00AF  			;Load trans temp
	BRCLR   L001D,#$08,LEAD3	;Error word #8, bra if b3==0, error 79 not set
					;previously
;
;-Error 79 set previously, check quals to reset
;											
	CMPA    $98,X			;L5B98, trans temp threshold to reset error 79
	BCC     LEAE5			;Bra if temp is >=, reset and return
;
;-Trans temp low enough to reset error
;											
	INY				;+1, error 79 timer
	CPY     $009A,X			;L5B9A, time threshold to set/reset error 79
	BCS     LEAE9			;Bra if time <
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Quals met to clear error 79, prolonged high trans temp
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;											
	BCLR    L001D,#$08		;Clear b3. error 79 flag										
	BRA     LEAE5			;Bra to reset and return
;
;-Check quals to set error 79
;							
LEAD3	CMPA    $99,X			;L5B99, threshold to set error 79
	BCS     LEAE5			;Bra if trans temp is less, reset and return
;
;-Trans temp out of range
;
	BRSET   L001B,#$40,LEAE5	;Error word, bra if b6==1, error 58, high trans temp,
					;already set
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Quals met to set error 79, prolonged high trans temp
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
	INY				;+1, increment error timer	 
	CPY     $009A,X			;L5B9A, time threshold to set/reset error 79
	BCS     LEAE9			;Bra if time <, return
;
;-Time threshold met to set error
;
	BSET    L001D,#$08		;Set b3, error 79 flag
;
LEAE5	LDY     #$0000			;Preclear Y to reset timer
;
LEAE9	STY     L0173			;Save it
;
LEAED	RTS				;Return
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; Routine to check for error 81, 2 -> 3 shift sol. failure
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; 
LEAEE	BRSET   L003B,#$04,LEB2E	;Status word, bra if b2==1
;
	LDX     #L5B00			;Index diag. masks
;	
	BRCLR   7,X,#$04,LEB2A		;L5B07, bra if b2==1, not flagging error 81
;	
;-Flagging error
;										
	LDAA    L0175			;Error 81 timer
	BRCLR   L0098,#$01,LEB04	;I/O status word, bra if b0==0
;
;-Check shift solenoid feedback
;											
	BRCLR   L0098,#$20,LEB1E	;Bra if b5==0, feedback reporting 2 -> 3 shift sol.
					;failure
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Output operational, log and reset previous errors
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;						
LEB04	BRSET   L0090,#$01,LEB2A	;Status word, bra if b0==1, error 81 logged already
;		
	BSET    L0090,#$01		;Set b0, error 81 logged for this session
;										
	BRCLR   L001D,#$04,LEB2A	;Error word #8, bra if b2==0, error 81 not set
;					
	LDAB    L002B			;Load error 81 counter
	CMPB    $9D,X			;L5B9D, number of errors for error 81 to remain set
	BCC     LEB2A			;Bra if number of errors >=, dont reset
;
	INCB				;+1
	INCB 				;+1, log additonal error
	STAB    L002B			;Save it, error 81 counter
	BCLR    L001D,#$04		;Clear b2, error 81 flag						
	BRA     LEB2A			;Bra to reset and return
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Output not operational, set error
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;										
LEB1E	BRSET   L001D,#$04,LEB2A	;Bra if b2==1, error 81 set already
;											 
	INCA				;+1, error 81 timer
	CMPA    $9C,X			;L5B9C, time threshold to set error
	BCS     LEB2B			;Bra if time <, dont set error yet
;
;-Time threshold met to set error
;
	BSET    L001D,#$04		;Set b2, error 81
;
LEB2A	CLRA				;Preclear A to reset error timer
;
LEB2B	STAA    L0175			;Save it, error 81 timer
;
LEB2E	RTS				;Return
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~     					
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; Routine to check for error 82, 1 -> 2 shift sol. failure
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; 
LEB2F	BRSET   L003B,#$04,LEB6F	;Status word, br aif b2==1, trans diags. disabled
;
	LDX     #L5B00			;Index diag. masks											;
	BRCLR   7,X,#$02,LEB6B		;L5B07, bra if b1==0, not flagging error 82
;	
;-Flagging error
;								
	LDAA    L0176			;Error 82 timer
	BRCLR   L0098,#$01,LEB45	;I/O status word, bra if b0==0
;
;-Check shift solenoid feedback
;
	BRCLR   L0098,#$10,LEB5F	;Bra if b4==0, feedback reporting 1 -> 2 shift sol.
					;failure
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Output operational, log and reset previous errors
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LEB45	BRSET   L0090,#$02,LEB6B	;Status word, bra if b1==1, error 81 logged already

	BSET    L0090,#$02		;Set b1, error 81 logged for this session
	BRCLR   L001D,#$02,LEB6B	;Error word #8, bra if b1==0, error 82 not set
;		
;-Error 82 set previously
;			
	LDAB    L002C			;Error 82 counter
	CMPB    $9F,X			;L5B9F, number of errors for error 82 to remain set
	BCC     LEB6B			;Bra if number of errors >=, dont reset
;	
;-Log additional error and reset
;										
	INCB				;+1
	INCB				;+1, log additional error
	STAB    L002C 			;Save it
	BCLR    L001D,#$02		;Clear b1, error 82 flag										
	BRA     LEB6B			;Bra to reset and return
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Output not operational, set error
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;											 
LEB5F	BRSET   L001D,#$02,LEB6B	;Bra if b1==1, error 82 set already
;											
	INCA				;+1, error 82 timer
	CMPA    $9E,X			;L5B9E, time threshold to set error 
	BCS     LEB6C			;Bra if time <, dont set error yet
;
;-Time threshold met to set error
;												
	BSET    L001D,#$02		;Set b1, error 82
											
LEB6B	CLRA				;Preclear A to reset error timer
;
LEB6C	STAA    L0176			;Save it, error 82 timer
;
LEB6F	RTS				;Return    					
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; Routine to check for error 83, shift solenoid driver failure
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; 
LEB70	BRSET   L003B,#$04,LEBB0	;Status word, bra if b2==1, trans diags. disabled
;
	LDX     #L5B00			;Index diag. masks
	BRCLR   7,X,#$01,LEBAC		;L5B07, bra if b0==0, not flagging error
;											
	LDAA    L0177			;Load error 83 timer
	BRCLR   L0098,#$01,LEB86	;I/O status word, bra if b0==0
;											
	BRSET   L0098,#$30,LEBA0	;*NOTE:	THIS IS A POSSIBLE ERROR. BOTH INDICATOR BITS
					;WILL BE '1' DURING NORMAL OPERATION. WHEN THE DRIVER
					;FAILS, THEN BOTH WOULD BE '0' DUE TO THE VOLTAGE
					;ON THE OUTPUT PINS REMAINING HIGH EVEN WITH THE 
					;OUTPUTS COMMANDED TO BE ACTIVE IN 1ST. THIS CODE IS 
					;ALSO DISABLED IN THE CALIBRATION SECTION*
;	
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; One or both outputs are operational, driver still functioning
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;										
LEB86	BRSET   L0090,#$04,LEBAC	;Status word, bra if b2==1, error 83 logged already
;											
	BSET    L0090,#$04		;Set b2, error 83 logged for this session
;
	BRCLR   L001D,#$01,LEBAC	;Error word #8, bra if b0==0, error 83 not set
					;previously
;
;-Error 83 set previously
;
	LDAB    L002D			;Load error 83 counter
	CMPB    $A1,X			;L5BA1, number of errors for error 83 to remain set
	BCC     LEBAC			;Bra if # of errors >=, dont reset error
;									
;-Log additional error and reset
;
	INCB				;+1
	INCB				;+1, log additional error 83
	STAB    L002D			;Save it, error 83 counter
	BCLR    L001D,#$01		;Clear b0, error 83 flag
;											
	BRA     LEBAC			;Bra to reset and return			
;	
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Both outputs still high when commanded on, driver has failed
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;											
LEBA0	BRSET   L001D,#$01,LEBAC	;Bra if b0==1, error 83 already set
;
	INCA				;+1, error 83 timer
	CMPA    $A0,X			;L5BA0, time threshold to set error
	BCS     LEBAD			;Bra if time <, dont set error yet
;
;-Time threshold to set error met
;
	BSET    L001D,#$01		;Set b0, error 83
;	 
LEBAC 	CLRA 				;Preclear A to reset timer
;
LEBAD	STAA    L0177			;Save it, error 83 timer
;
LEBB0	RTS				;Bra to return
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; Routine to check for error 89, excessive transmission slip
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; 
LEBB1	BRSET   L003B,#$04,LEC13	;Status word, bra if b2==1, trans diagnostics disabled
;
	LDX     #L5B00			;Index diag. masks										;
	BRCLR   8,X,#$04,LEC13		;L5B08, bra if b2==0, not flagging error 89
;											
	BRSET   L0090,#$40,LEBD6	;Status word, bra if b6==1, error 89 logged for this
					;session
;
;~~~~~~~~~~~~~~						
; Log error 89
;~~~~~~~~~~~~~~
;				
	BSET    L0090,#$40		;Set b6, error 89 logged
	BRCLR   L001E,#$04,LEC13	;Error word #9, bra if b2==0, error 89 not set
;
	LDAB    L0031			;Load error 89 log counter
	CMPB    $B8,X			;L5BB8, number of errors for erro 89 to remain
					;permanently set
	BCC     LEC13			;Bra if >=, error 89 to remain set
											
	INCB				;+1
	INCB				;+1
	STAB    L0031			;Error 89 log counter
	BCLR    L001E,#$04		;Clear b2, error 89										
	BRA     LEC13			;Bra to continue
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Here to check for excessive adaptive modifier
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;											
LEBD6	BRSET   L001E,#$04,LEC13	;Bra if b2==1, error 89 to remain set 
;
	LDAA    L0144			;Load current adaptive modifier
	ASLA				;x2
	BCS     LEBE1			;Bra if overflow
;
	ASLA				;x2
;
LEBE1	BCC     LEBE5			;Bra if no overflow 
;
	LDAA    #$FF			;Clear overflow
;
LEBE5	LDY     #LB74F			;Addr of set of cells to use based on shift pattern
	LDAB    L0099			;Load shift pattern word
	ANDB    #$06			;Clear all but b1=performance pattern req., b2=manual
					;pattern requested
	ABY				;Add in as offset
	LDY     0,Y			;Load in index for adaptive modifier cells
	CMPA    $14,Y			;Compare current adaptive modifier to max allowed
	BLT     LEC13			;Bra if adaptive modifier was <, return
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Adaptive modifier is out of bounds
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
	LDAB    L00D3			;Load gear #
	DECB				;-1 for look up
	BMI     LEC13			;Bra if already in first, return
;
	BNE     LEC15			;Bra if !=0, in 3rd or 4th
;
;-In second gear
;
	LDAB    L0132			;Load stored shift time for 1 -> 2 shift
	CMPB    $B2,X			;L5BB2, lower limit for shift time to set error
					;(1 -> 2)
	BLS     LEC4E			;Bra if shift time <=, previous short shift, return
;
	JSR     LEC4F			;Routine to check quals for error 89 diags.
;				
	BCS     LEC4E			;Bra if carry set, quals not met for error 89 diags.
;
;-Quals met, continue with setting error
;
	LDAA    L0032			;Load error 89 shift counter (2nd gear)
	INCA				;+1
	CMPA    $B5,X			;L5BB5, number of 1 -> 2 shifts threshold to set error
	BCC     LEC4B			;Bra if >=, adaptive modifier out of bounds for
					;too many shifts, set error
;
	STAA    L0032			;Save counter
;
LEC13	BRA     LEC4E			;Bra to return
;
;-Here if in 3rd or 4th gear
;
LEC15	CMPB    #$01			;3rd gear
	BNE     LEC30			;Bra if !=, in other gears
;
;~~~~~~~~~~~~~
; In 3rd gear
;~~~~~~~~~~~~~
;		
	LDAB    L0133			;Load stored shift time for 2 -> 3 shift
	CMPB    $00B3,X			;L5BB3, lower limit for shift time to set error
					;(2 -> 3)
	BLS     LEC4E			;Bra if shift time <=, previous short shift, dont set
					;error
;											
	JSR     LEC4F			;Routine to check quals for error 89 diags.
;										
	BCS     LEC4E			;Bra if carry set, quals not met for setting error 89
;	
;-Quals met, continue with setting error
;									
	LDAA    L0033			;Load error 89 shift counter (3rd gear)
	INCA				;+1
	CMPA    $B6,X			;L5BB6, number of 2 -> 3 shifts threshold to set error
	BCC     LEC4B			;Bra if >=, adaptive modifier out of bounds for
					;too many shifts, set error
;										
	STAA    L0033			;Store counter
	BRA     LEC4E			;Bra to return
;
;-Here if in 4th gear
;											
LEC30	CMPB    #$0002			;4th gear
	BNE     LEC4E			;Bra if !=, in other gears
;
;~~~~~~~~~~~~~
; In 4th gear
;~~~~~~~~~~~~~
;											
	LDAB    L0134			;Load stored shift time for 2 -> 3 shift
	CMPB    $B4,X			;L5BB4, lower limit for shift time to set error
					;(3 -> 4)
	BLS     LEC4E			;Bra if shift time <=, previous short shift, dont set
					;error
;											
	JSR     LEC4F			;Routine to check quals for error 89 diags.
;											
	BCS     LEC4E			;Bra if carry set, quals not met for setting error 89
;											
	LDAA    L0034			;Load error 89 shift counter (4th gear)
	INCA				;+!
	CMPA    $B7,X			;L5BB7, number of 2 -> 3 shifts threshold to set error
	BCC     LEC4B			;Bra if >=, adaptive modifier out of bounds for
					;too many shifts, set error
;											
	STAA    L0034			;Store counter
	BRA     LEC4E			;Bra to return
;	
;-Here to set error 89
;										
LEC4B	BSET    L001E,#$04		;Error word #9, set b2, error 89, excessive trans. 
					;slip indicated
;											
LEC4E	RTS				;Return   					
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; Routine to check quals for error 89 diags.
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LEC4F	SEC				;Pre-set carry bit
;
;-Check if external TFT sensor errors set
;
	BRSET   L001B,#$40,LEC7E	;Error word #6, bra if b6==1, error 58, high trans 
					;temp set
;
	BRSET   L001B,#$20,LEC7E	;Bra if b5==1, error 59, low trans temp set
;
;-Check temp threshold for diags.
;
	LDAB    L00AF  			;Load trans temp
	CMPB    $AF,X			;L5BAF, lower threshold for flagging error 89
	BCS     LEC7E			;Bra if trans temp <
;										
	SEC				;Pre-set carry bit
;
;-Check if external TPS errors set
;
	BRSET   L0016,#$01,LEC7E	;Error word #1, bra if b0, error 21, TPS high error 
					;set
;
	BRSET   L0017,#$80,LEC7E	;Error word #2, bra if b7, error 22, TPS low error 
					;set
;
;-Check TPS threshold for diags.
;
	LDAB    L00AC			;Load trans TPS
	CMPB    $B0,X			;L5BB0, lower TPS threshold for diagnostics
	BCS     LEC7E			;Bra if TPS <
;										
	SEC				;Pre-set carry bit
;
;-Check if VSS/TIS errors set
;
	BRSET   L0017,#$20,LEC7E	;Bra if b5, error 24, VSS low, set
;
	BRSET   L001C,#$04,LEC7E	;Error word #7, bra if b2, error 72, output speed lost,
					;set
;
	BRSET   L001C,#$01,LEC7E	;Error word #7, bra if b0==1, error 74, trans input 
					;speed loss, set
;
;-Check speed threshold for diags.
;
	LDAB    L00D1			;Load MPH x 2
	CMPB    $B1,X			;L5BB1, lower threshold for flagging error 89
;
;-Here to return with C bit set for no diags
;
LEC7E	RTS				;Return
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~   					
;
;~~~~~~~~~~~~~
;
; Not used...
;
;~~~~~~~~~~~~~
;											
LEC7F	RTS 				;Return
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; Error 66,  3 -> 2 shift solenoid quad driver failure
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; 
LEC80	BRSET   L003B,#$04,LECCC	;Status word, bra if b2==1, trans. diags. disabled
;
;-Transmission diags. ok'd
;
	LDX     #L5B00			;Start of diag. masks											;
	BRCLR   6,X,#$80,LECC8		;L5B06, bra if b7==0, not flagging error 66
;
	LDAA    L0169			;Load error 66 timer
	BRCLR   L0084,#$10,LECAF	;Status word, bra if b4==0, output did not pass
					;diags. 
;
;-Output operational here
;
	BCLR    L0084,#$10		;Clear b4 for next pass
	BRSET   L0090,#$80,LECC8	;Status word, bra if b7==1, error 66 logged already
;
	BSET    L0090,#$80		;Set b7, error 66 has been logged for this session
	BRCLR   L001C,#$80,LECC8	;Error word #7, bra if b7==0, error 66 not flagged
					;previously
;
;-Error previously set, check criteria for resetting error
;
	LDAB    L0026			;Error 66 fault counter
	CMPB    $72,X			;L5B72, number of faults before code 66 permanently
					;set
	BCC     LECC8			;Bra if greater, code 66 to remain set

	INCB				;+1,
	INCB				;+1, log one more error 66 reset
	STAB    L0026			;Save fault counter
	BCLR    L001C,#$80		;Clear b7, error 66
;
	BRA     LECC8			;Bra to return
;	
;-Output failed here
;									
LECAF	BRSET   L001C,#$80,LECC8	;Bra if b7==1, error 66 already flagged
;
	BRSET   L0093,#$08,LECC8	;Status word, bra if b3==1, high system voltage
;
	BRSET   L007F,#$80,LECC8	;Bra if b7==1, 3->2 shift modulation in progress
;
	TST     L0118			;Test 3->2 diag. DC
	BNE     LECC8			;Bra if !=0, diag. in effect
;
;-Here if quals met to set error 66
;
	INCA				;+1, error 66 timer
	CMPA    $71,X			;L5B71, time threshold to set error
	BCS     LECC9			;Bra if time threshold not yet met
;
	BSET    L001C,#$80		;Error word #6, set b7, error 66, 3 -> 2 shift 
					;solenoid quad driver failure
;
;-Here to reset timer
;
LECC8	CLRA				;Clear error 66 timer
;
LECC9	STAA    L0169			;Save timer
;
LECCC	RTS				;Return   					
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; Routine to check for error 67, TCC solenoid failure
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LECCD 	BRSET   L003B,#$04,LED09	;Status word, bra if b2==1, trans diags. disabled

	LDX     #L5B00			;Index diag. masks										;
	BRCLR   6,X,#$40,LED05		;L5B06, bra if b6==0, not flagging error 67
;
	LDAA    L016A			;Load error 67 timer
	BRCLR   L0085,#$02,LECF9	;Status word, bra if b1==0, TCC output failure/open 
					;ckt
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~
; TCC output is operational
;~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
	BRSET   L008F,#$80,LED05	;Status word, bra if b7==1, error 67 logged already
;
	BSET    L008F,#$80		;Set b7, error 67 logged for this session
	BRCLR   L001C,#$40,LED05	;Error word #7, bra if b6==0, error 67 not set
;
;-Error set previously, log error and reset 
;
	LDAB    L0027			;Load error 67 counter
	CMPB    $74,X			;L5B74,  # of errors for error 67 to remain set
	BCC     LED05			;Bra if # of errors is >=, error 67 to remain set
;
	INCB				;+1
	INCB				;+1, log additional error
	STAB    L0027			;Save it								
	BCLR    L001C,#$40		;Clear b6, error 67 flag										
	BRA     LED05			;Bra to reset and return
;
;~~~~~~~~~~~~~~~~~~~~~~~
; TCC output has failed
;~~~~~~~~~~~~~~~~~~~~~~~	
;
LECF9	BRSET   L001C,#$40,LED05	;Bra if b6==1, error 67 set previously
;			
	INCA				;+1
	CMPA    $73,X			;L5B73, time threshold to set error 67
	BCS     LED06			;Bra if time <
;
;-Time threshold met to set error
;
	BSET    L001C,#$40		;Error word #7, set b6, error 67
;
LED05	CLRA				;Preclear A to reset timer
;
LED06	STAA    L016A			;Save it, error 67 timer
;
LED09	RTS				;Return	
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;											
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; Routine to check for error 68, TCC slipping/not engaged
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; 
LED0A	BRSET   L003B,#$04,LED62	;Status word, bra if b2==1, trans diags. disabled
;
	LDX     #L5B00			;Index error masks
	BRCLR   6,X,#$20,LED5E		;L5B06, bra if b4==0, not flagging error 68
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Log error 68 if error set
;~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
	LDAA    L016B			;Load error 68 timer								
	BRSET   L008F,#$04,LED32	;Status word, bra if b2==1, error 68 logged for this 
					;run 
;
	BSET    L008F,#$04		;Set b2, error 68 logged for this run						
	BRCLR   L001C,#$20,LED5E	;Error word #6, bra if b5==0, error 66 not set
;
	LDAB    L0028			;Load error 68 log counter
	CMPB    $79,X			;L5B79, # of errors for error 68 to remain set
	BCC     LED5E			;Bra if # of errors >=, error to remain set, return
;
;-Log additional error and reset
;
	INCB				;+1
	INCB				;+1, log additional error
	STAB    L0028			;Save it
	BCLR    L001C,#$20		;Clear b5, error 68
	BRA     LED5E			;Bra to reset and return	
;			
;~~~~~~~~~~~~~~~~~~~~
; Perform TCC diags.
;~~~~~~~~~~~~~~~~~~~~
;
LED32	BRSET   L001C,#$20,LED5E	;Bra if b5==1, error 68 already set
;
	LDY     L00E5			;Load TCC slip
	CPY     $75,X			;L5B75, TCC slip threshold to check for TCC slipping
	BLT     LED5E			;Bra if TCC slip <
;
	BRCLR   L0085,#$20,LED5E	;Status word, bra if b5==0, TCC not locked
;
;-TCC locked and slipping
;
	LDAB    L00D3			;Load gear #
	BEQ     LED5E			;Bra if ==0, in 1st gear
;
;-In other gears, check for applicable errors
;
	BRSET   L001C,#$08,LED5E	;Bra if b3==1, error 71 set, engine speed low
;
	BRSET   L001C,#$01,LED5E	;Bra if b0==1, error 74 set, trans input speed lost
;
	BRSET   L0017,#$02,LED5E	;Error word #1, bra if b1==1, press. sw. manifold
					;error set  
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Make sure correct gear range is selected
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;

	BRCLR   L009C,#$0C,LED5E	;Range select word, bra if b2, drive, and b3, 
					;overdrive, are not selected
;
;-Drive or overdrive range selected, all quals met to set error
;
	INCA				;+1, inc. error 68 timer
	CMPA    $77,X			;L5B77, time threshold to set error 68
	BCS     LED5F			;Bra if time <
;
;-Time threshold met, set error 68
;
	BSET    L001C,#$20		;Error word #6, set b5, error 68
;
LED5E	CLRA				;Preclear A
;
LED5F	STAA    L016B			;Save, error 68 timer
;											
LED62	RTS     			;Return
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; Routine to check for error 69, TCC failed to disengage
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; 
LED63	BRSET   L003B,#$04,LEDD3	;Status word, bra if b2==1, trans diags. disabled
;
	LDX     #L5B00			;Index diag. masks
	BRCLR   6,X,#$10,LEDCF		;L5B06, bra if b4==0, not flagging error 69
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Check if error 69 set previously
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;	
	LDAA    L016C			;Load error 69 timer
	BRSET   L008F,#$08,LED81	;Status word, bra if b3==1, error 69 logged for this
					;session
;
	BSET    L008F,#$08		;Set b3, error 69 logged
	BRCLR   L001C,#$10,LEDCF	;Error word #6, bra if b4==0, error 69 not set, return 
;	
;-Error set previously, reset and return
;
	BCLR    L001C,#$10		;Clear b4, error 69 flag
	BRA     LEDCF			;Bra to return
;										
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Check thresholds to set error
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;											
LED81	BRSET   L001C,#$10,LEDCF	;Bra if b4==1, error 69 already set
;
	BRSET   L001C,#$04,LEDCF	;Bra if b2, error 72, output speed lost, set
;
	LDY     L00E5			;Load TCC slip
	CPY     $7C,X			;L5B7C, lower threshold for TCC to be locked
	BLE     LEDCF			;Bra if <=, TCC not locked
;
	CPY     $7A,X			;L5B7A, upper threshold for TCC to be locked
	BGE     LEDCF			;Bra if TCC slip >=, TCC not locked, bra to return
;
;-TCC is locked, check TPS 
;
	LDAB    L00AC			;Load trans TPS threshold
	CMPB    $7F,X			;L5B7F, lower trans TPS threshold to flag error 
	BLS     LEDCF			;Bra if TPS is <=, return
;	
;-Check if applicable errors have been set
;	
	BRSET   L0016,#$01,LEDCF	;Error word #1, bra if b0==1, error 21, high TPS set
;
	BRSET   L0017,#$80,LEDCF	;Error word #2, bra if b7==1, error 22, low TPS set
;
;-Check if TCC active
;
	BRCLR   L0085,#$70,LEDAA	;Status word, bra if b6, TCC fully applied, b5, TCC 
					;fully locked, and b4, TCC being released ==0
;
;-TCC being applied/locked/being released, return
;
	BRA     LEDCF			;Bra to reset and return
;	
;-TCC locked when commanded to be unlockedm check quals to set error
;										
LEDAA	LDAB    L00D3			;Load gear #
	BEQ     LEDCF			;Bra if ==0, in first gear
;
	BRSET   L001C,#$08,LEDCF	;Bra if b3==1, error 71 set, engine speed low, return
;	
	LDY     L019D			;Load engine RPM x 8
	CPY     $83,X			;L5B83, lower engine speed threshold to set error
	BLS     LEDCF			;Bra if engine RPMs <=, return
;												(CLR ERR 69 TIMER & EXIT via RTS) 
	BRSET   L001C,#$01,LEDCF	;Bra if b0==1, error 74 set, trans input speed lost
;
	BRSET   L0017,#$02,LEDCF	;Erorr word #2, bra if b1==1, error 28, press. sw. 
					;manifold, set
;													(CLR ERR 69 TIMER & EXIT via RTS) 
	BRCLR   L009C,#$0C,LEDCF	;Range select word, bra if b2, drive, and b3, 
					;overdrive, are not selected
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; All quals met to set error
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
	INCA				;+1, error 69 timer
	CMPA    $7E,X			;L5B7E, time threshold to set error
	BCS     LEDD0			;Bra if timer <, dont set error yet
;
;-Time threshold met to set error
;
	BSET    L001C,#$10		;Set b4, error 69, TCC failed to disengage
;
LEDCF	CLRA				;Preclear A
;
LEDD0	STAA    L016C			;Save it, error 69 timer
;			
LEDD3	RTS				;Return
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;~~~~~~~~~~~~~
;
; Not used...
;
;~~~~~~~~~~~~~
;
LEDD4	RTS				;Return
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;~~~~~~~~~~~~~
;
; Not used...
;
;~~~~~~~~~~~~~
;
LEDD5     RTS				;Return
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; Routine to check for error 86, ratio of engine to vehicle speed is low
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; 
LEDD6	BRSET   L003B,#$04,LEDFE	;Status word, bra if b2==1, trans diags. disabled

	LDX     #L5B00			;Index diag. masks										;
	BRCLR   8,X,#$20,LEDFC		;L5B08, bra if b5==0, not flagging error   
;
;-Flagging error
;
LEDE1	LDAA    L0179			;Error 86 timer
	BRSET   L0090,#$10,LEE00	;Status word, bra if b4==1, error 86 logged
					;already
;											
	BSET    L0090,#$10		;Set b4, error 86 logged for this session
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Log error if set previously
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
	BRCLR   L001E,#$20,LEE3F	;Error word #9, bra if b5==0, error 86 not set
					;previously
;
;-Error 86 set previously
;										
	LDAB    L002F			;Error 86 counter
	CMPB    $A8,X			;L5BA8, number of errors for 86 to remain set
	BCC     LEE3F			;Bra if # of errors >=, error to remain set
;
;-Log additional error
;
	INCB				;+1
	INCB				;+1, log additional error
	STAB    L002F			;Save it, error 86 counter
	BCLR    L001E,#$20		;Clear b5, reset error 86
;											
LEDFC	BRA     LEE3F			;Bra to reset and return
;				
LEDFE	BRA     LEE43			;Bra to return				
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~
; Check quals to set error
;~~~~~~~~~~~~~~~~~~~~~~~~~~
;					
LEE00	BRSET   L001E,#$20,LEE3F	;Bra if b5==1, error 86 already set 
;
;-Check gear #
;
	LDAB    L00D3			;Load gear #
	CMPB    #$01			;2nd gear
	BCC     LEE3F			;Bra if gear # >=
;
;-In 1st gear, P/N, or reverse, check ratio of engine RPMs to MPH
;
	LDY     L0190			;Load engine RPM/MPH ratio x 4
	CPY     $A3,X			;L5BA3, threshold to set error
	BHI     LEE3F			;Bra if ratio is higher
;
;-Ratio is low, check TPS threshold to set error
;
	BRSET   L0016,#$01,LEE3F	;Error word #1, bra if b0==1, error 21, high TPS
;
	BRSET   L0017,#$80,LEE3F	;Error word #2, bra if b7==1, error 22, low TPS
;
	LDAB    L00AC			;Load trans TPS
	CMPB    $A5,X			;L5BA5, TPS threshold to set error
	BLS     LEE3F			;Bra if TPS is less, dont set error
;
;-Check VSS threshold to set error
;
	BRSET   L0017,#$20,LEE3F	;Bra if b5==1, error 24, low VSS
;
	BRSET   L001C,#$04,LEE3F	;Error word #6, bra if b2==1, error 72, output speed
					;lost
;										
	LDAB    L00D1			;Load MPH x 2
	CMPB    $A6,X			;L5BA6, MPH threshold to set error
	BLS     LEE3F			;Bra if MPH <
;
;-Check to make sure a forward range is selected
;
	BRSET   L0017,#$02,LEE3F	;Error word #2, bra if b1==1, error 28, press. sw. 
					;manifold, set
;
	BRCLR   L009C,#$0F,LEE3F	;Trans range word, bra if b0-b4==0, no forward range
					;selected
;
;-All quals met, set error
;
	INCA				;+1, error 86 timer
	CMPA    $A7,X			;L5BA7, time threshold to set error
	BCS     LEE40			;Bra if time <
;
	BSET    L001E,#$20		;Set b5, error 86, RPM/MPH ratio low
;	
LEE3F	CLRA				;Preclear A to reset timer
;
LEE40	STAA    L0179			;Save it
;
LEE43	RTS				;Return
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~     					
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; Routine to check for error 87, ratio of engine speed to MPH is high
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; 
LEE44	BRSET   L003B,#$04,LEE6C	;Status word, bra if b2==1, trans diags. disabled
;
	LDX     #L5B00			;Index diag. masks
	BRCLR   8,X,#$10,LEE6A		;L5B08, bra if b4==0, not flagging error
;
;-Flagging error
;
	LDAA    L017A			;Load error 87 timer
;	
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Log and reset previous errors if applicable
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
	BRSET   L0090,#$20,LEE6E	;Status word, bra if b5==1, error 87 logged already
;										
	BSET    L0090,#$20		;Set b5, error 87 logged for this session
	BRCLR   L001E,#$10,LEE6A	;Error word #9, bra if b4==0, error 87 not set
					;previously
;
;-Error 87 set previously
;											
	LDAB    L0030			;Error 87 counter
	CMPB    $AE,X			;L5BAE, number of errors for error 87 to remain set
	BCC     LEEA5			;Bra if # of errors >=
;								
;-Log and reset error
;	
	INCB				;+1
	INCB				;+1, log additional error
	STAB    L0030			;Save it, error 87 counter
	BCLR    L001E,#$10		;Clear b4, reset error 87
;											
LEE6A	BRA     LEEA5			;Bra to reset error timer and return				
;											
LEE6C	BRA     LEEA9			;Bra to return				
;	
;~~~~~~~~~~~~~~~~~~~~~~~~~~
; Check quals to set error
;~~~~~~~~~~~~~~~~~~~~~~~~~~
;										
LEE6E	BRSET   L001E,#$10,LEEA5	;Bra if b4==1, error 87 set already
;										
	LDAB    L00D3			;Load gear #
	CMPB    #$03			;4th gear
	BCS     LEEA5			;Bra if gear # is <
;							
;-In 4th gear, check ratio of engine speed to MPH
;			
	LDY     L0190			;Load engine RPM/MPH ratio x 4
	CPY     $A9,X			;L5BA9, threshold to set error
	BCS     LEEA5			;Bra if threshold <
;	
;-Ratio of engine speed to MPH is high, check TPS 
;								
	BRSET   L0016,#$01,LEEA5	;Error word #1, bra if b0==1, error 21, high TPS
;
	BRSET   L0017,#$80,LEEA5	;Error word #2, bra if b7==1, error 22, low TPS
;
	LDAB    L00AC			;Load trans TPS 
	CMPB    $AB,X			;L5BAB, threshold to allow error to be set
	BLS     LEEA5			;Bra if TPS is <
;
;-Check transmission temp
;
	LDAB    L00AF  			;Load transmission temp
	CMPB    $AC,X			;L5BAC, threshold to allow error to be set
	BLS     LEEA5			;Bra if trans temp is <
;
;-Check to make sure a forward range is selected
;
	BRSET   L0017,#$02,LEEA5	;Error word #2, bra if b1==1, error 28, press. sw. 
					;manifold, set
;
	BRCLR   L009C,#$0F,LEEA5	;Trans range word, bra if b0-b4==0, no forward range
					;selected
;
	INCA				;+1, inc. error timer
	CMPA    $AD,X			;L5BAD, time threshold to set error
	BCS     LEEA6			;Bra if time <
;							
;-Time threshold met, set error 87, engine RPM to MPH ratio is high
;			
	BSET    L001E,#$10		;Set b4, error 87 flag
;										
LEEA5	CLRA				;Preclear A to reset timer
;
LEEA6	STAA    L017A			;Save it
;										   
LEEA9	RTS				;Return     				   
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~			
;													   
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; Lead in for A/D routine, Single and Multi-channel read													
;  										
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;		                                                            
LEEAA	LDX     #L3000			;Index CPU reg.
	LDAA    #$07			;0000 0111				
	JSR     LEF12			;Port D config routine
;
	LDAA    #$01			;0000 0001
	JSR     LEEFC			;A/D channel routine
;									
	LDAA    $34,X			;Get A/D results
	STAA    L082D          		;A/D cool temp
;		
	LDAA    #$06			;0000 0110				
	JSR     LEF12			;Port D config routine
;
	LDAA    #$01			;
	JSR     LEEFC			;A/D channel routine
;											
	LDAA    $34,X			;Load ch 1 results
	STAA    L0192			;Save it, A/D TPS		
;											
	LDAA    #$01			;
	BSR     LEF12   		;Port D config routine
;											
	LDAA    #$11			;0001 0001, Set A/D to multi-channel read 								
	BSR     LEEFC			;Go do A/D conversion
;
;-Get each of the results in
;
	LDAA    $32,X			;Get channel 1 results
	STAA    L082F			;Save it, ALDL switch voltage
	LDAA    $33,X			;Get channel 2 retults
	STAA    L009F  			;Save it, A/D transmission temp
	LDAA    $34,X			;Get channel 3 results
	STAA    L00A1			;A/D battery voltage
	BRA     LEEFB			;Bra to return
; 											
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; Lead in for A/D routine, Multi-channel read										
;										
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LEEE2	LDX     #L3000			;Base reg. address
	LDAA    #$14			;0001 0100, Set up for multi-channel read
	BSR     LEEFC			;A/D routine
;											
	LDAA    $31,X			;Get channel 4 results
	STAA    L00A0          		;A/D TPS
	BRSET   L0050,#$01,LEEF6	;Bra if b0==1, sync. MAP sensor reads
;											
	LDAA    $32,X			;Get channel 5 results
	STAA    L082E			;A/D MAP voltage
;											
LEEF6	LDAA    $33,X			;Get A/D channel 6 results
;
LEEF8	STAA    L01F8			;A/D O2 voltage
;
LEEFB	RTS 				;Return  
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~  
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; A/D Routine
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LEEFC	BCLR	L001A,#$02		;Clear b1, ERR 55, faulty PCM 11										;
	STAA    L3030			;Save A, A/D Contorl/Satus reg. 
					; b7=Conversion complete flag
					; b6= -
					; b5=Continuous scan control
					; b4=Multiple/Single channel control
					; b3=Channel select D
					; b2=Channel select C
					; b1=Channel select B
					; b0=Channel select A
;
	LDAB    #12			;A/D wait for data timer counter
											
LEF04	BRSET   $30,X,#$80,LEF11	;Bra if b7==1, A/D conversion complete
;
	DECB				;-1    
	BNE     LEF04			;Bra if !=0, continue
;
;-Here if A/D conversion failed
;
	BSET    L003A,#$01		;Status word, set b0, A/D failure
	BSET    L001A,#$02      	;SET b1, ERR 55, faulty PCM 11
;
;Here if A/D successful
;
LEF11	RTS				;Return with data
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;                                                            
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; Port D configuration routine
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LEF12	BCLR    8,X,#$38		;Clear b3-b5, 0x3008, (PORT D data)
	ASLA 				;Shift left   					
	ASLA  				;Shift left  					
	ASLA 				;Shift left   					
	ORAA    8,X			;				
	STAA    8,X			;Store it				
	RTS				;Return     
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; BLM reset routine
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LEF1D	LDX     #L02D6			;Start of BLM cells
	LDAA    #128			;Neutral BLM value
;
LEF22	STAA    0,X			;Reset cell
	INX 				;+1, adv. to next cell    
	CPX     #$02EA			;Last cell to clear
	BLS     LEF22			;Bra if <, continue
;
	RTS				;Return
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~     
;
;~~~~~~~~~~~~~~~~~~~~~
;
; BLM clamp routine
;
;~~~~~~~~~~~~~~~~~~~~~
;
LEF2B	LDAA    L48F3			;Upper limit for BLMs
	LDAB    L48F4			;Lower limit for BLMs
;
	LDX     #L02D6			;Addr of first BLM cell
LEF34	CMPA    0,X			;Compare upper lim. to BLM
	BCC     LEF3A			;Bra if BLM less
;
	STAA    0,X			;Save upper limit instead
;
LEF3A	CMPB    0,X			;Compare lower lim. to BLM
	BLS     LEF40			;Bra if lower limit is less
;
	STAB    0,X			;Save lower limit instead
;
LEF40   INX				;+1, advance to next cell
	CPX     #L02EA			;Last BLM cell
	BLS     LEF34			;Bra if not at last cell yet, continue
;
	RTS  				;Return
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~   
;                                                            
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; Checksum routine
;
; ENTER WITH:
;	 X pointing to start of checksum area
;
; EXIT WITH:
;	Results of checksum in Y
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LEF47	LDY     #0			;Preclear Y
	PSHX   				;Start address to stack 
	PULA				;Get MSB of start address off stack    
	PULA 				;Get LSB of start address off stack    
	CLRB				;Clear B, LSB x 256    
	LSRD 				;/2   
	LSRD 				;/2   
	LSRD				;/2, now address/8 for block transfer    
	PSHA				;MSB of lower addressb byte to stack    
	TBA  				;Get LSB into A   
	BEQ     LEF5F			;Bra if ==0, no leftovers, use block checksum
;
;-Single byte checksum
;
LEF56	LDAB    0,X			;Load up byte
	ABY  				;Add it into the checksum   
	INX 				;+1, next address    
	ADDA    #$20			;
	BNE     LEF56			;Bra if !=0
;
;-Block checksum
;
LEF5F	LDAB    #$08			;8 byte offset
	PULA  				;Get MSB of start area to clear  
;
LEF62	XGDY				;Swap Y and D, checksum in D
	ADDB    0,X			;Add in first address
	ADCA    #0			;Perform carry if needed
	ADDB    1,X			;Add in next address
	ADCA    #0			;
	ADDB    2,X			;
	ADCA    #0			;
	ADDB    3,X			;
	ADCA    #0			;
	ADDB    4,X			;
	ADCA    #0			;
	ADDB    5,X			;
	ADCA    #0			;
	ADDB    6,X			;
	ADCA    #0			;
	ADDB    7,X			;Add in last address
	ADCA    #0			;
	XGDY				;Swap Y and D back    
	ABX				;Add in offset, move to next block of memory     
	INCA 				;+1, moved to next block   
	BNE     LEF62			;Bra if no roll-over, continue, else do COP
;
;-Do COP
;
	LDAA    #$55			;0101 0101
	STAA    L303A			;Save it, COP reset
	COMA 				;Toggle bits   
	STAA    L303A			;Save it, COP reset
	CLRA 				;clear A   
	CPX     #0              	;Check for roll-over
	BNE     LEF62			;Bra if no roll-over, continue
;
	RTS				;Return     
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; RAM clear routine
;
; Enter:
;
;	X=Start address
;	D=Number of bytes to clear
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LEF9A	BITB    #$07			;Test b0, b1, b2
	BEQ     LEFA8			;Bra if b0, b1, b2==0, do mass clear of RAM
;
;-Here to clear byte at a time
;
	CLR     0,X			;Clear address
	INX 				;+1, advance to next address
	SUBD    #1			;-1 from remaining bytes to clear
	BNE     LEF9A			;Bra if !=0, still bytes remaining
;
	BRA     LEFC2			;Bra to return
;
;-Here to clear 8 byte blocks of RAM
;
LEFA8	LSRD				;/2   
	LSRD 				;/2   
	LSRD				;/2    
	TBA 				;Get LSB into A    
	LDAB    #$08			;Clearing  8 byte blocks
	LDY     #$0000 			;Preclear ind. reg. Y
LEFB2	STY     0,X			;clear byte 1,2
	STY     2,X			;clear byte 3,4
	STY     4,X			;ckear byte 5,6
	STY     6,X			;clear byte 7,8
	ABX   				;Add in offset, next block to clear  
	DECA    			;-1 from blocks remaining to clear
	BNE     LEFB2			;Bra if blocks remaining to clear
;
LEFC2	RTS 				;Return
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~	            
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; RAM checksum routine
; check $000A-$0014, flagged malfunction words
;
; ENTER WITH:
;	-
; EXIT WITH:
;	A = checksum
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LEFC3	LDX     #L0014			;Point to upppermost address to check
	LDAB    #10			;# of bytes to check
	CLRA				;Preclear A   
; 
LEFC9	SUBA    0,X			;- value of RAM addr.
	DEX				;-1, go to previous byte     
	DECB				;-1, byte counter   
	BNE     LEFC9			;Bra if !=0, continue
;
	RTS				;Return
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~				     
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; Routine to clear engine error flags/timers
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LEFD0	LDX     #$0009			;9 error words
	LDY     #LF01C			;Start of clearable errors mask 
;
;-Loop here to clear error words
;
LEFD7	LDAA    $0A,X			;Load flagged malfunction word 
	ANDA    0,Y			;And with mask, mask out stored flagged trans errors
	STAA    $0A,X			;Save it
;
	LDAA    $15,X			;Load error word 
	ANDA    0,Y			;And with mask
	STAA    $15,X			;Save it
										
	LDAA    $55,X			;Load stored error word
	ANDA    0,Y			;And with mask
	STAA    $55,X			;Save it
	DEY				;-1, move to next error word masks
	DEX				;-1, move to next error words
	BNE     LEFD7			;Bra if !=0, not done
;
	BCLR    L0004,#$80		;Malf. word #4, clear b6, Err 42, EST
	BCLR    L0044,#$80		;Status word, clear b7, EST failure flag
	BCLR    L0072,#$01		;
	BCLR    L006F,#$10		;
	BCLR    L0070,#$01		;Status word, clear b0, Err 32, EGR 										; 
	CLRA				;Preclear A
	STAA    L001F			;Err 21 counter 
	STAA    L0020			;Err 22 counter
	STAA    L0817			;Err 16 timer
	STAA    L023F			;Err 13 timer
	STAA    L081C			;Err 24 timer
	JSR     LEFC3			;Perform checksum
;
	STAA    L0015			;Malfunction word checksum
	RTS				;Return     					
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Clearable engine error masks
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; 0=clearable error	
;
LF014	FCB	$0E		;0000 1110, error word #1
	FCB	$0F		;0000 1111, error word #2
	FCB	$03		;0000 0111, error word #3
	FCB	$81		;1000 0001, error word #4
	FCB	$D9		;1101 1001, error word #5
	FCB	$FF		;1111 1111, error word #6
	FCB	$FF		;1111 1111, error word #7
	FCB	$FF		;1111 1111, error word #8
LF01C	FCB	$FF		;1111 1111, error word #9
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; Routine to clear transmission errors/timers
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LF01D	LDX	#$0009			;9 error words   
	LDY     #LF080			;Error word masks
;
;-Loop here to clear error words
;
LF024	LDAA    $0A,X			;Load error word 
	ANDA    0,Y			;And with mask
	STAA    $0A,X			;Save it
;
	LDAA    $15,X			;Load error word 
	ANDA    0,Y			;And with mask
	STAA    $15,X			;Save it
;						
	DEY				;-1, move to next error word masks
	DEX				;-1, move to next error words   
	BNE     LF024			;Bra if !=0, continue
;
	BCLR    L008E,#$F0		; CLR 1111 0000
	BCLR    L008A,#$80		; BR b7
;
;-Clear error timers
;
	CLRA				;    
	STAA    L001F			; ERR 21 CNT'R
	STAA    L0020			; ERR 22 CNT'R
	STAA    L0021			; ERR 24 CNT'R, LO OUT XMISH SPD
	STAA    L0022			; ERR 28 CNT'R
	STAA    L0023			; ERR 37 CNT'R, BRAKE STUCK ON
	STAA    L0024			; ERR 38 CNT'R, BRAKE STUCK OFF
	STAA    L0026			; ERR 37 CNT'R, 3 -> 2 SHFT QUAD DVR FAIL
	STAA    L0027			; ERR 67 CNT'R, TCC ENAB QUAD DVR FAIL
	STAA    L0028			; ERR 68 CNT'R
	STAA    L0029			; ERR 72 CNT'R
	STAA    L002A			; ERR 73 CNT'R, FORCE MOTOR CURREN
	STAA    L002B			; ERR 81 CNT'R, QUAD DVR 1 & SHFT B ERR
	STAA    L002C			; ERR 82 CNT'R, QUAD DVR 1 & SHFT A ERR
	STAA    L002D			; ERR 83 CNT'R, QUAD DVR 1 ERR
	STAA    L002F			; ERR 87 CNT'R,	LO RATIO
	STAA    L0030			; ERR 87 CNT'R, HI RATIO
	STAA    L0031			; ERR 89 CNT'R, MAX ADPT LONG SHIFT
	STAA    L017B			; ERR 37 TIMER
	STAA    L017C			;
	STAA    L017D			;
	STAA    L017E			;
	STAA    L0032			; ERR 89 CNT'S
	STAA    L0033			; ERR 89 CNT'R
	STAA    L0034			; ERR 89 CNT'R
	JSR     LEFC3			;Flagged malfunction words checksum routine
;
	STAA    L0015			;Malf. word checksum
	RTS				;Return     
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-Clearable transmission error masks
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; 0=clearable errors
;
LF078	FCB	$9E		;1001 1110, error word #1
	FCB	$5D		;0101 1101, error word #2
	FCB	$FC		;1111 1100, error word #3
	FCB	$7F		;0111 1111, error word #4
	FCB	$E7		;1110 0111, error word #5
	FCB	$99		;1001 1001, error word #6
	FCB	$00		;0000 0000, error word #7
	FCB	$50		;0101 0000, error word #8
LF080	FCB	$8B		;1000 1011, error word #9
;
;~~~~~~~~~~~~~~~~~~
;
; Hardware routine
;
;~~~~~~~~~~~~~~~~~~~
;    
LF081	LDAA	L3066			;Hardware	
	EORA	#$80			;Toggle b7
	STAA	L3066			;Save it, hardware	    
	RTS				;Return   
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;~~~~~~~~~~~~~~~~~~~~~
;
; Provide short delay
;
;~~~~~~~~~~~~~~~~~~~~~
;
LF08A	RTS				;Return  
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~    
;
;~~~~~~~~~~~~~~
;
; I/O routine
;
;~~~~~~~~~~~~~~
;
LF08B	SEI				;Disable interrupts     
	LDD     L3FFC			;I/O D port
	JSR     LF08A			;Delay


	ORAB    #$08			;Set b3,
	BRCLR   L0072,#$80,LF09A	;Bra if b7==0

	ANDB    #$F7			;Clear b3
;
LF09A	STD     L3FFC          		;Save, I/O port D
	LDAA    L3060			;
	ANDA    #$6F			;Mask, 0110 1111
	LDAB    L004B			;Load I/O word
	ANDB    #$90			;Mask, 1001 0000
	STAB    L3060			;Save it to hardware
	ABA				;Add in current hardware state
	STAA    L004B			;Save I/O word
;
;-Output to I/O port D
;											
	LDAA    L004C			;I/O port D word
	STAA    L3062			;Save, I/O PORT D
;
;-Read in I/O port C
;							
	LDAA    L3064			;I/O port C
	STAA    L004D			;I/O port C word
;											
	LDAA    L004E			;Load temp sensor status word
	STAA    L3067			;Save it to hardware
;
	CLI				;Enable interrupts
	RTS				;Return 			    
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; Entry point for filtering signed values
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;-Get difference between current and previous filtered value
;
LF0BD	PSHX 				;Push old value to stack
	TSX				;Get current stack pointer into X
	SUBD    0,X			;Subtract old filtered value from current value
	PSHB				;Push LSB to stack
	BLT     LF0CA			;Bra if result was < zero
;
;-Result was >= 0
;
	LDAB    0,Y			;Load filter coeff.
	MUL				;Filter coeff. x MSB delta value
	BRA     LF0E4			;Bra to continue
;
;-Result < 0
;	
LF0CA	LDAB    0,Y			;Load filter coeff.
	MUL				;Filter coeff x MSB of delta value
	SUBA    0,Y			;Subtract filter coeff.
	BRA     LF0E4			;Bra to continue
;
;~~~~~~~~~~~~~~~~~~~~~~~
;
; Filter routine
;
;~~~~~~~~~~~~~~~~~~~~~~~
;
; Enter with:
;	D = New value 
;	X = Old value 
;	Y = Addr. of filter coeff
;
; Exit with:
;	D = Filtered value
;
;-General form: D =Old value + Filter coeff x (New value - Old value)  
;
LF0D3	PSHX				;Old value to the stack
	TSX 				;Get current stack pointer into X
	SUBD    0,X			;New value - old value 
	PSHB				;LSB to stack
	LDAB    0,Y			;Load filter coeff
	BCS     LF0E0			;Bra if old value was greater
;							
;-Here if new value >= old value
;				
	MUL				;MSB of Delta value x filter coeff.
	BRA     LF0E4			;
;
;-Here if new value < old value
;											
LF0E0	MUL				;-MSB of Delta value x filter coeff
	SUBA    0,Y			;-filter coeff x 256, perform 2's comp., change sign
;
LF0E4	ADDD    0,X			;Add in old value
;
	STD     0,X			;Save result to the stack in place of old value
	PULA				;Get LSB of delta value back
	LDAB    0,Y			;Load filter coeff.
	MUL				;LSB x filter coeff.
	ADCA    1,X			;Add in LSB of previous result on stack and round if needed
	TAB				;MSB of (LSB x filter coeff) to B
	LDAA    0,X			;Load up MSB of previous result on stack
	ADCA    #0			;Perform carry from previous addition op. if applicable
;
;-Clean up the stack
;
	PULX				;
	RTS				;Return
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~  
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; 
; Lag filter routine
;
; Enter with:
;  A = New value
;  B = Filter coeff.
;  X = Old value
;
; Exit with:
;  AB = filtered value
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; Filtered value = old value + FC x (new value - old value)
;
LF0F6	PSHB				;Filter coeff. to stack
	PSHX				;Previous filtered value to stackl
	TSX				;Get current stack pointer
	SUBA    0,X			;New value - MSB old value
	BCS     LF100			;Bra if new value <
;
;-New value is greater then previous here
;											
	MUL				;Delta x filter coeff
	BRA     LF103			;Bra to continue
;
;-New value is less then previous here
;											
LF100	MUL				;Delta x filter coeff
	SUBA    2,X			;Sub. filter coeff.
;
LF103	PSHA				;Push result to stack
	PSHB				;
	LDD     1,X			;Load LSB of old value + FC
	NEGB				;Change sign of FC
	BEQ     LF10D			;Bra if ==0
;											
	MUL				;LSB Old value x -filter coeff
	ADCA    #0			;Round if needed
;
LF10D	PULB				;Get LSB of previous result off stack
	ABA				;Add it in
	TAB				;REsult to B
	PULA				;Get MSB of result off stack
	ADCA    0,X			;Add in previous value
;
;-Clean up stack
;
	PULX				;Pull X
	INS				;+1, stack pointer
	RTS				;Return
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; RPM filter routine
;
; Enter with:
;  D=Current RPMs
;  X=Addr. of Previous RPMs
;  Y=Addr. of filter coeff
;
; Exit with:
;  D=Filtered RPMs
;~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; 
;
;
LF116	SUBD    0,X			;Subtract previous RPMs
	PSHB				;Push LSB of result to stack
	LDAB    0,Y			;Load filter coeff.
	BCS     LF121			;Bra if underflow
;
;-Delta >= 0
;
	MUL				;(MSB of delta rpm) x coeff.
	BRA     LF125			;Bra to continue
;
;-Delta < 0
;
LF121	MUL				;(MSB of delta rpm) x coeff
	SUBA    0,Y			;-coeff., negate, now -coeff. x (65536 - delta)
;
LF125	ADDD    0,X			;Add to previous RPM val
	STD     0,X			;Save it
	PULA				;Get LSB back
	LDAB    0,Y			;coeff.
	MUL				;coeff. x LSB of delta RPMs 
	ADCA    1,X			;Add in LSB from previous calc. with carry
	TAB				;Get LSB of result into B
	LDAA    0,X			;Load MSB of result
	ADCA    #0			;Perform carry if needed
	RTS				;Return
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; Entry point for 2D lookup with UL with no offset
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LF136     CLRB				;Clear B, no offset
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; Entry point for 2D lookup with UL
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LF137	CMPA    0,X			;Start address of table upper limit for lookup			
	BLS     LF13D			;Bra if less

	LDAA    0,X			;Load upper limit
LF13D	INX				;+1, go to table
	BSR     LF15A			;Bra to do 2D lookup with SBA
;
	RTS				;Return
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; Alternate 2D lookup entry point
;
; Enter with: 
;  A = value used for lookup
;  X = Start addr of table and params
;
; Exit with:
;  A = value from lookup
;
; Table format: 
;  Addr = offset for lookup
;  Addr + 1 = Upper limit for offset value used for lookup 
;  Addr + 2 = # counts per line
;  Addr + 3 = Start of table
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LF141	PSHX				;Index to stack
	PSHB				;Push contents of B to stack
;
;-Subtract offset for lookup
;											
	SUBA    0,X			;Subtract offset from val. used for lookup
	BCC     LF148			;Bra if no underflow
;
	CLRA				;Clear underflow
;
;-Check upper limit for offset lookup value
;	
LF148	CMPA    1,X			;Upper limit for offset lookup value
	BLS     LF14E			;Bra if offset lookup value is <=
;
	LDAA    1,X			;Use limit instead
;											
LF14E	LDAB    #2			;2 byte offset
	ABX 				;Add in offset, skip to line count
	BRA     LF155			;Bra to do lookup				
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; Entry point for 2D lookup with line count
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;											
LF153	PSHX				;Push addr. to stack
	PSHB				;Pus contents of B to stack
;
LF155	LDAB    0,X			;Load line count
	INX				;+1, adv. to table
	BRA     LF162			;Bra to continue with lookup
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; Entry point for 2D lookup with SBA
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LF15A	SBA				;subtract offset from value used for lookup
	BCC     LF15E			;Bra if no underflow, do lookup
;
	CLRA  				;Clear underflow  
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; 2D lookup Routine
;
; Enter with:
;	A=value to use for lookup
;	X=Address of table
;
; Exit with:
;	A=looked up table value
;	B=Previous value on entry
;	X=Previous value on entry 
;  
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LF15E	PSHX				;Push table addr. to stack    
	PSHB 				;Push offset to stack 
	LDAB    #$10			;Multiplier for lookup, 16 counts/row
;
;-ALternate entry point
;
LF162	MUL				;Val x multiplier for lookup    
	PSHB				;remainder to stack    
	TAB				;Row offset into B     
	ABX				;Add in offset to X     
	LDD     0,X			;Load in table value and value following that
;
;-Perform interpolate 
;
	SBA 				;Subtract following value to get delta table value     
	PULB 				;Get remainder   
	BCC     LF172			;Bra if underflow, next table value is greater
;
;-Interpolate up
;
	NEGA 				;Make positive again   
	MUL 				;remainder x delta table value    
	ADCA    0,X			;Add in table value to interpolated value and round if needed
	BRA     LF178			;Bra to return
;
;-Interpolate down
;
LF172	MUL  				;remainder x delta table value   
	ADCA    #0			;Round if needed
	NEGA 				;Switch sign   
	ADDA    0,X			;Table value + (-interpolated value)
LF178	PULB				;Restore acc. B    
	PULX 				;Restore acc. X			   
	RTS				;Return     
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; 3D Lookup Routine
; 
; Enter with:
; 	A=Row value
;	B=Column. value
;	X=Table address
;
; Exit with:
; 	A=value from lookup
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LF17B   PSHY				;Reg. Y to stack    
	PSHB				;Col. val. to stack    
	PSHX 				;Table address to stack   
	SUBA    0,X			;Subtract row offset
	BCC     LF184			;Bra if no underflow
;	
	CLRA				;Clear out col. val.
;    
LF184	SUBB    1,X			;Subtract col offset
	BCC     LF189			;Bra if no underflow
;
	CLRB				;Clear underflow 
;   
LF189	PSHX				;Table addr. to stack    
	PULY 				;Get table addr. into Y  
	PSHA				;Row val. to the stack 

	LDAA    #$10			;Mult. for lookup
	MUL				;mult. x col. val, A now offset for lookup, B now remainder     
	PSHB 				;col. remainder to the stack    
	TAB 				;Get MSB into B   
	ABX				;Add the offset into X     
	PULA 				;Get col. remainder into A
	PULB				;Get row value back   
	PSHA				;Put the col. remainder back on the stack
;   
	LDAA    #$10			;mult. for lookup 
	MUL				;row value x mult. for lookup
	PSHB				;row remainder to the stack  
	LDAB    2,Y			;Load # of cols.
	MUL 				;# cols x row offset, skip to row in table    
	ABX				;Add the offset into X     
	PSHX				;Push address (current row) to stack    
	LDAB    2,Y			;Load # cols		
	ABX 				;Add it into X (Now next row)
;
;~~~~~~~~~~~~~~~~~~~~~~~
;-Perform interpolation
;~~~~~~~~~~~~~~~~~~~~~~~ 
;
;
;       V1= current stored value in table
;	V2-V4 are values located 
;	around the immediate table
;	value
;
;     -----------------
;   i |   V1  |   V2  |
;     -----------------
; i+1 |   V3  |   V4  |
;     -----------------
;         j      j+1
;
  
	TSY				;Get stack pointer + 1 into Y     
	LDD     3,X			;Get values in next row for interpolation
	SBA 				;Subtract table value in next col. from value in current col., now delta    
	LDAB    3,Y			;Load col. remainder from stack
	BCC     LF1B4			;Bra if delta value was negative
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Interpolate values in next row up
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;-Interpolate up
;
;	V2 + col. remainder x |(V3-V4)|
;
;
	NEGA				;Make delta positive   
	MUL 				;remainder from col. value x delta  
	ADCA    3,X			;Add it into table value in next row up
	BRA     LF1BA			;Bra
;
;-Interpolate down 
;
;	V2 - col. remainder x |(v3-V4)|
;	
;
LF1B4	MUL				;remainder from col. value x delta     
	ADCA    #0			;round if needed
	NEGA 				;Change sign   
	ADDA    3,X			;Add it into table value in next row up
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Interpolate table values in current row
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LF1BA	PULX				;Get current col/row address back    
	PSHA				;Push interpolated table value in next row to stack    
	LDD     3,X			;Get table values in current row		
	SBA				;Subtract table value in next col. from current value     
	LDAB    3,Y			;Load in col. remainder
	BCC     LF1CA			;Bra if next value is less
;											
	NEGA    			;Make positive
;
;-Interpolate up 
;
;	V1 + col. remainder x |(V1-V2)|
;                                            
	MUL				;remainder x delta     
	ADCA    3,X			;Add it into current value
	BRA     LF1D0			;Bra 		
;
;-Interpolate down
;
;	V1 - col. remainder x (V1-V2)
;
LF1CA	MUL				;remainder x delta    
	ADCA    #0			;Round if needed
	NEGA 				;change sign   
	ADDA    3,X			;Add in current value
;
;-Interpolate between the two rows
;
LF1D0	PULB 				;Get interpolated value from next row   
	PSHA				;Push interpolated value from current row to stack    
	SBA				;Current interpolated value - interpolated value in next row 
	LDAB    2,Y			;Load row remainder
	BCC     LF1DF			;Bra if delta was positive
										  
;
;-Interpolate up between rows
;
	NEGA				;change sign    					
	MUL 				;row remainder x delta value    
	ADCA    1,Y			;add to interpolated value in current row
	BRA     LF1E6			;Bra to 
;
;-Interpolate down between rows
;
LF1DF	MUL 				;row remainder x delta value    
	ADCA    #0			;Round up if needed
	NEGA				;change sign   
	ADDA    1,Y			;add it into the current interpolated value
;
;-Clean up the stack and resore X, Y, B
;
LF1E6	INS 				;Inc. stack pointer
	PULX 				;   
	PULX    			;
	PULB    			;
	PULY    			;
	RTS     			;Return
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~     
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; 8x16 multiply routine, (A x X*)/256
;
; Enter with:
;	A = 8 bit multiplier
;	X = Address of 16 bit multiplicand 
;
; Exit with:
;	AB = (mult. x mult'd) / 256
;
; (*) X points to address of mult'd
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LF1ED	PSHA				;Push mult. to stack   
	LDAB    1,X			;Get LSB of multiplicand
	MUL				;LSB x mult.     
	ADCA    #0			;Round if needed
	PULB				;Get multiplier back    
	PSHA				;Save product of LSB x mult., now /256
;  
	LDAA    0,X			;Get MSB of multiplicand
	MUL				;MSB x mult., upper portion of result/256   
	PSHX				;Push address of mult. to stack    
	TSX				;Get current stack pointer into X    
	ADDB    2,X			;Add in (LSB x mult.)/256
	ADCA    #0			;Perform carry if needed
;
;-Clean up the stack
;
	PULX				;Pull contents off stack  
	INS				;+1, reset the stack 			
	RTS				;Return
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; 16 x 16 multiply routine   
;
;Enter with:
;
; AB=mult'd
; X=mult.
;
;Exit with:
;
; AB = result = (mult. x mult'd)/256
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LF201	PSHA				;Make some space on the stack for tmp storage
	PSHX				;  
	PSHX 				;Push mult'd to stack   
	PSHB				;Push LSB of mult. to stack
	PSHA				;Push MSB of mult. to stack    
	TSX				;Get current stack pointer into X     
	LDAA    3,X			;Get LSB of mult'd
;
;-LSB x LSB
;
	MUL				;LSB of mult. x LSB of mult'd
	ADCA    #0			;Round if needed
	STAA    5,X			;Save result
;
;-LSB x MSB
;
	LDD     1,X			;Get LSB of mult. and MSB of mult'd						            	; & MSB of Mult'cnd in B
	MUL				;LSB mult. x MSB mult'd    
	ADDB    5,X			;Add in previous result
	ADCA    #0			;Perform carry if needed
	STD     4,X			;Save result
;
;-MSB x LSB
;
	LDAA    0,X			;Get MSB of mult.
	LDAB    3,X			;Get LSB of mult'd
	CLR     6,X			;Clear MSB of result
	MUL				;MSB mult. x LSB mult'd
	ADDD    4,X			;Add in previous result
;
	ROL     6,X			;Rotate C bit into MSB of result, perform carry
	STD     4,X			;Save lower two bytes of result
;
;-MSB x MSB
;
	LDAA    0,X			;Get MSB of mult.
	LDAB    2,X			;Get MSB of mult'd
	MUL				;MSB mult. x MSB mult'd  
	ADDB    4,X			;Add in middle byte of result
;
	ADCA    6,X			;Add in MSB of result with carry
;
	STAB    4,X			;Save LSB of result
	STD     2,X			;Save result, upper two bytes of final result
	TSTA				;Test uppermost byte of 32 bit result    
	BEQ     LF239			;Bra if ==0, result within range
;
	LDD     #$FFFF			;Clear overflow, result out of range
	STD     4,X			;Save as final result
;
;-Clean up the stack
;
LF239	PULX 				;  
	PULX				;				    
	PULA				;Get MSB of final result/256    
	PULB				;Get LSB of final result/256   
	INS				;+1, stack pointer     
	RTS				;Return     
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; 16 x 16 multiply routine   
;
;Enter with:
;
; AB=mult'd
; X=mult.
;
;Exit with:
;
; AB = result = (mult. x mult'd)/65536
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LF23F	PSHX				;Push mult. to stack   
	PSHA				;Push MSB of mult'd to stack    
	PSHX				;Push mult. to stack    
	PSHB				;Push mult'd to stack    
	PSHA				;    	
	TSX				;Get current stack pointer in
;
;-Begin multiplying AB x X
;     
	LDAA    3,X			;Load LSB of mult.
	MUL				;LSB of mult. x LSB of mult'd     
	STD     5,X			;Save result
	LDD     1,X			;Load LSB mult'd, MSB of mult.
	MUL				;MSB mult x LSB of mult'd     
	ADDB    5,X			;Add in MSB of previous MSB x LSB result
	ADCA    #0			;Perform carry if needed
	STD     4,X			;Save MSB x LSB of result
	LDAA    0,X			;Load MSB of mult'd
	LDAB    3,X			;Load LSB of mult.
	CLR     3,X			;Pre-clear MSB of result		
	MUL				;LSB of mult. x MSB of mult'd    
	ADDD    4,X			;Add in previous result
	ROL     3,X			;Perform carry if needed
	STD     4,X			;SAve result
	LDAA    0,X			;Load MSB of mult'd
	LDAB    2,X			;Load MSB of mult.
	MUL				;MSB of mult. x MSB of mult'd     
	ADDD    3,X			;Add in previous result
;
;-Clean up stack
;
	INS				;Inc. stack pointer     
	PULX				;   
	PULX				;
	PULX				;
	RTS				;Return     
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;~~~~~~~~~~~~~~~~~~
;
; Heads up routine
;
;~~~~~~~~~~~~~~~~~~
;
LF26C	BRCLR   L0082,#$40,LF273	;Status word, bra if b6==0, heads up not online
;
	JSR     L1806			;To HU vector table
;
LF273	LDAA    #$F7			;
	STAA    L0F00			;HU hardware
;
	RTS				;Return    
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; 
; Routine to init. mode 4 operation
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LF279	LDX     L1800			;Start of heads up section of EPROM
	CPX     #$7E18			;First two stored bytes in jump table
	BNE     LF284			;Bra if HU not present
;
;-HU is present
;
	JSR     L180C			;Routine to reset HU ram and enable HU
;											                   
LF284	RTS				;Return
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; ALDL scheduled message transmission routine
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LF285	LDAA    L0042			;ALDL diag. status word				
	ANDA    #$03			;Clear all but b0, engine diags., and b1, transmission
					;diags. 
	BNE     LF2EA			;Bra if either mode enabled
;
;-Not in transmission or engine diag. mode
;
	LDX     #L5185			;Load in start of ALDL transmission tables
	LDAB    L0002			;Loop counter
	ANDB    #$1F			;Clear all but b0-b4
	ABX				;Use as offset
	LDX     0,X			;Get addr.
	BEQ     LF2EA			;Bra if ==0, return
;
	LDAA    2,X			;Get next addr. 
	BEQ     LF2EA			;Bra if ==0, return
;
;-Check for transmission overrun
;
	BRSET   L003B,#$40,LF2E7	;Status word, bra if b6==1, 8192 xmission still in 
					;progress
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; No overrun, start xmission
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
	BSET    L003B,#$40		;Set b6, ALDL transmission in progress											;
	STX     L0364			;Save addr., pointer to ALDL transmission string
	LDAB    L302E			;Load SCI status reg
	STAA    L302F			;Save for transmission
	STAA    L0363			;Save it, ALDL message checksum
	LDAB    4,X			;Load in message length from ALDL transmission params
	STAB    L0367			;Save it, ALDL message length
	SEI				;Disable interrupts
	BEQ     LF2CF			;Bra if message length was ==0
;
	LDAA    3,X			;Load in memory flag
	BITA    #$C0			;Test b6, b7
	BNE     LF2CF			;Bra if b6, b7==1
;
	LDY     5,X			;Load in outgoing ALDL data buffer
;
;-Upload data to ALDL data buffer
;
LF2BF	PSHX    			;Push pointer to ALDL transmission string to stack
	LDX     9,X			;Load in addr. of data to transmit
	LDAA    0,X			;Load A with data
	STAA    0,Y			;Save it to ALDL data buffer
	INY				;+1, advance to next addr.     
	PULX 				;Get pointer back   
	INX				;+1     
	INX				;+1, advance to next addr
	DECB				;-1 from byte counter    
	BNE     LF2BF			;Bra if !=0, still have data to upload
;
;-Here if message length was ==0
;
LF2CF	LDAB    #$01			;Transmit 1 byte
	STAB    L0362			;Save it, ALDL message length
	LDD     L3FFC			;I/O port D
	JSR     LF08A			;Delay
;
	ORAB    #$04			;Set b2
	STD     L3FFC          		;Save it
	CLI				;Enable interrupts
;
;-Request SCI transmission interrupt
; 
	LDAA    #$88			;Set b3, transmit enable and b7, transmit interrupt
					;enable
	STAA    L302D			;Save it, SCI control reg. 2
	BRA     LF2EA			;Bra to return
;
;-Here if transmission exceeded time available
;
LF2E7	BSET    L003B,#$20		;ALDL status word, set b5, ALDL transmission overran
;
LF2EA	RTS				;Return   
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; Routine to handle mode 4 engine/trans. diagnostics
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;~~~~~~~~~~~~~~~~~~~~~~~
; Update TCC diag. word
;~~~~~~~~~~~~~~~~~~~~~~~
;
LF2EB	LDAA    L0088			;Load TCC status word
	ANDA    #$D9			;Mask the following:
					; b7, Excessive negative torque converter slip
					; b6, A/C on
					; b4, Rapid decel present
					; b3, Brakes have been applied 
					; b0, In manual range
;	
	LDAB    L0089			;Load TCC status word
	ANDB    #$02			;Mask b1, TCC disabled to prevent repeat
					;locking/unlocking
	ABA				;Add flags together 
	LDAB    L0085			;Load TCC status word	
	ANDB    #$20			;Mask b5, TCC locked
	ABA				;Add in the flag
	STAA    L012C			;Save it, TCC diag. word
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Update adaptive modifier word
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
	LDAA    L00F7			;
	ANDA    #$01			;
	LDAB    L008B			;Load adaptive modifier update status word
	ANDB    #$A4			;Mask the following:
					; b7, Excessive change in TPS
					; b5, Excessive transmission slip present
					; b2, TPS out of range for adaptive modifier update
;
	ABA				;Add in the flags
	LDAB    L008C			;Load adaptive modifier update status word
	ANDB    #$04			;Mask b2, change is speed to great 
	ASLB				;Shift left
	ASLB				;Shift left, flag now in b4 position
	ABA				;Add the flag in
	LDAB    L0083			;Load status word
	ANDB    #$02			;Mask b1, hot transmission flag
	ABA				;Add it in
	STAA    L012D			;Save it, adaptive modifier diag. word
;
;-Check if transmission diag. in effect
;
	BRCLR   L0042,#$02,LF36A	;ALDL diag. status word, bra if b1==0, transmission
					;diag. not in effect
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Transmission diag. in effect
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
	LDX     #L039B			;Load start of trans diag. data buffer 
	LDAA    L039B			;Load ALDL mode number (start of trans. diag. data
					;buffer)
	CMPA    #$04			;ALDL mode 4
	BNE     LF39F			;Bra if !=, mode 4 diags. not in effect
;
	LDD     L036C			;ALDL mode 4 re-enable delay timer
	BNE     LF39F			;Bra if !=0, not enough time has passed to re-enable
					;mode 4 diags.
;
;-ALDL mode 4 diags. in effect
;
	BSET    L003B,#$08		;Status word, set b3, in mode 4 (trans diags.)
;
;-See if shift solenoid state override in effect
;
	BRCLR   1,X,#$18,LF35E		;L039C of diag. buffer, bra if b3, b4==0, no override
					;specified
;
;-Shift solenoid override desired
;
	CLRA				;Clear desired solenoid state
	BRCLR   1,X,#$08,LF339		;L039C, bra if b3==0, no 1-2 solenoid override
;
	BRCLR   2,X,#$08,LF339		;L039D, bra if b3==0, no 1-2 solenoid override

	ORAA    #$01			;Set b0, 1-2 shift solenoid state to on
;					
LF339	BRCLR   1,X,#$10,LF343		;L039C, bra if b4==0, no 2-3 solenoid override
;
	BRCLR   2,X,#$10,LF343		;L039D, bra if b4==0, no 2-2 solenoid override
;
	ORAA    #$02			;Set b1, 2-3 shift solenoid state to on 
;
LF343	LDY     #LB3A1			;Index shift solenoid state table
	CLRB				;Preclear gear #
	CMPA    0,Y			;State for first gear
	BEQ     LF35A			;Bra if ==, continue
;
	INCB				;+1, second gear 
	CMPA    1,Y			;State for second gear
	BEQ     LF35A			;Bra if ==, continue
;
	INCB				;+1, third gear
	CMPA    2,Y			;State for third gear
	BEQ     LF35A			;Bra if ==, continue
;
	INCB				;+1, 4th gear
;
LF35A	INCB				;+1 so gear # is actual numeral corresponding to 
					;gear #
	STAB    L0103			;Mode 4 diag. gear #
;
;-Check if PWM TCC diag. requested
;
LF35E	BRCLR   5,X,#$80,LF36C		;L03A0, bra if b7==0, PWM TCC diag. not requested
;
	LDAA    L03A1			;Load diag. DC from data buffer
	STAA    L010A			;Save it, PWM TCC diag. DC
	BRA     LF376			;Bra to continue
;
;-Here if trans diag. not in effect
;										
LF36A	BRA     LF39F			;Bra to continue
;	
;-Check if 3->2 shift solenoid diag. requested
;										
LF36C	BRCLR   5,X,#$40,LF376		;Bra if b6==0, 3->2 shift solenoid diag. not requested
;										
	LDAA    L03A1			;Load diag. DC from data buffer
	STAA    L0118			;Save it, 3->2 shift sol. diag. DC	
;
LF376	BRCLR   L003B,#$08,LF399	;Bra if b3==0, no mode 4 trans diags.
;
;-Check if reset of adaptive modifier cells is desired
;
	BRCLR   7,X,#$40,LF399		;L03A2, bra if b6==0, adaptive modifier reset not
					;requested
;
	LDD     #$0000			;Clear AB to reset adaptive modifier cells
	STD     L031B			;
	STD     L031D			;
	STD     L031F			;
	STD     L0321			;
	STD     L0323			;
	STD     L0325			;
	STD     L0323			;
	STD     L0325			;
;
;-Load diag. transmission line pressure
;										 
LF399	LDAA    L03A3			;Load line pressure from data buffer
	STAA    L0111			;Save it, diag. line pressure
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Check and reset mode 4 transmission diags. flags and values
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LF39F	BRCLR   L003B,#$08,LF3BF	;Bra if b3==0, no mode 4 trans diags.
;
;-Mode 4 diags. where in effect
;
	BRCLR   L0042,#$02,LF3AC	;ALDL diag. status word, bra if b1==0, transmission
					;diag. not in effect
;	
;-Mode 4 trans. diags. still desired
;										
	LDD     L036C			;ALDL mode 4 re-enable delay timer
	BEQ     LF3BF			;Bra if ==0, time threshold met to re-enable diags.
;											
;-Trans diags. no longer in effect, perform resets
;
LF3AC	CLRA				;Clear A to perform resets
	STAA    L039B			;ALDL mode number
	STAA    L010A			;PWM TCC diag. DC
	STAA    L0103			;Mode 4 diag. gear #
	STAA    L0111			;Diag. line pressure
	STAA    L0118			;3->2 shift sol. diag. DC
	BCLR    L003B,#$08		;Reset mode 4 trans. diag. flag
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Check for ALDL mode 4 (engine diags)
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LF3BF	BRCLR   L0042,#$01,LF3E3	;Status word, bra if b0==0, not in ALDL mode 4
;
	LDAA    L0390			;Load ALDL mode number (start of eng. diag. data
					;buffer)
	CMPA    #$04			;ALDL mode 4
	BNE     LF3E3			;Bra if !=, not in ALDL mode 4
;
;-In ALDL mode 4 (Engine diags) 
;
	BSET    L003B,#$10		;Status word, set b4, in ALDL mode 4 (engine diags.) 
;
;-Check if BLM reset requested
;
	LDAA    L0393			;Load external request word from diag. buffer
	BITA    #$10			;Test b4, reset BLMs
	BEQ     LF3E0			;Bra if ==0, reset BLM command not sent
;
	BRSET   L0045,#$10,LF3E3	;Status word, bra if b4==1, BLMs already reset
;
	JSR     LEF1D			;Routine to reset BLMs
;
	BSET    L0045,#$10		;Set b4, BLMs reset
	BRA     LF3E3			;Bra to continue
;
;-Here if reset not requested
;
LF3E0	BCLR    L0045,#$10		;Clear b4, BLM reset flag
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~
; Check if in ALDL mode 10
;~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LF3E3	BRSET   L003B,#$02,LF3F2	;Status word, bra if b1==1, in ALDL mode 10, clear
					;error codes
;
;-In other modes here
;
	BRCLR   L003B,#$10,LF3FE	;Status word, bra if b4==0, not in mode 4 
;	
	LDAA    L0393			;Ext. request word
	BITA    #$40			;Test b6, clear error codes
	BEQ     LF3FE			;Bra if ==0, request to clear error codes not sent
;                     
LF3F2	BRSET   L0045,#$80,LF401	;Status word, bra if b7==1, error codes reset 
					;already
;
	JSR     LEFD0			;Routine to clear error flags/timers
;
	BSET    L0045,#$80		;Status word, set b7, error flags cleared
	BRA     LF401			;Bra to continue
;
;-Here if no request to clear codes
;											
LF3FE	BCLR    L0045,#$80		;Clear b7, error codes cleared flag
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; check if in ALDL mode 4 (trans. diags.)
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LF401	BRCLR   L003B,#$01,LF411	;Status word, bra if b0==0, not in mode 4 
;
	BRSET   L0045,#$40,LF414	;Status word, bra if b6==1, transmission error 
					;codes already reset
;
	JSR     LF01D			;Routine to clear trans error codes/timers
;										
	BSET    L0045,#$40		;Set b6, trans error codes cleared
	BRA     LF414			;Bra to continue
;
;-Here if no mode 4
;
LF411	BCLR    L0045,#$40		;Clear b6, trans. error codes cleared flag
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Check if enough time has passed to re-enable mode 4
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LF414	BRCLR   L003B,#$10,LF42A	;Status word, bra if b4==0, not in mode 4 (eng diags.)
;
	BRCLR   L0042,#$01,LF421	;Status word, bra if b0==0, not in ALDL engine
					;diags.
;
;-In mode 4
;
	LDD     L036C			;ALDL mode 4 re-enable delay timer
	BEQ     LF42A			;Bra if ==0, entry into mode 4 ok'd
;	
;-Not enough time has passed yet
;									
LF421	BCLR    L003B,#$10		;Status word, clear b4, bar ALDL mode 4
;
	CLR     L0390			;Clear ALDL mode # in diag. data buffer
	BCLR    L0044,#$04		;Status word, clear b2, skip EST test flag
;
LF42A	RTS				;Return     
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; ALDL mode 4/inactivity handling routine
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LF42B	SEI				;Disable interrupts     
	BRCLR   L0042,#$03,LF464	;Status word, bra if b0, ALDL engine diags.,
					;and b1, ALDL transmission diags., not enabled
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; ALDL diags. enabled, check for inactivity
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
	LDAA    L0369			;ALDL inactivity timer
	INCA				;+1    
	BEQ     LF43D			;Bra if roll-over, perform resets
;
	STAA    L0369			;Save timer
	CMPA    #200			;10 secs
	BCS     LF464			;Bra if timer <, dont reset yet
;
;-ALDL is inactive, perform resets
;
LF43D	CLR     L039B			;Start of ALDL transmission diag. buffer,
					;ALDL mode #
	CLR     L0390			;Start of ALDL engine diag. buffer, ALDL mode #
	BCLR    L0042,#$03		;Clear b0, b1 
	BCLR    L0045,#$D0		;Status word, clear b7, engine error codes reset,
					;b6, transmission error codes reset, and b4, BLMs
					;reset
;
	BCLR    L003B,#$43		;Status word, clear b4, ALDL transmission in 
					;progress, b1, in mode 4 (engine), and b0,
					;in mode 4, transmission
;
	LDD     L3FFC			;I/O port
	JSR     LF08A			;Short delay
;
	ANDB    #$00FB			;1111 1011
	STD     L3FFC           	;I/O port
;
	LDAA    #$26			;Set 1, SCI reciever in standby, b2, SCI reciever
					;enable, and b5, SCI interrupt request enabled
	STAA    L302D			;Save it, SCI control reg.
;
	CLRA				;
	CLRB				;
	STD     L0364			;Clear pointer to ALDL transmission string
	STD     L0362			;Clear ALDL message length
;
;-Check if ALDL mode 4 is in effect
;										
LF464	CLI				;Enable interrupts
	BRCLR   L003B,#$18,LF47D	;Bra if b3, in mode 4 (trans diags.) and b4, in mode
					;4, (engine diags.)
;
;~~~~~~~~~~~
; In mode 4
;~~~~~~~~~~~
;
	LDX     L036A			;ALDL mode 4 duration timer
	INX				;+1
	STX     L036A			;Save timer
	CPX     L51B1			;Time in mode 4 to prevent immediate re-entry
	BCS     LF493			;Bra if timer <
;
;-Enough time has passed to prevent immediate re-entry into ALDL mode 4
;
	LDX     #$0001			;
	STX     L036C			;Reset ALDL mode 4 re-enable delay timer
	BRA     LF493			;Bra to return
;
;~~~~~~~~~~~~~~~~
; In other modes
;~~~~~~~~~~~~~~~~
;
LF47D	LDX     L036C			;Mode 4 re-enable delay timer
	BEQ     LF493			;Bra if ==0, return
;	
	INX				;+1
	STX     L036C			;Save it
	CPX     L51B3			;Time to prevent re-entry into ALDL mode 4
	BCS     LF493			;Bra if timer <, no mode 4
;	
;-Enough time has passed, clear timers and allow re-entry ALDL mode 4 operation
;		
	CLRA				;Clear timers
	CLRB				;
	STD     L036C			; 
	STD     L036A			;
;
LF493	RTS				;Return    
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; SCI interrupt handling routine
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LF494    LDX     #L3000			;Start of CPU reg.
;
;-Check which function generated interrupt
;                                           
	BRCLR   $2D,X,#$20,LF4A4	;SCI control reg. 2, bra if b5==0, RX interrupt 
;					;not enabled, TX instead	
;-Check if data was recieved
;
	BRCLR   $2E,X,#$20,LF4CB	;SCI status register, bra if b5==0, RX data reg. empty
;
;~~~~~~~~~~~~~~~~~~~~~~~~
; Data has been recieved
;~~~~~~~~~~~~~~~~~~~~~~~~
;
	JSR     LF5B5			;SCI data reception handler
	BRA     LF4CB			;Bra to return
;
;-Check if TX interrupt was enabled
;                                         
LF4A4	BRCLR   $2D,X,#$80,LF4B1	;Bra if b7==0, TX interrupt not enabled
;
;-TX interrupt enabled
;
	BRCLR   $2E,X,#$80,LF4CB	;Bra if b7==0, TX register is empty
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Data is wating to be transmitted
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;											
	JSR     LF4CC			;ALDL transmission routine
;										
	BRA     LF4CB			;Bra to return
;
;-Check if transmission completed interrupt was set
;
LF4B1	BRCLR   $2D,X,#$40,LF4CB	;SCI control reg 2, bra if b6==0, no TX complete 
					;interrupt was set
;
;-TX complete interrupt was set
;                                            
	BRCLR   $2E,X,#$40,LF4CB	;SCI status register, bra if b6==0, transmitter
					;is buisy
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Transmission complete, reset SCI
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
	LDAA    #$26			;Set b1, reciever wakeup, b2, recieve enable, and 
					;b5, recieve interrupt enable
	STAA    $2D,X			;Save it, SCI control reg. 2
	LDD     L3FFC			;I/O port D
	JSR     LF08A			;Delay
;
	ANDB    #$FB			;Clear b2
	STD     L3FFC              	;Save it
	BCLR    L003B,#$40		;ALDL diag. status register, clear b6, ALDL 
					;transmission complete
;
LF4CB	RTI				;Return  
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~   
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; ALDL transmission routine
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~
;  
LF4CC	LDX     L0364			;Pointer to ALDL transmission string
	LDAB    L0362			;Message length counter
	DECB				;-1
	BNE     LF4FC			;Bra if !=0
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Here to transmit message length
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;							           
	LDAA    4,X			;Load in number of bytes to transmit
	BRCLR   L0042,#$03,LF4F4	;ALDL diag. status word, bra if b0, b1==0, 
					;engine/transmission diags. not enabled
;
;-Engine/transmission diags. enabled
;
	PSHA				;Push byte count to stack    
	LDAB    L0390			;Start of engine diag. buffer, load desired ALDL mode
	LDAA    L0366			;Current ALDL diag. ID
	CMPA    #$F5			;Diag. ID for transmission diags.
	BNE     LF4E9			;Bra if !=
;
;-Transmission diags. enabled
;
	LDAB    L039B			;Load start of trans. diag. buffer instead
;
LF4E9	PULA				;Get number of bytes to tx back    
	CMPB    #$03			;Mode 3, 8 byte addr. dump
	BNE     LF4F4			;Bra if not in mode 3
;
;-In mode 3, 8 byte memory dump here
;                                     
	LDAA    L0367			;ALDL message length
	DECA				;-1    
	ASRA				;/2 to get transmission length  
	INCA				;Restore message length
;
;-In other modes
;
LF4F4	STAA    L0367			;ALDL message length
	ADDA    #$55			;Add $55, used for error checking
	JMP     LF590			;Jump to xmit
;
;-Here if still transmitting data
;
LF4FC	DECB				;-1    
	BNE     LF513			;Bra if !=0

;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Here to transmit ALDL mode
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
	BRCLR   L0042,#$03,LF513	;ALDL diag. status word, bra if b1, b2==0, not in
					;engine or trans. diag. mode
;
;-In either engine or transmission diag. mode
;
	LDAA    L0390			;Start of engine diag. buffer, current ALDL mode
	LDAB    L0366			;Current ALDL diag. ID
	CMPB    #$F5			;Transmission diag. ID
	BNE     LF510			;Bra if !=, in engine diags.
;
;-In transmission diags.
;
	LDAA    L039B			;Start of transmission diag. data buffer, load ALDL
					;mode
LF510	JMP     LF590			;Go transmit ALDL mode
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Here to continue with general data transmission, find out which addr. mode is desired
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LF513	CMPB	L0367			;ALDL message length
	BCC     LF58A			;Bra if stored message length is >=, invalid
					;request
;
	BRSET   L003B,#$80,LF57E	;ALDL diag. status word, bra if b7==1, double
					;byte transmission in progress
;
	BRSET   3,X,#$80,LF52B		;ALDL transmission opt. byte, bra if b7==1, 
					;transmitting addr. from diag. table in prom
;
	BRSET   3,X,#$40,LF536		;ALDL transmission opt. byte, bra if b6==1, 
					;transmitting from ram/PROM in various modes
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Perform scheduled ALDL transmission
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
	LDX     5,X			;Load start addr. of ALDL data buffer
	ABX				;Add in byte count as offset  
	LDAA    0,X			;Load stored data from buffer
	BRA     LF590			;Bra to transmit			                                             
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Here to transmit ALDL using definitions in diagnostic tables in PROM
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LF52B	BRCLR   L0042,#$03,LF530	;ALDL diag. status word, bra if b0, b1==0, not in 
					;diagnostics mode
;
	DECB				;-1 from byte count
;
LF530	ASLB				;x2 for double byte addressing
	ABX				;Add in as offset
	LDX     9,X			;Load address of data to transmit from table in prom
	BRA     LF562			;Bra to continue
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Here to transmit from prom/RAM in various modes
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;										
LF536	DECB				;-1 from byte count
	LDX     5,X			;Load in addr. of outging data buffer
	PSHB				;Push byte count to stack
	LDAA    L0390			;Start of engine diag. data buffer, load ALDL mode								
	LDAB    L0366			;Current ALDL diag. ID
	CMPB    #$F4			;Engine diag. ID
	BEQ     LF54C			;Bra if ==, in engine diagnostics
;
;-Check if in transmission diags.
;                                     
	LDAA    L039B			;Start of transmission diag. data buffer, load ALDL
					;mode
	CMPB    #$F5			;Transmission diag. ID
	BEQ     LF54C			;Bra if ==, in transmission diags.
;
;-Not in engine/transmission diags.
;								
	CLRA				;
;
;-In engine/transmission diags.
;
LF54C	PULB				;Get byte count back
	CMPA    #$02			;ALDL mode 2, 64 byte memory dump 
	BEQ     LF55E			;
;
;-Transmitting from prom
;
	ASLB				;Byte count x2 for double byte addr.
	INCB				;+1
	CMPA    #$04			;ALDL mode 4, controller mode
	BNE     LF559			;Bra if !=
;
;-In mode 4, controller mode
;
	ADDB    #$0A			;Add in 10 byte offset
;
;-In other modes
;
LF559	ABX				;Add in byte count as offset
	LDX     0,X			;Load addr. of value to be transmitted
	BRA     LF562			;Bra to continue				
;
;-In mode 2 here, 64 addr. memory dump
;											
LF55E	INX				;+1
	LDX     0,X			;Load start addr. of data to begin transmitting
	ABX				;Add in byte count as offset     
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Check to see if addr. read will fall in the range of the CPU hardware
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;	
LF562	CPX     #$1000			;End of RAM
	BCS     LF586			;Bra if <
;
	CPX     #$3FFF			;End of hardware 
	BHI     LF586			;Bra if >
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-In addr. range of CPU hardware, prevent reads from hardware
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
	XGDX				;Swap address into D
	BRSET   L0050,#$08,LF573	;Status word, bra if b3==1, factory diag. requested
;
;-Not in factory diags., block reads from addr range $1400-$3000
;
	ANDA    #$CF			;Mask all but b4, b5 from address
;
;-In factory diags. allow all reads
;
LF573	XGDX				;Get address back
	LDD     0,X			;Load in value from addr
	STAB    L0368			;Save LSB for later transmission
	BSET    L003B,#$80		;ALDL diag. status word, set b7, double byte
					;transmission in progress
	BRA     LF590			;Bra to transmit
;
;-Here to transmit LSB of double byte value
;
LF57E	BCLR    L003B,#$80		;Clear b7, double byte transmission complete
	LDAA    L0368			;Load LSB of double byte value to transmit
	BRA     LF590			;Bra to transmit data
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Normal addr range requested
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LF586	LDAA    0,X			;Load value from specified location
	BRA     LF590			;Bra to transmit
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Here if no data/invalid request
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LF58A	BNE     LF59E			;Bra if message length was !=0, invalid request
;
;-Not transmitting data, just message checksum
;
	LDAA    L0363			;ALDL message checksum
	NEGA				;Negate
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Here to save data for transmission
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LF590	STAA    L302F			;SCI data register
	ADDA    L0363			;Add into ALDL message checksum
	STAA    L0363			;Save it
	INC     L0362			;+1, message byte counter
	BRA     LF5B4			;Bra to return
;
;~~~~~~~~~~~~~~~~~~~~~~~~~
; Here if invalid request
;~~~~~~~~~~~~~~~~~~~~~~~~~
;
LF59E	CLRA				;Clear regs.
	CLRB				;
	BCLR    L003B,#$80		;ALDL diag. status word, clear b7, double byte
					;transmission flag										
	STD     L0362			;Clear message byte counter, SCI data checksum
	STD     L0364			;Clear pointer for ALDL transmission string 										
	LDAA    L302E			;Access SCI status reg.
	LDAA    L302F			;Access SCI data register										
	LDAA    #$40			;Set b6, interrupt when transmission complete
	STAA    L302D			;SCI control reg. 2
;
LF5B4	RTS				;Return
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; Routine to handle ALDL data reception
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LF5B5	LDAB    L302E			;SCI status register
	LDAA    L302F			;SCI data register
	LDX     L0364			;Pointer to ALDL transmission string
	CLR     L0369			;ALDL inactivity timer
;
;-Check for transmission errors
;
	BITB    #$0E			;Test b3=overrun error flag, b2=noise flag, and 
					;b1=framing error flag
	BNE     LF5E3			;Bra if any errors detected
;
;-No errors, 
;
	TAB				;Get data into B     
	ADDB    L0363			;Add data into ALDL message checksum
	STAB    L0363			;Save it, ALDL message checksum
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Check if device ID has been recieved
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
	LDAB    L0362			;Check message legnth counter 
	BNE     LF5FF			;Bra if !=0, device ID already recieved, continue
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Device ID has been recieved
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
	LDX     #L51B5			;Start of ALDL entry addresses
	BRCLR   L0050,#$08,LF5DB	;Bra if b3==0, no factory test 
;
;-Factory test requested
;
	LDX     #LF881			;Start of factory diagnostics prams
;
LF5DB	CMPA    2,X			;Check ID code
	BEQ     LF5E5			;Bra if device ID matches current message table
;											
	LDX     0,X			;Skip to next message table
	BNE     LF5DB			;Bra if !=0, continue to follow addr. pointers to 
					;transmission diags. portion
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Here if errors during transmission
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;											
LF5E3	BRA     LF660			;
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Check which diagnostic streams to use
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LF5E5	CMPA    #$F4			;Device ID for engine diagnostics
	BNE     LF5EE			;Bra if device ID !=
;											
	BSET    L0042,#$01		;ALDL diag. mode word, set b0, ALDL engine diags. flag
	BRA     LF5F5			;Bra to continue
;
;-Check if transmission diagnostics desired
;
LF5EE	CMPA    #$F5			;Device ID for trans. diagnostics
	BNE     LF5F5			;Bra if !=, neither engine/transmission diag. modes 
					;requested
;											
	BSET    L0042,#$02		;Set b1, ALDL transmission diags. flag
;
LF5F5	STX     L0364			;Save current pointer
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Set up to recieve next serial transmission
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
	LDAA    #$24			;Set b5, recieve interrupt enable, and b2, enable SCI
					;reciever 
	STAA    L302D			;Save it, SCI control reg. 2
;
	BRA     LF65A			;Bra to inc. message length counter and return
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Here if the remainder of the ALDL request has been recievied
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LF5FF	DECB				;-1 from message length counter
	BNE     LF60F			;Bra if !=0, desired mode transmitted
;
;-Check recieved message length to ensure valid request
;
	SUBA    #$55			;-55, used as RX/TX error check
	BCS     LF660			;Bra if <, request not valied
;
	CMPA    #$22			;34, max message length
	BHI     LF660			;Bra if >, buffer overrun
;
	STAA    L0367			;Save it, ALDL message length
	BRA     LF65A			;Bra to continue
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Check to see which ALDL mode is desired
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LF60F	DECB				;-1 from message length counter
	CMPB    L0367			;ALDL message length
	BCC     LF663			;Bra if mode/desired ALDL transmision bytes already
					;recieved
;
	TSTB				;Test counter
	BNE     LF62E			;Bra if !=0, desired ALDL mode not recieved
;
;-The desired ALDL mode has been recieved
;
	BRCLR   L0042,#$03,LF655	;ALDL diag. status word, bra if b0,b1==0, neither in
					;ALDL engine or transmission diag. modes
;
;-In diagnostics mode, retrieve desired ALDL transmission sring 
;
	CMPA    #10			;Highest allowed mode
	BHI     LF660			;Bra if >, request not valid
;
	PSHB				;Push B to stack    
	TAB				;Get ALDL mode into B
	ASLB				;x2 to use as addr. offset    
	ABX				;Add it in    
	PULB				;Get B back    
	LDX     $0009,X			;Load in addr. desired transmission string
	BEQ     LF660			;Bra if ==0, request not valid
;
	STX     L0364			;Save it, pointer to ALDL transmission string
	BRA     LF655			;Bra to continue
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Check to see if desired ALDL transmission string has been recieved for mode 1 operation
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LF62E	BRCLR   L0042,#$03,LF655	;Bra if b0, b1==0 neither in engine or transmission
					;diag. mode
;
;-In diagnostics mode, check to see which ALDL mode was desired
;
	XGDY				;Swap A, B into Y    
	LDAA    L036E			;Start of ALDL data input buffer
	CMPA    #$01			;Mode 1, ALDL scan			
	XGDY				;Restore A, B   
	BNE     LF655			;Bra if !=1, no ALDL scan currently requested
;
;-Check byte number to verify that the transmission string has been recieved
;
	CMPB    #$01			;2nd byte recieved, desired ALDL transmission string
	BNE     LF655			;Bra if !=, save and return
;
;-Requested ALDL transmission string has been recieved
;
	CMPA    #$03			;First set of transmission params 
	BCS     LF646			;Bra if <, valid request
;
	CLRA				;Clear A, default to first pointer if invalid 
					;request
;  
;-Look up ALDL transmission string pointer for ALDL scan
; 
LF646	PSHB				;Push message length to stack    
	XGDX				;Swap A, B into X    
	SUBD    #$06			;Rewind to pointers
	XGDX				;Get A, B back    
	TAB				;Get recieved data into B     
	ASLB				;x2    
	ABX				;Add in as offset     
	LDX     0,X			;Load pointer
	PULB				;Get message length back    
	STX     L0364			;Save it, pointer to ALDL transmission string
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~
; Save ALDL data to buffer
;~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LF655	LDX     7,X			;Load start addr. of data input buffer
	ABX				;Add in message length as offset     
	STAA    0,X			;Save it to the buffer
;
LF65A	INC     L0362			;+1, message length counter
	JMP     LF704			;Jump to continue
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Here if error recieving transmission 
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LF660	JMP     LF6F7			;Go reset and return
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Perform additional ALDL functions 
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LF663	LDAB    L0363			;ALDL message checksum
	BNE     LF660			;Bra if !=0, errors transmitting
;
;-No serial transmission errors here
;
	LDAB    2,X			;Load in engine/transmission diag. ID
	STAB    L0366			;Save it, current ALDL diagnostics ID
	LDY     #L0390			;Addr of engine diag. data buffer
	LDAA    L0367			;ALDL message length
	BRSET   L0050,#$08,LF6BA	;Status word, bra if b3==1, factory diag. requested
;
;-No factory diags. requested
;
	CMPB    #$F5			;ALDL transmission mode ID
	BNE     LF680			;Bra if !=, in engine diags. mode
;
;-ALDL in transmission diagnostics mode
;
	LDY     #L039B			;Addr. of transmission diag. data buffer			
;
LF680	BCLR    L003B,#$03		;ALDL diag. status word, clear b0, b1, mode 
					;10 flags for engune and transmission diags.
	LDAB    L036E			;Start of ALDL data buffer, read in desired mode
	STAB    0,Y			;Save it
	BNE     LF69B			;Bra if !=0, not in mode 0
;
;-In mode 0, normal mode, ALDL engine/transmission diag. mode no longer desired
;
	LDAB    #$FE			;Clear b0, ALDL engine diag. mode flag
	CPY     #L039B			;Compare to addr. of transmission diags buffer
	BNE     LF695			;Bra if !=, in engine diag. mode
;
	LDAB    #$FD			;Clear b1, ALDL transmission diag. mode flag
;
LF695	ANDB    L0042			;Clear selected bits
	STAB    L0042			;Save it, ALDL diag. status word word
	BRA     LF6CB			;Bra to continue
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Engine/transmission diags. requested, find out which mode was desired
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LF69B	CMPB    #10			;Mode 10, ALDL error reset
	BNE     LF6B0			;Bra if not in mode 10
;
;-In mode 10, ALDL error reset
;
	LDAB    L0366			;Current ALDL diag. ID
	CMPB    #$F5			;ALDL transmission diag. mode ID
	BEQ     LF6AB			;Bra if transmission diag. mode
;
	BSET    L003B,#$02		;ALDL diag. status word, set b1, in mode 10 
					;(engine diags.)
	BRA     LF6CB			;Bra to continue
;
LF6AB	BSET    L003B,#$01		;ALDL diag. status word, set b0, in mode 10 
					;(trans. diags.)
	BRA     LF6CB			;Bra to continue
;
;-Check if mode 4 is in effect
;
LF6B0	CMPB    #$04			;Mode 4, controller mode
	BNE     LF6CB			;Bra if not in mode 4
;
;-In mode 4, external controller mode
;
	CMPA    #11			;11, max allowed ALDL message length
	BLS     LF6BA			;Bra if <=
;
	LDAA    #11			;Load max message length
;
LF6BA	TSTA				;Test message length    
	BEQ     LF6CB			;Bra if ==0
;
;-Retrieve data from controller
;								
	LDX     #L036E			;Addr. of ALDL data buffer
;
LF6C0	LDAB    0,X			;Load data from ALDL buffer
	STAB    0,Y			;Save it to diag. data buffer
	INX				;+1, move to next addr. 	    
	INY				;+1     
	DECA				;-1, ALDL message length
	BNE     LF6C0			;Bra if !=0, continue
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Set up for next ALDL transmission
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LF6CB	LDX     L0364			;Pointer to ALDL transmission string
	BRSET   4,X,#$80,LF6F7		;Number of bytes to XMIT, bra if b7==1,
					;not responding with data
;
;-Transmitting data here
;
LF6D2	STX     L0364			;Save pointer to ALDL transmission string
	LDAB    L302E			;SCI data register
	LDAA    2,X			;Load in ID byte
	STAA    L302F			;Save it, SCI data register
;
	STAA    L0363			;Save it, ALDL message checksum
	LDAB    #$01			;1, transmit one byte
	STAB    L0362			;Save it, ALDL message length
;
	LDD     L3FFC			;I/O port D
	JSR     LF08A			;Delay
;
	ORAB    #$04			;Set b2
	STD     L3FFC			;Save it
;
;-Enable SCI data transmission
; 
	LDAA    #$88			;Set b3, transmit enable, and b7, transmit interrupt
					;enable
	STAA    L302D			;Save it, SCI control register 2
	BRA     LF704			;Bra to continue
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Here if no transmission or errors, reset and return
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LF6F7	CLRA				;Clear A, B    
	CLRB				;    	
	STD     L0364			;Save it, pointer to ALDL transmission string
	STD     L0362			;ALDL message length and checksum
	LDAA    #$26			;Set b1, reciever wakeup, b2, recieve enable bit, and
					;b5, recieve interrupt enable
	STAA    L302D			;SCI control register 2
;
LF704	RTS				;Return  
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 
;
;~~~~~~~~~~~~~~~~~~~~~~
;
; Factory diagnostics
;
;~~~~~~~~~~~~~~~~~~~~~~
;
LF705	LDS     #L03FF         		; SET USR STX
	LDAA    #$01           		; SET UP TO READ DIAG INPUT
	LDX     #L3000			; INDEX CPU REG'S
;
	JSR     LEF12			; DELAY ROUTINE
;
	LDAA    #$01			;
	JSR     LEEFC			;
;
	LDAA    L3031			; GET A/D RESULT #1,
	CMPA    #100			; 2VDC
	BCC     LF72C			; BR IF VDC GT 100 bin, (2 VDC)
;
	CMPA    #40			; 800 mvdc 
	BCS     LF72C			; BR IF A/D VAL LT 800 mvdc
;
	LDAA    #$03			;
	JSR     LEEFC			;
;
	LDAA    L3031			; GET A/D RESULT #1
	CMPA    #26			;
	BHI     LF72D			;
;
LF72C	SWI				;
;     
LF72D	CLI				;     
;
;----------------
; TOGGLE COP REG
;----------------
;
	LDD     #$AA55			;
	STAA    L303A			; COP RST
	STAB    L303A			; COP RST
;
	LDAB    L0366			; ENG/XMISH MODE BYTE
	CMPB    #$02			;
	BEQ     LF749			;
;
	JSR     LF081			;
;
	CMPB    #$03			;
	BNE     LF753			;
;
	BRCLR   L0001,#$01,LF753	; BR IF BOT b0, 
;
LF749	BSET    L0001,#$01		; SET b0
	LDX     L0390			; ALDL MODE NUMBER
	JSR     0,X			;
;
	BRA     LF771			; EXIT TO WAIT FOR INTIRUP
;
LF753	CMPB    #$01			;
	BNE     LF774			; BR IF NOT RAM UP LD MSG
;
;---------------------
; UPLOAD RX'ED DATA
;
; XFER TO RAM STARTING
; AT  $02AE, (32 BYTES)
;---------------------
;
	BCLR    L0001,#$01		; BR IF b0
	LDX     L0390			; POINT TO MSG DESTINATION
					; (ALDL MSG MODE BYTE FOR ENGINE)
;
	LDY     #L0392           	; POINT TO MSG DATA ADDR
	LDAB    #32                	; 32 BY FIXED LEN MSG'S
;
LF763	LDAA    0,Y			; GET DATA BYTE 
	STAA    0,X			; XFER DATA TO DEST RAM LOC
	INY				; BUMP ADDR POINTERS
	INX				;     
	DECB				; DEC 32 BYTE CNT'R
	BNE     LF763			; LOOP TILL CNT'R = 0
;
	COM     L0366			; ENG/XMISH MODE BYTE
;
LF771	JMP     LF87E			; DO NOTHING LP, (WAIT FOR INIT ?)
;
;--------------------------------------------
; CK FOR OUTPUT	CYCLING DISABLE	 CONDITIONS
;
;
;--------------------------------------------
;
LF774	LDX     #LF8AB			; 
	TSTB    			; NZ, A VALID SERAIL DATA MSG
					; HASE BEEN RX'ED, DONT CYCLE										; OUTUTS
	BNE     LF7E3			;
;
	LDY     #L3064			; I/O DEV PORT
	BRCLR   0,Y,#$80,LF7E3		; BR IF NOT b7
;
	LDX     #$DFFF			; 57,343d
	STX     L3FCC			;
	JSR     LF880			; SHORT DELAY
;								
	LDX     #$DFFF			; 57,343d
	STX     L3FEA			;
	JSR     LF880			; SHORT DELAY
;
;---------------------------
; OUTPUT CYCLING 
; 
;---------------------------
;
	LDD     #197			; SET UP FOR 3msec PULSES
	STD     L3FCE			; SAVE TO EFI PW
	JSR     LF880			; SHORT CPU DELAY
;
	LDX     #66			; SET DWELL TO 1 Msec
	STX     L3FDC			; SPK WP, (DWELL)
	JSR     LF880			; EXIT via RTS
;
	LDD     #$FFF1			; 65,521d
	SUBD    #$05			;
	STD     L3FF6			; EST FALL CNT'R
					; (TIME FM DRP TO FIRE IGN)
	LDAB    L0002			; MJR LOOP SEGMENT COUNT; MAJOR LOOP COUNTER
	CMPB    #$4F			; 0010 1111
	BLS     LF7B7			; BR IF 
;									
	CLRB 				;   
;
LF7B7	STAB    L0002			; MJR LOOP SEGMENT COUNT
	BITB    #$0F			; 0000 1111
	BNE     LF771			;
;
	PSHB				;
	LDAA    L3062			; I/O PORT D
	TAB				; TO B reg
	ANDA    #$0C			; MASK 0000 1100
	BEQ     LF7CE			; BR IF NOT b2 & b3
;
	CMPA    #$0C			; b2 & b3
	BEQ     LF7CE			; BR IF NOT b2 & b3
;
	EORB    #$08			; b3
	BRA     LF7D0			;
;	
LF7CE	EORB    #$04			;
LF7D0 	ORAB    #$30			;
	STAB    L3062			;
	PULB				;
	LSRB				;
	LSRB				;
	LSRB				;
	LSRB				;
	LDAA    #$03			;
	MUL				;
;
;----------------------------------------
; MYSTERY TABLE
;
;----------------------------------------
;
	LDX     #LF89C			;
	ABX				;
	BRA     LF7F4			;				
;
;-----------------
; I/O D PORT
;
;-----------------
;
LF7E3	LDAA    L3062			; I/O PORT D
	ANDA    #$E3			; 0111 0011
	STAA    L3062			; I/O PORT D
	LDD     #$0000			;
	STD     L3FCE			;
;
	JSR     LF880			; EXIT via RTS
;
LF7F4	LDAA    L3060			; I/O PORT
	ANDA    #$6F			; 0110 1111
	ORAA    0,X			;
	STAA    L3060			; I/O PORT
;
;---------------
; I/O D PORT
;---------------
;
	LDAA    L3062			; I/O PORT D
	ANDA    #$FC			; 1111 1100
	ORAA    1,X			;
	STAA    L3062			; I/O PORT D
;
	SEI				;     
	LDD     L3FFC			; I/O D PORT ..
	JSR     LF880			; EXIT via RTS
;
	ANDB    #$F7			;
	ORAB    2,X			;
	STD     L3FFC			; I/O D PORT ..
;
	JSR     LF880			; EXIT via RTS
	CLI				;     
	CPX     #LF8AB			;
	BNE     LF842			;
;
	LDX     #$D000			;
	STX     L3FD4			;
	BSR     LF880			; EXIT via RTS
;
	STX     L3FD6			;
	BSR     LF880			; EXIT via RTS
;
	STX     L3FD8			;
	BSR     LF880			; EXIT via RTS
;
	STX     L3FDA			;
	STX     L306A			;
	STX     L306C			;
	STX     L306E			;
	STX     L3068			;
	BRA     LF87E			;
;
LF842	LDX     #$D399			; 
	STX     L3FD4			;
	BSR     LF880			; EXIT via RTS
;
	LDX     #$D366			;
	STX     L3FD6			;
	BSR     LF880			; EXIT via RTS
;
	LDX     #$D332			;
	STX     L3FD8			;
	BSR     LF880			; EXIT via RTS
;
	LDX     #$D2FF			;
	STX     L3FDA			;
	LDX     #$7FB3			;
	STX     L306A			;
	LDX     #$7FA6			;
	STX     L306C			;
	LDX     #$7F66			;
	STX     L306E			;
	LDX     #$0D73			;
	STX     L3068			;
	LDX     #LF88A			; INDEX MSG HANDLER
	JSR     LF6D2			;
;
LF87E	BRA     LF87E			;
;
LF880	RTS				;    
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;--------------------------------------------------
; FACTORY TEST MSG LIST 
;
; (SERIAL DAT MSG TBL)
; $0d          
;--------------------------------------------------
;
LF881	FDB  LF88A		; LINK TO NEXT MSG
	FCB  $01		; MESSAGE ID
	FCB  $00		; USE RAM BUFFER
	FCB  $00		; NUM OUTPUT BYTES
;
	FDB  L0390		; OCB ADDR 
	FDB  L036E		; ICB ADDR
;
;----------------------------------------
;
;                                
;----------------------------------------
;
LF88A	FDB  LF893		; LINK TO NEXT MSG
	FCB  $02		; MESSAGE ID
	FCB  $00		; USE RAM BUFFER
	FCB  $00		; NUM OUTPUT BYTES
;					
	FDB  L0390		; OCB ADDR
	FDB  L036E		; ICB ADDR
;
;----------------------------------------
;
;                                
;----------------------------------------
;
LF893	FDB  $0000		; END OF LINKED MSG LIST
	FCB  $03		; MESSAGE ID
	FCB  $00		; USE RAM BUFFER
	FCB  24			; NUM OUTPUT BYTES
;
	FDB  L036E		; OCB ADDR
	FDB  L036E		; ICB ADDR
;
;----------------------------------------
; MYSTERY TABLE
;*************************
;* OUTPUT CYCLING TABLE
;*************************
;                                
;----------------------------------------
;
LF89C  	FCB  $10		;
LF89D	FCB  $20   		;
LF89E  	FCB  $00		;
LF89F	FCB  $10		;
LF8A0  	FCB  $21		;
LF8A1	FCB  $08		; 
LF8A2  	FCB  $10		;
LF8A3	FCB  $22		; 
LF8A4  	FCB  $08		;
LF8A5	FCB  $00		; 
LF8A6  	FCB  $20		;
LF8A7	FCB  $08		; 
LF8A8  	FCB  $90		;
LF8A9	FCB  $20		; 
LF8AA  	FCB  $08		;
LF8AB	FCB  $10		; 
LF8AC  	FCB  $00		;
LF8AD	FCB  $08		; 
;
;~~~~~~~~~~~~~~~~~~~~
;
; SWI vector handler
;
;~~~~~~~~~~~~~~~~~~~~
;
LF8AE	BSET	L0000,#$08		;Status word, set b3, SWI requested
;
LF8B1	BRA	LF8B1			;Loop to wait for reset	
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;~~~~~~~~~~~~~~~~~~~~~~
;
; XIRQ Vector handler
;
;~~~~~~~~~~~~~~~~~~~~~~
;
LF8B3	BSET 	L0000,#$04		;Status word, set b2, XIRQ 
	PULA				;
	ORAA	#$40			;
	PSHA				;
	RTI				;Return from interrupt
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;~~~~~~~~~~~~~~~~~~~~~~
;
; Factory test routine
;
;~~~~~~~~~~~~~~~~~~~~~~
;
LF8BB	BSET    L0000,#$02		;Status word, set b1, factory test in effect
	LDAA    #$01			;Set b0, TIC 3, injector handling
	STAA    L3023			;Timer flag 1 reg. 
	RTI				;Return from interrupt     
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; Illegal opcode vector handler
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;
LF8C4	BSET    L0000,#$10		;Status word, set b4, illegal opcode encountered
;
LF8C7	BRA     LF8C7			;Loop to wait for reset
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; COP time out vector handler
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LF8C9	BSET    L0000,#$20		;Status word, set b5, COP timed out
	BRA     LF8D6			;Bra to restart
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; Clock fail vector handler
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LF8CE	BSET	L0000,#$40		;Status word, set b6, MPU clock failure
;
LF8D1	BRA     LF8D1			;Loop to wait for reset
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;~~~~~~~~~~~~~~~~~~~~~~
;
; Reset vector handler
;
;~~~~~~~~~~~~~~~~~~~~~~
;
LF8D3	BSET    L0000,#$80		;Status word, set b7, MPU has reset/powered up
;
LF8D6	JMP     L7200			;Main 160 Hz vector handler
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; Interrupt handler for:
; TOC 2, TIC 1, TIC 2, TIC 3, RTI
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
LF8D9	LDAA    #$A0			;1010 0000, Interrupt for TOC 1 and TOC 3 only
	STAA    L3022			;Timer Mask Reg. 1 (TMSK1)
	LDAA    #$5F			;0101 1111, reset all but TOC 1/3
	STAA    L3023			;Timer Interrupt Flag Reg. 1 (TFLG1)
	RTI 				;Return from interrupt    
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; PA input edge detect/PA, timer overflow vector handler
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;

LF8E4	LDAA    #$03			;Timer prescaler, E/16, 7.63 usec/count
	STAA    L3024			;Timer mask register 2
	LDAA    #$FF			;1111 1111
	STAA    L3025			;Timer flag register 2, clear all flags
;
LF8EE	RTI				;Return from interrupt     
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; 'Cool' linear temp vs. A/D counts
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; 				;Deg C
;
LF8EF	FCB   255		;151
	FCB   255		;151    
	FCB   255		;151        
	FCB   254		;151         
	FCB   237		;138          
	FCB   225		;129         
	FCB   215		;121          
	FCB   207		;115          
	FCB   201		;111          
	FCB   195		;106          
	FCB   190		;103         
	FCB   185		; 99            
	FCB   181		; 96           
	FCB   178		; 94          
	FCB   174		; 91          
	FCB   171		; 88         
	FCB   168		; 86          
	FCB   166		; 85           
	FCB   163		; 82          
	FCB   161		; 81          
	FCB   159		; 79        
	FCB   157		; 78           
	FCB   155    		; 76           
	FCB   153   		; 75          
	FCB   151    		; 73         
	FCB   149    		; 72          
	FCB   147    		; 70          
	FCB   146    		; 70           
	FCB   144    		; 68          
	FCB   143    		; 67          
	FCB   141    		; 66           
	FCB   140    		; 65       
	FCB   139    		; 64        
	FCB   137    		; 63       
	FCB   136    		; 62          
	FCB   135    		; 61           
	FCB   134    		; 61         
	FCB   133    		; 60         
	FCB   131    		; 58         
	FCB   130    		; 58          
	FCB   129    		; 57          
	FCB   128    		; 56         
	FCB   127    		; 55         
	FCB   126    		; 55         
	FCB   125    		; 54          
	FCB   124    		; 53          
	FCB   124    		; 53     
	FCB   123    		; 52         
	FCB   122    		; 52       
	FCB   121    		; 51      
	FCB   120    		; 50        
	FCB   119    		; 49      
	FCB   118    		; 49 
	FCB   118    		; 49    
	FCB   117   		; 48        
	FCB   116    		; 47         
	FCB   115    		; 46       
	FCB   114    		; 46      
	FCB   114    		; 46      
	FCB   113    		; 45         
	FCB   112    		; 44         
	FCB   112   		; 44         
	FCB   111    		; 43        
	FCB   110    		; 43        
	FCB   110    		; 43        
	FCB   109    		; 42         
	FCB   108    		; 41         
	FCB   108    		; 41         
	FCB   107    		; 40        
	FCB   106    		; 40        
	FCB   106    		; 40       
	FCB   105    		; 39        
	FCB   104    		; 38      
	FCB   104    		; 38       
	FCB   103    		; 37       
	FCB   103		; 37        
	FCB   102    		; 37          
	FCB   101    		; 36       
	FCB   101    		; 36   
	FCB   100    		; 35      
	FCB   100    		; 35      
	FCB   99     		; 34        
	FCB   99     		; 34       
	FCB   98     		; 34        
	FCB   98     		; 34 
	FCB   97		; 33	   
	FCB   96     		; 32        
	FCB   96     		; 32      
	FCB   95     		; 31          
	FCB   95     		; 31         
	FCB   94     		; 31          
	FCB   94     		; 31          
	FCB   93     		; 30           
	FCB   93     		; 30         
	FCB   92     		; 29          
	FCB   92     		; 29         
	FCB   91     		; 28          
	FCB   91     		; 28         
	FCB   90     		; 28         
	FCB   90     		; 28          
	FCB   89     		; 27        
	FCB   89     		; 27        
	FCB   88     		; 26         
	FCB   88     		; 26        
	FCB   87     		; 25         
	FCB   87     		; 25        
	FCB   86     		; 25        
	FCB   86     		; 25       
	FCB   86     		; 25       
	FCB   85     		; 24          
	FCB   85     		; 24       
	FCB   84     		; 23     
	FCB   84     		; 23       
	FCB   83     		; 22        
	FCB   83     		; 22      
	FCB   82     		; 22       
	FCB   82     		; 22         
	FCB   81     		; 21       
	FCB   81     		; 21      
	FCB   80     		; 20         
	FCB   80     		; 20       
	FCB   80     		; 20      
	FCB   79     		; 19      
	FCB   79     		; 19       
	FCB   78     		; 19      
	FCB   78     		; 19     
	FCB   77     		; 18        
	FCB   77     		; 18     
	FCB   77     		; 18       
	FCB   76     		; 17      
	FCB   76     		; 17     
	FCB   75     		; 16    
	FCB   75     		; 16    
	FCB   74     		; 16    
	FCB   74     		; 16      
	FCB   73     		; 15     
	FCB   73     		; 15      
	FCB   73     		; 15     
	FCB   72     		; 14       
	FCB   72     		; 14     
	FCB   71     		; 13         
	FCB   71     		; 13      
	FCB   70     		; 13        
	FCB   70     		; 13         
	FCB   70     		; 13     
	FCB   69     		; 12  
	FCB   69     		; 12      
	FCB   68     		; 11    
	FCB   68     		; 11      
	FCB   67     		; 10     
	FCB   67     		; 10  
	FCB   67     		; 10   
	FCB   66     		; 10   
	FCB   66     		; 10 
	FCB   65     		;  9  
	FCB   65     		;  9
	FCB   64     		;  8   
	FCB   64    		;  8     
	FCB   63     		;  7      
	FCB   63     		;  7    
	FCB   63     		;  7      
	FCB   62     		;  7      
	FCB   62     		;  7     
	FCB   61     		;  6         
	FCB   61     		;  6     
	FCB   60     		;  5   
	FCB   60     		;  5       
	FCB   59     		;  4        
	FCB   59     		;  4      
	FCB   59     		;  4     
	FCB   58     		;  4       
	FCB   58     		;  4    
	FCB   57     		;  3      
	FCB   57     		;  3     
	FCB   56     		;  2      
	FCB   56     		;  2        
	FCB   55     		;  1         
	FCB   55     		;  1        
	FCB   54     		;  1        
	FCB   54     		;  1        
	FCB   54     		;  1        
	FCB   53     		; -0     
	FCB   53     		; -0    
	FCB   52     		; -1    
	FCB   52     		; -1       
	FCB   51    		; -2     
	FCB   51     		; -2      
	FCB   50     		; -2      
	FCB   50    		; -2         
	FCB   49     		; -3        
	FCB   49     		; -3         
	FCB   48     		; -4      
	FCB   48     		; -4      
	FCB   47     		; -5       
	FCB   47     		; -5        
	FCB   46     		; -5       
	FCB   45     		; -6     
	FCB   45     		; -6      
	FCB   44     		; -7     
	FCB   44     		; -7        
	FCB   43     		; -8      
	FCB   43     		; -8        
	FCB   42     		; -8        
	FCB   42     		; -8        
	FCB   41     		; -9         
	FCB   40     		;-10       
	FCB   40     		;-10        
	FCB   39     		;-11  
	FCB   39     		;-11       
	FCB   38     		;-11      
	FCB   37     		;-12      
	FCB   37     		;-12       
	FCB   36     		;-13       
	FCB   35     		;-14       
	FCB   35     		;-14       
	FCB   34     		;-14   
	FCB   33     		;-15    
	FCB   33     		;-15      
	FCB   32     		;-16       
	FCB   31     		;-17      
	FCB   31     		;-17        
	FCB   30    		;-17     
	FCB   29     		;-18       
	FCB   28     		;-19       
	FCB   28     		;-19      
	FCB   27     		;-20      
	FCB   26     		;-20      
	FCB   25     		;-21        
	FCB   24     		;-22        
	FCB   23     		;-23       
	FCB   22     		;-23       
	FCB   21     		;-24        
	FCB   20     		;-25         
	FCB   19     		;-26          
	FCB   18     		;-26       
	FCB   17     		;-27     
	FCB   16     		;-28    
	FCB   15     		;-29      
	FCB   13     		;-30        
	FCB   12     		;-31        
	FCB   11     		;-32        
	FCB   9      		;-33       
	FCB   8      		;-34      
	FCB   6      		;-35      
	FCB   4      		;-37       
	FCB   3      		;-38    
	FCB   1      		;-39       
	FCB   0      		;-40    
	FCB   0      		;-40      
	FCB   0     		;-40         
	FCB   0      		;-40        
	FCB   0      		;-40        
	FCB   0      		;-40        
	FCB   0      		;-40         
	FCB   0      		;-40         
	FCB   0      		;-40    
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; 'Warm' linear temp vs. A/D counts
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
; 				;Deg C
;
LF9EF	FCB   255    		;151          
	FCB   255    		;151        
	FCB   255    		;151   
	FCB   255    		;151           
	FCB   255    		;151          
	FCB   255    		;151         
	FCB   255    		;151         
	FCB   255    		;151         
	FCB   255    		;151          
	FCB   255    		;151          
	FCB   255    		;151          
	FCB   255    		;151          
	FCB   255    		;151           
	FCB   255    		;151           
	FCB   255    		;151          
	FCB   255    		;151          
	FCB   255    		;151          
	FCB   255    		;151          
	FCB   255    		;151          
	FCB   255    		;151         
	FCB   255    		;151      
	FCB   255    		;151          
	FCB   255    		;151       
	FCB   255    		;151         
	FCB   255    		;151         
	FCB   255    		;151          
	FCB   255    		;151          
	FCB   255    		;151          
	FCB   255    		;151         
	FCB   255    		;151          
	FCB   255    		;151        
	FCB   253    		;150         
	FCB   251    		;148         
	FCB   249    		;147         
	FCB   247    		;145         
	FCB   245    		;144           
	FCB   243    		;142          
	FCB   241    		;141          
	FCB   239    		;139           
	FCB   237    		;138          
	FCB   236    		;137           
	FCB   234    		;136          
	FCB   232    		;134          
	FCB   231    		;133         
	FCB   229    		;132         
	FCB   228    		;131        
	FCB   227    		;130       
	FCB   225    		;129       
	FCB   224    		;128          
	FCB   222    		;127          
	FCB   221    		;126          
	FCB   220    		;125          
	FCB   219    		;124        
	FCB   217    		;123        
	FCB   216    		;122        
	FCB   215    		;121        
	FCB   214    		;121         
	FCB   213    		;120         
	FCB   211    		;118         
	FCB   210    		;118          
	FCB   209    		;117         
	FCB   208    		;116          
	FCB   207    		;115          
	FCB   206    		;115        
	FCB   205    		;114        
	FCB   204    		;113        
	FCB   203    		;112         
	FCB   202    		;112        
	FCB   201    		;111       
	FCB   200    		;110           
	FCB   199    		;109        
	FCB   198    		;109        
	FCB   197    		;108          
	FCB   196    		;107          
	FCB   195    		;106         
	FCB   195    		;106          
	FCB   194    		;106         
	FCB   193    		;105          
	FCB   192    		;104          
	FCB   191    		;103          
	FCB   190    		;103          
	FCB   189    		;102        
	FCB   189    		;102     
	FCB   188    		;101          
	FCB   187    		;100         
	FCB   186    		;100        
	FCB   185    		; 99         
	FCB   185    		; 99         
	FCB   184    		; 98        
	FCB   183    		; 97          
	FCB   182    		; 97          
	FCB   182    		; 97          
	FCB   181    		; 96          
	FCB   180    		; 95          
	FCB   179    		; 94          
	FCB   179    		; 94          
	FCB   178    		; 94          
	FCB   177    		; 93          
	FCB   176    		; 92          
	FCB   176    		; 92          
	FCB   175    		; 91   
	FCB   174    		; 91        
	FCB   174    		; 91       
	FCB   173    		; 90       
	FCB   172   		; 89        
	FCB   171   		; 88      
	FCB   171    		; 88     
	FCB   170    		; 88       
	FCB   169    		; 87       
	FCB   169    		; 87     
	FCB   168    		; 86       
	FCB   167   		; 85     
	FCB   167    		; 85       
	FCB   166    		; 85     
	FCB   165    		; 84         
	FCB   165    		; 84        
	FCB   164    		; 83      
	FCB   163    		; 82     
	FCB   163    		; 82     
	FCB   162    		; 82   
	FCB   161    		; 81         
	FCB   161    		; 81         
	FCB   160    		; 80        
	FCB   160    		; 80         
	FCB   159    		; 79        
	FCB   158    		; 79      
	FCB   158    		; 79      
	FCB   157    		; 78        
	FCB   156    		; 77       
	FCB   156    		; 77      
	FCB   155    		; 76        
	FCB   155    		; 76     
	FCB   154    		; 76       
	FCB   153    		; 75        
	FCB   153    		; 75         
	FCB   152    		; 74        
	FCB   151    		; 73        
	FCB   151    		; 73         
	FCB   150    		; 73         
	FCB   150    		; 73       
	FCB   149    		; 72   
	FCB   148    		; 71    
	FCB   148    		; 71       
	FCB   147    		; 70     
	FCB   147    		; 70      
	FCB   146    		; 70     
	FCB   145    		; 69        
	FCB   145    		; 69      
	FCB   144    		; 68        
	FCB   143    		; 67      
	FCB   143    		; 67       
	FCB   142    		; 67       
	FCB   142    		; 67       
	FCB   141    		; 66         
	FCB   140    		; 65          
	FCB   140    		; 65          
	FCB   139    		; 64       
	FCB   139    		; 64    
	FCB   138    		; 64     
	FCB   137    		; 63       
	FCB   137    		; 63      
	FCB   136    		; 62          
	FCB   135    		; 61        
	FCB   135    		; 61      
	FCB   134    		; 61        
	FCB   134    		; 61        
	FCB   133    		; 60        
	FCB   132    		; 59        
	FCB   132    		; 59         
	FCB   131    		; 58       
	FCB   130    		; 58         
	FCB   130    		; 58         
	FCB   129    		; 57      
	FCB   128    		; 56        
	FCB   128    		; 56       
	FCB   127    		; 55        
	FCB   126    		; 55        
	FCB   126    		; 55  
	FCB   125    		; 54    
	FCB   125    		; 54     
	FCB   124    		; 53     
	FCB   123    		; 52     
	FCB   122    		; 52     
	FCB   122    		; 52     
	FCB   121    		; 51        
	FCB   120    		; 50       
	FCB   120    		; 50      
	FCB   119    		; 49      
	FCB   118    		; 49       
	FCB   118    		; 49        
	FCB   117    		; 48      
	FCB   116    		; 47       
	FCB   116    		; 47    
	FCB   115    		; 46      
	FCB   114    		; 46      
	FCB   113    		; 45     
	FCB   113    		; 45        
	FCB   112    		; 44        
	FCB   111    		; 43       
	FCB   110    		; 43      
	FCB   109    		; 42         
	FCB   109    		; 42         
	FCB   108    		; 41          
	FCB   107    		; 40          
	FCB   106    		; 40          
	FCB   105    		; 39          
	FCB   105    		; 39          
	FCB   104    		; 38          
	FCB   103    		; 37          
	FCB   102    		; 37          
	FCB   101    		; 36          
	FCB   100    		; 35          
	FCB   99     		; 34          
	FCB   98     		; 34          
	FCB   97     		; 33          
	FCB   97     		; 33          
	FCB   96     		; 32          
	FCB   95     		; 31          
	FCB   94     		; 31          
	FCB   92     		; 29          
	FCB   91     		; 28          
	FCB   90     		; 28          
	FCB   89     		; 27          
	FCB   88    		; 26          
	FCB   87     		; 25          
	FCB   86     		; 25          
	FCB   85     		; 24          
	FCB   83     		; 22          
	FCB   82     		; 22          
	FCB   81     		; 21          
	FCB   79     		; 19          
	FCB   78     		; 19          
	FCB   76     		; 17          
	FCB   75     		; 16          
	FCB   73     		; 15          
	FCB   71     		; 13          
	FCB   70     		; 13          
	FCB   68     		; 11          
	FCB   66     		; 10          
	FCB   64     		;  8          
	FCB   62     		;  7          
	FCB   59     		;  4          
	FCB   57     		;  3          
	FCB   54     		;  1          
	FCB   51     		; -2          
	FCB   47     		; -5          
	FCB   43     		; -8          
	FCB   39     		;-11          
	FCB   34     		;-14          
	FCB   27     		;-20          
	FCB   18     		;-26          
	FCB   3      		;-38          
	FCB   0      		;-40          
	FCB   0      		;-40         
	FCB   0      		;-40      
	FCB   0      		;-40        
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Deg C MAT temp vs. AD MAT counts
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;				;Deg C
;
LFAEF  	FCB   0			;-40
	FCB   26		;-28 
	FCB   44		;-16 
	FCB   56		; -4  
	FCB   65		;  8  
	FCB   74		; 20 
	FCB   82		; 32 
	FCB   89		; 44
	FCB   97		; 56
	FCB   105		; 68 
	FCB   113		; 80 
	FCB   122		; 92 
	FCB   133		;104 
	FCB   147		;116 
	FCB   166		;128 
	FCB   199		;140 
	FCB   255		;152 	
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~
;-SES error blink out table
;~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;	Error #
;	Upper nibble = 10's blink out
;	Lower nibble = 1's blink out	;Error # counter
;
LFB00  	FCB  $12		; 0
;
;-Alternate start point for HU table
;
LFB01  	FCB  $13		; 1
	FCB  $14		; 2
	FCB  $15		; 3
	FCB  $16		; 4
	FCB  $17		; 5
	FCB  $18		; 6
	FCB  $19		; 7
	FCB  $21		; 8
	FCB  $22		; 9
	FCB  $23		;10 
	FCB  $24		;11
	FCB  $25		;12 
	FCB  $26		;13
	FCB  $27		;14
	FCB  $28		;15
	FCB  $29		;16
	FCB  $31		;17 
	FCB  $32		;18
	FCB  $33		;19 
	FCB  $34		;20
	FCB  $35		;21
	FCB  $36		;22
	FCB  $37		;23 
	FCB  $38		;24
	FCB  $39		;25 
	FCB  $41		;26 
	FCB  $42		;27
	FCB  $43		;28
	FCB  $44		;29
	FCB  $45		;30
	FCB  $46		;31
	FCB  $47		;32
	FCB  $48		;33
	FCB  $49		;34
	FCB  $51		;35
	FCB  $52		;36
	FCB  $53		;37
	FCB  $54		;38
	FCB  $55		;39
	FCB  $56		;40
	FCB  $57		;41
	FCB  $58		;42
	FCB  $59		;43
	FCB  $61		;44
	FCB  $62		;45
	FCB  $63		;46
	FCB  $64		;47
	FCB  $65		;48
	FCB  $66		;49
	FCB  $67		;50
	FCB  $68		;51
	FCB  $69		;52
	FCB  $71		;53 
	FCB  $72		;54
	FCB  $73		;55
	FCB  $74		;56
	FCB  $75		;57  
	FCB  $76		;58
	FCB  $77		;59
	FCB  $78		;60
	FCB  $79		;61
	FCB  $81		;62
	FCB  $82		;63
	FCB  $83		;64
	FCB  $84		;65
	FCB  $85		;66
	FCB  $86		;67
	FCB  $87		;68
	FCB  $88		;69
	FCB  $89		;70
	FCB  $91		;71
;
;-Alternate end point for HU table
;
	FCB  $92		;71
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~-END OF MAIN CODE-~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; Start of interrupt vectors
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;
        ORG  $FF90		;
        
FF90	FDB  L7E99		;
FF92	FDB  L7C7E		;
FF94	FDB  LA4D0		;
FF96	FDB  L7EEF		;
FF98	FDB  $C300		;
FF9A	FDB  $0000		;		
FF9C	FDB  $0000		;
FF9E	FDB  $0000		;
FFA0	FDB  LEF47		;
FFA2	FDB  LEEFC		;
FFA4	FDB  LEF12		;Port D config routine
FFA6	FDB  LF081		;
FFA8	FDB  L0035		;
FFAA	FDB  $036E		;
FFAC	FDB  $0000		;
FFAE	FDB  $0000		;
FFB0	FDB  L53D7		;
FFB2	FDB  $0000		;
FFB4	FDB  $0000		;
FFB6	FDB  $0000		;
FFB8	FDB  $0000		;
FFBA	FDB  $0000		;
FFBC	FDB  $0000		;
FFBE	FDB  $0000		;
FFC0	FDB  LF8EE		;
FFC2	FDB  LF8EE		;
FFC4	FDB  LF8EE		;		
FFC6	FDB  LF8EE		;
FFC8	FDB  LF8EE		;
FFCA	FDB  LF8EE		;
FFCC	FDB  LF8EE		;
FFCE	FDB  LF8EE		;
FFD0	FDB  LF8EE		;
FFD2	FDB  LF8EE		;
FFD4	FDB  LF8EE		;
;
;~~~~~~~~~~~~~~
;-Vector Table
;~~~~~~~~~~~~~~
;
	ORG  $FFD6
;
LFFD6	FDB  LF494		;SCI interrupt
LFFD8   FDB  LF8EE		;SPI vector to RTI
LFFDA	FDB  LF8E4		;Pulse acc. input edge vect.
LFFDC   FDB  LF8E4		;Pulse acc. overflow
LFFDE   FDB  LF8E4		;Timer overflow
LFFE0   FDB  L7922		;TOC 5, Inj. handling
LFFE2   FDB  L7986		;TOC 4, Inj. handling
LFFE4   FDB  L79EA		;TOC 3, Main vector handler
LFFE6   FDB  LF8D9		;TOC 2,
LFFE8   FDB  LCC8A		;TOC 1, Knock counter routine
LFFEA   FDB  LF8D9		;TIC 3, 1x cam sens. 
LFFEC   FDB  LF8D9		;TIC 2, VATS
LFFEE   FDB  LF8D9 		;TIC 1, 
LFFF0   FDB  LF8D9		;RTI
LFFF2   FDB  L7597		;IRQ, DRP/cranking fuel routine
LFFF4   FDB  LF8B3		;XIRQ 
LFFF6   FDB  LF8AE		;SWI 
LFFF8   FDB  LF8C4		;Illegal OP code
LFFFA   FDB  LF8C9		;COP time out
LFFFC   FDB  LF8CE		;Clock fail
LFFFE   FDB  LF8D3		;Power on reset vector

